;---------------------------------------------------------------
;+
; NAME: ANS_ANCIL2DATA
; PURPOSE:
;   To create SIO files of latitude, longitude, incident angle,
;   or radar look angle in SIO format based on fit data in
;   the ancillary data files
; CALLING SEQUENCE
;   ans_ancil2data, ancilfile
; INPUT
;   ancilfile    = Name of file with ancillary data
; OUTPUT
;  SIO file with the appropriate data
; KEYWORDS:
;   /quiet       = No screen output
;   /help        = Get help
;   /longitude   = Generate longitude SIO data
;   /latitude    = Generate latitude  SIO data
;   /incidence   = Generate nadir incident angle SIO data
;   /rlook       = Generate radar look diretion wrt to North
;   outputfile   = Name of output file.  The default is to base
;                = the output file name on the ancil file
;   outputdata   = Array of generated data
;   error=       = error
; MODIFICATION HISTORY:
;   2013 Apr 19  Frank Monaldo
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_ancil2data, ancilfile               , $
                    longitude  = longitude  , $
                    latitude   = latitude   , $
                    incidence  = incidence  , $
                    rlook      = rlook      , $
                    outputfile = outputfile , $
                    outputdata = outputdata , $
                    error      = error      , $
                    help       = help       , $
                    quiet      = quiet



if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_ANCIL2DATA
  print, ' PURPOSE:
  print, '   To create SIO files of latitude, longitude, incident angle,
  print, '   or radar look angle in SIO format based on fit data in
  print, '   the ancillary data files
  print, ' CALLING SEQUENCE
  print, '   ans_ancil2data, ancilfile
  print, ' INPUT
  print, '   ancilfile    = Name of file with ancillary data
  print, ' OUTPUT
  print, '  SIO file with the appropriate data
  print, ' KEYWORDS:
  print, '   /quiet       = No screen output
  print, '   /help        = Get help
  print, '   /longitude   = Generate longitude SIO data
  print, '   /latitude    = Generate latitude  SIO data
  print, '   /incidence   = Generate nadir incident angle SIO data
  print, '   /rlook       = Generate radar look diretion wrt to North
  print, '   outputfile   = Name of output file.  The default is to base
  print, '                = the output file name on the ancil file
  print, '   outputdata   = Array of generated data
  print, '   error=       = error
  error= -1
  return
endif

error=      0
!quiet=     1  ; Suppress compile statements
!except=    0  ; Suppress arithmetic error warnings
printflag=  1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if not file_test(ancilfile) then begin
  if printflag then print, 'Cannot find '+ ancilfile + ' Exiting ans_ancil2data.pro'
  error  = -1
  return
endif

if not keyword_set( longitude ) and $
   not keyword_set( latitude  ) and $
   not keyword_set( incidence ) and $
   not keyword_set( rlook )     then begin
   if printflag then print, 'No data type selected. Exiting ans_ancil2data.pro'
   error = -1
   return
endif

if keyword_set( longitude ) then type = 'LONGITUDE'
if keyword_set( latitude  ) then type = 'LATITUDE'
if keyword_set( incidence ) then type = 'INCIDENCE'
if keyword_set( rlook     ) then type = 'RADARLOOK'

if  not keyword_set( outputfile ) then begin
    position   =  strpos( ancilfile, 'ancil.txt' )
    case type of 
    'LONGITUDE' : filetail = 'longitude.dat'
    'LATITUDE'  : filetail = 'latitude.dat'
    'INCIDENCE' : filetail = 'incidence.dat'
    'RADARLOOK' : filetail = 'radarlook.dat'
    ELSE        : begin
                    if printflag then print, 'No data type selected. Exiting ans_andil2data.pro'
                    error = -1
                    return
                  end
    endcase
    outputfile =  strmid( ancilfile, 0, position ) + filetail
endif

ans_read_sio_info, ancilfile , info, quiet=quiet

case type of 
'LONGITUDE' : begin
                coef  =  info.lon_coef
                xexp  =  info.lon_xexp
                yexp  =  info.lon_yexp
              end
'LATITUDE'  : begin
                coef  =  info.lat_coef
                xexp  =  info.lat_xexp
                yexp  =  info.lat_yexp
              end
'INCIDENCE' : begin
                coef  =  info.incid_coef
                xexp  =  info.incid_xexp
                yexp  =  info.incid_yexp
              end
'RADARLOOK' : begin
                coef  =  info.rlook_coef
                xexp  =  info.rlook_xexp
                yexp  =  info.rlook_yexp
              end
ELSE        : begin
                if printflag then print, 'No data type selected. Exiting ans_andil2data.pro'
                error = -1
                return
              end
endcase

nx         =  long( info.pixels_sio_image )
ny         =  long( info.lines_sio_image  )
outputdata =  fltarr( nx, ny )
if printflag then begin
  print, 'Computing ' + strlowcase( type ) + '...'
  print, 'Nx = ' + string(nx)
  print, 'Ny = ' + string(ny)
endif


ii   =  dindgen(nx)
jj   =  dblarr(nx)
jstep = ny / 10
for j=0L, ny-1L do begin
  jj = jj * 0.0 + j
  outputdata[*,j] = ans_poly_cal( ii, jj, coef, xexp, yexp )
  if printflag and j mod jstep eq 0 then print, string(j, format='(I7)') + ' of ' +  string(ny-1, format='(I7)') + string(outputdata[0,j])
endfor





if printflag then print, 'Writing out SIO ' + type + ' data to: ' + outputfile + '.'

magic_number = -16678914L
nx           =  long(nx)
ny           =  long(ny)
data_type    =  3L
byte_size    =  4L

close,  1
openw,  1, outputfile
writeu, 1, magic_number, ny, nx, data_type, byte_size
writeu, 1, outputdata
close,  1

if printflag then print, 'Finished writing ' + outputfile + '.'


end
