;-----------------------------------------------------
;+
; NAME: ans_asf2sio
; PURPOSE:
;   To convert Radarasat-1/ERS-2 data to SIO files (and Level-1 and KMZ if selected)
; CALLING SEQUENCE
;   ans_asf2sio, deffile, sio_root
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
;                sio_kmz_dir, twobyte, rsat1_asf_dir,deg, ave_line, and ave_pixel, log_dir,
;                silent            
; OUTPUT
;   sio_root=    Root of the SIO filenames (full path)
;   SIO files, NetCDF level 1 (optional)
; KEYWORDS:
;   /quiet       No screen output
;   /help        Get help
; MODIFICATION HISTORY:
;   2010 Dec 29 Frank Monaldo. Change the architecture to separate out SIO generation
;                              by satellite
;   2010 Dec 30 Frank Monaldo. Returns sio_root
;   2011 Feb 05 Frank Monaldo. Add jsstart and pass it to ans_read_asf
;   2011 Feb 11 Frank Monaldo. Add working directory in call to ans_sio2kmz
;   2011 Feb 22 Frank Monaldo. Changed the .cdf extension to .nc
;   2012 Aug 16 Frank Monaldo. Changed message on leadging program
;   2012 Dec 31 Frank Monaldo. Change call to ans_read_asf to use defaultus file
;   2013 Aug 18 Frank Monaldo. Removed computation of ice mask.  This now done at the
;                              ans_sat2sio level.
; Copyright (C) 2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_asf2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=   1
printflag=1
if keyword_set(quiet) then printflag=0
sio_root=''

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ans_asf2sio
  print, ' PURPOSE:
  print, '    To convert Radarasat-1/ERS-2 data to SIO files (and Level-1 and KMZ if selected)
  print, ' CALLING SEQUENCE
  print, '   ans_asf2sio, deffile, sio_root
  print, ' INPUT
  print, '   deffile=     Full path to defaults file. This option will read a specified file
  print, '                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
  print, '                sio_kmz_dir, twobyte, rsat1_asf_dir,deg, ave_line, and ave_pixel, log_dir,
  print, '                silent
  print, '                
  print, ' OUTPUT
  print, '   sio_root=    Root of the SIO filenames (full path)
  print, '   SIO files, NetCDF level 1 (optional)
  print, ' KEYWORDS:
  print, '   /quiet       No screen output
  print, '   /help        Get help
  return
endif

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))



ans_read_defaults, deffile, df, /quiet
;
; Read defaults file
;
if tag_test(df, 'silent') then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 00

if printflag then begin
  print, '**************************'
  print, 'Begin ans_asf2sio. ' + systime(/utc)
  print, '************************** '
endif


;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir      
res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir           
res= file_search( df.log_dir,         /test_directory,  count=count)
if count eq 0 then file_mkdir, df.log_dir        


;
;  
;---------------------------------------------------------------
;------------------- Start Radarsat-1 or ERS-2 ASF processing ---------------
;---------------------------------------------------------------

if printflag then begin
  print,' '
  print, '**********Conversion of  Radarsat-1 or ERS-2 ASF to SIO *************'
  print,''
endif


leaderfile=    file_search( df.rsat1_asf_dir +'/*L', count=count)  
if count eq 0 then begin
  print, 'Cannot find file that matches ' + df.rsat1_asf_dir +'/*L'
  print, 'Exiting ans_asf2sio...'
  return
endif

leaderfile= leaderfile[0]
length= strlen(leaderfile)
datafile= strmid(leaderfile, 0,length-1) + 'D'
if printflag then print, 'Looking for ' + datafile
datafile=    file_search( datafile, count=count) 
if count eq 0 then begin
  print, 'Cannot find file that matches ' + datafile
  if strlen(df.already_processed) gt 1 then begin
    root= strmid(leaderfile, 0,length-1) + '*'
    listing= file_search(root,count=ct)
    for i=0, ct-1 do begin
      if i eq 0 then begin
        print, 'Moving ' + listing[i] 
      endif else begin
        print, 'and    ' + listing[i]
      endelse
    endfor
    print, 'to     ' + df.already_processed
    file_move, root, df.already_processed  + '/' , /overwrite
  endif
  print, 'Exiting ans_asf2sio..'
  sio_root=''
  return
endif


;
; Computing SIO files for Radarsat-1/ERS-2 
;


if printflag then print, 'Computing Radarsat-1 or ERS-2 ASF Level-1 SIO data...
error=1
ans_read_asf,  datafile, leaderfile, deffile, $
               outputroot  = outputroot,      $
               jsstart     = jsstart,         $
               quiet       = quiet,           $
               error       = error
if n_elements(error) gt 0 then begin
  if error lt 0 then return
endif
sio_root=      outputroot
;
; Generating file names
;
sio_root             = getwrd( sio_root, 0, /last, del='/' ) 
datafile             = df.sio_output_dir   + '/' + sio_root + '_image.dat'
infofile             = df.sio_output_dir   + '/' + sio_root + '_ancil.txt'
landfile             = df.sio_output_dir   + '/' + sio_root + '_mask.dat'
kmzfile              = df.sio_kmz_dir      + '/' + sio_root + '_sio.kmz'
netcdffile_level_1   = df.ncdf1_output_dir + '/' + sio_root + '_sar_level1.nc'
icefile              = df.sio_output_dir   + '/' + sio_root + '_ice.dat
;
; Computing Level 1 NetCDF files
;
logfile= df.log_dir + '/' + sio_root + '.log'
if strupcase(df.level1cdf) eq 'Y'  then begin
  if printflag then print, 'Computing Level-1 NetCDF data...     
  ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet

  time1=   dt_tm_tojs(systime(/utc))
  outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
endif


;
;
; SIO to KMZ
;
if df.siokmzfile eq 'Y' then begin
  if printflag then begin
     print, ''
     print, 'Computing KMZ files.'
  endif
  ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, /nrcs, quiet=quiet, $
               working_dir=df.working_directory
    res= file_search(df.log_dir, count=count)
  if count eq 0 then file_mkdir, df.log_dir
  logfile= df.log_dir + '/' + sio_root + '.log'
  time1=   dt_tm_tojs(systime(/utc))
  outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
endif

sio_root= df.sio_output_dir   + '/' + sio_root

;---------------------------------------------------------------
;--------------------- End Radarsat-1 or ERS-2 ASF processing ---------------
;---------------------------------------------------------------

end
