;+ ------------------------------------------
;  NAME:
;   ANS_DEF_ANCIL2ICEMASK
; PURPPOSE
;   To generate a ice mask consistent with the original SAR image using the defaults file and the
;   ancillary data file.  The ancillary data file is adjusted to contain information about the
;   ice data
; CALLING SEQUENCE
;   ans_def_ancil2icemask, deffile, ancilfile
; INPUT
;   deffile      =  defaults file
;   ancilfile    =  ancillary data file
; OUTPUT:
;   Unsigned one-byte ice mask written into an output file
; KEYWORDS:
;   outputfile  =   Outputfile name (default behavior is the generate a name consistent with the
;                   ancil data file name
;   /quiet      =   Do not print intermediate results.
;   /help       =   List help
;   error:          >=0 good, <0 bad  
;                   error    meaning
;                     -1     Help called or defaults file not found
;                     -2     Cannot find ancillary data file
;                     -3     Image south of min latitude
; NOTE:
;   0=  No data
;   1=  Water
;   2=  Land
;   3=  Sea Ice
;   4=  Snow
; MOFDICATION HISTORY:
;   2013 Aug 09 Frank Monaldo. Create
;   2013 Aug 13 Frank Monaldo. Push quiet setting to ans_find_ice_file program
;   2013 Aug 21 Frank Monaldo. Correct bug in the generation of a outputfile name, when none is
;                              supplied,
;   2013 Sep 30 Frank Monaldo. Add return if not in lattitude range. Opps then eliminated the return
;   2013 Oct 21 Frank Monalod. Corrected formating of write of icefile_time_js to file
;   2013 Nov 12 Frank Monaldo. Added error codes
;   2013 Nov 19 Frank Monaldo. Corrected error in help print out.
;   2014 Apr 16 Frank Monlado. Change call to nic_read_ice_cdf to minimize memory requirements. Only
;                              reads data in specified longitude and altitude range
;   2014 Jul 14 Chris Jackson  Correctly refernce the df structure in the call to tag value for ice_mask_n
;                              Add printout to screen for error = -3
;   2015 Nov 13 Frank Monaldo. Suppress error message printout for no icemask found unless printflag=1
;   2019 Dec 15 Chris Jackson. Change the info.basis_lon_fit to 2 is (basis is 1) but lonmax returned by 
;                              nic_read_ice_cdf is > 180.  Fix jstep calculation to an integer number.
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;--------------------------------------------------------

pro ans_def_ancil2icemask, deffile, ancilfile,               $
                          outputfile  =  outputfile        , $
                          quiet       =  quiet             , $
                          help        =  help              , $
                          error       =  error

error=  0
if keyword_set(help)  or (n_params(0) lt 1) then begin
  print, '  NAME:
  print, '   ANS_DEF_ANCIL2ICEMASK
  print, ' PURPPOSE
  print, '   To generate a ice mask consistent with the original SAR image using the defaults file and the
  print, '   ancillary data file.  The ancillary data file is adjusted to contain information about the
  print, '   ice data
  print, ' CALLING SEQUENCE
  print, '   ans_def_ancil2icemask, deffile, ancilfile
  print, ' INPUT
  print, '   deffile      =  defaults file
  print, '   ancilfile    =  ancillary data file
  print, ' OUTPUT:
  print, '   Unsigned one-byte ice mask written into an output file
  print, ' KEYWORDS:
  print, '   outputfile  =   Outputfile name (default behavior is the generate a name consistent with the
  print, '                   ancil data file name
  print, '   /quiet      =   Do not print intermediate results.
  print, '   /help       =   List help
  print, '   error:          >=0 good, <0 bad  
  print, '                   error    meaning
  print, '                     -1     Help called or defaults file not found
  print, '                     -2     Cannot find ancillary data file
  print, '                     -3     Image south of min latitude
  print, ' NOTE:
  print, '   0=  No data
  print, '   1=  Water
  print, '   2=  Land
  print, '   3=  Sea Ice
  print, '   4=  Snow
  error= -1
  return
endif

printflag= 1
error=     0
if keyword_set(quiet)   then printflag = 0
maxbyte=  255B
icefile=  'NONE'

quiet=  printflag eq 0
time1=  dt_tm_tojs( systime() ) 

if file_test( deffile) then begin
  ans_read_defaults, deffile, df, quiet=quiet
endif else begin
  print, 'Cannot find defaults file: ' + deffile
  error =  -1
  return
endelse

if file_test( ancilfile) then begin
  ans_read_sio_info, ancilfile, info, quiet=quiet
endif else begin
  print, 'Cannot find ancillary data file: ' + ancilfile
  error =  -2
  return
endelse

ice_mask_n=50.00;
if tag_value(df, 'ice_mask_n') then ice_mask_n=df.ice_mask_n;

;-----------------------------------------------
;  Defaults
;-----------------------------------------------

lon_arr        =  fltarr(4)
lat_arr        =  fltarr(4)
nx             =  long(info.pixels_sio_image)
ny             =  long(info.lines_sio_image)
lon_arr[0]     =  ans_poly_cal( 0,   0, info.lon_coef, info.lon_xexp, info.lon_yexp)
lon_arr[1]     =  ans_poly_cal(nx,   0, info.lon_coef, info.lon_xexp, info.lon_yexp)
lon_arr[2]     =  ans_poly_cal(nx,  ny, info.lon_coef, info.lon_xexp, info.lon_yexp)
lon_arr[3]     =  ans_poly_cal( 0,  ny, info.lon_coef, info.lon_xexp, info.lon_yexp)
lat_arr[0]     =  ans_poly_cal( 0,   0, info.lat_coef, info.lat_xexp, info.lat_yexp)
lat_arr[1]     =  ans_poly_cal(nx,   0, info.lat_coef, info.lat_xexp, info.lat_yexp)
lat_arr[2]     =  ans_poly_cal(nx,  ny, info.lat_coef, info.lat_xexp, info.lat_yexp)
lat_arr[3]     =  ans_poly_cal( 0,  ny, info.lat_coef, info.lat_xexp, info.lat_yexp)
if printflag then begin
  print, 'Longitude / latitude corners for icemask'
  print, '    i    j       Long         Lat  '
  print, '-----------------------------------------'
  for i=0, 3 do begin
    case i of
    0: begin
       x= 0
       y= 0
       end
    1: begin
       x= nx
       y= 0
       end
    2: begin
       x= nx
       y= ny
       end
    3: begin     
       x= 0
       y= ny
       end
    endcase
    print,  string(x,          format='(I5)')    + $
            string(y,          format='(I5)')    + $
            string(lon_arr[i], format='(F12.4)') + $
            string(lat_arr[i], format='(F12.4)')
  endfor
endif

if max(lat_arr) lt ice_mask_n then begin
  ;
  ; If the image does not extend north of ice_mask_n
  ; then produce no ice mask
  ;
  if printflag then begin
    print, 'Maximum latitude of image '+ string(max(lat_arr))+' is below the ';
    print, 'minimum specified ice mask latitude '+ string(ice_mask_n);
    print, 'No ice mask will be generated';
  endif
  error=-3
  return
endif
;
;  Find and read netCDF version of the ice file
;

dt     = getwrd(info.acq_date_time, 0)   
year   = getwrd(dt, 0, del='-')
month  = getwrd(dt, 1, del='-')
day    = getwrd(dt, 2, del='-')
tm     = getwrd(info.acq_date_time, 1)
hour   = getwrd(tm, 0, del=':')
minute = getwrd(tm, 1, del=':')
second = getwrd(tm, 2, del=':')

ans_find_ice_file, year, month, day, hour, minute, second, df.ice_mask_dir, icefile, quiet=quiet
if icefile eq 'NONE' then begin
  if printflag then begin
     print, 'Cannot find ice file for date: ' + info.acq_date_time + ' in '+ df.ice_mask_dir
  endif 
  error = -4 
  return
endif

if getwrd( icefile, 0, /last, del='/') eq tag_value(info, 'icefile')  then begin
  ;
  ; SIO file already created
  ; 
  if printflag then print, 'SIO ice mask file already created. Exiting ans_def_ancil2icemask...'
  return
endif
;
; Compute longitude and and latitude range

lonmax    =  max( lon_arr )
lonmin    =  min( lon_arr )
latmax    =  max( lat_arr )
latmin    =  min( lat_arr )
latdiff   =  latmax - latmin
londiff   =  lonmax - lonmin
lonmin    =  lonmin - 0.01 * londiff
lonmax    =  lonmax + 0.01 * londiff
latmin    =  latmin - 0.01 * latdiff
latmax    =  latmax + 0.01 * latdiff


nic_read_ice_cdf, icefile, ice,  $
                  latmin        =  latmin             , $
                  latmax        =  latmax             , $
                  lonmin        =  lonmin             , $
                  lonmax        =  lonmax             , $
                  basis_lon_fit =  info.basis_lon_fit , $
                  error         =  error              , $
                  quiet         =  quiet
;Deal with the odd case of the info.basis_lon_fit being 1 (-180 to 180)
;but the lonmax being returned by nic_read_ice_cdf is > 180.  
;This causes the mask to be computed incorrectly.  So reset the basis to 2   
if (lonmin gt 0 and lonmin lt 180 and lonmax gt 180 and  info.basis_lon_fit eq 1 ) then info.basis_lon_fit = 2

if error lt 0 then begin
  ;
  ; No overlap of data
  ;
  error= -3
  return
endif
;
; compute sub ice mask from icefile data
;
if printflag then print, 'Computing sub ice mask array...'
sz        =  size( ice.mask)
nxice     =  sz[1]
nyice     =  sz[2]


if info.basis_lon_fit eq 2 then begin
  ;
  ; Switch to 0/360 longitude basis
  ;
  if printflag then print, 'Adjusting the longitude from +/-180 to 0-360 longitude basis...'
  w= where(ice.longitude lt 0, icount)
  if (icount gt 0) then  ice.longitude[w] = ice.longitude[w] + 360.0
endif
  
w         =  where( ice.longitude le lonmax and $
                    ice.longitude ge lonmin and $
                    ice.latitude  le latmax and $
                    ice.latitude  ge latmin, count )


if count eq 0 then begin
  ;
  ; No overlap of data
  ;
  error= -3
  return
endif

one2two, w, ice.longitude, ix, iy

imin=  max( [       0, min(ix)-1 ] )
imax=  min( [ nxice-1, max(ix)+1 ] )
jmin=  max( [       0, min(iy)-1 ] )
jmax=  min( [ nyice-1, max(iy)+1 ] )

if (imax-imin) lt 4 or (jmax-jmin) lt 4 then begin
  ; Too few points in the ice mask.'
  ; Exiting ans_ans_def_ancil2icemask.pro.'
  error= -3 
  return
endif

icemask =  ice.mask[             imin:imax, jmin:jmax]
icelon  =  double(ice.longitude[ imin:imax, jmin:jmax] )
icelat  =  double(ice.latitude[  imin:imax, jmin:jmax] )

;
; Changing scaling of ice mask values. This will provide 
; more dynamic range in map re=projection
; 
maxice  =  max(icemask)
maxice  =  max( [ maxice, 1B] )
zscale  =  maxbyte / maxice
icemask =  icemask * zscale

if printflag then begin
  print, 'Rescaled ice mask'
  hlp, icemask, icelon, icelat
endif
  
sz      =  size(icemask)
nxi     =  sz[1]
nyi     =  sz[2]


;
; compute lon/lat ==> i,j relationship for sub ice mask
;
nxmap   =  2L * nyi
nymap   =  2L * nxi
loncen  =  0
latcen  =  0
if info.basis_lon_fit eq 2 then loncen = 180
;
;  Create ice map resampled to to rectilinear longitude and latitude grid.
;
if printflag then begin
  print, 'Creating linear project ice map: '
  print, 'Lat min = ' + string( latmin, format='(F9.4)' )
  print, 'Lat max = ' + string( latmax, format='(F9.4)' )
  print, 'Lon min = ' + string( lonmin, format='(F9.4)' )
  print, 'Lon max = ' + string( lonmax, format='(F9.4)' )
endif
set_plot, 'z'
device, set_resolution = [ nxmap, nymap ]
map_set2, latcen, loncen, /cycl,  /noborder, xmargin=[0,0],  ymargin=[0,0], $
         limit= [  latmin , lonmin, latmax, lonmax ]
mapped = map_patch( icemask, icelon, icelat, xstart=xstart, ystart=ystart, missing = 0)
if zscale ne 1 then mapped = mapped / zscale 
tv, mapped, xstart, ystart
icemap = tvrd()
sz     = size(icemap)
nxice  = sz[1]
nyice  = sz[2]


if printflag then print, 'Computing lon/lat ==> i,j relationship for sub ice mask'

ii_slope  = ( nxice - 1.0 ) / float( lonmax - lonmin )
ii_bias   = - ii_slope * lonmin
jj_slope  = ( nyice - 1.0 ) / float( latmax - latmin )
jj_bias   = - jj_slope * latmin


;
; Fill in ice mask
;
;
;  Note that we loop though indices in the SAR images, compute the
;  corresponding longitudes and latitudes.  These longitudes and 
;  latitudes are used to compute the correponding indices in the
;  sub ice maske array.  The sub image mask values are then placed
;  in the ice mask to be returned.
;

mask=  bytarr(nx,ny)


if printflag then begin
  print, 'Computing SIO ice mask...'
  print, 'Ice mask dimensions: ' + string( nx, format='(I7)') +' x ' +  string( ny, format='(I7)') 
endif
jstep = fix(ny/10)
id    = dblarr(nx,ny)
jd    = dblarr(nx,ny)
iline = findgen(nx)
jline = findgen(ny)

for i=0L, ny-1L do id[*,i] = iline
for i=0L, nx-1L do jd[i,*] = jline
; Compute longitudes and latitude of SAR image positions
inlon   = ans_poly_cal(id, jd, info.lon_coef, info.lon_xexp, info.lon_yexp)
inlat   = ans_poly_cal(id, jd, info.lat_coef, info.lat_xexp, info.lat_yexp)

; Compute indices of ice mask
outi    = long(ii_slope * inlon + ii_bias)
outj    = long(jj_slope * inlat + jj_bias)

; Make sure indices fall within the icemask array
w       = where( outi lt       0, /null) 
outi[w] = 0
w       = where( outi gt nxice-1, /null )
outi[w] = nxice-1
w       = where( outj lt       0, /null)
outj[w] = 0
w       = where( outj gt nyice-1, /null )
outj[w] = nyice-1  

for j=0, ny-1L do begin
  if (j mod jstep) eq 0 and printflag then print, 'Remapping ice mask into SAR array. ' + string(j, format='(I6)')
  for i= 0, nx-1L do mask[i,j] =  icemap[ outi[i,j], outj[i,j] ]
endfor

if n_elements(outputfile) eq 0 then begin
  ;
  ; if an outputfile not given then generate default file name
  ;
  filebreak, ancilfile, dir=d, file=f
  outputfile = d +'/'
  len =  strlen(ancilfile)
  corefile= strmid( ancilfile, 0, len-10 )
  outputfile=  corefile + '_ice.dat'
endif

if printflag then print, 'Writing out SIO image data to: ' + outputfile + '...'

magic_number= -16678914L
nx=            long(nx)
ny=            long(ny)
data_type=     1L               ; (unsigned integer)
byte_size=     1L               ; (one byte)

;
; Need to fill in the header data here.
; Note the ny, nx This was necessary for Chris Wackerman ship dectection code
; to properly interpret the data 
;


close,  1
openw,  1, outputfile
writeu, 1, magic_number, ny, nx, data_type, byte_size
writeu, 1, mask
close,  1

if printflag then print, 'Finished writing ' + outputfile + '.'

;
; Add ice data to ancillary data file
;
icefile = getwrd( icefile, 0, /last, del='/')
if tag_value( info, 'icefile') ne icefile then begin
  s= getfile(ancilfile)
  n= n_elements(s)
  close, 1
  openw, 1, ancilfile
  for i=0, n-1 do begin
     temp= strcompress(getwrd( s[i], 1, del='%' ))
     if temp eq 'icefile' then begin
       printf, 1, icefile + ' % icefile'
     endif else begin
       printf, 1, s[i]
     endelse
  endfor
  close,1
endif

icefile_time_js = double(ice.js)
icefile_time   = dt_tm_fromjs( ice.js, format='Y$-0n$-0d$ h$:m$:s$' ) + ' UTC'

s= getfile(ancilfile)
n= n_elements(s)
close, 1
openw, 1, ancilfile
ficefile         =  1
ficefile_time    =  1
ficefile_time_js =  1
;
; Substitute if ice lines are present
;
for i=0, n-1 do begin
 temp= strcompress(getwrd( s[i], 1, del='%' ))
 case temp of
 'icefile'         : begin
                     printf, 1, string(icefile)                         + ' % icefile'         
                     ficefile         =  0
                     end
 'icefile_time'    : begin
                     printf, 1, string(icefile_time, format='(A24)')    + ' % icefile_time'    
                     ficefile_time    =  0
                     end
 'icefile_time_js' : begin
                     printf, 1, string(icefile_time_js, format='(I24)') + ' % icefile_time_js' 
                     ficefile_time_js =  0
                     end
 ELSE              : printf, 1, s[i]
 endcase
endfor
close, 1
;
; Append if ice lines are present
;.cont
close, 1
openw, 1, ancilfile, /append
if ficefile         then printf, 1, string(icefile)                          + ' % icefile'
if ficefile_time    then printf, 1, string(icefile_time,    format='(A24)')  + ' % icefile_time'
if ficefile_time_js then printf, 1, string(icefile_time_js, format='(A24)')  + ' % icefile_time_js'
close,1 

time2=  dt_tm_tojs( systime())

if printflag then print, 'Time to compute and write ice mask: ' + strcompress( string(time2-time1) ) + ' s'

end
