;+ ------------------------------------------
;  NAME:
;    ANS_DEF_ANCIL2LANDMASK
; PURPPOSE
;   To generate a land mask consistent with the original SAR image.
; CALLING SEQUENCE
;   ans_def_ancil2landmask, deffile, ancilfile
; INPUT
;   deffile      =  Defaults file
;   ancilfile    =  SIO Ancillary data file
; OUTPUT:
;   Signed two-byte land mask written into the ouput file
; KEYWORDS:
;  /quiet:       =  Do not print intermediate results.
;  /help:        =  List help     
;  ouputfile     =  Land mask file name. If not set default is based on the
;                   form of the ancillary data file.
;   error:       =  0 good, -1 bad
; MOFDICATION HISTORY:
;  2013 Aug 01 Frank Monaldo. Creation. This is based on ans_write_land_mask.pro file
;  2013 Sep 09 Frank Monaldo. Check for error in call to ans_read_sio_info
;--------------------------------------------------------

pro  ans_def_ancil2landmask, deffile, ancilfile,     $
                             outputfile = outputfile, $
                             quiet      = quiet     , $
                             help       = help      , $
                             error      = error

                         
if keyword_set(help)  or (n_params(0) lt 1) then begin
  print, '  NAME:
  print, '    ANS_DEF_ANCIL2LANDMASK
  print, ' PURPPOSE
  print, '   To generate a land mask consistent with the original SAR image.
  print, ' CALLING SEQUENCE
  print, '   ans_def_ancil2landmask, deffile, ancilfile
  print, ' INPUT
  print, '   deffile      =  Defaults file
  print, '   ancilfile    =  SIO Ancillary data file
  print, ' OUTPUT:
  print, '   Signed two-byte land mask written into the ouput file
  print, ' KEYWORDS:
  print, '   /quiet:      =  Do not print intermediate results.
  print, '   /help:       =  List help     
  print, '  ouputfile     =  Land mask file name. If not set default is based on the
  print, '                   form of the ancillary data file.
  print, '   error:       =  0 good, -1 bad
  return
endif


printflag= 1
error=     0
if keyword_set(quiet) then printflag=0


if printflag then begin
  print, ''
  print, '----------------------------------'
  print, 'ans_def_ancil2landmask
  print, '----------------------------------'
  print, ''
endif

quiet= printflag eq 0
time1=  dt_tm_tojs( systime()) 

ans_read_defaults, deffile, df, error=error, quiet=quiet
if error lt 0 then return

ans_read_sio_info, ancilfile, info, quiet=quiet, error=error
if error lt 0 then return

	
maxpix         =  10E6				; Max section size in pixels.
maxint         =  32700
nx             =  long(info.pixels_sio_image  )
ny             =  long(info.lines_sio_image   )
lon_coef       =  info.lon_coef
lon_xexp       =  info.lon_xexp
lon_yexp       =  info.lon_yexp
lat_coef       =  info.lat_coef
lat_xexp       =  info.lat_xexp
lat_yexp       =  info.lat_yexp
pixel_spacing  =  float( info.pixel_spacing_m )
line_spacing   =  float( info.line_spacing_m  )
pixel_size     =  float(df.pixel_size_mask    )    

if printflag then begin                                        
  hlp, nx             
  hlp, ny             
  hlp, lon_coef       
  hlp, lon_xexp       
  hlp, lon_yexp       
  hlp, lat_coef       
  hlp, lat_xexp       
  hlp, lat_yexp       
  hlp, pixel_spacing  
  hlp, line_spacing   
  hlp, pixel_size     
endif

if printflag then begin
  print, 'Longitude / latitude corners for landmask'
  lon_arr=fltarr(4)
  lat_arr=fltarr(4)
  lon_arr[0]=  ans_poly_cal( 0,   0, lon_coef, lon_xexp, lon_yexp)
  lon_arr[1]=  ans_poly_cal(nx,   0, lon_coef, lon_xexp, lon_yexp)
  lon_arr[2]=  ans_poly_cal(nx,  ny, lon_coef, lon_xexp, lon_yexp)
  lon_arr[3]=  ans_poly_cal( 0,  ny, lon_coef, lon_xexp, lon_yexp)
  lat_arr[0]=  ans_poly_cal( 0,   0, lat_coef, lat_xexp, lat_yexp)
  lat_arr[1]=  ans_poly_cal(nx,   0, lat_coef, lat_xexp, lat_yexp)
  lat_arr[2]=  ans_poly_cal(nx,  ny, lat_coef, lat_xexp, lat_yexp)
  lat_arr[3]=  ans_poly_cal( 0,  ny, lat_coef, lat_xexp, lat_yexp)
  for i=0, 3 do begin
    print, 'Long: ' +string(lon_arr[i], format='(F12.4)') + '  Lat:' + string(lat_arr[i], format='(F12.4)')
  endfor
endif

;
; Compute scaling factors
;
m_x=    1L
m_y=    1L
if n_elements(pixel_size) gt 0  then begin
  if pixel_size ne 0 then begin
    if printflag then begin
      print, '-----------------------------------'
      print, '    Rescale land mask
      print, '-----------------------------------'    
    endif
    ;
    ; Compute x scaling
    ; 
    if pixel_size gt pixel_Spacing then begin
      for i=1, long(pixel_size/pixel_spacing) do begin
        sc_float=  float(nx) / float(i)
        sc_int=    long(nx)  / long(i)
        if sc_float eq sc_int then m_x= i
      endfor
    endif
    if printflag then begin
      print, 'Pixel spacing=             ' + string(pixel_spacing,        format='(F9.3)') + '  m '
      print, 'Maximum pixel spacing=     ' + string(pixel_size,           format='(F9.3)') + '  m '
      print, 'Selected pixel spacing=    ' + string(m_x * pixel_spacing,  format='(F9.3)') + '  m '
      print, 'Original number of pixels= ' + string(nx,                  format='(I8)')
      print, 'New number of pixels=      ' + string(nx/m_x,              format='(I8)')
      print, 'Scale factor=              ' + string(m_x,                  format='(I8)')
    endif
    pixel_spacing= m_x * pixel_spacing
    ;
    ; Compute y scaling
    ; 
    if pixel_size gt line_Spacing then begin
      for i=1, long(pixel_size/line_spacing) do begin
        sc_float=  float(ny) / float(i)
        sc_int=    long(ny)  / long(i)
        if sc_float eq sc_int then m_y= i
      endfor
    endif   
    if printflag then begin
      print, 'Line spacing=              ' + string(line_spacing,        format='(F9.3)') + '  m '
      print, 'Maximum line spacing=      ' + string(pixel_size,          format='(F9.3)') + '  m '
      print, 'Selected line spacing=     ' + string(m_y * line_spacing,  format='(F9.3)') + '  m '
      print, 'Original number of pixels= ' + string(ny,                 format='(I8)')
      print, 'New number of pixels=      ' + string(ny/m_y,             format='(I8)')
      print, 'Scale factor=              ' + string(m_y,                 format='(I8)')
    endif
    line_spacing=  m_y * line_spacing
  endif
endif
;
; Recomputing the i/j to lat/lon conversion
;
if m_x ne 1 or m_y ne 1 then begin
 nx=  nx / m_x
 ny=  ny / m_y
 nn=  n_elements(lon_coef)-1
 for i=0L, nn do lon_coef[i]=   lon_coef[i]   * double(m_x)^lon_xexp[i]   * double(m_y)^lon_yexp[i]
 nn= n_elements(lat_coef)-1
 for i=0L, nn do lat_coef[i]=   lat_coef[i]   * double(m_x)^lat_xexp[i]   * double(m_y)^lat_yexp[i]
 if printflag then begin
   print, 'Longitude / latitude corners for landmask using scaled coefficients'
   lon_arr=fltarr(4)
   lat_arr=fltarr(4)
   lon_arr[0]=  ans_poly_cal( 0,   0, lon_coef, lon_xexp, lon_yexp)
   lon_arr[1]=  ans_poly_cal(nx,   0, lon_coef, lon_xexp, lon_yexp)
   lon_arr[2]=  ans_poly_cal(nx,  ny, lon_coef, lon_xexp, lon_yexp)
   lon_arr[3]=  ans_poly_cal( 0,  ny, lon_coef, lon_xexp, lon_yexp)
   lat_arr[0]=  ans_poly_cal( 0,   0, lat_coef, lat_xexp, lat_yexp)
   lat_arr[1]=  ans_poly_cal(nx,   0, lat_coef, lat_xexp, lat_yexp)
   lat_arr[2]=  ans_poly_cal(nx,  ny, lat_coef, lat_xexp, lat_yexp)
   lat_arr[3]=  ans_poly_cal( 0,  ny, lat_coef, lat_xexp, lat_yexp)
   for i=0, 3 do begin
     print, 'Long: ' +string(lon_arr[i], format='(F12.4)') + '  Lat:' + string(lat_arr[i], format='(F12.4)')
   endfor
 endif
endif


;-----------------------------------------------
;  Initialize
;-----------------------------------------------
nbyt=  nx *ny			         ; Total bytes.
nsec=  ceil(nbyt/maxpix)		         ; # subsections.
nsec=  max( [nsec,2] )                     ; min nsec=2
iny=   round(maken(0,ny,nsec))<(ny-1)      ; Subsection offsets.
if printflag then print,' Making total land mask array ...'
msk = bytarr(nx,ny)			; Start mask.

;-----------------------------------------------
;  Loop over subsections
;  X indices are cross-track.
;  Y Indices are along-track.
;-----------------------------------------------

dd=   min( [pixel_spacing, line_spacing] )
ppd=   111000./dd
for i = 0, nsec-2 do begin
  lox = 0				; Subsection X indices.
  hix = nx-1
  loy = iny[i]				; Subsection Y indices.
  hiy = iny[i+1]
  if printflag then begin
    print,' ------------------------------------------------------'
    print,' On subsection '+strtrim(i+1,2)+' of '+ $
	    strtrim(nsec-1,2)+'.  Size = '+strtrim(hiy-loy+1,2)+ $
	   ' x '+strtrim(nx,2)
    print,' Making indices arrays ...'
  endif
  makexy,lox,hix,1,loy,hiy,1,ix,iy	; Subsection indices.
  if printflag then print,' Computing lon ...'
  lon = ans_poly_cal(ix,iy,lon_coef,lon_xexp,lon_yexp)	; Convert to lon,lat.
  if printflag then print,' Computing lat ...'
  lat = ans_poly_cal(ix,iy,lat_coef,lat_xexp,lat_yexp)
  lon1 = min(lon,max=lon2)		; Bounding box.
  lat1 = min(lat,max=lat2)
  mx = long((lon2-lon1)*ppd)		; Samples needed.
  my = long((lat2-lat1)*ppd)
  if printflag then print,' Making land mask: Size = '+strtrim(mx,2)+' x '+strtrim(my,2)
  maxdim= max([mx,my])
  mx1=    mx
  my1=    my
  if maxdim ge maxint then begin
    scaling=  float(maxint) / float(maxdim)
    mx1=      long( scaling * mx1)
    my1=      long( scaling * my1)
  endif
  gshhs_land_mask, lon1,lon2,lat1,lat2, mx1, my1, out=smsk, quiet=quiet
  if mx ne mx1 or my ne my1 then begin
    ; resize image back to requested size
    smsk= congrid( smsk, mx, my )
  endif
  jx = (mx-1)*(lon-lon1)/(lon2-lon1)	; Indices into submask.
  jy = (my-1)*(lat-lat1)/(lat2-lat1)
  sec = smsk[jx,jy]
  msk[0,loy] = sec
endfor ; 


;  
;  There is a problem for land mask data below 60S.  This is a hopefully temporary
;  fix.  Here were use the IDL rather than the GSHSS land mask for images that extend below this
;
lon_arr=     fltarr(4)
lat_arr=     fltarr(4)
lon_arr[0]=  ans_poly_cal(double( 0),  double( 0), lon_coef, lon_xexp, lon_yexp)
lon_arr[1]=  ans_poly_cal(double(nx),  double( 0), lon_coef, lon_xexp, lon_yexp)
lon_arr[2]=  ans_poly_cal(double(nx),  double(ny), lon_coef, lon_xexp, lon_yexp)
lon_arr[3]=  ans_poly_cal(double( 0),  double(ny), lon_coef, lon_xexp, lon_yexp)
lat_arr[0]=  ans_poly_cal(double( 0),  double( 0), lat_coef, lat_xexp, lat_yexp)
lat_arr[1]=  ans_poly_cal(double(nx),  double( 0), lat_coef, lat_xexp, lat_yexp)
lat_arr[2]=  ans_poly_cal(double(nx),  double(ny), lat_coef, lat_xexp, lat_yexp)
lat_arr[3]=  ans_poly_cal(double( 0),  double(ny), lat_coef, lat_xexp, lat_yexp)
if min( lat_arr) lt -60 then begin
  if printflag then print, 'Computing southern latitude mask fix.'
  latcen= 0.0
  loncen= -180
  latmin= min(lat_arr)
  lonmin= min(lon_arr)
  latmax= max(lat_arr)
  lonmax= max(lon_arr)
  if max(lon_arr) lt 0 then loncen=0
  set_plot,'z'
  device, set_resolution=[nx,ny]
  loadct,0
  map_set,  latcen, loncen, /CYL,limit=[latmin, lonmin, latmax, lonmax], xmargin=[0,0], ymargin=[0,0], /noborder
  erase, 0
  map_continents, /hires, /fill, color=1
  tvlct, r, g, b, /get
  z=tvrd()
  islope=  float(nx-1)/(lonmax-lonmin)
  ibias=   -lonmin*islope
  jslope=  float(ny-1)/(latmax-latmin)
  jbias=   -latmin*jslope
  for i=0L, nx-1 do begin
    ix= double(i)
    for j=0L, ny-1 do begin
      iy= double(j)
      lon= ans_poly_cal(ix,iy,lon_coef,lon_xexp,lon_yexp)	; Convert to lon,lat.
      lat= ans_poly_cal(ix,iy,lat_coef,lat_xexp,lat_yexp) 
      ii = long(islope * lon + ibias )
      jj=  long(jslope * lat + jbias )
      ;if i mod 100 eq 0 and j mod 100 eq 0 then begin
      ;   print, i, j, lon, lat, ii, jj
      ;endif
      if (ii ge      0) and $
         (ii le (nx-1)) and $
         (jj ge      0) and $
         (jj le (ny-1)) then begin
         msk[i,j]= z[ii,jj] 
      endif
    endfor
  endfor
endif
;
; End of southern latitude fix.

;-----------------------------------------------------------
;  Distance
;
;  Use morph_distance to compute distance in pixels from
;  any land.  This routine uses an approximation.
;  Pixels with 0 distance are land.
;  The edge pixels of d are also 0 so will be fixed by
;  copying adjacent lines or columns.
;-----------------------------------------------------------

if printflag then print, 'Computng distance from water to land...'
	
d = morph_distance(msk,NEIGHBOR_SAMPLING=2,/back)
;---  Fix edge effects  ---
d[0,0]       = d[*,1]
d[0,0]       = d[1,*]
d[0,ny-1]    = d[*,ny-2]
d[nx-1,0]    = d[nx-2,*]
;---  Fix corners  ---
d[0,0]       = d[1,1]
d[0,ny-1]    = d[1,ny-2]
d[nx-1,ny-1] = d[nx-2,ny-2]
d[nx-1,0]    = d[nx-2,1]



mask=  fix(d) * 0
sz= size(d)
nxd=  sz[1]
nyd=  sz[2]
for i=0, nxd-1 do begin
  d[i,*]= d[i,*] < maxint
  w= where(d[i,*] gt 0 , count) 
  if count gt 0 then mask[i,w]= -fix(d[i,w])
endfor

;
; Invert image
;
if printflag then print, 'Inverting msk...'

for i=0, nxd-1 do begin
  w= where(msk[i,*] eq 0, count)
  if count gt 0 then msk[i,w] = 255
  w= where(msk[i,*] eq 1, count)
  if count gt 0 then msk[i,w]=0
  w=where(msk[i,*] eq 255, count)
  if count gt 0 then msk[i,w]=1
endfor

if printflag then print, 'Computng distance from land to water...'
d = morph_distance(msk,NEIGHBOR_SAMPLING=2,/back)
;---  Fix edge effects  ---
d[0,0] = d[*,1]
d[0,0] = d[1,*]
d[0,ny-1] = d[*,ny-2]
d[nx-1,0] = d[nx-2,*]
;---  Fix corners  ---
d[0,0] = d[1,1]
d[0,ny-1] = d[1,ny-2]
d[nx-1,ny-1] = d[nx-2,ny-2]
d[nx-1,0] = d[nx-2,1]

for i=0, nxd-1 do begin
  d[i,*]= d[i,*] < maxint
  w= where(d[i,*] gt 0     , count) 
  if count gt 0 then mask[i,w]= fix(d[i,w])
endfor


magic_number= -16678914L
nx=            long(nx)
ny=            long(ny)
data_type=     2L               ; (Signed Integer)
byte_size=     2L               ; (two bytes)

if n_elements(outputfile) eq 0 then begin
  ;
  ; if an outputfile not given then generate default file name
  ;
  filebreak, ancilfile, dir=d, file=f
  outputfile = d +'/'
  len =  strlen(ancilfile)
  corefile= strmid( ancilfile, 0, len-10 )
  outputfile=  corefile + '_mask.dat'
endif
if printflag then print, 'Writing out SIO image data to: ' + outputfile + '...'
;
; Need to fill in the header data here.
; Note the ny, nx This was necessary for Chris Wackerman ship dectection code
; to properly interpret the data 
;
close,  1
openw,  1, outputfile
writeu, 1, magic_number, ny, nx, data_type, byte_size
writeu, 1, mask
close,  1

if printflag then print, 'Finished writing ' + outputfile + '.'


time2=  dt_tm_tojs( systime())

if printflag then print, 'Time to compute and write land mask: ' + strcompress( string(time2-time1) ) + ' s'
end
