;-----------------------------------------------------
;+
; NAME: ans_envisat2sio.pro
; PURPOSE:
;   To convert Envisat data to SIO files (and Level-1 and KMZ if selected)
; CALLING SEQUENCE
;   ans_envisat2sio, deffile, sio_root
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
;                sio_kmz_dir, twobyte, envisat_dir,deg, ave_line, and ave_pixel, log_dir,
;                silent
; OUTPUT
;   sio_root=    Root(s) of SIO data file names
;   SIO files, NetCDF level 1 and KMZ files if selected
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
; MODIFICATION HISTORY:
;   2010 Dec 31 Frank Monaldo. Converted from ws_process_envisat
;   2011 Feb 05 Frank Monaldo. Add jsstart
;   2011 Feb 07 Frank Monaldo. Add error catching
;   2011 Feb 22 Frank Monaldo. Changed the .cdf extension to .nc
;   2013 Oct 30 Chris Jackson. Changed the file search to accept files that begin with SAR..
;                              in addition to ASA* (SAR... files are ERS data in Envisat format)
; Copyright (C) 2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_envisat2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
sio_root= ''

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, 'NAME: ans_envisat2sio.pro
  print, 'PURPOSE:
  print, '  To convert Envisat data to SIO files (and Level-1 and KMZ if selected)
  print, 'CALLING SEQUENCE
  print, '  ans_envisat2sio, deffile, sio_root
  print, 'INPUT
  print, '  deffile=     Full path to defaults file. This option will read a specified file
  print, '               for the parameters: already_processsed, ncdf1_output_dir, working_directory,
  print, '               sio_kmz_dir, twobyte, envisat_dir,deg, ave_line, and ave_pixel, log_dir,
  print, '               silent
  print, 'OUTPUT
  print, '  sio_root=    Root(s) of SIO data file names
  print, '  SIO files, NetCDF level 1 and KMZ files if selected
  print, 'KEYWORDS:
  print, '  /quiet       No screen output
  print, '  /help;       Get help
  print, '
  return
endif

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))


;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Creating SIO files using ans_envisat2sio. ' + systime(/utc)
  print, ' '
endif

ans_read_defaults, deffile, df, quiet=quiet

if tag_test(df, 'silent') then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 0
;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory, count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir
res= file_search( df.sio_kmz_dir,      /test_directory, count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir  
res= file_search( df.log_dir,         /test_directory,  count=count)
if count eq 0 then file_mkdir, df.log_dir 

;
; Look for Envisat directories in base directory
;
ff1= file_search( df.envisat_dir +'/ASA*', count=count1)
ff2= file_search( df.envisat_dir +'/SAR*', count=count2)
if (count1 + count2) eq 0 then begin
  print, 'Cannot find files that match ' +'ASA*N1 of SAR*E1 (or E2)'
  print, 'in directory' +  df.envisat_dir
  print, 'Exiting ans_envisat2sio'
  sio_root=''
  return
endif
;
if count1 gt 0 and count2 eq 0 then ff=ff1
if count1 eq 0 and count2 gt 0 then ff=ff2
if count1 gt 0 and count2 gt 0 then ff=[ff1, ff2]

; Computing SIO files for all the Envisat data
;
datafile=   ff[0]
error=1
ans_read_envisat, datafile,   deffile,        $
                  jsstart    =  jsstart     , $
                  outputroot =  outputroot  , $
                  error      =  error       , $
                  quiet      =  quiet       
                  
if error lt 0 then return                  
                  
sio_root=         outputroot


;
; Loop through SIO files for full processing
;
n= n_elements(sio_root)


for i=0, n-1 do begin
  ;
  ; Generating file names
  ;
  if strlen(sio_root[i]) gt 0 then begin 
    root=                getwrd( sio_root[i], 0, /last, del='/' )
    datafile=            sio_root[i]         + '_image.dat'
    infofile=            sio_root[i]         + '_ancil.txt'
    landfile=            sio_root[i]         + '_mask.dat'
    kmzfile=             df.sio_kmz_dir      + root + '_sio.kmz'
    netcdffile_level_1=  df.ncdf1_output_dir + root + '_sar_level1.nc'
    res= file_search(df.log_dir, count=count)
    if count eq 0 then file_mkdir, df.log_dir
    logfile= df.log_dir + '/' + root + '.log'
    ;
    ; Computing Level 1 NetCDF files
    ;  
    if strupcase(df.level1cdf) eq 'Y'  then begin
      if printflag then print, 'Computing Envisat SAR Level-1 NetCDF data...   
      ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      print, outstr
      close, 1
    endif
    ;
    ; SIO to KMZ
    ;
    if df.siokmzfile eq 'Y' then begin
      if printflag then begin
         print, ''
         print, 'Computing KMZ files.'
      endif
      ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
                  /nrcs, quiet=quiet, working_dir=df.working_directory
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      print, outstr
      close, 1
    endif
  endif
endfor ; End loop through SIO files.


;---------------------------------------------------------------
;--------------------- End Envisat to SIO Processing ---------------
;---------------------------------------------------------------

end
