;-----------------------------------------------------
;+
; NAME: ANS_GAT2SIO
; PURPOSE:
;   To convert images from Radarsat-1 imagery processed at Gatineau to SIO format
; CALLING SEQUENCE
;  ans_gat2sio, deffile, sio_root
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters /twobyte, deg, ave_line, and ave_pixel
;                If the parameters are explicitly set. For example, ave_pixel=0 then
;                it will over-ride what ever is in the defaults file
; OUTPUT
;   SIO files, NetCDF level 1, and KMZ files
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
; MODIFICATION HISTORY:
;   2011 Jan 01 Frank Monaldo.
;   2011 Feb 05 Frank Moanldo. Add jsstart
;   2011 Feb 05 Frank Monaldo. Add error exit 
;   2011 Feb 22 Frank Monaldo. Change .cdf extension to .nc
; Copyright (C) 201, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_gat2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

sio_root=''

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_GAT2SIO
  print, ' PURPOSE:
  print, '   To convert images from Radarsat-1 imagery processed at Gatineau to SIO format
  print, ' CALLING SEQUENCE
  print, '  ans_gat2sio, deffile, sio_root
  print, ' INPUT
  print, '   deffile=     Full path to defaults file. This option will read a specified file
  print, '                for the parameters /twobyte, deg, ave_line, and ave_pixel
  print, '                If the parameters are explicitly set. For example, ave_pixel=0 then
  print, '                it will over-ride what ever is in the defaults file
  print, ' OUTPUT
  print, '   SIO files, NetCDF level 1, and KMZ files
  print, ' KEYWORDS:
  print, '   /quiet       No screen output
  print, '   /help;       Get help
  return
endif

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))


;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin wind speed processing using ans_gat2sio. ' + systime(/utc)
  print, ' '
endif


ans_read_defaults, deffile, df, quiet=quiet
;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir      
res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir           
res= file_search( df.log_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.log_dir    

;
; Look for Radarsat-1 Gatineau directories in base directory. Checking
; for *L *T and *D. The larger of the *L and *T files is what is used
; for the ancillary data, i.e., the effective leaderfile
;
trailerfile=    file_search( df.rsat1_gat_dir +'/*T', count=count)
trailerfile_sz= 0
ct=             count
roott=          ''
if count gt 0 then begin
  res =            file_info(trailerfile[0])
  if res.exists then trailerfile_sz=  res.size
  length=          strlen(trailerfile[0])
  roott=           strmid(trailerfile[0], 0, length-2)
endif

leaderfile=    file_search( df.rsat1_gat_dir +'/*L', count=count)
leaderfile_sz= 0
cl=            count
rootl=         ''
if count gt 0 then begin
  res =            file_info(leaderfile[0])
  if res.exists then leaderfile_sz=  res.size
  length=          strlen(leaderfile[0])
  rootl=           strmid(leaderfile[0], 0, length-2)
endif

datafile=    file_search( df.rsat1_gat_dir +'/*D', count=count) 
datafile_sz= 0
rootd=       ''
cd=          count
if count gt 0 then begin
  res =            file_info(datafile[0])
  if res.exists then datafile_sz=  res.size
  length=          strlen(datafile[0])
  rootd=           strmid(datafile[0], 0, length-2)
endif
if printflag then begin
  print, 'Trailer file=      ' + trailerfile[0] + $
         ' ' + string(trailerfile_sz, format='(I10)') + ' bytes'
  print, 'Leader file=       ' + leaderfile[0]+ $
         ' ' + string(leaderfile_sz, format='(I10)')  + ' bytes'
  print, 'Data file=         ' + datafile[0]+ $
         ' ' + string(datafile_sz, format='(I10)')    + ' bytes'
endif

if cd     eq 0     or  $
   rootd  ne rootl or  $
   rootd  ne roott then begin
  if printflag then print, 'Exiting ans_gat2sio..'
  sio_root=''
  return
endif
;
; Computing SIO files for Radarsat-1 Gatineau data
;

lfile=       trailerfile[0]
if leaderfile_sz gt trailerfile_sz then lfile= leaderfile[0]
dfile=       datafile[0]
if printflag then begin
 print, 'Input files selected:'
 print, '  Ancillary data file:' + lfile
 print, '  Image data file:    ' + dfile
endif

if printflag then print, 'Computing Radarsat-1 Gatineau Level-1 SIO data...

error=1
ans_read_gat_rsat,  dfile, lfile, deffile     ,  $
                    outputroot =  outputroot  ,  $
                    jsstart    =  jsstart     ,  $
                    error      =  error       ,  $
                    quiet      =  quiet
if n_elements(error) gt 0 then begin
  if error lt 0 then return
endif
                    
sio_root=       outputroot
;
; Generating file names

root=                getwrd( sio_root, 0, /last, del='/' )
datafile=            sio_root                    + '_image.dat'
infofile=            sio_root                    + '_ancil.txt'
landfile=            sio_root                    + '_mask.dat'
kmzfile=             df.sio_kmz_dir       + root + '_sio.kmz'
logfile=             df.log_dir           + root + '.log'
netcdffile_level_1=  df.ncdf1_output_dir  + root + '_sar_level1.nc'

;
; Computing Level 1 NetCDF files
;

if strupcase(df.level1cdf) eq 'Y'  then begin
  if printflag then print, 'Computing Radarsat-1 Gatineau NetCDF 1 data...   
  ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet
  time1= dt_tm_tojs(systime(/utc))
  outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' +  netcdffile_level_1
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  close, 1
endif
;
; SIO to KMZ
;
if df.siokmzfile eq 'Y' then begin
  if printflag then print, 'Computing KMZ files.'
  ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
               /nrcs, quiet=quiet, working_dir=df.working_directory
  time1= dt_tm_tojs(systime(/utc))
  outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' +  kmzfile
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  close, 1
endif


end
