;----------------------------------------------------------
;+
; NAME: ANS_GEN_SIO_VARIABLE
; PURPOSE:
;    To generate an SIO file of incidnent angle, radar look angle, or
;    longitude, or latitude
; CALLING SEQUENCE:'
;    ans_gen_sio_variable, infile
; INPUT:'
;   infile=       SIO ancillary data file
; OUTPUT:
;   output file in SIO format. SIO file is derivative of infile
; KEYWORDS:
;    /quiet=      Do not print intermediate results'
;    /help=       List help.'
;    dtype=       Data file type
;                 incid    , incident angle in degrees (default)
;                 rlook    , radar look angle in degrees wrt. north
;                 longitude, decimal degrees
;                 latitude,  decimal degrees
;   outfile=      Force this name for output file
; MODIFICATION
;  2012 Nov 14  Frank Monaldo. Creation
; Copyright (C) 2012, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_gen_sio_variable, infile,          $
                          quiet  = quiet,  $
                          help   = help,   $
                          dtype  = dtype,  $
                          outfile= outfile
                             
                             
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 )then begin
  print, ' NAME: ANS_GEN_SIO_VARIABLE
  print, ' PURPOSE:
  print, '    To generate an SIO file of incidnent angle, radar look angle, or
  print, '    longitude, or latitude
  print, ' CALLING SEQUENCE:'
  print, '    ans_gen_sio_variable, infile
  print, ' INPUT:'
  print, '   infile=       SIO ancillary data file
  print, ' OUTPUT:
  print, '   output file in SIO format. SIO file is derivative of infile
  print, ' KEYWORDS:
  print, '    /quiet=      Do not print intermediate results'
  print, '    /help=       List help.'
  print, '    dtype=       Data file type
  print, '                 incid    , incident angle in degrees (default)
  print, '                 rlook    , radar look angle in degrees wrt. north
  print, '                 longitude, decimal degrees
  print, '                 latitude,  decimal degrees
  return
endif

res= file_search( infile, count=count)

if count eq 0 then begin
  print, 'Cannot find file ' + infile
  print, 'Exit ans_gen_sio_variable'
endif
ans_read_sio_info, infile, info, quiet=quiet


ny      = long(info.lines_sio_image) 
nx      = long(info.pixels_sio_image)
iarr    = dblarr( nx, ny )
jarr    = dblarr( nx, ny )
iline   = dindgen( nx    )
jline   = dindgen( ny    )
for j=0, ny-1L do iarr[*,j]= iline 
for i=0, nx-1L do jarr[i,*]= jline

if n_elements(dtype) eq 0 then dtype='incid'

dtype=  strlowcase(dtype)
if dtype ne 'rlook'     and  $
   dtype ne 'longitude' and  $
   dtype ne 'latitude'  then dtype='incid'
   
   
case dtype of
'incid':     begin
               c=     info.incid_coef
               xexp=  info.incid_xexp
               yexp=  info.incid_yexp
               lab=   'incident angle'
             end
'rlook':     begin
               c=     info.rlook_coef
               xexp=  info.rlook_xexp
               yexp=  info.rlook_yexp
               lab=   'radar look angle'
             end
'longitude': begin
               c=     info.lon_coef
               xexp=  info.lon_xexp
               yexp=  info.lon_yexp
               lab=   'longitude'
             end
'latitude':  begin
               c=     info.lat_coef
               xexp=  info.lat_xexp
               yexp=  info.lat_yexp
               lab=   'latitude'
             end  
endcase

if printflag then print, 'Start computing ' + lab + '. ' + systime()
variable= float( ans_poly_cal(iarr, jarr, c, xexp, yexp) )
if printflag then print, 'End   computing ' + lab + '. ' + systime()

if n_elements(outfile) eq 0 then begin
  outfile=  stress(infile,  'R', 0, 'ancil',  dtype, ned)
  outfile=  stress(outfile, 'R', 0, '.txt' , '.dat', ned)
endif


magic_number= -16678914L
nx=            long(nx)
ny=            long(ny)
data_type=     3L
byte_size=     4L

;
; Need to fill in the header data here.
; Note the ny, nx This was necessary for Chris Wackerman ship dectection code
; to properly interpret the data 
;
if printflag then begin
  ;
  print, 'Statistics of saved image '  + lab + ' data'
  n=    n_elements(variable)
  nsub= 100
  if n lt 1000 then  nsub=10
  w= findgen(n/nsub) * nsub
  print, 'Number of elements =' + strcompress( string( n_elements( variable    ) ) )
  print, 'Number of samples  =' + strcompress( string( n_elements( w           ) ) )  
  print, 'nx                 =' + strcompress( string(        nx                 ) )  
  print, 'ny                 =' + strcompress( string(        ny                 ) )    
  print, 'Mean=              =' + strcompress( string(       mean( variable[w] ) ) )
  print, 'Standard Deviation =' + strcompress( string(       stdev(variable[w] ) ) )
  print, 'Maximum            =' + strcompress( string(       max(  variable[w] ) ) )
  print, 'Minimum            =' + strcompress( string(       min(  variable[w] ) ) )
endif

if printflag then print, 'Start    writing: ' + outfile + '. ' + systime()

close,  1
openw,  1, outfile
writeu, 1, magic_number, ny, nx, data_type, byte_size
writeu, 1, variable
close,  1

if printflag then print, 'Finished writing: ' + outfile + '. ' + systime()


end
