;-----------------------------------------------------
;+
; NAME: ANS_PSAR2SIO
; PURPOSE:
;   To convert ALOS PALSAR data to SIO files (and Level-1 and KMZ if selected)
; CALLING SEQUENCE
;   ans_psar2sio, deffile, sio_root
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
;                sio_kmz_dir, twobyte, psar_dir,deg, ave_line, and ave_pixel, log_dir,
;                silent
; OUTPUT
;   sio_root=    Root(s) of SIO data file names
;   SIO files
; KEYWORDS:
;   /quiet       No screen output
;   /help        Get help
; MODIFICATION HISTORY:
;   2011 Jan 01 Frank Monaldo Change the architecture to separate out SIO generation
;                              by satellite
;   2011 Jan 07 Frank Monaldo. Add ability to read ALOS provided renaming convention.
;   2011 Feb 05 Frank Monlado. Add jsstart
;   2011 Feb 08 Frank Monaldo. Return error=-1 if cannot read data
;   2011 Feb 22 Frank Monaldo. Change .cdf extenson to .nc
;   2015 May 19 Chris Jackson. Fix typos in the description / help
; Copyright (C) 2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_psar2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
sio_root=''

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_PSAR2SIO
  print, ' PURPOSE:
  print, '   To convert ALOS PALSAR data to SIO files (and Level-1 and KMZ if selected)
  print, ' CALLING SEQUENCE
  print, '   ans_psar2sio, deffile, sio_root
  print, ' INPUT
  print, '   deffile=     Full path to defaults file. This option will read a specified file
  print, '                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
  print, '                sio_kmz_dir, twobyte, psar_dir,deg, ave_line, and ave_pixel, log_dir,
  print, '                silent
  print, ' OUTPUT
  print, '   sio_root=    Root(s) of SIO data file names
  print, '   SIO files
  print, ' KEYWORDS:
  print, '   /quiet       No screen output
  print, '   /help        Get help
  return
endif
!quiet=  1 ; set idl to quiet

;  
;---------------------------------------------------------------
;------------------- Start ALOS PALSAR Conversio to SIO ---------------
;---------------------------------------------------------------
;
;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))

;
; Read defaults file
;

ans_read_defaults, deffile, df, quiet=quiet
if tag_test(df, 'silent') then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 0

;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory, count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir
res= file_search( df.sio_kmz_dir,      /test_directory, count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir  
res= file_search( df.log_dir, /test_directory,    count=count)
if count eq 0 then file_mkdir, df.log_dir
;
; Look for ALOS PALSAR directories in base directory
;
ff1=  file_search( df.psar_dir +'/*L',    count=ct1)
ff2=  file_search( df.psar_dir +'/LED*',  count=ct2)

if ct1 eq 0 and ct2 eq 0 then begin
  print, 'Cannot find files that match ' +  df.psar_dir +'/*L or 
  print, '                              ' +  df.psar_dir +'/LED*'
  print, 'Exiting ans_psar2sio'
  sio_root=''
  return
endif
if ct1 gt 0 and ct2 eq 0 then ff= ff1
if ct1 gt 0 and ct2 gt 0 then ff= [ ff1, ff2 ]
if ct1 eq 0 and ct2 gt 0 then ff= ff2
;
; Computing SIO files for ALOS PALSAR data
;
if printflag then print, 'Computing ALOS PALSAR Level-1 SIO data...

filebreak,    ff[0],directory=leaderdir, file=leaderfile
len=             strlen(leaderfile)
found=           0
name_convention= 0
if strupcase( strmid(leaderfile, len-1, 1) ) eq 'L'   then name_convention=1
if strupcase( strmid(leaderfile,     0, 3) ) eq 'LED' then name_convention=2
if name_convention eq 0 then begin
  print, 'Does not correspond to nameing convention', ff[0]
  print, 'Exiting ans_psar2sio'
  return
end

case name_convention of
1: datafile=  leaderdir + stress(leaderfile, 'R', 0, '.L', '.D', ned)
2: begin
     len=       strlen(leaderfile)
     root=      strmid(leaderfile,3, len-3)
     dfile=     file_search(leaderdir +'IMG*' + root, count=count)
     if count eq 0 then begin
       print, 'Cannot fine corresponding image file ' + leaderdir +'IMG*' + root
       print, 'Exiting ans_pat2sio'
     endif
     datafile=  dfile[0]
   end
endcase

ff=           file_search(datafile, count=count)

if count eq 0 then begin
  print, 'Cannot find datafile: ' + datafile
  print, 'Exiting ans_psar2sio'
  return
end

leaderfile=    leaderdir + leaderfile
error=1
ans_read_psar, datafile, leaderfile, deffile,      $
               outputroot  = outputroot         ,  $
               jsstart     = jsstart            ,  $
               error       = error              ,  $
               quiet       = quiet
if error lt 0 then begin
  print, 'Error reading either '
  print,  datafile + ' or '
  print,  leaderfile 
  error=  -1
  return
endif
sio_root=      outputroot

;
; Loop through SIO files for full processing
;
n= n_elements(sio_root)


for i=0, n-1 do begin
  ;
  ; Generating file names
  ;
  root=                getwrd( sio_root[i], 0, /last, del='/' )
  datafile=            sio_root[i]              + '_image.dat'
  infofile=            sio_root[i]              + '_ancil.txt'
  landfile=            sio_root[i]              + '_mask.dat'
  kmzfile=             df.sio_kmz_dir      + root + '_sio.kmz'
  netcdffile_level_1=  df.ncdf1_output_dir       + root + '_sar_level1.nc'
  ;
  ; Computing Level 1 NetCDF files
  ;
  logfile= df.log_dir + '/' + root + '.log'
  if strupcase(df.level1cdf) eq 'Y'  then begin
    if printflag then print, 'Computing ALOS PALSAR Level-1 NetCDF data...   
    ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet
    time1= dt_tm_tojs(systime(/utc))
    outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' +  netcdffile_level_1
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
    print, outstr
  endif
  
  ;
  ; SIO to KMZ
  ;
  if df.siokmzfile eq 'Y' then begin
    if printflag then begin
       print, ''
       print, 'Computing KMZ files.'
    endif
    ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
                 /nrcs, quiet=quiet, working_dir=df.working_directory
    time1= dt_tm_tojs(systime(/utc))
    outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
    print, outstr
  endif
endfor

;---------------------------------------------------------------
;------------ End ALOS PALSAR conversion to SIO  ---------------
;---------------------------------------------------------------

end
