;-----------------------------------------------------
;+
; NAME: ANS_RCM2SIO
; PURPOSE:
;   To convert RCM data to SIO files (and Level-1 and KMZ if selected)
; CALLING SEQUENCE
;  ans_rcm2sio, deffile, sio_root
; INPUT
;   deffile    =  Full path to defaults file. This option will read a specified file
;                 for the parameters: already_processsed, ncdf1_output_dir, working_directory,
;                 sio_kmz_dir, twobyte, rcm2_dir,deg, ave_line, and ave_pixel, log_dir,
;                 silent
; OUTPUT
;   sio_root   =  Root(s) of SIO data file names
; KEYWORDS:
;   /quiet        No screen output
;   /help         Get help
;   error         <0 if bad
;                 -1 No RCM data available in diretory
;                 -2 Cannot read RCM data
; MODIFICATION HISTORY:
;   2018 Jul 25 Frank Monaldo. Creation
;   2020 Feb 13 Frank Monaldo. Correct remarks
;   2020 Apr 02 Chris Jackson. Process the newest files first
;   2021 Apr 02 Chris Jackson. Add the creation/deletion of the SAFE lock file.  Move to "already_processed" relocated from ans_read_rcm.pro
; Copyright (C) 2010-2018, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_rcm2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

  !quiet=1
  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet=  printflag eq 0
  sio_root=0

  if keyword_set(help)  or (n_params(0) lt 1 ) then begin
    print, ' NAME: ANS_RCM2SIO
    print, ' PURPOSE:
    print, '   To convert RCM data to SIO files (and Level-1 and KMZ if selected)
    print, ' CALLING SEQUENCE
    print, '  ans_rcm2sio, deffile, sio_root
    print, ' INPUT
    print, '   deffile    =  Full path to defaults file. This option will read a specified file
    print, '                 for the parameters: already_processsed, ncdf1_output_dir, working_directory,
    print, '                 sio_kmz_dir, twobyte, rcm2_dir,deg, ave_line, and ave_pixel, log_dir,
    print, '                 silent
    print, ' OUTPUT
    print, '   sio_root   =  Root(s) of SIO data file names
    print, ' KEYWORDS:
    print, '   /quiet        No screen output
    print, '   /help         Get help
    print, '   error         <0 if bad
    print, '                 -1 No RCM data available in diretory
    print, '                 -2 Cannot read RCM data
    errpr = -1
    return
  endif

  ;
  ; Time to start processing
  ;

  jsstart=  dt_tm_tojs(systime(/utc))
  ;
  ; Read defaults file
  ;
  if printflag then begin
    print, ' '
    print, 'Begin converting RCM to SIO files. ' + systime(/utc)
    print, ' '
  endif

  ans_read_defaults, deffile, df, quiet=quiet, error=error
  if error le 0 then begin
    if printflag then begin
      print, 'Cannot read defaults file: ' + deffile
      print, 'Exiting ans_rcm2sio'
    endif
    error = -2
    return
  endif

  if tag_test(df, 'silent') then begin
    if df.silent eq 'Y' then printflag=0
    if df.silent eq 'N' then printflag=1
  endif
  quiet= printflag eq 0
  ;
  ;
  ; Create directories if they do not exist
  ;
  res= file_search( df.already_processed, /test_directory, count=count)
  if count eq 0 then file_mkdir, df.already_processed
  res= file_search( df.sio_output_dir,   /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.sio_output_dir
  res= file_search( df.log_dir,          /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.log_dir
  ;
  ;
  ;---------------------------------------------------------------
  ;------------------- Start RCM Conversion ---------------
  ;---------------------------------------------------------------
  ;
  ; Look for RCM sirectories in base directory
  ;
  ff= file_search( df.rcm_dir +'/R*', /test_directory, count=count) & cc=count
  if cc eq 0 then begin
    if printflag then print, 'Cannot find files in ' +  df.rcm_dir +'/R*'
    if printflag then print, 'Exiting ans_rcm2sio'
    sio_root=''
    error = -1
    return
  endif

  if count gt 1 then begin
    ;
    ; Sort to find the newest file
    ; Filenames like RCM2_OK1436518_PK1541186_1_SC50MB_20210402_212434_HH_HV_GRD.zip
    ; Default time to a large negative numer
    ;
    jsdir = dblarr(count) - 1e-45
    for i=0, count-1L do begin
      rcmdir  =  getwrd( ff[i], del='/', /last )
      tmstr   =  rcmdir.split('_')
      year    =  strmid( tmstr[5],  0, 4  )
      month   =  strmid( tmstr[5],  4, 2  )
      day     =  strmid( tmstr[5],  6, 2  )
      hour    =  strmid( tmstr[6],  0, 2  )
      minute  =  strmid( tmstr[6],  2, 2  )
      second  =  strmid( tmstr[6],  4, 2  )
      if isnumber( year   ) and $
        isnumber( month  ) and $
        isnumber( day    ) and $
        isnumber( hour   ) and $
        isnumber( minute ) and $
        isnumber( second ) then begin
        if long( month  ) gt  0 and $
          long( month  ) lt 13 and $
          long( day    ) gt  0 and $
          long( day    ) lt 32 and $
          long( hour   ) ge  0 and $
          long( hour   ) le 23 and $
          long( minute ) ge  0 and $
          long( minute ) le 59 and $
          long( second ) ge  0 and $
          long( second ) le 59 then begin
          inputstr = year + ' ' + month2name( long(month) ) + ' ' + day + ' '
          inputstr = inputstr + hour + ':' + minute + ':' + second
          jsdir[i] = dt_tm_tojs(inputstr)
          ;if printflag then print, inputstr, jsdir[i]
        endif ; Check for valid values
      endif   ; End test all elements begin numbers
    endfor     ; Looping through directories
    ind  = reverse( sort( jsdir) )
    ff  =  ff[ind]
  endif

  ;Look through the files and find the first unlocked file
  nfiles=n_elements(ff);
  lock_count = 0
  sio_root=''

  for i = 0,nfiles-1 do begin
    lock_file = ff[i]+'_lock'
    if (file_test(lock_file,/regular) eq 1 ) then begin
      lock_count++
      continue;
    endif

    ;Create lockfile
    close, 1 & openw, 1, lock_file  & printf, 1, '' & close, 1
    if printflag then print, 'Selected directory: ' + ff[i]

    ;
    ; Computing SIO files for all the RCM data
    ;
    if printflag then print, 'Converting RCM files to Level-1 SIO data...
    error=1
    ans_read_rcm,   ff[i],    deffile,           $
      outputroot =  outputroot  ,  $
      jsstart    =  jsstart     ,  $
      error      =  error       ,  $
      quiet      =  quiet       ,  $
      help       =  help

    if error lt 0 then begin
      if printflag then print, 'Error in ans_read_rcm'
      error = -2
      return
    endif

    sio_root=       outputroot
    ;Only process the first unlock file found
    break
  end
 
  if (lock_count eq nfiles) then begin
    print, 'There are no unlocked SAFE files in directory:'
    print, df.rcm_dir
    error = -1
    return
  endif

  ;
  ; Loop through SIO files for full processing
  ;
  w= where( strlen(sio_root) gt 0 , n)

  for k=0, n-1 do begin
    ;
    ; Generating file names
    ;
    root=                getwrd( sio_root[k], 0, /last, del='/' )
    datafile=            df.sio_output_dir   + '/' + root + '_image.dat'
    infofile=            df.sio_output_dir   + '/' + root + '_ancil.txt'
    landfile=            df.sio_output_dir   + '/' + root + '_mask.dat'
    kmzfile=             df.sio_kmz_dir      + '/' + root + '_sio.kmz'
    netcdffile_level_1=  df.ncdf1_output_dir + '/' + root + '_sar_level1.nc'
    ;
    ; Computing Level 1 NetCDF files
    ;
    logfile=  df.log_dir + '/' +root + '.log'
    if strupcase(df.level1cdf) eq 'Y'  then begin
      if printflag then print, 'Computing RCM Level-1 NetCDF data...
      ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
    ;
    ; SIO to KMZ
    ;
    if df.siokmzfile eq 'Y' then begin
      if printflag then begin
        print, ''
        print, 'Computing KMZ files.'
      endif
      ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
        /nrcs, quiet=quiet, working_dir=df.working_directory
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
  endfor

  ; Move files to already processed directory if move2dir set
  ;
  if printflag then begin
    print, 'Moving ' + ff[i]
    print, 'to     ' + df.already_processed + '/'
  endif
  file_move,   ff[i], df.already_processed  + '/', /overwrite, /ALLOW_SAME, /verbose
  ;

  ;Remove the SAFE lockfile
  if printflag then  print, 'Deleting Lockfile ' + lock_file
  file_delete, lock_file,/ALLOW_NONEXISTENT, /verbose


  ;---------------------------------------------------------------
  ;--------------------- End conversion to SIO processing ---------------
  ;---------------------------------------------------------------

end
