;+---------------------------
; NAME: ANS_READ_ASF
; PURPOSE:
;   The purpose of this program is to read Radarsat-1/ERS-2 data in ASF format and
;   create a SIO formatted binary image file, ASCII file
; CALLING SEQUENCE:
;    ans_read_asf, datafile, leaderfile, deffile
; INPUT:
;   datafile=      CEOS file with SAR data
;   leaderfile=    CEOS file if ancillary data information
;   deffile=       Defaults file
; OUTPUT:          
;   Binary files with image and land mask data and an ASCII file with ancillary data. The 
;   convention for file names is of the form:
;       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat      (NRCS       aray)
;       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt      (ancillary data file)
;   The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
;   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;   files include the SIO 20-byte header.
; KEYWORDS:
;   error=          error (0 if program completes or -1 if not)
;   jsstart=        Start of processing in julian secoons
;   outputroot      Output root SIO file name
;   /quiet=         Do not print intermediate results.
;   /help=          Print out help
; MODIFICATION HISTORY
;  2010 Mar 14 Frank Monaldo. Re-write of ans_read_asf_rsat.pro. 
;  2010 Mar 19 Frank Monaldo. Add ability to deal with negative js values
;  2010 May 30 Frank Monaldo. Add move2dir and outputroot
;  2010 Jun 03 Frank Monaldo. Set noise floor to zero for Radarsat-1
;  2010 Jun 05 Frank Monaldo. Correct spelling of lf.projection
;  2010 Jul 08 Frank Monaldo. Add ability to flip images from particular files.
;  2010 Jul 15 Frank Monaldo. Add facility for range bering correction of image.
;  2010 Jul 22 Frank Monaldo. Improve print outs
;                             Calculate stats on first 1/4 of the image to save time
;  2010 Dec 28 Frank Monaldo. Use tag_test for shift_file
;  2010 Dec 29 Frank Monaldo. Add _C to output file name
;  2010 Dec 31 Frank Monaldo. Add log directory
;  2011 Jan 20 Frank Monaldo. Correct spelling of log_dir
;  2011 Feb 04 Frank Monaldo. Add jsstart to lf structure
;  2011 Feb 05 Frank Monaldo. Added exit for bad leader file
;  2011 Feb 07 Frank Monaldo. Add original final name to start time print out
;                             to log file.
;  2011 Feb 13 Frank Monaldo. Changed unique name of directory to make more sense
;  2011 Feb 14 Frank Monaldo. Add check of .lock file
;  2011 Jun 21 Frank Monaldo. Corrected string conversion printing error
;  2012 Jul 12 Frank Monaldo. Remove hard wired directories
;  2012 Aug 21 Frank Monaldo. Move ans_gen_version program
;  2012 Dec 31 Frank Monaldo. Add ability to add ice mask and ability to 
;                             defaults file direction
;  2013 Apr 22 Frank Monaldo. Add the ability to generate and latitude, longitude, incidence, and 
;                             radar look angle SIO files
;  2013 May 04 Frank Monaldo. Correct error in check for df.twobyte
;  2013 Aug 18 Frank Monaldo. Move ability to generate and latitude, longitude, incidence, and 
;                             radar look angle SIO files to ans_sat2sio.  Remove deletion of 
;                             lock file
;  2013 Aug 20 Frank Monaldo. Remove generation of land mask.  The land mask is now computed
;                             ans_sat2sio program based on the defaults and ancillary data files.
;                             Removed /nomask and header options
;  2016 Jun 14 Chris Jackson. Made changes to properly name file when the satellite is ERS-1  
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;
 
pro ans_read_asf, datafile, leaderfile, deffile    , $
                  quiet          = quiet           , $
                  help           = help            , $
                  outputroot     = outputroot      , $
                  jsstart        = jsstart         , $
                  error          =  error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 )  then begin
  print, ' NAME: ANS_READ_ASF
  print, ' PURPOSE:
  print, '   The purpose of this program is to read Radarsat-1/ERS-2 data in ASF format and
  print, '   create a SIO formatted binary image file, ASCII file
  print, ' CALLING SEQUENCE:
  print, '    ans_read_asf, datafile, leaderfile, deffile
  print, ' INPUT:
  print, '   datafile=      CEOS file with SAR data
  print, '   leaderfile=    CEOS file if ancillary data information
  print, '   deffile=       Defaults file
  print, ' OUTPUT:          
  print, '   Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '   convention for file names is of the form:
  print, '       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat      (NRCS       aray)
  print, '       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt      (ancillary data file)
  print, '   The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
  print, '   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '   files include the SIO 20-byte header.
  print, ' KEYWORDS:
  print, '   error=          error (0 if program completes or -1 if not)
  print, '   jsstart=        Start of processing in julian secoons
  print, '   outputroot      Output root SIO file name
  print, '   /quiet=         Do not print intermediate results.
  print, '   /help=          Print out help
  error=  -1
  return
endif

!quiet=1 
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 


time0 = dt_tm_tojs(systime())
;
; Parcing input parameters
;

if not file_test(deffile)  then begin
  error= -1
  if printflag then begin
    print, 'Cannot find the defaults file: ' + deffile
    print, 'Existing ans_read_asf'
  endif
  return
endif

ans_read_defaults, deffile, df, /quiet

if tag_test(df, 'silent')  then begin
  if df.silent eq 'Y' then printflag=0
  quiet= 0
  quiet= printflag eq 0
endif

   

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet

print, answrs_version
   
;
; Determine if input files exist
; 

if not file_test(datafile) then begin
  print, 'Could not find ASF Radarsat-1/ERS-1/2 data file: ' + datafile
  print, 'Exiting ans_read_asf.pro '
  error= -1
  return
endif

if not file_test(leaderfile) then begin
  print, 'Could not find ASF Radarsat-1/ERS-1/2 leader file: ' + leaderfile
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif

;
; Create a new working directory
;
working_directory= df.working_directory + '/asf_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory

;
; Copy Radarsat-1 files to working directory.   Note the movement of Radarsat-1 directory to 
; and already processed directory must occur at a different level. It is not done in this program
;
if printflag then begin
  print, 'Copying ' + leaderfile
  print, 'and     ' + datafile
  print, 'to      ' + working_directory
endif
file_copy, leaderfile, working_directory  + '/'
file_copy, datafile,   working_directory  + '/'
;
; Moving files to appropriate directories if df.already_processed set
;

res= file_search(df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
if printflag then begin
  print, 'Moving  ' + leaderfile
  print, 'and     ' + datafile
  print, 'to      ' + df.already_processed
endif 
file_move, leaderfile ,  df.already_processed  + '/' , /overwrite
file_move, datafile   ,  df.already_processed  + '/' , /overwrite


sleaderfile=    getwrd(leaderfile, 0, /last, del='/')
sdatafile=      getwrd(datafile, 0, /last, del='/')


asf_read_leader,  working_directory  + '/' + sleaderfile,  good_leader, lf, quiet=quiet

if good_leader lt 0 then begin
  error= -1
  return
endif

lf.name_dir          =  sdatafile
lf.ave_pixel         =  df.ave_pixel
lf.ave_line          =  df.ave_line
lf.deg               =  df.deg
lf.nx1               =  lf.nx0 / lf.ave_pixel
lf.ny1               =  lf.ny0 / lf.ave_line
lf.working_directory =  working_directory
; Add to struture
temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
lf=      0
lf=      temp_lf


asf_create_outloc, lf, position_loc=df.position_loc,  quiet=quiet

;
; Note the calibration is done w/in this program.
;
ans_read_asf_raw, working_directory  + '/' + sdatafile, image, lf.nx0, lf.ny0, quiet=quiet, error=error

if n_elements(error) gt 0 then begin
  if error lt 0 then begin
    error=-1 
    return
  endif
endif
    
;
; Reading flipped file list
;

flip_file= 0  ; do not flip image, 1= flip image
if not file_test(df.flipped_file_list)  then begin
  ss=        getfile(flipped_file_list)
  ns=        n_elements(ss)
  shortfile= getwrd(datafile, 0, /last, del='/')
  length=    strlen(shortfile)
  shortfile= strmid(shortfile, 0, length-2)
  for i=0L, ns - 1 do begin
    if shortfile eq strcompress(ss[i]) then begin
      flip_file=1
      if printflag then print, shortfile + ' matches a file in the flipped list.'
    endif
  endfor
endif

if flip_file then begin
  if printflag then print, 'Flipping image.'
  image= reverse(image,2)
endif

;
; promote to float
;
image= float(image)

adj        = 1
nrcs_slope = df.rsat1_asf_slope
nrcs_bias  = df.rsat1_asf_bias
if lf.sar_sensor eq 'ERS-2' then begin
  date=    ydn2date(lf.data_year, lf.dn, format='Y$ n$ d$') + ' ' + $
           string(lf.hour,   format='(I2.2)') + ':' +  $
           string(lf.minute, format='(I2.2)') + ':' +  $
           string(lf.second, format='(I2.2)')
  js2=     dt_tm_tojs(date)
  js1=     dt_tm_tojs('1995 Apr 21')
  ;
  ;        Launch  - 1/ 1/2001   y = -0.0018x   + 0.0023
  ;        1/ 2/2001 - 2/26/2003 y = -0.002247x + 0.9296
  ;        2/27/2003 - current   y = -0.0018x   - 0.3323 + 3
  ;        where:
  ;        x = days since launch
  ;        y = gain adjustment (in db)
  ;
  jsa=     dt_tm_tojs('2001 Jan 01 00:00:00')
  jsb=     dt_tm_tojs('2003 Feb 26 00:00:00')
  slope=   -0.002247 & bias=  0.0023
  if  (js2 gt jsa) and (js2 lt jsb) then begin
    slope=   -0.0018 & bias=   0.9296
  endif
  if  (js2 ge jsb) then begin
    slope=   -0.0018 & bias=   3 - 0.3323 
  endif
  x=       (js2-js1) /86400. ; days 
  y=       slope * x + bias
  adj=     1.0
  adj=     float(10^(-y/10.0))
  nrcs_slope=  df.ers2_asf_slope
  nrcs_bias=   df.ers2_asf_bias  
endif


if printflag then begin
  print, 'Calibrating the image using ASF data in the leader file...'
  print, 'Stats before calibration.'
  hlp, image
  print, 'lf.a1=      ' + string(lf.a1)
  print, 'lf.a2=      ' + string(lf.a2)
  print, 'lf.a3=      ' + string(lf.a3)
  print, 'adj=        ' + string(adj)
  print, 'nrcs_slope= ' + string(nrcs_slope)
  print, 'nrcs_bias=  ' + string(nrcs_bias)
endif

nnoise=  interpol(lf.noise, lf.nx0)
image=   image * image
;
; Set noise to zero if not ERS-2, i.e. Radarsat-1
;
if strcmp(lf.sar_sensor,'RADARSAT',8,/FOLD_CASE) then lf.a1=0

for i=0L, lf.ny0-1L do begin
  image[*,i]= image[*,i] - float(lf.a1) * float(nnoise)
endfor
 
image= adj * float(lf.a2) * image + float(lf.a3) 

;
; Apply correction scaling if the slope ne 1 and bias ne 0
; This correction is read from the defaults file and used
; to tune the calibration.
;
if nrcs_slope ne 1.00 or nrcs_bias ne 0.00 then begin
  image= image * nrcs_slope + nrcs_bias  
  lf.nrcs_slope= nrcs_slope
  lf.nrcs_bias=  nrcs_bias
endif

if printflag then begin
  print, 'Stats after calibration.'
  hlp, image
endif


if df.ave_line ne 1 or df.ave_pixel ne 1 then begin
  ;
  ; Average down
  ;
  out= 'Averaging image from '
  out= out +          string(lf.nx0, format='(I5)') + ' x '  + string(lf.ny0, format='(I5)') 
  out= out + ' to ' + string(lf.nx1, format='(I5)') + ' x '  + string(lf.ny1, format='(I5)') 
  out= strcompress(out)
  if printflag then print, out
  image= smooth( image, [ave_pixel, ave_line])
  image= congrid( image, lf.nx1, lf.ny1)
endif
;
; Reorient if necessary
;
if printflag then begin
  help, lf.scansar
  help, lf.projection
endif
if lf.projection eq 'GROUNDRANGE' then lf.projection = 'GROUND RANGE'

if lf.scansar eq 1 then begin
  if printflag then print, 'Data in SCANSAR nmode so flipping.'
  image=reverse(image,2)		
endif

;
; Remove negative values.
;
w        = where(image lt 0, /null)
image[w] = 0
if printflag then begin
  print, 'a1, a2, a3 coef=', lf.a1, lf.a2, lf.a3
  print, 'Noise=', mean(nnoise)
  print, 'Adjustment=', adj
  ni=  n_elements(image)
  print, 'Mean NRCS image:' + strcompress( string(mean(image[0:ni/4]) )  )
  print, 'SD   NRCS image:' + strcompress( string(stdev(image[0:ni/4]))  )
endif


;
; Added to adjust geo location
;
range_shift=  0.0
bering_shift= 0.0

temp_lf= create_struct(lf, 'range_shift',  range_shift, $
                           'bering_shift', bering_shift )
lf=      0
lf=      temp_lf

if tag_test(df,'shift_file') then begin
  res=file_search( df.shift_file, count=count)
  if count gt 0 then begin
    s=getfile(df.shift_file)
    n= n_elements(s)
    for i=0L, n-1L do begin
       ff= getwrd(s[i], 0)
       if ff eq lf.name_dir then begin
         if printflag then print, s[i]
         lo1= float(getwrd(s[i], 3))
         la1= float(getwrd(s[i], 4))
         lo2= float(getwrd(s[i], 5))
         la2= float(getwrd(s[i], 6))
         ll2rb, lo1, la1, lo2, la2, r, b
         lf.range_shift=  r
         lf.bering_shift= b
         if printflag then begin
           print, 'Range shift=',  lf.range_shift
           print, 'Bering shift=', lf.bering_shift
         endif   
       endif    ; End check if file name found
    endfor     ; End loop lines of shift file
  endif        ; End check if shift file found
endif          ; End check if shift file specified


asf_lonlat2pix_fit, lf, quiet=quiet
asf_incid2pix_fit,  lf, quiet=quiet
asf_r_look,         lf, quiet=quiet
;
;
; Create file name root for output files.
;

lf.month = strupcase(lf.month)
case strupcase(lf.month) of
'JAN': month=1
'FEB': month=2
'MAR': month=3
'APR': month=4
'MAY': month=5
'JUN': month=6
'JUL': month=7
'AUG': month=8
'SEP': month=9
'OCT': month=10
'NOV': month=11
'DEC': month=12
ELSE:  month=1
endcase
;
;
; compute center geographic position
;


center_lat= string(abs(lf.center_lat), format='(F6.2)')
if lf.center_lat ge 0 then begin
  center_lat= center_lat + 'N'
endif else begin
  center_lat= center_lat + 'S'
endelse
center_lon= string(abs(lf.center_lon), format='(F6.2)')
if lf.center_lon ge 0 then begin
  center_lon= center_lon + 'E'
endif else begin
  center_lon= center_lon + 'W'
endelse

timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
         lf.month                               + ' ' + $
         string( lf.day,       format='(I2.2)') + ' ' + $
         string( lf.hour,      format='(I2.2)') + ':' + $
         string( lf.minute,    format='(I2.2)') + ':' + $
         string( lf.second,    format='(I2.2)')
js=       dt_tm_tojs(timestr)
if js ge 0 then begin
  js=            string( dt_tm_tojs(timestr), format='(I10.10)' ) 
endif else begin
  js=   '-' + string( abs(dt_tm_tojs(timestr)), format='(I9.9)' )
endelse
output_prefix= 'RSAT1'
if lf.sar_sensor eq 'ERS-1' then output_prefix='ERS1'
if lf.sar_sensor eq 'ERS-2' then output_prefix='ERS2'
output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + string( center_lon)
output_prefix= output_prefix + '_' + string( center_lat)
output_prefix= strcompress( output_prefix, /remove_all )

output_prefix2= output_prefix + '_' + strupcase(lf.polarization) + '_C'
; 
; Outputroot set
;
outputroot= df.sio_output_dir + '/'+ output_prefix2
outputroot= stress(  outputroot    , 'R', 0, '//' , '/', ned )
;
; Check and see if a lock file exits
;
lockfile=   outputroot + '*.lock'
;
res= file_search( lockfile, count=count)
if count gt 0 then begin
   print, 'Lockfile:  ' + lockfile + ' exists'
   print, 'Exiting ans_read_asf'
   error=-1
   return
endif

;
; Create lock file
;
lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '.lock'
lockfile= stress(  lockfile    , 'R', 0, '//' , '/', ned )
close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1

;
; Write out to SIO data file format. This is general for all.
; SAR data
;
scaling= 1.0
res= file_search(df.log_dir, count=count)
if count eq 0 then file_mkdir, df.log_dir
logfile= df.log_dir + '/' + output_prefix2 + '.log'

outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + lf.name_dir +' Processing starts'
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr

imageoutfile= df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
imageoutfile= stress(  imageoutfile , 'R', 0, '//' , '/', ned )
if df.twobyte eq 'Y' then begin
  ans_write_nrcs_sio, image, imageoutfile, scaling, /twobyte, quiet=quiet
endif else begin
  ans_write_nrcs_sio, image, imageoutfile, scaling, quiet=quiet
endelse
time4= dt_tm_tojs(systime(/utc))
outstr=    string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr

;
; Write out SIO ancillary file format. This program is explicitly written for ASF data
;
anciloutfile= df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
anciloutfile= stress(  anciloutfile , 'R', 0, '//' , '/', ned )
asf_write_ancil_sio, anciloutfile, lf, scaling, quiet=quiet
time5= dt_tm_tojs(systime(/utc))
close, 1
openw, 1, logfile, /append
outstr=  string(time5, format='(I10)')+ ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+  output_prefix2 + '_ancil.txt'
printf, 1, outstr
close, 1
print, outstr



if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive


end
