;+---------------------------
; NAME: ANS_READ_ASF_RSAT
; PURPOSE:
;   The purpose of this program is to read Radarsat-1/ERS-2 data in ASF format and
;   create a SIO formatted binary image file, ASCII file, and
;   binary land mask
; CALLING SEQUENCE:
;    ans_read_asf_rsat, datafile, leaderfile, output_dir, working_dir
; INPUT:
;   datafile=      CEOS file with SAR data
;   leaderfie=     CEOS file if ancillary data information
;   output_dir=    Directory where output files are placed
;   working_dir=   Working data were temporary files are place. These temporary
;                  files will be deleted at the completion of the program
; OUTPUT:          
;   Binary files with image and land mask data and an ASCII file with ancillary data. The 
;   convention for file names is of the form:
;       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
;   The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
;   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;   files include the SIO 20-byte header
; KEYWORDS:
;   /quiet=        Do not print intermediate results.
;   /help=         Print out help
;   /twobyte=      By default the data are stored floating point NRCS. The switch
;                  will convert the data to unsigned two-byte integers. The scaling will
;                  then be included in the ASCII output file.
;   deg=           degree of polynomial fit between index, long/lat,default=2
;   ave_line=      ave_line   ; Number of lines to average (default=1)
;   ave_pixel=     ave_pixel  ; Number of pixel to average (default=1)
;   deffile=       defaults file. This option will read a specified file
;                  for the parameters /twobyte, deg, ave_line, and ave_pixel
;                  If the parameters are explicitly set. For example, ave_pixel=0 then
;                  it will over-ride what ever is in the defaults file.
;   error=         error (0 if program completes or -1 if not)
;   gshhs_dir:     gshhs environmental variable pointing to the GSHHS data
;   asf_import:    Full path the asf_import executable
;   position_loc:  Full path to position_locator executable
;   /header:       Only save ancillary data
;   /nomask:       Suppress outpout of land mask file
;   pixel_size:   The horizontal dimensions of the mask array will be scaled so
;                  that the size of the mask will be scaled down in integer increments
;                  such that the mask pixels <= pixel size in meters. If not set then
;                  no scaling is performed.
; MODIFICATION HISTORY
;  2009 Dec 08 Frank Monaldo.
;  2009 Dec 16 Frank Monaldo. Corrected misspelling in gshhs_dir
;  2010 Jan 01 Frank Monaldo. Corrected misspelling in asf_incid2pix_fit call.
;              corrected time print out.
;  2010 Jan 02 Frank Monaldo. Corrected output prefix from RSAT to RSAT1 or ERS2
;  2010 Jan 05 Frank Monaldo. Change to ans_compute_fit / ans_poly_cal for fits
;  2010 Jan 15 Frank Monaldo. setenv gshhs_dir
;  2010 Jan 19 Frank Monaldo. Add /header option to only save ancillary data
;                             Add /nomask option to suppress land mask file output
;  2010 Jan 28 Frank Monaldo. Add option for creating ANSWRS version. This
;                             require the idlans directory name
;  2010 Feb 05 Frank Monaldo. Usd the help call to find the source file directory
;                             This is used to calculated the time of last modifications.
;                             This removes need for providing the name of source directory.
;                             Typing command w/no parameters also calls help. 
;  2011 Feb 14 Frank Monaldo. Add escape if lock file exits.
;  2012 Jul 05 Frank Monaldo. Eliminate hardwired directories.
;  2013 Apr 22 Frank Monaldo. Add ability create an ice mask and longitude, latitude,
;                             incidence and radar look angle. 
; Copyright (C) 1999-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;

pro ans_read_asf_rsat, datafile, leaderfile, output_dir, working_dir, $
                       quiet=         quiet,         $
                       help=          help,          $
                       twobyte=       twobyte,       $
                       deg=           deg,           $
                       ave_line=      ave_line,      $
                       ave_pixel=     ave_pixel,     $
                       deffile=       deffile,       $
                       gshhs_dir=     gshhs_dir,     $
                       asf_import=    asf_import,    $
                       position_loc=  position_loc,  $
                       header=        header,        $
                       nomask=        nomask,        $ 
                       pixel_size=    pixel_size,    $
                       error=         error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 )  then begin
  print, ' NAME: ANS_READ_ASF_RSAT
  print, '   The purpose of this program is to read Radarsat-1/ER-2 data in ASF format and
  print, '   create a SIO formatted binary image file, ASCII file, and
  print, '   binary land mask
  print, ' CALLING SEQUENCE:
  print, '    ans_read_asf_rsat, datafile, leaderfile, output_dir, working_dir
  print, ' INPUT:
  print, '   datafile=      CEOS file with SAR data
  print, '   leaderfie=     CEOS file if ancillary data information
  print, '   output_dir=    Directory where output files are placed
  print, '   working_dir=   Working data were temporary files are place. These temporary
  print, '                  files will be deleted at the completion of the program
  print, ' OUTPUT:          
  print, '   Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '   convention for file names is of the form:
  print, '       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '       RSAT1_ASF_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
  print, '   The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
  print, '   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '   files include the SIO 20-byte header/
  print, ' KEYWORDS:
  print, '   /quiet=        Do not print intermediate results.
  print, '   /help=         Print out help
  print, '   /twobyte=      By default the data are stored floating point NRCS. The switch
  print, '                  will convert the data to unsigned two-byte integers. The scaling will
  print, '                  then be included in the ASCII output file.
  print, '   deg=           degree of polynomial fit between index, long/lat,default=2
  print, '   ave_line=      ave_line   ; Number of lines to average (default=1)
  print, '   ave_pixel=     ave_pixel  ; Number of pixel to average (default=1)
  print, '   deffile=       defaults file. This option will read a specified file
  print, '                  for the parameters /twobyte, deg, ave_line, and ave_pixel
  print, '                  If the parameters are explicitly set. For example, ave_pixel=0 then
  print, '                  it will over-ride what ever is in the defaults file.
  print, '   error=         error (0 if program completes or -1 if not)
  print, '   gshhs_dir:     gshhs environmental variable pointing to the GSHHS data
  print, '   asf_import:    Full path the asf_import executable
  print, '   position_loc:  Full path to position_locator executable
  print, '   /header:       Only save ancillary data
  print, '   /nomask:       Suppress outpout of land mask file
  print, '   pixel_size:    The horizontal dimensions of the mask array will be scaled so
  print, '                  that the size of the mask will be scaled down in integer increments
  print, '                  such that the mask pixels <= pixel size in meters. If not set then
  print, '                  no scaling is performed.
  error= -1
  return
endif


printflag=1
if keyword_set(quiet) then printflag=0
help, calls=selfhelp
ss=   getwrd(selfhelp[0],1)              
ss=   stress(ss, 'D', 0, '<', '', ned)    
ss=   stress(ss, 'D', 0, '>', '', ned)    
ss=   stress(ss, 'D', 0, '(', '', ned)    
ss=   stress(ss, 'D', 0, ')', '', ned)    
ss=   getwrd(ss, 0)         
filebreak, ss, dir=idlans_dir 
ans_gen_version, answrs_version, idlans_dir=idlans_dir
defaults_file_read=0

time0 = dt_tm_tojs(systime())
;
; Parcing input parameters
;

if n_elements(deffile) then begin
  res=file_search(deffile, count=count)
  if count eq 0 then begin
    if printflag then begin
      print, 'Cannot file defaults file.' + deffile
      print, 'Using hardwired defaults or keywords that were explicitly set.'
    endif
  endif else begin
    ans_read_defaults, deffile, df
    defaults_file_read=1
  endelse
endif

; Note order of precendence:
;   1. Expicit input parameters
;   2. Default file input
;   3. Hardwired defaults.


if defaults_file_read and n_elements(ave_line)      eq 0  then ave_line=     df.ave_line
if defaults_file_read and n_elements(ave_pixel)     eq 0  then ave_pixel=    df.ave_pixel
if defaults_file_read and n_elements(deg)           eq 0  then deg=          df.deg
if defaults_file_read and n_elements(twobyte)       eq 0  then begin         
   twobyte=0                                                                 
   if strupcase(df.twobyte) eq 'Y' then twobyte=1                            
endif                                                                        
if defaults_file_read and n_elements(gshhs_dir)     eq 0  then gshhs_dir=    df.gshhs_dir
if defaults_file_read and n_elements(asf_import)    eq 0  then asf_import=   df.asf_import
if defaults_file_read and n_elements(position_loc)  eq 0  then position_loc= df.position_loc
if defaults_file_read and n_elements(idlans_dir)    eq 0  then idlans_dir=   df.idlans_dir
;
; Hardwired defaults
;
if n_elements(ave_line)     eq 0 then ave_line=     1
if n_elements(ave_pixel)    eq 0 then ave_pixel=    1
if n_elements(deg)          eq 0 then deg=          2
if n_elements(twobyte)      eq 0 then twobyte=      0
;
; Check for directories
;
if n_elements(gshhs_dir)    lt 1 then begin
  print, 'No gshhs_dir directory specified.'
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif
if n_elements(asf_import)   lt 1 then begin
  print, 'No asf_import file specified'
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif
if n_elements(position_loc) lt 1 then begin
  print, 'No position_loc file specified'
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif
if n_elements(idlans_dir)   lt 1 then begin
  print, 'No idlans_dir file specified'
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif

setenv, 'GSHHS_DIR='+ gshhs_dir
;
; Determine if input files exist
; 


if not file_test(datafile) then begin
  print, 'Could not find ASF Radarsat-1/ERS-2 data file: ' + datafile
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif

if not file_test(leaderfile) then begin
  print, 'Could not find ASF Radarsat-1 leader file: ' + leaderfile
  print, 'Exiting ans_read_rsat.pro '
  error= -1
  return
endif

;
; Create a new working directory
;
working_directory= working_dir + '/' + systime() + 'x' + string(randomu(seed)*100000,format='(I8)')
working_directory= strcompress( working_directory, /remove_all)
working_directory= repchr(working_directory, ':', 'x')
file_mkdir, working_directory

;
; Copy Radarsat-1 files to working directory.   Note the movement of Radarsat-1 directory to 
; and already processed directory must occur at a different level. It is not done in this program
;

file_copy, leaderfile, working_directory  + '/', /verbose
file_copy, datafile,   working_directory  + '/', /verbose
sleaderfile=    getwrd(leaderfile, 0, /last, del='/')
sdatafile=      getwrd(datafile, 0, /last, del='/')


asf_read_leader,  working_directory  + '/' + sleaderfile,  good_leader, lf, quiet=quiet

lf.name_dir          =  sdatafile
lf.ave_pixel         =  ave_pixel
lf.ave_line          =  ave_line
lf.deg               =  deg
lf.nx1               =  lf.nx0 / lf.ave_pixel
lf.ny1               =  lf.ny0 / lf.ave_line
lf.working_directory =  working_directory
; Add to struture
temp_lf              =  create_struct(lf, 'answrs_version', answrs_version)
lf                   =  0
lf                   =  temp_lf


asf_create_outloc, lf, position_loc=position_loc,  quiet=quiet

;
; Note the calibration is done w/in this program.
;
asf_read_data_file, working_directory  + '/' + sdatafile, image,  lf,   $
                    asf_import=asf_import, position_loc=postion_loc, quiet=quiet 
                    
asf_lonlat2pix_fit, lf 
asf_incid2pix_fit,  lf
asf_r_look,         lf
;
;
; Create file name root for output files.
;

lf.month = strupcase(lf.month)
case strupcase(lf.month) of
'JAN': month=1
'FEB': month=2
'MAR': month=3
'APR': month=4
'MAY': month=5
'JUN': month=6
'JUL': month=7
'AUG': month=8
'SEP': month=9
'OCT': month=10
'NOV': month=11
'DEC': month=12
ELSE:  month=1
endcase
;
;
; compute center geographic position
;


center_lat= string(abs(lf.center_lat), format='(F6.2)')
if lf.center_lat ge 0 then begin
  center_lat= center_lat + 'N'
endif else begin
  center_lat= center_lat + 'S'
endelse
center_lon= string(abs(lf.center_lon), format='(F6.2)')
if lf.center_lon ge 0 then begin
  center_lon= center_lon + 'E'
endif else begin
  center_lon= center_lon + 'W'
endelse

timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
         lf.month                               + ' ' + $
         string( lf.day,       format='(I2.2)') + ' ' + $
         string( lf.hour,      format='(I2.2)') + ':' + $
         string( lf.minute,    format='(I2.2)') + ':' + $
         string( lf.second,    format='(I2.2)')
         
js=            string( dt_tm_tojs(timestr), format='(I10.10)' ) 
output_prefix= 'RSAT1'
if lf.sar_sensor eq 'ERS-2' then output_prefix='ERS2'
output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + string( center_lon)
output_prefix= output_prefix + '_' + string( center_lat)
output_prefix= strcompress( output_prefix, /remove_all )

if n_elements(ave_pixel) eq 0 then ave_pixel=1
if n_elements(ave_line)  eq 0 then ave_line= 1


output_prefix2= output_prefix + '_' + strupcase(lf.polarization)


;
; Check and see if a lock file exits
;
lockfile=   output_prefix2 + '.lock'
;
res= file_search( lockfile, count=count)
if count gt 0 then begin
   print, 'Lockfile:  ' + lockfile + ' exists'
   error=-1
   return
endif
;
; Create lock file to protect other processes from creating SIO files
;
lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '.lock'
close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1
;
; Write out to SIO data file format. This is general for all.
; SAR data
;
scaling= 1.0
if not keyword_set(header) then begin
  time3= dt_tm_tojs(systime())
  if twobyte then begin
    ans_write_nrcs_sio, image, output_dir + '/'+ output_prefix2 + '_image.dat', scaling, /twobyte
  endif else begin
    ans_write_nrcs_sio, image, output_dir + '/'+ output_prefix2 + '_image.dat', scaling
  endelse
  time4= dt_tm_tojs(systime())
  if printflag then begin
    print, 'Time to write NRCS SIO file ' + strcompress( string(time4- time3), /remove_all) + ' s'
  endif
endif
;
; Write out SIO ancillary file format. This program is explicitly written for PALSAR data
;
asf_write_ancil_sio, output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling
time5= dt_tm_tojs(systime())

;
; Write out SIO mask file format.  This is general for all.
;
;
; Compute land mask and write out file. 
;
outputfile=  output_dir + '/'+ output_prefix2 + '_mask.dat'
;
; Only compute land mask for first of images
;
if not keyword_set(header) then begin
  if not keyword_set(nomask) then begin
    ans_write_land_mask, lf.lon_coef, lf.lon_xexp, lf.lon_yexp, lf.lat_coef, lf.lat_xexp, lf.lat_yexp, lf.nx1, lf.ny1,   $
                         lf.range_pixel   * lf.ave_pixel,            $
                         lf.azimuth_pixel * lf.ave_line,             $
                         outputfile,                                 $
                         quiet=       quiet,                         $
                         gshhs_dir=   gshhs_dir,                     $
                         pixel_size=  pixel_size
  endif
endif

time6= dt_tm_tojs(systime())
if printflag then begin
  print, 'Time to write land mask SIO file ' + strcompress( string(time6- time5), /remove_all) + ' s'
endif

;
; Remove lock file and working directory
;
file_delete, lockfile, /allow_nonexistent, /quiet

if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive, /verbose


end
