;+---------------------------
; NAME: ANS_READ_COSMOS 
; PURPOSE:
;    The purpose of this program is to read COSMOS SkyMed data and
;    create a SIO formatted binary image file and ASCII file.
; CALLING SEQUENCE:   
;    ans_read_cosmos, input_file, deffile
; INPUT:
;    input_file  = Cosmos SkyMed HDF data file
;    deffile     = Defaults file
; OUTPUT:          
;     Binary files with image data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;      COSMOS_ICUGS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;      COSMOS_ICUGS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;    The file name is delimited by _ and the entries are sensor, proessing facility, year, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header/  
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (1 if program completes or -1 if not)ta
;     outputroot   = root name of outputfiles
;     jsstart      = Processing start ime
; MODIFICATION HISTORY
;  2010 Oct 04
;  2011 Jan 21  Frank Monaldo. Add log_dir
;  2011 Jan 23  Frank Monaldo. Add time data to info structure
;  2011 Feb 07  Frank Monaldo. Add original file name to log file
;  2011 Feb 08  Frank Monaldo. Pass on error message.
;  2011 Feb 13  Frank Monaldo. Changed unique name of directory to make more sense
;                              Added better error catching
;  2011 Feb 14  Frank Monaldo. Add ability to exit if lock file encountered
;  2011 Mar 30  Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2011 Apr 20  Frank Monaldo. Correct spelling of gshss_dir that caused the
;                              program to default to the hard-wired values
;  2011 May 17  Frank Monaldo. Added '_X' file name
;  2011 May 23  Frank Monaldo. Add jsbegin and jsend to structure
;  2011 May 27  Frank Monaldo. Fill in jsbegin and jsend
;  2012 Jul 05  Frank Monaldo. Remove hardwired directories
;  2012 Aug 21  Frank Monaldo. Moved ans_gen_version
;  2012 Sep 07  Frank Monaldo. Add ability to deal with multiple images, by
;                              invoking ans_read_cosmos_hdf multiple times
;                              Pass on quite flag to ans_write_sio
;  2013 Feb 03  Frank Monaldo. Add ice mask and use only defaults file
;  2013 Feb 15  Frank Monaldo. Remove double forward slashes.
;  2013 Aug 14  Frank Monaldo. Move icemask, longitude, latitude, incident angle, radar look
;                              direction to ans_sat2sio.
;                              Remove delete lockfile
;  2013 Aug 21  Frank Monaldo. Creation of land mask file moved to ans_sat2sio
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_cosmos, input_file   , deffile,    $
                      quiet       = quiet,      $
                      help        = help,       $
                      outputroot  = outputroot, $
                      jsstart     = jsstart ,   $
                      error       = error
                    
error=     1
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_COSMOS 
  print, ' PURPOSE:
  print, '    The purpose of this program is to read COSMOS SkyMed data and
  print, '    create a SIO formatted binary image file and ASCII file.
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_cosmos, input_file, deffile
  print, ' INPUT:
  print, '    input_file  = Cosmos SkyMed HDF data file
  print, '    deffile     = Defaults file
  print, ' OUTPUT:          
  print, '    Binary files with image data and an ASCII file with ancillary data. The 
  print, '    convention for file names is of the form:
  print, '      COSMOS_ICUGS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '      COSMOS_ICUGS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '    The file name is delimited by _ and the entries are sensor, proessing facility, year, month, day, hour, minute
  print, '    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '    files include the SIO 20-byte header/  
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  print, '     error        = error (1 if program completes or -1 if not)ta
  print, '     outputroot   = root name of outputfiles
  print, '     jsstart      = Processing start 
  error= -1
  return
endif

scaling   = 1.0
error    = 1
printflag =1
if keyword_set(quiet) then printflag=0
if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 

;
; Read defaults file


ans_read_defaults, deffile, df, quiet=quiet

time0 = dt_tm_tojs(systime())

if tag_test(df, 'silent')  then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 0

if n_elements(df.gshhs_dir)  lt 1 then begin
  print, 'No gshhs_dir directory specified.'
  print, 'Exiting ans_read_cosmos.pro;
  error= -1
  return
endif

if n_elements(df.idlans_dir) lt 1 then begin
  print, 'No idlans_dir directory specified.'
  print, 'Exiting ans_read_cosmos.pro;
  error= -1
  return
endif
if n_elements(df.log_dir)    lt 1 then begin
  print, 'No log_dir directory specified.'
  print, 'Exiting ans_read_cosmos.pro;
  error= -1
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
setenv, 'GSHHS_DIR=' + df.gshhs_dir
;
; Determine if directory exists
; 
ff= file_search( input_file, count=count)

if count eq 0 then begin
  print, 'Could not find Cosmos-SkyMed: ' + input_file
  print, 'Exiting ans_read_cosmos.pro '
  error= -1
  return
endif


;
; Create a new working directory
;
working_directory= df.working_directory + '/csm_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory
;
; Copy Cosmos SkyMed file to working directory.   Note the movement of Envisat data to 
; an already processed directory must occur at a different level. It is not done in this program
;
file_mkdir, working_directory  + '/'
file_copy, input_file, working_directory  + '/', /recursive

res= file_search(df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
if printflag then begin
  print, 'Moving ' + input_file              + '/'   
  print, 'to     ' + df.already_processed    + '/' 
endif 
file_move, input_file,  df.already_processed  + '/' , /overwrite

;
; Keep only the input_file name and strip the path
;
inputfile= getwrd(input_file, 0, /last, del='/')

time1 = dt_tm_tojs(systime())

if printflag then print, 'Time to move Cosmos-Skymed file to working directory='+ string (time1-time0) +' s'


;
; Reading HDF file
;
error=1
version_no= float(!version.release)

if version_no lt 8 then begin
  print, 'This is version ' + !version.release
  print, 'This program will only work with versions 8+ because of the'
  print, 'iopen procedure.
  print, 'Exiting ans_read_cosmos'
  error= -1
  return
endif


data_set=    0
data_setmax= 0
while data_set le data_setmax do begin
  image=       0
  qlk=         0
  info=        !null
  outputroot=  ''
  ans_read_cosmos_hdf, working_directory + '/' + inputfile, image, qlk, info, image_no= data_set, quiet=quiet, error=error
  if fix( tag_value(info, 'no_of_images') ) gt 1 then begin
    data_setmax= 1
    outputroot= strarr(2)
  endif
  
  if error lt 0 then begin
    print, 'Cannot read ' + working_directory + '/' + inputfile
    error=-1
    return
  endif
  
  if printflag then print, 'Creating lf structure for Cosmos-Skymed data...'
  ;
  ; Note we have not yet built in mechanism for multiple polarizations. We will do that when we have
  ; some sample files to us 
  ;
  ;
  temp_info= !null
  
  temp_info=create_struct( info, $                            
          'ave_line'      , df.ave_line    , $ 
          'ave_pixel'     , df.ave_pixel   , $
          'deg'           , df.deg         , $
          'deg_incid'     , df.deg         , $
          'deg_rlook'     , df.deg         , $
          'center_incid'  , 0.0D           , $
          'center_lat'    , 0.0D           , $
          'center_lon'    , 0.0D           , $
          'answrs_version', answrs_version , $
          'jsstart'       , jsstart        , $
          'nrcs_slope'    , 1.0            , $
          'nrcs_bias'     , 0.0            , $
          'jsbegin'       , -3155673600D   , $   ; Default setting for start data
          'jsend'         , -3155673600D     )   ; Default setting for end  data           
         
  info=      !NULL                                                           
  info=      temp_info
  temp_info= !NULL
  case info.satellite_id of
  'CSKS1':begin 
          info.nrcs_slope= df.csm1_slope
          info.nrcs_bias=  df.csm1_bias
          end
  'CSKS2':begin 
          info.nrcs_slope= df.csm2_slope
          info.nrcs_bias=  df.csm2_bias
          end
  'CSKS3':begin 
          info.nrcs_slope= df.csm3_slope
          info.nrcs_bias=  df.csm3_bias
          end
  'CSKS4':begin 
          info.nrcs_slope= df.csm4_slope
          info.nrcs_bias=  df.csm4_bias
          end
  ELSE:  ; do nothing
  endcase
  ;
  
  if printflag then begin
    print, 'ans_read_cosmos'
    print, 'df.silent=' + df.silent
    print, 'quiet=    ' + string(quiet)
  endif
  
  cosmos_lonlat2pix_fit, info, quiet=quiet
  cosmos_incid2pix_fit,  info, quiet=quiet
  cosmos_rlook2pix_fit,  info, quiet=quiet
  ;
  ; Compute center incidence/lon/lat
  ;
  
  info.center_incid=   ans_poly_cal( info.nx/2, info.ny/2, info.incid_coef, info.incid_xexp, info.incid_yexp)
  info.center_lon=     ans_poly_cal( info.nx/2, info.ny/2, info.lon_coef,   info.lon_xexp,   info.lon_yexp)
  info.center_lat=     ans_poly_cal( info.nx/2, info.ny/2, info.lat_coef,   info.lat_xexp,   info.lat_yexp)
  ;
  ; Filling in more structure information
  ;
  ;
  ; Calibrate image;
  ;
  time1=  dt_tm_tojs( systime() )
  cosmos_dn2sigma, image, info, quiet=quiet
  ;
  ; Add correction to NRCS if set from defaults file.
  ;
  if info.nrcs_slope ne 1 or info.nrcs_bias ne 0 then begin
    image=  image * info.nrcs_slope + info.nrcs_bias
  endif
  ;
  time2=  dt_tm_tojs( systime() )
  if printflag then print, 'Time to calibrate: ' + string(time2-time1) + ' s'
  ;
  ; Create file name root for output files.
  ;
  center_lat= string(abs(float(info.scene_centre_geodetic_coordinates[0])), format='(F6.2)')
  if float(info.scene_centre_geodetic_coordinates[0]) ge 0 then begin
    center_lat= center_lat + 'N'
  endif else begin
    center_lat= center_lat + 'S'
  endelse
  center_lon= string(abs(float(info.scene_centre_geodetic_coordinates[1])), format='(F6.2)')
  if float(info.SCENE_CENTRE_GEODETIC_COORDINATES[0]) ge 0 then begin
    center_lon= center_lon + 'E'
  endif else begin
    center_lon= center_lon + 'W'
  endelse
  ;
  ; Compute center time
  ; 
  monthstr=      ['' , 'jan', 'feb', 'mar', 'apr', 'may', 'jun', $
                 'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
                 
  date1=         getwrd(info.scene_sensing_start_utc, 0)
  time1=         getwrd(info.scene_sensing_start_utc, 1)
  year1=         getwrd(date1, 0, del='-')
  month1=        getwrd(date1, 1, del='-')
  day1=          getwrd(date1, 2, del='-')
  date2=         getwrd(info.scene_sensing_stop_utc, 0)
  time2=         getwrd(info.scene_sensing_stop_utc, 1)
  year2=         getwrd(date1, 0, del='-')
  month2=        getwrd(date1, 1, del='-')
  day2=          getwrd(date1, 2, del='-') 
                 
  js1=           dt_tm_tojs(year1 + ' ' + monthstr[month1] + ' ' + day1 + ' ' + time1)
  js2=           dt_tm_tojs(year2 + ' ' + monthstr[month2] + ' ' + day2 + ' ' + time2)
  js=            (js1 + js2)/2  
  info.jsbegin=  js1
  info.jsend=    js2
  
  
  year=          dt_tm_fromjs(js, format='Y$'       )    
  month=         dt_tm_fromjs(js, format='0n$'      )
  day=           dt_tm_fromjs(js, format='0d$'      )
  hour=          dt_tm_fromjs(js, format='h$'       ) 
  minute=        dt_tm_fromjs(js, format='m$'       )
  second=        dt_tm_fromjs(js, format='s$'       )
  js=            string( js,      format='(I10.10)' )
  
  temp_info=create_struct( info,                           $                            
                           'data_year'      ,  year,       $ 
                           'month'          ,  month,      $
                           'day'            ,  day,        $
                           'hour'           ,  hour,       $
                           'minute'         ,  minute,     $
                           'second'         ,  second      )
         
  info=      !NULL                                                           
  info= temp_info
  temp_info= !NULL
  
  ;
  ; Create SIO prefix
  ;
  
  output_prefix= strcompress(info.satellite_id, /remove_all)
  output_prefix= output_prefix + '_' + strupcase( info.processing_center )
  output_prefix= output_prefix + '_' + year    
  output_prefix= output_prefix + '_' + month   
  output_prefix= output_prefix + '_' + day    
  output_prefix= output_prefix + '_' + hour   
  output_prefix= output_prefix + '_' + minute  
  output_prefix= output_prefix + '_' + second 
  output_prefix= output_prefix + '_' + js
  output_prefix= output_prefix + '_' + center_lon
  output_prefix= output_prefix + '_' + center_lat
  output_prefix= strcompress( output_prefix, /remove_all )   
  
  output_prefix2= output_prefix + '_' + strupcase(info.polarization[0])+ '_X'
  ;
  ; Check and see if a lock file exits
  ;
  lockfile=   df.sio_output_dir + '/'+ output_prefix2 + '*.lock'
  ;
  res= file_search( lockfile, count=count)
  if count gt 0 then begin
     print, 'Lockfile:  ' + lockfile + ' exists'
     error=-1
     return
  endif
  ;
  ; Write out to SIO data file format. This is general for all.
  ; SAR data
  ;
  scaling=1.0
  temp_info=create_struct( info,  'scaling',  scaling  )     
  info=      !NULL                                                           
  info= temp_info
  temp_info= !NULL
  
  outputroot[data_set]= df.sio_output_dir + '/'+ output_prefix2
  outputroot[data_set]= stress( outputroot[data_set], 'R', 0, '//', '/', ned)
  res= file_search(df.log_dir, count=count)
  if count eq 0 then file_mkdir, log_dir
  logfile= df.log_dir + '/' + output_prefix2 + '.log'
  
  outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd(info.file,0,/last,del='/') + ' Processing starts'
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  if strupcase(df.twobyte) eq 'Y' then begin
    ans_write_nrcs_sio, image, outputroot[data_set] + '_image.dat', scaling, /twobyte, quiet=quiet
  endif else begin
    ans_write_nrcs_sio, image, outputroot[data_set] + '_image.dat', scaling, quiet=quiet
  endelse
  time0= dt_tm_tojs(systime(/utc))
  outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + outputroot[data_set] + '_image.dat'
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr   
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for Cosmo SkyMeddata
  ;  
  ans_write_ancil_sio, outputroot[data_set] + '_ancil.txt', info, quiet=quiet
  time0= dt_tm_tojs(systime(/utc))
  outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + outputroot[data_set] + '_ancil.txt'
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1  
  print, outstr
  ;
  ; Incremenet data set value
  ;
  data_set= data_set + 1
endwhile  ; Looping through data sets.

;
; Remove
;

if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive, /quiet, /allow_nonexistent


end
