;-----------------------------------------------------
;+
; NAME: ANS_READ_COSMOS_HDF
; PURPOSE:
;   The purpose is the read an Cosmos SkyMed  HDF5 files and returns
;   and image and the ancillary data as they are stored
;   in the Cosmos-SkyMed file.  HDF5 files must have a .h5. This program
;   with only work with IDL 8
; CALLING SEQUENCE:
;   ans_read_cosmos_hdf, file, image, qlk, info
; INPUT:
;    file=     HDF5 file with .h5 extension
; OUTPUT:      
;    image=    Raw SAR image (original digital values)
;    qlk=      Quick look image
;    info=     Ancillary data structure with names from
;              HDF5 file
; KEYWORD      
;    /quiet    Print no intermediate output.
;    /help     Get help.
;    error=    If =-1 then cannot read file
;    /diag     Diagonistics turned on
;    image_no= (Return either the first or the second image. Only
;               valid for PINPONG images.  Valid values are 0 and 1.
;              If not set then the the the ith image and info structure
;              a returned)
; MODIFICATION HISTORY:
;    2010 Sep 30 Frank Monaldo
;    2011 Jan 12 Frank Monaldo. Add quick look data 
;    2011 Jan 23 Frank Monaldo. Change spelling of processing_center polarization to 
;                               American English spelling. 
;                               Added frequency band to structure
;    2011 Feb 08 Frank Monaldo. Add error handling if can not read data
;    2012 Aug 01 Frank Monaldo. Check for some tags.  Should extend to all tags
;    2012 Aug 02 Frank Monaldo. Add more tag tests.
;    2012 Aug 13 Frank Monaldo. Use the new tag_value command that will deal better with missing data.
;    2012 Aug 14 Frank Monaldo. Massive re-write to gracefully deal with different image modes.  
;    2012 Aug 32 Frank Monaldo. Add ability to read PINGPONG mode data.
;    2012 Sep 07 Frank Monaldo. Add ability to return either image from PINGPONG data
;    2012 Dec 26 Frank Monlado. Add HIMAGE data mode
; Copyright (C) 2010-12, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_read_cosmos_hdf, file, image, qlk, info,   $
                         image_no=       image_no   , $
                         help=           help       , $
                         quiet=          quiet      , $
                         error=          error      , $
                         diag=           diag

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_COSMOS_HDF
  print, ' PURPOSE:
  print, '   The purpose is the read an Cosmos SkyMed  HDF5 files and returns
  print, '   and image and the ancillary data as they are stored
  print, '   in the Cosmos-SkyMed file.  HDF5 files must have a .h5. This program
  print, '   with only work with IDL 8
  print, ' CALLING SEQUENCE:
  print, '   ans_read_cosmos_hdf, file, image, qlk, info
  print, ' INPUT:
  print, '    file=     HDF5 file with .h5 extension
  print, ' OUTPUT:
  print, '    image=    Raw SAR image (original digital values)
  print, '    qlk=      Quick look image
  print, '    info=     Ancillary data structure with names from
  print, '              HDF5 file
  print, ' KEYWORD
  print, '    /quiet    Print no intermediate output.
  print, '    /help     Get help.
  print, '    error     If =-1 then cannot read filed
  print, '    /diag     Diagonistics turned on
  error= -1
  return
endif

version_no= float(!version.release)

if version_no lt 8 then begin
  print, 'This is version ' + !version.release
  print, 'This program will only work with versions 8+ because of the'
  print, 'iopen procedure.
  print, 'Exiting ans_read_cosmos_hdf'
  error= -1
  return
endif

error= 0
;catch, error_status  
;;This statement begins the error handler:  
;
;if Error_status ne 0 then begin  
;   print, 'error index: ', error_status  
;   print, 'error message: ', !error_state.msg  
;   if error_status eq -249  or $
;      error_status eq -252  or $
;      error_status eq -261  or $
;      error_status eq -262  then begin
;      error= -1
;      return
;   endif
;   catch, /cancel  
;endif  

printflag= 1
!quiet=    1
if keyword_set(quiet)    then printflag=     0 
if n_elements(image_no) eq 0 then image_no=  0

res=  file_search(file, count=count)

if count eq 0 then begin
  print,  ''
  print, '************************************************************
  print, 'Cannot find file: ' + file
  print, 'Exiting ans_read_cosmos_hdf' 
  print, '============================================================
  print, ''
  error= -1
  return
endif
;
;  Reading HDF file into strucutre
;
if printflag then print, 'Begin reading: ' + file + ' ' + systime(/utc)
 
iopen, file, x

if printflag then print, 'End   reading: ' + file + ' ' + systime(/utc)

if printflag then begin
  print, ''
  print, 'FILE:             ' + getwrd(file, 0, /last, del='/')
  print, 'ACQUISITION MODE: ' + x.acquisition_mode._data
  print, ''
endif
;
; Common informaional values in single and multi-image files


case x.acquisition_mode._data of

'HUGEREGION': begin
  ;
  ;=====================================================================
  ;                HUGE REGION IMAGE FILE                                    
  ;=====================================================================
  if printflag then print, 'Parsing variables from HUGEREGION file'
  info={file                                             : tag_value( x,'_file'                                                ) , $  
        acquistion_mode                                  : tag_value( x,'acquisition_mode._data'                               ),  $         
        no_of_images                                     : 1L                                                                    , $  Assume we have one image
        icontype                                         : tag_value( x,'_icontype'                                            ) , $      
        leap_sign                                        : tag_value( x,'leap_sign._data'                                      ) , $        
        leap_utc                                         : tag_value( x,'leap_utc._data'                                       ) , $        
        orbit_number                                     : tag_value( x,'orbit_number._data'                                   ) , $    
        programmed_image_id                              : tag_value( x,'programmed_image_id._data'                            ) , $
        scene_sensing_start_utc                          : tag_value( x,'scene_sensing_start_utc._data'                        ) , $
        scene_sensing_stop_utc                           : tag_value( x,'scene_sensing_stop_utc._data'                         ) , $
        selective_availability_status                    : tag_value( x,'selective_availability_status._data'                  ) , $
        adc_characterization                             : tag_value( x,'adc_characterization._data'                           ) , $
        azimuth_polynomial_reference_time                : tag_value( x,'azimuth_polynomial_reference_time._data'              ) , $
        centroid_vs_range_time_polynomial                : tag_value( x,'centroid_vs_range_time_polynomial._data'              ) , $
        centroid_vs_azimuth_time_polynomial              : tag_value( x,'centroid_vs_azimuth_time_polynomial._data'            ) , $
        doppler_ambiguity_estimation_method              : tag_value( x,'doppler_ambiguity_estimation_method._data'            ) , $
        doppler_centroid_estimation_method               : tag_value( x,'doppler_centroid_estimation_method._data'             ) , $
        doppler_rate_estimation_method                   : tag_value( x,'doppler_rate_estimation_method._data'                 ) , $               
        doppler_rate_vs_azimuth_time_polynomial          : tag_value( x,'doppler_rate_vs_azimuth_time_polynomial._data'        ) , $         
        doppler_rate_vs_range_time_polynomial            : tag_value( x,'doppler_rate_vs_range_time_polynomial._data'          ) , $
        range_polynomial_reference_time                  : tag_value( x,'range_polynomial_reference_time._data'                ) , $
        bits_per_sample                                  : tag_value( x,'bits_per_sample._data'                                ) , $      
        columns_order                                    : tag_value( x,'columns_order._data'                                  ) , $        
        image_layers                                     : tag_value( x,'image_layers._data'                                   ) , $         
        image_scale                                      : tag_value( x,'image_scale._data'                                    ) , $          
        lines_order                                      : tag_value( x,'lines_order._data'                                    ) , $          
        sample_format                                    : tag_value( x,'sample_format._data'                                  ) , $        
        samples_per_pixel                                : tag_value( x,'samples_per_pixel._data'                              ) , $    
        acquisition_station_id                           : tag_value( x,'acquisition_station_id._data'                         ) , $
        delivery_mode                                    : tag_value( x,'delivery_mode._data'                                  ) , $        
        downlink_start_utc                               : tag_value( x,'downlink_start_utc._data'                             ) , $        
        downlink_stop_utc                                : tag_value( x,'downlink_stop_utc._data'                              ) , $         
        mission_id                                       : tag_value( x,'mission_id._data'                                     ) , $                
        processing_center                                : tag_value( x,'processing_centre._data'                              ) , $   ; Note the change in spelling      
        product_filename                                 : tag_value( x,'product_filename._data'                               ) , $          
        product_specification_document                   : tag_value( x,'product_specification_document._data'                 ) , $
        product_type                                     : tag_value( x,'product_type._data'                                   ) , $            
        satellite_id                                     : tag_value( x,'satellite_id._data'                                   ) , $            
        acquisition_mode                                 : tag_value( x,'acquisition_mode._data'                               ) , $        
        antenna_length                                   : tag_value( x,'antenna_length._data'                                 ) , $          
        azimuth_beamwidth                                : tag_value( x,'azimuth_beamwidth._data'                              ) , $       
        look_side                                        : tag_value( x,'look_side._data'                                      ) , $               
        multi_beam_id                                    : tag_value( x,'multi_beam_id._data'                                  ) , $           
        original_bit_quantisation                        : tag_value( x,'original_bit_quantisation._data'                      ) , $
        radar_frequency                                  : tag_value( x,'radar_frequency._data'                                ) , $             
        radar_wavelength                                 : tag_value( x,'radar_wavelength._data'                               ) , $            
        subswaths_number                                 : tag_value( x,'subswaths_number._data'                               ) , $            
        attitude_product_category                        : tag_value( x,'attitude_product_category._data'                      ) , $   
        doppler_ambiguity_confidence_measure             : tag_value( x,'doppler_ambiguity_confidence_measure._data'           ) , $
        doppler_centroid_confidence_measure              : tag_value( x,'doppler_centroid_confidence_measure._data'            ) , $
        doppler_centroid_estimation_accuracy             : tag_value( x,'doppler_centroid_estimation_accuracy._data'           ) , $
        doppler_rate_confidence_measure                  : tag_value( x,'doppler_rate_confidence_measure._data'                ) , $
        doppler_rate_estimation_accuracy                 : tag_value( x,'doppler_rate_estimation_accuracy._data'               ) , $
        pod_initial_position_accuracy                    : tag_value( x,'pod_initial_position_accuracy._data'                  ) , $    
        pod_overlap_position_accuracy                    : tag_value( x,'pod_overlap_position_accuracy._data'                  ) , $    
        pod_product_category                             : tag_value( x,'pod_product_category._data'                           ) , $             
        pod_quality_flag                                 : tag_value( x,'pod_quality_flag._data'                               ) , $                 
        attitude_quaternions                             : tag_value( x,'attitude_quaternions._data'                           ) , $             
        attitude_times                                   : tag_value( x,'attitude_times._data'                                 ) , $                  
        ecef_satellite_acceleration                      : tag_value( x,'ecef_satellite_acceleration._data'                    ) , $      
        ecef_satellite_position                          : tag_value( x,'ecef_satellite_position._data'                        ) , $          
        ecef_satellite_velocity                          : tag_value( x,'ecef_satellite_velocity._data'                        ) , $          
        inertial_reference_frame_id                      : tag_value( x,'inertial_reference_frame_id._data'                    ) , $      
        inertial_satellite_acceleration                  : tag_value( x,'inertial_satellite_acceleration._data'                ) , $  
        inertial_satellite_position                      : tag_value( x,'inertial_satellite_position._data'                    ) , $
        inertial_satellite_velocity                      : tag_value( x,'inertial_satellite_velocity._data'                    ) , $
        number_of_state_vectors                          : tag_value( x,'number_of_state_vectors._data'                        ) , $
        orbit_direction                                  : tag_value( x,'orbit_direction._data'                                ) , $
        pitch_rate                                       : tag_value( x,'pitch_rate._data'                                     ) , $
        roll_rate                                        : tag_value( x,'roll_rate._data'                                      ) , $
        satellite_height                                 : tag_value( x,'satellite_height._data'                               ) , $
        state_vectors_times                              : tag_value( x,'state_vectors_times._data'                            ) , $
        yaw_rate                                         : tag_value( x,'yaw_rate._data'                                       ) , $
        ecef_beam_centre_direction_for_processing        : tag_value( x,'ecef_beam_centre_direction_for_processing._data'      ) , $
        invalid_value                                    : tag_value( x,'invalid_value._data'                                  ) , $
        l0_software_version                              : tag_value( x,'l0_software_version._data'                            ) , $
        light_speed                                      : tag_value( x,'light_speed._data'                                    ) , $
        processing_history_bit_mask                      : tag_value( x,'processing_history_bit_mask._data'                    ) , $
        product_error_flag                               : tag_value( x,'product_error_flag._data'                             ) , $
        product_generation_utc                           : tag_value( x,'product_generation_utc._data'                         ) , $
        reference_utc                                    : tag_value( x,'reference_utc._data'                                  ) , $
        datum_rotation                                   : tag_value( x,'datum_rotation._data'                                 ) , $
        datum_scale                                      : tag_value( x,'datum_scale._data'                                    ) , $
        datum_shift                                      : tag_value( x,'datum_shift._data'                                    ) , $
        ellipsoid_designator                             : tag_value( x,'ellipsoid_designator._data'                           ) , $
        ellipsoid_semimajor_axis                         : tag_value( x,'ellipsoid_semimajor_axis._data'                       ) , $
        ellipsoid_semiminor_axis                         : tag_value( x,'ellipsoid_semiminor_axis._data'                       ) , $
        projection_id                                    : tag_value( x,'projection_id._data'                                  ) , $
        azimuth_coverage                                 : tag_value( x,'azimuth_coverage._data'                               ) , $
        centre_earth_radius                              : tag_value( x,'centre_earth_radius._data'                            ) , $
        ground_range_coverage                            : tag_value( x,'ground_range_coverage._data'                          ) , $
        scene_centre_geodetic_coordinates                : tag_value( x,'scene_centre_geodetic_coordinates._data'              ) , $
        scene_orientation                                : tag_value( x,'scene_orientation._data'                              ) , $
        doppler_ambiguity_confidence_measure_threshold   : tag_value( x,'doppler_ambiguity_confidence_measure_threshold._data' ) , $
        doppler_ambiguity_threshold                      : tag_value( x,'doppler_ambiguity_threshold._data'                    ) , $
        doppler_centroid_confidence_measure_threshold    : tag_value( x,'doppler_centroid_confidence_measure_threshold._data'  ) , $
        doppler_centroid_estimation_accuracy_threshold   : tag_value( x,'doppler_centroid_estimation_accuracy_threshold._data' ) , $
        doppler_rate_confidence_measure_threshold        : tag_value( x,'doppler_rate_confidence_measure_threshold._data'      ) , $
        doppler_rate_estimation_accuracy_threshold       : tag_value( x,'doppler_rate_estimation_accuracy_threshold._data'     ) , $
        raw_bias_threshold                               : tag_value( x,'raw_bias_threshold._data'                             ) , $
        raw_gain_imbalance_threshold                     : tag_value( x,'raw_gain_imbalance_threshold._data'                   ) , $
        raw_i_q_normality_threshold                      : tag_value( x,'raw_i_q_normality_threshold._data'                    ) , $
        raw_i_q_ortogonality_threshold                   : tag_value( x,'raw_i_q_ortogonality_threshold._data'                 ) , $
        raw_missing_lines_per_block_threshold            : tag_value( x,'raw_missing_lines_per_block_threshold._data'          ) , $
        raw_missing_lines_percentage_threshold           : tag_value( x,'raw_missing_lines_percentage_threshold._data'         ) , $
        raw_oversaturated_percentage_threshold           : tag_value( x,'raw_oversaturated_percentage_threshold._data'         ) , $
        raw_phase_uniformity_threshold                   : tag_value( x,'raw_phase_uniformity_threshold._data'                 ) , $
        raw_undersaturated_percentage_threshold          : tag_value( x,'raw_undersaturated_percentage_threshold._data'        ) , $
        rescaling_factor                                 : tag_value( x,'rescaling_factor._data'                               ) , $                                 
        adc_compensation                                 : tag_value( x,'adc_compensation._data'                               ) , $                                 
        antenna_pattern_compensation_reference_surface   : tag_value( x,'antenna_pattern_compensation_reference_surface._data' ) , $   
        azimuth_antenna_pattern_compensation_geometry    : tag_value( x,'azimuth_antenna_pattern_compensation_geometry._data'  ) , $    
        calibration_constant_compensation_flag           : tag_value( x,'calibration_constant_compensation_flag._data'         ) , $           
        incidence_angle_compensation_geometry            : tag_value( x,'incidence_angle_compensation_geometry._data'          ) , $            
        incidence_angle_compensation_reference_surface   : tag_value( x,'incidence_angle_compensation_reference_surface._data' ) , $   
        range_antenna_pattern_compensation_geometry      : tag_value( x,'range_antenna_pattern_compensation_geometry._data'    ) , $      
        range_spreading_loss_compensation_geometry       : tag_value( x,'range_spreading_loss_compensation_geometry._data'     ) , $       
        reference_incidence_angle                        : tag_value( x,'reference_incidence_angle._data'                      ) , $                        
        reference_slant_range                            : tag_value( x,'reference_slant_range._data'                          ) , $                            
        azimuth_focusing_weighting_coefficient           : tag_value( x,'azimuth_focusing_weighting_coefficient._data'         ) , $           
        azimuth_focusing_weighting_function              : tag_value( x,'azimuth_focusing_weighting_function._data'            ) , $              
        azimuth_processing_number_of_looks               : tag_value( x,'azimuth_processing_number_of_looks._data'             ) , $               
        focusing_algorithm_id                            : tag_value( x,'focusing_algorithm_id._data'                          ) , $                            
        l1a_software_version                             : tag_value( x,'l1a_software_version._data'                           ) , $                             
        multilook_filter_flag                            : tag_value( x,'multilook_filter_flag._data'                          ) , $                            
        range_focusing_weighting_coefficient             : tag_value( x,'range_focusing_weighting_coefficient._data'           ) , $             
        range_focusing_weighting_function                : tag_value( x,'range_focusing_weighting_function._data'              ) , $                
        range_processing_number_of_looks                 : tag_value( x,'range_processing_number_of_looks._data'               ) , $                 
        replica_reconstruction_method                    : tag_value( x,'replica_reconstruction_method._data'                  ) , $                    
        azimuth_geometric_resolution                     : tag_value( x,'azimuth_geometric_resolution._data'                   ) , $                     
        ground_range_geometric_resolution                : tag_value( x,'ground_range_geometric_resolution._data'              ) , $                
        inter_channel_co_registration                    : tag_value( x,'inter_channel_co_registration._data'                  ) , $                    
        irf_shape__10db__3db                             : tag_value( x,'irf_shape__10db__3db._data'                           ) , $                             
        irf_shape__6db__3db                              : tag_value( x,'irf_shape__6db__3db._data'                            ) , $                              
        islr                                             : tag_value( x,'islr._data'                                           ) , $                                             
        local_radiometric_stability                      : tag_value( x,'local_radiometric_stability._data'                    ) , $                      
        point_target_ambiguity                           : tag_value( x,'point_target_ambiguity._data'                         ) , $                           
        pslr                                             : tag_value( x,'pslr._data'                                           ) , $                                                                       
        sslr                                             : tag_value( x,'sslr._data'                                           ) , $                                             
        image_undersaturated_percentage_threshold        : tag_value( x,'image_undersaturated_percentage_threshold._data'      ) , $        
        image_oversaturated_percentage_threshold         : tag_value( x,'image_oversaturated_percentage_threshold._data'       ) , $         
        reference_slant_range_exponent                   : tag_value( x,'reference_slant_range_exponent._data'                 ) , $                   
        azimuth_multilooking_weighting_coefficient       : tag_value( x,'azimuth_multilooking_weighting_coefficient._data'     ) , $    
        azimuth_multilooking_weighting_function          : tag_value( x,'azimuth_multilooking_weighting_function._data'        ) , $       
        beam_mosaicking_policy                           : tag_value( x,'beam_mosaicking_policy._data'                         ) , $                        
        l1b_software_version                             : tag_value( x,'l1b_software_version._data'                           ) , $                          
        range_multilooking_weighting_coefficient         : tag_value( x,'range_multilooking_weighting_coefficient._data'       ) , $           
        ground_to_slant_polynomial                       : tag_value( x,'ground_to_slant_polynomial._data'                     ) , $                    
        slant_to_ground_polynomial                       : tag_value( x,'slant_to_ground_polynomial._data'                     ) , $                    
        spf_mean_intensity_ratio                         : tag_value( x,'spf_mean_intensity_ratio._data'                       ) , $                      
        spf_standard_deviation_intensity_ratio           : tag_value( x,'spf_standard_deviation_intensity_ratio._data'         ) , $        
        spf_target_contrast                              : tag_value( x,'spf_target_contrast._data'                            ) , $                           
        spf_targets                                      : tag_value( x,'spf_targets._data'                                    ) , $                                   
        spf_azimuth_moving_window                        : tag_value( x,'spf_azimuth_moving_window._data'                      ) , $                     
        spf_filter_type                                  : tag_value( x,'spf_filter_type._data'                                ) , $                               
        spf_iteration                                    : tag_value( x,'spf_iteration._data'                                  ) , $                                 
        spf_range_moving_window                          : tag_value( x,'spf_range_moving_window._data'                        ) , $                       
        spf_software_version                             : tag_value( x,'spf_software_version._data'                           ) , $                          
        geolocation_accuracy                             : tag_value( x,'geolocation_accuracy._data'                           ) , $                          
        geometric_conformity                             : tag_value( x,'geometric_conformity._data'                           ) , $                          
        zero_doppler_azimuth_first_time                  : tag_value( x,'mbi.zero_doppler_azimuth_first_time._data'            ) , $  
        zero_doppler_azimuth_last_time                   : tag_value( x,'mbi.zero_doppler_azimuth_last_time._data'             ) , $    
        zero_doppler_range_first_time                    : tag_value( x,'mbi.zero_doppler_range_first_time._data'              ) , $      
        zero_doppler_range_last_time                     : tag_value( x,'mbi.zero_doppler_range_last_time._data'               ) , $        
        image_max                                        : tag_value( x,'mbi.image_max._data'                                  ) , $                                              
        image_mean                                       : tag_value( x,'mbi.image_mean._data'                                 ) , $                                            
        image_min                                        : tag_value( x,'mbi.image_min._data'                                  ) , $                                              
        image_standard_deviation                         : tag_value( x,'mbi.image_standard_deviation._data'                   ) , $                
        column_spacing                                   : tag_value( x,'mbi.column_spacing._data'                             ) , $                                    
        equivalent_number_of_looks                       : tag_value( x,'mbi.equivalent_number_of_looks._data'                 ) , $            
        line_spacing                                     : tag_value( x,'mbi.line_spacing._data'                               ) , $                                        
        line_time_interval                               : tag_value( x,'mbi.line_time_interval._data'                         ) , $                            
        bottom_left_geodetic_coordinates                 : tag_value( x,'mbi.bottom_left_geodetic_coordinates._data'           ) , $ 
        bottom_right_geodetic_coordinates                : tag_value( x,'mbi.bottom_right_geodetic_coordinates._data'          ) , $
        far_incidence_angle                              : tag_value( x,'mbi.far_incidence_angle._data'                        ) , $                          
        far_look_angle                                   : tag_value( x,'mbi.far_look_angle._data'                             ) , $                                    
        near_incidence_angle                             : tag_value( x,'mbi.near_incidence_angle._data'                       ) , $                        
        near_look_angle                                  : tag_value( x,'mbi.near_look_angle._data'                            ) , $                                  
        top_left_geodetic_coordinates                    : tag_value( x,'mbi.top_left_geodetic_coordinates._data'              ) , $      
        top_right_geodetic_coordinates                   : tag_value( x,'mbi.top_right_geodetic_coordinates._data'             ) , $    
        image_oversaturated_percentage                   : tag_value( x,'mbi.image_oversaturated_percentage._data'             ) , $    
        image_undersaturated_percentage                  : tag_value( x,'mbi.image_undersaturated_percentage._data'            ) , $  
        subswath_change_column                           : tag_value( x,'mbi.subswath_change_column._data'                     ) , $
        quick_look_columns_order                         : tag_value( x,'qlk.quick_look_columns_order._data'                   ) , $
        quick_look_lines_order                           : tag_value( x,'qlk.quick_look_lines_order._data'                     ) , $
        quick_look_column_spacing                        : tag_value( x,'qlk.quick_look_column_spacing._data'                  ) , $
        quick_look_line_spacing                          : tag_value( x,'qlk.quick_look_line_spacing._data'                    ) , $   
        quick_look_projection_id                         : tag_value( x,'qlk.quick_look_projection_id._data'                   ) }
      
 
  qlk=  tag_value(x,'qlk._data')        ; Quick look image
  ;
  ; Added 2011 Jan 24
  ; Flipping the image to correspond to what is considered top and bottom
  ;
  if n_elements(qlk) gt 1 then qlk= reverse(qlk,2)
  ;
  ; Calibrated subswaths
  ;
  n_swaths=                         info.subswaths_number
  if info.subswaths_number eq'' then n_swaths=1
  calibration_constant=                dblarr(n_swaths)
  calibration_constant_estimation_utc= strarr(n_swaths)
  beam_id=                             strarr(n_swaths)
  synthetic_aperture_duration=         dblarr(n_swaths)
  polarisation=                        strarr(n_swaths)
  prf=                                 dblarr(n_swaths)
  for i=0L, n_swaths-1 do begin
    case i of
    0:    begin 
            calibration_constant[i]                = x.s01.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s01.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s01.beam_id._data
            synthetic_aperture_duration[i]         = x.s01.synthetic_aperture_duration._data
            polarisation[i]                        = x.s01.polarisation._data
            prf[i]                                 = x.s01.prf._data
          end
    1:    begin
            calibration_constant[i]                = x.s02.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s02.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s02.beam_id._data
            synthetic_aperture_duration[i]         = x.s02.synthetic_aperture_duration._data
            polarisation[i]                        = x.s02.polarisation._data
            prf[i]                                 = x.s02.prf._data
          end
    2:    begin
            calibration_constant[i]                = x.s03.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s03.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s03.beam_id._data
            synthetic_aperture_duration[i]         = x.s03.synthetic_aperture_duration._data
            polarisation[i]                        = x.s03.polarisation._data
            prf[i]                                 = x.s03.prf._data
          end
    3:    begin
            calibration_constant[i]                = x.s04.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s04.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s04.beam_id._data
            synthetic_aperture_duration[i]         = x.s04.synthetic_aperture_duration._data
            polarisation[i]                        = x.s04.polarisation._data
            prf[i]                                 = x.s04.prf._data
          end
    4:    begin
            calibration_constant[i]                = x.s05.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s05.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s05.beam_id._data
            synthetic_aperture_duration[i]         = x.s05.synthetic_aperture_duration._data         
            polarisation[i]                        = x.s05.polarisation._data
            prf[i]                                 = x.s05.prf._data                                 
          end
    5:    begin
            calibration_constant[i]                = x.s06.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s06.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s06.beam_id._data
            synthetic_aperture_duration[i]         = x.s06.synthetic_aperture_duration._data
            polarisation[i]                        = x.s06.polarisation._data
            prf[i]                                 = x.s06.prf._data
          end
    6:    begin
            calibration_constant[i]                = x.s07.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s07.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s07.beam_id._data
            synthetic_aperture_duration[i]         = x.s07.synthetic_aperture_duration._data
            polarisation[i]                        = x.s07.polarisation._data
            prf[i]                                 = x.s07.prf._data
          end
    7:    begin
            calibration_constant[i]                = x.s08.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s08.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s08.beam_id._data
            synthetic_aperture_duration[i]         = x.s08.synthetic_aperture_duration._data
            polarisation[i]                        = x.s08.polarisation._data
            prf[i]                                 = x.s08.prf._data
          end
    8:   begin
            calibration_constant[i]                         = x.s09.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s09.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s09.beam_id._data
            synthetic_aperture_duration[i]         = x.s09.synthetic_aperture_duration._data
            polarisation[i]                        = x.s09.polarisation._data
            prf[i]                                 = x.s09.prf._data
          end
    9:   begin
            calibration_constant[i]                         = x.s10.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s10.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s10.beam_id._data
            synthetic_aperture_duration[i]         = x.s10.synthetic_aperture_duration._data
            polarisation[i]                        = x.s10.polarisation._data
            prf[i]                                 = x.s10.prf._data
          end
    10:  begin
            calibration_constant[i]                = x.s11.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s11.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s11.beam_id._data
            synthetic_aperture_duration[i]         = x.s11.synthetic_aperture_duration._data
            polarisation[i]                        = x.s11.polarisation._data
            prf[i]                                 = x.s11.prf._data
          end
    11:  begin
            calibration_constant[i]                = x.s12.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s12.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s12.beam_id._data
            synthetic_aperture_duration[i]         = x.s12.synthetic_aperture_duration._data
            polarisation[i]                        = x.s12.polarisation._data
            prf[i]                                 = x.s12.prf._data
          end
          
    12:  begin
            calibration_constant[i]                = x.s13.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s13.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s13.beam_id._data
            synthetic_aperture_duration[i]         = x.s13.synthetic_aperture_duration._data
            polarisation[i]                        = x.s13.polarisation._data
            prf[i]                                 = x.s13.prf._data
          end
    ELSE: begin
          ; Do nothing
          end
    endcase
  endfor
  
  image=   x.mbi._data
  ;
  ; Added 2011 Jan 24
  ; Flipping the image to correspond to what is considered top and bottom
  ;
  image= reverse(image, 2)
  
  sz=     size(image)
  nx=     long(sz[1])
  ny=     long(sz[2])
  
  temp_info=create_struct( info, $                            
          'nx'                                    , nx                  , $ 
          'ny'                                    , ny                  , $
          'nx0'                                   , nx                  , $
          'ny0'                                   , ny                  , $
          'nx1'                                   , nx                  , $
          'ny1'                                   , ny                   )
         
  info=      !NULL                                                           
  info= temp_info
  temp_info= !NULL
  
  ; Clean up variables for info structure
  ;
  ; Add elements with the correct names for the ancillary data file.
  ;
  polarization=         'BLANK'
  if n_elements(polarization) gt 0 then polarization=         polarisation  
  asc_des=              strupcase( strmid(info.orbit_direction,0,1) )
  sensor_velocity=      0.0
  range_flag=           0
  if strpos(info.projection_id, 'GROUND RANGE') ge 0  then range_flag=1
  rcvr_gain=            0.0
  proc_gain=            0.0 
  flip=                 0
  platform_code=        'C'+strmid(info.satellite_id,4,1)
  valuetype=            'intensity'
  range_type=           info.projection_id
  early_azimuth_edge=   'B'
  near_range_edge=      'L'
  if info.lines_order   ne 'EARLY-LATE' then early_azimuth_edge= 'T'
  if info.columns_order ne 'NEAR-FAR'   then near_range_edge=    'R'
  range_swath=          info.nx * info.line_spacing
  azimuth_swath=        info.ny * info.column_spacing
  azimuth_looks=        info.azimuth_processing_number_of_looks
  range_looks=          sqrt( info.equivalent_number_of_looks^2 / azimuth_looks^2)
  ellipsoidscale=       1
  datumshiftparameters= '0 0 0'
  ns=                   n_elements(info.datum_shift)
  if ns gt 0 then begin
    datumshiftparameters=''
    for i=0, ns-1 do datumshiftparameters=datumshiftparameters + string(info.datum_shift[i]) + ' '
  endif
  integration_time=     0.0D
  software_version=     'L0: '+ info.l0_software_version + ' L1A:' + info.l1a_software_version + ' L1B:' + info.l1b_software_version
  slant_range_center=   0.0D
  slant_range_near=     0.0D
  lngswitchflag=       0
  ;
  ; Checking the lngwitchflag
  
  if abs(info.scene_centre_geodetic_coordinates[1] )  gt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  gt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  gt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  gt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  gt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 180 meridian'
          lngswitchflag=1
          if info.scene_centre_geodetic_coordinates[1] lt 0 then info.scene_centre_geodetic_coordinates[1]= 360 + lf.scene_centre_geodetic_coordinates[1] 
          if info.bottom_left_geodetic_coordinates[1]  lt 0 then info.bottom_left_geodetic_coordinates[1] = 360 + lf.bottom_left_geodetic_coordinates[1]  
          if info.bottom_right_geodetic_coordinates[1] lt 0 then info.bottom_right_geodetic_coordinates[1]= 360 + lf.bottom_right_geodetic_coordinates[1] 
          if info.top_left_geodetic_coordinates[1]     lt 0 then info.top_left_geodetic_coordinates[1]    = 360 + lf.top_left_geodetic_coordinates[1]     
          if info.top_right_geodetic_coordinates[1]    lt 0 then info.top_right_geodetic_coordinates[1]   = 360 + lf.top_right_geodetic_coordinates[1]    
        endif
  endif  
  if abs(info.scene_centre_geodetic_coordinates[1] )  lt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  lt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  lt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  lt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  lt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 0 meridian'
          lngswitchflag=0
          if info.scene_centre_geodetic_coordinates[1] gt 180 then info.scene_centre_geodetic_coordinates[1]= lf.scene_centre_geodetic_coordinates[1]- 360
          if info.bottom_left_geodetic_coordinates[1]  gt 180 then info.bottom_left_geodetic_coordinates[1] = lf.bottom_left_geodetic_coordinates[1] - 360
          if info.bottom_right_geodetic_coordinates[1] gt 180 then info.bottom_right_geodetic_coordinates[1]= lf.bottom_right_geodetic_coordinates[1]- 360
          if info.top_left_geodetic_coordinates[1]     gt 180 then info.top_left_geodetic_coordinates[1]    = lf.top_left_geodetic_coordinates[1]    - 360
          if info.top_right_geodetic_coordinates[1]    gt 180 then info.top_right_geodetic_coordinates[1]   = lf.top_right_geodetic_coordinates[1]   - 360
        endif
  endif  
  
  
  
  temp_info=create_struct( info, $                            
          'calibration_constant'                   , calibration_constant                   , $ 
          'calibration_constant_estimation_utc'    , calibration_constant_estimation_utc    , $
          'beam_id'                                , beam_id                                , $
          'synthetic_aperture_duration'            , synthetic_aperture_duration            , $
          'polarization'                           , polarization                           , $
          'pol'                                    , polarization[0]                        , $
          'prf'                                    , prf                                    , $
          'frequency_band'                         , 'X'                                    , $
          'producttype'                            , info.product_type                      , $
          'name_dir'                               , info.file                              , $
          'range_pixel'                            , info.column_spacing                    , $
          'azimuth_pixel'                          , info.line_spacing                      , $
          'sar_sensor'                             , info.satellite_id                      , $
          'platform_code'                          , platform_code                          , $
          'datumshiftparameters'                   , datumshiftparameters                   , $
          'proc_fac'                               , info.processing_center                 , $
          'software_version'                       , software_version                       , $ 
          'asc_des'                                , asc_des                                , $
          'beam_mode'                              , info.multi_beam_id                     , $
          'sensor_velocity'                        , sensor_velocity                        , $
          'nom_az_res'                             , info.azimuth_geometric_resolution      , $
          'nom_rg_res'                             , info.ground_range_geometric_resolution , $
          's_c_altitude'                           , info.satellite_height                  , $
          'rcvr_gain'                              , rcvr_gain                              , $  
          'proc_gain'                              , proc_gain                              , $
          'range_flag'                             , range_flag                             , $
          'flip'                                   , flip                                   , $
          'sar_imaging_mode'                       , info.multi_beam_id                     , $
          'datatake_id'                            , info.programmed_image_id               , $
          'valuetype'                              , valuetype                              , $
          'range_type'                             , range_type                             , $
          'heading'                                , info.scene_orientation                 , $
          'heading_rel'                            , 90.0                                   , $
          'illumination_dir'                       , 0.0                                    , $
          'early_azimuth_edge'                     , early_azimuth_edge                     , $
          'near_range_edge'                        , near_range_edge                        , $  
          'range_swath'                            , range_swath                            , $
          'azimuth_swath'                          , azimuth_swath                          , $
          'range_looks'                            , range_looks                            , $
          'azimuth_looks'                          , azimuth_looks                          , $
          'ellipsoidscale'                         , ellipsoidscale                         , $ 
          'ellipsoidname'                          , info.ellipsoid_designator              , $ 
          'semimajoraxis'                          , info.ellipsoid_semimajor_axis          , $ 
          'semiminoraxis'                          , info.ellipsoid_semiminor_axis          , $ 
          'slant_range_center'                     , slant_range_center                     , $
          'slant_range_near'                       , slant_range_near                       , $
          'integration_time'                       , integration_time                       , $
          'lngswitchflag'                          , lngswitchflag                          )
  
         
    info=      !NULL                                                           
    info= temp_info
    temp_info= !NULL       
  end
'PINGPONG': begin  
  ;=====================================================================
  ;               DUAL-POL IMAGE FILE                                  
  ;=====================================================================
  if printflag then print, 'Parsing variables from PINGPONG file'
  
  info={file                                             : tag_value( x,'_file'                                                ) , $  
        acquistion_mode                                  : tag_value( x,'acquisition_mode._data'                               ) , $         
        no_of_images                                     : 2L                                                                    , $  Assume we have one image
        icontype                                         : tag_value( x,'_icontype'                                            ) , $      
        leap_sign                                        : tag_value( x,'leap_sign._data'                                      ) , $        
        leap_utc                                         : tag_value( x,'leap_utc._data'                                       ) , $        
        orbit_number                                     : tag_value( x,'orbit_number._data'                                   ) , $    
        programmed_image_id                              : tag_value( x,'programmed_image_id._data'                            ) , $
        scene_sensing_start_utc                          : tag_value( x,'scene_sensing_start_utc._data'                        ) , $
        scene_sensing_stop_utc                           : tag_value( x,'scene_sensing_stop_utc._data'                         ) , $
        selective_availability_status                    : tag_value( x,'selective_availability_status._data'                  ) , $
        adc_characterization                             : tag_value( x,'adc_characterization._data'                           ) , $
        azimuth_polynomial_reference_time                : tag_value( x,'azimuth_polynomial_reference_time._data'              ) , $
        centroid_vs_range_time_polynomial                : tag_value( x,'centroid_vs_range_time_polynomial._data'              ) , $
        centroid_vs_azimuth_time_polynomial              : tag_value( x,'centroid_vs_azimuth_time_polynomial._data'            ) , $
        doppler_ambiguity_estimation_method              : tag_value( x,'doppler_ambiguity_estimation_method._data'            ) , $
        doppler_centroid_estimation_method               : tag_value( x,'doppler_centroid_estimation_method._data'             ) , $
        doppler_rate_estimation_method                   : tag_value( x,'doppler_rate_estimation_method._data'                 ) , $               
        doppler_rate_vs_azimuth_time_polynomial          : tag_value( x,'doppler_rate_vs_azimuth_time_polynomial._data'        ) , $         
        doppler_rate_vs_range_time_polynomial            : tag_value( x,'doppler_rate_vs_range_time_polynomial._data'          ) , $
        range_polynomial_reference_time                  : tag_value( x,'range_polynomial_reference_time._data'                ) , $
        bits_per_sample                                  : tag_value( x,'bits_per_sample._data'                                ) , $      
        columns_order                                    : tag_value( x,'columns_order._data'                                  ) , $        
        image_layers                                     : tag_value( x,'image_layers._data'                                   ) , $         
        image_scale                                      : tag_value( x,'image_scale._data'                                    ) , $          
        lines_order                                      : tag_value( x,'lines_order._data'                                    ) , $          
        sample_format                                    : tag_value( x,'sample_format._data'                                  ) , $        
        samples_per_pixel                                : tag_value( x,'samples_per_pixel._data'                              ) , $    
        acquisition_station_id                           : tag_value( x,'acquisition_station_id._data'                         ) , $
        delivery_mode                                    : tag_value( x,'delivery_mode._data'                                  ) , $        
        downlink_start_utc                               : tag_value( x,'downlink_start_utc._data'                             ) , $        
        downlink_stop_utc                                : tag_value( x,'downlink_stop_utc._data'                              ) , $         
        mission_id                                       : tag_value( x,'mission_id._data'                                     ) , $                
        processing_center                                : tag_value( x,'processing_centre._data'                              ) , $   ; Note the change in spelling      
        product_filename                                 : tag_value( x,'product_filename._data'                               ) , $          
        product_specification_document                   : tag_value( x,'product_specification_document._data'                 ) , $
        product_type                                     : tag_value( x,'product_type._data'                                   ) , $            
        satellite_id                                     : tag_value( x,'satellite_id._data'                                   ) , $            
        acquisition_mode                                 : tag_value( x,'acquisition_mode._data'                               ) , $        
        antenna_length                                   : tag_value( x,'antenna_length._data'                                 ) , $          
        azimuth_beamwidth                                : tag_value( x,'azimuth_beamwidth._data'                              ) , $       
        look_side                                        : tag_value( x,'look_side._data'                                      ) , $               
        multi_beam_id                                    : tag_value( x,'multi_beam_id._data'                                  ) , $           
        original_bit_quantisation                        : tag_value( x,'original_bit_quantisation._data'                      ) , $
        radar_frequency                                  : tag_value( x,'radar_frequency._data'                                ) , $             
        radar_wavelength                                 : tag_value( x,'radar_wavelength._data'                               ) , $            
        subswaths_number                                 : tag_value( x,'subswaths_number._data'                               ) , $            
        attitude_product_category                        : tag_value( x,'attitude_product_category._data'                      ) , $   
        doppler_ambiguity_confidence_measure             : tag_value( x,'doppler_ambiguity_confidence_measure._data'           ) , $
        doppler_centroid_confidence_measure              : tag_value( x,'doppler_centroid_confidence_measure._data'            ) , $
        doppler_centroid_estimation_accuracy             : tag_value( x,'doppler_centroid_estimation_accuracy._data'           ) , $
        doppler_rate_confidence_measure                  : tag_value( x,'doppler_rate_confidence_measure._data'                ) , $
        doppler_rate_estimation_accuracy                 : tag_value( x,'doppler_rate_estimation_accuracy._data'               ) , $
        pod_initial_position_accuracy                    : tag_value( x,'pod_initial_position_accuracy._data'                  ) , $    
        pod_overlap_position_accuracy                    : tag_value( x,'pod_overlap_position_accuracy._data'                  ) , $    
        pod_product_category                             : tag_value( x,'pod_product_category._data'                           ) , $             
        pod_quality_flag                                 : tag_value( x,'pod_quality_flag._data'                               ) , $                 
        attitude_quaternions                             : tag_value( x,'attitude_quaternions._data'                           ) , $             
        attitude_times                                   : tag_value( x,'attitude_times._data'                                 ) , $                  
        ecef_satellite_acceleration                      : tag_value( x,'ecef_satellite_acceleration._data'                    ) , $      
        ecef_satellite_position                          : tag_value( x,'ecef_satellite_position._data'                        ) , $          
        ecef_satellite_velocity                          : tag_value( x,'ecef_satellite_velocity._data'                        ) , $          
        inertial_reference_frame_id                      : tag_value( x,'inertial_reference_frame_id._data'                    ) , $      
        inertial_satellite_acceleration                  : tag_value( x,'inertial_satellite_acceleration._data'                ) , $  
        inertial_satellite_position                      : tag_value( x,'inertial_satellite_position._data'                    ) , $
        inertial_satellite_velocity                      : tag_value( x,'inertial_satellite_velocity._data'                    ) , $
        number_of_state_vectors                          : tag_value( x,'number_of_state_vectors._data'                        ) , $
        orbit_direction                                  : tag_value( x,'orbit_direction._data'                                ) , $
        pitch_rate                                       : tag_value( x,'pitch_rate._data'                                     ) , $
        roll_rate                                        : tag_value( x,'roll_rate._data'                                      ) , $
        satellite_height                                 : tag_value( x,'satellite_height._data'                               ) , $
        state_vectors_times                              : tag_value( x,'state_vectors_times._data'                            ) , $
        yaw_rate                                         : tag_value( x,'yaw_rate._data'                                       ) , $
        ecef_beam_centre_direction_for_processing        : tag_value( x,'ecef_beam_centre_direction_for_processing._data'      ) , $
        invalid_value                                    : tag_value( x,'invalid_value._data'                                  ) , $
        l0_software_version                              : tag_value( x,'l0_software_version._data'                            ) , $
        light_speed                                      : tag_value( x,'light_speed._data'                                    ) , $
        processing_history_bit_mask                      : tag_value( x,'processing_history_bit_mask._data'                    ) , $
        product_error_flag                               : tag_value( x,'product_error_flag._data'                             ) , $
        product_generation_utc                           : tag_value( x,'product_generation_utc._data'                         ) , $
        reference_utc                                    : tag_value( x,'reference_utc._data'                                  ) , $
        datum_rotation                                   : tag_value( x,'datum_rotation._data'                                 ) , $
        datum_scale                                      : tag_value( x,'datum_scale._data'                                    ) , $
        datum_shift                                      : tag_value( x,'datum_shift._data'                                    ) , $
        ellipsoid_designator                             : tag_value( x,'ellipsoid_designator._data'                           ) , $
        ellipsoid_semimajor_axis                         : tag_value( x,'ellipsoid_semimajor_axis._data'                       ) , $
        ellipsoid_semiminor_axis                         : tag_value( x,'ellipsoid_semiminor_axis._data'                       ) , $
        projection_id                                    : tag_value( x,'projection_id._data'                                  ) , $
        azimuth_coverage                                 : tag_value( x,'azimuth_coverage._data'                               ) , $
        centre_earth_radius                              : tag_value( x,'centre_earth_radius._data'                            ) , $
        ground_range_coverage                            : tag_value( x,'ground_range_coverage._data'                          ) , $
        scene_centre_geodetic_coordinates                : tag_value( x,'scene_centre_geodetic_coordinates._data'              ) , $
        scene_orientation                                : tag_value( x,'scene_orientation._data'                              ) , $
        doppler_ambiguity_confidence_measure_threshold   : tag_value( x,'doppler_ambiguity_confidence_measure_threshold._data' ) , $
        doppler_ambiguity_threshold                      : tag_value( x,'doppler_ambiguity_threshold._data'                    ) , $
        doppler_centroid_confidence_measure_threshold    : tag_value( x,'doppler_centroid_confidence_measure_threshold._data'  ) , $
        doppler_centroid_estimation_accuracy_threshold   : tag_value( x,'doppler_centroid_estimation_accuracy_threshold._data' ) , $
        doppler_rate_confidence_measure_threshold        : tag_value( x,'doppler_rate_confidence_measure_threshold._data'      ) , $
        doppler_rate_estimation_accuracy_threshold       : tag_value( x,'doppler_rate_estimation_accuracy_threshold._data'     ) , $
        raw_bias_threshold                               : tag_value( x,'raw_bias_threshold._data'                             ) , $
        raw_gain_imbalance_threshold                     : tag_value( x,'raw_gain_imbalance_threshold._data'                   ) , $
        raw_i_q_normality_threshold                      : tag_value( x,'raw_i_q_normality_threshold._data'                    ) , $
        raw_i_q_ortogonality_threshold                   : tag_value( x,'raw_i_q_ortogonality_threshold._data'                 ) , $
        raw_missing_lines_per_block_threshold            : tag_value( x,'raw_missing_lines_per_block_threshold._data'          ) , $
        raw_missing_lines_percentage_threshold           : tag_value( x,'raw_missing_lines_percentage_threshold._data'         ) , $
        raw_oversaturated_percentage_threshold           : tag_value( x,'raw_oversaturated_percentage_threshold._data'         ) , $
        raw_phase_uniformity_threshold                   : tag_value( x,'raw_phase_uniformity_threshold._data'                 ) , $
        raw_undersaturated_percentage_threshold          : tag_value( x,'raw_undersaturated_percentage_threshold._data'        ) , $
        rescaling_factor                                 : tag_value( x,'rescaling_factor._data'                               ) , $                                 
        adc_compensation                                 : tag_value( x,'adc_compensation._data'                               ) , $                                 
        antenna_pattern_compensation_reference_surface   : tag_value( x,'antenna_pattern_compensation_reference_surface._data' ) , $   
        azimuth_antenna_pattern_compensation_geometry    : tag_value( x,'azimuth_antenna_pattern_compensation_geometry._data'  ) , $    
        calibration_constant_compensation_flag           : tag_value( x,'calibration_constant_compensation_flag._data'         ) , $           
        incidence_angle_compensation_geometry            : tag_value( x,'incidence_angle_compensation_geometry._data'          ) , $            
        incidence_angle_compensation_reference_surface   : tag_value( x,'incidence_angle_compensation_reference_surface._data' ) , $   
        range_antenna_pattern_compensation_geometry      : tag_value( x,'range_antenna_pattern_compensation_geometry._data'    ) , $      
        range_spreading_loss_compensation_geometry       : tag_value( x,'range_spreading_loss_compensation_geometry._data'     ) , $       
        reference_incidence_angle                        : tag_value( x,'reference_incidence_angle._data'                      ) , $                        
        reference_slant_range                            : tag_value( x,'reference_slant_range._data'                          ) , $                            
        azimuth_focusing_weighting_coefficient           : tag_value( x,'azimuth_focusing_weighting_coefficient._data'         ) , $           
        azimuth_focusing_weighting_function              : tag_value( x,'azimuth_focusing_weighting_function._data'            ) , $              
        azimuth_processing_number_of_looks               : tag_value( x,'azimuth_processing_number_of_looks._data'             ) , $               
        focusing_algorithm_id                            : tag_value( x,'focusing_algorithm_id._data'                          ) , $                            
        l1a_software_version                             : tag_value( x,'l1a_software_version._data'                           ) , $                             
        multilook_filter_flag                            : tag_value( x,'multilook_filter_flag._data'                          ) , $                            
        range_focusing_weighting_coefficient             : tag_value( x,'range_focusing_weighting_coefficient._data'           ) , $             
        range_focusing_weighting_function                : tag_value( x,'range_focusing_weighting_function._data'              ) , $                
        range_processing_number_of_looks                 : tag_value( x,'range_processing_number_of_looks._data'               ) , $                 
        replica_reconstruction_method                    : tag_value( x,'replica_reconstruction_method._data'                  ) , $                    
        azimuth_geometric_resolution                     : tag_value( x,'azimuth_geometric_resolution._data'                   ) , $                     
        ground_range_geometric_resolution                : tag_value( x,'ground_range_geometric_resolution._data'              ) , $                
        inter_channel_co_registration                    : tag_value( x,'inter_channel_co_registration._data'                  ) , $                    
        irf_shape__10db__3db                             : tag_value( x,'irf_shape__10db__3db._data'                           ) , $                             
        irf_shape__6db__3db                              : tag_value( x,'irf_shape__6db__3db._data'                            ) , $                              
        islr                                             : tag_value( x,'islr._data'                                           ) , $                                             
        local_radiometric_stability                      : tag_value( x,'local_radiometric_stability._data'                    ) , $                      
        point_target_ambiguity                           : tag_value( x,'point_target_ambiguity._data'                         ) , $                           
        pslr                                             : tag_value( x,'pslr._data'                                           ) , $                                                                  
        sslr                                             : tag_value( x,'sslr._data'                                           ) , $                                             
        image_undersaturated_percentage_threshold        : tag_value( x,'image_undersaturated_percentage_threshold._data'      ) , $        
        image_oversaturated_percentage_threshold         : tag_value( x,'image_oversaturated_percentage_threshold._data'       ) , $         
        reference_slant_range_exponent                   : tag_value( x,'reference_slant_range_exponent._data'                 ) , $                   
        azimuth_multilooking_weighting_coefficient       : tag_value( x,'azimuth_multilooking_weighting_coefficient._data'     ) , $    
        azimuth_multilooking_weighting_function          : tag_value( x,'azimuth_multilooking_weighting_function._data'        ) , $       
        beam_mosaicking_policy                           : tag_value( x,'beam_mosaicking_policy._data'                         ) , $                        
        l1b_software_version                             : tag_value( x,'l1b_software_version._data'                           ) , $                          
        range_multilooking_weighting_coefficient         : tag_value( x,'range_multilooking_weighting_coefficient._data'       ) , $           
        ground_to_slant_polynomial                       : tag_value( x,'ground_to_slant_polynomial._data'                     ) , $                    
        slant_to_ground_polynomial                       : tag_value( x,'slant_to_ground_polynomial._data'                     ) , $                    
        spf_mean_intensity_ratio                         : tag_value( x,'spf_mean_intensity_ratio._data'                       ) , $                      
        spf_standard_deviation_intensity_ratio           : tag_value( x,'spf_standard_deviation_intensity_ratio._data'         ) , $        
        spf_target_contrast                              : tag_value( x,'spf_target_contrast._data'                            ) , $                           
        spf_targets                                      : tag_value( x,'spf_targets._data'                                    ) , $                                   
        spf_azimuth_moving_window                        : tag_value( x,'spf_azimuth_moving_window._data'                      ) , $                     
        spf_filter_type                                  : tag_value( x,'spf_filter_type._data'                                ) , $                               
        spf_iteration                                    : tag_value( x,'spf_iteration._data'                                  ) , $                                 
        spf_range_moving_window                          : tag_value( x,'spf_range_moving_window._data'                        ) , $                       
        spf_software_version                             : tag_value( x,'spf_software_version._data'                           ) , $                          
        geolocation_accuracy                             : tag_value( x,'geolocation_accuracy._data'                           ) , $                          
        geometric_conformity                             : tag_value( x,'geometric_conformity._data'                           ) }
       
  ;
  ; data dependent on image_no
  ;
  for i=0, image_no do begin
    case i of 
    0: begin
         zero_doppler_azimuth_first_time     = tag_value( x, 's01.sbi.zero_doppler_azimuth_first_time._data'   ) 
         zero_doppler_azimuth_last_time      = tag_value( x, 's01.sbi.zero_doppler_azimuth_last_time._data'    )   
         zero_doppler_range_first_time       = tag_value( x, 's01.sbi.zero_doppler_range_first_time._data'     )     
         zero_doppler_range_last_time        = tag_value( x, 's01.sbi.zero_doppler_range_last_time._data'      )       
         image_max                           = tag_value( x, 's01.sbi.image_max._data'                         )                                             
         image_mean                          = tag_value( x, 's01.sbi.image_mean._data'                        )                                           
         image_min                           = tag_value( x, 's01.sbi.image_min._data'                         )                                             
         image_standard_deviation            = tag_value( x, 's01.sbi.image_standard_deviation._data'          )               
         column_spacing                      = tag_value( x, 's01.sbi.column_spacing._data'                    )                                   
         equivalent_number_of_looks          = tag_value( x, 's01.sbi.equivalent_number_of_looks._data'        )           
         line_spacing                        = tag_value( x, 's01.sbi.line_spacing._data'                      )                                       
         line_time_interval                  = tag_value( x, 's01.sbi.line_time_interval._data'                )
         top_left_geodetic_coordinates       = tag_value( x, 's02.sbi.top_left_geodetic_coordinates._data'     )         
         top_right_geodetic_coordinates      = tag_value( x, 's02.sbi.top_right_geodetic_coordinates._data'    )    
         bottom_left_geodetic_coordinates    = tag_value( x, 's01.sbi.bottom_left_geodetic_coordinates._data'  ) 
         bottom_right_geodetic_coordinates   = tag_value( x, 's01.sbi.bottom_right_geodetic_coordinates._data' ) 
         far_incidence_angle                 = tag_value( x, 's01.sbi.far_incidence_angle._data'               )                         
         far_look_angle                      = tag_value( x, 's01.sbi.far_look_angle._data'                    )                                   
         near_incidence_angle                = tag_value( x, 's01.sbi.near_incidence_angle._data'              )                       
         near_look_angle                     = tag_value( x, 's01.sbi.near_look_angle._data'                   )                                 
         top_left_geodetic_coordinates       = tag_value( x, 's01.sbi.top_left_geodetic_coordinates._data'     )     
         top_right_geodetic_coordinates      = tag_value( x, 's01.sbi.top_right_geodetic_coordinates._data'    )   
         image_oversaturated_percentage      = tag_value( x, 's01.sbi.image_oversaturated_percentage._data'    )   
         image_undersaturated_percentage     = tag_value( x, 's01.sbi.image_undersaturated_percentage._data'   ) 
         subswath_change_column              = tag_value( x, 's01.sbi.subswath_change_column._data'            ) 
         quick_look_columns_order            = tag_value( x, 's01.qlk.quick_look_columns_order._data'          ) 
         quick_look_lines_order              = tag_value( x, 's01.qlk.quick_look_lines_order._data'            ) 
         quick_look_column_spacing           = tag_value( x, 's01.qlk.quick_look_column_spacing._data'         ) 
         quick_look_line_spacing             = tag_value( x, 's01.qlk.quick_look_line_spacing._data'           )  
         quick_look_projection_id            = tag_value( x, 's01.qlk.quick_look_projection_id._data'          ) 
         qlk                                 = tag_value( x, 's01.qlk._data'                                   )         
         image_undersaturated_percentage     = tag_value( x, 's01.sbi.image_undersaturated_percentage._data'   )
         image_oversaturated_percentage      = tag_value( x, 's01.sbi.image_oversaturated_percentage._data'    ) 
         subswath_change_column              = tag_value( x, 's01.sbi.subswath_change_column._data'            )
         equivalent_number_of_looks          = tag_value( x, 's01.sbi.equivalent_number_of_looks._data'        )      
         image                               = tag_value( x, 's01.sbi._data'                                   )   
         calibration_constant                = tag_value( x, 's01.calibration_constant._data'                  )
         calibration_constant_estimation_utc = tag_value( x, 's01.calibration_constant_estimation_utc._data'   )
         prf                                 = tag_value( x, 's01.prf._data'                                   )
         polarization                        = tag_value( x, 's01.polarisation._data'                          )
         calibration_sampling_rate           = tag_value( x, 's01.calibration_sampling_rate._data'             )
         beam_id                             = tag_value( x, 's01.beam_id._data'                               )
         synthetic_aperture_duration         = tag_value( x, 's01.synthetic_aperture_duration._data'           )
         prf                                 = tag_value( x, 's01.prf._data'                                   )
       end
    1: begin
         zero_doppler_azimuth_first_time     = tag_value( x, 's02.sbi.zero_doppler_azimuth_first_time._data'   ) 
         zero_doppler_azimuth_last_time      = tag_value( x, 's02.sbi.zero_doppler_azimuth_last_time._data'    )   
         zero_doppler_range_first_time       = tag_value( x, 's02.sbi.zero_doppler_range_first_time._data'     )     
         zero_doppler_range_last_time        = tag_value( x, 's02.sbi.zero_doppler_range_last_time._data'      )       
         image_max                           = tag_value( x, 's02.sbi.image_max._data'                         )                                             
         image_mean                          = tag_value( x, 's02.sbi.image_mean._data'                        )                                           
         image_min                           = tag_value( x, 's02.sbi.image_min._data'                         )                                             
         image_standard_deviation            = tag_value( x, 's02.sbi.image_standard_deviation._data'          )               
         column_spacing                      = tag_value( x, 's02.sbi.column_spacing._data'                    )                                   
         equivalent_number_of_looks          = tag_value( x, 's02.sbi.equivalent_number_of_looks._data'        )           
         line_spacing                        = tag_value( x, 's02.sbi.line_spacing._data'                      )                                       
         line_time_interval                  = tag_value( x, 's02.sbi.line_time_interval._data'                )
         top_left_geodetic_coordinates       = tag_value( x, 's02.sbi.top_left_geodetic_coordinates._data'     )         
         top_right_geodetic_coordinates      = tag_value( x, 's02.sbi.top_right_geodetic_coordinates._data'    )    
         bottom_left_geodetic_coordinates    = tag_value( x, 's02.sbi.bottom_left_geodetic_coordinates._data'  ) 
         bottom_right_geodetic_coordinates   = tag_value( x, 's02.sbi.bottom_right_geodetic_coordinates._data' ) 
         far_incidence_angle                 = tag_value( x, 's02.sbi.far_incidence_angle._data'               )                         
         far_look_angle                      = tag_value( x, 's02.sbi.far_look_angle._data'                    )                                   
         near_incidence_angle                = tag_value( x, 's02.sbi.near_incidence_angle._data'              )                       
         near_look_angle                     = tag_value( x, 's02.sbi.near_look_angle._data'                   )                                 
         top_left_geodetic_coordinates       = tag_value( x, 's02.sbi.top_left_geodetic_coordinates._data'     )     
         top_right_geodetic_coordinates      = tag_value( x, 's02.sbi.top_right_geodetic_coordinates._data'    )   
         image_oversaturated_percentage      = tag_value( x, 's02.sbi.image_oversaturated_percentage._data'    )   
         image_undersaturated_percentage     = tag_value( x, 's02.sbi.image_undersaturated_percentage._data'   ) 
         subswath_change_column              = tag_value( x, 's02.sbi.subswath_change_column._data'            ) 
         quick_look_columns_order            = tag_value( x, 's02.qlk.quick_look_columns_order._data'          ) 
         quick_look_lines_order              = tag_value( x, 's02.qlk.quick_look_lines_order._data'            ) 
         quick_look_column_spacing           = tag_value( x, 's02.qlk.quick_look_column_spacing._data'         ) 
         quick_look_line_spacing             = tag_value( x, 's02.qlk.quick_look_line_spacing._data'           )  
         quick_look_projection_id            = tag_value( x, 's02.qlk.quick_look_projection_id._data'          ) 
         qlk                                 = tag_value( x, 's02.qlk._data'                                   )         
         image_undersaturated_percentage     = tag_value( x, 's02.sbi.image_undersaturated_percentage._data'   )
         image_oversaturated_percentage      = tag_value( x, 's02.sbi.image_oversaturated_percentage._data'    ) 
         subswath_change_column              = tag_value( x, 's02.sbi.subswath_change_column._data'            )
         equivalent_number_of_looks          = tag_value( x, 's02.sbi.equivalent_number_of_looks._data'        )      
         image                               = tag_value( x, 's02.sbi._data'                                   )   
         calibration_constant                = tag_value( x, 's02.calibration_constant._data'                  )
         calibration_constant_estimation_utc = tag_value( x, 's02.calibration_constant_estimation_utc._data'   )
         prf                                 = tag_value( x, 's02.prf._data'                                   )
         polarization                        = tag_value( x, 's02.polarisation._data'                          )
         calibration_sampling_rate           = tag_value( x, 's02.calibration_sampling_rate._data'             ) 
         beam_id                             = tag_value( x, 's02.beam_id._data'                               )
         synthetic_aperture_duration         = tag_value( x, 's02.synthetic_aperture_duration._data'           )
         prf                                 = tag_value( x, 's02.prf._data'                                   )
       end
    endcase
  endfor
  if n_elements(qlk) gt 1 then qlk= reverse(qlk,2)
  sz=  size(image)
  nx=  sz[1]
  ny=  sz[2]
  temp_info= !NULL
  temp_info=create_struct( info, $   
          'zero_doppler_azimuth_first_time'       , zero_doppler_azimuth_first_time       , $
          'zero_doppler_azimuth_last_time'        , zero_doppler_azimuth_last_time        , $
          'zero_doppler_range_first_time'         , zero_doppler_range_first_time         , $
          'zero_doppler_range_last_time'          , zero_doppler_range_last_time          , $
          'image_max'                             , image_max                             , $
          'image_mean'                            , image_mean                            , $
          'image_min'                             , image_min                             , $
          'image_standard_deviation'              , image_standard_deviation              , $
          'column_spacing'                        , column_spacing                        , $
          'equivalent_number_of_looks'            , equivalent_number_of_looks            , $
          'line_spacing'                          , line_spacing                          , $
          'line_time_interval'                    , line_time_interval                    , $
          'top_left_geodetic_coordinates'         , top_left_geodetic_coordinates         , $
          'top_right_geodetic_coordinates'        , top_right_geodetic_coordinates        , $
          'bottom_left_geodetic_coordinates'      , bottom_left_geodetic_coordinates      , $
          'bottom_right_geodetic_coordinates'     , bottom_right_geodetic_coordinates     , $
          'far_incidence_angle'                   , far_incidence_angle                   , $
          'far_look_angle'                        , far_look_angle                        , $
          'near_incidence_angle'                  , near_incidence_angle                  , $
          'near_look_angle'                       , near_look_angle                       , $
          'image_oversaturated_percentage'        , image_oversaturated_percentage        , $
          'image_undersaturated_percentage'       , image_undersaturated_percentage       , $
          'subswath_change_column'                , subswath_change_column                , $
          'quick_look_columns_order'              , quick_look_columns_order              , $
          'quick_look_lines_order'                , quick_look_lines_order                , $
          'quick_look_column_spacing '            , quick_look_column_spacing             , $
          'quick_look_line_spacing'               , quick_look_line_spacing               , $
          'quick_look_projection_id'              , quick_look_projection_id              , $
          'nx'                                    , nx                                    , $ 
          'ny'                                    , ny                                    , $
          'nx0'                                   , nx                                    , $
          'ny0'                                   , ny                                    , $
          'nx1'                                   , nx                                    , $
          'ny1'                                   , ny                                    )     
  info=      !NULL                                                           
  info= temp_info
  temp_info= !NULL
  ;
  ; Clean up variables for info structure
  ;
  ; Add elements with the correct names for the ancillary data file.
  ; 
  asc_des=              strupcase( strmid(info.orbit_direction,0,1) )
  sensor_velocity=      0.0
  range_flag=           0
  if strpos(info.projection_id, 'GROUND RANGE') ge 0  then range_flag=1
  rcvr_gain=            0.0
  proc_gain=            0.0 
  flip=                 0
  platform_code=        'C'+strmid(info.satellite_id,4,1)
  valuetype=            'intensity'
  range_type=           info.projection_id
  early_azimuth_edge=   'B'
  near_range_edge=      'L'
  if info.lines_order   ne 'EARLY-LATE' then early_azimuth_edge= 'T'
  if info.columns_order ne 'NEAR-FAR'   then near_range_edge=    'R'
  range_swath=          info.nx * info.line_spacing
  azimuth_swath=        info.ny * info.column_spacing
  azimuth_looks=        info.azimuth_processing_number_of_looks
  range_looks=          sqrt( info.equivalent_number_of_looks^2 / azimuth_looks^2)
  ellipsoidscale=       1
  datumshiftparameters= '0 0 0'
  ns=                   n_elements(info.datum_shift)
  if ns gt 0 then begin
    datumshiftparameters=''
    for i=0, ns-1 do datumshiftparameters=datumshiftparameters + string(info.datum_shift[i]) + ' '
  endif
  integration_time=     0.0D
  software_version=     'L0: '+ info.l0_software_version + ' L1A:' + info.l1a_software_version + ' L1B:' + info.l1b_software_version
  slant_range_center=   0.0D
  slant_range_near=     0.0D
  lngswitchflag=       0
  ;
  ; Checking the lngwitchflag
  
  if abs(info.scene_centre_geodetic_coordinates[1] )  gt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  gt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  gt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  gt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  gt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 180 meridian'
          lngswitchflag=1
          if info.scene_centre_geodetic_coordinates[1] lt 0 then info.scene_centre_geodetic_coordinates[1]= 360 + lf.scene_centre_geodetic_coordinates[1] 
          if info.bottom_left_geodetic_coordinates[1]  lt 0 then info.bottom_left_geodetic_coordinates[1] = 360 + lf.bottom_left_geodetic_coordinates[1]  
          if info.bottom_right_geodetic_coordinates[1] lt 0 then info.bottom_right_geodetic_coordinates[1]= 360 + lf.bottom_right_geodetic_coordinates[1] 
          if info.top_left_geodetic_coordinates[1]     lt 0 then info.top_left_geodetic_coordinates[1]    = 360 + lf.top_left_geodetic_coordinates[1]     
          if info.top_right_geodetic_coordinates[1]    lt 0 then info.top_right_geodetic_coordinates[1]   = 360 + lf.top_right_geodetic_coordinates[1]    
        endif
  endif  
  if abs(info.scene_centre_geodetic_coordinates[1] )  lt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  lt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  lt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  lt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  lt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 0 meridian'
          lngswitchflag=0
          if info.scene_centre_geodetic_coordinates[1] gt 180 then info.scene_centre_geodetic_coordinates[1]= lf.scene_centre_geodetic_coordinates[1]- 360
          if info.bottom_left_geodetic_coordinates[1]  gt 180 then info.bottom_left_geodetic_coordinates[1] = lf.bottom_left_geodetic_coordinates[1] - 360
          if info.bottom_right_geodetic_coordinates[1] gt 180 then info.bottom_right_geodetic_coordinates[1]= lf.bottom_right_geodetic_coordinates[1]- 360
          if info.top_left_geodetic_coordinates[1]     gt 180 then info.top_left_geodetic_coordinates[1]    = lf.top_left_geodetic_coordinates[1]    - 360
          if info.top_right_geodetic_coordinates[1]    gt 180 then info.top_right_geodetic_coordinates[1]   = lf.top_right_geodetic_coordinates[1]   - 360
        endif
  endif  
  
  
  
  temp_info=create_struct( info, $                            
          'calibration_constant'                   , calibration_constant                   , $ 
          'calibration_constant_estimation_utc'    , calibration_constant_estimation_utc    , $
          'beam_id'                                , beam_id                                , $
          'synthetic_aperture_duration'            , synthetic_aperture_duration            , $
          'pol'                                    , polarization                           , $
          'polarization'                           , polarization                           , $
          'prf'                                    , prf                                    , $
          'frequency_band'                         , 'X'                                    , $
          'producttype'                            , info.product_type                      , $
          'name_dir'                               , info.file                              , $
          'range_pixel'                            , info.column_spacing                    , $
          'azimuth_pixel'                          , info.line_spacing                      , $
          'sar_sensor'                             , info.satellite_id                      , $
          'platform_code'                          , platform_code                          , $
          'datumshiftparameters'                   , datumshiftparameters                   , $
          'proc_fac'                               , info.processing_center                 , $
          'software_version'                       , software_version                       , $ 
          'asc_des'                                , asc_des                                , $
          'beam_mode'                              , info.multi_beam_id                     , $
          'sensor_velocity'                        , sensor_velocity                        , $
          'nom_az_res'                             , info.azimuth_geometric_resolution      , $
          'nom_rg_res'                             , info.ground_range_geometric_resolution , $
          's_c_altitude'                           , info.satellite_height                  , $
          'rcvr_gain'                              , rcvr_gain                              , $  
          'proc_gain'                              , proc_gain                              , $
          'range_flag'                             , range_flag                             , $
          'flip'                                   , flip                                   , $
          'sar_imaging_mode'                       , info.multi_beam_id                     , $
          'datatake_id'                            , info.programmed_image_id               , $
          'valuetype'                              , valuetype                              , $
          'range_type'                             , range_type                             , $
          'heading'                                , info.scene_orientation                 , $
          'heading_rel'                            , 90.0                                   , $
          'illumination_dir'                       , 0.0                                    , $
          'early_azimuth_edge'                     , early_azimuth_edge                     , $
          'near_range_edge'                        , near_range_edge                        , $  
          'range_swath'                            , range_swath                            , $
          'azimuth_swath'                          , azimuth_swath                          , $
          'range_looks'                            , range_looks                            , $
          'azimuth_looks'                          , azimuth_looks                          , $
          'ellipsoidscale'                         , ellipsoidscale                         , $ 
          'ellipsoidname'                          , info.ellipsoid_designator              , $ 
          'semimajoraxis'                          , info.ellipsoid_semimajor_axis          , $ 
          'semiminoraxis'                          , info.ellipsoid_semiminor_axis          , $ 
          'slant_range_center'                     , slant_range_center                     , $
          'slant_range_near'                       , slant_range_near                       , $
          'integration_time'                       , integration_time                       , $
          'lngswitchflag'                          , lngswitchflag                          )
  
         
    info=      !NULL                                                           
    info= temp_info
    temp_info= !NULL  
   end
'HIMAGE': begin
  ; 
  ; HIMAGE info
  ;
  info={file                                             : tag_value( x,'_file'                                                ) , $  
        acquistion_mode                                  : tag_value( x,'acquisition_mode._data'                               ),  $         
        no_of_images                                     : 1L                                                                    , $  Assume we have one image
        icontype                                         : tag_value( x,'_icontype'                                            ) , $      
        leap_sign                                        : tag_value( x,'leap_sign._data'                                      ) , $        
        leap_utc                                         : tag_value( x,'leap_utc._data'                                       ) , $        
        orbit_number                                     : tag_value( x,'orbit_number._data'                                   ) , $    
        programmed_image_id                              : tag_value( x,'programmed_image_id._data'                            ) , $
        scene_sensing_start_utc                          : tag_value( x,'scene_sensing_start_utc._data'                        ) , $
        scene_sensing_stop_utc                           : tag_value( x,'scene_sensing_stop_utc._data'                         ) , $
        selective_availability_status                    : tag_value( x,'selective_availability_status._data'                  ) , $
        adc_characterization                             : tag_value( x,'adc_characterization._data'                           ) , $
        azimuth_polynomial_reference_time                : tag_value( x,'azimuth_polynomial_reference_time._data'              ) , $
        centroid_vs_range_time_polynomial                : tag_value( x,'centroid_vs_range_time_polynomial._data'              ) , $
        centroid_vs_azimuth_time_polynomial              : tag_value( x,'centroid_vs_azimuth_time_polynomial._data'            ) , $
        doppler_ambiguity_estimation_method              : tag_value( x,'doppler_ambiguity_estimation_method._data'            ) , $
        doppler_centroid_estimation_method               : tag_value( x,'doppler_centroid_estimation_method._data'             ) , $
        doppler_rate_estimation_method                   : tag_value( x,'doppler_rate_estimation_method._data'                 ) , $               
        doppler_rate_vs_azimuth_time_polynomial          : tag_value( x,'doppler_rate_vs_azimuth_time_polynomial._data'        ) , $         
        doppler_rate_vs_range_time_polynomial            : tag_value( x,'doppler_rate_vs_range_time_polynomial._data'          ) , $
        range_polynomial_reference_time                  : tag_value( x,'range_polynomial_reference_time._data'                ) , $
        bits_per_sample                                  : tag_value( x,'bits_per_sample._data'                                ) , $      
        columns_order                                    : tag_value( x,'columns_order._data'                                  ) , $        
        image_layers                                     : tag_value( x,'image_layers._data'                                   ) , $         
        image_scale                                      : tag_value( x,'image_scale._data'                                    ) , $          
        lines_order                                      : tag_value( x,'lines_order._data'                                    ) , $          
        sample_format                                    : tag_value( x,'sample_format._data'                                  ) , $        
        samples_per_pixel                                : tag_value( x,'samples_per_pixel._data'                              ) , $    
        acquisition_station_id                           : tag_value( x,'acquisition_station_id._data'                         ) , $
        delivery_mode                                    : tag_value( x,'delivery_mode._data'                                  ) , $        
        downlink_start_utc                               : tag_value( x,'downlink_start_utc._data'                             ) , $        
        downlink_stop_utc                                : tag_value( x,'downlink_stop_utc._data'                              ) , $         
        mission_id                                       : tag_value( x,'mission_id._data'                                     ) , $                
        processing_center                                : tag_value( x,'processing_centre._data'                              ) , $   ; Note the change in spelling      
        product_filename                                 : tag_value( x,'product_filename._data'                               ) , $          
        product_specification_document                   : tag_value( x,'product_specification_document._data'                 ) , $
        product_type                                     : tag_value( x,'product_type._data'                                   ) , $            
        satellite_id                                     : tag_value( x,'satellite_id._data'                                   ) , $            
        acquisition_mode                                 : tag_value( x,'acquisition_mode._data'                               ) , $        
        antenna_length                                   : tag_value( x,'antenna_length._data'                                 ) , $          
        azimuth_beamwidth                                : tag_value( x,'azimuth_beamwidth._data'                              ) , $       
        look_side                                        : tag_value( x,'look_side._data'                                      ) , $               
        multi_beam_id                                    : tag_value( x,'multi_beam_id._data'                                  ) , $           
        original_bit_quantisation                        : tag_value( x,'original_bit_quantisation._data'                      ) , $
        radar_frequency                                  : tag_value( x,'radar_frequency._data'                                ) , $             
        radar_wavelength                                 : tag_value( x,'radar_wavelength._data'                               ) , $            
        subswaths_number                                 : tag_value( x,'subswaths_number._data'                               ) , $            
        attitude_product_category                        : tag_value( x,'attitude_product_category._data'                      ) , $   
        doppler_ambiguity_confidence_measure             : tag_value( x,'doppler_ambiguity_confidence_measure._data'           ) , $
        doppler_centroid_confidence_measure              : tag_value( x,'doppler_centroid_confidence_measure._data'            ) , $
        doppler_centroid_estimation_accuracy             : tag_value( x,'doppler_centroid_estimation_accuracy._data'           ) , $
        doppler_rate_confidence_measure                  : tag_value( x,'doppler_rate_confidence_measure._data'                ) , $
        doppler_rate_estimation_accuracy                 : tag_value( x,'doppler_rate_estimation_accuracy._data'               ) , $
        pod_initial_position_accuracy                    : tag_value( x,'pod_initial_position_accuracy._data'                  ) , $    
        pod_overlap_position_accuracy                    : tag_value( x,'pod_overlap_position_accuracy._data'                  ) , $    
        pod_product_category                             : tag_value( x,'pod_product_category._data'                           ) , $             
        pod_quality_flag                                 : tag_value( x,'pod_quality_flag._data'                               ) , $                 
        attitude_quaternions                             : tag_value( x,'attitude_quaternions._data'                           ) , $             
        attitude_times                                   : tag_value( x,'attitude_times._data'                                 ) , $                  
        ecef_satellite_acceleration                      : tag_value( x,'ecef_satellite_acceleration._data'                    ) , $      
        ecef_satellite_position                          : tag_value( x,'ecef_satellite_position._data'                        ) , $          
        ecef_satellite_velocity                          : tag_value( x,'ecef_satellite_velocity._data'                        ) , $          
        inertial_reference_frame_id                      : tag_value( x,'inertial_reference_frame_id._data'                    ) , $      
        inertial_satellite_acceleration                  : tag_value( x,'inertial_satellite_acceleration._data'                ) , $  
        inertial_satellite_position                      : tag_value( x,'inertial_satellite_position._data'                    ) , $
        inertial_satellite_velocity                      : tag_value( x,'inertial_satellite_velocity._data'                    ) , $
        number_of_state_vectors                          : tag_value( x,'number_of_state_vectors._data'                        ) , $
        orbit_direction                                  : tag_value( x,'orbit_direction._data'                                ) , $
        pitch_rate                                       : tag_value( x,'pitch_rate._data'                                     ) , $
        roll_rate                                        : tag_value( x,'roll_rate._data'                                      ) , $
        satellite_height                                 : tag_value( x,'satellite_height._data'                               ) , $
        state_vectors_times                              : tag_value( x,'state_vectors_times._data'                            ) , $
        yaw_rate                                         : tag_value( x,'yaw_rate._data'                                       ) , $
        ecef_beam_centre_direction_for_processing        : tag_value( x,'ecef_beam_centre_direction_for_processing._data'      ) , $
        invalid_value                                    : tag_value( x,'invalid_value._data'                                  ) , $
        l0_software_version                              : tag_value( x,'l0_software_version._data'                            ) , $
        light_speed                                      : tag_value( x,'light_speed._data'                                    ) , $
        processing_history_bit_mask                      : tag_value( x,'processing_history_bit_mask._data'                    ) , $
        product_error_flag                               : tag_value( x,'product_error_flag._data'                             ) , $
        product_generation_utc                           : tag_value( x,'product_generation_utc._data'                         ) , $
        reference_utc                                    : tag_value( x,'reference_utc._data'                                  ) , $
        datum_rotation                                   : tag_value( x,'datum_rotation._data'                                 ) , $
        datum_scale                                      : tag_value( x,'datum_scale._data'                                    ) , $
        datum_shift                                      : tag_value( x,'datum_shift._data'                                    ) , $
        ellipsoid_designator                             : tag_value( x,'ellipsoid_designator._data'                           ) , $
        ellipsoid_semimajor_axis                         : tag_value( x,'ellipsoid_semimajor_axis._data'                       ) , $
        ellipsoid_semiminor_axis                         : tag_value( x,'ellipsoid_semiminor_axis._data'                       ) , $
        projection_id                                    : tag_value( x,'projection_id._data'                                  ) , $
        azimuth_coverage                                 : tag_value( x,'azimuth_coverage._data'                               ) , $
        centre_earth_radius                              : tag_value( x,'centre_earth_radius._data'                            ) , $
        ground_range_coverage                            : tag_value( x,'ground_range_coverage._data'                          ) , $
        scene_centre_geodetic_coordinates                : tag_value( x,'scene_centre_geodetic_coordinates._data'              ) , $
        scene_orientation                                : tag_value( x,'scene_orientation._data'                              ) , $
        doppler_ambiguity_confidence_measure_threshold   : tag_value( x,'doppler_ambiguity_confidence_measure_threshold._data' ) , $
        doppler_ambiguity_threshold                      : tag_value( x,'doppler_ambiguity_threshold._data'                    ) , $
        doppler_centroid_confidence_measure_threshold    : tag_value( x,'doppler_centroid_confidence_measure_threshold._data'  ) , $
        doppler_centroid_estimation_accuracy_threshold   : tag_value( x,'doppler_centroid_estimation_accuracy_threshold._data' ) , $
        doppler_rate_confidence_measure_threshold        : tag_value( x,'doppler_rate_confidence_measure_threshold._data'      ) , $
        doppler_rate_estimation_accuracy_threshold       : tag_value( x,'doppler_rate_estimation_accuracy_threshold._data'     ) , $
        raw_bias_threshold                               : tag_value( x,'raw_bias_threshold._data'                             ) , $
        raw_gain_imbalance_threshold                     : tag_value( x,'raw_gain_imbalance_threshold._data'                   ) , $
        raw_i_q_normality_threshold                      : tag_value( x,'raw_i_q_normality_threshold._data'                    ) , $
        raw_i_q_ortogonality_threshold                   : tag_value( x,'raw_i_q_ortogonality_threshold._data'                 ) , $
        raw_missing_lines_per_block_threshold            : tag_value( x,'raw_missing_lines_per_block_threshold._data'          ) , $
        raw_missing_lines_percentage_threshold           : tag_value( x,'raw_missing_lines_percentage_threshold._data'         ) , $
        raw_oversaturated_percentage_threshold           : tag_value( x,'raw_oversaturated_percentage_threshold._data'         ) , $
        raw_phase_uniformity_threshold                   : tag_value( x,'raw_phase_uniformity_threshold._data'                 ) , $
        raw_undersaturated_percentage_threshold          : tag_value( x,'raw_undersaturated_percentage_threshold._data'        ) , $
        rescaling_factor                                 : tag_value( x,'rescaling_factor._data'                               ) , $                                 
        adc_compensation                                 : tag_value( x,'adc_compensation._data'                               ) , $                                 
        antenna_pattern_compensation_reference_surface   : tag_value( x,'antenna_pattern_compensation_reference_surface._data' ) , $   
        azimuth_antenna_pattern_compensation_geometry    : tag_value( x,'azimuth_antenna_pattern_compensation_geometry._data'  ) , $    
        calibration_constant_compensation_flag           : tag_value( x,'calibration_constant_compensation_flag._data'         ) , $           
        incidence_angle_compensation_geometry            : tag_value( x,'incidence_angle_compensation_geometry._data'          ) , $            
        incidence_angle_compensation_reference_surface   : tag_value( x,'incidence_angle_compensation_reference_surface._data' ) , $   
        range_antenna_pattern_compensation_geometry      : tag_value( x,'range_antenna_pattern_compensation_geometry._data'    ) , $      
        range_spreading_loss_compensation_geometry       : tag_value( x,'range_spreading_loss_compensation_geometry._data'     ) , $       
        reference_incidence_angle                        : tag_value( x,'reference_incidence_angle._data'                      ) , $                        
        reference_slant_range                            : tag_value( x,'reference_slant_range._data'                          ) , $                            
        azimuth_focusing_weighting_coefficient           : tag_value( x,'azimuth_focusing_weighting_coefficient._data'         ) , $           
        azimuth_focusing_weighting_function              : tag_value( x,'azimuth_focusing_weighting_function._data'            ) , $              
        azimuth_processing_number_of_looks               : tag_value( x,'azimuth_processing_number_of_looks._data'             ) , $               
        focusing_algorithm_id                            : tag_value( x,'focusing_algorithm_id._data'                          ) , $                            
        l1a_software_version                             : tag_value( x,'l1a_software_version._data'                           ) , $                             
        multilook_filter_flag                            : tag_value( x,'multilook_filter_flag._data'                          ) , $                            
        range_focusing_weighting_coefficient             : tag_value( x,'range_focusing_weighting_coefficient._data'           ) , $             
        range_focusing_weighting_function                : tag_value( x,'range_focusing_weighting_function._data'              ) , $                
        range_processing_number_of_looks                 : tag_value( x,'range_processing_number_of_looks._data'               ) , $                 
        replica_reconstruction_method                    : tag_value( x,'replica_reconstruction_method._data'                  ) , $                    
        azimuth_geometric_resolution                     : tag_value( x,'azimuth_geometric_resolution._data'                   ) , $                     
        ground_range_geometric_resolution                : tag_value( x,'ground_range_geometric_resolution._data'              ) , $                
        inter_channel_co_registration                    : tag_value( x,'inter_channel_co_registration._data'                  ) , $                    
        irf_shape__10db__3db                             : tag_value( x,'irf_shape__10db__3db._data'                           ) , $                             
        irf_shape__6db__3db                              : tag_value( x,'irf_shape__6db__3db._data'                            ) , $                              
        islr                                             : tag_value( x,'islr._data'                                           ) , $                                             
        local_radiometric_stability                      : tag_value( x,'local_radiometric_stability._data'                    ) , $                      
        point_target_ambiguity                           : tag_value( x,'point_target_ambiguity._data'                         ) , $                           
        pslr                                             : tag_value( x,'pslr._data'                                           ) , $                                                                       
        sslr                                             : tag_value( x,'sslr._data'                                           ) , $                                             
        image_undersaturated_percentage_threshold        : tag_value( x,'image_undersaturated_percentage_threshold._data'      ) , $        
        image_oversaturated_percentage_threshold         : tag_value( x,'image_oversaturated_percentage_threshold._data'       ) , $         
        reference_slant_range_exponent                   : tag_value( x,'reference_slant_range_exponent._data'                 ) , $                   
        azimuth_multilooking_weighting_coefficient       : tag_value( x,'azimuth_multilooking_weighting_coefficient._data'     ) , $    
        azimuth_multilooking_weighting_function          : tag_value( x,'azimuth_multilooking_weighting_function._data'        ) , $       
        beam_mosaicking_policy                           : tag_value( x,'beam_mosaicking_policy._data'                         ) , $                        
        l1b_software_version                             : tag_value( x,'l1b_software_version._data'                           ) , $                          
        range_multilooking_weighting_coefficient         : tag_value( x,'range_multilooking_weighting_coefficient._data'       ) , $           
        ground_to_slant_polynomial                       : tag_value( x,'ground_to_slant_polynomial._data'                     ) , $                    
        slant_to_ground_polynomial                       : tag_value( x,'slant_to_ground_polynomial._data'                     ) , $                    
        spf_mean_intensity_ratio                         : tag_value( x,'spf_mean_intensity_ratio._data'                       ) , $                      
        spf_standard_deviation_intensity_ratio           : tag_value( x,'spf_standard_deviation_intensity_ratio._data'         ) , $        
        spf_target_contrast                              : tag_value( x,'spf_target_contrast._data'                            ) , $                           
        spf_targets                                      : tag_value( x,'spf_targets._data'                                    ) , $                                   
        spf_azimuth_moving_window                        : tag_value( x,'spf_azimuth_moving_window._data'                      ) , $                     
        spf_filter_type                                  : tag_value( x,'spf_filter_type._data'                                ) , $                               
        spf_iteration                                    : tag_value( x,'spf_iteration._data'                                  ) , $                                 
        spf_range_moving_window                          : tag_value( x,'spf_range_moving_window._data'                        ) , $                       
        spf_software_version                             : tag_value( x,'spf_software_version._data'                           ) , $                          
        geolocation_accuracy                             : tag_value( x,'geolocation_accuracy._data'                           ) , $                          
        geometric_conformity                             : tag_value( x,'geometric_conformity._data'                           ) , $                          
        zero_doppler_azimuth_first_time                  : tag_value( x,'s01.sbi.zero_doppler_azimuth_first_time._data'        ) , $  
        zero_doppler_azimuth_last_time                   : tag_value( x,'s01.sbi.zero_doppler_azimuth_last_time._data'         ) , $    
        zero_doppler_range_first_time                    : tag_value( x,'s01.sbi.zero_doppler_range_first_time._data'          ) , $      
        zero_doppler_range_last_time                     : tag_value( x,'s01.sbi.zero_doppler_range_last_time._data'           ) , $        
        image_max                                        : tag_value( x,'s01.sbi.image_max._data'                              ) , $                                              
        image_mean                                       : tag_value( x,'s01.sbi.image_mean._data'                             ) , $                                            
        image_min                                        : tag_value( x,'s01.sbi.image_min._data'                              ) , $                                              
        image_standard_deviation                         : tag_value( x,'s01.sbi.image_standard_deviation._data'               ) , $                
        column_spacing                                   : tag_value( x,'s01.sbi.column_spacing._data'                         ) , $                                    
        equivalent_number_of_looks                       : tag_value( x,'s01.sbi.equivalent_number_of_looks._data'             ) , $            
        line_spacing                                     : tag_value( x,'s01.sbi.line_spacing._data'                           ) , $                                        
        line_time_interval                               : tag_value( x,'s01.sbi.line_time_interval._data'                     ) , $                            
        bottom_left_geodetic_coordinates                 : tag_value( x,'s01.sbi.bottom_left_geodetic_coordinates._data'       ) , $ 
        bottom_right_geodetic_coordinates                : tag_value( x,'s01.sbi.bottom_right_geodetic_coordinates._data'      ) , $
        far_incidence_angle                              : tag_value( x,'s01.sbi.far_incidence_angle._data'                    ) , $                          
        far_look_angle                                   : tag_value( x,'s01.sbi.far_look_angle._data'                         ) , $                                    
        near_incidence_angle                             : tag_value( x,'s01.sbi.near_incidence_angle._data'                   ) , $                        
        near_look_angle                                  : tag_value( x,'s01.sbi.near_look_angle._data'                        ) , $                                  
        top_left_geodetic_coordinates                    : tag_value( x,'s01.sbi.top_left_geodetic_coordinates._data'          ) , $      
        top_right_geodetic_coordinates                   : tag_value( x,'s01.sbi.top_right_geodetic_coordinates._data'         ) , $    
        image_oversaturated_percentage                   : tag_value( x,'mbi.image_oversaturated_percentage._data'             ) , $    
        image_undersaturated_percentage                  : tag_value( x,'mbi.image_undersaturated_percentage._data'            ) , $  
        subswath_change_column                           : tag_value( x,'mbi.subswath_change_column._data'                     ) , $
        quick_look_columns_order                         : tag_value( x,'qlk.quick_look_columns_order._data'                   ) , $
        quick_look_lines_order                           : tag_value( x,'qlk.quick_look_lines_order._data'                     ) , $
        quick_look_column_spacing                        : tag_value( x,'qlk.quick_look_column_spacing._data'                  ) , $
        quick_look_line_spacing                          : tag_value( x,'qlk.quick_look_line_spacing._data'                    ) , $   
        quick_look_projection_id                         : tag_value( x,'qlk.quick_look_projection_id._data'                   ) }
  ; 
  ; HIMAGE Swaths.  Note that we expect a single swath, but we kept provisions for multiple swahts.
  ;
  n_swaths=                            tag_value( info, 'subswaths_number')
  if n_swaths eq'' then n_swaths=1
  calibration_constant=                dblarr(n_swaths)
  calibration_constant_estimation_utc= strarr(n_swaths)
  beam_id=                             strarr(n_swaths)
  synthetic_aperture_duration=         dblarr(n_swaths)
  polarisation=                        strarr(n_swaths)
  prf=                                 dblarr(n_swaths)
  for i=0L, n_swaths-1 do begin
    case i of
    0:    begin 
            calibration_constant[i]                = x.s01.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s01.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s01.beam_id._data
            synthetic_aperture_duration[i]         = x.s01.synthetic_aperture_duration._data
            polarisation[i]                        = x.s01.polarisation._data
            prf[i]                                 = x.s01.prf._data
          end
    1:    begin
            calibration_constant[i]                = x.s02.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s02.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s02.beam_id._data
            synthetic_aperture_duration[i]         = x.s02.synthetic_aperture_duration._data
            polarisation[i]                        = x.s02.polarisation._data
            prf[i]                                 = x.s02.prf._data
          end
    2:    begin
            calibration_constant[i]                = x.s03.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s03.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s03.beam_id._data
            synthetic_aperture_duration[i]         = x.s03.synthetic_aperture_duration._data
            polarisation[i]                        = x.s03.polarisation._data
            prf[i]                                 = x.s03.prf._data
          end
    3:    begin
            calibration_constant[i]                = x.s04.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s04.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s04.beam_id._data
            synthetic_aperture_duration[i]         = x.s04.synthetic_aperture_duration._data
            polarisation[i]                        = x.s04.polarisation._data
            prf[i]                                 = x.s04.prf._data
          end
    4:    begin
            calibration_constant[i]                = x.s05.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s05.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s05.beam_id._data
            synthetic_aperture_duration[i]         = x.s05.synthetic_aperture_duration._data         
            polarisation[i]                        = x.s05.polarisation._data
            prf[i]                                 = x.s05.prf._data                                 
          end
    5:    begin
            calibration_constant[i]                = x.s06.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s06.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s06.beam_id._data
            synthetic_aperture_duration[i]         = x.s06.synthetic_aperture_duration._data
            polarisation[i]                        = x.s06.polarisation._data
            prf[i]                                 = x.s06.prf._data
          end
    6:    begin
            calibration_constant[i]                = x.s07.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s07.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s07.beam_id._data
            synthetic_aperture_duration[i]         = x.s07.synthetic_aperture_duration._data
            polarisation[i]                        = x.s07.polarisation._data
            prf[i]                                 = x.s07.prf._data
          end
    7:    begin
            calibration_constant[i]                = x.s08.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s08.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s08.beam_id._data
            synthetic_aperture_duration[i]         = x.s08.synthetic_aperture_duration._data
            polarisation[i]                        = x.s08.polarisation._data
            prf[i]                                 = x.s08.prf._data
          end
    8:   begin
            calibration_constant[i]                         = x.s09.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s09.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s09.beam_id._data
            synthetic_aperture_duration[i]         = x.s09.synthetic_aperture_duration._data
            polarisation[i]                        = x.s09.polarisation._data
            prf[i]                                 = x.s09.prf._data
          end
    9:   begin
            calibration_constant[i]                         = x.s10.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s10.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s10.beam_id._data
            synthetic_aperture_duration[i]         = x.s10.synthetic_aperture_duration._data
            polarisation[i]                        = x.s10.polarisation._data
            prf[i]                                 = x.s10.prf._data
          end
    10:  begin
            calibration_constant[i]                = x.s11.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s11.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s11.beam_id._data
            synthetic_aperture_duration[i]         = x.s11.synthetic_aperture_duration._data
            polarisation[i]                        = x.s11.polarisation._data
            prf[i]                                 = x.s11.prf._data
          end
    11:  begin
            calibration_constant[i]                = x.s12.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s12.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s12.beam_id._data
            synthetic_aperture_duration[i]         = x.s12.synthetic_aperture_duration._data
            polarisation[i]                        = x.s12.polarisation._data
            prf[i]                                 = x.s12.prf._data
          end
          
    12:  begin
            calibration_constant[i]                = x.s13.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s13.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s13.beam_id._data
            synthetic_aperture_duration[i]         = x.s13.synthetic_aperture_duration._data
            polarisation[i]                        = x.s13.polarisation._data
            prf[i]                                 = x.s13.prf._data
          end
    ELSE: begin
          ; Do nothing
          end
    endcase
  endfor
  ;
  ; Flipping the image to correspond to what is considered top and bottom
  ;
  qlk   =   x.s01.qlk._data
  image =   x.s01.sbi._data 
  qlk   =   reverse(  qlk,  2)
  image =   reverse( image, 2)
  sz    =   size(image)
  nx    =   long(sz[1])
  ny    =   long(sz[2])
  
  info  =   create_struct( info          , $                            
                           'nx'    , nx  , $ 
                           'ny'    , ny  , $
                           'nx0'   , nx  , $
                           'ny0'   , ny  , $
                           'nx1'   , nx  , $
                           'ny1'   , ny   )
         
  ;
  ; Clean up variables for info structure
  ;
  ; Add elements with the correct names for the ancillary data file.
  ;
  polarization=         polarisation[0]  
  asc_des=              strupcase( strmid(info.orbit_direction,0,1) )
  sensor_velocity=      0.0
  range_flag=           0
  if strpos(info.projection_id, 'GROUND RANGE') ge 0  then range_flag=1
  rcvr_gain=            0.0
  proc_gain=            0.0 
  flip=                 0
  platform_code=        'C'+strmid(info.satellite_id,4,1)
  valuetype=            'intensity'
  range_type=           info.projection_id
  early_azimuth_edge=   'B'
  near_range_edge=      'L'
  if info.lines_order   ne 'EARLY-LATE' then early_azimuth_edge= 'T'
  if info.columns_order ne 'NEAR-FAR'   then near_range_edge=    'R'
  range_swath=          info.nx * info.line_spacing
  azimuth_swath=        info.ny * info.column_spacing
  azimuth_looks=        info.azimuth_processing_number_of_looks
  range_looks=          sqrt( info.equivalent_number_of_looks^2 / azimuth_looks^2)
  ellipsoidscale=       1
  datumshiftparameters= '0 0 0'
  ns=                   n_elements(info.datum_shift)
  if ns gt 0 then begin
    datumshiftparameters=''
    for i=0, ns-1 do datumshiftparameters=datumshiftparameters + string(info.datum_shift[i]) + ' '
  endif
  integration_time=     0.0D
  software_version=     'L0: '+ info.l0_software_version + ' L1A:' + info.l1a_software_version + ' L1B:' + info.l1b_software_version
  slant_range_center=   0.0D
  slant_range_near=     0.0D
  lngswitchflag=       0
  ;
  ; Checking the lngwitchflag
  
  if abs(info.scene_centre_geodetic_coordinates[1] )  gt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  gt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  gt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  gt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  gt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 180 meridian'
          lngswitchflag=1
          if info.scene_centre_geodetic_coordinates[1] lt 0 then info.scene_centre_geodetic_coordinates[1]= 360 + lf.scene_centre_geodetic_coordinates[1] 
          if info.bottom_left_geodetic_coordinates[1]  lt 0 then info.bottom_left_geodetic_coordinates[1] = 360 + lf.bottom_left_geodetic_coordinates[1]  
          if info.bottom_right_geodetic_coordinates[1] lt 0 then info.bottom_right_geodetic_coordinates[1]= 360 + lf.bottom_right_geodetic_coordinates[1] 
          if info.top_left_geodetic_coordinates[1]     lt 0 then info.top_left_geodetic_coordinates[1]    = 360 + lf.top_left_geodetic_coordinates[1]     
          if info.top_right_geodetic_coordinates[1]    lt 0 then info.top_right_geodetic_coordinates[1]   = 360 + lf.top_right_geodetic_coordinates[1]    
        endif
  endif  
  if abs(info.scene_centre_geodetic_coordinates[1] )  lt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  lt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  lt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  lt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  lt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 0 meridian'
          lngswitchflag=0
          if info.scene_centre_geodetic_coordinates[1] gt 180 then info.scene_centre_geodetic_coordinates[1]= lf.scene_centre_geodetic_coordinates[1]- 360
          if info.bottom_left_geodetic_coordinates[1]  gt 180 then info.bottom_left_geodetic_coordinates[1] = lf.bottom_left_geodetic_coordinates[1] - 360
          if info.bottom_right_geodetic_coordinates[1] gt 180 then info.bottom_right_geodetic_coordinates[1]= lf.bottom_right_geodetic_coordinates[1]- 360
          if info.top_left_geodetic_coordinates[1]     gt 180 then info.top_left_geodetic_coordinates[1]    = lf.top_left_geodetic_coordinates[1]    - 360
          if info.top_right_geodetic_coordinates[1]    gt 180 then info.top_right_geodetic_coordinates[1]   = lf.top_right_geodetic_coordinates[1]   - 360
        endif
  endif  
  info=create_struct( info, $                            
          'calibration_constant'                   , calibration_constant                   , $ 
          'calibration_constant_estimation_utc'    , calibration_constant_estimation_utc    , $
          'beam_id'                                , beam_id                                , $
          'synthetic_aperture_duration'            , synthetic_aperture_duration            , $
          'polarization'                           , polarization                           , $
          'pol'                                    , polarization[0]                        , $
          'prf'                                    , prf                                    , $
          'frequency_band'                         , 'X'                                    , $
          'producttype'                            , info.product_type                      , $
          'name_dir'                               , info.file                              , $
          'range_pixel'                            , info.column_spacing                    , $
          'azimuth_pixel'                          , info.line_spacing                      , $
          'sar_sensor'                             , info.satellite_id                      , $
          'platform_code'                          , platform_code                          , $
          'datumshiftparameters'                   , datumshiftparameters                   , $
          'proc_fac'                               , info.processing_center                 , $
          'software_version'                       , software_version                       , $ 
          'asc_des'                                , asc_des                                , $
          'beam_mode'                              , info.multi_beam_id                     , $
          'sensor_velocity'                        , sensor_velocity                        , $
          'nom_az_res'                             , info.azimuth_geometric_resolution      , $
          'nom_rg_res'                             , info.ground_range_geometric_resolution , $
          's_c_altitude'                           , info.satellite_height                  , $
          'rcvr_gain'                              , rcvr_gain                              , $  
          'proc_gain'                              , proc_gain                              , $
          'range_flag'                             , range_flag                             , $
          'flip'                                   , flip                                   , $
          'sar_imaging_mode'                       , info.multi_beam_id                     , $
          'datatake_id'                            , info.programmed_image_id               , $
          'valuetype'                              , valuetype                              , $
          'range_type'                             , range_type                             , $
          'heading'                                , info.scene_orientation                 , $
          'heading_rel'                            , 90.0                                   , $
          'illumination_dir'                       , 0.0                                    , $
          'early_azimuth_edge'                     , early_azimuth_edge                     , $
          'near_range_edge'                        , near_range_edge                        , $  
          'range_swath'                            , range_swath                            , $
          'azimuth_swath'                          , azimuth_swath                          , $
          'range_looks'                            , range_looks                            , $
          'azimuth_looks'                          , azimuth_looks                          , $
          'ellipsoidscale'                         , ellipsoidscale                         , $ 
          'ellipsoidname'                          , info.ellipsoid_designator              , $ 
          'semimajoraxis'                          , info.ellipsoid_semimajor_axis          , $ 
          'semiminoraxis'                          , info.ellipsoid_semiminor_axis          , $ 
          'slant_range_center'                     , slant_range_center                     , $
          'slant_range_near'                       , slant_range_near                       , $
          'integration_time'                       , integration_time                       , $
          'lngswitchflag'                          , lngswitchflag                          )
        end
'WIDEREGION': begin         
  ;=====================================================================
  ;                WIDE REGION IMAGE FILE                                    
  ;=====================================================================
    if printflag then print, 'Parsing variables from WIDEREGION file'

  info={file                                             : tag_value( x,'_file'                                                ) , $  
        acquistion_mode                                  : tag_value( x,'acquisition_mode._data'                               ),  $         
        no_of_images                                     : 1L                                                                    , $  Assume we have one image
        icontype                                         : tag_value( x,'_icontype'                                            ) , $      
        leap_sign                                        : tag_value( x,'leap_sign._data'                                      ) , $        
        leap_utc                                         : tag_value( x,'leap_utc._data'                                       ) , $        
        orbit_number                                     : tag_value( x,'orbit_number._data'                                   ) , $    
        programmed_image_id                              : tag_value( x,'programmed_image_id._data'                            ) , $
        scene_sensing_start_utc                          : tag_value( x,'scene_sensing_start_utc._data'                        ) , $
        scene_sensing_stop_utc                           : tag_value( x,'scene_sensing_stop_utc._data'                         ) , $
        selective_availability_status                    : tag_value( x,'selective_availability_status._data'                  ) , $
        azimuth_polynomial_reference_time                : tag_value( x,'azimuth_polynomial_reference_time._data'              ) , $
        centroid_vs_range_time_polynomial                : tag_value( x,'centroid_vs_range_time_polynomial._data'              ) , $
        centroid_vs_azimuth_time_polynomial              : tag_value( x,'centroid_vs_azimuth_time_polynomial._data'            ) , $
        doppler_ambiguity_estimation_method              : tag_value( x,'doppler_ambiguity_estimation_method._data'            ) , $
        doppler_centroid_estimation_method               : tag_value( x,'doppler_centroid_estimation_method._data'             ) , $
        doppler_rate_estimation_method                   : tag_value( x,'doppler_rate_estimation_method._data'                 ) , $               
        doppler_rate_vs_azimuth_time_polynomial          : tag_value( x,'doppler_rate_vs_azimuth_time_polynomial._data'        ) , $         
        doppler_rate_vs_range_time_polynomial            : tag_value( x,'doppler_rate_vs_range_time_polynomial._data'          ) , $
        range_polynomial_reference_time                  : tag_value( x,'range_polynomial_reference_time._data'                ) , $
        bits_per_sample                                  : tag_value( x,'bits_per_sample._data'                                ) , $      
        columns_order                                    : tag_value( x,'columns_order._data'                                  ) , $        
        image_layers                                     : tag_value( x,'image_layers._data'                                   ) , $         
        image_scale                                      : tag_value( x,'image_scale._data'                                    ) , $          
        lines_order                                      : tag_value( x,'lines_order._data'                                    ) , $          
        sample_format                                    : tag_value( x,'sample_format._data'                                  ) , $        
        samples_per_pixel                                : tag_value( x,'samples_per_pixel._data'                              ) , $    
        acquisition_station_id                           : tag_value( x,'acquisition_station_id._data'                         ) , $
        delivery_mode                                    : tag_value( x,'delivery_mode._data'                                  ) , $        
        downlink_start_utc                               : tag_value( x,'downlink_start_utc._data'                             ) , $        
        downlink_stop_utc                                : tag_value( x,'downlink_stop_utc._data'                              ) , $         
        mission_id                                       : tag_value( x,'mission_id._data'                                     ) , $                
        processing_center                                : tag_value( x,'processing_centre._data'                              ) , $   ; Note the change in spelling      
        product_filename                                 : tag_value( x,'product_filename._data'                               ) , $          
        product_specification_document                   : tag_value( x,'product_specification_document._data'                 ) , $
        product_type                                     : tag_value( x,'product_type._data'                                   ) , $            
        satellite_id                                     : tag_value( x,'satellite_id._data'                                   ) , $            
        acquisition_mode                                 : tag_value( x,'acquisition_mode._data'                               ) , $        
        antenna_length                                   : tag_value( x,'antenna_length._data'                                 ) , $          
        azimuth_beamwidth                                : tag_value( x,'azimuth_beamwidth._data'                              ) , $       
        look_side                                        : tag_value( x,'look_side._data'                                      ) , $               
        multi_beam_id                                    : tag_value( x,'multi_beam_id._data'                                  ) , $           
        original_bit_quantisation                        : tag_value( x,'original_bit_quantisation._data'                      ) , $
        radar_frequency                                  : tag_value( x,'radar_frequency._data'                                ) , $             
        radar_wavelength                                 : tag_value( x,'radar_wavelength._data'                               ) , $            
        subswaths_number                                 : tag_value( x,'subswaths_number._data'                               ) , $            
        attitude_product_category                        : tag_value( x,'attitude_product_category._data'                      ) , $   
        doppler_ambiguity_confidence_measure             : tag_value( x,'doppler_ambiguity_confidence_measure._data'           ) , $
        doppler_centroid_confidence_measure              : tag_value( x,'doppler_centroid_confidence_measure._data'            ) , $
        doppler_centroid_estimation_accuracy             : tag_value( x,'doppler_centroid_estimation_accuracy._data'           ) , $
        doppler_rate_confidence_measure                  : tag_value( x,'doppler_rate_confidence_measure._data'                ) , $
        doppler_rate_estimation_accuracy                 : tag_value( x,'doppler_rate_estimation_accuracy._data'               ) , $
        pod_initial_position_accuracy                    : tag_value( x,'pod_initial_position_accuracy._data'                  ) , $    
        pod_overlap_position_accuracy                    : tag_value( x,'pod_overlap_position_accuracy._data'                  ) , $    
        pod_product_category                             : tag_value( x,'pod_product_category._data'                           ) , $             
        pod_quality_flag                                 : tag_value( x,'pod_quality_flag._data'                               ) , $                 
        attitude_quaternions                             : tag_value( x,'attitude_quaternions._data'                           ) , $             
        attitude_times                                   : tag_value( x,'attitude_times._data'                                 ) , $                  
        ecef_satellite_acceleration                      : tag_value( x,'ecef_satellite_acceleration._data'                    ) , $      
        ecef_satellite_position                          : tag_value( x,'ecef_satellite_position._data'                        ) , $          
        ecef_satellite_velocity                          : tag_value( x,'ecef_satellite_velocity._data'                        ) , $          
        inertial_reference_frame_id                      : tag_value( x,'inertial_reference_frame_id._data'                    ) , $      
        inertial_satellite_acceleration                  : tag_value( x,'inertial_satellite_acceleration._data'                ) , $  
        inertial_satellite_position                      : tag_value( x,'inertial_satellite_position._data'                    ) , $
        inertial_satellite_velocity                      : tag_value( x,'inertial_satellite_velocity._data'                    ) , $
        number_of_state_vectors                          : tag_value( x,'number_of_state_vectors._data'                        ) , $
        orbit_direction                                  : tag_value( x,'orbit_direction._data'                                ) , $
        pitch_rate                                       : tag_value( x,'pitch_rate._data'                                     ) , $
        roll_rate                                        : tag_value( x,'roll_rate._data'                                      ) , $
        satellite_height                                 : tag_value( x,'satellite_height._data'                               ) , $
        state_vectors_times                              : tag_value( x,'state_vectors_times._data'                            ) , $
        yaw_rate                                         : tag_value( x,'yaw_rate._data'                                       ) , $
        ecef_beam_centre_direction_for_processing        : tag_value( x,'ecef_beam_centre_direction_for_processing._data'      ) , $
        invalid_value                                    : tag_value( x,'invalid_value._data'                                  ) , $
        l0_software_version                              : tag_value( x,'l0_software_version._data'                            ) , $
        light_speed                                      : tag_value( x,'light_speed._data'                                    ) , $
        processing_history_bit_mask                      : tag_value( x,'processing_history_bit_mask._data'                    ) , $
        product_error_flag                               : tag_value( x,'product_error_flag._data'                             ) , $
        product_generation_utc                           : tag_value( x,'product_generation_utc._data'                         ) , $
        reference_utc                                    : tag_value( x,'reference_utc._data'                                  ) , $
        datum_rotation                                   : tag_value( x,'datum_rotation._data'                                 ) , $
        datum_scale                                      : tag_value( x,'datum_scale._data'                                    ) , $
        datum_shift                                      : tag_value( x,'datum_shift._data'                                    ) , $
        ellipsoid_designator                             : tag_value( x,'ellipsoid_designator._data'                           ) , $
        ellipsoid_semimajor_axis                         : tag_value( x,'ellipsoid_semimajor_axis._data'                       ) , $
        ellipsoid_semiminor_axis                         : tag_value( x,'ellipsoid_semiminor_axis._data'                       ) , $
        projection_id                                    : tag_value( x,'projection_id._data'                                  ) , $
        azimuth_coverage                                 : tag_value( x,'azimuth_coverage._data'                               ) , $
        centre_earth_radius                              : tag_value( x,'centre_earth_radius._data'                            ) , $
        ground_range_coverage                            : tag_value( x,'ground_range_coverage._data'                          ) , $
        scene_centre_geodetic_coordinates                : tag_value( x,'scene_centre_geodetic_coordinates._data'              ) , $
        scene_orientation                                : tag_value( x,'scene_orientation._data'                              ) , $
        doppler_ambiguity_confidence_measure_threshold   : tag_value( x,'doppler_ambiguity_confidence_measure_threshold._data' ) , $
        doppler_ambiguity_threshold                      : tag_value( x,'doppler_ambiguity_threshold._data'                    ) , $
        doppler_centroid_confidence_measure_threshold    : tag_value( x,'doppler_centroid_confidence_measure_threshold._data'  ) , $
        doppler_centroid_estimation_accuracy_threshold   : tag_value( x,'doppler_centroid_estimation_accuracy_threshold._data' ) , $
        doppler_rate_confidence_measure_threshold        : tag_value( x,'doppler_rate_confidence_measure_threshold._data'      ) , $
        doppler_rate_estimation_accuracy_threshold       : tag_value( x,'doppler_rate_estimation_accuracy_threshold._data'     ) , $
        raw_bias_threshold                               : tag_value( x,'raw_bias_threshold._data'                             ) , $
        raw_gain_imbalance_threshold                     : tag_value( x,'raw_gain_imbalance_threshold._data'                   ) , $
        raw_i_q_normality_threshold                      : tag_value( x,'raw_i_q_normality_threshold._data'                    ) , $
        raw_i_q_ortogonality_threshold                   : tag_value( x,'raw_i_q_ortogonality_threshold._data'                 ) , $
        raw_missing_lines_per_block_threshold            : tag_value( x,'raw_missing_lines_per_block_threshold._data'          ) , $
        raw_missing_lines_percentage_threshold           : tag_value( x,'raw_missing_lines_percentage_threshold._data'         ) , $
        raw_oversaturated_percentage_threshold           : tag_value( x,'raw_oversaturated_percentage_threshold._data'         ) , $
        raw_phase_uniformity_threshold                   : tag_value( x,'raw_phase_uniformity_threshold._data'                 ) , $
        raw_undersaturated_percentage_threshold          : tag_value( x,'raw_undersaturated_percentage_threshold._data'        ) , $
        rescaling_factor                                 : tag_value( x,'rescaling_factor._data'                               ) , $                                 
        adc_compensation                                 : tag_value( x,'adc_compensation._data'                               ) , $                                 
        antenna_pattern_compensation_reference_surface   : tag_value( x,'antenna_pattern_compensation_reference_surface._data' ) , $   
        azimuth_antenna_pattern_compensation_geometry    : tag_value( x,'azimuth_antenna_pattern_compensation_geometry._data'  ) , $    
        calibration_constant_compensation_flag           : tag_value( x,'calibration_constant_compensation_flag._data'         ) , $           
        incidence_angle_compensation_geometry            : tag_value( x,'incidence_angle_compensation_geometry._data'          ) , $            
        incidence_angle_compensation_reference_surface   : tag_value( x,'incidence_angle_compensation_reference_surface._data' ) , $   
        range_antenna_pattern_compensation_geometry      : tag_value( x,'range_antenna_pattern_compensation_geometry._data'    ) , $      
        range_spreading_loss_compensation_geometry       : tag_value( x,'range_spreading_loss_compensation_geometry._data'     ) , $       
        reference_incidence_angle                        : tag_value( x,'reference_incidence_angle._data'                      ) , $                        
        reference_slant_range                            : tag_value( x,'reference_slant_range._data'                          ) , $                            
        azimuth_focusing_weighting_coefficient           : tag_value( x,'azimuth_focusing_weighting_coefficient._data'         ) , $           
        azimuth_focusing_weighting_function              : tag_value( x,'azimuth_focusing_weighting_function._data'            ) , $              
        azimuth_processing_number_of_looks               : tag_value( x,'azimuth_processing_number_of_looks._data'             ) , $               
        focusing_algorithm_id                            : tag_value( x,'focusing_algorithm_id._data'                          ) , $                            
        l1a_software_version                             : tag_value( x,'l1a_software_version._data'                           ) , $                             
        multilook_filter_flag                            : tag_value( x,'multilook_filter_flag._data'                          ) , $                            
        range_focusing_weighting_coefficient             : tag_value( x,'range_focusing_weighting_coefficient._data'           ) , $             
        range_focusing_weighting_function                : tag_value( x,'range_focusing_weighting_function._data'              ) , $                
        range_processing_number_of_looks                 : tag_value( x,'range_processing_number_of_looks._data'               ) , $                 
        replica_reconstruction_method                    : tag_value( x,'replica_reconstruction_method._data'                  ) , $                    
        azimuth_geometric_resolution                     : tag_value( x,'azimuth_geometric_resolution._data'                   ) , $                     
        ground_range_geometric_resolution                : tag_value( x,'ground_range_geometric_resolution._data'              ) , $                                  
        irf_shape__10db__3db                             : tag_value( x,'irf_shape__10db__3db._data'                           ) , $                             
        irf_shape__6db__3db                              : tag_value( x,'irf_shape__6db__3db._data'                            ) , $                              
        islr                                             : tag_value( x,'islr._data'                                           ) , $                                             
        local_radiometric_stability                      : tag_value( x,'local_radiometric_stability._data'                    ) , $                      
        point_target_ambiguity                           : tag_value( x,'point_target_ambiguity._data'                         ) , $                           
        pslr                                             : tag_value( x,'pslr._data'                                           ) , $                                                                      
        sslr                                             : tag_value( x,'sslr._data'                                           ) , $                                             
        image_undersaturated_percentage_threshold        : tag_value( x,'image_undersaturated_percentage_threshold._data'      ) , $        
        image_oversaturated_percentage_threshold         : tag_value( x,'image_oversaturated_percentage_threshold._data'       ) , $         
        reference_slant_range_exponent                   : tag_value( x,'reference_slant_range_exponent._data'                 ) , $                   
        azimuth_multilooking_weighting_coefficient       : tag_value( x,'azimuth_multilooking_weighting_coefficient._data'     ) , $    
        azimuth_multilooking_weighting_function          : tag_value( x,'azimuth_multilooking_weighting_function._data'        ) , $       
        beam_mosaicking_policy                           : tag_value( x,'beam_mosaicking_policy._data'                         ) , $                        
        l1b_software_version                             : tag_value( x,'l1b_software_version._data'                           ) , $                          
        range_multilooking_weighting_coefficient         : tag_value( x,'range_multilooking_weighting_coefficient._data'       ) , $           
        ground_to_slant_polynomial                       : tag_value( x,'ground_to_slant_polynomial._data'                     ) , $                    
        slant_to_ground_polynomial                       : tag_value( x,'slant_to_ground_polynomial._data'                     ) , $                                     
        spf_azimuth_moving_window                        : tag_value( x,'spf_azimuth_moving_window._data'                      ) , $                     
        spf_filter_type                                  : tag_value( x,'spf_filter_type._data'                                ) , $                               
        spf_iteration                                    : tag_value( x,'spf_iteration._data'                                  ) , $                                 
        spf_range_moving_window                          : tag_value( x,'spf_range_moving_window._data'                        ) , $                       
        spf_software_version                             : tag_value( x,'spf_software_version._data'                           ) , $                          
        geolocation_accuracy                             : tag_value( x,'geolocation_accuracy._data'                           ) , $                          
        geometric_conformity                             : tag_value( x,'geometric_conformity._data'                           ) , $                          
        zero_doppler_azimuth_first_time                  : tag_value( x,'mbi.zero_doppler_azimuth_first_time._data'            ) , $  
        zero_doppler_azimuth_last_time                   : tag_value( x,'mbi.zero_doppler_azimuth_last_time._data'             ) , $    
        zero_doppler_range_first_time                    : tag_value( x,'mbi.zero_doppler_range_first_time._data'              ) , $      
        zero_doppler_range_last_time                     : tag_value( x,'mbi.zero_doppler_range_last_time._data'               ) , $        
        image_max                                        : tag_value( x,'mbi.image_max._data'                                  ) , $                                              
        image_mean                                       : tag_value( x,'mbi.image_mean._data'                                 ) , $                                            
        image_min                                        : tag_value( x,'mbi.image_min._data'                                  ) , $                                              
        image_standard_deviation                         : tag_value( x,'mbi.image_standard_deviation._data'                   ) , $                
        column_spacing                                   : tag_value( x,'mbi.column_spacing._data'                             ) , $                                    
        equivalent_number_of_looks                       : tag_value( x,'mbi.equivalent_number_of_looks._data'                 ) , $            
        line_spacing                                     : tag_value( x,'mbi.line_spacing._data'                               ) , $                                        
        ;line_time_interval                               : tag_value( x,'mbi.line_time_interval._data'                         ) , $                            
        bottom_left_geodetic_coordinates                 : tag_value( x,'mbi.bottom_left_geodetic_coordinates._data'           ) , $ 
        bottom_right_geodetic_coordinates                : tag_value( x,'mbi.bottom_right_geodetic_coordinates._data'          ) , $
        far_incidence_angle                              : tag_value( x,'mbi.far_incidence_angle._data'                        ) , $                          
        far_look_angle                                   : tag_value( x,'mbi.far_look_angle._data'                             ) , $                                    
        near_incidence_angle                             : tag_value( x,'mbi.near_incidence_angle._data'                       ) , $                        
        near_look_angle                                  : tag_value( x,'mbi.near_look_angle._data'                            ) , $                                  
        top_left_geodetic_coordinates                    : tag_value( x,'mbi.top_left_geodetic_coordinates._data'              ) , $      
        top_right_geodetic_coordinates                   : tag_value( x,'mbi.top_right_geodetic_coordinates._data'             ) , $    
        image_oversaturated_percentage                   : tag_value( x,'mbi.image_oversaturated_percentage._data'             ) , $    
        image_undersaturated_percentage                  : tag_value( x,'mbi.image_undersaturated_percentage._data'            ) , $  
        ;subswath_change_column                           : tag_value( x,'mbi.subswath_change_column._data'                     ) , $
        quick_look_columns_order                         : tag_value( x,'qlk.quick_look_columns_order._data'                   ) , $
        quick_look_lines_order                           : tag_value( x,'qlk.quick_look_lines_order._data'                     ) , $
        quick_look_column_spacing                        : tag_value( x,'qlk.quick_look_column_spacing._data'                  ) , $
        quick_look_line_spacing                          : tag_value( x,'qlk.quick_look_line_spacing._data'                    ) , $   
        quick_look_projection_id                         : tag_value( x,'qlk.quick_look_projection_id._data'                   ) }
      
 
  qlk=  tag_value(x,'qlk._data')        ; Quick look image
  ;
  ; Added 2011 Jan 24
  ; Flipping the image to correspond to what is considered top and bottom
  ;
  if n_elements(qlk) gt 1 then qlk= reverse(qlk,2)
  ;
  ; Calibrated subswaths
  ;
  n_swaths=                         info.subswaths_number
  if info.subswaths_number eq'' then n_swaths=1
  calibration_constant=                dblarr(n_swaths)
  calibration_constant_estimation_utc= strarr(n_swaths)
  beam_id=                             strarr(n_swaths)
  synthetic_aperture_duration=         dblarr(n_swaths)
  polarisation=                        strarr(n_swaths)
  prf=                                 dblarr(n_swaths)
  for i=0L, n_swaths-1 do begin
    case i of
    0:    begin 
            calibration_constant[i]                = x.s01.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s01.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s01.beam_id._data
            synthetic_aperture_duration[i]         = x.s01.synthetic_aperture_duration._data
            polarisation[i]                        = x.s01.polarisation._data
            prf[i]                                 = x.s01.prf._data
          end
    1:    begin
            calibration_constant[i]                = x.s02.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s02.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s02.beam_id._data
            synthetic_aperture_duration[i]         = x.s02.synthetic_aperture_duration._data
            polarisation[i]                        = x.s02.polarisation._data
            prf[i]                                 = x.s02.prf._data
          end
    2:    begin
            calibration_constant[i]                = x.s03.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s03.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s03.beam_id._data
            synthetic_aperture_duration[i]         = x.s03.synthetic_aperture_duration._data
            polarisation[i]                        = x.s03.polarisation._data
            prf[i]                                 = x.s03.prf._data
          end
    3:    begin
            calibration_constant[i]                = x.s04.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s04.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s04.beam_id._data
            synthetic_aperture_duration[i]         = x.s04.synthetic_aperture_duration._data
            polarisation[i]                        = x.s04.polarisation._data
            prf[i]                                 = x.s04.prf._data
          end
    4:    begin
            calibration_constant[i]                = x.s05.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s05.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s05.beam_id._data
            synthetic_aperture_duration[i]         = x.s05.synthetic_aperture_duration._data         
            polarisation[i]                        = x.s05.polarisation._data
            prf[i]                                 = x.s05.prf._data                                 
          end
    5:    begin
            calibration_constant[i]                = x.s06.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s06.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s06.beam_id._data
            synthetic_aperture_duration[i]         = x.s06.synthetic_aperture_duration._data
            polarisation[i]                        = x.s06.polarisation._data
            prf[i]                                 = x.s06.prf._data
          end
    6:    begin
            calibration_constant[i]                = x.s07.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s07.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s07.beam_id._data
            synthetic_aperture_duration[i]         = x.s07.synthetic_aperture_duration._data
            polarisation[i]                        = x.s07.polarisation._data
            prf[i]                                 = x.s07.prf._data
          end
    7:    begin
            calibration_constant[i]                = x.s08.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s08.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s08.beam_id._data
            synthetic_aperture_duration[i]         = x.s08.synthetic_aperture_duration._data
            polarisation[i]                        = x.s08.polarisation._data
            prf[i]                                 = x.s08.prf._data
          end
    8:   begin
            calibration_constant[i]                         = x.s09.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s09.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s09.beam_id._data
            synthetic_aperture_duration[i]         = x.s09.synthetic_aperture_duration._data
            polarisation[i]                        = x.s09.polarisation._data
            prf[i]                                 = x.s09.prf._data
          end
    9:   begin
            calibration_constant[i]                         = x.s10.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s10.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s10.beam_id._data
            synthetic_aperture_duration[i]         = x.s10.synthetic_aperture_duration._data
            polarisation[i]                        = x.s10.polarisation._data
            prf[i]                                 = x.s10.prf._data
          end
    10:  begin
            calibration_constant[i]                = x.s11.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s11.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s11.beam_id._data
            synthetic_aperture_duration[i]         = x.s11.synthetic_aperture_duration._data
            polarisation[i]                        = x.s11.polarisation._data
            prf[i]                                 = x.s11.prf._data
          end
    11:  begin
            calibration_constant[i]                = x.s12.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s12.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s12.beam_id._data
            synthetic_aperture_duration[i]         = x.s12.synthetic_aperture_duration._data
            polarisation[i]                        = x.s12.polarisation._data
            prf[i]                                 = x.s12.prf._data
          end
          
    12:  begin
            calibration_constant[i]                = x.s13.calibration_constant._data
            calibration_constant_estimation_utc[i] = x.s13.calibration_constant_estimation_utc._data
            beam_id[i]                             = x.s13.beam_id._data
            synthetic_aperture_duration[i]         = x.s13.synthetic_aperture_duration._data
            polarisation[i]                        = x.s13.polarisation._data
            prf[i]                                 = x.s13.prf._data
          end
    ELSE: begin
          ; Do nothing
          end
    endcase
  endfor
  
  image=   x.mbi._data
  ;
  ; Added 2011 Jan 24
  ; Flipping the image to correspond to what is considered top and bottom
  ;
  image= reverse(image, 2)
  
  sz=     size(image)
  nx=     long(sz[1])
  ny=     long(sz[2])
  
  temp_info=create_struct( info, $                            
          'nx'                                    , nx                  , $ 
          'ny'                                    , ny                  , $
          'nx0'                                   , nx                  , $
          'ny0'                                   , ny                  , $
          'nx1'                                   , nx                  , $
          'ny1'                                   , ny                   )
         
  info=      !NULL                                                           
  info= temp_info
  temp_info= !NULL
  
  ; Clean up variables for info structure
  ;
  ; Add elements with the correct names for the ancillary data file.
  ;
  polarization=         'BLANK'
  if n_elements(polarization) gt 0 then polarization=         polarisation  
  asc_des=              strupcase( strmid(info.orbit_direction,0,1) )
  sensor_velocity=      0.0
  range_flag=           0
  if strpos(info.projection_id, 'GROUND RANGE') ge 0  then range_flag=1
  rcvr_gain=            0.0
  proc_gain=            0.0 
  flip=                 0
  platform_code=        'C'+strmid(info.satellite_id,4,1)
  valuetype=            'intensity'
  range_type=           info.projection_id
  early_azimuth_edge=   'B'
  near_range_edge=      'L'
  if info.lines_order   ne 'EARLY-LATE' then early_azimuth_edge= 'T'
  if info.columns_order ne 'NEAR-FAR'   then near_range_edge=    'R'
  range_swath=          info.nx * info.line_spacing
  azimuth_swath=        info.ny * info.column_spacing
  azimuth_looks=        info.azimuth_processing_number_of_looks
  range_looks=          sqrt( info.equivalent_number_of_looks^2 / azimuth_looks^2)
  ellipsoidscale=       1
  datumshiftparameters= '0 0 0'
  ns=                   n_elements(info.datum_shift)
  if ns gt 0 then begin
    datumshiftparameters=''
    for i=0, ns-1 do datumshiftparameters=datumshiftparameters + string(info.datum_shift[i]) + ' '
  endif
  integration_time=     0.0D
  software_version=     'L0: '+ info.l0_software_version + ' L1A:' + info.l1a_software_version + ' L1B:' + info.l1b_software_version
  slant_range_center=   0.0D
  slant_range_near=     0.0D
  lngswitchflag=       0
  ;
  ; Checking the lngwitchflag
  
  if abs(info.scene_centre_geodetic_coordinates[1] )  gt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  gt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  gt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  gt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  gt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 180 meridian'
          lngswitchflag=1
          if info.scene_centre_geodetic_coordinates[1] lt 0 then info.scene_centre_geodetic_coordinates[1]= 360 + lf.scene_centre_geodetic_coordinates[1] 
          if info.bottom_left_geodetic_coordinates[1]  lt 0 then info.bottom_left_geodetic_coordinates[1] = 360 + lf.bottom_left_geodetic_coordinates[1]  
          if info.bottom_right_geodetic_coordinates[1] lt 0 then info.bottom_right_geodetic_coordinates[1]= 360 + lf.bottom_right_geodetic_coordinates[1] 
          if info.top_left_geodetic_coordinates[1]     lt 0 then info.top_left_geodetic_coordinates[1]    = 360 + lf.top_left_geodetic_coordinates[1]     
          if info.top_right_geodetic_coordinates[1]    lt 0 then info.top_right_geodetic_coordinates[1]   = 360 + lf.top_right_geodetic_coordinates[1]    
        endif
  endif  
  if abs(info.scene_centre_geodetic_coordinates[1] )  lt 90 and $
        (info.bottom_left_geodetic_coordinates[1]  )  lt 90 and $
        (info.bottom_right_geodetic_coordinates[1] )  lt 90 and $
        (info.top_left_geodetic_coordinates[1]     )  lt 90 and $
        (info.top_right_geodetic_coordinates[1]    )  lt 90 then begin
        signarr=sign([info.scene_centre_geodetic_coordinates[1]  , $
                      info.bottom_left_geodetic_coordinates[1]   , $
                      info.bottom_right_geodetic_coordinates[1]  , $
                      info.top_left_geodetic_coordinates[1]      , $
                      info.top_right_geodetic_coordinates[1]     ])
        maxsign=max(signarr)
        minsign=min(signarr)
        if minsign lt 0 and maxsign gt 0 then begin 
          if printflag gt 1 then print, 'Data crosses 0 meridian'
          lngswitchflag=0
          if info.scene_centre_geodetic_coordinates[1] gt 180 then info.scene_centre_geodetic_coordinates[1]= lf.scene_centre_geodetic_coordinates[1]- 360
          if info.bottom_left_geodetic_coordinates[1]  gt 180 then info.bottom_left_geodetic_coordinates[1] = lf.bottom_left_geodetic_coordinates[1] - 360
          if info.bottom_right_geodetic_coordinates[1] gt 180 then info.bottom_right_geodetic_coordinates[1]= lf.bottom_right_geodetic_coordinates[1]- 360
          if info.top_left_geodetic_coordinates[1]     gt 180 then info.top_left_geodetic_coordinates[1]    = lf.top_left_geodetic_coordinates[1]    - 360
          if info.top_right_geodetic_coordinates[1]    gt 180 then info.top_right_geodetic_coordinates[1]   = lf.top_right_geodetic_coordinates[1]   - 360
        endif
  endif  
  
  
  
  temp_info=create_struct( info, $                            
          'calibration_constant'                   , calibration_constant                   , $ 
          'calibration_constant_estimation_utc'    , calibration_constant_estimation_utc    , $
          'beam_id'                                , beam_id                                , $
          'synthetic_aperture_duration'            , synthetic_aperture_duration            , $
          'polarization'                           , polarization                           , $
          'pol'                                    , polarization[0]                        , $
          'prf'                                    , prf                                    , $
          'frequency_band'                         , 'X'                                    , $
          'producttype'                            , info.product_type                      , $
          'name_dir'                               , info.file                              , $
          'range_pixel'                            , info.column_spacing                    , $
          'azimuth_pixel'                          , info.line_spacing                      , $
          'sar_sensor'                             , info.satellite_id                      , $
          'platform_code'                          , platform_code                          , $
          'datumshiftparameters'                   , datumshiftparameters                   , $
          'proc_fac'                               , info.processing_center                 , $
          'software_version'                       , software_version                       , $ 
          'asc_des'                                , asc_des                                , $
          'beam_mode'                              , info.multi_beam_id                     , $
          'sensor_velocity'                        , sensor_velocity                        , $
          'nom_az_res'                             , info.azimuth_geometric_resolution      , $
          'nom_rg_res'                             , info.ground_range_geometric_resolution , $
          's_c_altitude'                           , info.satellite_height                  , $
          'rcvr_gain'                              , rcvr_gain                              , $  
          'proc_gain'                              , proc_gain                              , $
          'range_flag'                             , range_flag                             , $
          'flip'                                   , flip                                   , $
          'sar_imaging_mode'                       , info.multi_beam_id                     , $
          'datatake_id'                            , info.programmed_image_id               , $
          'valuetype'                              , valuetype                              , $
          'range_type'                             , range_type                             , $
          'heading'                                , info.scene_orientation                 , $
          'heading_rel'                            , 90.0                                   , $
          'illumination_dir'                       , 0.0                                    , $
          'early_azimuth_edge'                     , early_azimuth_edge                     , $
          'near_range_edge'                        , near_range_edge                        , $  
          'range_swath'                            , range_swath                            , $
          'azimuth_swath'                          , azimuth_swath                          , $
          'range_looks'                            , range_looks                            , $
          'azimuth_looks'                          , azimuth_looks                          , $
          'ellipsoidscale'                         , ellipsoidscale                         , $ 
          'ellipsoidname'                          , info.ellipsoid_designator              , $ 
          'semimajoraxis'                          , info.ellipsoid_semimajor_axis          , $ 
          'semiminoraxis'                          , info.ellipsoid_semiminor_axis          , $ 
          'slant_range_center'                     , slant_range_center                     , $
          'slant_range_near'                       , slant_range_near                       , $
          'integration_time'                       , integration_time                       , $
          'lngswitchflag'                          , lngswitchflag                          )
  
         
    info=      !NULL                                                           
    info= temp_info
    temp_info= !NULL   
    end
endcase
       
       
       
       
       
       
       
          

end
