;+---------------------------
; NAME: ANS_READ_ENVISAT 
; PURPOSE:
;    The purpose of this program is to read ENIVSAT data and
;    create a SIO formatted binary image file and ASCII file
; CALLING SEQUENCE:   
;    ans_read_envisat, input_file, deffile
; INPUT:
;    input_file =    ENVISAT data file
;    deffile    =    Defaults file.
; OUTPUT:          
;    Binary files with image and land mask data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;        ENVI_ESA_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;        ENVI_ESA_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;        ERS2_D-PAF_2008_08_16_14_35_04_0272212504_118.43E_20.33N_VV_image.dat (image file)
;        ERS2_D-PAF_2008_08_16_14_35_04_0272212504_118.43E_20.33N_VV_ancil.txt (ancillary data file)
;    The file name is delimited by _ and the entries are Sensor, proessing facility, year, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header/
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     outputroot   = root name of outputfiles
;     jsstart      = Start processing time
; MODIFICATION HISTORY
;  2009 Nov 24 Frank Monaldo
;              Program to convert ENVISAT data to SIO files
;              based upon ans_read_rsat2.pro
;  2010 Jan 06 Frank Monaldo. Correct for use of ans_compute_fit
;  2010 Jan 15 Frank Monaldo. setenv gshhs_dir
;  2010 Jan 19 Frank Monaldo. Add /header to only print out the ancillary data
;                             Add /nomas to suppress output of land mask
;  2010 Jan 26 Frank Monaldo. Add pixel_size
;  2010 Jan 28 Frank Monaldo. Add option for creating ANSWRS version. This
;                             require the idlans directory name
;  2010 Feb 05 Frank Monaldo. Usd the help call to find the source file directory
;                             This is used to calculated the time of last modifications.
;                             This removes need for providing the name of source directory.
;                             Typing command w/no parameters also calls help. 
;  2010 May 27 Frank Monaldo. Add outputroot
;  2010 May 28 Frank Monaldo. Add outputoot array
;  2010 Dec 30 Frank Monaldo. Pass on quiet setting, Add log_dir
;  2011 Jan 20 Frank Monaldo. Typo is gshhs
;  2011 Feb 05 Frank Monlado. Add jsstart
;  2011 Feb 07 Frank Monaldo. Add file name to processing start printout in log
;  2011 Feb 07 Frank Monaldo. Add error catching
;  2011 Feb 13 Frank Monaldo. Changed unique name of directory to make more sense
;  2011 Feb 14 Frank Monaldo. Add ability to escape if lock file encountered.
;  2011 Mar 29 Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2011 Apr 20 Frank Monaldo. Correct spelling of gshss_dir that caused the
;                             program to default to the hard-wired values
;  2011 Oct 17 Frank Monaldo. Exist of lon_coef=0 indicating bad longitude fit. 
;  2011 Jul 05 Frank Monaldo. Remove hardwired directories and files 
;  2012 Aug 21 Frank Monaldo. Move ans_gen_version
;  2013 Feb 01 Frank Monaldo. Add ice mask and re-write to rely only on defaults file
;                             Add additional dependence on defaults file
;  2013 Apr 25 Frank Monaldo. Add ability to generate longitude, latitude, incident angle,
;                             and radar look direction.
;  2013 Aug 14 Frank Monaldo. Move icemask, longitude, latitude, incident angle, radar look
;                             direction to ans_sat2sio.
;                             Remove delete lockfile
;  2013 Aug 20 Frank Monaldo. Move land mask generation to ans_sat2sio
;  2013 Oct 30 Chris Jackson. Modified the naming convention to properly work with files named
;                             SAR...E1 (or E2) which are ERS data in Envisat format
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_envisat, input_file, deffile,      $
                      quiet       = quiet,      $
                      help        = help,       $
                      outputroot  = outputroot, $
                      jsstart     = jsstart,    $
                      error       = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_ENVISAT 
  print, ' PURPOSE:
  print, '    The purpose of this program is to read ENIVSAT data and
  print, '    create a SIO formatted binary image file and ASCII file
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_envisat, input_file, deffile
  print, ' INPUT:
  print, '    input_file =    ENVISAT data file
  print, '    deffile    =    Defaults file.
  print, ' OUTPUT:          
  print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '    convention for file names is of the form:
  print, '        ENVI_ESA_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '        ENVI_ESA_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '        ERS2_D-PAF_2008_08_16_14_35_04_0272212504_118.43E_20.33N_VV_image.dat (image file)
  print, '        ERS2_D-PAF_2008_08_16_14_35_04_0272212504_118.43E_20.33N_VV_ancil.txt (ancillary data file)
  print, '    The file name is delimited by _ and the entries are Sensor, proessing facility, year, month, day, hour, minute
  print, '    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '    files include the SIO 20-byte header/
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  print, '     outputroot   = root name of outputfiles
  print, '     jsstart      = Start processing time
  error= -1
  return
endif

scaling = 1.0
; Parcing input parameters
;
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 

time0 = dt_tm_tojs(systime())
;
; Parcing input parameters
;

if not file_test(deffile) then begin
  if printflag then begin
    print, 'Cannot file defaults file.' + deffile
    print, 'Exiting ans_read_envisat.pro.'
  endif
  error = -1
  return
endif

ans_read_defaults,deffile, df, quiet=quiet

if n_elements(df.gshhs_dir) lt 1 then begin
  print, 'gshhs_dir directory file not specified.;
  print, 'Exiting ans_read_envisat.pro '
  error= -1
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet= quiet

setenv, 'GSHHS_DIR=' + df.gshhs_dir
;
; Determine if directory exists
; 
ff= file_search( input_file, count=count)

if count eq 0 then begin
  print, 'Could not find Envisat data: ' + input_file
  print, 'Exiting ans_read_envisat.pro '
  error= -1
  return
endif

;
; Create a new working directory
;
working_directory= df.working_directory + '/envisat_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory
;
; Copy Envisat file to working directory.   Note the movement of Envisat data to 
; an already processed directory must occur at a different level. It is not done in this program
;
file_mkdir, working_directory  + '/'
if printflag then begin
  print, 'Coping ' + input_file    
  print, 'to     ' + working_directory + '/'
endif
file_copy, input_file, working_directory  + '/', /recursive, /overwrite

res= file_search(df.sio_output_dir, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
if printflag then begin
  print, 'Moving ' + input_file    
  print, 'to     ' + df.already_processed + '/'
endif 
file_move, input_file,  df.already_processed  + '/' , /overwrite

;
; Keep only the input_file name and strip the path
;
inputfile= getwrd(input_file, 0, /last, del='/')

time1 = dt_tm_tojs(systime())

if printflag then print, 'Time to Envisat file to working directory='+ string (time1-time0) +' s'

if printflag then print, 'Creating lf structure for Envisat data...'

env_wsm_create_lf, lf, /quiet

lf.nrcs_slope=  df.envisat_slope
lf.nrcs_bias=   df.envisat_bias
;
; Filling in more structure information
;
lf.ave_line=  df.ave_line
lf.ave_pixel= df.ave_pixel
lf.deg=       df.deg
;

error= 1

if printflag then print, 'Reading main product header from '  + working_directory + '/' + inputfile
env_wsm_main_product_header_read,     working_directory + '/' + inputfile, lf, quiet=quiet, error=error

if printflag then print, 'Reading main product header from '  + working_directory + '/' + inputfile

if error lt 0 then return

;==========================================================
; Loop through data sets
;==========================================================

if printflag then print, 'Number of Envisat data sets: ' + string(lf.num_data_sets, format='(I3)')

; Add to struture
temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
lf=      0
lf=      temp_lf


data_set=    0
data_setmax= 0
iout=        0

while data_set le data_setmax do begin
  ;
  ; Specific product header
  ;
  if printflag then print, 'Reading specific product header from ' + working_directory + '/' + inputfile + $
                           ' for data set ' + strcompress(string(data_set))  
  error=1 
  env_wsm_specific_product_header_read, working_directory + '/' + inputfile, lf, data_set=data_set, /quiet, error=error
  if error lt 0 then return
  ;
  ; Check for second data set
  ;
  w=where(lf.ds_name eq 'MDS2', count)
  if count eq 1 and strcompress((lf.mds2_tx_rx_polar),/remove_all) ne '' then data_setmax=1
  if printflag then begin
    print, 'Count on MDS2=       ' + string(count)
    print, 'lf.mds2_tx_rx_polar= ' + lf.mds2_tx_rx_polar
    print, 'data_setmax=         ' + string(data_setmax)
  endif
  ;
  ; Geolocation data
  ;
  if printflag then print, 'Reading geolocation data from '      + working_directory + '/' + inputfile + $
                            ' for data set ' + strcompress(string(data_set)) 
  error=1  
  env_wsm_geoloc_read, working_directory + '/' + inputfile , lf, /quiet, error=error
  if error lt 0 then return
  ;
  ; Main processing parameters
  ;
  if printflag then print, 'Reading main processing parameters from ' + working_directory + '/' + inputfile + $
                           ' for data set ' + strcompress(string(data_set))
  env_wsm_mpp_read, working_directory + '/' + inputfile, lf, /quiet
  ;
  ; Fill in some parameter in structure
  ;
  
  lf.pixel_spacing=   lf.range_spacing   ; Fill GAT leader values
  lf.line_spacing=    lf.azimuth_spacing
  lf.range_pixel=     lf.range_spacing   ; Fill ASF values
  lf.azimuth_pixel=   lf.azimuth_spacing
  lf.polarization=    'VV'
  strpol=             strupcase(lf.mds1_tx_rx_polar)  
  if data_set eq 1 then strpol=  strupcase(lf.mds2_tx_rx_polar)  
  case strpol of
  'H/H': lf.polarization='HH'
  'V/V': lf.polarization='VV'
  'H/V': lf.polarization='HV'
  'V/H': lf.polarization='VH'
  ELSE:  lf.polarization='VV' 
  endcase
  ;
  ; Reading in image data
  ;
  if printflag then print, 'Reading data from ' + working_directory + '/' + inputfile+ $
                           'for data set ' + strcompress(string(data_set))   
  error= 1                         
  env_wsm_mds_read, working_directory + '/' + inputfile, lf, image,    $
                    quiet=quiet, data_set=data_set,                    $
                    data_present_flag=data_present_flag, error=error 
  if error lt 0 then return                 
  if data_set eq 0 then begin              
    ;
    ; Compute whether to swith longitude basis. The basis starts from -180 to 180. 
    ;
    lf.lngswitchflag=  0
    w=where(lf.first_longs ne 0, count)
    if count gt 0 then begin
      max_longitude=max(lf.first_longs(w) )
      min_longitude=min(lf.first_longs(w)  )
    endif
    w=where(lf.last_longs  ne 0, count)
    if count gt 0 then begin
      max_longitude=max( [max_longitude,lf.last_longs(w)] )
      min_longitude=min( [min_longitude,lf.last_longs(w)]  )
    endif
    ;
    lf.lngswitchflag=0 ; Assume the longitude bais is +/-180
    ;
    ; Check and see if the we are using the 0-360 basis and
    ; change longitude basis if crossing a discontinuity
    ;
    if printflag then begin
      print, '-------Selecting longitude basis -----'
      print, 'Assuming a +/-180 basis. lf.lngswithflag=0'
      print, 'Maximum longitude= ' + string(max_longitude)
      print, 'Minimum longitdue= ' + string(min_longitude)
    endif
    if max_longitude gt 180 then begin 
      ; The basis in the original data is 0-360
      lf.lngswitchflag=  1
      if printflag then begin
        print,'Longitudes greater than 180.'
        print,'Change lf.lngswitchflag to 1'
      endif 
      if min_longitude lt 90 and max_longitude gt 270 then begin
        ; 
        ; The data cross the Prime Meridian. Switch to a
        ; +/-180 longitude basis
        ;
        if printflag then begin
          print, 'Data cross the Prime Meridian. Switching to a'
          print, '+/-180 longitude basis, lf.lngswitchflag=0'
        endif
        w=where(lf.first_longs gt 180, count)
        if count gt 0 then lf.first_longs(w)=lf.first_longs(w) - 360.0
        w=where(lf.last_longs  gt 180, count)
        if count gt 0 then lf.last_longs(w)= lf.last_longs(w)  - 360.0
        lf.lngswitchflag=0
      endif
    endif else begin
      ;
      ; The basis is assumed  to be -180 to 180 then check 
      ; if the data cross
      ; the International data line
      ;
      if max_longitude gt 160 and min_longitude lt -160 then begin
        ;
        ; The data cross the International Date Line and we must shift
        ; longitude basis
        ;
        if printflag then begin
          print, 'Data cross the Internationl Date Lien. Switching to a'
          print, '0-360 longitude basis, lf.lngswitchflag=1'
        endif
        lf.lngswitchflag=  1
        w=where(lf.first_longs lt 0, count)
        if count gt 0 then lf.first_longs(w)=360.0+lf.first_longs(w)
        w=where(lf.last_longs  lt 0, count)
        if count gt 0 then lf.last_longs(w)=360.0+lf.last_longs(w)
      endif
    endelse
    if printflag then print, '-------------------------------'
    ;
    ;
    env_lonlat2pix_fit, lf, quiet=quiet, error=latlonerror
    env_incid2pix_fit,  lf, quiet=quiet 
    env_r_look,         lf, quiet=quiet 
  endif
  ;
  ; Calibrate image
  ;
  time1=  dt_tm_tojs( systime() )
  env_wsm_dn2sigma, lf, image, quiet=quiet
  ;
  ; Add correction to NRCS if set from defaults file.
  ;
  if lf.nrcs_slope ne 1 or lf.nrcs_bias ne 0 then begin
    image=  image * lf.nrcs_slope + lf.nrcs_bias
  endif
  
  
  scaling= 1
  time2=  dt_tm_tojs( systime() )
  if printflag then print, 'Time to calibrate: ' + string(time2-time1) + ' s'
  ;
  ; Create file name root for output files.
  ;
  if data_set eq 0 then begin
    center_lat= string(abs(lf.center_lat), format='(F6.2)')
    if lf.center_lat ge 0 then begin
       center_lat= center_lat + 'N'
    endif else begin
      center_lat= center_lat + 'S'
    endelse
    center_lon= string(abs(lf.center_lon), format='(F6.2)')
    if lf.center_lon ge 0 then begin
      center_lon= center_lon + 'E'
    endif else begin
      center_lon= center_lon + 'W'
    endelse
    lf.month = strupcase(lf.month)
    case strupcase(lf.month) of
    'JAN': month=1
    'FEB': month=2
    'MAR': month=3
    'APR': month=4
    'MAY': month=5
    'JUN': month=6
    'JUL': month=7
    'AUG': month=8
    'SEP': month=9
    'OCT': month=10
    'NOV': month=11
    'DEC': month=12
    ELSE:  month=1
    endcase
    timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
                     lf.month                               + ' ' + $
                     string( lf.day,       format='(I2.2)') + ' ' + $
                     string( lf.hour,      format='(I2.2)') + ':' + $
                     string( lf.minute,    format='(I2.2)') + ':' + $
                     string( lf.second,    format='(I2.2)')
    ;js=      string( dt_tm_tojs(timestr), format='(I10.10)' )
    js=      string( dt_tm_tojs(timestr))
    jss      =  sign(js)
    js       =  string( abs(js), format='(I11)')
    if jss lt 0 then js = '-' + js
    js       =  strcompress(js, /remove_all)

    output_prefix= 'ENVI'
    idstr = getwrd(inputfile, 0, /last, del='.')
    if (idstr eq 'E1') then output_prefix= 'ERS1'
    if (idstr eq 'E2') then output_prefix= 'ERS2'
    output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
    output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
    output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
    output_prefix= output_prefix + '_' + js
    output_prefix= output_prefix + '_' + string( center_lon)
    output_prefix= output_prefix + '_' + string( center_lat)
    output_prefix= strcompress( output_prefix, /remove_all )   
  endif
  output_prefix2= output_prefix + '_' + strupcase(lf.polarization) + '_C'
  ;
  ; Check and see if a lock file exits
  ;
  lockfile=   df.sio_output_dir + '/'+ output_prefix2 + '*.lock'
  ;
  res= file_search( lockfile, count=count)
  if not file_test(lockfile) gt 0 then begin
     print, 'Lockfile:  ' + lockfile + ' exists'
     error=-1
     return
  endif
  ;
  ; Create lock file
  ;
  lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '.lock'
  close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1
  ;
  ; Write out to SIO data file format. This is general for all.
  ; SAR data
  ;
  scaling=1.0
  res= file_search(df.log_dir, count=count)
  if count eq 0 then file_mkdir, log_dir
  logfile= df.log_dir + '/' + output_prefix2 + '.log'
  ;
  outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + lf.product + ' Processing starts'
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  if total(lf.lon_coef) eq 0 or latlonerror lt 0 then begin
    jsc=   dt_tm_tojs(systime(/utc))
    outstr=string(jsc, format='(I10)') + ' '+ dt_tm_fromjs(jsc) + ' ' + lf.product + ' !!!! WARNING: Data provide bad geographic fit.'
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
    print, outstr
    error=-1
    return
  endif
  ;
  ; Write out data
  ;
  if df.twobyte eq 'Y' then begin
    ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, /twobyte, quiet=quiet
  endif else begin
    ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, quiet=quiet
  endelse
  time4= dt_tm_tojs(systime(/utc))
  outstr=    string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  ;
  if iout eq 0 then begin
    outputroot= df.sio_output_dir + '/'+ output_prefix2
    iout=1
  endif else begin
    outputroot= [outputroot, df.sio_output_dir + '/'+ output_prefix2 ]
  endelse
  
  time4 = dt_tm_tojs(systime())
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for Envisat data
  ;
  env_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
  time4= dt_tm_tojs(systime(/utc))
  outstr=    string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  data_set=  data_set + 1 ; Increment data set value
endwhile

;
; Remove
;

if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive, quiet=quiet


end
