;+---------------------------
; NAME: ANS_READ_ENVISAT_ANCIL
; PURPOSE:
;    The purpose of this program is to read ENIVSAT data a, ASCII file
;    of ancillary data
; CALLING SEQUENCE:   
;    ans_read_envisat_ancil, input_file, deffile
; INPUT:
;    input_file =    ENVISAT data file
;    deffile    =    Defaults file.
; OUTPUT:          
;    ASCII file with ancillary data
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
; MODIFICATION HISTORY
;  2013 Feb 06 Frank Monaldo. Modification of ancil_read_envisat_ancil
;  2013 Oct 30 Chris Jackson. Modified the naming convention to properly work with files named
;                             SAR...E1 (or E2) which are ERS data in Envisat format
; Copyright (C) 1999-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_envisat_ancil, input_file, deffile,      $
                            quiet       = quiet,      $
                            help        = help,       $
                            error       = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_ENVISAT_ANCIL
  print, ' PURPOSE:
  print, '    The purpose of this program is to read ENIVSAT data a, ASCII file
  print, '    of ancillary data
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_envisat_ancil, input_file, deffile
  print, ' INPUT:
  print, '    input_file =    ENVISAT data file
  print, '    deffile    =    Defaults file.
  print, ' OUTPUT:          
  print, '    ASCII file with ancillary data
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  error= -1
  return
endif

scaling = 1.0
; Parcing input parameters
;
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

jsstart= dt_tm_tojs(systime(/utc) )

ans_read_defaults, deffile, df, error=error, quiet=quiet
if error lt 0 then begin
  if printflag then begin
    print, 'Cannot find defaults file. ' + deffile
    error=-1
    return
  endif
endif


ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet


if printflag then print, 'Creating lf structure for Envisat data...'

env_wsm_create_lf, lf, /quiet

lf.nrcs_slope=  df.envisat_slope
lf.nrcs_bias=   df.envisat_bias
;
; Filling in more structure information
;
lf.ave_line=  df.ave_line
lf.ave_pixel= df.ave_pixel
lf.deg=       df.deg
;

error= 1

if printflag then print, 'Reading main product header from '  + input_file
env_wsm_main_product_header_read,  input_file, lf, quiet=quiet, error=error


if printflag then print, 'Reading main product header from '  + input_file

if error lt 0 then return

;==========================================================
; Loop through data sets
;==========================================================

if printflag then print, 'Number of Envisat data sets: ' + string(lf.num_data_sets, format='(I3)')

; Add to struture
temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
lf=      0
lf=      temp_lf


data_set=    0
data_setmax= 0
iout=        0

while data_set le data_setmax do begin
  ;
  ; Specific product header
  ;
  if printflag then print, 'Reading specific product header from ' + input_file + ', Data set ' + strcompress(string(data_set))  
  error=1 
  env_wsm_specific_product_header_read, input_file, lf, data_set=data_set, /quiet, error=error
  if error lt 0 then return
  ;
  ; Check for second data set
  ;
  w=where(lf.ds_name eq 'MDS2', count)
  if count eq 1 and strcompress((lf.mds2_tx_rx_polar),/remove_all) ne '' then data_setmax=1
  if printflag then begin
    print, 'Count on MDS2=       ' + string(count)
    print, 'lf.mds2_tx_rx_polar= ' + lf.mds2_tx_rx_polar
    print, 'data_setmax=         ' + string(data_setmax)
  endif
  ;
  ; Geolocation data
  ;
  if printflag then print, 'Reading geolocation data from ' + input_file + ', Data set ' + strcompress(string(data_set))
  error=1  
  env_wsm_geoloc_read, input_file , lf, /quiet, error=error
  if error lt 0 then return
  ;
  ; Main processing parameters
  ;
  if printflag then print, 'Reading main processing parameters from ' + input_file + ', Data set ' + strcompress(string(data_set)) 
  env_wsm_mpp_read,  input_tfile, lf, /quiet
  ;
  ; Fill in some parameter in structure
  ;
  
  lf.pixel_spacing=   lf.range_spacing   ; Fill GAT leader values
  lf.line_spacing=    lf.azimuth_spacing
  lf.range_pixel=     lf.range_spacing   ; Fill ASF values
  lf.azimuth_pixel=   lf.azimuth_spacing
  lf.polarization=    'VV'
  strpol=             strupcase(lf.mds1_tx_rx_polar)  
  if data_set eq 1 then strpol=  strupcase(lf.mds2_tx_rx_polar)  
  case strpol of
  'H/H': lf.polarization='HH'
  'V/V': lf.polarization='VV'
  'H/V': lf.polarization='HV'
  'V/H': lf.polarization='VH'
  ELSE:  lf.polarization='VV' 
  endcase
  ;
  ; Reading in in image data
  ;
  if printflag then print, 'Reading data from ' + input_file+ 'for data set ' + strcompress(string(data_set))
  error= 1
  env_wsm_mds_read, input_file, lf, image,  quiet=quiet, data_set=data_set,  $
                    data_present_flag=data_present_flag, error=error
  if error lt 0 then return

  if data_set eq 0 then begin              
    ;
    ; Compute whether to swith longitude basis. The basis starts from -180 to 180. 
    ;
    lf.lngswitchflag=  0
    w=where(lf.first_longs ne 0, count)
    if count gt 0 then begin
      max_longitude=max(lf.first_longs(w) )
      min_longitude=min(lf.first_longs(w)  )
    endif
    w=where(lf.last_longs  ne 0, count)
    if count gt 0 then begin
      max_longitude=max( [max_longitude,lf.last_longs(w)] )
      min_longitude=min( [min_longitude,lf.last_longs(w)]  )
    endif
    ;
    lf.lngswitchflag=0 ; Assume the longitude bais is +/-180
    ;
    ; Check and see if the we are using the 0-360 basis and
    ; change longitude basis if crossing a discontinuity
    ;
    if printflag then begin
      print, '-------Selecting longitude basis -----'
      print, 'Assuming a +/-180 basis. lf.lngswithflag=0'
      print, 'Maximum longitude= ' + string(max_longitude)
      print, 'Minimum longitdue= ' + string(min_longitude)
    endif
    if max_longitude gt 180 then begin 
      ; The basis in the original data is 0-360
      lf.lngswitchflag=  1
      if printflag then begin
        print,'Longitudes greater than 180.'
        print,'Change lf.lngswitchflag to 1'
      endif 
      if min_longitude lt 90 and max_longitude gt 270 then begin
        ; 
        ; The data cross the Prime Meridian. Switch to a
        ; +/-180 longitude basis
        ;
        if printflag then begin
          print, 'Data cross the Prime Meridian. Switching to a'
          print, '+/-180 longitude basis, lf.lngswitchflag=0'
        endif
        w=where(lf.first_longs gt 180, count)
        if count gt 0 then lf.first_longs(w)=lf.first_longs(w) - 360.0
        w=where(lf.last_longs  gt 180, count)
        if count gt 0 then lf.last_longs(w)= lf.last_longs(w)  - 360.0
        lf.lngswitchflag=0
      endif
    endif else begin
      ;
      ; The basis is assumed  to be -180 to 180 then check 
      ; if the data cross
      ; the International data line
      ;
      if max_longitude gt 160 and min_longitude lt -160 then begin
        ;
        ; The data cross the International Date Line and we must shift
        ; longitude basis
        ;
        if printflag then begin
          print, 'Data cross the Internationl Date Lien. Switching to a'
          print, '0-360 longitude basis, lf.lngswitchflag=1'
        endif
        lf.lngswitchflag=  1
        w=where(lf.first_longs lt 0, count)
        if count gt 0 then lf.first_longs(w)=360.0+lf.first_longs(w)
        w=where(lf.last_longs  lt 0, count)
        if count gt 0 then lf.last_longs(w)=360.0+lf.last_longs(w)
      endif
    endelse
    if printflag then print, '-------------------------------'
    ;
    ;
    env_lonlat2pix_fit, lf, quiet=quiet, error=latlonerror
    env_incid2pix_fit,  lf, quiet=quiet 
    env_r_look,         lf, quiet=quiet 
  endif
  ;
  ; Create file name root for output files.
  ;
  if data_set eq 0 then begin
    center_lat= string(abs(lf.center_lat), format='(F6.2)')
    if lf.center_lat ge 0 then begin
       center_lat= center_lat + 'N'
    endif else begin
      center_lat= center_lat + 'S'
    endelse
    center_lon= string(abs(lf.center_lon), format='(F6.2)')
    if lf.center_lon ge 0 then begin
      center_lon= center_lon + 'E'
    endif else begin
      center_lon= center_lon + 'W'
    endelse
    lf.month = strupcase(lf.month)
    case strupcase(lf.month) of
    'JAN': month=1
    'FEB': month=2
    'MAR': month=3
    'APR': month=4
    'MAY': month=5
    'JUN': month=6
    'JUL': month=7
    'AUG': month=8
    'SEP': month=9
    'OCT': month=10
    'NOV': month=11
    'DEC': month=12
    ELSE:  month=1
    endcase
    timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
                     lf.month                               + ' ' + $
                     string( lf.day,       format='(I2.2)') + ' ' + $
                     string( lf.hour,      format='(I2.2)') + ':' + $
                     string( lf.minute,    format='(I2.2)') + ':' + $
                     string( lf.second,    format='(I2.2)')
    js=      string( dt_tm_tojs(timestr), format='(I10.10)' )

    ; Get only the input_file name and strip the path
    inputfile= getwrd(input_file, 0, /last, del='/')
    output_prefix= 'ENVI'
    idstr = getwrd(inputfile, 0, /last, del='.')
    if (idstr eq 'E1') then output_prefix= 'ERS1'
    if (idstr eq 'E2') then output_prefix= 'ERS2'
    output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
    output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
    output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
    output_prefix= output_prefix + '_' + js
    output_prefix= output_prefix + '_' + string( center_lon)
    output_prefix= output_prefix + '_' + string( center_lat)
    output_prefix= strcompress( output_prefix, /remove_all )   
  endif
  output_prefix2= output_prefix + '_' + strupcase(lf.polarization) + '_C'
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for Envisat data
  ;
  env_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
  data_set= data_set+1
endwhile

end
