;+---------------------------
; NAME: ANS_READ_GAT_RSAT
; PURPOSE:
;   The purpose of this program is to read Radarsat-1 data in Gatiney format and
;   create a SIO formatted binary image filea and ASCII file. The Gatineau output is consistent with the CSTARS
;   processed data. Both processing facilities produce leader and trailer
;   files, but the leader file for one is the trailer file for the other. The "leaderfile"
;   used here is the larger of the two files.
; CALLING SEQUENCE:
;   ans_read_gat_rsat, datafile, leaderfile, deffile
; INPUT:
;   datafile  =   CEOS file with SAR data
;   leaderfie =   CEOS file if ancillary data information
;   deffile   =   Defaults files
; OUTPUT:          
;   Binary files with image and land mask data and an ASCII file with ancillary data. The 
;   convention for file names is of the form:
;       RSAT1_GAT_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;       RSAT1_GAT_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;   The file name is delimited by _ and the entries are Sensor, year, processing facility, month, day, hour, minute
;   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;   files include the SIO 20-byte header.
; KEYWORDS:
;   /quiet=        Do not print intermediate results.
;   /help=         Print out help
;   /diagnostic    Print out geometric location fit comparisons
;   outputroot:    Output of the SIO files
;   jsstart:       Start processing time in Julian seconds
; MODIFICATION HISTORY
;  2009 Dec 09 Frank Monaldo. Rewrote for ANSWRS 2.0
;  2009 Dec 16 Frank Monaldo. Corrected misspelling of gshhs_dir
;  2010 Jan 06 Frank Monaldo. Change to use ans_poly_cal
;  2010 Jan 15 Frank Monaldo. setenv gshhs_dir
;  2010 Jan 19 Frank Monaldo. Add /header option to only save ancillary data file
;                             Add /nomask to suppress land mask data output
;  2010 Jan 23 Frank Monaldo. Moved time4 to the proper place
;  2010 Jan 26 Frank Monaldo. Add pixel_size
;  2010 Jan 28 Frank Monaldo. Add option for creating ANSWRS version. This
;                             require the idlans directory name
;  2010 Feb 05 Frank Monaldo. Use the help call to find the source file directory
;                             This is used to calculated the time of last modifications.
;                             This removes need for providing the nanme of source directory.
;                             Typing command w/no parameters also calls help. 
;  2010 May 29 Frank Monaldo. Add move2dir and outputroot options
;  2011 Jan 01 Frank Monaldo. Add log_dir, and pass on quiet flag
;  2011 Feb 04 Frank Monaldo. Add jsstart
;  2011 Feb 05 Frank Monaldo. Add additional error handling
;  2011 Feb 07 Frank Monaldo. Add file or directory name to line in
;                             log file with the orignal file/directory name.
;  2011 Feb 13 Frank Monaldo. Change working directory name to something that makes moe sense.
;  2011 Feb 14 Frank Monaldo. Add ability to escape if .lock file encountered.
;  2011 Mar 29 Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2011 Dec 01 Frank Monaldo. Add corrected spelling error (gshhs)
;  2012 Jul 05 Frank Monaldo. Removed hardwired directories
;  2012 Aug 21 Frank Monaldo. Move ans_gen_version
;  2013 Jan 19 Frank Monaldo. Replace undescores with hypends in processing facility name.
;  2013 Jan 31 Frank Monaldo. Add ability to add ice mask and ability to 
;                             defaults file direction.  Re-write call to this
;                             program to rely on the defaults file
;  2013 Aug 08 Frank Monaldo. Move computation icemask, longitude, latitude, incidence, and
;                             radar look to ans_sat2sio.  Remove this function from
;                             ans_read_gat_rsat. Remove delection of lock file from this program
;                             this function is now in ans_sat2sio
;  2013 Aug 20 Frank Monaldo. Move generation of land mask to ans_sat2sio. Remove nomask, header options
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;

pro ans_read_gat_rsat, datafile, leaderfile, deffile , $
                       quiet         = quiet         , $
                       help          = help          , $
                       outputroot    = outputroot    , $
                       jsstart       = jsstart       , $
                       diagnostic    = diagnostic    , $
                       error         = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_READ_GAT_RSAT
  print, ' PURPOSE:
  print, '   The purpose of this program is to read Radarsat-1 data in Gatiney format and
  print, '   create a SIO formatted binary image filea and ASCII file. The Gatineau output is consistent with the CSTARS
  print, '   processed data. Both processing facilities produce leader and trailer
  print, '   files, but the leader file for one is the trailer file for the other. The "leaderfile"
  print, '   used here is the larger of the two files.
  print, ' CALLING SEQUENCE:
  print, '   ans_read_gat_rsat, datafile, leaderfile, deffile
  print, ' INPUT:
  print, '   datafile  =   CEOS file with SAR data
  print, '   leaderfie =   CEOS file if ancillary data information
  print, '   deffile   =   Defaults files
  print, ' OUTPUT:          
  print, '   Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '   convention for file names is of the form:
  print, '       RSAT1_GAT_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '       RSAT1_GAT_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '   The file name is delimited by _ and the entries are Sensor, year, processing facility, month, day, hour, minute
  print, '   second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '   files include the SIO 20-byte header.
  print, ' KEYWORDS:
  print, '   /quiet=        Do not print intermediate results.
  print, '   /help=         Print out help
  print, '   /diagnostic    Print out geometric location fit comparisons
  print, '   outputroot:    Output of the SIO files
  print, '   jsstart:       Start processing time in Julian seconds
  error= -1
  return
endif

!quiet=   1                                                          
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 

time0 = dt_tm_tojs(systime())
;
; Parcing input parameters
;
if file_test(deffile) eq 0 then begin
   error  = -1
   print, 'Cannot find defaults file:' + deffile
   print, 'Exiting ans_read_gat_rsat.pro'
   return
endif

ans_read_defaults, deffile, df, quiet=quiet

if strupcase(df.silent) eq 'Y' then quiet=1

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
setenv, 'GSHHS_DIR=' + df.gshhs_dir

; Determine if input files exist
; 


if file_test(datafile) eq 0 then begin
  print, 'Could not find GAT Radarsat-1 data file: ' + datafile
  print, 'Exiting ans_read_gat_rsat.pro '
  error= -1
  return
endif

if file_test(leaderfile) eq 0 then begin
  print, 'Could not find GAT Radarsat-1 data file: ' + leaderfile
  print, 'Exiting ans_read_gat_rsat.pro '
  error= -1
  return
endif


time1=dt_tm_tojs(systime())

;
; Create a new working directory
;
working_directory= df.working_directory + '/gat_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory

;
; Copy Radarsat-1 files to working directory.   Note the movement of Radarsat-1 directory to 
; and already processed directory must occur at a different level. It is not done in this program
;
if printflag then begin
  print, 'Moving  ' + leaderfile
  print, 'and     ' + datafile
  print, 'to      ' + working_directory
endif
file_copy, leaderfile, working_directory  + '/'
file_copy, datafile,   working_directory  + '/'
;
; Moving files to appropriate directories
;

res= file_search(df.already_processed, /test_directory, count=count)

if count eq 0 then file_mkdir, df.already_processed
length = strlen(leaderfile)
source = strmid(leaderfile,0,length-1) +'*'
if printflag then begin
  print, 'Moving ' + source
  print, 'to     ' + df.already_processed
endif 
file_move, source,  df.already_processed  + '/' , /overwrite


sleaderfile=    getwrd(leaderfile, 0, /last, del='/')
sdatafile=      getwrd(datafile  , 0, /last, del='/')

;
; Read leader file
;

gat_read_leader, working_directory  + '/' + sleaderfile, good_leader, lf, quiet=quiet

lf.nrcs_slope=  df.rsat1_gat_slope
lf.nrcs_bias=   df.rsat1_gat_bias

if good_leader eq -1 or $
   good_leader eq -2 or $
   good_leader eq -4 then begin
   print, 'Return from gat_read_leader=' + string(good_leader)
   error= -1
   return
endif

lf.name_dir   = sdatafile
lf.ave_pixel  = df.ave_pixel
lf.ave_line   = df.ave_line
lf.deg        = df.deg

; Add to struture

temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
lf=      0
lf=      temp_lf

;
; Read data file
;
gat_read_data_file,   working_directory  + '/' + sdatafile, image, lf, quiet=quiet

;
; Compute i/j to incidence angle fit
gat_incid2pix_fit,   lf, diagnostic=diagnostic, quiet=quiet
;
; Convert digital number to sigma-naught. This program
; also averages the image in range and azimuth
gat_dn2sigma, image, lf, quiet=quiet  

;
; Add correction to NRCS if set from defaults file.
;
if lf.nrcs_slope ne 1 or lf.nrcs_bias ne 0 then begin
  image=  image * lf.nrcs_slope + lf.nrcs_bias
endif

lf.nx0= lf.n_pixel
lf.ny0= lf.n_line
lf.nx1= lf.n_pixel / lf.ave_pixel
lf.ny1= lf.n_line  / lf.ave_line

; Compute foward and invers reltionships between i and j <=> latitude
; and longitude
gat_lonlat2pix_fit,  lf,  quiet= quiet
gat_r_look       ,   lf,  quiet= quiet


lf.center_lat=    ans_poly_cal( lf.nx1 /2.0, lf.ny1/2.0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
lf.center_lon=    ans_poly_cal( lf.nx1 /2.0, lf.ny1/2.0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
;



time2=dt_tm_tojs(systime())

if printflag then begin
  print, 'Time to move/read Radarsat-1 files in Gatineau format: ' + strcompress(string(time2-time1), /remove_all) + ' s'
endif
;
; Create file name root for output files.
;

lf.month = strupcase(lf.month)
case strupcase(lf.month) of
'JAN': month=1
'FEB': month=2
'MAR': month=3
'APR': month=4
'MAY': month=5
'JUN': month=6
'JUL': month=7
'AUG': month=8
'SEP': month=9
'OCT': month=10
'NOV': month=11
'DEC': month=12
ELSE:  month=1
endcase
;
;
; compute center geographic position
;


center_lat= string(abs(lf.center_lat), format='(F6.2)')
if lf.center_lat ge 0 then begin
  center_lat= center_lat + 'N'
endif else begin
  center_lat= center_lat + 'S'
endelse
center_lon= string(abs(lf.center_lon), format='(F6.2)')
if lf.center_lon ge 0 then begin
  center_lon= center_lon + 'E'
endif else begin
  center_lon= center_lon + 'W'
endelse

timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
         lf.month                               + ' ' + $
         string( lf.day,       format='(I2.2)') + ' ' + $
         string( lf.hour,      format='(I2.2)') + ':' + $
         string( lf.minute,    format='(I2.2)') + ':' + $
         string( lf.second,    format='(I2.2)')

jsf=           dt_tm_tojs(timestr)
if jsf lt 0 then begin
  js=            '-'+ string(abs(jsf), format='(I10.10)' )
endif else begin
  js=            string(jsf, format='(I10.10)' ) 
endelse
output_prefix= 'RSAT1'
output_prefix= output_prefix + '_' + stress(strupcase(lf.proc_fac),'R',0,'_','-',ned)
output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + string( center_lon)
output_prefix= output_prefix + '_' + string( center_lat)
output_prefix= strcompress( output_prefix, /remove_all )

ave_pixel  = df.ave_pixel
ave_line   = df.ave_line 


output_prefix2= output_prefix + '_' + strupcase(lf.polarization) + '_C'

;
; Check and see if a lock file exits
;
lockfile  = df.sio_output_dir + '/'+ output_prefix2 + '.lock'
lockfile  = stress( lockfile + '/', 'R', 0, '//', '/', ned)
;
res= file_search( lockfile, count=count)
if count gt 0 then begin
   print, 'Lockfile:  ' + lockfile + ' exists'
   error=-1
   return
endif
;
; Create lockfile
; 
close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1


; Write out to SIO data file format. This is general for all.
; SAR data
;
scaling=1.0
logfile=  df.log_dir + '/'+ output_prefix2 + '.log'
close, 1 &  openw, 1, logfile  & close, 1
   
outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + lf.name_dir + ' Processing starts'
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr


time3= dt_tm_tojs(systime())
if strupcase(df.twobyte) eq'Y' then begin
  ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, /twobyte, quiet=quiet
endif else begin       
  ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, quiet=quiet
endelse
time4= dt_tm_tojs(systime())
if printflag then begin
  print, 'Time to write NRCS SIO file ' + strcompress( string(time4- time3), /remove_all) + ' s'
endif
time4= dt_tm_tojs(systime(/utc))
outstr  =  string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
outstr  = stress( outstr, 'R', 0, '//', '/', ned)
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr


time4= dt_tm_tojs(systime())
; 
; Outputroot set
;
outputroot= df.sio_output_dir + '/'+ output_prefix2
; 
; Write out SIO ancillary file format. This program is explicitly written for PALSAR data
;
gat_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet= quiet
time5= dt_tm_tojs(systime())
if printflag then begin
  print, 'Time to write ancillary data SIO file ' + strcompress( string(time5- time4), /remove_all) + ' s'
endif
time4= dt_tm_tojs(systime(/utc))
outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
outstr  = stress( outstr, 'R', 0, '//', '/', ned)
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr

;====================================
; Ice Mask
; ===================================
;
; Write out SIO ice mask file format.  
;

; Remove
;

file_delete,  working_directory  + '/', /recursive, quiet=quiet


end
