;----------------------------------------------------------
;+
; NAME: ANS_READ_L1_CDF'
; PURPOSE:'
;   Read Level-1 SAR windretrieval data.
; CALLING SEQUENCE:'
;   ans_read_l1_cdf, file, info, sigma, mask
;   file:    Level 1 SAR file in CDF format
; OUTPUT:
;   info:    Structure with ancillary information
;   sigma:   Normalized radar cross section
;   mask:    Land mask data
; KEYWORDS:  
;   /quiet=  Do not print intermediate results'
;   /help=   List help.
; MOFICATION HISTORY
;   2010 Mar 09 Frank Monaldo. Add global attributes to info structure.
;   2010 Jun 14 Frank Monaldo. Corrected spelling
;   2011 Jun 01 Frank Monaldo. Add nrcs_slope, nrcs_bias
;   2011 Jun 13 Frank Monaldo. Hardwire ids if not read properly
; Copyright (C) 2010-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-


pro  ans_read_l1_cdf, file, info, sigma, mask, quiet= quiet, help=help

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_READ_L1_CDF'
  print, ' PURPOSE:'
  print, '   Read Level-1 SAR windretrieval data.
  print, ' CALLING SEQUENCE:'
  print, '   ans_read_l1_cdf, file, info, sigma, mask
  print, '   file:    Level 1 SAR file in CDF format
  print, ' OUTPUT:
  print, '   info:    Structure with ancillary information
  print, '   sigma:   Normalized radar cross section
  print, '   mask:    Land mask data
  print, ' KEYWORDS:  
  print, '   /quiet=  Do not print intermediate results'
  print, '   /help=   List help.
  return                
endif
;
; Parcing input parameters  
;
printflag= 1
if keyword_set(quiet) then printflag=0

res=file_search(file, count=count)
if count eq 0 then begin
  print, 'Cannot find file '+ file + ' .'
  print, 'Exiting ans_read_l1_cdf....'
  return
endif

id = NCDF_OPEN(file, /nowrite) 
;
; Global Attributes
;  
NCDF_ATTGET, id, /GLOBAL, 'ANSWRS_Version',                        answrs_version
NCDF_ATTGET, id, /GLOBAL, 'Platform_Name',                         platform_name
NCDF_ATTGET, id, /GLOBAL, 'Acquisition_Time',                      acq_date_time
NCDF_ATTGET, id, /GLOBAL, 'Acquisition_Time_Julian_Seconds',       acq_date_time_js
NCDF_ATTGET, id, /GLOBAL, 'Acquisition_Time_Start_Julian_Seconds', jsbegin
NCDF_ATTGET, id, /GLOBAL, 'Acquisition_Time_End_Julian_Seconds',   jsend
NCDF_ATTGET, id, /GLOBAL, 'Processing_Time',                       sio_create_date_time
NCDF_ATTGET, id, /GLOBAL, 'SAR_File_or_Directory_Name',            root_file
NCDF_ATTGET, id, /GLOBAL, 'Radar_Frequency_Hz',                    sensor_frequency_hz
NCDF_ATTGET, id, /GLOBAL, 'Radar_Wavelength_m',                    sensor_wavelength_m
NCDF_ATTGET, id, /GLOBAL, 'Radar_Band',                            frequency_band
NCDF_ATTGET, id, /GLOBAL, 'Polarization',                          polarization
NCDF_ATTGET, id, /GLOBAL, 'product_type',                          product_type               
NCDF_ATTGET, id, /GLOBAL, 'processing_agency',                     processing_agency          
NCDF_ATTGET, id, /GLOBAL, 'processing_facility',                   processing_facility        
NCDF_ATTGET, id, /GLOBAL, 'sar_processor_version',                 sar_processor_version      
NCDF_ATTGET, id, /GLOBAL, 'scene_identifier',                      scene_identifier        
NCDF_ATTGET, id, /GLOBAL, 'scene_reference_number',                scene_reference_number     
NCDF_ATTGET, id, /GLOBAL, 'orbit_number',                          orbit_number       
NCDF_ATTGET, id, /GLOBAL, 'sar_platform_code',                     sar_platform_code          
NCDF_ATTGET, id, /GLOBAL, 'sensor_imaging_mode',                   sensor_imaging_mode        
NCDF_ATTGET, id, /GLOBAL, 'calibration_flag',                      calibration_flag          
NCDF_ATTGET, id, /GLOBAL, 'image_pixel_value_units',               image_pixel_value_units                                           
NCDF_ATTGET, id, /GLOBAL, 'orbit_pass_flag',                       orbit_pass_flag                   
NCDF_ATTGET, id, /GLOBAL, 'sar_beam_code',                         sar_beam_code             
NCDF_ATTGET, id, /GLOBAL, 'map_descriptor',                        map_descriptor
NCDF_ATTGET, id, /GLOBAL, 'range_type',                            range_type
NCDF_ATTGET, id, /GLOBAL, 'ref_ellipsoid',                         ref_ellipsoid
NCDF_ATTGET, id, /GLOBAL, 'datum_shifts_m',                        datum_shifts_m
NCDF_ATTGET, id, /GLOBAL, 'datum_scale',                           datum_scale
NCDF_ATTGET, id, /GLOBAL, 'source_lonlat_fit',                     source_lonlat_fit 
NCDF_ATTGET, id, /GLOBAL, 'source_incid_fit',                      source_incid_fit 
NCDF_ATTGET, id, /GLOBAL, 'source_rlook_fit',                      source_rlook_fit 
NCDF_ATTGET, id, /GLOBAL, 'nrcs_slope',                            nrcs_slope
NCDF_ATTGET, id, /GLOBAL, 'nrcs_bias',                             nrcs_bias
;

info= { answrs_version:              string(answrs_version),         $
        platform_name:               string(platform_name),          $
        acq_date_time:               string(acq_date_time),          $
        acq_date_time_js:            string(acq_date_time_js),       $
        jsbegin:                     string(jsbegin),                $
        jsend:                       string(jsend),                  $       
        sio_create_date_time:        string(sio_create_date_time),   $
        root_file:                   string(root_file),              $ 
        sensor_frequency_hz:         string(sensor_frequency_hz),    $
        sensor_wavelength_m:         string(sensor_wavelength_m),    $
        frequency_band:              string(frequency_band),         $
        polarization:                string(polarization),           $
        product_type:                string(product_type),           $              
        processing_agency:           string(processing_agency),      $         
        processing_facility:         string(processing_facility),    $       
        sar_processor_version:       string(sar_processor_version),  $     
        scene_identifier:            string(scene_identifier),       $        
        scene_reference_number:      string(scene_reference_number), $    
        orbit_number:                string(orbit_number),           $       
        sar_platform_code:           string(sar_platform_code),      $         
        sensor_imaging_mode:         string(sensor_imaging_mode),    $       
        calibration_flag:            string(calibration_flag),       $          
        image_pixel_value_units:     string(image_pixel_value_units),$     
        orbit_pass_flag:             string(orbit_pass_flag),        $             
        sar_beam_code:               string(sar_beam_code),          $             
        map_descriptor:              string(map_descriptor),         $
        range_type:                  string(range_type),             $
        ref_ellipsoid:               string(ref_ellipsoid),          $
        datum_shifts_m:              string(datum_shifts_m),         $
        datum_scale:                 string(datum_scale),            $
        source_lonlat_fit:           string(source_lonlat_fit),      $ 
        source_incid_fit:            string(source_incid_fit),       $ 
        source_rlook_fit:            string(source_rlook_fit),       $
        nrcs_slope:                  string(nrcs_slope),             $
        nrcs_bias:                   string(nrcs_bias)               } 



if printflag then begin
  print, 'Global attributes.                '  
  print , '----------------------------------------
  print, 'answrs_version:             ' + info.answrs_version      
  print, 'platform_name:              ' + info.platform_name       
  print, 'acq_date_time:              ' + info.acq_date_time       
  print, 'acq_date_time_js:           ' + info.acq_date_time_js    
  print, 'sio_create_date_time:       ' + info.sio_create_date_time
  print, 'root_file:                  ' + info.root_file           
  print, 'sensor_frequency_hz:        ' + info.sensor_frequency_hz 
  print, 'sensor_wavelength_m:        ' + info.sensor_wavelength_m 
  print, 'frequency_band:             ' + info.frequency_band      
  print, 'polarization:               ' + info.polarization        
  print, 'product_type:               ' + info.product_type        
  print, 'processing_agency:          ' + info.processing_agency   
  print, 'processing_facility:        ' + info.processing_facility 
  print, 'sar_processor_version:      ' + info.sar_processor_version
  print, 'scene_identifier:           ' + info.scene_identifier    
  print, 'scene_reference_number:     ' + info.scene_reference_number
  print, 'orbit_number:               ' + info.orbit_number        
  print, 'sar_platform_code:          ' + info.sar_platform_code    
  print, 'sensor_imaging_mode:        ' + info.sensor_imaging_mode 
  print, 'calibration_flag:           ' + info.calibration_flag    
  print, 'image_pixel_value_units:    ' + info.image_pixel_value_units
  print, 'orbit_pass_flag:            ' + info.orbit_pass_flag     
  print, 'sar_beam_code:              ' + info.sar_beam_code       
  print, 'map_descriptor:             ' + info.map_descriptor      
  print, 'range_type:                 ' + info.range_type          
  print, 'ref_ellipsoid:              ' + info.ref_ellipsoid       
  print, 'datum_shifts_m:             ' + info.datum_shifts_m      
  print, 'datum_scale:                ' + info.datum_scale         
  print, 'source_lonlat_fit:          ' + info.source_lonlat_fit   
  print, 'source_incid_fit:           ' + info.source_incid_fit    
  print, 'source_rlook_fit:           ' + info.source_rlook_fit    
endif
 
;
; Scalar values
;
; 
;
if printflag then print, 'Declaring IDs for scalar quantities for netCDF file.'

scaling_id=                      NCDF_VARID(id, 'scaling'                       )  
nominal_azimuth_resolution_m_id= NCDF_VARID(id, 'nominal_azimuth_resolution_m'  )  
nominal_range_resolution_m_id=   NCDF_VARID(id, 'nominal_range_resolution_m'    )  
pixel_spacing_m_id=              NCDF_VARID(id, 'pixel_spacing_m'               )  
line_spacing_m_id=               NCDF_VARID(id, 'line_spacing_m'                )  
heading_north_deg_id=            NCDF_VARID(id, 'heading_north_deg_id'          )  
heading_image_deg_id=            NCDF_VARID(id, 'heading_image_deg_id'          )  
range_swath_width_m_id=          NCDF_VARID(id, 'range_swath_width_m_id'        )  
azimuth_swath_width_m_id=        NCDF_VARID(id, 'azimuth_swath_width_m_id'      )  
sensor_altitude_m_id=            NCDF_VARID(id, 'sensor_altitude_m_id'          )  
incid_angle_center_deg_id=       NCDF_VARID(id, 'incid_angle_center_deg_id'     )  
slant_range_center_m_id=         NCDF_VARID(id, 'slant_range_center_m_id'       )  
slant_range_near_m_id=           NCDF_VARID(id, 'slant_range_near_m_id'         )  
lines_sio_image_id=              NCDF_VARID(id, 'lines_sio_image_id'            )  
pixels_sio_image_id=             NCDF_VARID(id, 'pixels_sio_image_id'           )  
lines_original_image_id=         NCDF_VARID(id, 'lines_original_image_id'       )  
pixels_original_image_id=        NCDF_VARID(id, 'pixels_original_image_id'      )  
looks_range_id=                  NCDF_VARID(id, 'looks_range_id'                )  
looks_azimuth_id=                NCDF_VARID(id, 'looks_azimuth_id'              )  
ref_semi_major_axis_m_id=        NCDF_VARID(id, 'ref_semi_major_axis_m_id'      )  
ref_semi_minor_axis_m_id=        NCDF_VARID(id, 'ref_semi_minor_axis_m_id'      )  
basis_lon_fit_id=                NCDF_VARID(id, 'basis_lon_fit_id'              )  
deg_lonlat_fit_id=               NCDF_VARID(id, 'deg_lonlat_fit_id'             )  
deg_incid_fit_id=                NCDF_VARID(id, 'deg_incid_fit_id'              )  
deg_rlook_fit_id=                NCDF_VARID(id, 'deg_rlook_fit_id'              )  


if scaling_id                       eq -1 then  scaling_id=                      0
if nominal_azimuth_resolution_m_id  eq -1 then  nominal_azimuth_resolution_m_id= 1 
if nominal_range_resolution_m_id    eq -1 then  nominal_range_resolution_m_id=   2
if pixel_spacing_m_id               eq -1 then  pixel_spacing_m_id=              3
if line_spacing_m_id                eq -1 then  line_spacing_m_id=               4
if heading_north_deg_id             eq -1 then  heading_north_deg_id=            5
if heading_image_deg_id             eq -1 then  heading_image_deg_id=            6
if range_swath_width_m_id           eq -1 then  range_swath_width_m_id=          7
if azimuth_swath_width_m_id         eq -1 then  azimuth_swath_width_m_id=        8
if sensor_altitude_m_id             eq -1 then  sensor_altitude_m_id=            9
if incid_angle_center_deg_id        eq -1 then  incid_angle_center_deg_id=       10
if slant_range_center_m_id          eq -1 then  slant_range_center_m_id=         11
if slant_range_near_m_id            eq -1 then  slant_range_near_m_id=           12
if lines_sio_image_id               eq -1 then  lines_sio_image_id=              13
if pixels_sio_image_id              eq -1 then  pixels_sio_image_id=             14
if lines_original_image_id          eq -1 then  lines_original_image_id=         15
if pixels_original_image_id         eq -1 then  pixels_original_image_id=        16
if looks_range_id                   eq -1 then  looks_range_id=                  17
if looks_azimuth_id                 eq -1 then  looks_azimuth_id=                18
if ref_semi_major_axis_m_id         eq -1 then  ref_semi_major_axis_m_id=        19
if ref_semi_minor_axis_m_id         eq -1 then  ref_semi_minor_axis_m_id=        20
if basis_lon_fit_id                 eq -1 then  basis_lon_fit_id=                21
if deg_lonlat_fit_id                eq -1 then  deg_lonlat_fit_id=               22
if deg_incid_fit_id                 eq -1 then  deg_incid_fit_id=                23
if deg_rlook_fit_id                 eq -1 then  deg_rlook_fit_id=                24


if printflag then begin
  print,   ' ids'
  print,   'scaling_id                       ' + string(scaling_id                     , format='(I8)')
  print,   'nominal_azimuth_resolution_m_id  ' + string(nominal_azimuth_resolution_m_id, format='(I8)')
  print,   'nominal_range_resolution_m_id    ' + string(nominal_range_resolution_m_id  , format='(I8)')
  print,   'pixel_spacing_m_id               ' + string(pixel_spacing_m_id             , format='(I8)')
  print,   'line_spacing_m_id                ' + string(line_spacing_m_id              , format='(I8)')
  print,   'heading_north_deg_id             ' + string(heading_north_deg_id           , format='(I8)')
  print,   'heading_image_deg_id             ' + string(heading_image_deg_id           , format='(I8)')
  print,   'range_swath_width_m_id           ' + string(range_swath_width_m_id         , format='(I8)')
  print,   'azimuth_swath_width_m_id         ' + string(azimuth_swath_width_m_id       , format='(I8)')
  print,   'sensor_altitude_m_id             ' + string(sensor_altitude_m_id           , format='(I8)')
  print,   'incid_angle_center_deg_id        ' + string(incid_angle_center_deg_id      , format='(I8)')
  print,   'slant_range_center_m_id          ' + string(slant_range_center_m_id        , format='(I8)')
  print,   'slant_range_near_m_id            ' + string(slant_range_near_m_id          , format='(I8)')
  print,   'lines_sio_image_id               ' + string(lines_sio_image_id             , format='(I8)')
  print,   'pixels_sio_image_id              ' + string(pixels_sio_image_id            , format='(I8)')
  print,   'lines_original_image_id          ' + string(lines_original_image_id        , format='(I8)')
  print,   'pixels_original_image_id         ' + string(pixels_original_image_id       , format='(I8)')
  print,   'looks_range_id                   ' + string(looks_range_id                 , format='(I8)')
  print,   'looks_azimuth_id                 ' + string(looks_azimuth_id               , format='(I8)')
  print,   'ref_semi_major_axis_m_id         ' + string(ref_semi_major_axis_m_id       , format='(I8)')
  print,   'ref_semi_minor_axis_m_id         ' + string(ref_semi_minor_axis_m_id       , format='(I8)')
  print,   'basis_lon_fit_id                 ' + string(basis_lon_fit_id               , format='(I8)')
  print,   'deg_lonlat_fit_id                ' + string(deg_lonlat_fit_id              , format='(I8)')
  print,   'deg_incid_fit_id                 ' + string(deg_incid_fit_id               , format='(I8)')
  print,   'deg_rlook_fit_id                 ' + string(deg_rlook_fit_id               , format='(I8)')
endif


if printflag then print, 'Reading scalar quantities
scaling                      = '0'
nominal_azimuth_resolution_m = '0'
nominal_range_resolution_m   = '0'
pixel_spacing_m              = '0'
line_spacing_m               = '0'
heading_north_deg            = '0'
heading_image_deg            = '0'
range_swath_width_m          = '0'
azimuth_swath_width_m        = '0'
sensor_altitude_m            = '0'
incid_angle_center_deg       = '0'
slant_range_center_m         = '0'
slant_range_near_m           = '0'
lines_sio_image              = '0'
pixels_sio_image             = '0'
lines_original_image         = '0'
pixels_original_image        = '0'
looks_range                  = '0'
looks_azimuth                = '0'
ref_semi_major_axis_m        = '0'
ref_semi_minor_axis_m        = '0'
basis_lon_fit                = '0'
deg_lonlat_fit               = '0'
deg_incid_fit                = '0'
deg_rlook_fit                = '0'



if  scaling_id                      gt -1 then NCDF_VARGET1, id, scaling_id,                       scaling                     
if  nominal_azimuth_resolution_m_id gt -1 then NCDF_VARGET1, id, nominal_azimuth_resolution_m_id,  nominal_azimuth_resolution_m
if  nominal_range_resolution_m_id   gt -1 then NCDF_VARGET1, id, nominal_range_resolution_m_id,    nominal_range_resolution_m  
if  pixel_spacing_m_id              gt -1 then NCDF_VARGET1, id, pixel_spacing_m_id,               pixel_spacing_m             
if  line_spacing_m_id               gt -1 then NCDF_VARGET1, id, line_spacing_m_id,                line_spacing_m              
if  heading_north_deg_id            gt -1 then NCDF_VARGET1, id, heading_north_deg_id,             heading_north_deg           
if  heading_image_deg_id            gt -1 then NCDF_VARGET1, id, heading_image_deg_id,             heading_image_deg           
if  range_swath_width_m_id          gt -1 then NCDF_VARGET1, id, range_swath_width_m_id,           range_swath_width_m         
if  azimuth_swath_width_m_id        gt -1 then NCDF_VARGET1, id, azimuth_swath_width_m_id,         azimuth_swath_width_m       
if  sensor_altitude_m_id            gt -1 then NCDF_VARGET1, id, sensor_altitude_m_id,             sensor_altitude_m           
if  incid_angle_center_deg_id       gt -1 then NCDF_VARGET1, id, incid_angle_center_deg_id,        incid_angle_center_deg      
if  slant_range_center_m_id         gt -1 then NCDF_VARGET1, id, slant_range_center_m_id,          slant_range_center_m        
if  slant_range_near_m_id           gt -1 then NCDF_VARGET1, id, slant_range_near_m_id,            slant_range_near_m          
if  lines_sio_image_id              gt -1 then NCDF_VARGET1, id, lines_sio_image_id,               lines_sio_image             
if  pixels_sio_image_id             gt -1 then NCDF_VARGET1, id, pixels_sio_image_id,              pixels_sio_image            
if  lines_original_image_id         gt -1 then NCDF_VARGET1, id, lines_original_image_id,          lines_original_image        
if  pixels_original_image_id        gt -1 then NCDF_VARGET1, id, pixels_original_image_id,         pixels_original_image       
if  looks_range_id                  gt -1 then NCDF_VARGET1, id, looks_range_id,                   looks_range                 
if  looks_azimuth_id                gt -1 then NCDF_VARGET1, id, looks_azimuth_id,                 looks_azimuth               
if  ref_semi_major_axis_m_id        gt -1 then NCDF_VARGET1, id, ref_semi_major_axis_m_id,         ref_semi_major_axis_m       
if  ref_semi_minor_axis_m_id        gt -1 then NCDF_VARGET1, id, ref_semi_minor_axis_m_id,         ref_semi_minor_axis_m       
if  basis_lon_fit_id                gt -1 then NCDF_VARGET1, id, basis_lon_fit_id,                 basis_lon_fit               
if  deg_lonlat_fit_id               gt -1 then NCDF_VARGET1, id, deg_lonlat_fit_id,                deg_lonlat_fit              
if  deg_incid_fit_id                gt -1 then NCDF_VARGET1, id, deg_incid_fit_id,                 deg_incid_fit               
if  deg_rlook_fit_id                gt -1 then NCDF_VARGET1, id, deg_rlook_fit_id,                 deg_rlook_fit               

temp_info= create_struct( info,                                                                 $
                          'scaling',                      string(scaling),                      $
                          'nominal_azimuth_resolution_m', string(nominal_azimuth_resolution_m), $
                          'nominal_range_resolution_m',   string(nominal_range_resolution_m),   $
                          'pixel_spacing_m',              string(pixel_spacing_m),              $
                          'line_spacing_m',               string(line_spacing_m),               $
                          'heading_north_deg',            string(heading_north_deg),            $
                          'heading_image_deg',            string(heading_image_deg),            $
                          'range_swath_width_m',          string(range_swath_width_m),          $
                          'azimuth_swath_width_m',        string(azimuth_swath_width_m),        $
                          'sensor_altitude_m',            string(sensor_altitude_m),            $
                          'incid_angle_center_deg',       string(incid_angle_center_deg),       $
                          'slant_range_center_m',         string(slant_range_center_m),         $
                          'slant_range_near_m',           string(slant_range_near_m),           $
                          'lines_sio_image',              string(lines_sio_image),              $
                          'pixels_sio_image',             string(pixels_sio_image),             $
                          'lines_original_image',         string(lines_original_image),         $
                          'pixels_original_image',        string(pixels_original_image),        $
                          'looks_range',                  string(looks_range),                  $
                          'looks_azimuth',                string(looks_azimuth),                $
                          'ref_semi_major_axis_m',        string(ref_semi_major_axis_m),        $
                          'ref_semi_minor_axis_m',        string(ref_semi_minor_axis_m),        $
                          'basis_lon_fit',                string(basis_lon_fit),                $
                          'deg_lonlat_fit',               string(deg_lonlat_fit),               $
                          'deg_incid_fit',                string(deg_incid_fit),                $
                          'deg_rlook_fit',                string(deg_rlook_fit)                 )

info= 0
info= temp_info

if printflag then begin                         
  print, 'Scalar values'
  print, '------------------------------------------------------------------------------------'
  print, 'scaling                       = ' +  strcompress(info.scaling                       )
  print, 'nominal_azimuth_resolution_m  = ' +  strcompress(info.nominal_azimuth_resolution_m  )
  print, 'nominal_range_resolution_m    = ' +  strcompress(info.nominal_range_resolution_m    )
  print, 'pixel_spacing_m               = ' +  strcompress(info.pixel_spacing_m               )
  print, 'line_spacing_m                = ' +  strcompress(info.line_spacing_m                )
  print, 'heading_north_deg             = ' +  strcompress(info.heading_north_deg             )
  print, 'heading_image_deg             = ' +  strcompress(info.heading_image_deg             )
  print, 'range_swath_width_m           = ' +  strcompress(info.range_swath_width_m           )
  print, 'azimuth_swath_width_m         = ' +  strcompress(info.azimuth_swath_width_m         )
  print, 'sensor_altitude_m             = ' +  strcompress(info.sensor_altitude_m             )
  print, 'incid_angle_center_deg        = ' +  strcompress(info.incid_angle_center_deg        )
  print, 'slant_range_center_m          = ' +  strcompress(info.slant_range_center_m          )
  print, 'slant_range_near_m            = ' +  strcompress(info.slant_range_near_m            )
  print, 'lines_sio_image               = ' +  strcompress(info.lines_sio_image               )
  print, 'pixels_sio_image              = ' +  strcompress(info.pixels_sio_image              )
  print, 'lines_original_image          = ' +  strcompress(info.lines_original_image          )
  print, 'pixels_original_image         = ' +  strcompress(info.pixels_original_image         )
  print, 'looks_range                   = ' +  strcompress(info.looks_range                   )
  print, 'looks_azimuth                 = ' +  strcompress(info.looks_azimuth                 )
  print, 'ref_semi_major_axis_m         = ' +  strcompress(info.ref_semi_major_axis_m         )
  print, 'ref_semi_minor_axis_m         = ' +  strcompress(info.ref_semi_minor_axis_m         )
  print, 'basis_lon_fit                 = ' +  strcompress(info.basis_lon_fit                 )
  print, 'deg_lonlat_fit                = ' +  strcompress(info.deg_lonlat_fit                )
  print, 'deg_incid_fit                 = ' +  strcompress(info.deg_incid_fit                 )
  print, 'deg_rlook_fit                 = ' +  strcompress(info.deg_rlook_fit                 )
endif

if printflag then print, 'Declaring IDs for array quantities for netCDF file.'
sigmaid=        NCDF_VARID( id, 'sigma'         ) 
maskid=         NCDF_VARID( id, 'mask'          ) 
lon_coefid=     NCDF_VARID( id, 'lon_coef'      ) 
lon_xexpid=     NCDF_VARID( id, 'lon_xexp'      ) 
lon_yexpid=     NCDF_VARID( id, 'lon_yxep'      ) 
lat_coefid=     NCDF_VARID( id, 'lat_coef'      ) 
lat_xexpid=     NCDF_VARID( id, 'lat_xexp'      ) 
lat_yexpid=     NCDF_VARID( id, 'lat_yexp'      ) 
i_coefid=       NCDF_VARID( id, 'i_coef'        ) 
i_xexpid=       NCDF_VARID( id, 'i_xexp'        ) 
i_yexpid=       NCDF_VARID( id, 'i_yxep'        ) 
j_coefid=       NCDF_VARID( id, 'j_coef'        ) 
j_xexpid=       NCDF_VARID( id, 'j_xexp'        ) 
j_yexpid=       NCDF_VARID( id, 'j_yxep'        ) 
incid_coefid=   NCDF_VARID( id, 'incid_coef'    ) 
incid_xexpid=   NCDF_VARID( id, 'incid_xexp'    ) 
incid_yexpid=   NCDF_VARID( id, 'incid_yxep'    ) 
rlook_coefid=   NCDF_VARID( id, 'rlook_coef'    ) 
rlook_xexpid=   NCDF_VARID( id, 'rlook_xexp'    ) 
rlook_yexpid=   NCDF_VARID( id, 'rlook_yxep'    ) 


NCDF_VARGET,  id, sigmaid,       sigma 
NCDF_VARGET,  id, maskid,        mask 
NCDF_VARGET,  id, lon_coefid,    lon_coef 
NCDF_VARGET,  id, lon_xexpid,    lon_xexp 
NCDF_VARGET,  id, lon_yexpid,    lon_yexp 
NCDF_VARGET,  id, lat_coefid,    lat_coef 
NCDF_VARGET,  id, lat_xexpid,    lat_xexp 
NCDF_VARGET,  id, lat_yexpid,    lat_yexp   
NCDF_VARGET,  id, i_coefid,      i_coef 
NCDF_VARGET,  id, i_xexpid,      i_xexp 
NCDF_VARGET,  id, i_yexpid,      i_yexp 
NCDF_VARGET,  id, j_coefid,      j_coef 
NCDF_VARGET,  id, j_xexpid,      j_xexp 
NCDF_VARGET,  id, j_yexpid,      j_yexp 
NCDF_VARGET,  id, incid_coefid,  incid_coef
NCDF_VARGET,  id, incid_xexpid,  incid_xexp
NCDF_VARGET,  id, incid_yexpid,  incid_yexp
NCDF_VARGET,  id, rlook_coefid,  rlook_coef
NCDF_VARGET,  id, rlook_xexpid,  rlook_xexp
NCDF_VARGET,  id, rlook_yexpid,  rlook_yexp

if printflag then begin
  print, 'sigma      ', format='(8A, $)' &  hlp, sigma 
  print, 'mask       ', format='(8A, $)' &  hlp, mask 
  print, 'lon_coef   ', format='(8A, $)' &  hlp, lon_coef 
  print, 'lon_xexp   ', format='(8A, $)' &  hlp, lon_xexp 
  print, 'lon_yexp   ', format='(8A, $)' &  hlp, lon_yexp 
  print, 'lat_coef   ', format='(8A, $)' &  hlp, lat_coef 
  print, 'lat_xexp   ', format='(8A, $)' &  hlp, lat_xexp 
  print, 'lat_yexp   ', format='(8A, $)' &  hlp, lat_yexp   
  print, 'i_coef     ', format='(8A, $)' &  hlp, i_coef 
  print, 'i_xexp     ', format='(8A, $)' &  hlp, i_xexp 
  print, 'i_yexp     ', format='(8A, $)' &  hlp, i_yexp 
  print, 'j_coef     ', format='(8A, $)' &  hlp, j_coef 
  print, 'j_xexp     ', format='(8A, $)' &  hlp, j_xexp 
  print, 'j_yexp     ', format='(8A, $)' &  hlp, j_yexp 
  print, 'incid_coef ', format='(8A, $)' &  hlp, incid_coef
  print, 'incid_xexp ', format='(8A, $)' &  hlp, incid_xexp
  print, 'incid_yexp ', format='(8A, $)' &  hlp, incid_yexp
  print, 'rlook_coef ', format='(8A, $)' &  hlp, rlook_coef
  print, 'rlook_xexp ', format='(8A, $)' &  hlp, rlook_xexp
  print, 'rlook_yexp ', format='(8A, $)' &  hlp, rlook_yexp
endif
;
; Fill in info structure
;
temp_info=0
temp_info= create_struct( info,                                             $
                          'lon_coef',               lon_coef,               $ 
                          'lon_xexp',               lon_xexp,               $ 
                          'lon_yexp',               lon_yexp,               $ 
                          'lat_coef',               lat_coef,               $    
                          'lat_xexp',               lat_xexp,               $ 
                          'lat_yexp',               lat_yexp,               $   
                          'i_coef',                 i_coef,                 $ 
                          'i_xexp',                 i_xexp,                 $ 
                          'i_yexp',                 i_yexp,                 $ 
                          'j_coef',                 j_coef,                 $ 
                          'j_xexp',                 j_xexp,                 $ 
                          'j_yexp',                 j_yexp,                 $ 
                          'incid_coef',             incid_coef,             $
                          'incid_xexp',             incid_xexp,             $
                          'incid_yexp',             incid_yexp,             $
                          'rlook_coef',             rlook_coef,             $
                          'rlook_xexp',             rlook_xexp,             $
                          'rlook_yexp',             rlook_yexp              )
        

NCDF_CLOSE, id

if printflag then print, 'Finished reading: '+ file

	

end