;----------------------------------------------------------
;+
; NAME: ANS_READ_LLCORNERS_L2CDF
; PURPOSE:'
;   Read Level-2 SAR level-2 netCDF file and extract the 4 corner latitide and longitude. This version
;   of the program can now read level2 netCDF files that are both CF compliant and readable by
;   panopoly
; CALLING SEQUENCE:'
;   ans_read_wind_l2_cdf2, file
; INPUT:'
;   file:    Level 2 wind retrieval file in CDF format
; OUTPUT:
;
; KEYWORDS:
;   /quiet        = Do not print intermediate results'
;   /help         = List help.
;   error         = <0 bad
; MOFICATION HISTORY
;   2013 Dec 10 Chris Jackson. Adapted from ans_read_wind_l2_ccdf2
; Copyright (C) 2010-2013, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-


pro ans_read_llcorners_l2cdf, file, $
                    quiet        = quiet,       $
                    help         = help,        $
                    error        = error

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' PURPOSE:'
  print, '   
  print, '   
  print, '   
  print, ' CALLING SEQUENCE:'
  print, '   ans_read_llcorners_l2cdf, file
  print, ' INPUT:'
  print, '   file:    Level 2 wind retrieval file in CDF format
  print, ' OUTPUT:
  print, '   
  print, ' KEYWORDS:
  print, '   /quiet        = Do not print intermediate results'
  print, '   /help         = List help.
  print, '   error         = <0 bad
  error=-1
  return
endif
;
; Parcing input parameters
;
error    = 0
printflag= 1
if keyword_set(quiet) then printflag=0

res=file_search(file, count=count)
if count eq 0 then begin
  print, 'Cannot find file '+ file + ' .'
  print, 'Exiting ans_read_llcorners_l2cdf2...'
  return
endif

id = NCDF_OPEN(file, /nowrite)
;
; Global Attributes.
;
platform_name             = 'BLANK'
acq_date_time             = 'BLANK'

r = NCDF_ATTINQ(id, 'answrs:platform_name'                 , /GLOBAL) & fplatform_name              = r.datatype ne 'UNKNOWN'
r = NCDF_ATTINQ(id, 'answrs:acquistion_time'               , /GLOBAL) & facquistion_time            = r.datatype ne 'UNKNOWN'

if fplatform_name              then NCDF_ATTGET, id, /GLOBAL, 'answrs:platform_name'                  , platform_name
if facquistion_time            then NCDF_ATTGET, id, /GLOBAL, 'answrs:acquistion_time'                , acq_date_time

platform_name                  = strcompress(string( platform_name            ))
acq_date_time                  = strcompress(string( acq_date_time            ))
;
;
; Scalar values
;
;
;
if printflag then print, 'Declaring IDs for scalar quantities for netCDF file.'
upper_left_longitude_id=    NCDF_VARID(id, 'upper_left_longitude'   )
upper_right_longitude_id=   NCDF_VARID(id, 'upper_right_longitude'  )
upper_left_latitude_id=     NCDF_VARID(id, 'upper_left_latitude'    )
upper_right_latitude_id=    NCDF_VARID(id, 'upper_right_latitude'   )
lower_left_longitude_id=    NCDF_VARID(id, 'lower_left_longitude'   )
lower_right_longitude_id=   NCDF_VARID(id, 'lower_right_longitude'  )
lower_left_latitude_id=     NCDF_VARID(id, 'lower_left_latitude'    )
lower_right_latitude_id=    NCDF_VARID(id, 'lower_right_latitude'   )


upper_left_longitude      = 0.0
upper_right_longitude     = 0.0
upper_left_latitude       = 0.0
upper_right_latitude      = 0.0
lower_left_longitude      = 0.0
lower_right_longitude     = 0.0
lower_left_latitude       = 0.0
lower_right_latitude      = 0.0

if  upper_left_longitude_id   ge 0 then NCDF_VARGET1, id, upper_left_longitude_id,   upper_left_longitude
if  upper_right_longitude_id  ge 0 then NCDF_VARGET1, id, upper_right_longitude_id,  upper_right_longitude
if  upper_left_latitude_id    ge 0 then NCDF_VARGET1, id, upper_left_latitude_id,    upper_left_latitude
if  upper_right_latitude_id   ge 0 then NCDF_VARGET1, id, upper_right_latitude_id,   upper_right_latitude
if  lower_left_longitude_id   ge 0 then NCDF_VARGET1, id, lower_left_longitude_id,   lower_left_longitude
if  lower_right_longitude_id  ge 0 then NCDF_VARGET1, id, lower_right_longitude_id,  lower_right_longitude
if  lower_left_latitude_id    ge 0 then NCDF_VARGET1, id, lower_left_latitude_id,    lower_left_latitude
if  lower_right_latitude_id   ge 0 then NCDF_VARGET1, id, lower_right_latitude_id,   lower_right_latitude

if printflag then begin
 print, 'upper_left_longitude   =' + string( upper_left_longitude  )
 print, 'upper_right_longitude  =' + string( upper_right_longitude )
 print, 'upper_left_latitude    =' + string( upper_left_latitude   )
 print, 'upper_right_latitude   =' + string( upper_right_latitude  )
 print, 'lower_left_longitude   =' + string( lower_left_longitude  )
 print, 'lower_right_longitude  =' + string( lower_right_longitude )
 print, 'lower_left_latitude    =' + string( lower_left_latitude   )
 print, 'lower_right_latitude   =' + string( lower_right_latitude  )
endif



ncdf_close, id

print,'<Placemark>'
print,'   <name>'+acq_date_time+'</name>'
print,'		<styleUrl>#m_ylw-pushpin</styleUrl>'
print,'		<Polygon>'
print,'			<tessellate>1</tessellate>'
print,'			<outerBoundaryIs>'
print,'				<LinearRing>'
print, '<coordinates>'+strcompress(string(upper_right_longitude))+','+ strcompress(string( upper_left_latitude))+',0 ' + $
strcompress(string(upper_left_longitude))+','+ strcompress(string( upper_left_latitude ))+',0 ' +  $
strcompress(string(lower_left_longitude))+','+ strcompress(string( lower_left_latitude ))+',0 ' +  $
strcompress(string(lower_right_longitude))+','+ strcompress(string( lower_left_latitude))+',0 ' +  $
strcompress(string(upper_right_longitude))+','+ strcompress(string( upper_left_latitude))+',0 </coordinates>'
print,'				</LinearRing>'
print,'			</outerBoundaryIs>'
print,'		</Polygon>'
print,'</Placemark>'


end
