;+---------------------------
; NAME: ANS_READ_PSAR
; PURPOSE:
;    The purpose of this program is to read ALOS PALSAR data and
;    create a SIO formatted binary image file and ASCII file
; CALLING SEQUENCE:
;    ans_read_psar, datafile, leaderfile, deffile
; INPUT:
;    datafile    =  CEOS file with SAR data
;    leaderfie   =  CEOS file if ancillary data information
;    deffile     =  Defaults file
; OUTPUT:          
;    Binary files with image and land mask data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;        PSAR_JAX_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;        PSAR_JAX_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;    The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header.
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (0 if program completes or -1 if not)
;     outputroot   = Output root name of files
;     jsstart      = Start processing time
; MODIFICATION HISTORY
;  2009 Nov 19 Frank Monaldo.  
;  2009 Dec 16 Frank Monaldo. Corrected misspelling of gshhs_dir
;  2010 Jan 15 Frank Monaldo. setenv gshhs_dir             
;  2010 Jan 19 Frank Monaldo. Add /header option to only print out ancil_data
;                             Add /nomask to suppress output of land mask
;  2010 Jan 26 Frank Monaldo. Add pixel_size
;  2010 Jan 28 Frank Monaldo. Add option for creating ANSWRS version. This
;                             require the idlans directory name
;  2010 Feb 05 Frank Monaldo. Usd the help call to find the source file directory
;                             This is used to calculated the time of last modifications.
;                             This removes need for providing the name of source directory.
;                             Typing command w/no parameters also calls help. 
;  2010 May 26 Frank Monaldo. Move2dir added
;  2010 May 27 Frank Monaldo. 
;  2011 Jan 01 Frank Monaldo. Log_dir added, Pass on quiet flag
;  2011 Jan 07 Frank Monlado. Add the ability to deal with files in the original naming conventon
;  2011 Feb 05 Frank Monaldo. Add jsstart
;  2011 Feb 07 Frank Monaldo. Add original file name to output log
;  2011 Feb 13 Frank Monaldo. Change unique working directory name to something that makes more sense
;  2011 Feb 14 Frank Monaldo. Add ability to escape if .lock file encountered
;  2011 Mar 30 Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2012 Jul 05 Frank Monaldo. Removed hardwired directories and files
;  2012 Aug 21 Frank Monaldo. Move ans_gen_version
;  2013 Feb 02 Frank Monaldo. Add ice mask generation and rely solely on defaults file
;  2013 Apr 23 Frank Monaldo. Add option to produce longitude, latitude, incident angle, and
;                             radar look direction SIO files.
;  2013 Aug 14 Frank Monaldo. Move production of icemask, longitude, latitude, incident angle,
;                             and radar look direction to ans_sat2sio. Remove delete lockfile
;  2013 Aug 20 Frank Monaldo. Move land mask generation to ans_sat2sio
;  2014 May 12 Chris Jackson. Add a "2" to the end of the PSAR prefix if data are from ALOS2
;  2017 Aug 08 Chris Jackson. Forced the format of the output center longitude/latitude to F06.2/F05.2 respectively  
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;

pro ans_read_psar, datafile, leaderfile, deffile, $
                    quiet       = quiet,      $
                    help        = help,       $
                    outputroot  = outputroot, $
                    jsstart     = jsstart,    $
                    error       = error     
                    
error=     0
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_PSAR
  print, ' PURPOSE:
  print, '    The purpose of this program is to read ALOS PALSAR data and
  print, '    create a SIO formatted binary image file and ASCII file
  print, ' CALLING SEQUENCE:
  print, '    ans_read_psar, datafile, leaderfile, deffile
  print, ' INPUT:
  print, '    datafile    =  CEOS file with SAR data
  print, '    leaderfie   =  CEOS file if ancillary data information
  print, '    deffile     =  Defaults file
  print, ' OUTPUT:          
  print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '    convention for file names is of the form:
  print, '        PSAR_JAX_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '        PSAR_JAX_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '    The file name is delimited by _ and the entries are Sensor, processing facility, year, month, day, hour, minute
  print, '    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '    files include the SIO 20-byte header.
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  print, '     error        = error (0 if program completes or -1 if not)
  print, '     outputroot   = Output root name of files
  print, '     jsstart      = Start processing timee
  error= -1
  return
endif


;
; Parcing input parameters
;
printflag=1
if keyword_set(quiet) then printflag=0
quiet=  printflag eq 0
error   = 1

if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 


time0 = dt_tm_tojs(systime())

ans_read_defaults, deffile, df, quiet=quiet, error=error
if error lt 0 then return

if n_elements(df.gshhs_dir)     lt 1 then begin
  print, 'gshhs_dir not specified.'
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif
if n_elements(df.idlans_dir)    lt 1 then begin
  print, 'idlans_dir not specified.
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif
if n_elements(df.log_dir)       lt 1 then begin
  print, 'lof_dir not specified.
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet

setenv, 'GSHHS_DIR=' + df.gshhs_dir

; Determine if input files exist
; 
ff= file_search( datafile, count=count)

if count eq 0 then begin
  print, 'Could not find PALASR data file: ' + datafile
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif
ff= file_search( leaderfile, count=count)

if count eq 0 then begin
  print, 'Could not find PALASR leader file: ' + leaderfile
  print, 'Exiting ans_read_psar.pro '
  error= -1
  return
endif


time1=dt_tm_tojs(systime())

;
; Create a new working directory
;
working_directory= df.working_directory + '/psar_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory

;
; Copy PALSAR files to working directory.   Note the movement of PALSAR directory to 
; and already processed directory must occur at a different level. It is not done in this program
;
filebreak, leaderfile, directory=directory, file=file

len             = strlen(file)
found           = 0
name_convention = 0
if strupcase( strmid(file, len-1, 1) ) eq 'L'   then name_convention=1
if strupcase( strmid(file,     0, 3) ) eq 'LED' then name_convention=2

if name_convention eq 0 then begin
  print, 'Does not correspond to naming convention', ff[0]
  print, 'Exiting ans_read_psar'
  error = -1
  return
end

case name_convention of
1: begin
     length=      strlen(leaderfile)
     trailerfile= strmid(leaderfile, 0, length-2) +'.T'
     volumefile=  strmid(leaderfile, 0, length-2) +'.V'
     res=         file_search(trailerfile, count=tcount)
     res=         file_search(volumefile,  count=vcount)
   end
2: begin
     len=         strlen(file)
     root=        strmid(file, 3, len-3)
     trailerfile= file_search(directory +'TRL*' + root, count=tcount)
     volumefile=  file_search(directory +'VOL*' + root, count=vcount)
   end
endcase
;
; Copy input files to working directory
;
if printflag then begin
  print, 'Copying ' + leaderfile + ' and '
  print, '        ' + datafile   + ' to '
  print, working_directory
endif 
file_copy, leaderfile, working_directory  + '/' 
file_copy, datafile,   working_directory  + '/' 
sleaderfile=    getwrd(leaderfile, 0, /last, del='/')
sdatafile=      getwrd(datafile,   0, /last, del='/')
;
; Moving input files to alreaduy processed files
;
res= file_search(df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, move2dir
if printflag then begin
  print, 'Moving ' + leaderfile    
  print, '       ' + datafile     
  if tcount gt 0 then print, '       ' + trailerfile   
  if vcount gt 0 then print, '       ' + volumefile    
  print, 'to     ' + df.already_processed
endif 
file_move, leaderfile,  df.already_processed  + '/' , /overwrite
file_move, datafile,    df.already_processed  + '/' , /overwrite
if tcount gt 0 then file_move, trailerfile[0], df.already_processed  + '/' , /overwrite   
if vcount gt 0 then file_move, volumefile[0],  df.already_processed  + '/' , /overwrite
;
; Create leader file structure and read ancillary data
;
psar_create_lf,        lf, quiet=quiet
psar_read_leader,      working_directory  + '/' + sleaderfile, lf, good_leader, quiet=quiet
if good_leader lt 0 then begin
  print, 'Error reading '+working_directory  + '/' + sleaderfile
  error=-1
  return
endif

lf.name_dir            = datafile
lf.ave_pixel           = df.ave_pixel
lf.ave_line            = df.ave_line
lf.deg                 = df.deg
lf.nrcs_slope          = df.psar_slope
lf.nrcs_bias           = df.psar_bias
;
; Read image data
;
psar_read_image_file,  working_directory  + '/' + sdatafile, image, lf, quiet=quiet, error=error
if error lt 0 then begin
  print, 'Error reading '+working_directory  + '/' + sdatafile
  error=-1
  return
endif


psar_r_look,           lf, quiet=quiet
psar_dn2sigma, image,  lf, quiet=quiet   
;
; Correct NRCS if necessary.
;
if lf.nrcs_slope ne 1 or lf.nrcs_bias ne 0 then begin
  image=  image * lf.nrcs_slope + lf.nrcs_bias
endif
 
; Add to struture
temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
lf=      0
lf=      temp_lf

time2=dt_tm_tojs(systime())

if printflag then begin
  print, 'Time to move/read PALSAR files: ' + strcompress(string(time2-time1), /remove_all) + ' s'
endif
;
; Create file name root for output files.
;

lf.month = strupcase(lf.month)
case strupcase(lf.month) of
'JAN': month=1
'FEB': month=2
'MAR': month=3
'APR': month=4
'MAY': month=5
'JUN': month=6
'JUL': month=7
'AUG': month=8
'SEP': month=9
'OCT': month=10
'NOV': month=11
'DEC': month=12
ELSE:  month=1
endcase
;
;
; compute center geographic position
;


center_lat= string(abs(lf.center_lat), format='(F05.2)')
if lf.center_lat ge 0 then begin
  center_lat= center_lat + 'N'
endif else begin
  center_lat= center_lat + 'S'
endelse
center_lon= string(abs(lf.center_lon), format='(F06.2)')
if lf.center_lon ge 0 then begin
  center_lon= center_lon + 'E'
endif else begin
  center_lon= center_lon + 'W'
endelse

timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
         lf.month                               + ' ' + $
         string( lf.day,       format='(I2.2)') + ' ' + $
         string( lf.hour,      format='(I2.2)') + ':' + $
         string( lf.minute,    format='(I2.2)') + ':' + $
         string( lf.second,    format='(I2.2)')
         
js=            string( dt_tm_tojs(timestr), format='(I10.10)' ) 
output_prefix= 'PSAR'
;if (lf.data_year ge 2014) then output_prefix= 'PSAR2'
output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
if (lf.data_year ge 2014) then output_prefix= output_prefix + '2'
output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + string( center_lon)
output_prefix= output_prefix + '_' + string( center_lat)
output_prefix= strcompress( output_prefix, /remove_all )


output_prefix2= output_prefix + '_' + strupcase(lf.polarization) + '_L'

;
; Check and see if a lock file exits
;
lockfile=   df.sio_output_dir + '/'+ output_prefix2  + '*.lock'
;
res= file_search( lockfile, count=count)
if count gt 0 then begin
   print, 'Lockfile:  ' + lockfile + ' exists'
   error=-1
   return
endif
lockfile=   df.sio_output_dir + '/'+ output_prefix2  + '.lock'
close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1

;
; Write out to SIO data file format. This is general for all.
; SAR data
;
scaling=1.0
logfile= df.log_dir + '/' + output_prefix2 + '.log'
outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd(lf.name_dir, 0, /last, del='/')  + ' Processing starts'
outstr  = stress( outstr, 'R', 0, '//', '/', ned)
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr
if not keyword_set(header) then begin
  time3= dt_tm_tojs(systime())
  if strupcase(df.twobyte) eq 'Y' then begin
    ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, /twobyte, quiet=quiet
  endif else begin
    ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, quiet=quiet
  endelse
  time4= dt_tm_tojs(systime())
  if printflag then begin
    print, 'Time to write NRCS SIO file ' + strcompress( string(time4- time3), /remove_all) + ' s'
  endif
  time4= dt_tm_tojs(systime(/utc))
  outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
endif
;
; Write out SIO ancillary file format. This program is explicitly written for PALSAR data
;
time4= dt_tm_tojs(systime())
psar_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
time5= dt_tm_tojs(systime())
if printflag then begin
  print, 'Time to write ancillary data SIO file ' + strcompress( string(time5- time4), /remove_all) + ' s'
endif
time4= dt_tm_tojs(systime(/utc))
outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
outstr  = stress( outstr, 'R', 0, '//', '/', ned)
outstr  = stress( outstr, 'R', 0, '//', '/', ned)
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr

if printflag then print, 'Deleting ' + working_directory 
file_delete, working_directory  + '/', /recursive, quiet=quiet , /allow_nonexistent
;
; Generate root output file name
outputroot= df.sio_output_dir + '/'+ output_prefix2 



end
