;+---------------------------
; NAME: ANS_READ_RCM
; PURPOSE:
;  The purpose of this program is to read RCM data and
;  create a SIO formatted binary image file, ASCII file
; CALLING SEQUENCE:
;  ans_read_rcm, input_dir, deffile
; INPUT:
;  input_dir=    RCM come in directories. This input is the name
;                of such a directory. A directory might look like
;                RCM1_OK76385_PK678063_3_SC30MA_20080415_143807_HH_HV_GRD/ 
;  deffile  =    Defaults files
; OUTPUT:          
;  Binary files with image and land mask data and an ASCII file with ancillary data. The 
;  convention for file names is of the form:
;      RCM-1_CSA_2008_04_15_14_38_07_0261585487_129.12W_33.43N_HH_image.dat (image file)
;      RCM-1_CSA_2008_04_15_14_38_07_0261585487_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;  The file name is delimited by _ and the entries are Sensor, processin facility, year, month, day, hour, minute
;  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;  files include the SIO 20-byte header/  
; KEYWORDS:
;  outputroot  =  Full path name of output root file name (Output from program)
;  jssatrt     =  Processing start time (Output from program )
;  error       =  Error (0 if program completes or -1 if not)
;  /quiet      =  Do not print intermediate results.
;  /help       =  Print out help
; MODIFICATION HISTORY
;  2018 Jul 13 Frank Monaldo. Update sio_cross_pol use
;  2020 Feb 13 Frank Monaldo. Spelling error in comments
;  2020 Feb 21 Frank Monaldo. If the img data are complex, square and convert to real 
;                             before writing out SIO data file.
;                             If we ever want to deal with complex data in the SIO
;                             we will have to change this.
;  2020 May 26 Chris Jackson.  Update format statements for center_lat and center_lon to maitain field size with leading zeros
;  2021 Mar 02 Chris Jackson.  Put in logic to process only the polarization selected by the sio_cross_pol parameter in the defaults file
;                              Remove the dash "-" in the RCM designator in the output file (RCM# instead of RCM-#) 
;  2021 Mar 17 Chris Jackon.  Adjust values in lutsigma_org when lf.pixeltimeordering is 'DECREASING' (See note near test)
;  2021 Apr 02 Chris Jackson. Eliminate the copy of the S1 directory to the working_directory.
;                             Remove the move to already processed (now in ans_sen2sio)
;                             and include a delete of the sio lock file in this routine.  
; Copyright (C) 1999-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;

pro ans_read_rcm, input_dir, deffile          , $
                  output_dir  =  output_dir   , $
                  outputroot  =  outputroot   , $
                  error       =  error        , $          
                  jsstart     =  jsstart      , $                    
                  header      =  header       , $
                  quiet       =  quiet        , $
                  help        =  help     
                    
error=     1

if keyword_set(help) or n_params(0) lt 1 then begin
  print, ' NAME: ANS_READ_RCM
  print, ' PURPOSE:
  print, '  The purpose of this program is to read RCM data and
  print, '  create a SIO formatted binary image file, ASCII file
  print, ' CALLING SEQUENCE:
  print, '  ans_read_rcm, input_dir, deffile
  print, ' INPUT:
  print, '  input_dir=    RCM come in directories. This input is the name
  print, '                of such a directory. A directory might look like
  print, '                RCM1_OK76385_PK678063_3_SC30MA_20080415_143807_HH_HV_GRD/ 
  print, '  deffile  =    Defaults files
  print, ' OUTPUT:          
  print, '  Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '  convention for file names is of the form:
  print, '      RCM-1_CSA_2008_04_15_14_38_07_0261585487_129.12W_33.43N_HH_image.dat (image file)
  print, '      RCM-1_CSA_2008_04_15_14_38_07_0261585487_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '  The file name is delimited by _ and the entries are Sensor, processin facility, year, month, day, hour, minute
  print, '  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '  files include the SIO 20-byte header/  
  print, ' KEYWORDS:
  print, '  outputroot  =  Full path name of output root file name (Output from program)
  print, '  jssatrt     =  Processing start time (Output from program )
  print, '  error       =  Error (0 if program completes or -1 if not)
  print, '  /quiet      =  Do not print intermediate results.
  print, '  /help       =  Print out help
  error= -1
  return
endif

printflag =1
if keyword_set(quiet) then printflag=0
quiet=  printflag eq 0
if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 

time0 = dt_tm_tojs(systime())
                   
if file_test( deffile) then ans_read_defaults, deffile, df, quiet=quiet, error=error
if error lt 0 then begin
  if printflag then begin
     print, 'Cannot read file: ' + deffile 
     print, 'Existing ans_read_rcm.pro'
  endif
  error = -2
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
;
; Check values
;
if n_elements(df.gshhs_dir)      lt 1 then begin
  if printflag then begin
    print, 'gshhs_dir not specified.'
    print, 'Exiting ans_read_rcm.pro '
  endif
  error= -3
  return
endif
if n_elements(df.idlans_dir)     lt 1 then  begin
  if printflag then begin
    print, 'idlans_dir not specified.'
    print, 'Exiting ans_read_rcm.pro '
  endif
  error= -4
  return
endif
if n_elements(df.log_dir)        lt 1 then  begin
  if printflag then begin
    print, 'log_dir not specified.'
    print, 'Exiting ans_read_rcm.pro '
  endif
  error= -5
  return
endif

;
; Determine if directory exists
; 
if printflag then print, 'Checking for exitence of RCM directory...'
time1= dt_tm_tojs(systime())

ff= file_search( input_dir, /test_directory, count=count)

if count eq 0 then begin
  print, 'Could not find RCM directory: ' + input_dir
  print, 'Exiting ans_read_rcm.pro '
  error= -6
  return
endif

print, 'Using Directory ' + ff[0]
;
; Find the product.xml file and the base_directory for processing, ff= full path to product.xml file
; Find leader file
;
leaderfile=  file_search( ff[0],'product.xml', count=count)
if count eq 0 then begin
  print, 'Cannot find product.xml in ' + ff[0]
  print, 'Exiting ans_read_rcm.pro '
  error= -1
  return
endif

rcm_read_product, leaderfile, lf, ii, jj, incidence, longitude, latitude, quiet=quiet

;
; Setting slope and bias corrections
case lf.sar_sensor of
'RCM-1': begin
           for i=0,3 do begin
             p      =  getwrd( lf.polarization[i] )
             lf.pol = p
             case p of
             ''   : ; No operation
             'HH' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_hh
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_hh
                    end
             'VV' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_vv
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_vv
                    end
             'VH' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_vh
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_vh
                    end
             'HV' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_hv
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_hv
                    end
             'CV' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_cv
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_cv
                    end
             'CH' : begin
                    lf.nrcs_slope_array[i] = df.rcm1_slope_ch
                    lf.nrcs_bias_array[i]  = df.rcm1_bias_ch
                    end
             ELSE : ; No operation
             endcase
           endfor
         end
'RCM-2': begin
          for i=0,3 do begin
             p      =  getwrd( lf.polarization[i] )
             case p of
             ''   : ; No operation
             'HH' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_hh
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_hh
                    end
             'VV' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_vv
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_vv
                    end
             'VH' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_vh
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_vh
                    end
             'HV' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_hv
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_hv
                    end
             'CV' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_cv
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_cv
                    end
             'CH' : begin
                    lf.nrcs_slope_array[i] = df.rcm2_slope_ch
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_ch
                    end
             ELSE : ; No operation
             endcase
           endfor
         end
'RCM-3': begin
          for i=0,3 do begin
             p      =  getwrd( lf.polarization[i] )
             case p of
             ''   : ; No operation
             'HH' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_hh
                    lf.nrcs_bias_array[i]  = df.rcm3_bias_hh
                    end
             'VV' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_vv
                    lf.nrcs_bias_array[i]  = df.rcm3_bias_vv
                    end
             'VH' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_vh
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_vh
                    end
             'HV' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_vh
                    lf.nrcs_bias_array[i]  = df.rcm3_bias_hv
                    end
             'CV' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_cv
                    lf.nrcs_bias_array[i]  = df.rcm2_bias_cv
                    end
             'CH' : begin
                    lf.nrcs_slope_array[i] = df.rcm3_slope_ch
                    lf.nrcs_bias_array[i]  = df.rcm3_bias_ch
                    end
             ELSE : ; No operation
             endcase
           endfor
         end
endcase


;
; Note that lf is an information structure.
; ii, jj, incidence, longitude, latitude are returned so that we can compute the relationship
; between index, incidence, longitude and latitude
;
;
; Compute whether to swith longitude basis. The basis starts from -180 to 180. 
;
lf.lngswitchflag=  0
 
max_longitude=  max(longitude)
min_longitude=  min(longitude)
if max_longitude gt 180 then begin 
 ; The basis in the original data is 0-360
 lf.lngswitchflag=  1
endif else begin
 ;
 ; The basis is -180 to 180 then check if the data cross
 ; the International data line
 ;
 if max_longitude gt 120 and min_longitude lt -120 then begin
   ; The data cross the International Date Line and we must shift
   ; longitude basis
   lf.lngswitchflag=  1
   w=where(longitude lt 0, count)
   if count gt 0 then longitude(w)=360.0+longitude(w)
 endif
endelse
;0
; Add to struture
;
lf.nx1     =   lf.nx0 / df.ave_pixel
lf.ny1     =   lf.ny0 / df.ave_line
temp_lf= create_struct(lf, 'deg',            df.deg         ,  $
                           'ave_pixel',      df.ave_pixel   ,  $
                           'ave_line' ,      df.ave_line    ,  $
                           'answrs_version', answrs_version ,  $
                           'jsstart',        jsstart        )
lf=  0
lf=  temp_lf

;
; Computing fits. Using the same fitting programs as RSAT-2
;
; Longitude and latitude
;
rsat2_lonlat2pix_fit, lf, ii, jj, longitude, latitude, quiet=quiet

;
; Incident angle
rsat2_incid2pix_fit,  lf, ii, jj, incidence, incid_coef, quiet=quiet
;
; Compute polynomial coefficients for radar look direction
; 
rsat2_compute_rlook, lf, quiet=quiet

                           

;
; Determine the short name of the directory with the files
;
name_dir=   getwrd(input_dir, 0, /last, del='/')
                                  
temp_lf= create_struct(lf, 'name_dir',    name_dir)
lf=               0
lf=               temp_lf
                             
;
; Create file name root for output files.
;

case strupcase(lf.month) of
'JAN': month=1
'FEB': month=2
'MAR': month=3
'APR': month=4
'MAY': month=5
'JUN': month=6
'JUL': month=7
'AUG': month=8
'SEP': month=9
'OCT': month=10
'NOV': month=11
'DEC': month=12
ELSE:  month=1
endcase
;
; compute center geographic position
;

ic= lf.nx1 / 2
jc= lf.ny1 / 2

lf.center_lon=  ans_poly_cal( double(ic), double(jc), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
lf.center_lat=  ans_poly_cal( double(ic), double(jc), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)

center_lat= string(abs(lf.center_lat), format='(F05.2)')
if lf.center_lat ge 0 then begin
    center_lat= center_lat + 'N'
endif else begin
    center_lat= center_lat + 'S'
endelse
center_lon= string(abs(lf.center_lon), format='(F06.2)')
if lf.center_lon ge 0 then begin
    center_lon= center_lon + 'E'
endif else begin
    center_lon= center_lon + 'W'
endelse

timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
         lf.month                               + ' ' + $
         string( lf.day,       format='(I2.2)') + ' ' + $
         string( lf.hour,      format='(I2.2)') + ':' + $
         string( lf.minute,    format='(I2.2)') + ':' + $
         string( lf.second,    format='(I2.2)')
js=            dt_tm_tojs(timestr)     
if js ge 0 then begin
  js=            string( dt_tm_tojs(timestr), format='(I10.10)' ) 
endif else begin
  js=   '-' + string( abs(dt_tm_tojs(timestr)), format='(I9.9)' )
endelse
output_prefix= lf.sar_sensor.Replace('RCM-','RCM')
output_prefix= output_prefix + '_' + strupcase( lf.proc_cent)
output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + string( center_lon)
output_prefix= output_prefix + '_' + string( center_lat)
output_prefix= strcompress( output_prefix, /remove_all )


if printflag then print, 'Computed output prefix for SIO files:' + output_prefix

time2= dt_tm_tojs(systime())

if printflag then print, 'Time read product ancillary data file and process='+ string (time2-time1) +' s'
;
; Loop through different images in the file. A set of three files will be generated for each image
;
w = where (lf.imagefiles ne '' ,nfiles)
if n_elements(ave_pixel) eq 0 then ave_pixel=1
if n_elements(ave_line)  eq 0 then ave_line= 1
outputroot= strarr(nfiles)
;
; Create the output directory if it does not exist
;
res= file_search( df.sio_output_dir, /test_directory, count=count) 
if count eq 0 then file_mkdir, df.sio_output_dir
;

for i=0, nfiles-1 do begin
  ;
  ; Decision to process file base of polarizaton
  ;
  polarization= strupcase(lf.polarization[i])
  lf.pol=       polarization
  polflag     = 1B  
  if strupcase( df.sio_cross_pol ) eq '00' then polflag = 0B
  if strupcase( df.sio_cross_pol ) eq '01' then begin
    case strupcase( polarization ) of
      'HV' : polflag = 1B
      'VH' : polflag = 1B
      'HH' : polflag = 0B
      'VV' : polflag = 0B
      ELSE : polflag = 1B
    endcase
  endif
  if strupcase( df.sio_cross_pol ) eq '10' then begin
    case strupcase( polarization ) of
      'HV' : polflag = 0B
      'VH' : polflag = 0B
      'HH' : polflag = 1B
      'VV' : polflag = 1B
      ELSE : polflag = 1B
    endcase
  endif
  remove_noise_flag                                     = 0B
  cpol_flag                                             = 1B
  xpol_flag                                             = 0B
  if strupcase( polarization  ) ne 'HH' and $
    strupcase( polarization  ) ne 'VV' then begin
    cpol_flag                                          = 0B
    xpol_flag                                          = 1B
  endif
  if cpol_flag and $
    strupcase(df.rcm_cpol_remove_noise) eq 'Y' then begin
    remove_noise_flag                                  = 1B
  endif
  if xpol_flag and $
    strupcase(df.rcm_xpol_remove_noise) eq 'Y' then begin
    remove_noise_flag                                  = 1B
  endif
  if strlen(polarization) eq 0 then polflag=0
  if printflag then begin
    print, 'Polarization  : ' + polarization
    print, 'Imagefile     : ' + lf.imagefiles[i]
    print, 'sio_cross_pol : ' + tag_value( df, 'sio_cross_pol')
    print, 'polflag       : ' + string( fix(polflag), format='(I1)' )
  endif  


  if polflag ne 1 then continue; 

  ;
  ; Read and fill in sigma_ne
  ;
  nofile  =  file_search( ff[0],lf.noisefiles[i], count=count)
  if count gt 0 then begin
    rcm_read_noise_xml, nofile, info_noise_level, ii, noise_level
    rcm_sigma_ne_fill, lf, ii, noise_level, sigma_ne
  endif

  lf.nrcs_slope = lf.nrcs_slope_array[i]
  lf.nrcs_bias  = lf.nrcs_bias_array[i]
  ;
  output_prefix2= output_prefix + '_' + strupcase(polarization) + '_C'
  ;
  ; Calibration
  ;
  time3 = dt_tm_tojs(systime(/utc))
  lutfile  =  file_search( ff[0],'lutSigma_'+lf.polarization[i]+'.xml', count=count)
  
  rcm_read_lutsigma_xml, lutfile, info_lutsigma, ii, lutsigma_org, error=error
  if error lt 0 then begin
    error = -1
    if printflag then begin
      print, 'Can not find calibration file: ' + ff[0],'lutSigma_'+lf.polarization[i]+'.xml'
      print, 'Exiting ans_read_rcm.pro'
    endif
    error = -2
   return
  endif 
  rcm_sigma_ne_fill, lf, ii, lutsigma_org, lutsigma, error=error
  ;All RCM georeferenced images are oriented with north nomially up and east nominally on the right 
  ;According to the documentation this means that the Descending passess a "flipped" Left-Right
  ;and Ascending are flipped Top-Bottom
  ;But apparently the LUTS are not flipped.... so flip it here (Note lutsigma_org is 1D in range)  
  if strcmp(lf.pixeltimeordering,'DECREASING',/FOLD_CASE) then lutsigma_org = rotate(lutsigma_org,5)
  ;
  ; Read image file
  ;
  imgfile  =  file_search( ff[0],lf.imagefiles[i], count=count)

  rcm_read_image, imgfile, df.ave_pixel, df.ave_line, lf, img, error=error
  ;
  if printflag then print, 'Return from rcm_read_image
  if error le 0 then begin
    if printflag then begin
      print, 'Cannot read file: ' + imgfile
      print, 'Exiting ans_read_rcm.pro
      error = -1
    endif 
    return
  endif
  sz  = size(img)
  if sz[3] eq 6 or sz[3] eq 9 then lf.datatype ='COMPLEX'
  print, lf.datatype
  if strupcase(lf.datatype) ne 'COMPLEX' then begin
       rcm_dn2sigma, img, info_lutsigma.offset, lutsigma_org, quiet=quiet, error=error 
  endif else begin
       rcm_dn2sigma, img, info_lutsigma.offset, lutsigma_org, /complexdata, quiet=quiet, error=error 
  endelse
  print, 'Return from rcm_dn2sigma'
  ;
  ; Correct nrcs if necessary
  ;
  if lf.nrcs_slope_array[i] ne 1 or lf.nrcs_bias_array[i] ne 0 then begin
    image=  image * lf.nrcs_slope_array[i] + lf.nrcs_bias_array[i]
  endif
  time4 = dt_tm_tojs(systime(/utc))
  if printflag then print, 'Time to read/cal ' + ff[0] + lf.imagefiles[i] +  ' ' + string (time4-time3) +' s'
  logfile= df.log_dir + '/' + output_prefix2 + '.log'
  outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd( lf.name_dir, 0, /last, del='/')+ ' Processing starts'
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  ;
  ; Write out to SIO data file format. This is general for all.
  ; SAR data
  ;
  scaling= 1.0
  outputroot[i]= df.sio_output_dir + '/'+ output_prefix2 
  ;
  ; Check and see if a lock file exits
  ;
  lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '*.lock'
  ;
  res= file_search( lockfile, count=count)
  if count gt 0 then begin
     print, 'Lockfile:  ' + lockfile + ' exists'
     error=-1
     return
  endif
  ;
  ; Create lock file
  ;
  lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '.lock'
  close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1
  ;
  ;
  outputfile=    outputroot[i] + '_image.dat' 
  ;
  ; If data are complex then square and make real
  ;
  sz        =    size(img)
  if sz[3] eq 6 or sz[3] eq 9 then img = abs(img) * abs(img)
  print, 'In ans_read_rcm. If data complex then square
  if not keyword_set(header) then begin
    if df.twobyte eq 'Y' then begin
       ans_write_nrcs_sio, img, outputfile, scaling, /twobyte, quiet=quiet
    endif else begin       
       ans_write_nrcs_sio, img, outputfile, scaling, quiet=quiet
    endelse
  endif
  logfile= df.log_dir + '/' + output_prefix2 + '.log'
  time1= dt_tm_tojs(systime(/utc))
  outstr  = string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + outputfile
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for RCM data
  ;
  rsat2_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
  time1= dt_tm_tojs(systime(/utc))
  outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  close, 1
  openw, 1, logfile, /append
  printf, 1, outstr
  close, 1
  print, outstr
endfor ; Looping through polarizations
;
; Eliminate outputroot file that are empty
w          =  where( strlen(outputroot) gt 0, /null )
outputroot =  outputroot[w]
;

end
