;+---------------------------
; NAME: ANS_READ_SEASAT
; PURPOSE:
;    The purpose of this program is to read Seasat data and
;    create a SIO formatted binary image file, ASCII file, and
;    binary land mask. This works with HDF files.
; CALLING SEQUENCE:   
;    ans_read_seasat, input_file, deffile
; INPUT:
;    input_file  = Seasat ASF HDF data file
;    deffile     = Defaults file
; OUTPUT:          
;    Binary files with image and land mask data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_image.dat (image file)
;      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
;    The file name is delimited by _ and the entries are sensor, proessing facility, year, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header/  
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (1 if program completes or -1 if not)ta
;     outputroot   = root name of outputfiles
;     jsstart      = Processing start ime
; MODIFICATION HISTORY
;   2013 Jul 02 Frank Monaldo. Creation 
;   2013 Aug 14 Frank Monaldo. Remove ice creation file.
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_seasat,  input_file   , deffile,    $
                      quiet       = quiet,      $
                      help        = help,       $
                      outputroot  = outputroot, $
                      jsstart     = jsstart ,   $
                      error       = error
                    
error=     1
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_SEASAT
  print, ' PURPOSE:
  print, '    The purpose of this program is to read Seasat data and
  print, '    create a SIO formatted binary image file, ASCII file, and
  print, '    binary land mask. This works with HDF files.
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_seasat, input_file, deffile
  print, ' INPUT:
  print, '    input_file  = Seasat ASF HDF data file
  print, '    deffile     = Defaults file
  print, ' OUTPUT:          
  print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '    convention for file names is of the form:
  print, '      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '      SEASAT_ASF_1978_09_28_15_20_00_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
  print, '    The file name is delimited by _ and the entries are sensor, proessing facility, year, month, day, hour, minute
  print, '    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '    files include the SIO 20-byte header/  
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  print, '     error        = error (1 if program completes or -1 if not)ta
  print, '     outputroot   = root name of outputfiles
  error= -1
  return
endif

scaling   = 1.0
error     = 1
printflag =1
if keyword_set(quiet) then printflag=0
if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) ) 

;
; Read defaults file


ans_read_defaults, deffile, df, quiet=quiet, error=error

if error lt 0 then begin
  print, 'Could not read defaults file ' + deffile
  print, 'Exiting ans_read_seasat'
  error= -1
  return
endif

time0 = dt_tm_tojs(systime())

if tag_test(df, 'silent')  then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 0

if n_elements(df.gshhs_dir)  lt 1 then begin
  print, 'No gshhs_dir directory specified.'
  print, 'Exiting ans_read_cosmos.pro;
  error= -1
  return
endif

if n_elements(df.idlans_dir) lt 1 then begin
  print, 'No idlans_dir directory specified.'
  print, 'Exiting ans_read_cosmos.pro;
  error= -1
  return
endif
if n_elements(df.log_dir)    lt 1 then begin
  print, 'No log_dir directory specified.'
  print, 'Exiting ans_read_seasat.pro;
  error= -1
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
setenv, 'GSHHS_DIR=' + df.gshhs_dir
;
; Determine if directory exists
; 

if not file_test(input_file) then begin
  print, 'Could not find Seasat: ' + input_file
  print, 'Exiting ans_read_seasat.pro '
  error= -1
  return
endif


;
; Create a new working directory
;
working_directory= df.working_directory + '/seasat_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory
;
; Copy Seast file to working directory.   Note the movement of Envisat data to 
; an already processed directory must occur at a different level. It is not done in this program
;
file_mkdir, working_directory  + '/'
file_copy, input_file, working_directory  + '/', /recursive


res= file_search(df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
if printflag then begin
  print, 'Moving ' + input_file              + '/'   
  print, 'to     ' + df.already_processed    + '/' 
endif 

file_move, input_file,  df.already_processed  + '/' , /overwrite

;
; Keep only the input_file name and strip the path
;
input_file= getwrd(input_file, 0, /last, del='/')

time1 = dt_tm_tojs(systime())

if printflag then print, 'Time to move Seasat file to working directory='+ string (time1-time0) +' s'


;
; Reading HDF file
;
error=1
version_no= float(!version.release)

if version_no lt 8 then begin
  print, 'This is version ' + !version.release
  print, 'This program will only work with versions 8+ because of the'
  print, 'iopen procedure.
  print, 'Exiting ans_read_seasat'
  error= -1
  return
endif




img=       0
info=      !null
outputroot=  ''
;
; Reading Seasat HDF file
;
ans_read_seasat_hdf, working_directory + '/' + input_file, img, longitude, latitude, info, error=error, quiet=quiet
if error lt 0 then begin
  print, 'Cannot read ' + working_directory + '/' + input_file
  print, 'Exiting ans_read_seasat.'
  error=-1
  return
endif

if printflag then hlp, img, longitude, latitude

;
; Setting up to compute fits
;
; 
;

nx0   =  11
ny0   =  11
xskip =  info.nx /10
yskip =  info.ny /10
ii    =  dblarr(nx0, ny0)
jj    =  dblarr(nx0, ny0)
lo    =  dblarr(nx0, ny0)
la    =  dblarr(nx0, ny0)
rk    =  dblarr(nx0, ny0)
for i=0, nx0-1 do begin
  for j=0, ny0-1 do begin
    ii[i,j] = i * xskip
    in      = ii[i,j]
    ip      = in + 1
    if ip gt  info.nx -1 then begin 
      in= info.nx - 2
      ip= info.nx - 1
    endif
    jj[i,j] = j * yskip
    jn      = jj[i,j]
    jp      = jn + 1
    if jp gt  info.ny -1 then begin 
      jn= info.ny - 2
      jp= info.ny - 2
    endif
    lo[i,j] = longitude[ ip, jp ]
    la[i,j] = latitude[  ip, jp ]
    ll2rb,    longitude[ in, jn ], latitude[ in, jn], $
              longitude[ ip, jp ], latitude[ ip, jp], r, b
    rk[i,j] = b
  endfor    
endfor
nx=   info.nx
ny=   info.ny
id   = [ 0D,    0D,   0D, nx/2, nx/2, nx/2, nx/2, nx-1, nx-1 ]
jd   = [ 0D,  ny/2, ny-1,   0D, ny/2, ny-1,   0D, ny/2, ny-1 ]
inc  = [ double(info.incidcorner_ll)    , $
         double(info.incidcorner_ll)    , $
         double(info.incidcorner_ll)    , $
         double(info.incidcenter)       , $
         double(info.incidcenter)       , $
         double(info.incidcenter)       , $
         double(info.incidcorner_ur)    , $
         double(info.incidcorner_ur)    , $
         double(info.incidcorner_ur)    ]
;
; Rearrange to generate a one dimensonal array
;
n    = n_elements(ii)
ii2  = dblarr(n)
jj2  = dblarr(n)
lo2  = dblarr(n)
la2  = dblarr(n)
for i=0, n-1 do begin 
  ii2[i]  =  ii[i]
  jj2[i]  =  jj[i]
  lo2[i]  =  lo[i] 
  la2[i]  =  la[i] 
endfor
;
; Computing fits.
;
ans_compute_fit, ii2, jj2, lo2,  df.deg,   lon_coef,  lon_xexp,     lon_yexp, quiet=quiet
ans_compute_fit, ii2, jj2, la2,  df.deg,   lat_coef,  lat_xexp,     lat_yexp, quiet=quiet
ans_compute_fit, lo2, la2, ii2,  df.deg,     i_coef,    i_xexp,       i_yexp, quiet=quiet
ans_compute_fit, lo2, la2, jj2,  df.deg,     j_coef,    j_xexp,       j_yexp, quiet=quiet
ans_compute_fit, lo2, la2, jj2,  df.deg,     j_coef,    j_xexp,       j_yexp, quiet=quiet
ans_compute_fit, ii2, jj2, la2,  df.deg, rlook_coef, rlook_xexp,  rlook_yexp, quiet=quiet
ans_compute_fit,  id,  jd, inc,  df.deg, incid_coef, incid_xexp,  incid_yexp, quiet=quiet

  
info=create_struct( info                               , $
                    'ave_line'      , df.ave_line      , $
                    'ave_pixel'     , df.ave_pixel     , $
                    'deg'           , df.deg           , $
                    'deg_incid'     , df.deg           , $
                    'deg_rlook'     , df.deg           , $
                    'center_lat'    , 0.0D             , $
                    'center_lon'    , 0.0D             , $
                    'answrs_version', answrs_version   , $
                    'jsstart'       , jsstart          , $
                    'jsbegin'       , -3155673600D     , $
                    'jsend'         , -3155673600D     , $
                    'lon_coef'      , lon_coef         , $
                    'lon_xexp'      , lon_xexp         , $
                    'lon_yexp'      , lon_yexp         , $
                    'lat_coef'      , lat_coef         , $
                    'lat_xexp'      , lat_xexp         , $
                    'lat_yexp'      , lat_yexp         , $
                    'i_coef'        , i_coef           , $
                    'i_xexp'        , i_xexp           , $
                    'i_yexp'        , i_yexp           , $
                    'j_coef'        , j_coef           , $
                    'j_xexp'        , j_xexp           , $
                    'j_yexp'        , j_yexp           , $
                    'rlook_coef'    , rlook_coef       , $
                    'rlook_xexp'    , rlook_xexp       , $
                    'rlook_yexp'    , rlook_yexp       , $
                    'incid_coef'    , incid_coef       , $
                    'incid_xexp'    , incid_xexp       , $
                    'incid_yexp'    , incid_yexp       , $
                    'nrcs_slope'    , df.seasat_slope  , $
                    'nrcs_bias'     , df.seasat_bias   )

 
if printflag then begin
  print, 'ans_read_cosmos'
  print, 'df.silent=' + df.silent
  print, 'quiet=    ' + string(quiet)
endif
  
;
; Compute center incidence/lon/lat
;
  
info.center_lon=     ans_poly_cal( info.nx/2, info.ny/2, info.lon_coef,   info.lon_xexp,   info.lon_yexp)
info.center_lat=     ans_poly_cal( info.nx/2, info.ny/2, info.lat_coef,   info.lat_xexp,   info.lat_yexp)

;
; Add correction to NRCS if set from defaults file.
;

if info.nrcs_slope ne 1 or info.nrcs_bias ne 0 then img=img * info.nrcs_slope + info.nrcs_bias
;
; Create file name root for output files.
;
center_lat= string(abs(info.center_lat), format='(F6.2)')
if float(info.center_lat) ge 0 then begin
  center_lat= center_lat + 'N'
endif else begin
  center_lat= center_lat + 'S'
endelse
center_lon= string(abs(info.center_lon), format='(F6.2)')
if float(info.center_lon) ge 0 then begin
  center_lon= center_lon + 'E'
endif else begin
  center_lon= center_lon + 'W'
endelse
;
; Compute center time
; 
monthstr =   ['' , 'jan', 'feb', 'mar', 'apr', 'may', 'jun', $
            'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
            
date     =  getwrd(info.center_time, 0, del='T')
time     =  getwrd(info.center_time, 1, del='T')
year     =  getwrd(date, 0, del='-')
month    =  getwrd(date, 1, del='-')
day      =  getwrd(date, 2, del='-')
hour     =  getwrd(time, 0, del=':')
minute   =  getwrd(time, 1, del=':')
second   =  getwrd(time, 2, del=':')
              
js       =  dt_tm_tojs(year + ' ' + monthstr[fix(month)] + ' ' + day + ' ' + time)
jss      =  sign(js)
js       =  string( abs(js), format='(I11)')
if jss lt 0 then js = '-' + js
js       =  strcompress(js, /remove_all)

info=create_struct( info,                          $
                   'data_year'      ,  year,       $
                   'month'          ,  month,      $
                   'day'            ,  day,        $
                   'hour'           ,  hour,       $
                   'minute'         ,  minute,     $
                   'second'         ,  second      )
;
;  Compute heading
;
lo1     = ans_poly_cal( info.nx/2.0D,              0D, info.lon_coef, info.lon_xexp, info.lon_yexp )
la1     = ans_poly_cal( info.nx/2.0D,              0D, info.lat_coef, info.lat_xexp, info.lat_yexp )
lo2     = ans_poly_cal( info.nx/2.0D, double(info.ny), info.lon_coef, info.lon_xexp, info.lon_yexp )
la2     = ans_poly_cal( info.nx/2.0D, double(info.ny), info.lat_coef, info.lat_xexp, info.lat_yexp )
ll2rb, lo1, la1, lo2, la2, r, b
heading =  b
info = create_struct( info, 'heading', heading)


;
; Create SIO prefix
;
  
output_prefix= strcompress(info.platform, /remove_all)
output_prefix= output_prefix + '_' + strupcase( info.processing_center )
output_prefix= output_prefix + '_' + year    
output_prefix= output_prefix + '_' + month   
output_prefix= output_prefix + '_' + day    
output_prefix= output_prefix + '_' + hour   
output_prefix= output_prefix + '_' + minute  
output_prefix= output_prefix + '_' + second 
output_prefix= output_prefix + '_' + js
output_prefix= output_prefix + '_' + center_lon
output_prefix= output_prefix + '_' + center_lat
output_prefix= strcompress( output_prefix, /remove_all )   
output_prefix2= output_prefix + '_' + strupcase(info.polarization)+ '_L'
;
; Check and see if a lock file exits
;
lockfile=   df.sio_output_dir + '/'+ output_prefix2 + '*.lock'
;
res= file_search( lockfile, count=count)
if count gt 0 then begin
   print, 'Lockfile:  ' + lockfile + ' exists'
   error=-1
   return
endif
;
; Write out to SIO data file format. This is general for all.
; SAR data
;

info=create_struct( info,  'scaling',  1.000  )     
outputroot= df.sio_output_dir + '/'+ output_prefix2
outputroot= stress( outputroot, 'R', 0, '//', '/', ned)
  
res= file_search(df.log_dir, count=count)
if count eq 0 then file_mkdir, log_dir
logfile= df.log_dir + '/' + output_prefix2 + '.log'
outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd(info.file,0,/last,del='/') + ' Processing starts'
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr

if strupcase(df.twobyte) eq 'Y' then begin
  ans_write_nrcs_sio, img, outputroot + '_image.dat', scaling, /twobyte, quiet=quiet
endif else begin 
  ans_write_nrcs_sio, img, outputroot + '_image.dat', scaling, quiet=quiet
endelse
time0= dt_tm_tojs(systime(/utc))
outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + outputroot + '_image.dat'
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1
print, outstr   
;
; Write out SIO ancillary file format.
;  
ans_write_ancil_sio, outputroot+ '_ancil.txt', info, quiet=quiet
time0= dt_tm_tojs(systime(/utc))
outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + outputroot + '_ancil.txt'
close, 1
openw, 1, logfile, /append
printf, 1, outstr
close, 1  
;
; Write out SIO mask file format.  This is general for all.
;
;
; Compute land mask and write out file. 
;
outputfile=  outputroot+ '_mask.dat'
;
; Compute land mask
; 
ans_write_land_mask, info.lon_coef, info.lon_xexp, info.lon_yexp, $
                          info.lat_coef, info.lat_xexp, info.lat_yexp, $
                          info.nx1, info.ny1,                          $
                          info.range_pixel   * info.ave_pixel,         $
                          info.azimuth_pixel * info.ave_line,          $
                          outputfile,                                  $
                          quiet=        quiet,                         $
                          gshhs_dir=    df.gshhs_dir,                  $
                          pixel_size=   df.pixel_size_mask

time0= dt_tm_tojs(systime(/utc))
outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + outputroot + '_mask.dat'
close,  1
openw,  1, logfile, /append
printf, 1, outstr
close,  1
print, outstr
;


;
; Remove
;

if printflag then print, 'Deleting ' + working_directory
file_delete,  working_directory  + '/', /recursive


end
