;-----------------------------------------------------
;+
; NAME: ANS_READ_SEASAT_HDF
; PURPOSE:
;   The purpose is the read an Seasat HDF5 files and returns
;   and image and the ancillary data as they are stored
;   in theSeasat file.  HDF5 files must have a .h5. This program
;   with only work with IDL 8
; CALLING SEQUENCE:
;   ans_read_seasat_hdf, file, img, longitude, latitude, info
; INPUT:
;    file=     HDF5 file with .h5 extension
; OUTPUT:
;    img       = Raw SAR image (original digital values) faux calibration
;    longitude = Array of longitude values
;    latitude  = Array of lattitude values
;    info      = Ancillary data structure with names from
;                HDF5 file
; KEYWORD
;    /quiet    Print no intermediate output.
;    /help     Get help.
;    error=    If =-1 then cannot read file
;    /diag     Diagonistics turned on
; MODIFICATION HISTORY:
;    2013 Jul 02 Frank Monaldo. Creation
; Copyright (C) 2013, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro  ans_read_seasat_hdf, file, img, longitude, latitude, info,   $
                           help=           help       , $
                           quiet=          quiet      , $
                           error=          error      , $
                           diag=           diag

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_SEASAT_HDF
  print, ' PURPOSE:
  print, '   The purpose is the read an Seasat HDF5 files and returns
  print, '   and image and the ancillary data as they are stored
  print, '   in theSeasat file.  HDF5 files must have a .h5. This program
  print, '   with only work with IDL 8
  print, ' CALLING SEQUENCE:
  print, '   ans_read_seasat_hdf, file, img, longitude, latitude, info
  print, ' INPUT:
  print, '    file=     HDF5 file with .h5 extension
  print, ' OUTPUT:
  print, '    img       = Raw SAR image (original digital values) faux calibration
  print, '    longitude = Array of longitude values
  print, '    latitude  = Array of lattitude values
  print, '    info      = Ancillary data structure with names from
  print, '                HDF5 file
  print, ' KEYWORD
  print, '    /quiet    Print no intermediate output.
  print, '    /help     Get help.
  print, '    error=    If =-1 then cannot read file
  print, '    /diag     Diagonistics turned on
  error= -1
  return
endif

version_no= float(!version.release)

if version_no lt 8 then begin
  print, 'This is version ' + !version.release
  print, 'This program will only work with versions 8+ because of the'
  print, 'iopen procedure.
  print, 'Exiting ans_read_seasat_hdf'
  error= -1
  return
endif

error= 0
;catch, error_status
;;This statement begins the error handler:
;
;if Error_status ne 0 then begin
;   print, 'error index: ', error_status
;   print, 'error message: ', !error_state.msg
;   if error_status eq -249  or $
;      error_status eq -252  or $
;      error_status eq -261  or $
;      error_status eq -262  then begin
;      error= -1
;      return
;   endif
;   catch, /cancel
;endif

printflag= 1
!quiet=    1
if keyword_set(quiet)    then printflag=     0



if not file_test(file) then begin
  print, ''
  print, '************************************************************
  print, 'Cannot find file: ' + file
  print, 'Exiting ans_read_seasat_hdf'
  print, '============================================================
  print, ''
  error= -1
  return
endif
;
;  Reading HDF file into strucutre
;
if printflag then print, 'Begin reading: ' + file + ' ' + systime(/utc)

iopen, file, x

if printflag then print, 'End   reading: ' + file + ' ' + systime(/utc)
;
; Common informaional values in single and multi-image files
;
c                 =  double(299792458.0)  ; Speed of light in m/s
filename          =  getwrd(tag_value( x,'_name'  ), 0, /last, del='/')
institution       =  x.institution._data
processing_center =  'ASF'
if  strupcase(institution) ne 'ALASKA SATELLITE FACILITY' then processing_center='UNKNOWN'
asc_des           =  'U'
asc_des           =  x.metadata.productinfo.missioninfo.orbitdirection._data
if asc_des eq 'ASCENDING'  then asc_des='A'
if asc_des eq 'DESCENDING' then asc_des='D'
pol               =  x.metadata.productinfo.acquisitioninfo.polarizationlist.pollayer._data
polarization      =  pol[0]
pol               =  pol[0]
software_version  =  x.metadata.generalheader.generationsystem._data
lookdirection     =  x.metadata.productinfo.acquisitioninfo.lookdirection._data
beammode          =  x.metadata.productinfo.acquisitioninfo.imagingmode._data
sz                =  tag_value(x.data.hh, '_dimensions')
nx                =  sz[0]
ny                =  sz[1]
units             =  x.data.hh.units_description._data
platform          =  x.metadata.productinfo.missioninfo.mission._data
time_epoch        =  x.data.time.units._data
time_js           =  x.data.time._data
center_time       =  x.metadata.platform.orbit.statevec.timeutc._data
start_time        =  x.metadata.productinfo.sceneinfo.start.timeutc._data
end_time          =  x.metadata.productinfo.sceneinfo.start.timeutc._data
sceneid           =  x.metadata.productinfo.sceneinfo.sceneid._data
earthradius       =  x.metadata.productinfo.sceneinfo.earthradius._data
satelliteheight   =  x.metadata.productinfo.sceneinfo.satelliteheight._data
absorbit          =  x.metadata.productinfo.missioninfo.absorbit._data
frequency         =  x.metadata.instrument.radarparameters.centerfrequency._data
azimuth_looks     =  x.metadata.productinfo.IMAGEDATAINFO.IMAGERASTER.azimuthlooks._data
range_looks       =  x.metadata.processing.processingparameter.rangelooks._data
line_size_m       =  x.metadata.productinfo.IMAGEDATAINFO.IMAGERASTER.azimuthresolution._data
pixel_size_m      =  x.metadata.productinfo.IMAGEDATAINFO.IMAGERASTER.groundrangeresolution._data
near_range_s      =  x.metadata.productinfo.sceneinfo.rangetime.firstpixel._data ; s
near_range_m      =  c * near_range_s
far_range_s       =  x.metadata.productinfo.sceneinfo.rangetime.lastpixel._data ; s
far_range_m       =  c * far_range_s
far_range_m       =  far_range_m
cen_range_s       =  x.metadata.productinfo.sceneinfo.SCENECENTERCOORD.rangetime._data
cen_range_m       =  c * cen_range_s
incidcenter       =  x.metadata.productinfo.sceneinfo.scenecentercoord.incidenceangle._data
incidcorner       =  x.metadata.productinfo.sceneinfo.scenecornercoord.incidenceangle._data
incidence_ll      =  incidcorner[0]
incidence_lr      =  incidcorner[1]
incidence_ul      =  incidcorner[2]
incidence_ur      =  incidcorner[3]
xpos              =  x.metadata.platform.orbit.statevec.posx._data
ypos              =  x.metadata.platform.orbit.statevec.posy._data
zpos              =  x.metadata.platform.orbit.statevec.posz._data
xvel              =  x.metadata.platform.orbit.statevec.velx._data
yvel              =  x.metadata.platform.orbit.statevec.vely._data
zvel              =  x.metadata.platform.orbit.statevec.velx._data
ellipsoidname     =  x.metadata.platform.orbit.orbitheader.statevectorrefframe._data
sensor_velocity   =  sqrt( xvel[0]^2 + yvel[0]^2 + zvel[0]^2 )
r                 =  (near_range_m + far_range_m) /2.0         
wavelength        =  c / frequency
integration       =  wavelength * r /2.0  / sensor_velocity / float(line_size_m)
range_swath       =  pixel_size_m * float(nx)
azimuth_swath     =  line_size_m  * float(ny)
semiMajorAxis     =  6378137.0
semiMinorAxis     =  6356752.3142
ellipsoidscale    =  1


img       =  x.data.hh._data
latitude  =  x.data.latitude._data
longitude =  x.data.longitude._data

;
; Check for longitude basis
;
; Make sure that by default the longitude fall between +/- 180

w=              where( longitude gt 360, count)
lngswitchflag = 0
if count gt 0 then longitude[w]  = longitude[w] - 360.00

;
; If data cross the internationl date line then
; switch to 0-360 longitude basis
;
if max(longitude) gt 150 and min(longitude) lt -150 then begin
  w=              where( longitude lt 0, count)
  if count gt 0 then begin
    longitude[w]  = longitude[w] + 360.00
    lngswitchflag = 1
  endif
endif  

info={file                         : filename           , $
      name_dir                     : filename           , $
      institution                  : institution        , $
      processing_center            : processing_center  , $
      proc_fac                     : processing_center  , $
      software_version             : software_version   , $
      datatake_id                  : sceneid            , $
      asc_des                      : asc_des            , $
      polarization                 : polarization       , $
      pol                          : pol                , $
      frequency                    : frequency          , $
      radar_frequency              : frequency          , $
      frequency_band               : 'L'                , $
      nx                           : nx                 , $
      ny                           : ny                 , $
      lookdirection                : lookdirection      , $
      beammode                     : beammode           , $
      beam_mode                    : beammode           , $
      nx0                          : nx                 , $
      ny0                          : ny                 , $
      nx1                          : nx                 , $
      ny1                          : ny                 , $
      units                        : units              , $
      platform                     : platform           , $
      time_epoch                   : time_epoch         , $
      time_js                      : time_js            , $
      center_time                  : center_time        , $
      start_time                   : start_time         , $
      end_time                     : end_time           , $
      heading_rel                  : 90                 , $
      sceneid                      : sceneid            , $
      lngswitchflag                : lngswitchflag      , $
      earthradius                  : earthradius        , $
      satelliteheight              : satelliteheight    , $
      absorbit                     : absorbit           , $
      azimuth_looks                : azimuth_looks      , $
      range_looks                  : range_looks        , $
      line_size_m                  : line_size_m        , $
      pixel_size_m                 : pixel_size_m       , $
      azimuth_pixel                : line_size_m        , $
      range_pixel                  : pixel_size_m       , $
      near_range_m                 : near_range_m       , $
      far_range_m                  : far_range_m        , $
      incidcenter                  : incidcenter        , $
      incidcorner_ll               : incidence_ll       , $
      incidcorner_lr               : incidence_lr       , $
      incidcorner_ul               : incidence_ul       , $
      incidcorner_ur               : incidence_ur       , $
      datumshiftparameters         : '0 0 0'            , $
      sar_sensor                   : 'SEASAT'           , $
      productType                  : 'SEASAT STD'       , $
      platform_code                : 'SE'               , $
      orbit_number                 : absorbit           , $
      scansar                      : 0                  , $
      valuetype                    : 'intensity'        , $
      sensor_velocity              : sensor_velocity    , $
      integration_time             : integration        , $
      nom_az_res                   : 25.0               , $
      nom_rg_res                   : 25.0               , $
      rcvr_gain                    : 0.0                , $
      proc_gain                    : 0.0                , $
      range_type                   : 'MAP'              , $
      range_flag                   : 1                  , $
      headling_rel                 : 90                 , $
      flip                         : 0                  , $
      illumination_dir             : 0                  , $
      range_swath                  : range_swath        , $
      azimuth_swath                : azimuth_swath      , $
      s_c_altitude                 : satelliteheight    , $
      center_incid                 : incidcenter        , $
      slant_range_center           : cen_range_m        , $
      slant_range_near             : near_range_m       , $
      semiMajorAxis                : semiMajorAxis      , $
      semiMinorAxis                : semiMinorAxis      , $
      ellipsoidname                : ellipsoidname      , $
      ellipsoidscale               : ellipsoidscale     }

if printflag then begin
  print, 'End of ans_read_seasat_hdf program....'
  hlp, img, longitude, latitude
endif
end
