;+---------------------------
; NAME: ANS_READ_SEN
; PURPOSE:
;    The purpose of this program is to read Sentinel data and
;    create a SIO formatted binary image file, ASCII file, and
;    binary land mask
; CALLING SEQUENCE:
;    ans_read_sen, input_dir, deffile
; INPUT:
;    input_dir  =  Sentinel data come in directories. This input is the name
;                  of such a directory. A directory might look like
;                  S1A_WV_SLC__1SSV_20120101T040741_20120101T040828_001771_000001_55D4.SAFE/
;    deffile   =   Defaults file
; OUTPUT:
;    Binary files with image and land mask data and an ASCII file with ancillary data. The
;    convention for file names is of the form:
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_image.dat (image file)
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_ancil.txt (ancillary data file)
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_mask.dat  (land mask)
;    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization[swathnumber], frequency band,
;    file type and extension. The data
;    files include the SIO 20-byte header.  The
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (0 if program completes or -1 if not)
;     outputroot   = Root name of SIO files
;     jsstart      = Processing start time
; MODIFICATION HISTORY
;  2013 Sep 30 Frank Monaldo. Created for ANSWRS 2.0
;  2014 May 15 Frank Monakdo. Added quiet sen_img2sigma call
;  2014 Sep 26 Frank Monaldo. Respect sio_cross_pol
;  2014 Sep 29 Frank Monaldo. Corrected a bug if outputroot variable does
;                             not exist.
;  2014 Sep 30 Frank Monaldo. Removed land mask generation.  This is now ans_sat2sio
;  2015 Oct 08 Chris Jackson. Add block of code to create lockfile before sio write
;  2016 Jan 05 Chris Jackson. Modify the format statements for center lon/lat in the standard
;                             name to include leading zeros (F6.2 to F06.2)
;  2016 Jan 29 Chris Jackson. Modified the check to add data set number to filename (after polarization).
;                             Now tests for SLC string in imgfile
;  2016 Apr 25 Frank Monaldo. Changed sen_img2sigma to sen_img2sigma_noise which subtracts
;                             off the noise floor.
;  2016 Jun 09 Frank Monaldo. Reads defaults file to determine whether or not to subtract noise
;  2016 Jun 10 Frank Monaldo. Corrected spelling remove_noise_flag
;  2016 Nov 02 Frank Monaldo. Extend usefullness sio_cross_pol functionality
;  2019 Apr 29 Chris Jackson. Update calls to sen_img2sigma_noise passing in the 0th array element
;  2020 Apr 02 Chris Jackson. Eliminate the copy of the S1 directory to the working_directory.
;                             Remove the move to already processed (now in ans_sen2sio)
;                             and include a delete of the sio lock file in this routine.
;  2020 Jun 20 Frank Monaldo. Add minus sign if noise removed
; Copyright (C) 2013-2016, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;

pro ans_read_sen,  input_dir,    deffile, $
  quiet       = quiet           , $
  help        = help            , $
  outputroot  = outputroot      , $
  jsstart     = jsstart         , $
  error       = error

  error=     0
  if keyword_set(help) or (n_params(0) lt 1) then begin
    print, ' NAME: ANS_READ_SEN
    print, ' PURPOSE:
    print, '    The purpose of this program is to read Sentinel data and
    print, '    create a SIO formatted binary image file, ASCII file, and
    print, '    binary land mask
    print, ' CALLING SEQUENCE:
    print, '    ans_read_sen, input_dir, deffile
    print, ' INPUT:
    print, '    input_dir  =  Sentinel data come in directories. This input is the name
    print, '                  of such a directory. A directory might look like
    print, '                  S1A_WV_SLC__1SSV_20120101T040741_20120101T040828_001771_000001_55D4.SAFE/
    print, '    deffile   =   Defaults file
    print, ' OUTPUT:
    print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The
    print, '    convention for file names is of the form:
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_image.dat (image file)
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_ancil.txt (ancillary data file)
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_mask.dat  (land mask)
    print, '    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
    print, '    second, julian seconds, center longitude, center latitude, polarization[swathnumber], frequency band,
    print, '    file type and extension. The data
    print, '    files include the SIO 20-byte header.  The
    print, ' KEYWORDS:
    print, '     /quiet       = Do not print intermediate results.
    print, '     /help        = Print out help
    print, '     error        = error (0 if program completes or -1 if not)
    print, '     outputroot   = Root name of SIO files
    print, '     jsstart      = Processing start time
    error= -1
    return
  endif

  !quiet=1
  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  error=1
  time0 = dt_tm_tojs(systime(/utc))
  ;
  ; Read defaults file
  ;
  ans_read_defaults, deffile, df, quiet=quiet, error=error
  if error lt 0 then return

  if n_elements(df.idlans_dir) lt 1 then begin
    print, 'idlans_dir not specified.'
    print, 'Exiting ans_read_sen.pro'
    error= -1
    return
  endif
  if n_elements(df.log_dir)    lt 1 then begin
    print, 'log_dir not specified.'
    print, 'Exiting ans_read_sen.pro'
    error= -1
    return
  endif
  ;
  ; Determine if input directory exists
  ;

  ff= file_search( input_dir, /test_directory, count=count)

  if count eq 0 then begin
    print, 'Could not find Sentinel directory: ' + input_dir
    print, 'Exiting ans_read_sen.pro '
    error= -1
    return
  endif

  print, 'Using Directory ' + ff[0]

  ; Find the s*.xml file and the base_directory for processing, ff= full path to product.xml file
  ;
  ; Locating leader file
  ;
  xfiles=   file_search( ff[0], 's*xml', count=data_setmax)
  if data_setmax eq 0 then begin
    print, 'Cannot file xml data in: ' + ff[0]
    print, 'Exiting ans_read_sen.pro'
    error = -1
    return
  endif
  ;
  ; Loop through data sets.
  ;

  for data_set= 0, data_setmax-1 do begin
    base_directory = ff[0]
    ;
    ; Fill information into lf structure, Read mapping grid information
    ; and read geo reference dagta
    ;
    root_file = getwrd( input_dir, 0, /last, del='/')
    if n_elements(jsstart) eq 0 then jsstart=dt_tm_tojs(systime(/utc))
    corename  = getwrd( xfiles[data_set], 0, /last, del='/' )
    len       = strlen( corename )
    corename  = strmid( corename, 0, len-4 )

    sen_fillin_lf,  xfiles[data_set], deffile,  root_file, jsstart, lf,  quiet=quiet
    if data_set eq 0 then begin
      ;
      ; Create file name root for output files.
      ;
      ; compute center geographic position
      ;
      center_lat= string(abs(lf.center_lat), format='(F05.2)')
      if lf.center_lat ge 0 then begin
        center_lat= center_lat + 'N'
      endif else begin
        center_lat= center_lat + 'S'
      endelse
      center_lon= string(abs(lf.center_lon), format='(F06.2)')
      if lf.center_lon ge 0 then begin
        center_lon= center_lon + 'E'
      endif else begin
        center_lon= center_lon + 'W'
      endelse
      js            = lf.acq_date_time_js
      output_prefix = lf.platform_name
      output_prefix = output_prefix + '_' + strupcase( lf.processing_facility)
      output_prefix = output_prefix + '_' + dt_tm_fromjs( js, format='Y$_0n$_0d$_h$_m$_s$')
      if js ge 0 then begin
        output_prefix = output_prefix + '_'  + string(      js, format='(I10.10)' )
      endif else begin
        output_prefix = output_prefix + '_-' + string( abs(js), format='(I10.10)' )
      endelse
      output_prefix = output_prefix + '_' + string( center_lon) + '_' + string( center_lat)
      output_prefix = strcompress( output_prefix, /remove_all )
      if printflag then print, 'Computed output prefix for SIO files:' + output_prefix
      time2= dt_tm_tojs(systime(/utc))
    endif
    ;
    ; Set polflag to decide whether to create SIO files
    ; of cross-pol data
    ;
    ;
    ; Set polflag to decide whether to create SIO files
    ; depending on polarization
    ;
    polflag                                               = 1B
    if strupcase( df.sio_cross_pol ) eq '00' then polflag = 0B
    if strupcase( df.sio_cross_pol ) eq '01' then begin
      case strupcase( lf.polarization ) of
        'HV' : polflag = 1B
        'VH' : polflag = 1B
        'HH' : polflag = 0B
        'VV' : polflag = 0B
        ELSE : polflag = 1B
      endcase
    endif
    if strupcase( df.sio_cross_pol ) eq '10' then begin
      case strupcase( lf.polarization ) of
        'HV' : polflag = 0B
        'VH' : polflag = 0B
        'HH' : polflag = 1B
        'VV' : polflag = 1B
        ELSE : polflag = 1B
      endcase
    endif
    remove_noise_flag                                     = 0B
    cpol_flag                                             = 1B
    xpol_flag                                             = 0B
    if strupcase( lf.polarization  ) ne 'HH' and $
      strupcase( lf.polarization  ) ne 'VV' then begin
      cpol_flag                                          = 0B
      xpol_flag                                          = 1B
    endif
    if cpol_flag and $
      strupcase(df.sen_cpol_remove_noise) eq 'Y' then begin
      remove_noise_flag                                  = 1B
    endif
    if xpol_flag and $
      strupcase(df.sen_xpol_remove_noise) eq 'Y' then begin
      remove_noise_flag                                  = 1B
    endif
    if polflag then begin
      ;
      ; Read and calibrate image file
      ;
      imgfile       = file_search( base_directory, corename + '.tiff', count= count)
      if count eq 0 then begin
        print, 'Cannot find image file ' + corename + '.tiff'
        error = -1
        return
      endif
      calfile         = file_search( base_directory, 'calibration*' + corename + '*', count= count)
      if count eq 0 then begin
        print, 'Cannot find calibration file ' + 'calibration*' + corename + '*'
        error = -1
        return
      endif
      noisefile='BLANK'
      if remove_noise_flag then begin
        noisefile = file_search( base_directory, 'noise*'       + corename + '*', count= count)
        if count eq 0 then begin
          print, 'Cannot find noise file ' + 'noise*' + corename + '*'
          error = -1
          return
        endif
      endif
      if remove_noise_flag then begin
        ;
        ; Remove noise floor
        ;
        sen_img2sigma_noise, imgfile, calfile[0], noisefile[0], img, $
          quiet       =  quiet                  , $
          dn2_mean    =  dn2_mean               ,  $
          noise_mean  =  noise_mean             ,  $
          final_mean  =  final_mean
      endif else begin
        ;
        ; Keep noise floor
        ;
        sen_img2sigma_noise, imgfile, calfile[0], noisefile[0], img, $
          quiet       =  quiet                  , $
          /keepnoise                            , $
          dn2_mean    =  dn2_mean               ,  $
          noise_mean  =  noise_mean             ,  $
          final_mean  =  final_mean
      endelse
      if n_elements(dn2_mean) gt 0 then begin
        sdn2_mean  = string(dn2_mean, format = '(E15.9)' )
        if tag_value(lf, 'dn2_mean') ne '' then begin
          lf.dn2_mean =sdn2_mean
        endif else begin
          lf          = create_struct( lf, 'dn2_mean' ,  sdn2_mean )
        endelse
      endif
      if n_elements(noise_mean) gt 0 then begin
        snoise_mean  = string(noise_mean, format = '(E15.9)' )
        if tag_value(lf, 'noise_mean') ne '' then begin
          lf.noise_mean =snoise_mean
        endif else begin
          lf          = create_struct( lf, 'noise_mean' ,  snoise_mean )
        endelse
      endif
      if n_elements(final_mean) gt 0 then begin
        sfinal_mean  = string(final_mean, format = '(E15.9)' )
        if tag_value(lf, 'final_mean') ne '' then begin
          lf.final_mean =sfinal_mean
        endif else begin
          lf          = create_struct( lf, 'final_mean' ,  sfinal_mean )
        endelse
      endif
      ;
      ; Adjust NRCS if necessary
      ;
      if lf.nrcs_slope ne 1 or lf.nrcs_bias ne 0 then begin
        img=  img * lf.nrcs_slope + lf.nrcs_bias
      endif
      output_prefix2= output_prefix + '_' + strupcase(lf.polarization)
      if (strmatch(imgfile,'*SLC*',/FOLD_CASE) eq 1) then begin
        len=    strlen(corename)
        output_prefix2 = output_prefix2  + strmid(corename, len-3, 3 )
      endif

      output_prefix2= output_prefix2 + '_' + strupcase(lf.frequency_band)
      if remove_noise_flag then output_prefix2 = output_prefix2 + '-'
      if n_elements(outputroot) eq 0 then begin
        outputroot= df.sio_output_dir + '/'+ output_prefix2
      endif else begin
        outputroot= [outputroot, df.sio_output_dir + '/'+ output_prefix2]
      endelse
      outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd(root_file, 0, /last, del='/')  + ' Processing starts'
      ;
      ; Check for existence of log_dir
      ;
      res= file_search(df.log_dir, count=count)
      if count gt 0 then begin
        logfile= df.log_dir + '/' + output_prefix2 + '.log'
        close, 1  & openw, 1, logfile & close, 1
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
      endif
      print, outstr
      ;
      ; Check and see if a lock file exists
      ;
      lockfile=    df.sio_output_dir + '/'+ output_prefix2 + '.lock'
      ;
      res= file_search( lockfile, count=count)
      if count gt 0 then begin
        print, 'Lockfile:  ' + lockfile + ' exists'
        error=-1
        ;Check if the proper sio files are created.  This can happen because of the GRDH / GRDM pairing issue
        ;WHen both files are being processed near the same time
        if file_test(df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', /regular) $
          and file_test(df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt',/regular) then begin
          error = 0
          print,'Image and Ancil files already exist: ' + output_prefix2
        endif
        return
      endif

      ; Create lock file
      ;
      lockfile=   df.sio_output_dir + '/'+ output_prefix2 +  '.lock'
      close, 1 & openw, 1, lockfile & printf, 1, '' & close, 1

      ;
      ; Write out SIO data file
      ;
      if strupcase(df.twobyte) eq 'Y' then begin
        ans_write_nrcs_sio, img, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', lf.scaling, /twobyte, quiet=quiet
      endif else begin
        ans_write_nrcs_sio, img, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', lf.scaling, quiet=quiet
      endelse
      time4= dt_tm_tojs(systime(/utc))
      outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
      outstr  = stress( outstr, 'R', 0, '//', '/', ned)
      res= file_search(df.log_dir, count=count)
      if count gt 0 then begin
        logfile= df.log_dir + '/' + output_prefix2 + '.log'
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        print, ' '
        print, ' '
        close, 1
        print, outstr
      endif
      ;
      ; Write out SIO ancillary file format. This program is explicitly written for Sentinel data
      ;
      sen_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, quiet=quiet
      time2= dt_tm_tojs(systime(/utc))
      outstr  = string(time2, format='(I10)') + ' '+ dt_tm_fromjs(time2) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
      outstr  = stress( outstr, 'R', 0, '//', '/', ned)
      res= file_search(df.log_dir, count=count)
      if count gt 0 then begin
        logfile= df.log_dir + '/' + output_prefix2 + '.log'
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
      endif
      print, outstr

      ;Delete the lock file for the SIO files
      file_delete, lockfile, /allow_nonexistent, /quiet

    endif   ; End if polflag eq 1



  endfor




end
