;----------------------------------------------------------
;+
; NAME: ANS_READ_SIO_INFO'
; PURPOSE:'
;   This program reads ancillary information in SIO format. The 
;   input file is and ASCII file using a % character as a
;   delimiter
; CALLING SEQUENCE:'
;   ans_read_sio_info, filename, lf'
; INPUT:'
;   filename=    A string containing the SIO ancillary data
;                file to read'
; OUTPUT:'
;   lf=          Structure with information
; KEYWORDS:
;    /quiet=      Do not print intermediate results'
;    /help=       List help.
;    error=       <0 bad
; MODIFICATION
;  2009 Nov 01 Frank Monaldo
;  2009 Dec 03 Frank Monaldo. Changed the loop index to a long
;  2010 Jan 28 Frank Monaldo. Added ability to read ANSWRS version
;  2010 Feb 01 Frank Monaldo. Added ability to give help w/no parameters
;  2010 Feb 04 Frank Monaldo. Added start_processing time
;  2011 Mar 29 Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2011 May 23 Frank Monaldo. Add jsbegin and jsend for acquisition
;  2013 Aug 13 Frank Monaldo. Add ability to read ice mask information data.
;  2013 Sep 09 Frank Monaldo. Add error flag
;  2015 Dec 26 Chris Jackson. Fixed nc variable in case deg = 3 (mis-called as n)
;  2016 Jun 10 Frank Monaldo. Added sen_cpol_remove_noise and sen_xpol_remove_noise
;  2020 Sep 25 Chris Jackson. Change sen_cpol_remove_noise/sen_xpol_remove_noise attributes to  generic cpol_remove_noise/xpol_remove_noise
; Copyright (C) 2009-2016, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_read_sio_info, filename, lf, quiet=quiet, help=help, error=error


if keyword_set(help)  or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_READ_SIO_INFO'
  print, ' PURPOSE:'
  print, '   This program reads ancillary information in SIO format. The 
  print, '   input file is and ASCII file using a % character as a
  print, '   delimiter
  print, ' CALLING SEQUENCE:'
  print, '   ans_read_sio_info, filename, lf'
  print, ' INPUT:'
  print, '   filename=    A string containing the SIO ancillary data
  print, '                file to read'
  print, ' OUTPUT:'
  print, '   lf=          Structure with information
  print, ' KEYWORDS:
  print, '    /quiet=      Do not print intermediate results'
  print, '    /help=       List help.'
  print, '    error=       <0 bad
  error = -1
  return
endif

printflag=1
if keyword_set(quiet) then printflag=0
error = 0

res=file_search(filename, count= count)
if count eq 0 then begin
 print, 'Cannot find file: ',  filename
 print, 'Exiting program ans_read_sio_info...'
 error = -1
 return
endif

if printflag then print, 'Input filename: ' + filename + '.'

;
; Create the lf structure
;
lf={root_file:                    '', $
    answrs_version:               '', $
    acquisition_date:             '', $  
    acquisition_time:             '', $
    acq_date_time_js:             '', $
    acq_date_time:                '', $
    jsbegin:                      '', $
    jsend:                        '', $
    processing_start:             '', $
    sio_create_date_time:         '', $
    product_type:                 '', $
    processing_agency:            '', $
    processing_facility:          '', $
    sar_processor_version:        '', $
    scene_identifier:             '', $
    scene_reference_number:       '', $
    orbit_number:                 '', $
    platform_name:                '', $
    sar_platform_code:            '', $
    sensor_imaging_mode:          '', $
    calibration_flag:             '', $
    image_pixel_value_units:      '', $
    polarization:                 '', $
    frequency_band:               '', $
    flag_sar_band_polarization:   '', $
    orbit_pass_flag:              '', $
    scaling:                      '', $
    sar_beam_code:                '', $
    sensor_wavelength_m:          '', $
    sensor_frequency_hz:          '', $
    sensor_vel_rel_ground_mps:    '', $
    integration_time_s:           '', $
    nominal_azimuth_resolution_m: '', $
    nominal_range_resolution_m:   '', $
    receiver_gain_db:             '', $
    relative_processor_gain_db:   '', $
    map_descriptor:               '', $
    pixel_spacing_m:              '', $
    line_spacing_m:               '', $
    range_type:                   '', $
    heading_north_deg:            '', $
    heading_image_deg:            '', $
    flip_flag:                    '', $
    illumination_dir_deg:         '', $
    early_azimuth_edge:           '', $
    near_range_edge:              '', $
    range_swath_width_m:          '', $
    azimuth_swath_width_m:        '', $
    sensor_altitude_m:            '', $
    incid_angle_center_deg:       '', $
    slant_range_center_m:         '', $
    slant_range_near_m:           '', $
    lines_original_image:         '', $
    pixels_original_image:        '', $
    lines_sio_image:              '', $
    pixels_sio_image:             '', $
    ref_corner_line:              '', $
    ref_corner_pixel:             '', $
    ref_line_incr:                '', $
    ref_pixel_incr:               '', $
    looks_range:                  '', $
    looks_azimuth:                '', $
    ref_ellipsoid:                '', $
    ref_semi_major_axis_m:        '', $
    ref_semi_minor_axis_m:        '', $
    datum_shifts_m:               '', $
    datum_scale:                  '', $
    ul_outer_lat:                 '', $
    ul_outer_lon:                 '', $
    ur_outer_lat:                 '', $
    ur_outer_lon:                 '', $
    ll_outer_lat:                 '', $
    ll_outer_lon:                 '', $
    lr_outer_lat:                 '', $
    lr_outer_lon:                 '', $
    ul_inner_lat:                 '', $
    ul_inner_lon:                 '', $
    ur_inner_lat:                 '', $
    ur_inner_lon:                 '', $
    ll_inner_lat:                 '', $
    ll_inner_lon:                 '', $
    lr_inner_lat:                 '', $
    lr_inner_lon:                 '', $
    source_lonlat_fit:            '', $
    basis_lon_fit:                '', $
    deg_lonlat_fit:               '', $
    source_incid_fit:             '', $
    deg_incid_fit:                '', $
    source_rlook_fit:             '', $
    deg_rlook_fit:                '', $
    source_sigma_ne_fit:          '', $
    deg_sigma_ne_fit:             '', $
    nrcs_slope:                   '', $
    nrcs_bias:                    '', $
    icefile:                      '', $
    icefile_time_js:              '', $
    icefile_time:                 '', $
    sen_cpol_remove_noise:        '', $  ;Keeping this for backward compatability
    sen_xpol_remove_noise:        '', $   
    cpol_remove_noise:            '', $
    xpol_remove_noise:            ''  }

;
; Add coefficients to the struture onCe we know
; hoW big they are.
;

s=   getfile(filename)
n=   n_elements(s)

for i=0L, n-1 do begin
  res=strpos(s[i], '%')
  if res gt -1 then begin  ; there must a % sign or else we skip
    first=   getwrd( s[i], 0, del='%' )
    second=  getwrd( s[i], 1, del='%' )
    if strlen(first) ne 0 and strlen(second) ne 0 then begin  ;  The first have info
      first=   strcompress(first)
      second=  strlowcase( strcompress(second, /remove_all) )
      lsecond= strlen(second)
      if lsecond gt 30 then second= strmid( second, 0, 30 )
      ;if printflag then print, string(i, format='(I4.4)') + ' ' +second
      case second of                                                     
      'originalrootfileordirectorynam':  lf.root_file=                    first
      'answrsversion':                   lf.answrs_version=               first
      'acquisitiondate':                 lf.acquisition_date=             first
      'acquisitiontime(gmt)':            lf.acquisition_time=             first
      'juliansecondsepoch=2000jan100:':  lf.acq_date_time_js=             first 
      'acquisitiondateandtimeformat2':   lf.acq_date_time=                first
      'acquisitionstarttimeinjulianse':  lf.jsbegin=                      first
      'acquisitionendtimeinjulianseco':  lf.jsend=                        first
      'processingstarttime':             lf.processing_start=             first
      'siocreationdateandtime':          lf.sio_create_date_time=         first
      'producttype':                     lf.product_type=                 first
      'processingagency':                lf.processing_agency=            first
      'facility':                        lf.processing_facility=          first
      'sarprocessorversion':             lf.sar_processor_version=        first
      'sceneidentifier':                 lf.scene_identifier=             first
      'scenereferencenumber(orbit-fra':  lf.scene_reference_number=       first
      'orbit(revolution)number':         lf.orbit_number=                 first
      'platformname':                    lf.platform_name=                first
      'sarplatformcode':                 lf.sar_platform_code=            first
      'sensorimagingmode(originalpixe':  lf.sensor_imaging_mode=          first
      'calibrationflag(-1=antennacorr':  lf.calibration_flag=             first 
      'imagepixelvalueunits(note:ampl':  lf.image_pixel_value_units=      first
      'polarization':                    lf.polarization=                 first
      'frequencyband':                   lf.frequency_band=               first
      'flagforsarbandandpolarizationm':  lf.flag_sar_band_polarization=   first                                              
      'orbitalpassflag(a=ascending/d=':  lf.orbit_pass_flag=              first
      'scaling':                         lf.scaling=                      first
      'sarbeammodecode':                 lf.sar_beam_code=                first
      'sensorwavelength(m)':             lf.sensor_wavelength_m=          first
      'sensorfrequency(hz)':             lf.sensor_frequency_hz=          first
      'sensorvelocityrelativetoimageg':  lf.sensor_vel_rel_ground_mps=    first
      'imagedwellorintegrationtime(se':  lf.integration_time_s=           first
      'nominalazimuthresolution(m)':     lf.nominal_azimuth_resolution_m= first
      'nominalrangeresolution(m)':       lf.nominal_range_resolution_m=   first
      'receivergain(indb)':              lf.receiver_gain_db=             first
      'relativeprocessorgain(indb)':     lf.relative_processor_gain_db=   first
      'mapdescriptor':                   lf.map_descriptor=               first
      'element/rangespacing(m)':         lf.pixel_spacing_m=              first
      'line/azimuthspacing(m)':          lf.line_spacing_m=               first
      'rangetypeindicatorflag(0=slant':  lf.range_type=                   first
      'headingangleindegreesrelativet':  lf.heading_north_deg=            first
      'headinganglerelativetooutputim':  lf.heading_image_deg=            first
      'flipflag(0=notflipped|1=flippe':  lf.flip_flag=                    first
      'actualilluminationdirectionang':  lf.illumination_dir_deg=         first      
      'earlyazimuthedge{l|r|t|b}':       lf.early_azimuth_edge=           first
      'nearrangeedge{l|r|t|b}':          lf.near_range_edge=              first
      'rangeswathwidth(m)':              lf.range_swath_width_m=          first
      'azimuthswathwidth(m)':            lf.azimuth_swath_width_m=        first
      'sensoraltitude(m)corresponding':  lf.sensor_altitude_m=            first
      'incidenceangletoimagecenterpix':  lf.incid_angle_center_deg=       first
      'slantrangedistancetoimagecente':  lf.slant_range_center_m=         first
      'slantrangedistancetonearimagee':  lf.slant_range_near_m=           first 
      'totalnumberoflinesinsioimage':    lf.lines_sio_image=              first
      'totalnumberofelements(perline)':  lf.pixels_sio_image=             first
      'maximumnumberoflinesinmaster(o':  lf.lines_original_image=         first
      'maximumnumberofelementsinmaste':  lf.pixels_original_image=        first
      'referencecornerlinenumberrelat':  lf.ref_corner_line=              first
      'referencecornerelementnumberre':  lf.ref_corner_pixel=             first
      'referencelinespacingincrementr':  lf.ref_line_incr=                first 
      'referenceelementspacingincreme':  lf.ref_pixel_incr=               first
      'numberofrangelooksinoriginalim':  lf.looks_range=                  first
      'numberofazimuthlooksinoriginal':  lf.looks_azimuth=                first
      'referenceellipsoid(datum)':       lf.ref_ellipsoid=                first
      'ref.ellipsoidsemi-majoraxisinm':  lf.ref_semi_major_axis_m=        first
      'ref.ellipsoidsemi-minoraxisinm':  lf.ref_semi_minor_axis_m=        first
      'datum(ellipsoid)shiftsdx,dy,dz':  lf.datum_shifts_m=               first
      'datum(ellipsoid)scalefactor':     lf.datum_scale=                  first
      'ulouterimagecornerlatitude(deg':  lf.ul_outer_lat=                 first
      'ulouterimagecornerlongitude(de':  lf.ul_outer_lon=                 first 
      'urouterimagecornerlatitude(deg':  lf.ur_outer_lat=                 first
      'urouterimagecornerlongitude(de':  lf.ur_outer_lon=                 first
      'llouterimagecornerlatitude(deg':  lf.ll_outer_lat=                 first
      'llouterimagecornerlongitude(de':  lf.ll_outer_lon=                 first
      'lrouterimagecornerlatitude(deg':  lf.lr_outer_lat=                 first
      'lrouterimagecornerlongitude(de':  lf.lr_outer_lon=                 first
      'ulinnerimagetruecornerlatitude':  lf.ul_inner_lat=                 first
      'ulinnerimagetruecornerlongitud':  lf.ul_inner_lon=                 first 
      'urinnerimagetruecornerlatitude':  lf.ur_inner_lat=                 first
      'urinnerimagetruecornerlongitud':  lf.ur_inner_lon=                 first
      'llinnerimagetruecornerlatitude':  lf.ll_inner_lat=                 first
      'llinnerimagetruecornerlongitud':  lf.ll_inner_lon=                 first
      'lrinnerimagetruecornerlatitude':  lf.lr_inner_lat=                 first
      'lrinnerimagetruecornerlongitud':  lf.lr_inner_lon=                 first
      'sourceforlongitude/latitudepol':  lf.source_lonlat_fit=            first
      'polynomialtransformorder(inall':  lf.basis_lon_fit=                first
      'degreeofpolynomialforlongitude':  lf.deg_lonlat_fit=               first
      'sourceforincidentanglepolynomi':  lf.source_incid_fit=             first
      'degreeofpolynomialforincidenta':  lf.deg_incid_fit=                first
      'sourceforradarlookanglepolynom':  lf.source_rlook_fit=             first
      'degreeofpolynomialforlookangle':  lf.deg_rlook_fit=                first
      'sourceforsigmanepolynomialtran':  lf.source_sigma_ne_fit=          first
      'degreeofpolynomialforsigmane'  :  lf.deg_sigma_ne_fit=             first
      'nrcsslopecorrection'           :  lf.nrcs_slope=                   first
      'nrcsbiascorrection'            :  lf.nrcs_bias=                    first
      'icefile'                       :  lf.icefile=                      first
      'icefile_time_js'               :  lf.icefile_time_js=              first
      'icefile_time'                  :  lf.icefile_time=                 first
      'sentinelcpolremovenoise'       :  lf.sen_cpol_remove_noise=        first
      'sentinelxpolremovenoise'       :  lf.sen_xpol_remove_noise=        first
      'cpolremovenoise'               :  lf.cpol_remove_noise=            first
      'xpolremovenoise'               :  lf.xpol_remove_noise=            first
      ELSE:                           ; do nothing
      endcase
    endif
  endif
endfor
    

;-----------------------------------------------------
;read coefficients for converting i,j to latitude
;------------------------------------------------------
deg=      fix ( lf.deg_lonlat_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   lat_coef= dblarr(nc)
   lat_xexp= dblarr(nc)
   lat_yexp= dblarr(nc)
   lat_xexp= [0,1,0]
   lat_yexp= [0,0,1]
   end
2: begin
   nc=6
   lat_coef= dblarr(nc)
   lat_xexp= dblarr(nc)
   lat_yexp= dblarr(nc)
   lat_xexp= [0,1,0,1,2,0]
   lat_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   lat_coef= dblarr(nc)
   lat_xexp= dblarr(nc)
   lat_yexp= dblarr(nc)
   lat_xexp= [0,1,0,1,2,0,2,1,3,0]
   lat_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase
for j=0, nc-1 do begin
  lat_coef(j)=  0
  searchstr= 'latitude transform coef for element^'+ $
              strcompress(string(lat_xexp(j), format='(I)'), /remove_all)  + $
              ' line^'                                          + $
              strcompress(string(lat_yexp(j), format='(I)'), /remove_all) 
  searchstr= strcompress( searchstr, /remove_all)
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  lat_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'lat_coef',  lat_coef, $
                            'lat_xexp',  lat_xexp, $
                            'lat_yexp',  lat_yexp )
lf=   0
lf=   temp_lf


;-----------------------------------------------------
;read coefficients for converting i,j to longitude
;------------------------------------------------------
deg=      fix ( lf.deg_lonlat_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   lon_coef= dblarr(nc)
   lon_xexp= dblarr(nc)
   lon_yexp= dblarr(nc)
   lon_xexp= [0,1,0]
   lon_yexp= [0,0,1]
   end
2: begin
   nc=6
   lon_coef= dblarr(nc)
   lon_xexp= dblarr(nc)
   lon_yexp= dblarr(nc)
   lon_xexp= [0,1,0,1,2,0]
   lon_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   lon_coef= dblarr(nc)
   lon_xexp= dblarr(nc)
   lon_yexp= dblarr(nc)
   lon_xexp= [0,1,0,1,2,0,2,1,3,0]
   lon_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase
for j=0, nc-1 do begin
  lon_coef(j)=  0
  searchstr1= 'longitude (-180to180) transform coef for element^'+ $
              strcompress(string(lon_xexp(j), format='(I)'), /remove_all)  + $
              ' line^'                                          + $
              strcompress(string(lon_yexp(j), format='(I)'), /remove_all) 
  searchstr1= strcompress( searchstr1, /remove_all)  
  searchstr2= 'longitude (0to360) transform coef for element^'+ $
              strcompress(string(lon_xexp(j), format='(I)'), /remove_all)  + $
              ' line^'                                          + $
              strcompress(string(lon_yexp(j), format='(I)'), /remove_all) 
  searchstr2= strcompress( searchstr2, /remove_all)  
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr1 or second eq searchstr2 then  lon_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'lon_coef',  lon_coef, $
                            'lon_xexp',  lon_xexp, $
                            'lon_yexp',  lon_yexp )
lf=   0
lf=   temp_lf

;-----------------------------------------------------
;read coefficients for converting i,j to incidence
;------------------------------------------------------
deg=      fix ( lf.deg_incid_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   incid_coef= dblarr(nc)
   incid_xexp= dblarr(nc)
   incid_yexp= dblarr(nc)
   incid_xexp= [0,1,0]
   incid_yexp= [0,0,1]
   end
2: begin
   nc=6
   incid_coef= dblarr(nc)
   incid_xexp= dblarr(nc)
   incid_yexp= dblarr(nc)
   incid_xexp= [0,1,0,1,2,0]
   incid_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   incid_coef= dblarr(nc)
   incid_xexp= dblarr(nc)
   incid_yexp= dblarr(nc)
   incid_xexp= [0,1,0,1,2,0,2,1,3,0]
   incid_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase
for j=0, nc-1 do begin
  incid_coef(j)=  0
  searchstr= 'incident angle (deg)  transform coef for element^'+ $
              strcompress(string(incid_xexp(j), format='(I)'), /remove_all)  + $
              ' line^'                                          + $
              strcompress(string(incid_yexp(j), format='(I)'), /remove_all) 
  searchstr= strcompress(searchstr, /remove_all)
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  incid_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'incid_coef',  incid_coef, $
                            'incid_xexp',  incid_xexp, $
                            'incid_yexp',  incid_yexp )
lf=   0
lf=   temp_lf

;-----------------------------------------------------
;read coefficients for converting i,j to rlook
;------------------------------------------------------
deg=      fix ( lf.deg_rlook_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   rlook_coef= dblarr(nc)
   rlook_xexp= dblarr(nc)
   rlook_yexp= dblarr(nc)
   rlook_xexp= [0,1,0]
   rlook_yexp= [0,0,1]
   end
2: begin
   nc=6
   rlook_coef= dblarr(nc)
   rlook_xexp= dblarr(nc)
   rlook_yexp= dblarr(nc)
   rlook_xexp= [0,1,0,1,2,0]
   rlook_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   rlook_coef= dblarr(nc)
   rlook_xexp= dblarr(nc)
   rlook_yexp= dblarr(nc)
   rlook_xexp= [0,1,0,1,2,0,2,1,3,0]
   rlook_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase

for j=0, nc-1 do begin
  rlook_coef(j)=  0
  searchstr= 'radar look direction (deg)  transform coef for element^'+ $
              strcompress(string(rlook_xexp(j), format='(I)'), /remove_all)  + $
              ' line^'                                          + $
              strcompress(string(rlook_yexp(j), format='(I)'), /remove_all) 
  searchstr= strcompress(searchstr, /remove_all)
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  rlook_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'rlook_coef',  rlook_coef, $
                            'rlook_xexp',  rlook_xexp, $
                            'rlook_yexp',  rlook_yexp )
lf=   0
lf=   temp_lf

;----------------------------------------------------
; read coefficients for convertin i to sigma_ne
;----------------------------------------------------
deg=  fix(lf.deg_sigma_ne_fit)
if deg eq 0 then begin
  sigma_ne_coef=0
endif else begin
  sigma_ne_coef= dblarr(deg+1)
  for j=0, deg do begin
    searchstr= 'sigma ne (dB) transform coef for element^'+ $
                strcompress(string(j, format='(I)'), /remove_all) 
    searchstr= strcompress(searchstr, /remove_all)
    for i=0, n-1 do begin
      res=strpos(s[i], '%')
      if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  sigma_ne_coef(j)= double( first )
      endif
    endfor
  endfor
endelse
temp_lf= create_struct( lf, 'sigma_ne_coef',  sigma_ne_coef )
lf=   0
lf=   temp_lf

;-----------------------------------------------------
;read coefficients for converting longitude and latitude to j
;------------------------------------------------------
deg=      fix ( lf.deg_lonlat_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   j_coef= dblarr(nc)
   j_xexp= dblarr(nc)
   j_yexp= dblarr(nc)
   j_xexp= [0,1,0]
   j_yexp= [0,0,1]
   end
2: begin
   nc=6
   j_coef= dblarr(nc)
   j_xexp= dblarr(nc)
   j_yexp= dblarr(nc)
   j_xexp= [0,1,0,1,2,0]
   j_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   j_coef= dblarr(nc)
   j_xexp= dblarr(nc)
   j_yexp= dblarr(nc)
   j_xexp= [0,1,0,1,2,0,2,1,3,0]
   j_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase
for j=0, nc-1 do begin
  j_coef(j)=  0
  searchstr= 'line transform coef for longitude^'+ $
              strcompress(string(j_xexp(j), format='(I)'), /remove_all)  + $
              ' latitude^'                                          + $
              strcompress(string(j_yexp(j), format='(I)'), /remove_all) 
  searchstr= strcompress( searchstr, /remove_all)
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  j_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'j_coef',  j_coef, $
                            'j_xexp',  j_xexp, $
                            'j_yexp',  j_yexp )
lf=   0
lf=   temp_lf

;-----------------------------------------------------
;read coefficients for converting longitude and latitude to i
;------------------------------------------------------
deg=      fix ( lf.deg_lonlat_fit )
if deg eq 0 then deg=3
case deg of
1: begin
   nc=3
   i_coef= dblarr(nc)
   i_xexp= dblarr(nc)
   i_yexp= dblarr(nc)
   i_xexp= [0,1,0]
   i_yexp= [0,0,1]
   end
2: begin
   nc=6                                                               
   i_coef= dblarr(nc)
   i_xexp= dblarr(nc)
   i_yexp= dblarr(nc)
   i_xexp= [0,1,0,1,2,0]
   i_yexp= [0,0,1,1,0,2]
   end
3: begin 
   nc=10
   i_coef= dblarr(nc)
   i_xexp= dblarr(nc)
   i_yexp= dblarr(nc)
   i_xexp= [0,1,0,1,2,0,2,1,3,0]
   i_yexp= [0,0,1,1,0,2,1,2,0,3]
   end
endcase
for j=0, nc-1 do begin
  i_coef(j)=  0
  searchstr= 'element transform coef for longitude^'+ $
              strcompress(string(i_xexp(j), format='(I)'), /remove_all)  + $
              ' latitude^'                                          + $
              strcompress(string(i_yexp(j), format='(I)'), /remove_all) 
  searchstr=  strcompress(searchstr, /remove_all)
  for i=0, n-1 do begin
     res=strpos(s[i], '%')
     if res gt -1 then begin  ; there must a % sign or else we skip
        first=   getwrd( s[i], 0, del='%' )
        second=  getwrd( s[i], 1, del='%' )
        second=  strcompress(second, /remove_all)
        if second eq searchstr then  i_coef(j)= double( first )
      endif
  endfor
endfor
temp_lf= create_struct( lf, 'i_coef',  i_coef, $
                            'i_xexp',  i_xexp, $
                            'i_yexp',  i_yexp )
lf=   0
lf=   temp_lf


end
