;+---------------------------
; NAME: ANS_READ_SMAP
; PURPOSE:
;    The purpose of this program is to read NASA SMAP HDF file and
;    create a SIO formatted binary image file and ASCII info file.
; CALLING SEQUENCE:
;    ans_read_smap, input_file, deffile
; INPUT:
;    input_file  = SMAP HDF data file
;    deffile     = Defaults file
; OUTPUT:
;   sio_out      = Array containing the list of base frame names written out
;     Binary files with image data and an ASCII file with ancillary data.  The
;    convention for file names is of the form:
;      SMAP-[AF]_NASA_YYYY_MM_DD_HH_MI_SS_JJJJJJJJJJ_CLON[EW]_CLAT[NS]_[HV][HV]_image.dat (image file)
;      SMAP-[AF]_NASA_YYYY_MM_DD_HH_MI_SS_JJJJJJJJJJ_CLON[EW]_CLAT[NS]_[HV][HV]_ancil.txt (ancillary data file)
;    The file name is delimited by _ and the entries are sensor (-F = Fore, -A = Aft) , processing facility, year,
;    month, day, hour, minute,  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header.
;    SMAP HDF files contain HH, VV and HV Polarization for both a Fore and Aft look of the radar
;    In addition the latitude, longitude, radar look angle and incidence angle are given as arrays in the hdf file
; KEYWORDS:
;     /detail      = Print out some addtional detail on the processing (%Valid, %Ocean)
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (1 if program completes or -1 if not)ta
;     jsstart      = Processing start time
;
; MODIFICATION HISTORY
;
;  2015 Dec 16  Chris Jackson. Initial Version
;  2016 Nov 02  Frank Monaldo. Extend functional of sio_cross_pol flag
; Copyright (C) 1999-2013, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;

pro ans_read_smap, input_file   , deffile, sio_root,   $
    quiet       = quiet,      $
    help        = help,       $
    detail      = detail,     $
    jsstart     = jsstart ,   $
    error       = error
    
  error=     1
  if keyword_set(help) or (n_params(0) lt 1 ) then begin
    print, ' NAME: ANS_READ_SMAP
    print, ' PURPOSE:
    print, '    The purpose of this program is to read NASA SMAP HDF file and
    print, '    create a SIO formatted binary image file and ASCII info file.
    print, ' CALLING SEQUENCE:
    print, '    ans_read_smap, input_file, deffile
    print, ' INPUT:
    print, '    input_file  = SMAP HDF data file
    print, '    deffile     = Defaults file
    print, ' OUTPUT:
    print, '   sio_out      = Array containing the list of base frame names written out
    print, '     Binary files with image data and an ASCII file with ancillary data. The
    print, '    convention for file names is of the form:
    print, '      SMAP-[AF]_NASA_YYYY_MM_DD_HH_MI_SS_JJJJJJJJJJ_CLON[EW]_CLAT[NS]_[HV][HV]_image.dat (image file)
    print, '      SMAP-[AF]_NASA_YYYY_MM_DD_HH_MI_SS_JJJJJJJJJJ_CLON[EW]_CLAT[NS]_[HV][HV]_ancil.txt (ancillary data file)
    print, '    The file name is delimited by _ and the entries are sensor (-F = Fore, -A = Aft) , processing facility, year,
    print, '    month, day, hour, minute,  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
    print, '    files include the SIO 20-byte header.
    print, '    SMAP HDF files contain HH, VV and HV Polarization for both a Fore and Aft look of the radar
    print, '    In addition the latitude, longitude, radar look angle and incidence angle are given as arrays in the hdf file
    print, ' KEYWORDS:
    print, '     sio_out      = Array containing the list of base frame names written out
    print, '     /detail      = Print out some addtional detail on the processing (%Valid, %Ocean)
    print, '     /quiet       = Do not print intermediate results.
    print, '     /help        = Print out help
    print, '     error        = error (1 if program completes or -1 if not)ta
    print, '     outputroot   = root name of outputfiles
    print, '     jsstart      = Processing start
    error= -1
    return
  endif
  
  scaling   = 1.0
  error    = 1
  printflag =1
  if     keyword_set(quiet) then printflag=0
  if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) )
  
  ;
  ; Read defaults file
  ;
  ans_read_defaults, deffile, df, quiet=quiet
  
  time0 = dt_tm_tojs(systime())
  
  if tag_test(df, 'silent')  then begin
    if df.silent eq 'Y' then printflag=0
    if df.silent eq 'N' then printflag=1
  endif
  quiet= printflag eq 0
  
  ff= file_search(df.log_dir, count=count)
  if count eq 0 then  begin
    cd , current=log_dir
    df.log_dir = log_dir
    print, 'No log_dir directory exists'
    print, 'Setting to log_dir to current directory - '+df.log_dir
  ;    error= -1
  ;    return
  endif
  
  ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
  ;
  ; Determine if directory exists
  ;
  ff= file_search( input_file, count=count)
  if count eq 0 then begin
    print, 'Could not find SMAP: ' + input_file
    print, 'Exiting ans_read_smap.pro '
    error= -1
    return
  endif
  
  
  
  ;
  ; Create a new working directory
  ;
  working_directory= df.working_directory + '/smap_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
  working_directory= strcompress( working_directory, /remove_all)
  file_mkdir, working_directory
  ;
  ; Copy SMAP file to working directory.
  ;
  file_mkdir, working_directory  + '/'
  file_copy, input_file, working_directory  + '/', /recursive
  
  ;  ;!!!!!!!!!!!!!!!!!!!!!!Temporary!!!!!!!
  ;  cd, current=working_directory
  ;  cd, current=sio_output_dir
  ;  df.sio_output_dir = sio_output_dir
  ;  ;!!!!!!!!!!!!!!!!!!!!!!!!!!Remove on Linux box
  
  res= file_search(df.already_processed, /test_directory, count=count)
  if count eq 0 then file_mkdir, df.already_processed
  if printflag then begin
    print, 'Moving ' + input_file              + '/'
    print, 'to     ' + df.already_processed    + '/'
  endif
  file_move, input_file,  df.already_processed  + '/' , /overwrite
  time1 = dt_tm_tojs(systime())
  
  if printflag then print, 'Time to move SMAP file to working directory='+ string (time1-time0) +' s'
  
  ;
  ; Keep only the input_file name and strip the path
  filebreak, input_file, file=inputfile ;Puts the filename only into inputfile
  ;inputfile= getwrd(input_file, 0, /last, del='/')
  input_file2 = working_directory + '/' + inputfile
  ;
  ; Reading HDF file
  ;;
  ; Load the lf array to get some information about the data
  ;
  ;Input file might need to change with the working directory copy
  smap_fillin_lf, input_file2, deffile, lf, jsstart = jsstart, quiet = quiet
  nlines = lf.lines_original_image
  npixels = lf.pixels_original_image
  
  ;Get the time array from the HDF5 file and convert to JS
  ; Spacecraft_Data/along_track_time_utc (UTC time stamp when spacecraft passed over midpoint of the along track grid cell row along the nadir path)
  ;This is a 1-D array whose length equals the number of lines
  ans_read_smap_hdf, input_file2, data_utctime, /track_time,/quiet
  data_jstime = dt_tm_tojs(data_utctime,format='yyyy-nn-ddThh:mm:ssssssZ',/ymd)
  
  ;Convert back to check
  data_utctime2 = dt_tm_fromjs(data_jstime, format='Y$-0n$-0d$Th$:m$:s$f$Z',decimal=3 )
  ;Syntax for this function - just for reference
  ; print, dt_tm_fromjs(dt_tm_tojs('2015-05-15T07:51:00.951Z',format='yyyy-nn-ddThh:mm:ssssssZ',/ymd), format='Y$-0n$-0d$ h$:m$:s$f$ UTC',decimal=3 )
  ;Check the difference in data_jstime with ts_diff(x,1)
  ;Put in a checl to make sure there are not "bad" times or big jumps?
  
  ;Potentially have to read up to 3 data sets (hh,vv and hv) for each SMAP file
  ;then break it up in the along track direction into "square" image
  ;However the NRCS SMAP data do not fill the whole HDF data arrays (fill value = -9999)
  ;so read in the HH data set and determine the first valid data start and stop locations
  
  fore = 1
  aft = 0
  if strcmp(df.smap_radar,'A',1,/FOLD_CASE) then begin
    fore =0
    aft = 1
  endif
  
  ;Read the HH data
  if (fore eq 1) then begin
    ans_read_smap_hdf, input_file2, data_hh, /fore, /hh
  endif else begin
    ans_read_smap_hdf, input_file2, data_hh, /aft, /hh
  endelse
  
  ;  ;Start crudely by just finding the beginning and ending of the valid data
  ;  ;Find the first line with a valid data point
  ;  idx = where(data_hh ge -0.01,count,/null)
  ;  start_line = floor(idx[0]/npixels)
  ;  end_line = ceil(idx[n_elements(idx)-1.0]/npixels)
  
  ;Or do something a little better (supercedes above) and find the first line that contains more than 15% valid data values.
  line_percent = fltarr(nlines)
  for i = 0,nlines-1L do begin
    xx=where(data_hh[*,i] ge -0.01, count)
    line_percent[i] = float(count)
  endfor
  line_percent = line_percent/npixels
  idx = where(line_percent ge 0.15,count,/null)
  start_line = idx[0]
  end_line = idx[n_elements(idx)-1L]
  
  if printflag then print, 'Valid SMAP data span lines: '+strcompress(string(start_line,format='(I5)'))+' to '+ strcompress(string(end_line,format='(I5)'))
  
  nframes = (float(end_line - start_line))/npixels
  ; Test to make sure there are some frames to process
  if (nframes le 0 ) then begin
    print, 'No valid data was found in the Sigma0 HH-' +strupcase(radar_dir) + ' array of ' + input_file
    print, 'Exiting ans_read_smap.pro;
    error= -1
    return
  endif
  fframes = nframes - floor(nframes)
  nframes = floor(nframes)
  
  if printflag then print, 'Reading Sigma0 Data Arrays'
  
  ;Read the VV data - Get the incidence and azimuth (look) angle array as well
  if (fore eq 1) then begin
    ans_read_smap_hdf, input_file2, data_vv, /fore, /vv, /quiet
    ans_read_smap_hdf, input_file2, incid, /fore, /incid, /quiet
    ans_read_smap_hdf, input_file2, rlook, /fore, /rlook, /quiet
    ;Read the HV (xpol) data if necessary
      if tag_value(df,'sio_cross_pol') eq '01' or $
         tag_value(df,'sio_cross_pol') eq '11' then ans_read_smap_hdf, input_file2, data_hv, /fore, /hv
  endif else begin
    ans_read_smap_hdf, input_file2, data_vv, /aft, /vv, /quiet
    ans_read_smap_hdf, input_file2, incid, /aft, /incid,/quiet
    ans_read_smap_hdf, input_file2, rlook, /aft, /rlook,/quiet
    ;Read the HV (xpol) data if necessary
    if tag_value(df,'sio_cross_pol') eq '01' or $
       tag_value(df,'sio_cross_pol') eq '11' then ans_read_smap_hdf, input_file2, data_hv, /aft, /hv
  endelse
  
  ;Read in the longitude, latitude, gradient array
  if printflag then print, 'Reading Longitude/Latitude/Incidence/Azimuth Data Arrays'
  ans_read_smap_hdf, input_file2, lon_array, /longitude, /quiet
  ans_read_smap_hdf, input_file2, lat_array, /latitude, /quiet
  ans_read_smap_hdf, input_file2, grad_array, /gradient, /quiet
  
  frame_names = strarr(2*(nframes+1))
  base_names = strarr(2*(nframes+1))
  diag_array = fltarr(4,2*(nframes+1))
  if tag_value(df,'sio_cross_pol') eq '01' or $
     tag_value(df,'sio_cross_pol') eq '11' then begin
    frame_names = strarr(3*(nframes+1))
    base_names  = strarr(3*(nframes+1))
    diag_array  = strarr(4,3*(nframes+1))
  endif
  fncnt = 0;
  
  ;Loop over the frames
  for i = 0, nframes do begin
  
    fstart_line = start_line + i*npixels
    fstop_line = fstart_line + npixels -1
    ;Deal with the fractional final frame
    if (i eq nframes) then begin
      fstop_line = end_line
      ;If the fractional frames is less than 50% then get a full frame back npixels from the last line instead.
      if (fframes lt 0.5) then begin
        fstart_line = fstop_line -(npixels-1)
      endif
    endif
    
    
    if printflag then print, 'Processing SMAP sub-scene:' + strcompress(string(i+1,format='(I4)')) +' of ' + strcompress(string(nframes+1,format='(I3)')) $
      +'   Lines: '+strcompress(string(fstart_line,format='(I5)'))+' to '+ strcompress(string(fstop_line,format='(I5)'))
      
    fstart_js = data_jstime[fstart_line] ;For working with along_track_time_utc
    fstop_js = data_jstime[fstop_line] ;For working with along_track_time_utc
    
    diag_array[2,i] = fstart_line
    diag_array[3,i] = fstop_line
    
    ;Create a frame lf structure
    frame_lf = lf
    frame_lf.jsbegin = fstart_js
    frame_lf.jsend = fstop_js
    center_js      = (fstart_js + fstop_js) /2.0D
    frame_lf.acquisition_date       = dt_tm_fromjs( center_js, format='0n$-0d$-Y$' )
    frame_lf.acquisition_time       = dt_tm_fromjs( center_js, format='h$:m$:s$' )
    frame_lf.acq_date_time_js       = center_js
    frame_lf.acq_date_time          = dt_tm_fromjs( center_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
    frame_lf.lines_sio_image  =  fstop_line-fstart_line+1
    frame_lf.pixels_sio_image = npixels
    frame_lf.nx1 = npixels
    frame_lf.ny1 = fstop_line-fstart_line+1
    
    ;Extract the frame data from the full data array  (Start with HH - remaining are below)
    fdata_hh = data_hh[*,fstart_line:fstop_line]*lf.nrcs_slope+lf.nrcs_bias
    
    ;Check the number of valid data points in the frame
    idx_valid = where(fdata_hh ge -0.01, valid_count)
    ;If there are less than 10% valid nrcs measurements skip the scene.
    if (n_elements(fdata_hh) eq 0) or (float(valid_count)/float(n_elements(fdata_hh)) le 0.1) then begin
      print, 'Frame ' + string(i)+ ' Contains less than 10% valid NRCS measurements'
      print, 'Valid Percent = '+string(100.0*float(valid_count)/float(n_elements(fdata_hh)), format='(F6.3)')
      print, 'Skipping creation of sio/ancil file for ' + frame_lf.acq_date_time
      continue
    endif
    
    ;Check the number of gradient = 0 points in the frame (assume gradient = 0 is ocean)
    fgrad = grad_array[*,fstart_line:fstop_line]
    ;But like lat and lon the gradient is defined everywhere so only incude valid values
    idx_grad = where(round(fgrad) eq 0 and fdata_hh ge -0.01, grad0_count)
    if (float(grad0_count)/float(valid_count) le 0.05) then begin
      print, 'Frame ' + string(i)+ ' appears to contain < 5% ocean'
      print, 'Ocean Percent = '+string(100.0*float(grad0_count)/float(valid_count), format='(F6.3)')
      print, 'Skipping creation of sio/ancil file for ' + frame_lf.acq_date_time
      continue
    endif
    
    ;Extract the remaining frame data from the full data array  VV, HV,
    fdata_vv = data_vv[*,fstart_line:fstop_line]*lf.nrcs_slope+lf.nrcs_bias
    if tag_value(df,'sio_cross_pol') eq '01' or $
       tag_value(df,'sio_cross_pol') eq '11' then  fdata_hv = data_hv[*,fstart_line:fstop_line]*lf.nrcs_slope+lf.nrcs_bias
    
    ;Longitude, Latitude
    flon  = lon_array[*,fstart_line:fstop_line]
    flat  = lat_array[*,fstart_line:fstop_line]
    
    ;Incidence and Radar Look angle (azimuth)
    ;Unlike Lat and Lon the incidence and look angle arrays are not totally filled.
    ;If there is no data the arrays contain -9999.
    fincid = incid[*,fstart_line:fstop_line]
    frlook = rlook[*,fstart_line:fstop_line]
    
    ;Perform some polynomial fits
    if printflag then print, 'Performing Polynomial Fits Lon/Lat/i/j'
    ;Longitude / Latitude / Pixel (i) / Line (j)
    smap_lonlat2pix_fit, flon, flat, frame_lf, /quiet
    ;   For checking
    ;Set up the ii and jj arrays
    ;        sz = size(flon)
    ;        iii = lonarr(sz[1],sz[2])
    ;        for i2 =0,sz[1]-1L do iii(*,i2) = lindgen(sz[2])
    ;        jji = rotate(iii,1)
    ;        clon=ans_poly_cal( iii, jji, frame_lf.lon_coef, frame_lf.lon_xexp, frame_lf.lon_yexp )
    ;        clat=ans_poly_cal( iii, jji, frame_lf.lat_coef, frame_lf.lat_xexp, frame_lf.lat_yexp )
    ;
    ;Get the frame center longitude and latitude
    clon0=ans_poly_cal( frame_lf.nx1/2.0, frame_lf.ny1/2.0, frame_lf.lon_coef, frame_lf.lon_xexp, frame_lf.lon_yexp )
    clat0=ans_poly_cal( frame_lf.nx1/2.0, frame_lf.ny1/2.0, frame_lf.lat_coef, frame_lf.lat_xexp, frame_lf.lat_yexp )
    frame_lf.center_lat = clat0
    frame_lf.center_lon = clon0
    ;Temporary Check the result of the center pixel
    iid=ans_poly_cal( clon0, clat0, frame_lf.i_coef, frame_lf.i_xexp, frame_lf.i_yexp )
    jjd=ans_poly_cal( clon0, clat0, frame_lf.j_coef, frame_lf.j_xexp, frame_lf.j_yexp )
    ;print, i, clon0,clat0, iid, jjd, ' ', systime(/utc)
    
    ;Fit Incidence Angle
    ;Incidence to i/j
    if printflag then print, 'Performing Polynomial Fit Incidence/i/j'
    smap_incid2pix_fit, fincid, frame_lf, /quiet
    ;   sz = size(fincid)
    ;   ;Set up the ii and jj arrays
    ;  iii = lonarr(sz[1],sz[2])
    ;  for i2 =0,sz[1]-1L do iii(*,i2) = lindgen(sz[2])
    ;  jji = rotate(iii,1)
    ;   cincid=ans_poly_cal( iii, jji, frame_lf.incid_coef, frame_lf.incid_xexp, frame_lf.incid_yexp )
    ;   im = image((fincid-cincid), min_value = 0, max_value = 30)
    ;   c = COLORBAR(TARGET=im, ORIENTATION=1)
    
    ;Fit Radar Look Angle
    ;Radar Look Angle to i/j
    if printflag then print, 'Performing Polynomial Fit Radar_Look/i/j'
    smap_rlook2pix_fit, frlook, frame_lf, /quiet
    ;    sz = size(frlook)
    ;    ;Set up the ii and jj arrays
    ;    iii = lonarr(sz[1],sz[2])
    ;    for i2 =0,sz[1]-1L do iii(*,i2) = lindgen(sz[2])
    ;    jji = rotate(iii,1)
    ;    crlook=ans_poly_cal( iii, jji, frame_lf.rlook_coef, frame_lf.rlook_xexp, frame_lf.rlook_yexp )
    ;    fidx = where(frlook ge -180.0,/null);
    ;    if ( max(frlook[fidx]) - min(frlook[fidx]) gt 160.0) then frlook[fidx] = (frlook[fidx] + 360) MOD 360
    ;    im = image((frlook-crlook), min_value = 0, max_value = 10)
    ;    c = COLORBAR(TARGET=im, ORIENTATION=1)
    
    ;Build-up the filename of the form
    ;SMAP-[AF]_NASA_YYYY_MM_DD_HH_MI_SS_JJJJJJJJJJ_CLON[EW]_CLAT[NS]_[HV][HV]_L_image.dat (image file)
    ;
    smap_output_str = 'SMAP-F_NASA_'
    if (aft eq 1) then smap_output_str = 'SMAP-A_NASA_'
    smap_output_str = smap_output_str+dt_tm_fromjs(center_js,  format='Y$_0n$_0d$_h$_m$_s$')+'_'+string(center_js,format='(I10.10)')
    clon_str = string(abs(frame_lf.center_lon),format='(F06.2)')+'E'
    if (frame_lf.center_lon lt 0) then clon_str=repchr(clon_str,'E','W')
    clat_str =  string(abs(frame_lf.center_lat),format='(F05.2)')+'N'
    if (frame_lf.center_lat lt 0) then clat_str=repchr(clat_str,'N','S')
    smap_output_str = smap_output_str+'_'+clon_str+'_'+clat_str
    
    ;Put the basename into an array to return
    ;==================================
    ;   HH Output Block
    ;==================================
    ;Write out the HH _image.dat sio and _ancil.txt file
    scaling=1.0
    frame_lf.polarization  = 'HH'
    frame_lf.pol           = 'HH'
    frame_lf.flag_sar_band_polarization = '32'
    smap_output_str = smap_output_str+'_'+frame_lf.polarization+'_L'
    base_names[fncnt] = smap_output_str
    full_output_str= df.sio_output_dir + '/'+ smap_output_str
    full_output_str= stress( full_output_str, 'R', 0, '//', '/', ned)
    frame_names[fncnt] = full_output_str;
    diag_array[0,fncnt] = 100.0*float(valid_count)/float(n_elements(fdata_hh))
    diag_array[1,fncnt] = 100.0*float(grad0_count)/float(valid_count)
    fncnt++
    
    ; Check and see if a lock file exits
    lockfile=   df.sio_output_dir + '/'+ smap_output_str + '*.lock'
    res= file_search( lockfile, count=count)
    if count gt 0 then begin
      print, 'Lockfile:  ' + lockfile + ' exists'
      print, 'No SIO data written - moving to next sub-scene'
      error=-1
      continue
    endif
    
    ;Create and Write out log file entries
    logfile= df.log_dir + '/' + smap_output_str +'.log'
    log_outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + input_file + ' Processing starts'
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    
    ;Write out HH SIO data (_image.dat) file
    if strupcase(df.twobyte) eq 'Y' then begin
      ans_write_nrcs_sio, fdata_hh, full_output_str + '_image.dat', scaling, /twobyte, quiet=quiet
    endif else begin
      ans_write_nrcs_sio, fdata_hh, full_output_str + '_image.dat', scaling, quiet=quiet
    endelse
    time0= dt_tm_tojs(systime(/utc))
    log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_image.dat'
    close, 1
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    ;
    ; Write out HH SIO ancillary file format.
    ;
    ans_write_ancil_sio, full_output_str+'_ancil.txt', frame_lf, quiet=quiet
    time0= dt_tm_tojs(systime(/utc))
    log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_ancil.txt'
    close, 1
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    
    ;Produce SIO file of longitude, latitude,incidence and/or radar look (azimuth)
    if (strcmp(df.sio_longitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flon , smap_output_str+'_longitude.dat'
    if (strcmp(df.sio_latitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flat , smap_output_str+'_latitude.dat'
    ;Might be necessary to use the polynomials to fill in -9999 values in incidence and radarlook arrays (???)
    if (strcmp(df.sio_incidence,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,fincid, smap_output_str+'_incidence.dat'
    if (strcmp(df.sio_rlook,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,frlook , smap_output_str+'_radarlook.dat'
    
    
    ;==================================
    ;   VV Output Block
    ;==================================
    ;Write out the VV _image.dat sio and _ancil.txt file
    scaling=1.0
    frame_lf.polarization  = 'VV'
    frame_lf.pol           = 'VV'
    frame_lf.flag_sar_band_polarization = '31'
    strput, smap_output_str, 'VV',strpos(smap_output_str,'HH',/REVERSE_SEARCH)
    base_names[fncnt] = smap_output_str
    full_output_str= df.sio_output_dir + '/'+ smap_output_str
    full_output_str= stress( full_output_str, 'R', 0, '//', '/', ned)
    frame_names[fncnt] = full_output_str;
    diag_array[0,fncnt] = 100.0*float(valid_count)/float(n_elements(fdata_vv))
    diag_array[1,fncnt] = 100.0*float(grad0_count)/float(valid_count)
    fncnt++
    
    ; Check and see if a lock file exits
    lockfile=   df.sio_output_dir + '/'+ smap_output_str + '*.lock'
    res= file_search( lockfile, count=count)
    if count gt 0 then begin
      print, 'Lockfile:  ' + lockfile + ' exists'
      print, 'No SIO data written - moving to next sub-scene'
      error=-1
      continue
    endif
    
    ;Create and Write out log file entries
    logfile= df.log_dir + '/' + smap_output_str +'.log'
    log_outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + input_file + ' Processing starts'
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    
    ;Write out VV SIO data (_image.dat) file
    if strupcase(df.twobyte) eq 'Y' then begin
      ans_write_nrcs_sio, fdata_vv, full_output_str + '_image.dat', scaling, /twobyte, quiet=quiet
    endif else begin
      ans_write_nrcs_sio, fdata_vv, full_output_str + '_image.dat', scaling, quiet=quiet
    endelse
    time0= dt_tm_tojs(systime(/utc))
    log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_image.dat'
    close, 1
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    ;
    ; Write out VV SIO ancillary file format.
    ;
    ans_write_ancil_sio, full_output_str+'_ancil.txt', frame_lf, quiet=quiet
    time0= dt_tm_tojs(systime(/utc))
    log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_ancil.txt'
    close, 1
    openw, 1, logfile, /append
    printf, 1, log_outstr
    close, 1
    print, log_outstr
    
    ;Produce SIO file of longitude, latitude,incidence and/or radar look (azimuth)
    if (strcmp(df.sio_longitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flon , smap_output_str+'_longitude.dat'
    if (strcmp(df.sio_latitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flat , smap_output_str+'_latitude.dat'
    ;Might be necessary to use the polynomials to fill in -9999 values in incidence and radarlook arrays (???)
    if (strcmp(df.sio_incidence,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,fincid, smap_output_str+'_incidence.dat'
    if (strcmp(df.sio_rlook,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,frlook , smap_output_str+'_radarlook.dat'
    
    ;==================================
    ;   XPOL (HV) Output Block
    ;==================================
    if n_elements(data_hv) gt 0 then begin
      ;Write out the HV _image.dat sio and _ancil.txt file
      scaling=1.0
      frame_lf.polarization  = 'HV'
      frame_lf.pol           = 'HV'
      frame_lf.flag_sar_band_polarization = '33'
      strput, smap_output_str, 'HV',strpos(smap_output_str,'VV',/REVERSE_SEARCH)
      base_names[fncnt] = smap_output_str
      full_output_str= df.sio_output_dir + '/'+ smap_output_str
      full_output_str= stress( full_output_str, 'R', 0, '//', '/', ned)
      frame_names[fncnt] = full_output_str;
      diag_array[0,fncnt] = 100.0*float(valid_count)/float(n_elements(fdata_hh))
      diag_array[1,fncnt] = 100.0*float(grad0_count)/float(valid_count)
      fncnt++
      
      ; Check and see if a lock file exits
      lockfile=   df.sio_output_dir + '/'+ smap_output_str + '*.lock'
      res= file_search( lockfile, count=count)
      if count gt 0 then begin
        print, 'Lockfile:  ' + lockfile + ' exists'
        print, 'No SIO data written - moving to next sub-scene'
        error=-1
        continue
      endif
      
      ;Create and Write out log file entries
      logfile= df.log_dir + '/' + smap_output_str +'.log'
      log_outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + input_file + ' Processing starts'
      openw, 1, logfile, /append
      printf, 1, log_outstr
      close, 1
      print, log_outstr
      
      ;Write out HV SIO data (_image.dat) file
      if strupcase(df.twobyte) eq 'Y' then begin
        ans_write_nrcs_sio, fdata_hv, full_output_str + '_image.dat', scaling, /twobyte, quiet=quiet
      endif else begin
        ans_write_nrcs_sio, fdata_hv, full_output_str + '_image.dat', scaling, quiet=quiet
      endelse
      time0= dt_tm_tojs(systime(/utc))
      log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_image.dat'
      close, 1
      openw, 1, logfile, /append
      printf, 1, log_outstr
      close, 1
      print, log_outstr
      ;
      ; Write out HV SIO ancillary file format.
      ;
      ans_write_ancil_sio, full_output_str+'_ancil.txt', frame_lf, quiet=quiet
      time0= dt_tm_tojs(systime(/utc))
      log_outstr=    string(time0, format='(I10)') + ' '+ dt_tm_fromjs(time0) + ' ' + smap_output_str + '_ancil.txt'
      close, 1
      openw, 1, logfile, /append
      printf, 1, log_outstr
      close, 1
      print, log_outstr
    endif
    
    ;Produce SIO file of longitude, latitude,incidence and/or radar look (azimuth)
    if (strcmp(df.sio_longitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flon , smap_output_str+'_longitude.dat'
    if (strcmp(df.sio_latitude,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,flat , smap_output_str+'_latitude.dat'
    ;Might be necessary to use the polynomials to fill in -9999 values in incidence and radarlook arrays (???)
    if (strcmp(df.sio_incidence,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,fincid, smap_output_str+'_incidence.dat'
    if (strcmp(df.sio_rlook,'Y',/FOLD_CASE)) then ans_write_nrcs_sio,frlook , smap_output_str+'_radarlook.dat'
    
  endfor
  
  sio_root = frame_names
  
  if keyword_set(detail) then begin
    detail_str = 'Frame_Detail_'+stress(inputfile,'D',0,'h5')+'txt'
    close,2
    openw, 2, detail_str
    print, 'Frame', 'Frame_Names', '%Valid', '%Ocean', 'Start_line', 'Stop_Line'
    for j = 0,n_elements(base_names)-1,1 do begin
      print,format='(I2,2x,A64,2x,2(F6.2,2x),I6,2x,I6)', j,base_names[j],diag_array[0,j],diag_array[1,j],diag_array[2,j],diag_array[3,j]
      printf,2,format='(I2,2x,A64,2x,2(F6.2,2x),I6,2x,I6)', j,base_names[j],diag_array[0,j],diag_array[1,j],diag_array[2,j],diag_array[3,j]
    endfor
    
    close,2
    
  endif
  ;
  ; Remove
  ;
  if printflag then print, 'Deleting ' + working_directory
  file_delete,  working_directory  + '/', /recursive, /quiet, /allow_nonexistent
  
  
  
  
end


