;-----------------------------------------------------
;+
; NAME: ANS_READ_SMAP_HDF
; PURPOSE:
;   The purpose is the read a NASA SMAP Radar Level-1C HDF5 files and return
;   and image and the ancillary data.  HDF5 files must have a .h5 extension.
;   This program with only work with IDL 8 or greater
; CALLING SEQUENCE:
;   ans_read_smap_hdf, file, data
; INPUT:
;    file=     HDF5 file with .h5 extension
; OUTPUT:
;    data=     The data aray from SMAP Sigma0_Data Group
; KEYWORD
;    /fore     The "Fore" radar data
;    /aft      The "Aft" radar data
;    /hh         HH Polarization (specify fore or aft)
;    /vv         VV Polarization (specify fore or aft)
;    /hv         HV Polarization (specify fore or aft)
;    /rlook      Radar Look Azimuth (to True North) (specify fore or aft)
;    /incid      Radar Incidence Angle (specify fore or aft)
;    /latitude   Latitude
;    /longitude  Longitude
;    /gradient   Gradient (Representative land slope degrees)
;    /track_time UTC time stamp for the midpoint of the along track cell (Format YYYY-MM-DDTHH:MI:SS.SSSZ)
;    /quiet    Print no intermediate output.
;    /help     Get help.
;    error=    If =-1 then cannot read file
;    /diag     Diagonistics turned on
;
; MODIFICATION HISTORY:
;    2015 Dec 16 Chris Jackson.   Initial Version
; Copyright (C) 2010-15, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_read_smap_hdf, file, data, $
    fore=       fore   , $
    aft =       aft  ,$
    hh =        hh, $
    vv =        vv, $
    hv =        hv, $
    latitude = latitude, $
    longitude = longitude, $
    gradient = gradient, $
    rlook     = rlook, $
    incid     = incid, $
    track_time = track_time ,$
    help=           help       , $
    quiet=          quiet      , $
    error=          error      , $
    diag=           diag
    
  if keyword_set(help)  or (n_params(0) lt 1 ) then begin
    print, ' NAME: ANS_READ_SMAP_HDF
    print, ' PURPOSE:
    print, '   The purpose is the read a NASA SMAP Level-1C HDF5 files and return
    print, '   and image and the ancillary data.  HDF5 files must have a .h5.
    print, '   This program with only work with IDL 8 or greater
    print, ' CALLING SEQUENCE:
    print, '   ans_read_smap_hdf, file, data, info
    print, ' INPUT:
    print, '    file=     HDF5 file with .h5 extension
    print, ' OUTPUT:
    print, '    data=    Raw SAR image (original digital values)
    print, ' KEYWORD
    print, '    /fore       The "Fore" radar data
    print, '    /aft        The "Aft" radar data
    print, '    /hh         HH Polarization (specify fore or aft)
    print, '    /vv         VV Polarization (specify fore or aft)
    print, '    /hv         HV Polarization (specify fore or aft)
    print, '    /rlook      Radar Look Azimuth (to True North) (specify fore or aft)
    print, '    /incid      Radar Incidence Angle (specify fore or aft)
    print, '    /latitude   Latitude
    print, '    /longitude  Longitude
    print, '    /gradient   Gradient (Representative land slope degrees)
    print, '    /track_time UTC time stamp for the midpoint of the along track cell (Format YYYY-MM-DDTHH:MI:SS.SSSZ)
    print, '    /quiet      Print no intermediate output.
    print, '    /help       Get help.
    print, '    error       If =-1 then cannot read filed
    print, '    /diag       Diagonistics turned on
    error= -1
    return
  endif
  
  version_no= float(!version.release)
  
  if version_no lt 8 then begin
    print, 'This is version ' + !version.release
    print, 'This program will only work with versions 8+ because of the'
    print, 'iopen procedure.
    print, 'Exiting ans_read_smap_hdf'
    error= -1
    return
  endif
  
  error= 0
  
  printflag= 1
  !quiet=    1
  if keyword_set(quiet)    then printflag=     0
  
  res=  file_search(file, count=count)
  
  if count eq 0 then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Cannot find file: ' + file
      print, 'Exiting ans_read_smap_hdf'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  ;
  ;Check to make sure a valid set of keywords has been set
  if (n_elements(fore)+n_elements(aft)+ n_elements(hh)+n_elements(vv)+n_elements(hv)+n_elements(latitude)+ $
    n_elements(longitude)+n_elements(rlook)+n_elements(incidence)+n_elements(gradient)+n_elements(track_time)$
    eq 0 ) then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'No Keywords Set - Dont know what to return'
      print, 'Exiting ans_read_smap_hdf'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  ;Check to make sure a valid set of keywords has been set
  if (n_elements(hh)+n_elements(vv)+n_elements(hv)+n_elements(rlook)+n_elements(incidence) gt 0) $
    and (n_elements(fore)+n_elements(aft) eq 0) then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Fore or Aft Keyword needs to be set in order to return'
      print, '       hh,vv,hv,rlook or incidence data arrays'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  ;
  ;  Reading HDF file into strucutre
  ;
  if printflag then print, 'Begin reading: ' + file + ' ' + systime(/utc)
  fileid = 0
  fileid = h5f_open(file)
  
  if (fileid le 0) then begin
    print,  ''
    print, '============================================================
    print, 'H5F_OPEN failed for file: ' + file
    print, 'Exiting ans_read_smap_hdf'
    print, '============================================================
    print, ''
    error = -1
    return
  endif
  
  ;Get Data
  ;Complete list of available data is given at the end of the file
  ;For ANSWRS the variables or interest are..
  ;Sigma0_Data Group
  ;DataSet               Valid_Min Valid_Max Units
  ;cell_lat                    -90   90     degrees
  ;cell_lon                   -180  179.999 degrees
  ;cell_gradient                 0   90     degrees
  ;slant_range_fore         700000  1100000 meters
  ;earth_azimuth_mean_fore       0  359.999 degrees
  ;earth_incidence_mean_fore     0   90     degrees
  ;cell_sigma0_hh_fore       -0.01   10     n/a
  ;cell_sigma0_vv_fore       -0.01   10     n/a
  ;cell_sigma0_xpol_fore     -0.01   10     n/a
  ;earth_azimuth_mean_aft        0  359.999 degrees
  ;earth_incidence_mean_aft      0   90     degrees
  ;cell_sigma0_hh_aft        -0.01   10     n/a
  ;cell_sigma0_vv_aft        -0.01   10     n/a
  ;cell_sigma0_xpol_aft      -0.01   10     n/a
  
  ;Spacecraft_Data Group
  ;along_tract_time_utc
  
  if keyword_set(latitude)   then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_lat'))
  if keyword_set(longitude)  then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_lon'))
  if keyword_set(gradient)   then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_gradient'))
  if keyword_set(track_time) then data = H5D_READ(H5D_OPEN(fileid, '/Spacecraft_Data/along_track_time_utc'))
  
  if keyword_set(fore) then begin
    if keyword_set(rlook) then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/earth_azimuth_mean_fore'))
    if keyword_set(incid) then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/earth_incidence_mean_fore'))
    if keyword_set(hh)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_hh_fore'))
    if keyword_set(vv)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_vv_fore'))
    if keyword_set(hv)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_xpol_fore'))
  endif
  
  if keyword_set(aft) then begin
    if keyword_set(rlook) then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/earth_azimuth_mean_aft'))
    if keyword_set(incid )then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/earth_incidence_mean_aft'))
    if keyword_set(hh)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_hh_aft'))
    if keyword_set(vv)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_vv_aft'))
    if keyword_set(hv)    then data = H5D_READ(H5D_OPEN(fileid, '/Sigma0_Data/cell_sigma0_xpol_aft'))
  endif
  
  h5f_close, fileid 
  
  ;Put in check for data array is empty
  sz = size(data)
  if (sz[0] eq 0) then begin 
    print,  ''
    print, '============================================================
    print,   'Data array is empty'
    print, 'Exiting ans_read_smap_hdf'
    print, '============================================================
    print, ''
    error = -1
    return
  endif
  
  if printflag then print, 'End   reading: ' + file + ' ' + systime(/utc)
  
  
;Complete list of variables available in the Sigma0_Data Group
;cell_lat
;cell_lon
;cylindrical_grid_row_index
;cylindirical_grid_column_index
;polar_grid_row_index
;polar_grid_column_index
;cell_altitude
;cell_altitude_std_dev
;cell_gradient
;number_of_cell_looks_fore
;number_of_cell_looks_aft
;cell_radar_mode_flag
;cell_sigma0_qual_flag_hh
;cell_sigma0_qual_flag_vv
;cell_sigma0_qual_flag_xpol
;slant_range_fore
;earth_azimuth_mean_fore
;earth_azimuth_std_dev_fore
;earth_incidence_mean_fore
;earth_incidence_std_dev_fore
;cell_sigma0_hh_fore
;cell_sigma0_vv_fore
;cell_sigma0_xpol_fore
;cell_sigma0_hh_fore_noise
;cell_sigma0_vv_fore_noise
;cell_sigma0_xpol_fore_noise
;cell_area_fore
;cell_kp_hh_fore
;cell_kp_vv_fore
;cell_kp_xpol_fore
;slant_range_aft
;earth_azimuth_mean_aft
;earth_azimuth_std_dev_aft
;earth_incidence_mean_aft
;earth_incidence_std_dev_aft
;cell_sigma0_hh_aft
;cell_sigma0_vv_aft
;cell_sigma0_xpol_aft
;cell_sigma0_hh_aft_noise
;cell_sigma0_vv_aft_noise
;cell_sigma0_xpol_aft_noise
;cell_area_aft
;cell_kp_hh_aft
;cell_kp_vv_aft
;cell_kp_xpol_aft
;selected_frequency_aft
;selected_frequency_fore
;polarization_rotation_angle_aft
;polarization_rotation_angle_fore
;faraday_rotation_angle_fore
;faraday_rotation_angle_aft
;faraday_rotation_corr_hh_fore
;faraday_rotation_corr_vv_fore
;faraday_rotation_corr_xpol_fore
;faraday_rotation_corr_hh_aft
;faraday_rotation_corr_vv_aft
;faraday_rotation_corr_xpol_aft
;xpol_correlation_fore
;xpol_correlation_aft
  
  
  
end
