;+---------------------------
; NAME: ANS_READ_TSAR 
; PURPOSE:
;    The purpose of this program is to read TerraSAR-X data and
;    create a SIO formatted binary image file, ASCII file, and
;    binary land mask
; CALLING SEQUENCE:
;    ans_read_tsar, input_dir, deffile
; INPUT:
;    input_dir  =  TerraSAR-X come in directories. This input is the name
;                  of such a directory. A directory might look like
;                  dims_op_oc_dfd2_202312801_1/ 
;    deffile   =   Defaults file
; OUTPUT:          
;    Binary files with image and land mask data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;        TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
;        TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
;        TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
;    The file name is delimited by _ and the entries are Sensor, year, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
;    files include the SIO 20-byte header
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (0 if program completes or -1 if not)
;     gshhs_dir    = gshhs environmental variable pointing to the GSHHS data
;     outputroot   = Root name of SIO files
;     jsstart      = Processing start time
; MODIFICATION HISTORY    
;  2009 Dec 04 Frank Monaldo. Created for ANSWRS 2.0
;  2009 Jan 05 Frank Monaldo. Change to use new ans_compute_fir/ans_poly_cal_routines
;  2009 Jan 15 Frank Monaldo. setenv gshhs_dir         
;  2010 Jan 19 Frank Monaldo. Add /header option to only print out ancillary data file
;  2010 Jan 26 Frank Monaldo. Add pixel_size
;  2010 Jan 28 Frank Monaldo. Add option for creating ANSWRS version. This
;                             require the idlans directory name
;  2010 Feb 05 Frank Monaldo. Us the help call to find the source file directory
;                             This is used to calculated the time of last modifications.
;                             This removes need for providing the name of source directory.
;                             Typing command w/no parameters also calls help. ;  2011 Jan 01 Frank Monaldo. Add log_dir and pass on quiet flag
;  2011 Feb 04 Frank Monaldo. Add jsstart
;  2011 Feb 07 Frank Monaldo. Print original file/dir to log file
;  2011 Feb 08 Frank Monaldo. Add error catching if cannot read files
;  2011 Feb 13 Frank Monaldo. Change the unique directory name to something that makes more sense
;  2011 Feb 14 Frank Monaldo. Add ability to escape if .lock file encountered.
;  2011 May 30 Frank Monaldo. Add nrcs_slope and nrcs_bias
;  2011 Jul 12 Frank Monaldo. Corrected spelling error that kept me from moving files
;  2011 Sep 16 Frank Monaldo. Supply a jsstart if none provided.
;  2011 Sep 19 Frank Monaldo. If log_dir not set use default directory. If the default directory
;                             then do not print log file
;  2011 Sep 21 Frank Monaldo  Placed outstr in a different place.
;  2012 Jul 05 Frank Monaldo. Remove hardwired directories.
;                             Corrected spelling of gshhs_dir
;  2012 Aug 21 Frank Monaldo. Move ans_gen_version
;  2013 Feb 03 Frank Monaldo. Add ice mask and use only defaults file
;  2013 Aug 14 Frank Monaldo. Move icemask, long/lat, incidence, and rlook generation to
;                             ans_sat2sio. Remove deletion of lock file
;  2013 Sep 30 Chris Jackson  Modified the output prefrix for Terrasar to properly distinguish
;                             between TerraSAR (TSX1) and Tandem-X (TDX1)
; Copyright (C) 1999-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;

pro ans_read_tsar,  input_dir,    deffile, $
                    quiet       = quiet           , $
                    help        = help            , $
                    outputroot  = outputroot      , $
                    jsstart     = jsstart         , $
                    error       = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, '  NAME: ANS_READ_TSAR 
  print, '  PURPOSE:
  print, '     The purpose of this program is to read TerraSAR-X data and
  print, '     create a SIO formatted binary image file, ASCII file, and
  print, '     binary land mask
  print, '  CALLING SEQUENCE:
  print, '     ans_read_tsar, input_dir, deffile
  print, '  INPUT:
  print, '     input_dir  =  TerraSAR-X come in directories. This input is the name
  print, '                   of such a directory. A directory might look like
  print, '                   dims_op_oc_dfd2_202312801_1/ 
  print, '     deffile   =   Defaults file
  print, '  OUTPUT:          
  print, '     Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '     convention for file names is of the form:
  print, '         TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)
  print, '         TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)
  print, '         TSAR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_mask.dat  (land mask)
  print, '     The file name is delimited by _ and the entries are Sensor, year, month, day, hour, minute
  print, '     second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data
  print, '     files include the SIO 20-byte header
  print, '  KEYWORDS:
  print, '      /quiet       = Do not print intermediate results.
  print, '      /help        = Print out help
  print, '      error        = error (0 if program completes or -1 if not)
  print, '      gshhs_dir    = gshhs environmental variable pointing to the GSHHS data
  print, '      outputroot   = Root name of SIO files
  print, '      jsstart      = Processing start time
  error= -1
  return
endif

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
error=1
time0 = dt_tm_tojs(systime(/utc))
;
; Read defaults file
;
ans_read_defaults, deffile, df, quiet=quiet, error=error
if error lt 0 then return

if n_elements(df.gshhs_dir)  lt 1 then begin 
  print, 'gshhs_dir not specified.'
  print, 'Exiting ans_read_tsar.pro'
  error= -1
  return
endif
if n_elements(df.idlans_dir) lt 1 then begin 
  print, 'idlans_dir not specified.'
  print, 'Exiting ans_read_tsar.pro'
  error= -1
  return
endif
if n_elements(df.log_dir)    lt 1 then begin 
  print, 'log_dir not specified.'
  print, 'Exiting ans_read_tsar.pro'
  error= -1
  return
endif

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet

setenv, 'GSHHS_DIR=' + df.gshhs_dir
;
; Determine if input directory exists
; 
ff= file_search( input_dir, /test_directory, count=count)

if count eq 0 then begin
  print, 'Could not find TerraSAR-X directory: ' + input_dir
  print, 'Exiting ans_read_tsar.pro '
  error= -1
  return
endif

;
; Create a new working directory
;
working_directory= df.working_directory + '/tsar_' + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_directory= strcompress( working_directory, /remove_all)
file_mkdir, working_directory
;
; Copy TerrSAR-X directory to working directory.   Note the movement of TerraSAR directory to 
; and already processed directory must occur at a different level. It is not done in this program
;
if printflag then begin
 print, 'Copying ' + ff[0]             + '/'
 print, 'to      ' + working_directory + '/'
endif
file_copy,   ff[0], working_directory  + '/', /recursive
;
; Move files to already processed directory if move2dir set
;
res= file_search(df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, move2dir
if printflag then begin
  print, 'Moving ' + ff[0]
  print, 'to     ' + df.already_processed + '/'
endif
file_move,   ff[0], df.already_processed  + '/', /overwrite, /ALLOW_SAME
;
; Find the TSAR*.xml file and the base_directory for processing, ff= full path to product.xml file
; 
;

time1 = dt_tm_tojs(systime(/utc))

if printflag then print, 'Time to copy files to working directory='+ string (time1-time0) +' s'

;
; Locating leader file
;
data_set=    0
data_setmax= 0
while data_set le data_setmax do begin
  if data_set eq 0 then begin
    leaderfile1=  file_search( working_directory,'TSX*.xml' , count=count1)
    leaderfile2=  file_search( working_directory,'TDX1*.xml', count=count2)
    if (count1 + count2) eq 0 then begin
      print, 'Cannot find TSAR or TDX1 leaderfile.'
      print, 'Exiting ans_read_tsar program.'
      error=-1
      return
    endif
    if count1 gt 0 and count2 eq 0 then leaderfile=leaderfile1
    if count1 eq 0 and count2 gt 0 then leaderfile=leaderfile2
    if count1 gt 0 and count2 gt 0 then leaderfile=[leaderfile1, leaderfile2]
    leaderfile=  leaderfile[0]
    sleaderfile= getwrd(leaderfile, 0, /last, del='/')
    pos= strpos(leaderfile, sleaderfile)
    if pos eq 0 then begin
      base_directory='./'
    endif else begin
      base_directory= strmid(leaderfile, 0, pos)
    endelse
    ;
    ; Read in base ghead information into the ghead struture.
    ;
    tsar_read_gheader,  base_directory+sleaderfile, ghead, quiet=quiet, error=error
    if error lt 0 then begin
       error=-1
       print, 'Cannot read file ' + base_directory+sleaderfile
       return
    endif   
    plen=2
    for i=0, 3 do begin
      length=strlen(ghead.pollayer[i])       
      if length  eq 0 and plen eq 2 then data_setmax=i-1
      plen= length
    endfor
    if printflag then print, 'data_setmax=' + string(data_setmax)
  endif else begin
    ; 
    ; For data_set=0, the lf structure is filled later.
    ;
    lf.polarization=lf.pollayer[data_set]
    base_directory=lf.base_directory
  endelse
  ;
  ; Fill information into lf structure, Read mapping grid information
  ; and read geo reference dagta
  ;
  tsar_fillin_lf,  ghead,  data_set ,df.ave_pixel, df.ave_line, lf, quiet=quiet
  lf.nrcs_slope=  df.tsar_slope
  lf.nrcs_bias=   df.tsar_bias
  error=1
  tsar_read_mapping_grid, base_directory+ghead.mapping_gridfile[0], ghead, t1, tau1, i, j, quiet=quiet, error=error
  if error lt 0 then begin
     error=-1
     print, 'Cannot read file ' + base_directory+ghead.mapping_gridfile[0]
     return
  endif 
  error=1
  tsar_read_georef, base_directory+ghead.georeffile[0], lf, incidence, latitude, longitude, t2, tau2, quiet=quiet, error=error
  if error lt 0 then begin
     error=-1
     print, 'Cannot read file ' + base_directory+ghead.georeffile[0]
     return
  endif 
  ; Add to struture
  if n_elements(jsstart) eq 0 then jsstart=dt_tm_tojs(systime(/utc))
  temp_lf= create_struct(lf, 'answrs_version', answrs_version, 'jsstart', jsstart)
  lf=      0
  lf=      temp_lf
  ;
  ; Change sign of "t2" if ascending pass
  ;
  if strupcase(lf.orbitdirection) ne 'DESCENDING' and printflag then begin 
     if printflag then print, 'Ascending pass. Shifting the t2 variable'
     t2=-t2
  endif
  max_longitude=  max(longitude)
  min_longitude=  min(longitude)
  lf.lngswitchflag=  0
  if max_longitude gt 180 then begin 
    ; The basis in the original data is 0-360
    lf.lngswitchflag=  1
   endif else begin
     ;
     ; The basis is -180 to 180 then check if the data cross
     ; the International data line
     ;
     if max_longitude gt 120 and min_longitude lt -120 then begin
       ; The data cross the International Date Line and we must shift
       ; longitude basis
       lf.lngswitchflag=  1
       w=where(longitude lt 0, count)
       if count gt 0 then longitude(w)=360.0+longitude(w)
     endif
  endelse
  ;
  ; Perform longitude latitude fits
  ;
  temp_lf=  create_struct( lf, 'deg'        , df.deg        , $
                               'ave_pixel'  , df.ave_pixel  , $
                               'ave_line'   , df.ave_line     )
  lf=               0
  lf=               temp_lf      
  tsar_lonlatinc_fit, i, j, t1, tau1, t2, tau2, longitude, latitude, incidence, lf, quiet=quiet
  
  lf.rlook= float(ghead.headingangle) +90
  
  lf.layer=         data_set
  lf.polarization=  lf.pollayer[data_set]
  name_dir=   getwrd(input_dir, 0, /last, del='/')
  if data_set eq 0 then begin
    lf.base_directory=base_directory
    ;
    ; Create file name root for output files.
    ; 
    case strupcase(lf.month) of
    'JAN': month=1
    'FEB': month=2
    'MAR': month=3
    'APR': month=4
    'MAY': month=5
    'JUN': month=6
    'JUL': month=7
    'AUG': month=8
    'SEP': month=9
    'OCT': month=10
    'NOV': month=11
    'DEC': month=12
    ELSE:  month=1
    endcase
    ;
    ; compute center geographic position
    ;
    ic= lf.nx1 / 2
    jc= lf.ny1 / 2                                           
    lf.center_lon=  ans_poly_cal( double(ic), double(jc), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
    lf.center_lat=  ans_poly_cal( double(ic), double(jc), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
    center_lat= string(abs(lf.center_lat), format='(F6.2)')
    if lf.center_lat ge 0 then begin
      center_lat= center_lat + 'N'
    endif else begin
      center_lat= center_lat + 'S'
    endelse
    center_lon= string(abs(lf.center_lon), format='(F6.2)')
    if lf.center_lon ge 0 then begin
      center_lon= center_lon + 'E'
    endif else begin
     center_lon= center_lon + 'W'
    endelse
    timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
    lf.month                               + ' ' + $
    string( lf.day,       format='(I2.2)') + ' ' + $
    string( lf.hour,      format='(I2.2)') + ':' + $
    string( lf.minute,    format='(I2.2)') + ':' + $
    string( lf.second,    format='(I2.2)')
    js=            string( dt_tm_tojs(timestr), format='(I10.10)' ) 
    output_prefix = getwrd(sleaderfile, 0, del='_')
    output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
    output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
    output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
    output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
    output_prefix= output_prefix + '_' + js
    output_prefix= output_prefix + '_' + string( center_lon)
    output_prefix= output_prefix + '_' + string( center_lat)
    output_prefix= strcompress( output_prefix, /remove_all )
    if printflag then print, 'Computed output prefix for SIO files:' + output_prefix
    time2= dt_tm_tojs(systime(/utc))
  endif
  tsar_r_look, lf, quiet=quiet
  error=1
  tsar_read_image, base_directory+lf.imagedatafiles[data_set], lf, image, quiet=quiet, error=error
  if error lt 0 then begin
    error=-1
    print, 'Cannot read file ' + base_directory+lf.imagedatafiles[data_set]
    return
  endif
  if strupcase(lf.orbitdirection) ne 'DESCENDING' then image=reverse(image,2)
  tsar_dn2sigma, image, lf, quiet=quiet
  ; Adjust NRCS if necessary
  if lf.nrcs_slope ne 1 or lf.nrcs_bias ne 0 then begin
    image=  image * lf.nrcs_slope + lf.nrcs_bias
  endif
  output_prefix2= output_prefix + '_' + strupcase(lf.polarization)+ '_X'
  if data_set eq 0 then begin
    outputroot= df.sio_output_dir + '/'+ output_prefix2
  endif else begin
    outputroot= [outputroot, df.sio_output_dir + '/'+ output_prefix2]
  endelse
  scaling= 1.0
  outstr=    string(jsstart, format='(I10)') + ' '+ dt_tm_fromjs(jsstart) + ' ' + getwrd(lf.name_dir, 0, /last, del='/')  + ' Processing starts'
  ;
  ; Check for existence of log_dir
  ;
  res= file_search(df.log_dir, count=count)
  if count gt 0 then begin
    logfile= df.log_dir + '/' + output_prefix2 + '.log'
    close, 1  & openw, 1, logfile & close, 1
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
  endif
  print, outstr
  ;
  ; Check and see if a lock file exists
  ;
  lockfile=    df.sio_output_dir + '/'+ output_prefix2 + '*.lock'
  ;
  res= file_search( lockfile, count=count)
  if count gt 0 then begin
     print, 'Lockfile:  ' + lockfile + ' exists'
     error=-1
     return
  endif
  ;
  ; Write out SIO data file
  ;
  if strupcase(df.twobyte) eq 'Y' then begin
     ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, /twobyte, quiet=quiet
  endif else begin
     ans_write_nrcs_sio, image, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', scaling, quiet=quiet
  endelse
  time4= dt_tm_tojs(systime(/utc))
  outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  res= file_search(df.log_dir, count=count)
  if count gt 0 then begin
    logfile= df.log_dir + '/' + output_prefix2 + '.log'
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    print, ' '
    print, ' '
    close, 1
    print, outstr
  endif
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for Radarsat-2 data
  ;
  tsar_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
  time2= dt_tm_tojs(systime(/utc))
  outstr  = string(time2, format='(I10)') + ' '+ dt_tm_fromjs(time2) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  res= file_search(df.log_dir, count=count)
  if count gt 0 then begin
    logfile= df.log_dir + '/' + output_prefix2 + '.log'
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
  endif
  print, outstr
  ;
  ; Compute land mask and write out file. 
  ;
  outputfile=  df.sio_output_dir + '/'+ output_prefix2 + '_mask.dat'
  ;
  ; Only compute land mask for first of images
  ;
  if data_set eq 0 then begin
     ans_write_land_mask, lf.lon_coef, lf.lon_xexp, lf.lon_yexp, lf.lat_coef, lf.lat_xexp, lf.lat_yexp, lf.nx1, lf.ny1, $
                          lf.range_pixel   * lf.ave_pixel,          $
                          lf.azimuth_pixel * lf.ave_line,           $
                          outputfile,                               $
                          quiet       = quiet                    ,  $
                          gshhs_dir   = df.gshhs_dir             ,  $
                          pixel_size  = df.pixel_size_mask
     oldfile=outputfile
  endif else begin
     futil, oldfile, to=outputfile, /copy
  endelse
  time2= dt_tm_tojs(systime(/utc))
  outstr  = string(time2, format='(I10)') + ' '+ dt_tm_fromjs(time2) + ' ' + outputfile
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  res= file_search(df.log_dir, count=count)
  if count gt 0 then begin
    logfile= df.log_dir + '/' + output_prefix2 + '.log'
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
  endif
  print, outstr
  ;
  ; Increment data set
  data_set= data_set + 1
endwhile
;
; Delete working directory
;
if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive, quiet=quiet


end
