;+---------------------------
; NAME: ANS_READ_UAVSAR
; PURPOSE:
;    The purpose of this program is to read the UAVSAR data files
; CALLING SEQUENCE:   
;    ans_read_uavsar, deffile, outputroot
; INPUT:
;    Binary files with image and land mask data and an ASCII file with ancillary data. The 
;    convention for file names is of the form:
;        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_image.dat (image file)
;        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_ancil.txt (ancillary data file)
;        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_mask.dat  (land mask)
;    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization, frequency band, 
;    file type and extension. The data
;    files include the SIO 20-byte header.  

; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     ancil        = Input specific *.ann file
;     outputroot   = returned outputroot
;     /help        = Print out help
;     error        = <0 is bad
;                  = -1 Return from help
;  2019 Dec 12 Frank Monaldo. Creation
;  2020 Jan 24 Frank Monaldo. Allow the specification of an ancillary data file
; Copyright (C) 2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_uavsar, deffile, outputroot   , $
                     ancil        = ancil    , $
                     quiet        = quiet    , $
                     help         = help     , $
                     error        = error
                    
error=     0

if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_UAVSAR
  print, ' PURPOSE:
  print, '    The purpose of this program is to read the UAVSAR data files
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_uavsar, deffile, outputroot
  print, ' INPUT:
  print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The 
  print, '    convention for file names is of the form:
  print, '        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_image.dat (image file)
  print, '        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_ancil.txt (ancillary data file)
  print, '        UAVSAR_JPL_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HHHH_C_mask.dat  (land mask)
  print, '    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
  print, '    second, julian seconds, center longitude, center latitude, polarization, frequency band, 
  print, '    file type and extension. The data
  print, '    files include the SIO 20-byte header.  
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     ancil        = Input specific *.ann file
  print, '     outputroot   = returned outputroot
  print, '     /help        = Print out help
  print, '     error        = <0 is bad
  print, '                  = -1 Return from help
  error= -1
  return
endif
;
; Parcing input parameters
;
error     = 1
printflag = 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

time0 = dt_tm_tojs(systime(/utc))
;
; Read defaults file
;
ans_read_defaults, deffile, df, quiet=quiet, error=error
if error lt 0 then begin
  error = -2
  if printflag then begin
    print, 'Cannot read defaults
    print, 'Exiting ans_read_uavsar.pro 
  endif
  return
endif

if n_elements(df.idlans_dir) lt 1 then begin 
  print, 'idlans_dir not specified.'
  print, 'Exiting ans_read_uavsar.pro 
  error= -2
  return
endif
if n_elements(df.log_dir)    lt 1 then begin 
  print, 'log_dir not specified.'
  print, 'Exiting ans_read_uavsar.pro 
  error= -2
  return
endif


; Determine if ancillary data file exists
; 

input_dir = tag_value(df, 'uavsar_dir' )
if keyword_set(ancil) then begin
  ff= ancil 
  if file_test(ff) then count = 1
  ancil=''
endif else begin
  ff= file_search( input_dir, '*.ann', count=count)
endelse
if count eq 0 then begin
  print, 'Could not find UAVSAR directory: ' + input_dir
  print, 'Exiting ans_read_uavsar '
  error= -3
  return
endif

;
; Create a new working directory
;
working_directory = df.working_directory + '/uavsar_' 
working_directory = working_directory    + dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-0d$-h$-m$-s$' ) 
working_directory = working_directory    + '-' +  string(randomu(seed)*100000000, format='(I9)') + '/'
working_directory = strcompress( working_directory, /remove_all)
working_directory = stress( working_directory, 'R', 0, '//', '/',ned)
file_mkdir, working_directory
;
; Copy UAVSAR files to working directory.   Note the movement of UAVSAR directory to 
; and already processed directory must occur at a different level. It is not done in this program
; I must read this file to get the file names
;

if printflag then print, 'Reading ancillary data file : ' + ff[0]

ans_read_uavsar_ancil, ff[0], ancil, /quiet, error=error
if error lt 0 then begin
  error = -4
  if printflag then begin
    print, 'Cannot read ancillary data file : ' + ff[0]
    print, 'Exiting ans_read_uavsar'
  endif
endif

if printflag then begin
  print, 'Copying files listed in the ancillary data file : ' 
  print, '       ' + ff[0]
  print, ' to  : '  
  print, '       ' + working_directory 
endif
;
destination = stress(  working_directory  + '/' ,'R' , 0, '//', '/', ned )
origin      = stress(  input_dir          + '/' ,'R' , 0, '//', '/', ned )  
if printflag then print, 'Copying ' + ff[0]   + ' to ' + destination
filebreak,  ff[0], file = ancil_file
file_copy,  ff[0]   , destination

if tag_value(ancil,'slcHH') ne '' then begin
  if file_test(origin  + ancil.slcHH  ) then begin
    if printflag then print, 'Copying ' + origin  + string(ancil.slcHH, format='(A-60)' )  + ' to ' + destination
    file_copy, origin  + ancil.slcHH   , destination  
  endif  
endif     
;    
if tag_value(ancil, 'slcHV') ne  '' then begin 
  if file_test(origin  + ancil.slcHV  ) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.slcHV, format='(A-60)' )   + ' to ' + destination    
    file_copy, origin  + ancil.slcHV   , destination     
  endif 
endif  
;
if tag_value(ancil, 'slcVH') ne  '' then begin 
  if file_test(origin  + ancil.slcVH  ) then begin
    if printflag then print, 'Copying ' + origin  + string( ancil.slcVH, format='(A-60)' )     + ' to ' + destination 
    file_copy, origin  + ancil.slcVH   , destination   
  endif 
endif                  
;
if tag_value(ancil, 'slcVV') ne  '' then begin 
  if file_test(origin  + ancil.slcVV  ) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.slcVV, format='(A-60)' )    + ' to ' + destination 
    file_copy, origin  + ancil.slcVV   , destination  
  endif      
endif   
;
if tag_value(ancil, 'mlcHHHH') ne  '' then begin               
  if file_test(origin  + ancil.mlcHHHH) then begin      
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcHHHH, format='(A-60)' )   + ' to ' + destination  
    file_copy, origin  + ancil.mlcHHHH , destination 
  endif    
endif
;
if tag_value(ancil, 'mlcHVHV') ne  '' then begin              
  if file_test(origin  + ancil.mlcHVHV) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcHVHV, format='(A-60)' )  + ' to ' + destination    
    file_copy, origin  + ancil.mlcHVHV , destination        
  endif  
endif
;
if tag_value(ancil, 'mlcVVVV') ne  '' then begin                 
  if file_test(origin  + ancil.mlcVVVV) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcVVVV, format='(A-60)' )  + ' to ' + destination 
    file_copy, origin  + ancil.mlcVVVV , destination
  endif  
endif
;
if tag_value(ancil, 'mlcHHHV') ne  '' then begin                        
  if file_test(origin  + ancil.mlcHHHV) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcHHHV, format='(A-60)' )  + ' to ' + destination  
    file_copy, origin  + ancil.mlcHHHV , destination

  endif 
endif
;
if tag_value(ancil, 'mlcHHVV') ne  '' then begin                                              
  if file_test(origin  + ancil.mlcHHVV) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcHHVV, format='(A-60)' )  + ' to ' + destination  
    file_copy, origin  + ancil.mlcHHVV , destination     
  endif   
endif
;
if tag_value(ancil, 'mlcHVVV') ne  '' then begin                 
  if file_test(origin  + ancil.mlcHVVV) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.mlcHVVV, format='(A-60)' )  + ' to ' + destination   
    file_copy, origin  + ancil.mlcHVVV , destination  
  endif  
endif
;
if tag_value(ancil, 'grdHHHH') ne  '' then begin                                                     
  if file_test(origin  + ancil.grdHHHH) then begin
    if printflag then print, 'Copying ' + origin  +  string(ancil.grdHHHH, format='(A-60)' )  + ' to ' + destination
    file_copy, origin  + ancil.grdHHHH , destination  
  endif 
endif
; 
if tag_value(ancil, 'grdHVHV') ne  '' then begin                    
  if file_test(input_dir + '/' + ancil.grdHVHV) then begin
    if printflag then print, 'Copying ' + origin + string(ancil.grdHVHV, format='(A-60)' )  + ' to ' + destination   
    file_copy, origin + ancil.grdHVHV , destination     
  endif  
endif
;                 
if tag_value(ancil, 'grdVVVV') ne  '' then begin 
  if file_test(input_dir + '/' + ancil.grdVVVV) then begin
    if printflag then print, 'Copying ' + origin  + string(ancil.grdVVVV, format='(A-60)' )  + ' to ' + destination   
    file_copy, origin + ancil.grdVVVV ,  destination  
  endif  
endif
;
if tag_value(ancil, 'grdHHHV') ne  '' then begin                      
  if file_test(input_dir + '/' + ancil.grdHHHV) then begin
    if printflag then print, 'Copying ' + origin + string(ancil.grdHHHV, format='(A-60)' )  + ' to ' + destination
    file_copy, origin + ancil.grdHHHV ,destination  
  endif
endif
;
if tag_value(ancil, 'grdHHVV') ne  '' then begin                       
  if file_test(input_dir + '/' + ancil.grdHHVV) then begin
    if printflag then print, 'Copying ' + origin + string(ancil.grdHHVV, format='(A-60)' )  + ' to ' + destination 
    file_copy, origin  + ancil.grdHHVV , destination     
  endif    
endif
;
if tag_value(ancil, 'grdHVVV') ne  '' then begin               
  if file_test(origin  + ancil.grdHVVV) then begin
    if printflag then print, 'Copying ' + origin  + string(ancil.grdHVVV, format='(A-60)' )  + ' to ' + destination  
    file_copy, origin  + ancil.grdHVVV ,destination
  endif  
endif
;
if tag_value(ancil, 'hgt') ne  '' then begin                       
  if file_test(origin  + ancil.hgt    ) then begin
    if printflag then print, 'Copying ' + origin  + string(ancil.hgt, format='(A-60)' )     + ' to ' + destination
    file_copy, origin  + ancil.hgt     , destination
  endif 
endif                  
;
if tag_value(ancil, 'inc') ne  '' then begin                                    
  if file_test(origin  + ancil.inc    ) then begin
     if printflag then print, 'Copying ' + origin  + string(ancil.inc, format='(A-60)' )     + ' to ' + destination    
     file_copy, origin  + ancil.inc     , destination              
  endif 
endif
;
if tag_value(ancil, 'slope') ne  '' then begin            
  if file_test(origin  + ancil.slope  ) then begin
    if printflag then print, 'Copying ' + origin  + string(ancil.slope, format='(A-60)' )    + ' to ' +  destination  
    file_copy, origin  + ancil.slope   , destination
  endif
endif

time1 = dt_tm_tojs(systime(/utc))

if printflag then print, 'Time to copy files to working directory ='+ string (time1-time0) +' s'
;
; Move files to already processed directory if move2dir set
;
; move2dir= file_search(df.already_processed, /test_directory, count=count)
; if count eq 0 then file_mkdir, move2dir
; if printflag then begin
;   print, 'Copying files associated with the ancillary data file : ' +ff[0]
;   print, ' to ' + working_directory + '/'
; endif

; if file_test(origin  + ancil.slcHH  ) then file_move, origin  + ancil.slcHH   , working_directory  + '/'                         
; if file_test(origin  + ancil.slcHV  ) then file_move, origin  + ancil.slcHV   , working_directory  + '/'                         
; if file_test(origin  + ancil.slcVH  ) then file_move, origin  + ancil.slcVH   , working_directory  + '/'                         
; if file_test(origin  + ancil.slcVV  ) then file_move, origin  + ancil.slcVV   , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcHHHH) then file_move, origin  + ancil.mlcHHHH , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcHVHV) then file_move, origin  + ancil.mlcHVHV , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcVVVV) then file_move, origin  + ancil.mlcVVVV , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcHHHV) then file_move, origin  + ancil.mlcHHHV , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcHHVV) then file_move, origin  + ancil.mlcHHVV , working_directory  + '/'                         
; if file_test(origin  + ancil.mlcHVVV) then file_move, origin  + ancil.mlcHVVV , working_directory  + '/'                                                       
; if file_test(origin  + ancil.grdHHHH) then file_move, origin  + ancil.grdHHHH , working_directory  + '/'                         
; if file_test(origin  + ancil.grdHVHV) then file_move, origin  + ancil.grdHVHV , working_directory  + '/'                         
; if file_test(origin  + ancil.grdVVVV) then file_move, origin  + ancil.grdVVVV , working_directory  + '/'                         
; if file_test(origin  + ancil.grdHHHV) then file_move, origin  + ancil.grdHHHV , working_directory  + '/'                         
; if file_test(origin  + ancil.grdHHVV) then file_move, origin  + ancil.grdHHVV , working_directory  + '/'                         
; if file_test(origin  + ancil.grdHVVV) then file_move, origin  + ancil.grdHVVV , working_directory  + '/'                         
; if file_test(origin  + ancil.hgt    ) then file_move, origin  + ancil.hgt     , working_directory  + '/'                                                         
; if file_test(origin  + ancil.inc    ) then file_move, origin  + ancil.inc     , working_directory  + '/'                         
; if file_test(origin  + ancil.slope  ) then file_move, origin  + ancil.slope   , working_directory  + '/'
;
; Only real (not complex) data sets
;
polarization = [ 'HH', 'HV', 'VV']
hgt_file     = tag_value( ancil, 'hgt' )
inc_file     = tag_value( ancil, 'inc' )
if printflag then begin
   print, 'Filling in info structure using file : ' + destination + ancil_file
endif
uavsar_fillin_info, destination + ancil_file, deffile, info, quiet=quiet, error = error
if error lt 0 then begin
  if printflag then begin
    print, 'Cannot read ' + destination + ancil_file
    print, 'Exiting ans_read_uavsar.pro'
  endif
  error=-1
endif
outputroot = strarr(3)
for d = 0, 2 do begin ; Loop through polarizations
  ; 
  ; Fill in the necessary data into and info structure to be eventually written into the *.ancil file.
  ;
  info.polarization = polarization[d]
  if d eq 0 then begin
    ;
    ; Create file name root for output files.
    ;
    ; compute center geographic position
    ;
    center_lat= string(abs(info.center_lat), format='(F05.2)')
    if info.center_lat ge 0 then begin
      center_lat= center_lat + 'N'
    endif else begin
      center_lat= center_lat + 'S'
    endelse
    center_lon= string(abs(info.center_lon), format='(F06.2)')
    if info.center_lon ge 0 then begin
      center_lon= center_lon + 'E'
    endif else begin
     center_lon= center_lon + 'W'
    endelse
    js            = double(info.acq_date_time_js)
    output_prefix = info.platform_name
    output_prefix = output_prefix + '_' + strupcase( info.processing_facility)
    output_prefix = output_prefix + '_' + dt_tm_fromjs( js, format='Y$_0n$_0d$_h$_m$_s$')
    if js ge 0 then begin
      output_prefix = output_prefix + '_'  + string(      js, format='(I10.10)' )
    endif else begin
      output_prefix = output_prefix + '_-' + string( abs(js), format='(I10.10)' )
    endelse
    output_prefix = output_prefix + '_' + string( center_lon) + '_' + string( center_lat)
    output_prefix = strcompress( output_prefix, /remove_all )
    if printflag then print, 'Computed output prefix for SIO files:' + output_prefix
  endif
  ;
  ; Read radar image
  ;
  img = 1
  case d of
  0 : begin
      polflag = 0B
      ans_read_uavsar_ancil_img, destination + ancil_file, info2, grdHHHH = img, quiet = quiet 
      uavsar_fillin_info,  destination + ancil_file, deffile, info2, quiet=quiet, error = error
      output_prefix2= output_prefix + '_HH_L'
      end
  1 : begin
      polflag = 1B
      grdHVHV = 1
      ans_read_uavsar_ancil_img, destination + ancil_file, info2, grdHVHV = img, quiet = quiet
      uavsar_fillin_info,  destination + ancil_file, deffile, info2, quiet=quiet, error = error
      output_prefix2= output_prefix + '_HV_L'
      end
  2 : begin
      grdVVVV  = 1
      polflag = 0B
      ans_read_uavsar_ancil_img, destination + ancil_file, info2, grdVVVV = img, quiet = quiet
      uavsar_fillin_info,  destination + ancil_file, deffile, info2, quiet=quiet, error = error
      output_prefix2= output_prefix + '_VV_L'
      end
  endcase
  info2.polarization = polarization[d]
  case polarization[d] of
  'HH':   info2.flag_sar_band_polarization = '22'
  'VV':   info2.flag_sar_band_polarization = '21'
  'VH':   info2.flag_sar_band_polarization = '24'
  'HV':   info2.flag_sar_band_polarization = '23'
  ELSE:   info2.flag_sar_band_polarization  = '00'
  endcase
  ;
  ; Write out SIO data file
  ;
  
  if strupcase(df.twobyte) eq 'Y' then begin
       ans_write_nrcs_sio, img, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat',info.scaling, /twobyte, quiet=quiet
  endif else begin
       ans_write_nrcs_sio, img, df.sio_output_dir + '/'+ output_prefix2 + '_image.dat', info.scaling, quiet=quiet
  endelse
  time4   = dt_tm_tojs(systime(/utc))
  outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_image.dat'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  res= file_search(df.log_dir, count=count)
  if count gt 0 then begin
      logfile= df.log_dir + '/' + output_prefix2 + '.log'
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      print, ' '
      print, ' '
      close, 1
      print, outstr
  endif
  ;
  ; Write out SIO ancillary file format. This program is explicitly written for UAVSAR data
  ;
  uavsar_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', info2, quiet=quiet

  time4   = dt_tm_tojs(systime(/utc))
  outstr  = string(time4, format='(I10)') + ' '+ dt_tm_fromjs(time4) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
  outstr  = stress( outstr, 'R', 0, '//', '/', ned)
  res     = file_search(df.log_dir, count=count)
  if count gt 0 then begin
    logfile= df.log_dir + '/' + output_prefix2 + '.log'
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
     print, outstr
  endif
  outputroot[d]  = df.sio_output_dir + '/' + output_prefix2 
endfor  ; End d loop through images of different polarizations.
;
; Delete working directory
;
if printflag then print, 'Deleting ' + working_directory  
file_delete,  working_directory  + '/', /recursive, quiet=quiet

end
