;+---------------------------
; NAME: ANS_READ_UAVSAR_ANCIL
; PURPOSE:
;    The purpose of this program is to read the UAVSAR ancillary data and return
;    a structure with the ancillary values
; CALLING SEQUENCE:   
;    ans_read_uavsar_ancil, file, ancil
; INPUT:
;    file  = UAVSAR ancillary data file
; OUTPUT:          
;    ancil = Structure with ancillary data
; KEYWORDS:
;     /quiet   = Do not print intermediate results.
;     /help    = Print out help
;     error    = <0 is bad
;                -1 return from help
;                -2 No ancillary data file
; MODIFICATION HISTORY
;  2019 Dec 09 Frank Monaldo. Creation
;  2020 Jan 06 Frank Monaldo. Trim space in variable
; Copyright (C) 2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro  ans_read_uavsar_ancil, file,     ancil , $
                            quiet   = quiet , $
                            help    = help  , $
                            error   = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_UAVSAR_ANCIL
  print, ' PURPOSE:
  print, '    The purpose of this program is to read the UAVSAR ancillary data and return
  print, '    a structure with the ancillary values
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_uavsar_ancil, file, ancil
  print, ' INPUT:
  print, '    file  = UAVSAR ancillary data file
  print, ' OUTPUT:          
  print, '    ancil = Structure with ancillary data
  print, ' KEYWORDS:
  print, '     /quiet   = Do not print intermediate results.
  print, '     /help    = Print out help
  print, '     error    = <0 is bad
  print, '                -1 return from help
  print, '                -2 No ancillary data file
  error= -1
  return
endif
;
; Parcing input parameters
;
error     = 1
printflag = 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if file_test(file) eq 0 then begin
  error = -2
  if printflag then begin
    print, 'Cannot find file : ' + file
    print, 'Exiting ans_read_uavsar_ancil
  endif
  return
endif else begin
  s = getfile(file)
endelse



    
n = n_elements( s )

ancil = {ancil  : 'UAVSAR ancillary data'}

for i = 0, n - 1  do begin       
  ;
  ; Proceed if the line does not begin with ';', a remark line
  ;
  if strpos( s[i], ';') ne 0  and strlen( s[i] ) gt 0 then begin
     variable_name = getwrd( s[i], 0, del = '=')
     variable_name = strcompress( variable_name)
     variable_name = stress(variable_name , 'D', 0, ' (&)',      x, ned)  ; Remove space (&)   
     variable_name = stress(variable_name , 'R', 0, ' '  ,     '_', ned)  ; Replace space with '_'
     variable_name = stress(variable_name , 'R', 0, '.'  ,     '_', ned)  ; Replace . with '_'
     variable_name = stress(variable_name , 'D', 0, ','  ,       x, ned)  ; Remove ,    
     variable_name = stress(variable_name , 'D', 0, ')'  ,       x, ned)  ; Remove )
     variable_name = stress(variable_name , 'D', 0, '('  ,       x, ned)  ; Remove (
     variable_name = stress(variable_name , 'R', 0, '/'  , '_per_', ned)  ; Replace / with _per_
     variable_name = stress(variable_name , 'D', 0, '-'  ,       x, ned)  ; Remove -
     variable_name = strcompress( variable_name, /remove_all)
     value         = getwrd( s[i], 1, del = '=')
     value         = strcompress(value, /remove_all)
     if strpos( s[i], ';') gt 1 then begin
        last_byte = strpos( value, ';')
        value = strmid( value, 0, last_byte)
     endif
     if printflag then print, string( variable_name, format='(A50)' ) + ' = ' + value
     ;
     ; test to avoid duplicates
     ;
     if tag_test(ancil, variable_name ) eq 0 then ancil =create_struct( ancil, variable_name, value)
  endif
  ;

endfor

end
