;+---------------------------
; NAME: ANS_READ_UAVSAR_GRD
; PURPOSE:
;    The purpose of this program is to read the UAVSAR ground projected data
; CALLING SEQUENCE:   
;    ans_read_uavsar_grd, file, grd, nx, ny
; INPUT:
;    file  = UAVSAR *.grd data file
;    nx    = columns
;    ny    = rows
; OUTPUT:          
;    grd   = radar values
; KEYWORDS:
;     /quiet   = Do not print intermediate results.
;     /help    = Print out help
;     error    = <0 is bad
;                -1 return from help
;                -2 No grd file
;                -3 Byte size not four or eight
;  2019 Dec 10 Frank Monaldo. Creation
;  2020 Jan 22 Frank Monaldo. Flipping image vertically. 
; Copyright (C) 2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro  ans_read_uavsar_grd, file, grd, nx, ny, $
                            quiet   = quiet , $
                            help    = help  , $
                            error   = error
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_UAVSAR_GRD
  print, ' PURPOSE:
  print, '    The purpose of this program is to read the UAVSAR ground projected data
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_uavsar_grd, file, grd, nx, ny
  print, ' INPUT:
  print, '    file  = UAVSAR *.grd data file
  print, '    nx    = columns
  print, '    ny    = rows
  print, ' OUTPUT:          
  print, '    grd   = radar values
  print, ' KEYWORDS:
  print, '     /quiet   = Do not print intermediate results.
  print, '     /help    = Print out help
  print, '     error    = <0 is bad
  print, '                -1 return from help
  print, '                -2 No grd file
  print, '                -3 Byte size not four or eight
  error= -1
  return
endif
;
; Parcing input parameters
;
error     = 1
printflag = 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0


if file_test(file) eq 0 then begin
  error = -2
  if printflag then begin
    print, 'Cannot find file : ' + file
    print, 'Exiting ans_read_uavsar_grd'
  endif
  return
endif 

f                = file_info(file)
pixel_byte_size  = f.size / long(nx) / long(ny)


if printflag then print, 'Start reading file : ' + file

close, 1
case pixel_byte_size of
4    : begin
         grd  = fltarr(nx, ny)
         openr,  1, file
         readu,  1, grd
       end
8    : begin
         grd  = complexarr(nx, ny)
         ii1  = lindgen(nx) * 2L
         ii2  = ii1 + 1
         ln   = fltarr( 2L * nx )
         openr,  1, file
         for j = 0L, ny - 1 do begin
            readu,  1, ln
            grd[ *, j] = complex( ln[ii1] , ln[ii2] )
         endfor
       end
ELSE : begin
         error = -3
         if printflag then begin
           print, 'pixel_byte_size ne 4 and 8'
           print, 'Exiting ans_read_uavsar_grd...'
         endif
         return
       end
endcase
close, 1

if printflag then print, 'End   reading file : ' + file
grd  = reverse( grd, 2 )
end
