;+---------------------------
; NAME: ANS_READ_UAVSAR_SLOPE
; PURPOSE:
;    The purpose of this program is to read the UAVSAR slope file
; CALLING SEQUENCE:   
;    ans_read_uavsar_slope, file, slope_east, slope_north, nx, ny
; INPUT:
;    file        = UAVSAR slope data file
;    nx          = columns
;    ny          = rows
; OUTPUT:          
;    slope_east  = Slope angle in radians in east
;    slope_north = Slope angle in radians in north 
; KEYWORDS:
;     /quiet     = Do not print intermediate results.
;     /help      = Print out help
;     error      = <0 is bad
;                  -1 return from help
;                  -2 No slope file
;  2019 Dec 10 Frank Monaldo. Creation
;  2020 Jan 22 Frank Monaldo. Flipping image vertically. 
; Copyright (C) 2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.   
;-
;

pro ans_read_uavsar_slope, file, slope_east, slope_north, nx, ny , $
                            quiet   = quiet    , $
                            help    = help     , $
                            error   = error
                    
error=     0

if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_READ_UAVSAR_SLOPE
  print, ' PURPOSE:
  print, '    The purpose of this program is to read the UAVSAR slope file
  print, ' CALLING SEQUENCE:   
  print, '    ans_read_uavsar_slope, file, slope_east, slope_north, nx, ny
  print, ' INPUT:
  print, '    file        = UAVSAR slope data file
  print, '    nx          = columns
  print, '    ny          = rows
  print, ' OUTPUT:          
  print, '    slope_east  = Slope angle in radians in east
  print, '    slope_north = Slope angle in radians in north 
  print, ' KEYWORDS:
  print, '     /quiet     = Do not print intermediate results.
  print, '     /help      = Print out help
  print, '     error      = <0 is bad
  print, '                  -1 return from help
  print, '                  -2 No slope file
  error= -1
  return
endif
;
; Parcing input parameters
;
error     = 1
printflag = 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0


if file_test(file) eq 0 then begin
  error = -2
  if printflag then begin
    print, 'Cannot find file : ' + file
    print, 'Exiting ans_read_uavsar_slope'
  endif
  return
endif 

slope_east  = fltarr( nx, ny )
slope_north = fltarr( nx, ny )
ln          = fltarr( 2L * nx )

if printflag then print, 'Start reading file : ' + file

close,  1
openr,  1, file
for j = 0L, ny - 1 do begin
  ii1 = lindgen(nx) * 2L
  ii2 = ii1 + 1
  readu,  1, ln
  slope_east[*,j]   = ln[ ii1  ]
  slope_north[*,j]  = ln[ ii2 ]
endfor
close,  1

if printflag then print, 'End   reading file : ' + file
slope_east  = reverse(slope_east ,2)
slope_north = reverse(slope_north,2)
end
