;----------------------------------------------------------
;+
; NAME: ANS_READ_WIND_AWIPS1_CDF'
; PURPOSE:'
;   Read AWIPS SAR wind retrieval data.
; CALLING SEQUENCE:'
;   ans_read_wind_awips1_cdf, file, info, sar_wind 
; INPUT:'
;   file       :   AWIPS-1 netCDF file of SAR wind
; OUTPUT:
;   info       :  Structure with ancillary information
;   sar_wind   :  2D SAR wind array
; KEYWORDS:
;   /quiet     =  Do not print intermediate results'
;   /help      =  List help.
;   error      =  <0 is bad
; MOFICATION HISTORY
;   2013 Jul 26 Frank Monaldo.  
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-


pro  ans_read_wind_awips1_cdf, file, info, sar_wind , $
                          quiet= quiet, help=help, error=error

error =  0
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, 'NAME: ANS_READ_WIND_AWIPS1_CDF'
  print, 'PURPOSE:'
  print, '  Read AWIPS SAR wind retrieval data.
  print, 'CALLING SEQUENCE:'
  print, '  ans_read_wind_awips1_cdf, file, info, sar_wind 
  print, 'INPUT:'
  print, '  file       :   AWIPS-1 netCDF file of SAR wind
  print, 'OUTPUT:
  print, '  info       :  Structure with ancillary information
  print, '  sar_wind   :  2D SAR wind array
  print, 'KEYWORDS:
  print, '  /quiet     =  Do not print intermediate results'
  print, '  /help      =  List help.
  print, '  error      =  <0 is bad
  error        =  -1
  return                
endif
;
; Parcing input parameters  
;
printflag= 1
if keyword_set(quiet) then printflag=0


if not file_test(file) then begin
  print, 'Cannot find file '+ file + ' .'
  print, 'Exiting ans_read_wind_awipd1_cdf....'
  return
endif

id = NCDF_OPEN(file, /nowrite) 
;
; Global Attributes
;  
data_record_time = 'BLANK'
depictorName     = 'BLANK'
projIndex        = 'BLANK'
projName         = 'BLANK''
centralLon       = 0.00000
centralLat       = 0.00000
xMin             = 0.00000
xMax             = 0.00000
yMin             = 0.00000
yMax             = 0.00000
lat00            = 0.00000
lon00            = 0.00000
latNxNy          = 0.00000
lonNxNy          = 0.00000
rotation         = 0.00000
dxKm             = 0.00000
dyKm             = 0.00000
latDxDy          = 0.00000
lonDxDy          = 0.00000
source           = 'BLANK'

; Determine if global attributes are present

r_data_record_time = NCDF_ATTINQ( id, 'data_record_time' , /GLOBAL )
r_depictorName     = NCDF_ATTINQ( id, 'depictorName'     , /GLOBAL )
r_projIndex        = NCDF_ATTINQ( id, 'projIndex'        , /GLOBAL )
r_projName         = NCDF_ATTINQ( id, 'projName'         , /GLOBAL )
r_centralLon       = NCDF_ATTINQ( id, 'centralLon'       , /GLOBAL )
r_centralLat       = NCDF_ATTINQ( id, 'centralLat'       , /GLOBAL )
r_xMin             = NCDF_ATTINQ( id, 'xMin'             , /GLOBAL )
r_xMax             = NCDF_ATTINQ( id, 'xMax'             , /GLOBAL )
r_yMin             = NCDF_ATTINQ( id, 'yMin'             , /GLOBAL )
r_yMax             = NCDF_ATTINQ( id, 'yMax'             , /GLOBAL )
r_lat00            = NCDF_ATTINQ( id, 'lat00'            , /GLOBAL )
r_lon00            = NCDF_ATTINQ( id, 'lon00'            , /GLOBAL )
r_latNxNy          = NCDF_ATTINQ( id, 'latNxNy'          , /GLOBAL )
r_lonNxNy          = NCDF_ATTINQ( id, 'lonNxNy'          , /GLOBAL )
r_rotation         = NCDF_ATTINQ( id, 'rotation'         , /GLOBAL )
r_dxKm             = NCDF_ATTINQ( id, 'dxKm'             , /GLOBAL )
r_dyKm             = NCDF_ATTINQ( id, 'dyKm'             , /GLOBAL )
r_latDxDy          = NCDF_ATTINQ( id, 'latDxDy'          , /GLOBAL )
r_lonDxDy          = NCDF_ATTINQ( id, 'lonDxDy'          , /GLOBAL )
r_source           = NCDF_ATTINQ( id, 'source'           , /GLOBAL )

if printflag then begin
  if r_data_record_time.datatype  eq 'UNKNOWN' then print, 'Did not read attribute data_record_time'
  if r_depictorName.datatype      eq 'UNKNOWN' then print, 'Did not read attribute depictorName    '
  if r_projIndex.datatype         eq 'UNKNOWN' then print, 'Did not read attribute projIndex       '
  if r_projName.datatype          eq 'UNKNOWN' then print, 'Did not read attribute projName        '
  if r_centralLon.datatype        eq 'UNKNOWN' then print, 'Did not read attribute centralLon      '
  if r_centralLat.datatype        eq 'UNKNOWN' then print, 'Did not read attribute centralLat      '
  if r_xMin.datatype              eq 'UNKNOWN' then print, 'Did not read attribute xMin            '
  if r_xMax.datatype              eq 'UNKNOWN' then print, 'Did not read attribute xMax            '
  if r_yMin.datatype              eq 'UNKNOWN' then print, 'Did not read attribute yMin            '
  if r_yMax.datatype              eq 'UNKNOWN' then print, 'Did not read attribute yMax            '
  if r_lat00.datatype             eq 'UNKNOWN' then print, 'Did not read attribute lat00           '
  if r_lon00.datatype             eq 'UNKNOWN' then print, 'Did not read attribute lon00           '
  if r_latNxNy.datatype           eq 'UNKNOWN' then print, 'Did not read attribute latNxNy         '
  if r_lonNxNy.datatype           eq 'UNKNOWN' then print, 'Did not read attribute lonNxNy         '
  if r_rotation.datatype          eq 'UNKNOWN' then print, 'Did not read attribute rotation        '
  if r_dxKm.datatype              eq 'UNKNOWN' then print, 'Did not read attribute dxKm            '
  if r_dyKm.datatype              eq 'UNKNOWN' then print, 'Did not read attribute dyKm            '
  if r_latDxDy.datatype           eq 'UNKNOWN' then print, 'Did not read attribute latDxDy         '
  if r_lonDxDy.datatype           eq 'UNKNOWN' then print, 'Did not read attribute lonDxDy         '
  if r_source.datatype            eq 'UNKNOWN' then print, 'Did not read attribute source          '
endif
  

if r_data_record_time.datatype  ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'data_record_time' ,data_record_time 
if r_depictorName.datatype      ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'depictorName'     ,depictorName 
if r_projIndex.datatype         ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'projIndex'        ,projIndex    
if r_projName.datatype          ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'projName'         ,projName     
if r_centralLon.datatype        ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'centralLon'       ,centralLon   
if r_centralLat.datatype        ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'centralLat'       ,centralLat   
if r_xMin.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'xMin'             ,xMin         
if r_xMax.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'xMax'             ,xMax         
if r_yMin.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'yMin'             ,yMin         
if r_yMax.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'yMax'             ,yMax         
if r_lat00.datatype             ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'lat00'            ,lat00        
if r_lon00.datatype             ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'lon00'            ,lon00        
if r_latNxNy.datatype           ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'latNxNy'          ,latNxNy      
if r_lonNxNy.datatype           ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'lonNxNy'          ,lonNxNy      
if r_rotation.datatype          ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'rotation'         ,rotation     
if r_dxKm.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'dxKm'             ,dxKm         
if r_dyKm.datatype              ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'dyKm'             ,dyKm         
if r_latDxDy.datatype           ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'latDxDy'          ,latDxDy      
if r_lonDxDy.datatype           ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'lonDxDy'          ,lonDxDy      
if r_source.datatype            ne 'UNKNOWN' then NCDF_ATTGET, id, /GLOBAL, 'source'           ,source       

 
data_record_time  =  string( data_record_time  )    
depictorName      =  string( depictorName      )    
projIndex         =  string( projIndex         )    
projName          =  string( projName          )    
source            =  string( source            )    




if printflag then begin
  print, 'Global attributes.                '
  print, '------------------------------------'
  print, 'data_record_time  = ' , data_record_time
  print, 'depictorName      = ' , depictorName 
  print, 'projIndex         = ' , projIndex        
  print, 'projName          = ' , projName     
  print, 'centralLon        = ' , centralLon   
  print, 'centralLat        = ' , centralLat   
  print, 'xMin              = ' , xMin         
  print, 'xMax              = ' , xMax         
  print, 'yMin              = ' , yMin         
  print, 'yMax              = ' , yMax         
  print, 'lat00             = ' , lat00        
  print, 'lon00             = ' , lon00        
  print, 'latNxNy           = ' , latNxNy      
  print, 'lonNxNy           = ' , lonNxNy      
  print, 'rotation          = ' , rotation     
  print, 'dxKm              = ' , dxKm         
  print, 'dyKm              = ' , dyKm         
  print, 'latDxDy           = ' , latDxDy      
  print, 'lonDxDy           = ' , lonDxDy      
  print, 'source            = ' , source       
endif

;
; Fill in info structure
;
info= { data_record_time : data_record_time , $
        depictorName     : depictorName     , $
        projIndex        : projIndex        , $
        projName         : projName         , $'
        centralLon       : centralLon       , $
        centralLat       : centralLat       , $
        xMin             : xMin             , $
        xMax             : xMax             , $
        yMin             : yMin             , $
        yMax             : yMax             , $
        lat00            : lat00            , $
        lon00            : lon00            , $
        latNxNy          : latNxNy          , $
        lonNxNy          : lonNxNy          , $
        rotation         : rotation         , $
        dxKm             : dxKm             , $
        dyKm             : dyKm             , $
        latDxDy          : latDxDy          , $
        lonDxDy          : lonDxDy          , $
        source           : source           }
        


sar_windid=     NCDF_VARID( id, 'sar_wind'      ) 

NCDF_VARGET,  id, sar_windid,    sar_wind 



;

NCDF_CLOSE, id

if printflag then print, 'Finished reading: '+ file

end
