;-----------------------------------------------------
;+
; NAME: ANS_RSAT22SIO
; PURPOSE:
;   To convert Radarasat-2 data to SIO files (and Level-1 and KMZ if selected)
; CALLING SEQUENCE
;   ans_rsat22sio, deffile, sio_root
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
;                sio_kmz_dir, twobyte, rsat2_dir,deg, ave_line, and ave_pixel, log_dir,
;                silent
; OUTPUT
;   sio_root=    Root(s) of SIO data file names
;   SIO files
; KEYWORDS:
;   /quiet       No screen output
;   /help        Get help
;   error        <0 if bad 
;                -1 No Radarsat-2 data available in diretory
;                -2 Cannot read Radarsat-2 data
; MODIFICATION HISTORY:
;   2010 Dec 30 Frank Monaldo Change the architecture to separate out SIO generation
;                              by satellite
;   2011 Feb 05 Frank Monaldo. Add jsstart
;   2011 Feb 22 Frank Monaldo. Change .cdf extension to .nc
;   2011 Jun 23 Frank Monaldo. Corrected misspelling
;   2013 Aug 09 Frank Monaldo. Move ice mask generation to ans_sat2io
;   2014 Apr 15 Frank Monaldo. Add error catching if reading the image fails
;   2014 Oct 30 Chris Jackson. Narrowed the file search to RS* directories only
;   2018 Jul 12 Frank Monaldo. Narrowed the file search to R*  directories only
; Copyright (C) 2010-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_rsat22sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet=  printflag eq 0
sio_root=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print,  ' NAME: ANS_RSAT22SIO
  print,  ' PURPOSE:
  print,  '   To convert Radarasat-2 data to SIO files (and Level-1 and KMZ if selected)
  print,  ' CALLING SEQUENCE
  print,  '   ans_rsat22sio, deffile, sio_root
  print,  ' INPUT
  print,  '   deffile=     Full path to defaults file. This option will read a specified file
  print,  '                for the parameters: already_processsed, ncdf1_output_dir, working_directory,
  print,  '                sio_kmz_dir, twobyte, rsat2_dir,deg, ave_line, and ave_pixel, log_dir,
  print,  '                silent
  print,  ' OUTPUT
  print,  '   sio_root=    Root(s) of SIO data file names
  print,  '   SIO files
  print,  ' KEYWORDS:
  print,  '   /quiet       No screen output
  print,  '   /help        Get help
  print,  '   error        <0 if bad
  errpr = -1
  return
endif

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))


;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin converting RSAT2 to SIO files. ' + systime(/utc)
  print, ' '
endif

ans_read_defaults, deffile, df, quiet=quiet

if tag_test(df, 'silent') then begin
  if df.silent eq 'Y' then printflag=0
  if df.silent eq 'N' then printflag=1
endif
quiet= printflag eq 0
;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir         
res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir            
res= file_search( df.log_dir,         /test_directory,  count=count)
if count eq 0 then file_mkdir, df.log_dir 


;
;  
;---------------------------------------------------------------
;------------------- Start Radarsat-2 Conversion ---------------
;---------------------------------------------------------------


;
; Look for Radarsat-2 directories in base directory
;
ff= file_search( df.rsat2_dir +'/R*', /test_directory, count=count) & cc=count
if cc eq 0 then begin
  if printflag then print, 'Cannot find files in ' +  df.rsat2_dir +'/R*'
  if printflag then print, 'Exiting ans_rsat22Ssio'
  sio_root=''
  error = -1 
  return
endif

;
; Computing SIO files for all the Radarsat-2 data
;
if printflag then print, 'Computing Radarsat-2 Level-1 SIO data...

error=1
ans_read_rsat2, ff[0],    deffile,           $
                outputroot =  outputroot  ,  $
                jsstart    =  jsstart     ,  $
                error      =  error       ,  $
                quiet      =  quiet       ,  $
                help       =  help            

if error lt 0 then begin
  print, 'Error in ans_read_rsat2'
  error = -2
  return
endif

sio_root=       outputroot
;
; Loop through SIO files for full processing
;
w= where( strlen(sio_root) gt 0 , n)

for i=0, n-1 do begin
  ;
  ; Generating file names
  ;
  root=                getwrd( sio_root[i], 0, /last, del='/' ) 
  datafile=            df.sio_output_dir   + '/' + root + '_image.dat'
  infofile=            df.sio_output_dir   + '/' + root + '_ancil.txt'
  landfile=            df.sio_output_dir   + '/' + root + '_mask.dat'
  kmzfile=             df.sio_kmz_dir      + '/' + root + '_sio.kmz'
  netcdffile_level_1=  df.ncdf1_output_dir + '/' + root + '_sar_level1.nc'
  ;
  ; Computing Level 1 NetCDF files
  ;
  logfile=  df.log_dir + '/' +root + '.log'
  if strupcase(df.level1cdf) eq 'Y'  then begin
    if printflag then print, 'Computing Radarsat-2 Level-1 NetCDF data...   
    ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1, quiet=quiet
    time1=   dt_tm_tojs(systime(/utc))
    outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
    print, outstr
  endif
  ;
  ; SIO to KMZ
  ;
  if df.siokmzfile eq 'Y' then begin
    if printflag then begin
       print, ''
       print, 'Computing KMZ files.'
    endif
    ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
                 /nrcs, quiet=quiet, working_dir=df.working_directory
    time1=   dt_tm_tojs(systime(/utc))
    outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
    close, 1
    openw, 1, logfile, /append
    printf, 1, outstr
    close, 1
    print, outstr
  endif
endfor

;---------------------------------------------------------------
;--------------------- End conversion to SIO processing ---------------
;---------------------------------------------------------------

end
