;+---------------------------
; NAME: ANS_RSAT2_ANCIL_TR '
; PURPOSE:'
;  The purpose of this program is to read RSAT2 data and'
;  create an SIO formatted ancillary text file'
; CALLING SEQUENCE:'
;  ans_rsat2_ancil_tr, input_dir, deffile'
; INPUT:'
;  input_dir=    Radarsat-2 come in directories. This input is the name'
;                of such a directory. A directory might look like'
;                RS2_OK1455_PK12972_DK14754_U4_20080509_112327_HH_SGX/ '
;  deffile  =    Defaults files'
; OUTPUT:          '
;  An ASCII file with ancillary data. The convention for file names is of the form:'
;      RSAT2_DLR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)'
;      RSAT2_DLR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)'
;  The file name is delimited by _ and the entries are Sensor, processin facility, year, month, day, hour, minute'
;  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data'
;  files include the SIO 20-byte header/  '
; KEYWORDS:'
;  outputroot  =  Full path name of output root file name (Output from program)'
;  jssatrt     =  Processing start time (Output from program )'
;  error       =  Error (0 if program completes or -1 if not)'
;  /quiet      =  Do not print intermediate results.'
;  /help       =  Print out help'
; MODIFICATION HISTORY
;  2020 Dec 10 Tyler Ruff. Adapted from the original hacked version of ans_read_rsat2 with updates.
;-
;

pro ans_rsat2_ancil_tr, input_dir, deffile          , $
  output_dir  =  output_dir   , $
  outputroot  =  outputroot   , $
  error       =  error        , $
  jsstart     =  jsstart      , $
  header      =  header       , $
  quiet       =  quiet        , $
  help        =  help

  error=     0

  if keyword_set(help) or n_params(0) lt 1 then begin
    print, ' NAME: ANS_RSAT2_ANCIL_TR '
    print, ' PURPOSE:'
    print, '  The purpose of this program is to read RSAT2 data and'
    print, '  create an SIO formatted ancillary text file'
    print, ' CALLING SEQUENCE:'
    print, '  ans_rsat2_ancil_tr, input_dir, deffile'
    print, ' INPUT:'
    print, '  input_dir=    Radarsat-2 come in directories. This input is the name'
    print, '                of such a directory. A directory might look like'
    print, '                RS2_OK1455_PK12972_DK14754_U4_20080509_112327_HH_SGX/ '
    print, '  deffile  =    Defaults files'
    print, ' OUTPUT:          '
    print, '  ASCII file with ancillary data. The convention for file names is of the form:'
    print, '      RSAT2_DLR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_image.dat (image file)'
    print, '      RSAT2_DLR_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH_ancil.txt (ancillary data file)'
    print, '  The file name is delimited by _ and the entries are Sensor, processin facility, year, month, day, hour, minute'
    print, '  second, julian seconds, center longitude, center latitude, polarization, file type and extension. The data'
    print, '  files include the SIO 20-byte header/  '
    print, ' KEYWORDS:'
    print, '  outputroot  =  Full path name of output root file name (Output from program)'
    print, '  jssatrt     =  Processing start time (Output from program )'
    print, '  error       =  Error (0 if program completes or -1 if not)'
    print, '  /quiet      =  Do not print intermediate results.'
    print, '  /help       =  Print out help'
    error= -1
    return
  endif

  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet=  printflag eq 0
  if not keyword_set(jsstart) then jsstart= dt_tm_tojs(systime(/utc) )



  time0 = dt_tm_tojs(systime())

  if file_test( deffile) then ans_read_defaults, deffile, df, quiet=quiet, error=error
  if error lt 0 then begin
    if printflag then begin
      print, 'Cannot find file: ' + deffile
      print, 'Existing ans_read_rsat2.pro'
    endif
    error = -2
    return
  endif

  ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet

  ;setenv, 'GSHHS_DIR=' + df.gshhs_dir
  ;
  ; Determine if directory exists
  ;
  if printflag then print, 'Checking for exitence of Radarsat-2 directory...'

  ff= file_search( input_dir, /test_directory, count=count)

  if count eq 0 then begin
    print, 'Could not find Radarsat-2 directory: ' + input_dir
    print, 'Exiting ans_read_rsat2.pro '
    error= -6
    return
  endif

 
  ;
  ; Find the product.xml file and the base_directory for processing, ff= full path to product.xml file
  ;
  ;

  time1 = dt_tm_tojs(systime(/utc))

  rsat2_find_files, input_dir  + '/', ff, base_dir, quiet=quiet, error=error
  if error lt 0 then begin
    if printflag then begin
      print, 'Cannot find base_directory'
      print, 'Exiting ans_read_rsat2...'
    endif
    error=-7
    return
  endif
  ;
  ; Find leader file
  ;
  leaderfile=  file_search( base_dir,'product.xml', count=count)
  if count eq 0 then begin
    print, 'Cannot find product.xml in ' + base_dir
    print, 'Exiting ans_read_rsat2.pro '
    error= -1
    return
  endif

  ; rsat2_read_product, ff[0], lf, ii, jj, incidence, longitude, latitude, sigma_ne, quiet=quiet
  rsat2_read_product_fast, ff[0], lf, ii, jj, incidence, longitude, latitude, sigma_ne, quiet=quiet
  ;
  lf.nrcs_slope= df.rsat2_slope
  lf.nrcs_bias=  df.rsat2_bias
  ;
  ; Note that lf is an information structure.
  ; ii, jj, incidence, longitude, latitude are returned so that we can compute the relationship
  ; between index, incidence, longitude and latitude
  ;
  ;
  ; Compute whether to swith longitude basis. The basis starts from -180 to 180.
  ;
  lf.lngswitchflag=  0

  max_longitude=  max(longitude)
  min_longitude=  min(longitude)
  if max_longitude gt 180 then begin
    ; The basis in the original data is 0-360
    lf.lngswitchflag=  1
  endif else begin
    ;
    ; The basis is -180 to 180 then check if the data cross
    ; the International data line
    ;
    if max_longitude gt 120 and min_longitude lt -120 then begin
      ; The data cross the International Date Line and we must shift
      ; longitude basis
      lf.lngswitchflag=  1
      w=where(longitude lt 0, count)
      if count gt 0 then longitude(w)=360.0+longitude(w)
    endif
  endelse
  ;

  ;
  ;
  ; Add to struture
  temp_lf= create_struct(lf, 'deg',            df.deg,            $
    'ave_pixel',      df.ave_pixel,      $
    'ave_line' ,      df.ave_line,       $
    'answrs_version', answrs_version, $
    'jsstart',        jsstart         )


  lf=  0
  lf=  temp_lf

  ;
  ; Computing fits
  ;
  ; Longitude and latitude
  ;
  rsat2_lonlat2pix_fit, lf, ii, jj, longitude, latitude, quiet=quiet

  ;
  ; Incident angle
  rsat2_incid2pix_fit,  lf, ii, jj, incidence, incid_coef, quiet=quiet
  ;
  ; Compute polynomial coefficients for radar look direction
  ;
  rsat2_compute_rlook, lf, quiet=quiet
  ;
  ; sigam_ne
  ;
  rsat2_sigma_ne_fit, lf, sigma_ne


  ;
  ; Determine the short name of the directory with the files
  ;
  name_dir=   getwrd(input_dir, 0, /last, del='/')

  temp_lf= create_struct(lf, 'name_dir',    name_dir,    $
    'pol',                '',   $
    'cpol_remove_noise',  'UNK',   $
    'xpol_remove_noise',  'UNK')
  lf=               0
  lf=               temp_lf

  if tag_value( df, 'rsat2_cpol_remove_noise') ne '' then lf.cpol_remove_noise = tag_value( df, 'rsat2_cpol_remove_noise')
  if tag_value( df, 'rsat2_xpol_remove_noise') ne '' then lf.xpol_remove_noise = tag_value( df, 'rsat2_xpol_remove_noise')

  ;
  ; Create file name root for output files.
  ;

  case strupcase(lf.month) of
    'JAN': month=1
    'FEB': month=2
    'MAR': month=3
    'APR': month=4
    'MAY': month=5
    'JUN': month=6
    'JUL': month=7
    'AUG': month=8
    'SEP': month=9
    'OCT': month=10
    'NOV': month=11
    'DEC': month=12
    ELSE:  month=1
  endcase
  ;
  ; compute center geographic position
  ;

  ic= lf.nx1 / 2
  jc= lf.ny1 / 2

  lf.center_lon=  ans_poly_cal( double(ic), double(jc), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lf.center_lat=  ans_poly_cal( double(ic), double(jc), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)

  center_lat= string(abs(lf.center_lat), format='(F05.2)')
  if lf.center_lat ge 0 then begin
    center_lat= center_lat + 'N'
  endif else begin
    center_lat= center_lat + 'S'
  endelse
  center_lon= string(abs(lf.center_lon), format='(F06.2)')
  if lf.center_lon ge 0 then begin
    center_lon= center_lon + 'E'
  endif else begin
    center_lon= center_lon + 'W'
  endelse
  timestr= string( lf.data_year, format='(I4.4)') + ' ' + $
    lf.month                               + ' ' + $
    string( lf.day,       format='(I2.2)') + ' ' + $
    string( lf.hour,      format='(I2.2)') + ':' + $
    string( lf.minute,    format='(I2.2)') + ':' + $
    string( lf.second,    format='(I2.2)')
  js=            dt_tm_tojs(timestr)
  if js ge 0 then begin
    js=            string( dt_tm_tojs(timestr), format='(I10.10)' )
  endif else begin
    js=   '-' + string( abs(dt_tm_tojs(timestr)), format='(I9.9)' )
  endelse
  output_prefix= 'RSAT2'
  output_prefix= output_prefix + '_' + strupcase( lf.proc_fac)
  output_prefix= output_prefix + '_' + string( lf.data_year, format='(I4.4)')
  output_prefix= output_prefix + '_' + string( month,        format='(I2.2)')
  output_prefix= output_prefix + '_' + string( lf.day,       format='(I2.2)')
  output_prefix= output_prefix + '_' + string( lf.hour,      format='(I2.2)')
  output_prefix= output_prefix + '_' + string( lf.minute,    format='(I2.2)')
  output_prefix= output_prefix + '_' + string( lf.second,    format='(I2.2)')
  output_prefix= output_prefix + '_' + js
  output_prefix= output_prefix + '_' + string( center_lon)
  output_prefix= output_prefix + '_' + string( center_lat)
  output_prefix= strcompress( output_prefix, /remove_all )


  if printflag then print, 'Computed output prefix for SIO files:' + output_prefix

  time2= dt_tm_tojs(systime(/utc))

  if printflag then print, 'Time read product ancillary data file and process='+ string(time2-time1) +' s'
  ;
  ; Loop through different images in the file. A set of three files will be generated for each image
  ;
  nfiles=n_elements(lf.imagefiles)
  if n_elements(ave_pixel) eq 0 then ave_pixel=1
  if n_elements(ave_line)  eq 0 then ave_line= 1
  outputroot= strarr(nfiles)
  ;
  ; Create the output directory if it does not exist
  ;
  res= file_search( df.sio_output_dir, /test_directory, count=count)
  if count eq 0 then file_mkdir, df.sio_output_dir
  ;
  ;
  ; Loop through image files
  ;
  for i=0, nfiles-1 do begin
    ;
    ; Decision to process file base of polarizaton
    ;
    polarization= strupcase(getwrd(lf.polarization, i))
    lf.pol=       polarization
    polflag     = 1B
    if strupcase( df.sio_cross_pol ) eq '00' then polflag = 0B
    if strupcase( df.sio_cross_pol ) eq '01' then begin
      case strupcase( polarization ) of
        'HV' : polflag = 1B
        'VH' : polflag = 1B
        'HH' : polflag = 0B
        'VV' : polflag = 0B
        ELSE : polflag = 1B
      endcase
    endif
    if strupcase( df.sio_cross_pol ) eq '10' then begin
      case strupcase( polarization ) of
        'HV' : polflag = 0B
        'VH' : polflag = 0B
        'HH' : polflag = 1B
        'VV' : polflag = 1B
        ELSE : polflag = 1B
      endcase
    endif
    remove_noise_flag                                     = 0B
    cpol_flag                                             = 1B
    xpol_flag                                             = 0B
    if strupcase( polarization  ) ne 'HH' and $
      strupcase( polarization  ) ne 'VV' then begin
      cpol_flag                                          = 0B
      xpol_flag                                          = 1B
    endif
    if cpol_flag and $
      strupcase(df.rsat2_cpol_remove_noise) eq 'Y' then begin
      remove_noise_flag                                  = 1B
    endif
    if xpol_flag and $
      strupcase(df.rsat2_xpol_remove_noise) eq 'Y' then begin
      remove_noise_flag                                  = 1B
    endif
    if strlen(polarization) eq 0 then polflag=0
    if printflag then begin
      print, 'Polarization  : ' + polarization
      print, 'Imagefile     : ' + lf.imagefiles[i]
      print, 'sio_cross_pol : ' + tag_value( df, 'sio_cross_pol')
      print, 'polflag       : ' + string( fix(polflag), format='(I1)' )
    endif
    if polflag eq 1 then begin
      ;
      output_prefix2= output_prefix + '_' + strupcase(polarization) + '_C'
      ;
      ; Subtract noise
      ;
      poltag                                                          = 'C'  ; Co-polarizaton
      if ( polarization eq 'HV' or  polarization eq 'VH') then poltag = 'X'  ; X-polarization
      if printflag then begin
        print, 'This image'
        print, '  poltag                     = ' + poltag
        print, '  df.rsat2_cpol_remove_noise = ' + tag_value(df,'rsat2_cpol_remove_noise')
        print, '  df.rsat2_xpol_remove_noise = ' + tag_value(df,'rsat2_xpol_remove_noise')
      endif
      if ( strupcase(tag_value(df,'rsat2_cpol_remove_noise')) eq 'Y' and poltag eq 'C' ) or $
        ( strupcase(tag_value(df,'rsat2_xpol_remove_noise')) eq 'Y' and poltag eq 'X' ) then begin
        output_prefix2= output_prefix2 + '-'
      
      endif
      ;
      ; Write out to SIO data file format. This is general for all.
      ; SAR data
      ;
      scaling= 1.0
      outputroot[i]= df.sio_output_dir + '/'+ output_prefix2
      if printflag then print, 'Output file name : ' + outputroot[i]
      ;
      ; Write out SIO ancillary file format. This program is explicitly written for Radarsat-2 data
      ;
      rsat2_write_ancil_sio, df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt', lf, scaling, quiet=quiet
      time1= dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + df.sio_output_dir + '/'+ output_prefix2 + '_ancil.txt'
      outstr  = stress( outstr, 'R', 0, '//', '/', ned)
      if printflag then print, outstr
    endif         ; End if polflag eq 1
  endfor
  ;
  ; Eliminate outputroot file that are empty
  w          =  where( strlen(outputroot) gt 0, count )
  if count gt 0 then begin
    outputroot =  outputroot[w]
  endif else begin
    outputroot = ''
  endelse

end
