;-----------------------------------------------------
;+
; NAME: ANS_SAT2SIO
; PURPOSE:
;   To compute SIO image of the first satellite data
;   it runs into
; CALLING SEQUENCE
;   ans_sat2sio, deffile
; INPUT
;   deffile=     Full path to defaults file.
; OUTPUT
;   Level-1, SIO, and KMZ files (as specified)
; KEYWORDS:
;   /quiet       : No screen output
;   /help        : Get help
;   /asf         : Radarsat-1   data and ERS-2 data from ASF
;   /gat         : Radarsat-1   data from Gatineau (or CSTARS)
;   /envisat     : Envisat      data
;   /psar        : ALOS PALSAR  data
;   /rsat2       : Radarsat-2   data
;   /rcm         : Radarsat Constelation data
;   /tsar        : TerraSAR-X   data
;   /csm         : Cosmo SkyMed data
;   /sen         : Sentinel-1
;   /smap        : NASA SMAP 
;   /all         : All          data
;   error        : error (<0 bad)
; MODIFICATION HISTORY:
;   2011 Jan 07 Frank Monaldo.
;   2011 Jan 20 Frank Monaldo  Add Cosmos Sky Med
;   2012 Sep 18 Frank Monaldo. Add error code 
;   2013 Aug 14 Frank Monaldo. Moved the computation of SIO files of longitude,
;                              latitude, icemask, incidence angle, and
;                              radar look direciotn from the ans_read_[sat] files
;                              to ans_sat2sio
;   2013 Sep 09 Frank Monaldo. Added error catching for failure to read SIO ancillary data
;                              files.
;   2013 Sep 30 Frank Monaldo. Correced an error. Previous behavior: If an ice mask is not created, the program
;                              would exit without deleting the lock file.  Now the lock file is properly
;                              deleted. Removing return in computation of longitude array error.
;   2013 Oct 10 Frank Monaldo. Add Sentinel-1 call
;   2013 Nov 18 Frank Monaldo. Corrected latitude output
;   2014 Jan 22 Frank Monaldo. Remove the effect of an error returned if image is south of
;                              minimum latitude for compurtation of ice mask
;   2014 Apr 16 Frank Monaldo. Corrected an error where ice mask file was said to be produced in the
;                              mini-log file, when in fact it was not.
;   2014 Jun 17 Frank Monaldo  Correct error that would stop  processing in no ice data present.
;   2014 Jul 23 Frank Monaldo  Send up lower level error code
;   2014 Sep 24 Frank Monlado  We ran into problems with multiple images in the files and trying to
;                              avoid recalculation of landmask etc. We are now re-calculating even
;                              when not strictly required.
;   2015 Dec 29 Chris Jackson  Add SMAP     
;                              Skip creation of _longitude, _latitude, _incidence, _rlook of files already exist
;   2016 Jan 06 Chris Jackson  Move file_delete, lockfile to inside image loop.                     
;   2020 Feb 06 Chris Jackson  Add RCM options
; Copyright (C) 2011-2015, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_sat2sio, deffile,            $
                 asf      = asf     , $
                 gat      = gat     , $
                 psar     = psar    , $
                 envisat  = envisat , $
                 rsat2    = rsat2   , $
                 rcm      = rcm     , $
                 tsar     = tsar    , $
                 csm      = csm     , $
                 sen      = sen     , $
                 smap     = smap    , $
                 uavsar  =  uavsar  , $
                 help     = help    , $ 
                 all      = all     , $
                 error    = error   , $
                 quiet    = quiet



if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_SAT2SIO
  print, ' PURPOSE:
  print, '   To compute SIO image of the first satellite data
  print, '   it runs into
  print, ' CALLING SEQUENCE
  print, '   ans_sat2sio, deffile
  print, ' INPUT
  print, '   deffile=     Full path to defaults file.
  print, ' OUTPUT
  print, '   Level-1, SIO, and KMZ files (as specified)
  print, ' KEYWORDS:
  print, '   /quiet       : No screen output
  print, '   /help        : Get help
  print, '   /asf         : Radarsat-1   data and ERS-2 data from ASF
  print, '   /gat         : Radarsat-1   data from Gatineau (or CSTARS)
  print, '   /envisat     : Envisat      data
  print, '   /psar        : ALOS PALSAR  data
  print, '   /rsat2       : Radarsat-2   data
  print, '   /rcm         : Radarsat Constelation data
  print, '   /tsar        : TerraSAR-X   data
  print, '   /csm         : Cosmo SkyMed data
  print, '   /sen         : Sentinel-1
  print, '   /smap        : NASA SMAP
  print, '   /uavsasr     : UAVSAR'
  print, '   /all         : All          data
  error= -1
  return
endif

error=      0
!quiet=     1  ; Suppress compile statements
!except=    0  ; Suppress arithmetic error warnings
printflag=  1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

res= file_search(deffile, count=count)

if count eq 0 then begin
  print, 'There is no defaults file: ' + deffile
  print, 'Exiting ans_sat2sio'
  error = -1
  return
endif

;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin conversion to SIO files. ' + systime(/utc)
  print, ' '
endif

ans_read_defaults, deffile, df, quiet=quiet, error=error

if error lt 0 then begin
  error = -2
  print, 'Cannot find defaults file'
  return
endif

sat=['NONE']
if keyword_set(asf)      then   sat=[sat, 'asf'     ]
if keyword_set(gat)      then   sat=[sat, 'gat'     ]
if keyword_set(envisat)  then   sat=[sat, 'envisat' ]
if keyword_set(psar)     then   sat=[sat, 'psar'    ]
if keyword_set(rsat2)    then   sat=[sat, 'rsat2'   ]
if keyword_set(rcm)      then   sat=[sat, 'rcm'     ]
if keyword_set(tsar)     then   sat=[sat, 'tsar'    ]
if keyword_set(csm)      then   sat=[sat, 'csm'     ]
if keyword_set(sen)      then   sat=[sat, 'sen'     ]
if keyword_set(smap)     then   sat=[sat, 'smap'    ]
if keyword_set(uavsar)   then   sat=[sat, 'uavsar'  ]
if keyword_set(all)      then  begin
   sat=['NONE',  'asf', 'gat', 'envisat', 'psar', 'rsat2', 'rcm', 'tsar', 'csm', 'sen','smap','uavsar'] 
endif

ns= n_elements( sat )

if ns eq 1 then begin
  print, 'No satellites specified.'
  error = -3
  return
endif


for i=1, ns-1 do begin ; Loop though satellites
  ;
  ; Generate SIO file, Level-1 CDF and KMZ
  ;

  case sat[i] of 
  'asf'      : ans_asf2sio      , deffile, sio_root, quiet=quiet
  'gat'      : ans_gat2sio      , deffile, sio_root, quiet=quiet
  'envisat'  : ans_envisat2sio  , deffile, sio_root, quiet=quiet
  'psar'     : ans_psar2sio     , deffile, sio_root, quiet=quiet
  'rsat2'    : ans_rsat22sio    , deffile, sio_root, quiet=quiet
  'rcm'      : ans_rcm2sio      , deffile, sio_root, quiet=quiet
  'tsar'     : ans_tsar2sio     , deffile, sio_root, quiet=quiet
  'csm'      : ans_csm2sio      , deffile, sio_root, quiet=quiet
  'sen'      : ans_sen2sio      , deffile, sio_root, quiet=quiet
  'smap'     : ans_smap2sio     , deffile, sio_root, quiet=quiet
  'uavsar'   : ans_uavsar2sio   , deffile, sio_root, quiet=quiet  
  endcase
  if error lt 0 then begin
    if error eq -2 then error = -4
    return
  endif
  ;
  ; Create ice mask, longitude, latitude, incidence, and/or radar look files
  ;
  w        =  where( strlen(sio_root) gt 1, /null )
  sio_root = sio_root[w]
  n_sio    = n_elements( sio_root)
  if n_sio gt 0 then begin  ; Loop through images
    for s= 0, n_sio-1 do begin
      lockfile     =  sio_root[s] +'.lock'
      logfile      =  df.log_dir + getwrd( sio_root[s], 0, /last, del='/')  + '.log'               
      longfile     =  sio_root[s] + '_longitude.dat'
      latfile      =  sio_root[s] + '_latitude.dat'
      incidfile    =  sio_root[s] + '_incidence.dat'
      rlookfile    =  sio_root[s] + '_radarlook.dat'
      icemaskfile  =  sio_root[s] + '_ice.dat'
      ancilfile    =  sio_root[s] + '_ancil.txt'
      landmaskfile =  sio_root[s] + '_mask.dat'
      ;
      ;  Land mask file
      ;
 
       ans_def_ancil2landmask, deffile, ancilfile, outputfile = landmaskfile, quiet=quiet, error= error
       if error lt 0 then begin
         print, 'Error in ans_def_ancil2landmask call in ans_sat2sio while generating landmask'
         error = -5
       endif
      ;
      tm      = dt_tm_tojs(systime(/utc))
      outstr  = string(tm, format='(I10)') + ' '+ dt_tm_fromjs(tm) + ' ' +  landmaskfile
      outstr  = stress( outstr, 'R', 0, '//', '/', ned)
      close,  1
      openw,  1, logfile, /append
      printf, 1, outstr
      close,  1
      print, outstr
      ;
      ;  Ice File
      ;
      if strupcase( tag_value(df, 'ice_mask') ) eq 'Y' then begin
        ;
        ans_def_ancil2icemask, deffile, ancilfile, outputfile = icemaskfile, quiet=quiet, error= error
        if error lt 0  and error ne -3 and error ne -4 then begin
          msg  = 'Error in ans_def_ancil2landmask call in ans_sat2sio while generating icemask. '
          msg  = msg  + ' Error=' + string(error)
          error = 0
          icelogflag = 0
        endif
        ;
        ; Error -3 is not a problem. It just means that the
        ; image is further south of the limit for computing
        ; the ice mask. Error=-4 means the no original ice file
        ; can proceed without ice file
        ;
        if error eq -3 or error eq -4 then error = 0  	
        ;
        tm      = dt_tm_tojs(systime(/utc))
        outstr  = string(tm, format='(I10)') + ' '+ dt_tm_fromjs(tm) + ' ' +  icemaskfile
        outstr  = stress( outstr, 'R', 0, '//', '/', ned)
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
        print, outstr
      endif                       ; End check if ice mask turned on
      ;
      ; Longitude SIO file
      ;
      longfile2 = df.sio_output_dir + '/' +  longfile
      ff= file_search( longfile2, count=count)
      if (strupcase( tag_value(df, 'sio_longitude') ) eq 'Y') and (count eq 0) then begin
         ans_ancil2data, ancilfile, outputfile=longfile, /longitude, quiet=quiet
         close, 1
         openw, 1, logfile, /append
                  tm   = dt_tm_tojs(systime(/utc))
         tm      = dt_tm_tojs(systime(/utc))
         outstr = string(tm, format='(I10)')+ ' '+ dt_tm_fromjs(tm) + ' ' + longfile
         outstr = stress( outstr, 'R', 0, '//', '/', ned)
         printf, 1, outstr
         close, 1
         print, outstr
      endif
      ;
      ; Latitude SIO file
      ;
      latfile2 = df.sio_output_dir + '/' +  latfile
      ff= file_search( latfile2, count=count)
      if (strupcase( tag_value(df, 'sio_latitude') ) eq 'Y') and (count eq 0) then begin
         ans_ancil2data, ancilfile, outputfile=latfile, /latitude, quiet=quiet
         close, 1
         openw, 1, logfile, /append
         tm      = dt_tm_tojs(systime(/utc))
         outstr = string(tm, format='(I10)')+ ' '+ dt_tm_fromjs(tm) + ' ' + latfile
         outstr = stress( outstr, 'R', 0, '//', '/', ned)
         printf, 1, outstr
         close, 1
         print, outstr
      endif
      ;
      ; Incident angle SIO file
      ;
      incidfile2 = df.sio_output_dir + '/' +  incidfile
      ff= file_search( incidfile2, count=count)
      if (strupcase( tag_value(df, 'sio_incidence') ) eq 'Y') and (count eq 0) then begin
         ans_ancil2data, ancilfile, outputfile=incidfile, /incidence, quiet=quiet
         close, 1
         openw, 1, logfile, /append
         tm      = dt_tm_tojs(systime(/utc))
         outstr = string(tm, format='(I10)')+ ' '+ dt_tm_fromjs(tm) + ' ' + incidfile
         outstr = stress( outstr, 'R', 0, '//', '/', ned)
         printf, 1, outstr
         close, 1
         print, outstr
      endif
      ;
      ; Radar look angle SIO file
      ;
      rlookfile2 = df.sio_output_dir + '/' +  rlookfile
      ff= file_search( rlookfile2, count=count)
      if (strupcase( tag_value(df, 'sio_rlook') ) eq 'Y') and (count eq 0) then begin
         ans_ancil2data, ancilfile, outputfile=rlookfile, /rlook, quiet=quiet
         close, 1
         openw, 1, logfile, /append
         tm      = dt_tm_tojs(systime(/utc))
         outstr = string(tm, format='(I10)')+ ' '+ dt_tm_fromjs(tm) + ' ' + rlookfile
         outstr = stress( outstr, 'R', 0, '//', '/', ned)
         printf, 1, outstr
         close, 1
         print, outstr
      endif

     file_delete, lockfile, /allow_nonexistent, /quiet

    endfor                        ; End loop through different image loop
  endif                           ; If number of sio root files > 0
endfor                            ; End looping through satellites
;

end
