;-----------------------------------------------------
;+
; NAME: ans_sen2sio
; PURPOSE:
;   To convert Sentinel-1 imagery to SIO format
; CALLING SEQUENCE
;   ans_sen2sio, deffile, sio_out
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters /twobyte, deg, ave_line, and ave_pixel
;                If the parameters are explicitly set. For example, ave_pixel=0 then
;                it will over-ride what ever is in the defaults file
; OUTPUT
;   sio_out=     Root file name(s) for SIO files
;   SIO files, NetCDF level and KMZ files
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
;   error:       -1 if bad
; MODIFICATION HISTORY:
;   2013 Sep 30  Frank Monaldo. Adapted from ans_tsar2sio
;   2014 Oct 30  Chris Jackson. Narrowed the file search to S1* directories only
;   2015 May 05  Frank Monaldo. Process the newest files first
;   2020 Mar 31  Chris Jackson. Updated search to S1*SAFE, Add the creation/deletion of the SAFE lock file.
;   2020 Apr 02  Chris Jackson. Move to "already_processed" relocated from ans_read_sen.pro
; Copyright (C) 2013-2015, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_sen2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

  !quiet=1
  printflag=1
  error = 0
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  sio_root=''

  if keyword_set(help)   or (n_params(0) lt 1 )  then begin
    print, ' NAME: ans_sen2sio
    print, ' PURPOSE:
    print, '   To convert Sentinel-1 imagery to SIO format
    print, ' CALLING SEQUENCE
    print, '   ans_sen2sio, deffile, sio_out
    print, ' INPUT
    print, '   deffile=     Full path to defaults file. This option will read a specified file
    print, '                for the parameters /twobyte, deg, ave_line, and ave_pixel
    print, '                If the parameters are explicitly set. For example, ave_pixel=0 then
    print, '                it will over-ride what ever is in the defaults file
    print, ' OUTPUT
    print, '   sio_out=     Root file name(s) for SIO files
    print, '   SIO files, NetCDF level and KMZ files
    print, ' KEYWORDS:
    print, '   /quiet       No screen output
    print, '   /help;       Get help
    print, '   error:       -1 if bad
    return
  endif
  !quiet=  1 ; set idl to quiet

  ;
  ; Time to start processing
  ;

  jsstart=  dt_tm_tojs(systime(/utc))


  ;
  ; Read defaults file
  ;
  if printflag then begin
    print, ' '
    print, 'Begin Sentinel to SIO. ' + systime(/utc)
    print, ' '
  endif
  ans_read_defaults, deffile, df, quiet=quiet
  ;
  ;
  ; Create directories if they do not exist
  ;
  res= file_search( df.already_processed, /test_directory, count=count)
  if count eq 0 then file_mkdir, df.already_processed
  res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.ncdf1_output_dir
  res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.sio_kmz_dir
  res= file_search( df.log_dir, /test_directory,     count=count)
  if count eq 0 then file_mkdir, df.log_dir
  ;
  ;
  ;---------------------------------------------------------------
  ;----------- Start Sentinel data conversion to SIO  ---------------
  ;---------------------------------------------------------------

  ;
  ; Look for Sentinel directories in base directory
  ;
  ff= file_search( df.sen_dir +'/S1*SAFE', /test_directory, count=count) & cc=count
  if cc eq 0 then begin
    print, 'Cannot find files in ' +  df.sen_dir +'/*'
    print, 'Exiting ans_sen2sio'
    sio_root=''
    error = -1
    return
  endif
  if count gt 1 then begin
    ;
    ; Sort to find the newest file
    ;
    ; Default time to a large negative numer
    ;
    jsdir = dblarr(count) - 1e-45
    for i=0, count-1L do begin
      sendir  =  getwrd( ff[i], del='/', /last )
      tmstr   =  getwrd( sendir, 4, del='_')
      year    =  strmid( tmstr,  0, 4  )
      month   =  strmid( tmstr,  4, 2  )
      day     =  strmid( tmstr,  6, 2  )
      hour    =  strmid( tmstr,  9, 2  )
      minute  =  strmid( tmstr, 11, 2  )
      second  =  strmid( tmstr, 13, 2  )
      if isnumber( year   ) and $
        isnumber( month  ) and $
        isnumber( day    ) and $
        isnumber( hour   ) and $
        isnumber( minute ) and $
        isnumber( second ) then begin
        if long( month  ) gt  0 and $
          long( month  ) lt 13 and $
          long( day    ) gt  0 and $
          long( day    ) lt 32 and $
          long( hour   ) ge  0 and $
          long( hour   ) le 23 and $
          long( minute ) ge  0 and $
          long( minute ) le 59 and $
          long( second ) ge  0 and $
          long( second ) le 59 then begin
          inputstr = year + ' ' + month2name( long(month) ) + ' ' + day + ' '
          inputstr = inputstr + hour + ':' + minute + ':' + second
          jsdir[i] = dt_tm_tojs(inputstr)
          ;if printflag then print, inputstr, jsdir[i]
        endif ; Check for valid values
      endif   ; End test all elements begin numbers
    endfor     ; Looping through directories
    ind  = reverse( sort( jsdir) )
    ff  =  ff[ind]
  endif

  ;Look through the files and find the first unlocked file
  nfiles=n_elements(ff);
  lock_count = 0
  sio_root=''

  for i = 0,nfiles-1 do begin
    lock_file = ff[i]+'_lock'
    if (file_test(lock_file,/regular) eq 1 ) then begin
      lock_count++
      continue;
    endif

    ;Create lockfile
    close, 1 & openw, 1, lock_file  & printf, 1, '' & close, 1
    if printflag then print, 'Selected directory: ' + ff[i]

    ;
    ; Computing SIO files for all the Sentinel data
    ;
    error= 1
    if printflag then print, 'Computing Sentinel Level-1 SIO data...
    ans_read_sen,   ff[i]        , deffile    , $
      outputroot   = outputroot , $
      jsstart      = jsstart    , $
      error        = error      , $
      quiet        = quiet

    if error lt 0 then begin
      print, 'Cannot read Sentinel data'
      print, 'Exiting ans_sen2sio.pro'
      error= -1
      return
    endif

    sio_root= outputroot
    ;Only process the first unlock file found
    break
  end

  if (lock_count eq nfiles) then begin
    print, 'There are no unlocked SAFE files in directory:'
    print, df.sen_dir
    error = -1
    return
  endif

  ;
  ; Loop through SIO files for full processing
  ;
  ; Computing Level 1 NetCDF files and KMZ
  ;
  n= n_elements(sio_root)

  for ii=0, n-1 do begin
    if strlen( sio_root[ii] ) gt 0 then begin
      root=                getwrd( sio_root[ii], 0, /last, del='/' )
      datafile=            sio_root[ii]          + '_image.dat'
      infofile=            sio_root[ii]          + '_ancil.txt'
      kmzfile=             df.sio_kmz_dir       + root + '_sio.kmz'
      netcdffile_level_1=  df.ncdf1_output_dir  + root + '_sar_level1.nc'
      logfile=  df.log_dir + '/' + root + '.log'
      ;
      ; SIO to NetCDF
      ;
      if printflag then print, 'Computing Sentinel Level-1 NetCDF data...
      if df.level1cdf eq 'Y' then begin
        ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1
        time1=   dt_tm_tojs(systime(/utc))
        outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
        print, outstr
      endif
      ;
      ; SIO to KMZ
      ;
      if df.siokmzfile eq 'Y' then begin
        if printflag then begin
          print, ''
          print, 'Computing KMZ files.'
        endif
        ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
          /nrcs, quiet= quiet, working_dir=df.working_directory
        time1=   dt_tm_tojs(systime(/utc))
        outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
        print, outstr
      endif
    endif
  endfor

  ; Move files to already processed directory if move2dir set
  ;
  if printflag then begin
    print, 'Moving ' + ff[i]
    print, 'to     ' + df.already_processed + '/'
  endif
  file_move,   ff[i], df.already_processed  + '/', /overwrite, /ALLOW_SAME, /verbose
  ;

  ;Remove the SAFE lockfile
  if printflag then  print, 'Deleting Lockfile ' + lock_file
  file_delete, lock_file,/ALLOW_NONEXISTENT, /verbose


  ;---------------------------------------------------------------
  ;--------------------- End Sentinel processing ---------------
  ;---------------------------------------------------------------

end
