;-----------------------------------------------------
;+
; NAME: ans_sen_tiff2tiff
; PURPOSE:
;   The is an exploratory program to time the generation of reprojected text file
;   from Sentinel images
; CALLING SEQUENCE
;   ans_sen_tiff2tiff, sen_dir, deffile, tiff_file
; INPUT
;   sen_dir   = Sentinel directory
;   deffile   = Defaults file
; OUTPUT
;   tiff_file = Output tiff file
; KEYWORDS:
;   /quiet    =  No screen output
;   /help     =  Get help
;   error     =  -1 if bad
; MODIFICATION HISTORY:
;   2015 May 22  Frank Monaldo. Adapted from ans_tsar2sio
; Copyright (C) 2013-2015, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_sen_tiff2tiff, sen_dir, deffile, tiff_file, help= help,  quiet= quiet, error=error

!quiet    = 1
printflag = 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)   or (n_params(0) lt 1 )  then begin
  print, ' NAME: ans_sen_tiff2tiff
  print, ' PURPOSE:
  print, '   The is an exploratory program to time the generation of reprojected text file
  print, '   from Sentinel images
  print, ' CALLING SEQUENCE
  print, '   ans_sen_tiff2tiff, sen_dir, deffile, tiff_file
  print, '   deffile   = Defaults file
  print, ' INPUT
  print, '   sen_dir   = Sentinel directory
  print, '   deffile   = Defaults file
  print, ' OUTPUT
  print, '   tiff_file = Output tiff file
  print, ' KEYWORDS:
  print, '   /quiet    =  No screen output
  print, '   /help     =  Get help
  print, '   error     =  -1 if bad
  return
endif
!quiet=  1 ; set idl to quiet

jsstart=  dt_tm_tojs(systime(/utc))

;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin Sentinel to SIO. ' + systime(/utc)
  print, ' '
endif
;
;  
;---------------------------------------------------------------
;----------- Read Sentinel data  ---------------
;---------------------------------------------------------------

;
; Look for Sentinel directories in base directory
;
res=  file_search(deffile, count=count)
if count eq 0 then begin
  print, 'Cannot defaults file: ' + deffile
  print, 'Exiting ans_sen_tiff2tiff'
  error = -1
  return
endif
;
ff= file_search( sen_dir, /test_directory, count=count) & cc=count
if cc eq 0 then begin
  print, 'Cannot find files in ' +  sen_dir
  print, 'Exiting ans_senn_tiff2tiff
  error = -1
  return
endif
;
if printflag then print, 'Selected directory: ' + ff[0]
;
; Computing SIO files for all the Sentinel data
;
error= 1
if printflag then print, 'Computing Sentinel Level-1 SIO data...
;
; Locating leader file
; 
;

xfiles=   file_search( sen_dir, 's*xml', count=data_setmax)
if data_setmax eq 0 then begin
  print, 'Cannot file xml data in: ' + working_directory
  print, 'Exiting ans_read_sen.pro'
  error = -1
  return
endif
for data_set= 0, data_setmax-1 do begin
  base_directory = sen_dir
  ;
  ; Fill information into lf structure, Read mapping grid information
  ; and read geo reference dagta
  ;
  corename  = getwrd( xfiles[data_set], 0, /last, del='/' )
  len       = strlen( corename )
  corename  = strmid( corename, 0, len-4 )
  sen_fillin_lf,  xfiles[data_set], deffile,  corename, jsstart, lf,  /quiet
  ; 
  ; Proceed for only like polarization.
  ;
  if printflag then print, 'Polarization : ' + lf.polarization 
  if  strupcase( lf.polarization  ) eq 'VV' or  $
      strupcase( lf.polarization  ) eq 'HH' then begin
      imgfile  = file_search( base_directory, corename + '.tiff', count= count)
      if count eq 0 then begin
        print, 'Cannot find image file ' + corename + '.tiff'
        error = -1
        return
     endif
    ;
    ; Read TIFF image
    ;
    if printflag then print, 'Read image from file: ' + imgfile
    t1 = dt_tm_tojs(systime(/utc))
    img =  read_tiff( imgfile[data_set] )
    t2 = dt_tm_tojs(systime(/utc))
    if printflag then print, 'Time to read file: ' + string( t2-t1, format='(I6)' ) + 's'
    ;
    ; Create tiff image
    ; 
    sz = size(img)
    nx   = sz[1]
    ny   = sz[2]
    xout = nx
    yout = ny
    ;
    ; lower corner locations
    ;
    lonmin= ans_poly_cal( double(0), double(0), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
    lonmax= lonmin
    latmin= ans_poly_cal( double(0), double(0), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
    latmax= latmin
    ;
    ; Bottom
    y= double(0L)
    for i=0L, nx do begin              
      lng=    ans_poly_cal( double(i), y, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
      lat=    ans_poly_cal( double(i), y, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
      lonmin= min( [lonmin, lng] ) 
      lonmax= max( [lonmax, lng] )
      latmin= min( [latmin, lat] ) 
      latmax= max( [latmax, lat] )  
    endfor
    ; top
    y= double(ny)
    for i=0L, nx do begin
      lng=    ans_poly_cal( double(i), y, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
      lat=    ans_poly_cal( double(i), y, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
      lonmin= min( [lonmin, lng] ) 
      lonmax= max( [lonmax, lng] )
      latmin= min( [latmin, lat] ) 
      latmax= max( [latmax, lat] )  
    endfor  
    ; left
    x= double(0L)
    for j=0L, ny do begin
      lng=    ans_poly_cal( x, double(j), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
      lat=    ans_poly_cal( x, double(j), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
      lonmin= min( [lonmin, lng] ) 
      lonmax= max( [lonmax, lng] )
      latmin= min( [latmin, lat] ) 
      latmax= max( [latmax, lat] )  
    endfor 
    ; right
    x= double(nx)
    for j=0L, ny do begin
      lng=    ans_poly_cal( x, double(j), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
      lat=    ans_poly_cal( x, double(j), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
      lonmin= min( [lonmin, lng] ) 
      lonmax= max( [lonmax, lng] )
      latmin= min( [latmin, lat] ) 
      latmax= max( [latmax, lat] )
    endfor
  endif   ; End check for like-polarization
endfor    ; End loop over data_set
;
; Create longitude and latitude arrays
; for rectilinear grid
;
lng    = fltarr(nx,ny)
lat    = fltarr(nx,ny)
slope  = (lonmax-lonmin) / (nx-1.0)
rlon   = findgen(nx) * slope + lonmin
for j=0, ny-1L do lng[*,j]= rlon
slope  = (latmax-latmin) / (ny-1.0)  
rlat   = findgen(ny) * slope + latmin
for i=0, nx-1 do lat[i,*]= rlat

;
; Resmapling the image to a rectilinear longitude/latitude grid.
;
img2  = fltarr( nx, ny )
nxs   = strcompress( string( nx, '(I6)') )
istep = nx/10 
t1    = dt_tm_tojs( systime(/utc) )
if printflag then print, 'Begin copmputing longitude latitude arrays.' + systime(/utc)
is    = long( ans_poly_cal( lng, lat, lf.i_coef, lf.i_xexp, lf.i_yexp) )
js    = long( ans_poly_cal( lng, lat, lf.j_coef, lf.j_xexp, lf.j_yexp) )
if printflag then print, 'End  copmputing longitude latitude arrays.' + systime(/utc)


for i=0L, nx-1 do begin
  if i mod istep eq 0 and printflag then begin
    out= 'Resample image set to rectilinear lon/lat'
    out=  out + string(i, format='(I6)') + ' of ' + nxs 
    print, out + ' ' +systime()
  endif
  for j=0L, ny-1 do begin
    if is[i,j] ge    0 and   $
       is[i,j] lt nx-1 and   $
       js[i,j] ge    0 and   $
       js[i,j] lt ny-1 then begin
       img2[i,j] =  img[is[i,j],js[i,j]]
    endif
  endfor
endfor
t2    = dt_tm_tojs( systime(/utc) )
;
;
;
print, t1,t2
if printflag then print, 'Time for resampling image:' + string(t2-t1, format='(I7)') + ' s'

;
; Set up mapping area for cylindrical projection
;
set_plot,'z'
loadct, 0
device, set_resolution=[ nx, ny ]
if printflag then begin
  print, 'TIFF image dimensions:'
  print, 'X= ' + string(nx)
  print, 'y= ' + string(ny) 
endif


latcen=  0.0
loncen=  (lonmin+lonmax) / 2.0 

map_set2, latcen, loncen, /CYL, limit=[latmin,lonmin,latmax,lonmax], $
          xmargin=[0,0], ymargin=[0,0], /noborder, $
          position=[0.00, 0.00, 1.00, 1.00] 
if printflag then print, 'Start re-projection     :' + systime(/utc)
img3   = map_image(img2, sx, sy ,  missing=0, compress=1,    $
                  latmin=latmin,  latmax=latmax,             $
                  lonmin=lonmin,  lonmax=lonmax   )   
if printflag then print, 'End   re-projection     :' + systime(/utc)
if printflag then print, 'Start display to device :' + systime(/utc)
tvscl, ls(img3, /quiet), sx,sy
if printflag then print, 'End   display to device :' + systime(/utc)
if printflag then print, 'Start write to file     :' + systime(/utc)
;
; Save GeoTFF image
;
save_tiff_image:

tvlct,  r, g, b, /get

z=             tvrd()
sz=            size(z)
nx=            sz[1]
ny=            sz[2]
rr=            byte(r[z])
gg=            byte(g[z])
bb=            byte(b[z])

; Flip image so origin is upper left corner, following remote sensing
; convention.
rr=            rotate(rr, 7)
gg=            rotate(gg, 7)
bb=            rotate(bb, 7)
sz=            size(rr)
nx=            sz[1]
ny=            sz[2]
img4=           bytarr(3,nx,ny)

img4[0, *, * ]= rr
img4[1, *, * ]= gg
img4[2, *, * ]= bb
;
; Set up the appropriate geokey structure
; Reference: http://www.remotesensing.org/geotiff/spec/geotiff2.7.html#2.7.3

xres=  double(lonmax-lonmin) / nx
yres=  double(latmax-latmin) / ny

geokeys = { GTModelTypeGeoKey    : 2S,    $   ; geographic
            GTRasterTypeGeoKey   : 1S,    $   ; RasterPixelIsArea
            GeographicTypeGeoKey : 4326S, $   ; WGS-84
            ModelTiepointTag     : [0.0D, 0.0D, 0.0D, double(lonmin), double(latmax), 0.0D], $
            ModelPixelScaleTag   : [xres, yres, 0.0D] $
          }

if printflag then print, 'Writing: ' + tiff_file

write_tiff, tiff_file, img4, geotiff=geokeys, compression=1
if printflag then print, 'End   write to file     :' + systime(/utc)
end
