;----------------------------------------------------------
;+
; NAME: ANS_SIO2KMZ'
; PURPOSE:'
;   This program reads data in SIO format and generates a 
;   corresponding KMZ file
; CALLING SEQUENCE:'
;   ans_sio2kmz, datafile, infofile, outputfile'
; INPUT:'
;   datafile=     The name of the SIO NRCS for land mask file
;   infofile=     The name of the SIO info file
; OUTPUT:'
;   outputfile=   Name of the output KMZ file
; KEYWORDS:
;    /quiet=      Do not print intermediate results'
;    /help=       List help.'
;    avefactor=   Factor to average down size of the image
;                 If avefactor=2 then then image size is 
;                 reduced by 2 in both directions
;    /mask=       Intpret as land mask file. This will produce
;                 kmz file with a clear land/wate boundary.  if this 
;                 is not selected, land mask files are displayed
;                 in their natural format. Negative values are land values
;                 in units of pixel values from land/sea boundary. 
;                 Positive values  are water values in units of pixel 
;                 values from land/sea boundary.
;    /ice         Interpret as ice mask
;    /usecongrid  Use congrid to resame when averaging. congrid takes longer
;                 but is a little more accurate. Can crash for lack of memory
;                 for very large image
;    maxsize=     Maximum number of pixels along an image dimension before a 
;                 different, slower resizing is used. default=60000
;    xout=        Number of horizontal pixels for image in KMZ file, default=2000
;    yout=        Number of vertical   pixels for image in KMZ file, default=2000
;    /flatten     When no a mask file flatten the image. i.e. remove low-freq trends
;    /plotlook    Draw arrows for radar look direction
;    /nrcs        Plot NRCS distribution.
;    /cutoff      Maximum NRCS for display and PDF
;    /incid       Print out incident angle values
;    /kmzdb       Display KMZ file in dB (10*ALOG10(sigma))
;    /amplitude   Display the square root of the input image.
;    working_dir  Working directory to create  sio file
; NOTES:
;    Requires zip
; MODIFICATION
;    2009 Nov 25 Frank Monaldo
;    2009 Dec 01 Frank Monaldo
;                Frank Monaldo. Corrected a bug where rlat size was xout not yout
;    2009 Dec 02 Frank Monaldo. The code is validated with the 0-360 longitude basis
;    2009 Dec 04 Frank Monaldo. Add option to flatten image. Check the entire image
;                               edges for max lat and lng, not just corners
;                               Change maxsize default to 60000. FOrce avefactor=5
;                               for very large image dimension > maxsize
;    2009 Dec 05 Frank Monaldo. Added option to draw radar look arrows
;    2009 Dec 10 Frank Monaldo. Add image scaling options
;    2009 Dec 11 Frank Monaldo. Add incident angle
;    2010 Jan 02 Frank Monaldo. Dealt with situration where there are no values >0
;    2010 Jan 08 Frank Monaldo. When plotting radar look direction, increased the number
;                               of arrowss per image from three to nine.
;    2010 Jan 10 Frank Monlado. Corrected incident angle calculation so that it would
;                               not be effected by avefactor.
;    2010 Jan 21 Frank Monaldo. Corrected typo in help pointed out by Chris Jackson.
;    2010 Jan 30 Frank Monaldo. Corrected misspelling of print/usecongrid commands
;    2010 Feb 25 Frank Monaldo. Corrected misspelling of rectilinear
;    2010 Mar 29 Frank Monaldo. Add option for display of dB
;    2010 Jun 14 Frank Monaldo. Try to add transparency.
;    2010 Jul 03 Frank Monaldo. Improve KMZ writing to that I do not have the
;                               same file names. This means that there is no chance of
;                               writing over in processing.
;    2010 Dec 29 Frank Monaldo. Pass on quiet info
;    2011 Jan 01 Frank Monaldo. Removed hlp command that was slowing things down.
;    2011 Feb 11 Frank Monaldo. Add working directory option
;    2011 Aug 15 Frank Monaldo. Do not remove ./
;    2011 Aug 30 Frank Monaldo. Add /amplitude option
;    2012 Sep 06 Frank Monaldo. Deal with bug where KMZ file is written in working
;                               and erased
;    2013 Jan 19 Frank Monaldo. Add ice mask display opition
;    2014 Jan 28 Frank Monaldo. Make 2000 pixels the default size
;    2016 Nov 14 Chris Jackson. Converted the "getenv" call to "cd current=" to save the PWD to a variable 
; Copyright (C) 2009-2012, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  ans_sio2kmz, datafile, infofile, outputfile,  $
                  avefactor=    avefactor,         $
                  mask=         mask,              $
                  usecongrid=   usecongrid,        $
                  maxsize=      maxsize,           $
                  xout=         xout,              $
                  yout=         yout,              $
                  flatten=      flatten,           $
                  plotlook=     plotlook,          $
                  nrcs=         nrcs,              $
                  kmzdb=        kmzdb,             $
                  amplitude=    amplitude,         $
                  working_dir=  working_dir,       $
                  cutoff=       cutoff,            $
                  incid=        incid,             $
                  ice=          ice,               $
                  quiet=        quiet,             $
                  help=         help


if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_SIO2KMZ'
  print, ' PURPOSE:'
  print, '   This program reads data in SIO format and generates a 
  print, '   corresponding KMZ file
  print, ' CALLING SEQUENCE:'
  print, '   ans_sio2kmz, datafile, infofile, outputfile'
  print, ' INPUT:'
  print, '   datafile=     The name of the SIO NRCS for land mask file
  print, '   infofile=     The name of the SIO info file
  print, ' OUTPUT:'
  print, '   outputfile=   Name of the output KMZ file
  print, ' KEYWORDS:
  print, '    /quiet=      Do not print intermediate results'
  print, '    /help=       List help.'
  print, '    avefactor=   Factor to average down size of the image
  print, '                 If avefactor=2 then then image size is 
  print, '                 reduced by 2 in both directions
  print, '    /mask=       Intpret as land mask file. This will produce
  print, '                 kmz file with a clear land/wate boundary.  if this 
  print, '                 is not selected, land mask files are displayed
  print, '                 in their natural format. Negative values are land values
  print, '                 in units of pixel values from land/sea boundary. 
  print, '                 Positive values  are water values in units of pixel 
  print, '                 values from land/sea boundary.
  print, '    /ice         Interpret as ice mask
  print, '    /usecongrid  Use congrid to resame when averaging. congrid takes longer
  print, '                 but is a little more accurate. an crash for lack of memory
  print, '                 for very large image
  print, '    maxsize=     Maximum number of pixels along an image dimension before a 
  print, '                 different, slower resizing is used. default=60000
  print, '    xout=        Number of horizontal pixels for image in KMZ file, default=2000
  print, '    yout=        Number of vertical   pixels for image in KMZ file, default=2000
  print, '    /flatten     When no a mask file flatten the image. i.e. remove low-freq trends
  print, '    /plotlook    Draw arrows for radar look direction
  print, '    /nrcs        Plot NRCS distribution
  print, '    /cutoff      Maximum NRCS for display and PDF
  print, '    /incid       Print out incident angle values
  print, '    /amplitude   Display the square root of the input image.
  print, '    /kmzdb       Display KMZ file in dB (10*ALOG10(sigma))
  print, '    working_dir  Working directory to create  sio file
  print, ' NOTES:
  print, '    Requires zip
  return
endif

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if not keyword_set(avefactor) then avefactor=1
 
res=file_search(datafile, count=count)
if count eq 0 then begin
  print, 'Cannot find: ' + datafile
  print, 'Exiting ans_sio2kmz'
  return
endif

res=file_search(infofile, count=count)
if count eq 0 then begin
  print, 'Cannot find: ' + infofile
  print, 'Exiting ans_sio2kmz'
  return
endif

ans_read_sio_data, datafile, image, quiet=quiet
ans_read_sio_info, infofile, lf,  quiet=quiet



if not keyword_set(maxsize) then maxsize= 60000

if not keyword_set(xout) then xout=2000
if not keyword_set(yout) then yout=2000
nx0=  long(lf.pixels_sio_image)
ny0=  long(lf.lines_sio_image)


; 
; Upscale landmask if landmask has been downscaled
;
sz   = size(image) 
nxi  = sz[1]
nyi  = sz[2]

if (nxi ne nx0) or (nyi ne ny0) then begin
  if printflag then begin
    print, 'Rescale image from ' + string( nxi, format='(I5)' ) +'x' + string( nxi, format='(I5)' ) +' to ' + $
                                   string( nx0, format='(I5)' ) +'x' + string( ny0, format='(I5)' )
  endif
  if (nx0 mod nxi) eq 0 and (ny0 mod nyi) then begin 
    image =  rebin(   image, nx0, ny0 )
  endif else begin
    image =  congrid( image, nx0, ny0 )
  endelse
endif

;
; Resize 
;
if nx0 gt maxsize or ny0 gt maxsize and avefactor lt 5 then begin
  ;
  ; kuldge for very big image
  ;
  if printflag then print, 'WARNING. Image very large. Setting avefactor to 5'
  avefactor=5
endif
if printflag and not keyword_set(mask) then begin
  if nx0 gt 2000 and ny0 gt 2000 then begin
    if printflag then print, 'Computing image stats from 2000x2000 subset'
    tmp= image[0:1999,0:1999]
  endif else begin
    if printflag then print, 'Computing image stats from original image'
    tmp= image
  endelse
  w= where(tmp gt 0, count )
  if count gt 0 then begin
    mean_image= mean(tmp[w])
    std_image=  stdev(tmp[w])
    print, 'Stats of original input image'
    print, 'Mean=' + string(mean_image)
    print, 'SD=  ' + string(std_image)
  endif
endif
   
   
if avefactor gt 1 then begin
  if not keyword_set(usecongrid) then begin
    if printflag then print, 'Reszing image using REBIN...'
    nx=     nx0
    ny=     ny0
    nx1=    long(nx)  / avefactor
    ny1=    long(ny)  / avefactor
    nx2=    nx1 *  avefactor 
    ny2=    ny1 *  avefactor 
    image=  rebin( image[0:nx2-1,0:ny2-1], nx1, ny1)
  endif else begin
    if printflag then print, 'Reszing image using CONGRID...'
    nx=     nx0 
    ny=     ny0
    nx1=    long(nx)  / avefactor
    ny1=    long(ny)  / avefactor
    image=  smooth( image, [avefactor, avefactor])
    image=  congrid( image, nx1, ny1)
  endelse
  if printflag and not keyword_set(mask) then begin
    w= where(image gt 0, count )
    if count gt 0 then begin
      mean_image= mean(image[w])
      std_image=  stdev(image[w])
      print, 'Stats of averaged input image'
      print, 'Mean=' + string(mean_image)
      print, 'SD=  ' + string(std_image)
    endif
  endif
endif 
;
; Convert to amplitude
;
if keyword_set(amplitude) then begin
  w= where( image gt 0, count)
  if count gt 0 then begin
    if printflag then print, 'Computing the square root of the image...'
    image[w]= sqrt(image[w])
  endif
endif

;
; Convert to dB
;
if keyword_set(kmzdb) then begin
  w= where( image gt 0, count)
  if count gt 0 then begin
    if printflag then print, 'Converting to image from linear to dB...'
    image[w]= 10*alog10(image[w])
  endif
endif

;
; Compute NRCS PDF
;
if not keyword_set(cutoff) then cutoff=max(image)
if keyword_set(nrcs) then begin
  if printflag then print, 'Computing NCRS probabilty density function...'
  w=where(image gt 0 and image lt cutoff, count)
  if count gt 0 then begin
    h=float( hist(image[w],nrcs_values, nbin=100) )
    h=h/total(h)
  endif else begin
    nrcs=0
  endelse
endif
nx=    nx0 / avefactor 
ny=    ny0 / avefactor 

;
; Make land mask file have single values for land or water
;
if keyword_set(mask) then begin
  w=where(image ge 0, count)
  if count gt 0 then image[w]=255
  w=where(image lt 0, count)
  if count gt 0 then image[w]=0
endif else begin 
  
endelse
;
; Make ice mask file easier to interpet
;
if keyword_set(ice) then begin
  w=where(image le 1, /null)
  image[w] = 0
  w=where(image eq 2, /null)
  image[w] = 80
  w=where(image eq 3, /null)
  image[w] = 150
  w=where(image eq 4, /null)
  image[w] = 255
endif

if keyword_set(flatten) and  not keyword_set(mask) and keyword_set(ice) then begin
  image=image-opfit2d(image,2,2)
endif
;
; bottom edge
;
lonmin= ans_poly_cal( double(0), double(0), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
lonmax= lonmin
latmin= ans_poly_cal( double(0), double(0), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
latmax= latmin

; Bottom
y= double(0L)
for i=0L, nx0 do begin              
  lng=    ans_poly_cal( double(i), y, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat=    ans_poly_cal( double(i), y, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lonmin= min( [lonmin, lng] ) 
  lonmax= max( [lonmax, lng] )
  latmin= min( [latmin, lat] ) 
  latmax= max( [latmax, lat] )  
endfor


; top

y= double(ny0)
for i=0L, nx0 do begin
  lng=    ans_poly_cal( double(i), y, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat=    ans_poly_cal( double(i), y, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lonmin= min( [lonmin, lng] ) 
  lonmax= max( [lonmax, lng] )
  latmin= min( [latmin, lat] ) 
  latmax= max( [latmax, lat] )  
endfor  


; left

x= double(0L)
for j=0L, ny0 do begin
  lng=    ans_poly_cal( x, double(j), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat=    ans_poly_cal( x, double(j), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lonmin= min( [lonmin, lng] ) 
  lonmax= max( [lonmax, lng] )
  latmin= min( [latmin, lat] ) 
  latmax= max( [latmax, lat] )  
endfor 


; right

x= double(nx0)
for j=0L, ny0 do begin
  lng=    ans_poly_cal( x, double(j), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat=    ans_poly_cal( x, double(j), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lonmin= min( [lonmin, lng] ) 
  lonmax= max( [lonmax, lng] )
  latmin= min( [latmin, lat] ) 
  latmax= max( [latmax, lat] )  
endfor 

;
if printflag then begin
  print, 'Creating PNG image'
  print, 'Longitude from ' + string(lonmin) + ' to ' + string(lonmax)
  print, 'Latitude  from ' + string(latmin) + ' to ' + string(latmax)
endif

;
; Create longitude and latitude arrays
; for rectilinear grid
;

slope=   (lonmax-lonmin) / (xout-1.0)
rlon=    findgen(xout) * slope + lonmin
slope=   (latmax-latmin) / (yout-1.0)  
rlat=    findgen(yout) * slope + latmin

img=     fltarr(xout,yout) 
if keyword_set(incid) then incidence=img

istep=   xout/10.
xout_s=  string(xout, format='(I6)') 
maximg=  max(image)
minimg=  maximg/250.
for i=0L, xout-1 do begin
  if i mod istep eq 0 and printflag then begin
    out= 'Resample image     to rectilinear longitude and latitude. '
    out=  out + string(i, format='(I6)') + ' of ' + xout_s
    print, out
  endif
  for j=0L, yout-1 do begin
    is=  ans_poly_cal( rlon[i], rlat[j], lf.i_coef, lf.i_xexp, lf.i_yexp) 
    is=  round( is / avefactor )
    if is gt 0 and is lt nx then begin
      js=  ans_poly_cal( rlon[i], rlat[j], lf.j_coef, lf.j_xexp, lf.j_yexp) 
      js=  round( js / avefactor )
      if js gt 0 and js lt ny then img[i,j]=  image[is,js]
    endif
  endfor
endfor

if keyword_set(incid) then begin
  for i=0L, xout-1 do begin
    if i mod istep eq 0 and printflag then begin
      out= 'Resample incidence to rectilinear longitude and latitude. '
      out=  out + string(i, format='(I6)') + ' of ' + xout_s
      print, out
    endif
    for j=0L, yout-1 do begin
      is=              ans_poly_cal( rlon[i], rlat[j], lf.i_coef, lf.i_xexp, lf. i_yexp) 
      js=              ans_poly_cal( rlon[i], rlat[j], lf.j_coef, lf.j_xexp, lf. j_yexp) 
      incidence[i,j]=  ans_poly_cal( is, js, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
    endfor
  endfor
  if printflag then begin
    print, 'Incident angle statistics.'
    print, 'Mean: ', mean(incidence)
    print, 'Max:  ', max(incidence)
    print, 'Min:  ', min(incidence)
  endif
endif


if printflag and not keyword_set(mask) then begin
    w= where(img gt 0, count )
    if count gt 0 then begin
      mean_image= mean(img[w])
      std_image=  stdev(img[w])
      print, 'Stats of reseample image'
      print, 'Mean=' + string(mean_image)
      print, 'SD=  ' + string(std_image)
    endif
endif
;
; Create KMZ file
;
if not keyword_set(working_dir) then working_dir='./'
working_dir=  working_dir +  '/kmz_'+dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + '-' + string(randomu(seed))
working_dir=  strcompress( working_dir, /remove_all) +'/'
sdatafile=    getwrd(datafile, 0, /last, del='/')
nl=           string( byte(10) )

imgdir=       working_dir + '/images'
file_mkdir,   imgdir
kmlfile=      working_dir + 'doc.kml'
newfile=      working_dir + '/' + 'images/image.png'
snewfile=     getwrd(newfile, 0, /last, del='/')
nrcsfile=     working_dir + '/' + 'images/nrcs.png'
snrcsfile=    getwrd(nrcsfile, 0, /last, del='/')
meanlong=     ( lonmin  + lonmax ) / 2.0
meanlat=      ( latmin  + latmax ) / 2.0

if printflag then begin
  print, 'Image directory for KMZ creation=' + imgdir
  print, 'KML file for KMZ creation=       ' + kmlfile
  print, 'PNG file for KMZ creation=       ' + newfile
  print, 'PNG file for KMZ creation=       ' + snewfile
  print, 'PNG file for KMZ creation=       ' + nrcsfile
  print, 'PNG file for KMZ creation=       ' + snewfile
endif

;
; Convert meanlong into -/+180 of necessary

if meanlong gt 180 then meanlong= meanlong-360.



tt=      '<?xml version="1.0" encoding="UTF-8"?>'                + nl
tt= tt + '<kml xmlns="http://earth.google.com/kml/2.0">'         + nl
tt= tt + '<Folder>'        + nl
tt= tt + '  <name> '+ sdatafile+ '</name>'                       + nl
tt= tt + '  <description> SIO Image </description>'        + nl
tt= tt + '  <LookAt>'                                            + nl
tt= tt + '    <longitude>' + string(meanlong) + '</longitude>'   + nl
tt= tt + '    <latitude> ' + string(meanlat)  + '</latitude> '   + nl
tt= tt + '    <range>600000</range>'                             + nl
tt= tt + '    <tilt>0</tilt>'                                    + nl
tt= tt + '    <heading>0</heading>'                              + nl
tt= tt + '  </LookAt>'                                           + nl


sz= size(img)
nx= sz[1]
ny= sz[2]
set_plot, 'z'
device, set_resolution=[nx,ny]
w= where(img le 0, count)
;if not keyword_set(mask) then img=ls(img)
if count gt 0 then img[w]=0
loadct, 0
erase,  0

threshold= max(img)
if strpos(datafile,'image.dat') ge 0 then begin
  w= where(img gt 0, count)
  if count gt 0 then begin
    mean_image=  mean(img[w])
    sd=          stdev(img[w]) 
    threshold=   min([mean_image + 3.*sd, cutoff])
  endif else begin
    threshold= 1
  endelse
  if printflag then print, threshold
  tvscl,     ls( img <  threshold, /quiet)
endif else  begin
  tvscl,     img
endelse
if keyword_set(incid) then begin
  istep= nx/7
  jstep= ny/7
  for i=0L, nx-1L, istep do begin
    for j=0L, ny-1L, jstep do begin 
      incid_str=string(incidence[i,j], format='(F6.3)')
      if incidence[i,j] ne 0 then $
          xyouts, i, j, incid_str, /dev, charsize=1, align=0.5
    endfor
   if printflag then  print, 'Writing incidence angle on image. ' +$
       string(i, format='(I6)') + ' ' + incid_str
  endfor
endif
tvlct,   r, g, b, /get
z=       tvrd()
write_png, newfile, z, r, g, b, TRANSPARENT = [0,1]


;if lonmin gt 180 then lonmin= lonmin-360.0
;if lonmax gt 180 then lonmax= lonmax-360.0
tt= tt + '  <Folder>'                                     + nl
tt= tt + '  <name> ' + sdatafile +    ' NRCS   </name>'   + nl
tt= tt + '  <GroundOverlay>'                              + nl
tt= tt + '    <name> NRCS image                  </name>' + nl
tt= tt + '    <color>ffffffff</color>'                    + nl
tt= tt + '    <Icon>'                                     + nl
tt= tt + '      <href> ' + 'images/' + snewfile + '</href>'  + nl
tt= tt + '      <viewBoundScale>0.75</viewBoundScale>'    + nl
tt= tt + '    </Icon>'                                    + nl
tt= tt + '    <LatLonBox>'                                + nl
tt= tt + '      <north>' + string(latmax)+ '</north>'     + nl
tt= tt + '      <south>' + string(latmin)+ '</south>'     + nl
tt= tt + '      <west>'  + string(lonmin)+ '</west>'      + nl
tt= tt + '      <east>'  + string(lonmax)+ '</east>'      + nl
tt= tt + '    </LatLonBox>'                               + nl
tt= tt + '  </GroundOverlay>'                             + nl
tt= tt + '  </Folder>'                                    + nl

if keyword_set(plotlook) then begin
  ;
  ; Plot arrows for radar look direction
  ;
  if printflag then print, 'Plotting radar look direction..'
  ; Determine diagonal distance for arrow scaling. Arrow is
  ; 1/10 of distance along the bottom of the image
  ll2rb, ans_poly_cal(double(0),   double(0), lf.lon_coef, lf.lon_xexp, lf.lon_yexp), $
         ans_poly_cal(double(0),   double(0), lf.lat_coef, lf.lat_xexp, lf.lat_yexp), $
         ans_poly_cal(double(nx0), double(0), lf.lon_coef, lf.lon_xexp, lf.lon_yexp), $
         ans_poly_cal(double(nx0), double(0), lf.lat_coef, lf.lat_xexp, lf.lat_yexp), $
         dist, bearing
  dist= dist / 10.0 
  nx2=  nx0 / 2L 
  ny2=  ny0 / 2L
  tt=   tt + '  <Folder>'                                                   + nl
  tt=   tt + '    <name> Look Direction Arrows  </name>'   + nl
  for m=0L, 2L do begin
    x= m* double(nx2)
    for k=0L, 2L do begin
      y= k * double(ny2) 
      tt= tt + '    <Placemark>'                   + nl 
      tt= tt + '      <visibility>1</visibility>'  + nl                
      tt= tt + '      <LineString>'                + nl               
      tt= tt + '        <coordinates>'             + nl  
      dir=  ans_poly_cal( double(x), y, lf.rlook_coef,lf.rlook_xexp, lf.rlook_yexp)
      if printflag then print, 'LOOK DIRECTION:' , dir
      lngp= ans_poly_cal( x, y, lf.lon_coef, lf.lon_xexp,   lf.lon_yexp)
      latp= ans_poly_cal(x, y, lf.lat_coef, lf.lat_xexp,   lf.lat_yexp)
      rb2ll, lngp, latp, dist, dir, lngn, latn   
      if printflag then print, nx2, y, dir,lngp, latp, lngp, latn
      if lngp lt  -90 and lngn gt  90  then lngp=360 + lngp
      if lngp gt   90 and lngn lt  90  then lngn=360 + lngn
      dir1= ( dir + 270 ) mod 360 ; -90 + 360 take mod 360
      dir2= ( dir + 450 ) mod 360 ; +90 + 360 take mod 360
      dist4= dist / 4.0
      dist2= dist / 2.0
      rb2ll, lngn, latn, dist4, dir1, lngn2, latn2
      rb2ll, lngp, latp, (dist + dist2), dir, lngn3, latn3
      rb2ll, lngn, latn, dist4, dir2, lngn4, latn4
      ; 
      ; Convert to -180 to 180 range
      ;
      if lngp  gt 180 then  lngp=   lngp  - 360
      if lngn  gt 180 then  lngn=   lngn  - 360
      if lngn2 gt 180 then  lngn2=  lngn2 - 360
      if lngn3 gt 180 then  lngn3=  lngn3 - 360
      if lngn4 gt 180 then  lngn4=  lngn4 - 360
      tt= tt + '       ' + string(lngp,  format='(F15.8)') + ',' + $
                           string(latp,  format='(F15.8)') + ', 0' + nl
      tt= tt + '       ' + string(lngn,  format='(F15.8)') + ',' + $
                           string(latn,  format='(F15.8)') + ', 0' + nl
      tt= tt + '       ' + string(lngn2, format='(F15.8)') + ',' + $
                           string(latn2, format='(F15.8)') + ', 0' + nl
      tt= tt + '       ' + string(lngn3, format='(F15.8)') + ',' + $
                           string(latn3, format='(F15.8)') + ', 0' + nl
      tt= tt + '       ' + string(lngn4, format='(F15.8)') + ',' + $
                           string(latn4, format='(F15.8)') + ', 0' + nl
      tt= tt + '       ' + string(lngn,  format='(F15.8)') + ',' + $
                           string(latn,  format='(F15.8)') + ', 0' + nl 
      tt= tt + '        </coordinates>' + nl                            
      tt= tt + '      </LineString>'    + nl                               
      tt= tt + '    </Placemark>'       + nl
    endfor
  endfor
  tt=   tt + '  </Folder>' +nl
endif

if keyword_set(nrcs) gt 0 then begin
  nrcsfile= working_dir + 'images/nrcs.png'
  set_plot, 'z'
  device, set_resolution=[400, 400]
  loadct, 0
  tvlct, 255, 255, 255, 255 ; Set color 255 to white	
  tvlct,   0,   0,   0,  0  ; Set color   0 to black
  tvlct,   r, g, b, /get
  erase, 255
  ;
  ; Compute NRCS probability distribution
  ;
  ww=where(h gt 0,count)
  xt= 'NRCS (linear)'
  if keyword_set(kmzdb) then xt= 'NRCS (dB)'
  if count gt 0 then begin
    plot_io, nrcs_values[ww], h[ww], $
             xtitle=   xt,           $
             ytitle=   'PDF',        $
             color=    0,            $
             /noerase,               $
             /xstyle,                $
             xrange=   [0, max(nrcs_values(ww)) ]
  endif
  z=       tvrd()
  write_png, nrcsfile, z, r, g, b, transparent=[0,1]
  tt= tt +  '  <Folder>'                                              + nl
  tt= tt + '     <name> NRCS Probability Density Function  </name>'   + nl
  tt= tt +  '    <ScreenOverlay id="Wind Speed Legend">'              + nl
  tt= tt +  '     <visibility>1</visibility>'                         + nl
  tt= tt +  '     <Icon>'                                             + nl
  tt= tt +  '       <href>' + ' images/' + snrcsfile + '</href>'      + nl
  tt= tt +  '     </Icon>'                                            + nl
  tt= tt +  '     <overlayXY x="0" y="0" xunits="fraction" yunits="fraction"/>' + nl
  tt= tt +  '     <screenXY  x="0" y="0" xunits="fraction" yunits="fraction"/>'+ nl
  tt= tt +  '    </ScreenOverlay>'                                   + nl 
  tt= tt +  '  </Folder>'                                            + nl 
endif
tt= tt + '</Folder>'                                                 + nl
tt= tt + '</kml>'  
close,  1
;
; Write KML file
;
openw,  1, kmlfile
printf, 1, tt
close,  1
cd, current=pwd ;Puts current directory into pwd
cd,  working_dir
if  strpos(outputfile,'/') lt 0 then outputfile= pwd  + '/'  + outputfile
cmd= 'zip -q ' + outputfile + ' doc.kml  images/* '
if printflag then print, cmd
spawn, cmd
cd, pwd

if printflag then begin
  print, 'Outputfile: ' + outputfile
  if working_dir ne './' then print, 'Removing:   ' + working_dir
endif

if working_dir ne './' then begin
  file_delete,  working_dir, /recursive
endif

end
