;----------------------------------------------------------
;+
; NAME: ANS_SIO2l1CDF'
; PURPOSE:'
;   This program reads data in SIO format and generates a 
;   corresponding netCDF file file
; CALLING SEQUENCE:'
;   ans_sio2l1cdf, datafile, infofile, landfile, outputfile
; INPUT:'
;   datafile=     The name of the SIO NRCS for land mask file
;   infofile=     The name of the SIO info file
;   landfile=     The name of the land mask file.
; OUTPUT:'
;   outputfile=   Level-1 netCDF file
; KEYWORDS:
;    /quiet=      Do not print intermediate results'
;    /help=       List help.'
; MODIFICATION
;    2010 Mar 08 Frank Monaldo. 
;    2010 May 22 Frank Monaldo. Quit program if varaibles are too
;                big for netCDF 3.6
;    2010 May 24 Check for scene identifier and reference number empty values.
;    2010 May 25 Frank Monaldo. Check all attribute for empty strings and fill with 
;                               BLANK
;                               Changing print statements to include Level-1 label
;    2010 Jun 14 Frank Monaldo. Corrected spelling
;    2010 Jun 14 Frank Monaldo. Add option for no parameters
;    2010 Dec 29 Frank Monaldo. Pass through silent
;    2011 May 23 Frank Monaldo. Add jsbegin and jsend
;    2011 Jun 02 Frank Monaldo. Add nrcs_slope nrcs_bias
;    2011 Jun 13 Frank Monaldo. Corrected help
;    2011 Oct 11 Frank Monaldo. Corrected print error
;    2012 Jul 18 Frank Monaldo. Removed redundant beam code check. Corrected spelling of info
; Copyright (C) 2010-12, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  ans_sio2l1cdf, datafile, infofile, landfile, outputfile,             $
                    quiet=quiet, help=help


if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_SIO2l1CDF'
  print, '  PURPOSE:'
  print, '    This program reads data in SIO format and generates a 
  print, '    corresponding netCDF file file
  print, '  CALLING SEQUENCE:'
  print, '    ans_sio2l1cdf, datafile, infofile, landfile, outputfile
  print, '  INPUT:'
  print, '    datafile=     The name of the SIO NRCS for land mask file
  print, '    infofile=     The name of the SIO info file
  print, '    landfile=     The name of the land mask file.
  print, '  OUTPUT:'
  print, '    outputfile=   Level-1 netCDF file
  print, '  KEYWORDS:
  print, '     /quiet=      Do not print intermediate results'
  print, '     /help=       List help.'
 
  return
endif

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
;
; Reading in data
;
res= file_search(datafile, count=count)
if count eq 0 then begin
  print, 'Cannot find data file:' + datafile
  print, 'Exiting ans_sio1l1cdf...'
  return
endif
res= file_search(infofile, count=count)
if count eq 0 then begin
  print, 'Cannot find data file:' + infofile
  print, 'Exiting ans_sio1l1cdf...'
  return
endif
ans_read_sio_data, datafile, image, quiet=quiet
;
; Check size
; 
total_bytes=float(n_elements(image)) * 4.0 
if total_bytes gt 2E9 then begin
  print, 'The image is too big for a netCDF file.
  print, 'Image size= ' + string(total_bytes)
  print, 'Not creating a netCDF file.'
  print, 'Exiting ans_sio2l1cdf
  return
endif
; 
ans_read_sio_info, infofile, info, quiet=quiet

res= file_search(landfile, count=count)
landflag=1
if count eq 0 then begin
  print, 'Cannot find data file:' + landfile
  landflag= 0
  msk= fix(image) * 0
endif else begin
  ans_read_sio_data, landfile, msk, quiet=quiet
endelse

sz=  size(msk)
nxx= sz(1)
nyy= sz(2)
;
; Write NetCDF file
;
id= NCDF_CREATe( outputfile, /clobber)  
;
; Do not fill the file with default data.
;
NCDF_CONTROL, id, /fil
;
; Global Attributes
;  
info.answrs_version=        stress(info.answrs_version,        'R',0,':','-')
info.acq_date_time=         stress(info.acq_date_time,         'R',0,':','-')
info.sio_create_date_time=  stress(info.sio_create_date_time,  'R',0,':','-')

if strlen( info.answrs_version             ) eq 0 then info.answrs_version             = 'BLANK'
if strlen( info.platform_name              ) eq 0 then info.platform_name              = 'BLANK'
if strlen( info.acq_date_time              ) eq 0 then info.acq_date_time              = 'BLANK'
if strlen( info.acq_date_time_js           ) eq 0 then info.acq_date_time_js           = 'BLANK'
if strlen( info.jsbegin                    ) eq 0 then info.jsbegin                    = 'BLANK'
if strlen( info.jsend                      ) eq 0 then info.jsend                      = 'BLANK'
if strlen( info.sio_create_date_time       ) eq 0 then info.sio_create_date_time       = 'BLANK'
if strlen( info.root_file                  ) eq 0 then info.root_file                  = 'BLANK'
if strlen( info.sensor_frequency_hz        ) eq 0 then info.sensor_frequency_hz        = 'BLANK'
if strlen( info.sensor_wavelength_m        ) eq 0 then info.sensor_wavelength_m        = 'BLANK'
if strlen( info.frequency_band             ) eq 0 then info.frequency_band             = 'BLANK'
if strlen( info.polarization               ) eq 0 then info.polarization               = 'BLANK'
if strlen( info.product_type               ) eq 0 then info.product_type               = 'BLANK'
if strlen( info.processing_agency          ) eq 0 then info.processing_agency          = 'BLANK'
if strlen( info.processing_facility        ) eq 0 then info.processing_facility        = 'BLANK'
if strlen( info.sar_processor_version      ) eq 0 then info.sar_processor_version      = 'BLANK'
if strlen( info.scene_identifier           ) eq 0 then info.scene_identifier           = 'BLANK'
if strlen( info.scene_reference_number     ) eq 0 then info.scene_reference_number     = 'BLANK'
if strlen( info.orbit_number               ) eq 0 then info.orbit_number               = 'BLANK'
if strlen( info.sar_platform_code          ) eq 0 then info.sar_platform_code          = 'BLANK'
if strlen( info.sensor_imaging_mode        ) eq 0 then info.sensor_imaging_mode        = 'BLANK'
if strlen( info.calibration_flag           ) eq 0 then info.calibration_flag           = 'BLANK'
if strlen( info.image_pixel_value_units    ) eq 0 then info.image_pixel_value_units    = 'BLANK'
if strlen( info.orbit_pass_flag            ) eq 0 then info.orbit_pass_flag            = 'BLANK'
if strlen( info.sar_beam_code              ) eq 0 then info.sar_beam_code              = 'BLANK'
if strlen( info.map_descriptor             ) eq 0 then info.map_descriptor             = 'BLANK'
if strlen( info.range_type                 ) eq 0 then info.range_type                 = 'BLANK'
if strlen( info.ref_ellipsoid              ) eq 0 then info.ref_ellipsoid              = 'BLANK'
if strlen( info.datum_shifts_m             ) eq 0 then info.datum_shifts_m             = 'BLANK'
if strlen( info.datum_scale                ) eq 0 then info.datum_scale                = 'BLANK'
if strlen( info.source_lonlat_fit          ) eq 0 then info.source_lonlat_fit          = 'BLANK'
if strlen( info.source_incid_fit           ) eq 0 then info.source_incid_fit           = 'BLANK'
if strlen( info.source_rlook_fit           ) eq 0 then info.source_rlook_fit           = 'BLANK'
if strlen( info.nrcs_slope                 ) eq 0 then info.nrcs_slope                 = 'BLANK'
if strlen( info.nrcs_bias                  ) eq 0 then info.nrcs_bias                  = 'BLANK'

NCDF_ATTPUT, id, /GLOBAL, 'sar_beam_code'                  , strcompress(lf.sar_beam_code , /remove_all)


NCDF_ATTPUT, id, /GLOBAL, 'ANSWRS_Version',                        info.answrs_version
NCDF_ATTPUT, id, /GLOBAL, 'Platform_Name',                         info.platform_name
NCDF_ATTPUT, id, /GLOBAL, 'Acquisition_Time',                      info.acq_date_time
NCDF_ATTPUT, id, /GLOBAL, 'Acquisition_Time_Julian_Seconds',       info.acq_date_time_js
NCDF_ATTPUT, id, /GLOBAL, 'Acquisition_Time_Start_Julian_Seconds', info.jsbegin
NCDF_ATTPUT, id, /GLOBAL, 'Acquisition_Time_End_Julian_Seconds',   info.jsend
NCDF_ATTPUT, id, /GLOBAL, 'Processing_Time',                       info.sio_create_date_time
NCDF_ATTPUT, id, /GLOBAL, 'SAR_File_or_Directory_Name',            info.root_file
NCDF_ATTPUT, id, /GLOBAL, 'Radar_Frequency_Hz',                    info.sensor_frequency_hz
NCDF_ATTPUT, id, /GLOBAL, 'Radar_Wavelength_m',                    info.sensor_wavelength_m
NCDF_ATTPUT, id, /GLOBAL, 'Radar_Band',                            info.frequency_band
NCDF_ATTPUT, id, /GLOBAL, 'Polarization',                          info.polarization
NCDF_ATTPUT, id, /GLOBAL, 'product_type',                          info.product_type
NCDF_ATTPUT, id, /GLOBAL, 'processing_agency',                     info.processing_agency          
NCDF_ATTPUT, id, /GLOBAL, 'processing_facility',                   info.processing_facility        
NCDF_ATTPUT, id, /GLOBAL, 'sar_processor_version',                 info.sar_processor_version      
NCDF_ATTPUT, id, /GLOBAL, 'scene_identifier',                      info.scene_identifier   
NCDF_ATTPUT, id, /GLOBAL, 'scene_reference_number',                info.scene_reference_number     
NCDF_ATTPUT, id, /GLOBAL, 'orbit_number',                          info.orbit_number       
NCDF_ATTPUT, id, /GLOBAL, 'sar_platform_code',                     info.sar_platform_code          
NCDF_ATTPUT, id, /GLOBAL, 'sensor_imaging_mode',                   info.sensor_imaging_mode        
NCDF_ATTPUT, id, /GLOBAL, 'calibration_flag',                      info.calibration_flag          
NCDF_ATTPUT, id, /GLOBAL, 'image_pixel_value_units',               info.image_pixel_value_units                                           
NCDF_ATTPUT, id, /GLOBAL, 'orbit_pass_flag',                       info.orbit_pass_flag                   
NCDF_ATTPUT, id, /GLOBAL, 'sar_beam_code',                         info.sar_beam_code             
NCDF_ATTPUT, id, /GLOBAL, 'map_descriptor',                        info.map_descriptor
NCDF_ATTPUT, id, /GLOBAL, 'range_type',                            info.range_type
NCDF_ATTPUT, id, /GLOBAL, 'ref_ellipsoid',                         info.ref_ellipsoid
NCDF_ATTPUT, id, /GLOBAL, 'datum_shifts_m',                        info.datum_shifts_m
NCDF_ATTPUT, id, /GLOBAL, 'datum_scale',                           info.datum_scale
NCDF_ATTPUT, id, /GLOBAL, 'source_lonlat_fit',                     info.source_lonlat_fit 
NCDF_ATTPUT, id, /GLOBAL, 'source_incid_fit',                      info.source_incid_fit 
NCDF_ATTPUT, id, /GLOBAL, 'source_rlook_fit',                      info.source_rlook_fit 
NCDF_ATTPUT, id, /GLOBAL, 'nrcs_slope',                            info.nrcs_slope
NCDF_ATTPUT, id, /GLOBAL, 'nrcs_bias',                             info.nrcs_bias
NCDF_ATTPUT, id, /GLOBAL, 'sar_beam_code',                         strcompress(info.sar_beam_code , /remove_all)

if printflag then begin
  print, ''
  print, 'Global Variables written by ans_sio2l1cdf...'
  print, 'ANSWRS_Version:                          ' +  info.answrs_version
  print, 'Platform_Name:                           ' +  info.platform_name
  print, 'Acquisition_Time:                        ' +  info.acq_date_time
  print, 'Acquisition_Time_Julian_Seconds:         ' +  info.acq_date_time_js
  print, 'Acquisition_Time_Start_Julian_Seconds:   ' +  info.jsbegin
  print, 'Acquisition_Time_End_Julian_Seconds:     ' +  info.jsend
  print, 'Processing_Time:                         ' +  info.sio_create_date_time
  print, 'SAR_File_or_Directory_Name:              ' +  info.root_file
  print, 'Radar_Frequency_Hz:                      ' +  info.sensor_frequency_hz
  print, 'Radar_Wavelength_m:                      ' +  info.sensor_wavelength_m
  print, 'Radar_Band:                              ' +  info.frequency_band
  print, 'Polarization:                            ' +  info.polarization
  print, 'product_type:                            ' +  info.product_type               
  print, 'processing_agency:                       ' +  info.processing_agency          
  print, 'processing_facility:                     ' +  info.processing_facility        
  print, 'sar_processor_version:                   ' +  info.sar_processor_version      
  print, 'scene_identifier:                        ' +  info.scene_identifier   
  print, 'scene_reference_number:                  ' +  info.scene_reference_number     
  print, 'orbit_number:                            ' +  info.orbit_number       
  print, 'sar_platform_code:                       ' +  info.sar_platform_code          
  print, 'sensor_imaging_mode:                     ' +  info.sensor_imaging_mode        
  print, 'calibration_flag:                        ' +  info.calibration_flag          
  print, 'image_pixel_value_units:                 ' +  info.image_pixel_value_units    
  print, 'orbit_pass_flag:                         ' +  info.orbit_pass_flag            
  print, 'sar_beam_code:                           ' +  info.sar_beam_code             
  print, 'map_descriptor:                          ' +  info.map_descriptor
  print, 'range_type:                              ' +  info.range_type
  print, 'ref_ellipsoid:                           ' +  info.ref_ellipsoid
  print, 'datum_shifts_m:                          ' +  info.datum_shifts_m
  print, 'datum_scale:                             ' +  info.datum_scale
  print, 'source_lonlat_fit:                       ' +  info.source_lonlat_fit 
  print, 'source_incid_fit:                        ' +  info.source_incid_fit 
  print, 'source_rlook_fit:                        ' +  info.source_rlook_fit 
  print, 'nrcs_slope:                              ' +  info.nrcs_slope
  print, 'nrcs_bias:                               ' +  info.nrcs_bias  
  print, ''
endif

;
; Declare dimensions
; 
sz=         size(msk)
nxx=        sz[1]
nyy=        sz[2]
sz=         size(image)
nx=         sz[1]
ny=         sz[2]
nlon_fit=   n_elements(info.lon_coef)
nlat_fit=   n_elements(info.lat_coef)
ni_fit=     n_elements(info.i_coef)
nj_fit=     n_elements(info.j_coef)
nincid_fit= n_elements(info.incid_coef)
nrlook_fit= n_elements(info.rlook_coef)


if printflag then print, 'Declaring dimensions for Level-1 netCDF file.'
nxid=          NCDF_DIMDEF(id, 'x',      nx        )
nyid=          NCDF_DIMDEF(id, 'y',      ny        )
nxmid=         NCDF_DIMDEF(id, 'xx',     nxx       )
nymid=         NCDF_DIMDEF(id, 'yy',     nyy       ) 
nlon_fitid=    NCDF_DIMDEF(id, 'xlon',   nlon_fit  )
nlat_fitid=    NCDF_DIMDEF(id, 'xlat',   nlat_fit  )
ni_fitid=      NCDF_DIMDEF(id, 'xi',     ni_fit    )
nj_fitid=      NCDF_DIMDEF(id, 'xj',     nj_fit    )
nincid_fitid=  NCDF_DIMDEF(id, 'xincid', nincid_fit)
nrlook_fitid=  NCDF_DIMDEF(id, 'xrlook', nrlook_fit)

 
;
; Scalar values
;
; 
if printflag then print, 'Declaring IDs for scalar quantities for Level-1 netCDF file in.'
scaling_id=                      NCDF_VARDEF(id, 'scaling',                       /float   )  
nominal_azimuth_resolution_m_id= NCDF_VARDEF(id, 'nominal_azimuth_resolution_m',  /float   )  
nominal_range_resolution_m_id=   NCDF_VARDEF(id, 'nominal_range_resolution_m',    /float   )  
pixel_spacing_m_id=              NCDF_VARDEF(id, 'pixel_spacing_m',               /float   )  
line_spacing_m_id=               NCDF_VARDEF(id, 'line_spacing_m',                /float   )  
heading_north_deg_id=            NCDF_VARDEF(id, 'heading_north_deg',             /float   )  
heading_image_deg_id=            NCDF_VARDEF(id, 'heading_image_deg',             /float   )  
range_swath_width_m_id=          NCDF_VARDEF(id, 'range_swath_width_m',           /float   )  
azimuth_swath_width_m_id=        NCDF_VARDEF(id, 'azimuth_swath_width_m',         /float   )  
sensor_altitude_m_id=            NCDF_VARDEF(id, 'sensor_altitude_m',             /float   )  
incid_angle_center_deg_id=       NCDF_VARDEF(id, 'incid_angle_center_deg',        /float   )  
slant_range_center_m_id=         NCDF_VARDEF(id, 'slant_range_center_m',          /float   )  
slant_range_near_m_id=           NCDF_VARDEF(id, 'slant_range_near_m',            /float   )  
lines_sio_image_id=              NCDF_VARDEF(id, 'lines_sio_image',               /long    )  
pixels_sio_image_id=             NCDF_VARDEF(id, 'pixels_sio_image',              /long    )  
lines_original_image_id=         NCDF_VARDEF(id, 'lines_original_image',          /long    )  
pixels_original_image_id=        NCDF_VARDEF(id, 'pixels_original_image',         /long    )  
looks_range_id=                  NCDF_VARDEF(id, 'looks_range',                   /float   )  
looks_azimuth_id=                NCDF_VARDEF(id, 'looks_azimuth',                 /float   )  
ref_semi_major_axis_m_id=        NCDF_VARDEF(id, 'ref_semi_major_axis_m',         /float   )  
ref_semi_minor_axis_m_id=        NCDF_VARDEF(id, 'ref_semi_minor_axis_m_',        /float   )  
basis_lon_fit_id=                NCDF_VARDEF(id, 'basis_lon_fit',                 /float   )  
deg_lonlat_fit_id=               NCDF_VARDEF(id, 'deg_lonlat_fit',                /float   )  
deg_incid_fit_id=                NCDF_VARDEF(id, 'deg_incid_fit',                 /float   )  
deg_rlook_fit_id=                NCDF_VARDEF(id, 'deg_rlook_fit',                 /float   )  


;
; Arrays
;
if printflag then print, 'Declaring IDs for array quantities for Level-1 netCDF file.'
sigmaid=                         NCDF_VARDEF(id, 'sigma',       [nxid,  nyid ], /float)
maskid=                          NCDF_VARDEF(id, 'mask',        [nxmid, nymid], /short)
lon_coefid=                      NCDF_VARDEF(id, 'lon_coef',    [nlon_fitid],   /double)
lon_xexpid=                      NCDF_VARDEF(id, 'lon_xexp',    [nlon_fitid],   /float)
lon_yexpid=                      NCDF_VARDEF(id, 'lon_yxep',    [nlon_fitid],   /float)
lat_coefid=                      NCDF_VARDEF(id, 'lat_coef',    [nlat_fitid],   /double)
lat_xexpid=                      NCDF_VARDEF(id, 'lat_xexp',    [nlat_fitid],   /float)
lat_yexpid=                      NCDF_VARDEF(id, 'lat_yexp',    [nlat_fitid],   /float)
i_coefid=                        NCDF_VARDEF(id, 'i_coef',      [ni_fitid],     /double)
i_xexpid=                        NCDF_VARDEF(id, 'i_xexp',      [ni_fitid],     /float)
i_yexpid=                        NCDF_VARDEF(id, 'i_yxep',      [ni_fitid],     /float)
j_coefid=                        NCDF_VARDEF(id, 'j_coef',      [nj_fitid],     /double)
j_xexpid=                        NCDF_VARDEF(id, 'j_xexp',      [nj_fitid],     /float)
j_yexpid=                        NCDF_VARDEF(id, 'j_yxep',      [nj_fitid],     /float)
incid_coefid=                    NCDF_VARDEF(id, 'incid_coef',  [nincid_fitid], /double)
incid_xexpid=                    NCDF_VARDEF(id, 'incid_xexp',  [nincid_fitid], /float)
incid_yexpid=                    NCDF_VARDEF(id, 'incid_yxep',  [nincid_fitid], /float)
rlook_coefid=                    NCDF_VARDEF(id, 'rlook_coef',  [nrlook_fitid], /double)
rlook_xexpid=                    NCDF_VARDEF(id, 'rlook_xexp',  [nrlook_fitid], /float)
rlook_yexpid=                    NCDF_VARDEF(id, 'rlook_yxep',  [nrlook_fitid], /float)
;
; units
;
if printflag then print, 'Declaring units for  quantities for Level-1 netCDF file.'


NCDF_ATTPUT, id, scaling_id,                      'units', 'NRCS / digital number'                    
NCDF_ATTPUT, id, nominal_azimuth_resolution_m_id, 'units', 'meters'
NCDF_ATTPUT, id, nominal_range_resolution_m_id,   'units', 'meters' 
NCDF_ATTPUT, id, pixel_spacing_m_id,              'units', 'meters'            
NCDF_ATTPUT, id, line_spacing_m_id,               'units', 'meters'               
NCDF_ATTPUT, id, heading_north_deg_id,            'units', 'degrees with respect to N'          
NCDF_ATTPUT, id, heading_image_deg_id,            'units', 'degrees with respect to N'          
NCDF_ATTPUT, id, range_swath_width_m_id,          'units', 'meters'        
NCDF_ATTPUT, id, azimuth_swath_width_m_id,        'units', 'meters'      
NCDF_ATTPUT, id, sensor_altitude_m_id,            'units', 'meters'          
NCDF_ATTPUT, id, incid_angle_center_deg_id,       'units', 'degrees'     
NCDF_ATTPUT, id, slant_range_center_m_id,         'units', 'meters'       
NCDF_ATTPUT, id, slant_range_near_m_id,           'units', 'meters'         
NCDF_ATTPUT, id, lines_sio_image_id,              'units', 'Dimension less'            
NCDF_ATTPUT, id, pixels_sio_image_id,             'units', 'Dimension less'           
NCDF_ATTPUT, id, lines_original_image_id,         'units', 'Dimension less'       
NCDF_ATTPUT, id, pixels_original_image_id,        'units', 'Dimension less'      
NCDF_ATTPUT, id, looks_range_id,                  'units', 'Dimension less'                
NCDF_ATTPUT, id, looks_azimuth_id,                'units', 'Dimension less'              
NCDF_ATTPUT, id, ref_semi_major_axis_m_id,        'units', 'meters'      
NCDF_ATTPUT, id, ref_semi_minor_axis_m_id,        'units', 'meters'      
NCDF_ATTPUT, id, basis_lon_fit_id,                'units', 'Dimension less'              
NCDF_ATTPUT, id, deg_lonlat_fit_id,               'units', 'Dimension less'             
NCDF_ATTPUT, id, deg_incid_fit_id,                'units', 'Dimension less'              
NCDF_ATTPUT, id, deg_rlook_fit_id,                'units', 'Dimension less'              
NCDF_ATTPUT, id, sigmaid,                         'units', 'Normalized Radar Cross Section.'
NCDF_ATTPUT, id, maskid,                          'units', 'Pixel from shore, positive values land, negative values water'
NCDF_ATTPUT, id, lon_coefid,                      'units', 'Coefficients to compute longitude in degs from pixel/lines'
NCDF_ATTPUT, id, lon_xexpid,                      'units', 'Longitude pixel exponents'
NCDF_ATTPUT, id, lon_yexpid,                      'units', 'Longitude line exponents'
NCDF_ATTPUT, id, lat_coefid,                      'units', 'Coefficients to compute latitude in degs from pixel/lines'
NCDF_ATTPUT, id, lat_xexpid,                      'units', 'Latitude pixel exponents'
NCDF_ATTPUT, id, lat_yexpid,                      'units', 'Latitude line exponents'  
NCDF_ATTPUT, id, i_coefid,                        'units', 'Coefficients to compute pixel from longitue/latitude'
NCDF_ATTPUT, id, i_xexpid,                        'units', 'Pixel longitude exponents'
NCDF_ATTPUT, id, i_yexpid,                        'units', 'Pixel latitude exponents'
NCDF_ATTPUT, id, j_coefid,                        'units', 'Coefficients to compute line from longitue/latitude'
NCDF_ATTPUT, id, j_xexpid,                        'units', 'Line longitude exponents'
NCDF_ATTPUT, id, j_yexpid,                        'units', 'Line latitude exponents'
NCDF_ATTPUT, id, incid_coefid,                    'units', 'Coefficients to compute incid in degs from pixel/lines' 
NCDF_ATTPUT, id, incid_xexpid,                    'units', 'Incid pixel exponents'
NCDF_ATTPUT, id, incid_yexpid,                    'units', 'Incid line exponents'  
NCDF_ATTPUT, id, rlook_coefid,                    'units', 'Coefficients to compute radar look direction in degs from pixel/lines'                                          
NCDF_ATTPUT, id, rlook_xexpid,                    'units', 'Radar look direction pixel exponents'
NCDF_ATTPUT, id, rlook_yexpid,                    'units', 'Radar look direction line exponents'  
;                                                 
; Long variable names;
; 
if printflag then print, 'Declaring long variable names for array quantities for Level-1 netCDF file.'

NCDF_ATTPUT, id, scaling_id,                      'long_name', 'NRCS / digital number'                    
NCDF_ATTPUT, id, nominal_azimuth_resolution_m_id, 'long_name', 'Nominal Azimuth Resolution (m)'
NCDF_ATTPUT, id, nominal_range_resolution_m_id,   'long_name', 'Nominal Range Resolution (m)' 
NCDF_ATTPUT, id, pixel_spacing_m_id,              'long_name', 'Pixel Spacing (m)'            
NCDF_ATTPUT, id, line_spacing_m_id,               'long_name', 'Line Spacing (m)'               
NCDF_ATTPUT, id, heading_north_deg_id,            'long_name', 'degrees with respect to N'          
NCDF_ATTPUT, id, heading_image_deg_id,            'long_name', 'degrees with respect to N'          
NCDF_ATTPUT, id, range_swath_width_m_id,          'long_name', 'meters'        
NCDF_ATTPUT, id, azimuth_swath_width_m_id,        'long_name', 'meters'      
NCDF_ATTPUT, id, sensor_altitude_m_id,            'long_name', 'meters'          
NCDF_ATTPUT, id, incid_angle_center_deg_id,       'long_name', 'degrees'     
NCDF_ATTPUT, id, slant_range_center_m_id,         'long_name', 'meters'       
NCDF_ATTPUT, id, slant_range_near_m_id,           'long_name', 'meters'         
NCDF_ATTPUT, id, lines_sio_image_id,              'long_name', 'Dimension less'            
NCDF_ATTPUT, id, pixels_sio_image_id,             'long_name', 'Dimension less'           
NCDF_ATTPUT, id, lines_original_image_id,         'long_name', 'Dimension less'       
NCDF_ATTPUT, id, pixels_original_image_id,        'long_name', 'Dimension less'      
NCDF_ATTPUT, id, looks_range_id,                  'long_name', 'Dimension less'                
NCDF_ATTPUT, id, looks_azimuth_id,                'long_name', 'Dimension less'              
NCDF_ATTPUT, id, ref_semi_major_axis_m_id,        'long_name', 'meters'      
NCDF_ATTPUT, id, ref_semi_minor_axis_m_id,        'long_name', 'meters'      
NCDF_ATTPUT, id, basis_lon_fit_id,                'long_name', 'Dimension less'              
NCDF_ATTPUT, id, deg_lonlat_fit_id,               'long_name', 'Dimension less'             
NCDF_ATTPUT, id, deg_incid_fit_id,                'long_name', 'Dimension less'              
NCDF_ATTPUT, id, deg_rlook_fit_id,                'long_name', 'Dimension less'  
NCDF_ATTPUT, id, sigmaid,                         'long_name', 'Normalized Radar Cross Section.''
NCDF_ATTPUT, id, maskid,                          'long_name', 'Interpolated land mask'
NCDF_ATTPUT, id, lon_coefid,                      'long_name', 'Coefficients to compute longitude in degs from pixel/lines'
NCDF_ATTPUT, id, lon_xexpid,                      'long_name', 'Longitude pixel exponents'
NCDF_ATTPUT, id, lon_yexpid,                      'long_name', 'Longitude line exponents'
NCDF_ATTPUT, id, lat_coefid,                      'long_name', 'Coefficients to compute latitude in degs from pixel/lines'
NCDF_ATTPUT, id, lat_xexpid,                      'long_name', 'Latitude pixel exponents'
NCDF_ATTPUT, id, lat_yexpid,                      'long_name', 'Latitude line exponents'  
NCDF_ATTPUT, id, i_coefid,                        'long_name', 'Coefficients to compute pixel from longitue/latitude'
NCDF_ATTPUT, id, i_xexpid,                        'long_name', 'Pixel longitude exponents'
NCDF_ATTPUT, id, i_yexpid,                        'long_name', 'Pixel latitude exponents'
NCDF_ATTPUT, id, j_coefid,                        'long_name', 'Coefficients to compute line from longitue/latitude'
NCDF_ATTPUT, id, j_xexpid,                        'long_name', 'Line longitude exponents'
NCDF_ATTPUT, id, j_yexpid,                        'long_name', 'Line latitude exponents'
NCDF_ATTPUT, id, incid_coefid,                    'long_name', 'Coefficients to compute incid in degs from pixel/lines' 
NCDF_ATTPUT, id, incid_xexpid,                    'long_name', 'Incid pixel exponents'
NCDF_ATTPUT, id, incid_yexpid,                    'long_name', 'Incid line exponents'  
NCDF_ATTPUT, id, rlook_coefid,                    'long_name', 'Coefficients to compute radar look direction in degs from pixel/lines'                                          
NCDF_ATTPUT, id, rlook_xexpid,                    'long_name', 'Radar look direction pixel exponents'
NCDF_ATTPUT, id, rlook_yexpid,                    'long_name', 'Radar look direction line exponents'  
;
; Exit control mode and switch to data model
;

ncdf_control, id, /endef
if printflag then print, 'File id=' + string(id,  format='(I8)')
;
; Write the scalar data
;
if printflag then begin
  print, ''
  print, 'Writing scalar data Level-1 netCDF file.'
  print, 'scaling_id                       ' + string(scaling_id                     , format='(I8)')+ '   ' +string(info.scaling                      ) 
  print, 'nominal_azimuth_resolution_m_id  ' + string(nominal_azimuth_resolution_m_id, format='(I8)')+ '   ' +string(info.nominal_azimuth_resolution_m )
  print, 'nominal_range_resolution_m_id    ' + string(nominal_range_resolution_m_id  , format='(I8)')+ '   ' +string(info.nominal_range_resolution_m   )
  print, 'pixel_spacing_m_id               ' + string(pixel_spacing_m_id             , format='(I8)')+ '   ' +string(info.pixel_spacing_m              )
  print, 'line_spacing_m_id                ' + string(line_spacing_m_id              , format='(I8)')+ '   ' +string(info.line_spacing_m               )
  print, 'heading_north_deg_id             ' + string(heading_north_deg_id           , format='(I8)')+ '   ' +string(info.heading_north_deg            )
  print, 'heading_image_deg_id             ' + string(heading_image_deg_id           , format='(I8)')+ '   ' +string(info.heading_image_deg            )
  print, 'range_swath_width_m_id           ' + string(range_swath_width_m_id         , format='(I8)')+ '   ' +string(info.range_swath_width_m          )
  print, 'azimuth_swath_width_m_id         ' + string(azimuth_swath_width_m_id       , format='(I8)')+ '   ' +string(info.azimuth_swath_width_m        )
  print, 'sensor_altitude_m_id             ' + string(sensor_altitude_m_id           , format='(I8)')+ '   ' +string(info.sensor_altitude_m            )
  print, 'incid_angle_center_deg_id        ' + string(incid_angle_center_deg_id      , format='(I8)')+ '   ' +string(info.incid_angle_center_deg       )
  print, 'slant_range_center_m_id          ' + string(slant_range_center_m_id        , format='(I8)')+ '   ' +string(info.slant_range_center_m         )
  print, 'slant_range_near_m_id            ' + string(slant_range_near_m_id          , format='(I8)')+ '   ' +string(info.slant_range_near_m           )
  print, 'lines_sio_image_id               ' + string(lines_sio_image_id             , format='(I8)')+ '   ' +string(info.lines_sio_image              )
  print, 'pixels_sio_image_id              ' + string(pixels_sio_image_id            , format='(I8)')+ '   ' +string(info.pixels_sio_image             )
  print, 'lines_original_image_id          ' + string(lines_original_image_id        , format='(I8)')+ '   ' +string(info.lines_original_image         )
  print, 'pixels_original_image_id         ' + string(pixels_original_image_id       , format='(I8)')+ '   ' +string(info.pixels_original_image        )
  print, 'looks_range_id                   ' + string(looks_range_id                 , format='(I8)')+ '   ' +string(info.looks_range                  )
  print, 'looks_azimuth_id                 ' + string(looks_azimuth_id               , format='(I8)')+ '   ' +string(info.looks_azimuth                )
  print, 'ref_semi_major_axis_m_id         ' + string(ref_semi_major_axis_m_id       , format='(I8)')+ '   ' +string(info.ref_semi_major_axis_m        )
  print, 'ref_semi_minor_axis_m_id         ' + string(ref_semi_minor_axis_m_id       , format='(I8)')+ '   ' +string(info.ref_semi_minor_axis_m        )
  print, 'basis_lon_fit_id                 ' + string(basis_lon_fit_id               , format='(I8)')+ '   ' +string(info.basis_lon_fit                )
  print, 'deg_lonlat_fit_id                ' + string(deg_lonlat_fit_id              , format='(I8)')+ '   ' +string(info.deg_lonlat_fit               )
  print, 'deg_incid_fit_id                 ' + string(deg_incid_fit_id               , format='(I8)')+ '   ' +string(info.deg_incid_fit                )
  print, 'deg_rlook_fit_id                 ' + string(deg_rlook_fit_id               , format='(I8)')+ '   ' +string(info.deg_rlook_fit                )   
  print, ''
endif

NCDF_VARPUT, id, scaling_id,                      info.scaling                   
NCDF_VARPUT, id, nominal_azimuth_resolution_m_id, info.nominal_azimuth_resolution_m
NCDF_VARPUT, id, nominal_range_resolution_m_id,   info.nominal_range_resolution_m   
NCDF_VARPUT, id, pixel_spacing_m_id,              info.pixel_spacing_m       
NCDF_VARPUT, id, line_spacing_m_id,               info.line_spacing_m               
NCDF_VARPUT, id, heading_north_deg_id,            info.heading_north_deg            
NCDF_VARPUT, id, heading_image_deg_id,            info.heading_image_deg            
NCDF_VARPUT, id, range_swath_width_m_id,          info.range_swath_width_m          
NCDF_VARPUT, id, azimuth_swath_width_m_id,        info.azimuth_swath_width_m        
NCDF_VARPUT, id, sensor_altitude_m_id,            info.sensor_altitude_m            
NCDF_VARPUT, id, incid_angle_center_deg_id,       info.incid_angle_center_deg       
NCDF_VARPUT, id, slant_range_center_m_id,         info.slant_range_center_m         
NCDF_VARPUT, id, slant_range_near_m_id,           info.slant_range_near_m           
NCDF_VARPUT, id, lines_sio_image_id,              info.lines_sio_image              
NCDF_VARPUT, id, pixels_sio_image_id,             info.pixels_sio_image             
NCDF_VARPUT, id, lines_original_image_id,         info.lines_original_image         
NCDF_VARPUT, id, pixels_original_image_id,        info.pixels_original_image        
NCDF_VARPUT, id, looks_range_id,                  info.looks_range            
NCDF_VARPUT, id, looks_azimuth_id,                info.looks_azimuth                
NCDF_VARPUT, id, ref_semi_major_axis_m_id,        info.ref_semi_major_axis_m        
NCDF_VARPUT, id, ref_semi_minor_axis_m_id,        info.ref_semi_minor_axis_m        
NCDF_VARPUT, id, basis_lon_fit_id,                info.basis_lon_fit            
NCDF_VARPUT, id, deg_lonlat_fit_id,               info.deg_lonlat_fit               
NCDF_VARPUT, id, deg_incid_fit_id,                info.deg_incid_fit                
NCDF_VARPUT, id, deg_rlook_fit_id,                info.deg_rlook_fit                
   



; Write the array data
;
if printflag then print, 'Writing array data Level-1 netCDF file.'
if n_elements(wnd1) eq 0 then wnd1= image * 0.0
NCDF_VARPUT, id, sigmaid,       image
NCDF_VARPUT, id, maskid,        msk
NCDF_VARPUT, id, lon_coefid,    info.lon_coef
NCDF_VARPUT, id, lon_xexpid,    info.lon_xexp  
NCDF_VARPUT, id, lon_yexpid,    info.lon_yexp  
NCDF_VARPUT, id, lat_coefid,    info.lat_coef  
NCDF_VARPUT, id, lat_xexpid,    info.lat_xexp  
NCDF_VARPUT, id, lat_yexpid,    info.lat_yexp  
NCDF_VARPUT, id, i_coefid,      info.i_coef   
NCDF_VARPUT, id, i_xexpid,      info.i_xexp    
NCDF_VARPUT, id, i_yexpid,      info.i_yexp    
NCDF_VARPUT, id, j_coefid,      info.j_coef    
NCDF_VARPUT, id, j_xexpid,      info.j_xexp    
NCDF_VARPUT, id, j_yexpid,      info.j_yexp    
NCDF_VARPUT, id, incid_coefid,  info.incid_coef
NCDF_VARPUT, id, incid_xexpid,  info.incid_xexp
NCDF_VARPUT, id, incid_yexpid,  info.incid_yexp
NCDF_VARPUT, id, rlook_coefid,  info.rlook_coef
NCDF_VARPUT, id, rlook_xexpid,  info.rlook_xexp
NCDF_VARPUT, id, rlook_yexpid,  info.rlook_yexp

;
; Note that error -62 may appear here if an object to too big
; The one case we had this error the image was 24101 x 28596 
; a four bytes per elements, the size is 2.8 GBytes. 
;


NCDF_CLOSE, id

if printflag then begin
  print, 'Data stored in file: ' + outputfile
  print, '---------------------------------------------
  print, ''
endif

end
