;-----------------------------------------------------
;+
; NAME: ANS_SIO_NRCS2AMP
; PURPOSE:
;    Convert SIO NRCS file to amplitude
; CALLING SEQUENCE
;   ans_sio_nrcs2amp, in_data_file, in_info_file, out_data_file, out_info_file
; INPUT
;   in_data_file=    Input  SIO data file in NRCS units
;   in_info_file=    Input  SIO info file.
; OUTPUT
;   out_data_file=   Output SIO data file in Sqrt(NRCS) units
;   out_info_file=   Output SIO info file in amp/intensity line changed
; KEYWORDS:
;   /quiet           No screen output
;   /help            Get help
;   error:           error, -1 is bad
;   ave_pixel=       ave_pixel  number of pixels to average (default=1)
;   ave_line=        ave_line   number of lines  to average (default=1)
;   in_mask_file=    In  mask file to be read if changing image size
;   out_mask_file=   Out mask file to be written
;   /noconvert       Do not convert to amplitude
; MODIFICATION HISTORY:
;   2011 Jun 29 Frank Monaldo.
;   2011 Jul 18 Frank Monaldo. Adding ability to change size
;   2011 Jul 20 Frank Monaldo. Debugging the conversion
;   2012 Aug 13 Frank Monaldo. Change n values to long
;
; Copyright (C) 2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_sio_nrcs2amp, in_data_file, in_info_file, out_data_file, out_info_file, $
                      in_mask_file= in_mask_file, out_mask_file=out_mask_file,  $
                      ave_pixel= ave_pixel,                                     $
                      ave_line=  ave_line,                                      $
                      noconvert= noconvert,                                     $
                      help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0

quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ANS_SIO_NRCS2AMP
  print, ' PURPOSE:
  print, '    Convert SIO NRCS file to amplitude
  print, ' CALLING SEQUENCE
  print, '   ans_sio_nrcs2amp, in_data_file, in_info_file, out_data_file, out_info_file
  print, ' INPUT
  print, '   in_data_file=    Input  SIO data file in NRCS units
  print, '   in_info_file=    Input  SIO info file.
  print, ' OUTPUT
  print, '   out_data_file=   Output SIO data file in Sqrt(NRCS) units
  print, '   out_info_file=   Output SIO info file in amp/intensity line changed
  print, ' KEYWORDS:
  print, '   /quiet           No screen output
  print, '   /help            Get help
  print, '   error:           error, -1 is bad
  print, '   ave_pixel=       ave_pixel  number of pixels to average (default=1)
  print, '   ave_line=        ave_line   number of lines  to average (default=1)
  print, '   in_mask_file=    In  mask file to be read if changing image size
  print, '   out_mask_file=   Out mask file to be written
  print, '   /noconvert       Do not convert to amplitude
  return
endif

if n_elements(ave_pixel) eq 0 then ave_pixel= 1
if n_elements(ave_line)  eq 0 then ave_line=  1


error=1



;---------------------------------------------------------------
;                Start SIO to NRCS to Amplitude Inversion
;---------------------------------------------------------------
;
; Check for files
;



imgfile= file_search(in_data_file, count=idata)
if idata eq 0 then begin
  print, 'Cannot find file ', in_data_file
  print, "Exiting ans_sio_nrcs2amp...'
  error=-1
  return
endif
  
ancfile= file_search(in_info_file, count=iinfo)
if idata eq 0 then begin 
  print, 'Cannot find file ', in_info_file
  print, "Exiting ans_sio_nrcs2amp...'
  error=-1
  return
endif


ans_read_sio_data,  in_data_file, image, quiet=quiet
ans_read_sio_info,  in_info_file, info,  quiet=quiet



if strupcase(info.image_pixel_value_units) eq 'AMPLITUDE' then begin
  print, 'Data already amplitude.'
  print, "Exiting ans_sio_nrcs2amp...'
  error=-1
  return
endif  

w= where(finite(image) and image ge 0, count)

if count eq 0 then begin
  print, 'No finite data.'
  print, "Exiting ans_sio_nrcs2amp...'
  error=-1
  return
endif  

if printflag then begin
  print, 'Before'
  print, 'Mean image: ' + string(mean(image[w]))  + ' Number of finite points >= 0 ' + string(count)
  print, 'STD  image: ' + string(stdev(image[w])) 
endif

if not keyword_set(noconvert) then image[w]=  sqrt( image[w] )

if printflag then begin
  print, 'After'
  print, 'Mean image: ' + string(mean(image[w]))  + ' Number of finite points >= 0 ' + string(count)
  print, 'STD  image: ' + string(stdev(image[w])) 
endif


if (ave_pixel eq 1) and (ave_line eq 1) then ans_write_nrcs_sio, image, out_data_file, 1.0

;
; Changing the size of the image and the location coefficients
;
if (ave_pixel gt 1) or (ave_line gt 1) then begin
  sz=        size(image)
  nx=        long(sz[1])
  ny=        long(sz[2])
  nx0=       long( nx / ave_pixel )  ; New image size
  ny0=       long( ny / ave_line  )  ; New image size
  xscale=    long( nx / nx0       )
  yscale=    long( ny / ny0       )
  nx1=       nx0 * xscale
  ny1=       ny0 * yscale
  printstr=  'Changing image from ' + string(nx)  + ' x ' + string(ny)
  printstr=  strcompress( printstr     + ' to  ' + string(nx0) + ' x ' + string(ny0) )
  if printflag then print, printstr
  image=     rebin( image[0: nx1-1, 0:ny1-1], nx0, ny0 )
  ans_write_nrcs_sio, image, out_data_file, 1.0
  ;
  ;
  res=       file_search( in_mask_file, count= count )
  if n_elements(in_mask_file) eq 0 then begin
    print, 'There is no in_mask_file specified'
    print, 'Exiting ans_sio_nrcs2amp'
    error=0
    return
  endif
  if count eq 0 then begin
    error=0
    print, 'Cannot find ' + in_mask_file
    print, 'Exiting ans_sio_nrcs2amp'
    return
  endif
  ans_read_sio_data,  in_mask_file, mask, quiet=quiet
  mask= rebin( mask, nx, ny )
  mask= rebin( mask[0: nx1-1, 0:ny1-1], nx0, ny0 )
  ans_write_nrcs_sio, mask, out_mask_file, 1.0, /twobyte
  ;
  ; Recompute polynomial conversions
  ;
  n= n_elements(info.lat_coef)
  for i=0, n-1 do info.lat_coef[i]=   info.lat_coef[i]   * ave_pixel^info.lat_xexp[i]   *  ave_line^info.lat_yexp[i]
  n= n_elements(info.lon_coef)
  for i=0, n-1 do info.lon_coef[i]=   info.lon_coef[i]   * ave_pixel^info.lon_xexp[i]   *  ave_line^info.lon_yexp[i]
  n= n_elements(info.rlook_coef)
  for i=0, n-1 do info.rlook_coef[i]= info.rlook_coef[i] * ave_pixel^info.rlook_xexp[i] *  ave_line^info.rlook_yexp[i]
  n= n_elements(info.incid_coef)                                                     
  for i=0, n-1 do info.incid_coef[i]= info.incid_coef[i] * ave_pixel^info.incid_xexp[i] *  ave_line^info.incid_yexp[i]
  info.i_coef= info.i_coef / ave_pixel
  info.j_coef= info.j_coef / ave_line
  pixel_spacing_m=         info.pixel_spacing_m * ave_pixel
  line_spacing_m=          info.line_spacing_m  * ave_line
  range_swath_width_m=     float(nx0) * pixel_spacing_m
  azimuth_swath_width_m=   float(ny0) * line_spacing_m
  looks_range=             info.looks_range   * ave_pixel
  looks_azimuth=           info.looks_azimuth * ave_line
endif

s= getfile( in_info_file)
n= n_elements(s)

for i=0, n-1 do begin
  first=  getwrd( s[i], 0, del='%' )
  if strupcase(first) eq 'INTENSITY' and (not keyword_set(noconvert) ) then begin
     s[i]=   '               amplitude % image pixel value units (note: amplitude==magnitude; intensity==power==squared magnitude)'
  endif
endfor



if ave_pixel gt 1 or ave_line gt 1 then begin
  ;
  ; Create array of second items original file
  second_item= strarr(n)  
  for i=1, n-1 do begin
    second_item[i]=  strcompress(getwrd(s[i], 1, del='%'), /remove_all )
    second_item[i]=  strlowcase(second_item[i] )  
  endfor
  ;  
  ; range resolution correction
  ;
  second=   'element/rangespacing(m)'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]= string(pixel_spacing_m,      format='(F24.6)') + ' % element/range spacing (m)' 
    endif
  endfor  
  ;
  ; azimuth resolution correction
  ;
  second=   'line/azimuthspacing(m)'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]= string(line_spacing_m,      format='(F24.6)') + ' % line/azimuth spacing (m)' 
    endif
  endfor  
  ;
  ; range swath width correction
  ;    
  second=   'rangeswathwidth(m)' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]= string(range_swath_width_m,       format='(F24.6)') + ' % range swath width (m)' 
    endif
  endfor
  ;
  ; azimuth swath width correction
  ;
  second=   'azimuthswathwidth(m)'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(azimuth_swath_width_m,     format='(F24.6)') + ' % azimuth swath width (m)' 
    endif
  endfor
  ;
  ; Total lines in SIO image correction
  ;
  second=  'totalnumberoflinesinsioimage'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(ny0,               format='(I24)')   + ' % total number of lines in sio image' 
    endif
  endfor
  ;
  ; Total numer of elements in SIO image correction
  ;
  second=  'totalnumberofelements(perline)insioimage'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(nx0,               format='(I24)')   + ' % total number of elements (per line) in sio image'
    endif
  endfor  
  ;
  ; Lines in SIO image correction
  ;
  second= 'maximumnumberoflinesinmaster(original)ceosimage'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(ny0,               format='(I24)')   + ' % maximum number of lines in master (original) CEOS image'
    endif
  endfor  
  ;
  ; Lines in SIO image correction
  ;
  second= 'maximumnumberofelementsinmaster(original)ceosimage'
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(nx0,               format='(I24)')   + ' % maximum number of elements in master (original) CEOS image'
    endif
  endfor  
  ;
  ; UL inner image true corner line location
  ;
  second=  'ulinnerimagetruecornerlinelocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(ny0,               format='(I24)')   + ' % UL inner image true corner line location'
    endif
  endfor  
  ;
  ; UR inner image true corner line location
  ;
  second=  'urinnerimagetruecornerlinelocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(ny0,               format='(I24)')   + ' % UR inner image true corner line location'
    endif
  endfor 
  ;
  ; UR inner image true corner element location
  ;
  second=  'urinnerimagetruecornerelementlocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(nx0,               format='(I24)')   + ' % UR inner image true corner element location'
    endif
  endfor  
  ;
  ; LR inner image true corner element location
  ;
  second=  'lrinnerimagetruecornerelementlocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(nx0,               format='(I24)')   + ' % LR inner image true corner element location'
    endif
  endfor 
  ;
  ; center inner image line location
  ;
  second=  'centerinnerimagelinelocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(ny0/2,               format='(I24)')   + ' % center inner image line location'
    endif
  endfor 
  ;
  ; center inner image element location
  ;
  second=  'centerinnerimageelementlocation' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(nx0/2,               format='(I24)')   + ' % center inner image element location'
    endif
  endfor 
  ;
  ; Looks range
  ;
  second=  'numberofrangelooksinoriginalimage' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(looks_range,       format='(F24.3)') + ' % Number of range   looks in original image
    endif
  endfor  
  ;
  ; Looks azimuth
  ;
  second=  'numberofazimuthlooksinoriginalimage' 
  for i=0, n-1 do begin 
    if second eq second_item[i] then begin
      s[i]=  string(looks_azimuth,     format='(F24.3)') + ' % Number of azimuth looks in original image
    endif
  endfor  
  ;
  ; ==============================================
  ;  TRANSFORM COEFFICIENTS
  ; ==============================================
  ;
  ;  Latitude from i/j
  ;
  n1= n_elements(info.lat_coef)
  tt1=    ' % latitude transform coef for element^'
  tt2=    ' line^'
  for i=0, n1-1 do begin
    tt= string(info.lat_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.lat_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.lat_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,   1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if sec2 eq sec1 then s[j]= tt
    endfor
  endfor
  ; 
  ; Longitude form i/j
  ;
  n1= n_elements(info.lon_coef)
  tt1=    ' % longitude (-180to180) transform coef for element^'
  tt2=    ' line^'
  for i=0, n1-1 do begin
    tt= string(info.lon_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.lon_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.lon_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,   1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if sec2 eq sec1 then s[j]= tt
    endfor
  endfor
  ;
  ; Longitude frm i/j different bias
  ;
  n1= n_elements(info.lon_coef)
  tt1=    ' % longitude (0to360) transform coef for element^'
  tt2=    ' line^'
  for i=0, n1-1 do begin
    tt= string(info.lon_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.lon_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.lon_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,   1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if sec2 eq sec1 then s[j]= tt
    endfor
  endfor
  ; 
  ; Incident angle from i/j
  ;
  n1= n_elements(info.incid_coef)
  tt1=    ' % incident angle (deg) transform coef for element^'
  tt2=    ' line^'
  for i=0, n1-1 do begin
    tt= string(info.incid_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.incid_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.incid_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,   1, del='%'),/remove_all)
      sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
    endfor
  endfor
  ;
  ; Radar look angle from i/j
  ;
  n1= n_elements(info.rlook_coef)
  tt1=    ' % radar look direction (deg) transform coef for element^'
  tt2=    ' line^'
  for i=0, n1-1 do begin
    tt= string(info.rlook_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.rlook_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.rlook_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,   1, del='%'),/remove_all)
      sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
    endfor
  endfor
  ;
  ; i from latitude and longitude
  ;
  n1= n_elements(info.i_coef)
  tt1=    ' % element transform coef for longitude^' 
  tt2=    ' latitude^'
  for i=0, n1-1 do begin
    tt= string(info.i_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.i_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.i_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,  1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if sec2 eq sec1 then s[j]= tt
    endfor
  endfor
  ;
  ; j from latitude and longitude
  ;
  n1= n_elements(info.j_coef)
  tt1=    ' % line transform coef for longitude^' 
  tt2=    ' latitude^'
  for i=0, n1-1 do begin
    tt= string(info.j_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(info.j_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(info.j_yexp(i), format='(I)'), /remove_all)
    sec2=''
    for j=0, n-1 do begin
      sec1=  strcompress(getwrd(tt,  1, del='%'),/remove_all)
      if strlen(s[j]) gt 0 then sec2=  strcompress(getwrd(s[j], 1, del='%'),/remove_all)
      if sec2 eq sec1 then s[j]= tt
    endfor
  endfor
endif ; End check if ave_pixel or ave_line gt 0


if printflag then print, 'Writing: ' + out_info_file
close, 1
openw, 1, out_info_file
for i=0, n-1 do begin
 printf, 1, s[i]
endfor 
close, 1

end
