;----------------------------------------------------------
;+
; NAME: ANS_SIO_WIND_DIR_2PNG
; PURPOSE:'
;   Read SIO data and wind data and create a PNG image of NRCS and
;   wind direction vectors
; CALLING SEQUENCE:'
;   ans_sio_wind_dir_2png, imagefile, ancilfile, maskfile, dirfile
; INPUT:'
;   imagefile=   NRCS image in SIO format
;   maskfile=    Land mask file in SIO format
;   ancilfile=   Ancillary data file in SIO format
;   dirfile=     Wind diretion file in netCDf format
; OUTPUT:'
;   Image of NRCS with wind directions,
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.'
;   xout=        Number of horizontal pixels for image in PNG file, default=1000
;   yout=        Number of vertical   pixels for image in PNG file, default=1000
;   error=       Negative if program fails
;   outfile=     Name of file to save the PNG image
; MODIFICATION
;    2012 Apr 23 Frank Monaldo. Created. 
; Copyright (C) 2012, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  ans_sio_wind_dir_2png, imagefile, ancilfile,  maskfile, dirfile, $              
                    xout=               xout,               $
                    yout=               yout,               $
                    outfile=            outfile,            $
                    error=              error,              $                    
                    quiet=              quiet,              $
                    help=               help


if keyword_set(help) or (n_params(0) lt 1)  then begin
  print, ' NAME: ANS_SIO_WIND_DIR_2PNG
  print, ' PURPOSE:'
  print, '   Read SIO data and wind data and create a PNG image of NRCS and
  print, '   wind direction vectors
  print, ' CALLING SEQUENCE:'
  print, '   ans_sio_wind_dir_2png, imagefile, ancilfile, maskfile, dirfile
  print, ' INPUT:'
  print, '   imagefile=   NRCS image in SIO format
  print, '   maskfile=    Land mask file in SIO format
  print, '   ancilfile=   Ancillary data file in SIO format
  print, '   dirfile=     Wind diretion file in netCDf format
  print, ' OUTPUT:'
  print, '   Image of NRCS with wind directions,
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.'
  print, '   xout=        Number of horizontal pixels for image in PNG file, default=1000
  print, '   yout=        Number of vertical   pixels for image in PNG file, default=1000
  print, '   error=       Negative if program fails
  print, '   outfile=     Name of file to save the PNG image
  error=  -1
  return
endif

error=      0
!quiet   =  1
printflag=  1
re       =  6378.00  ; Earth radius
if     keyword_set(quiet)  then printflag=  0
quiet= printflag eq 0


if not keyword_set(xout) then xout= 1000
if not keyword_set(yout) then yout= 1000
;


;------------------------------------------------
; Try to find files. Exit if not found
;------------------------------------------------
res=file_search(imagefile, count=c1)
res=file_search(maskfile,  count=c2)
res=file_search(ancilfile, count=c3)
res=file_search(dirfile,   count=c4)

if c1 eq 0 or $
   c2 eq 0 or $
   c3 eq 0 or $
   c4 eq 0 then begin
   if c1 eq 0 then print, 'Cannot find file: '+ imagefile
   if c2 eq 0 then print, 'Cannot find file: '+ maskfile
   if c3 eq 0 then print, 'Cannot find file: '+ ancilfile
   if c4 eq 0 then print, 'Cannot find file: '+ dirfile
   print, 'ans_sio_wind_dir_2png'
   error= -1
  return
endif

; ---------------------------------------------------------
; Reading data, mask, ancillary and wind direction files.
; ---------------------------------------------------------
ans_read_sio_data, imagefile, image, quiet=quiet
ans_read_sio_data, maskfile,  mask,  quiet=quiet
ans_read_sio_info, ancilfile, info,  quiet=quiet
; Note that the SAR wind direction and model direction file are in the same format
ans_read_model_cdf, dirfile, model

;-------------------------------------------------------------
; Make sure that the the image and land mask are the same size
;-------------------------------------------------------------
sz=    size(image)
nxi=   sz[1]
nyi=   sz[2]

sz=    size(mask)
nxm=   sz[1]
nym=   sz[2]

if nxm ne nxi or nym ne nyi then begin
  mask= congrid(mask, nxi, nyi)
endif

;----------------------------------------------------------------------------
; Average mimage and mask to about the size of the the final  display image
;----------------------------------------------------------------------------

nx=     info.pixels_sio_image
ny=     info.lines_sio_image

;
; Compute avefactor
;
avefactor=    1
avefactor=    ceil( ny / xout)
avefactor=    max( [1, avefactor] )
if printflag then print, 'Computed avefactor for PNG image: ' + string(avefactor, format='(I4)')

if avefactor gt 1 then begin
  if printflag then begin
    print, 'Avefactor=',avefactor
    print, 'Smoothing and reducing size of input images.'
    print, nx,ny, long(nx)  / avefactor, long(ny)  / avefactor
  endif
  nx=          long(nx)  / avefactor
  ny=          long(ny)  / avefactor
  image=       smooth(image,       [avefactor, avefactor])
  mask=        smooth(mask,        [avefactor, avefactor])
  image=       congrid(image, nx, ny)
  mask=        congrid(mask, nx, ny)
  ;
  ; Recompute  conversions
  ;
  info.i_coef=  info.i_coef / double(avefactor)
  info.j_coef=  info.j_coef / double(avefactor)
  n= n_elements(info.lon_coef)-1
  for i=0, n do info.lon_coef[i]=   info.lon_coef[i]   * double(avefactor)^info.lon_xexp[i]   * double(avefactor)^info.lon_yexp[i]
  n= n_elements(info.lat_coef)-1
  for i=0, n do info.lat_coef[i]=   info.lat_coef[i]   * double(avefactor)^info.lat_xexp[i]   * double(avefactor)^info.lat_yexp[i]
  n= n_elements(info.incid_coef)-1
  for i=0, n do info.incid_coef[i]= info.incid_coef[i] * double(avefactor)^info.incid_xexp[i] * double(avefactor)^info.incid_yexp[i]
  n= n_elements(info.rlook_coef)-1
  for i=0, n do info.rlook_coef[i]= info.rlook_coef[i] * double(avefactor)^info.rlook_xexp[i] * double(avefactor)^info.rlook_yexp[i] 
endif 



if n_elements(outfile) eq 0 then begin
  ;
  ; Display on screen
  ;
  if printflag then print, 'Plotting vector...'
  set_plot, 'x'
  device, decomp=0
  colortable=    0
  window, xsize= xout, ysize=yout
endif else begin
  ;
  ; Create a PNG image flie
  ;
  set_plot, 'z'
  device, set_resolution=[xout, yout]
endelse
;
; Display background NRCS image
;
loadct, 0
tvscl, ls(image)
; display vectors
nw= n_elements(model.ucomp)
;
; Compute arrow length
;
distance= fltarr(nw) 
for i=1, nw-1 do begin
  ll2rb, model.longitude[0], model.latitude[0], model.longitude[i], model.latitude[i],r, b
  distance[i]= r
endfor   
length = min( distance[1,*] ) / 2 ; in radians
;
; Plot arrows
;
for i=0, nw - 1 do begin
  i1=   ans_poly_cal( double(model.longitude[i]), double(model.latitude[i]), info.i_coef, info.i_xexp, info.i_yexp)
  j1=   ans_poly_cal( double(model.longitude[i]), double(model.latitude[i]), info.j_coef, info.j_xexp, info.j_yexp)
  b=    atan(model.ucomp[i], model.vcomp[i]) * 180.0 / !pi
  rb2ll, model.longitude[i], model.latitude[i], length, b, longitude, latitude
  if min(model.longitude lt 0) and longitude gt 0 then longitude=longitude-360.00
  i2=   ans_poly_cal( double(longitude), double(latitude), info.i_coef, info.i_xexp, info.i_yexp)
  j2=   ans_poly_cal( double(longitude), double(latitude), info.j_coef, info.j_xexp, info.j_yexp)    
  arrow, i1, j1, i2, j2, color=255, /device
endfor

if  n_elements(outfile) gt 0 then begin
  tvlct,   r, g, b, /get
  z=       tvrd()
  if printflag then print, 'Writing: ' + outfile
  write_png, outfile, z, r, g, b  
endif


end
