;-----------------------------------------------------
;+
; NAME: ans_smap2sio
; PURPOSE:
;   To convert SMAP Sigma0 imagery to SIO format
; CALLING SEQUENCE
;   ans_smap2sio, deffile, sio_out
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters /twobyte, deg, ave_line, and ave_pixel
;                If the parameters are explicitly set. For example, ave_pixel=0 then
;                it will over-ride what ever is in the defaults file
; OUTPUT
;   sio_out=     Root file name(s) for SIO files
;   SIO files, NetCDF level and KMZ files
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
;   error:       -1 if bad
; MODIFICATION HISTORY:
;   2015 Dec 29  Chris Jackson. Initial Version - Based on ans_sen2sio
; Copyright (C) 2013-2015, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_smap2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

  !quiet=1
  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  sio_root=''
  
  if keyword_set(help)   or (n_params(0) lt 1 )  then begin
    print, ' NAME: ans_smap2sio
    print, ' PURPOSE:
    print, '   To convert SMAP-1 imagery to SIO format
    print, ' CALLING SEQUENCE
    print, '   ans_smap2sio, deffile, sio_out
    print, ' INPUT
    print, '   deffile=     Full path to defaults file. This option will read a specified file
    print, '                for the parameters /twobyte, deg, ave_line, and ave_pixel
    print, '                If the parameters are explicitly set. For example, ave_pixel=0 then
    print, '                it will over-ride what ever is in the defaults file
    print, ' OUTPUT
    print, '   sio_out=     Root file name(s) for SIO files
    print, '   
    print, ' KEYWORDS:
    print, '   /quiet       No screen output
    print, '   /help;       Get help
    print, '   error:       -1 if bad
    return
  endif
  !quiet=  1 ; set idl to quiet

  ;
  ; Time to start processing
  ;
  
  jsstart=  dt_tm_tojs(systime(/utc))
  
  
  ;
  ; Read defaults file
  ;
  if printflag then begin
    print, ' '
    print, 'Begin SMAP to SIO. ' + systime(/utc)
    print, ' '
  endif
  ans_read_defaults, deffile, df, quiet=quiet
  ;
  ;
  ; Create directories if they do not exist
  ;
  res= file_search( df.already_processed, /test_directory, count=count)
  if count eq 0 then file_mkdir, df.already_processed
  res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.ncdf1_output_dir
  res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
  if count eq 0 then file_mkdir, df.sio_kmz_dir
  res= file_search( df.log_dir, /test_directory,     count=count)
  if count eq 0 then file_mkdir, df.log_dir
  ;
  ;
  ;---------------------------------------------------------------
  ;----------- Start SMAP data conversion to SIO  ---------------
  ;---------------------------------------------------------------
  
  ;
  ; Look for SMAP directories in base directory
  ;
  ff= file_search( df.smap_dir +'/SMAP*', count=count) 
  if count eq 0 then begin
    print, 'Cannot find files in ' +  df.smap_dir +'/*'
    print, 'Exiting ans_smap2sio'
    sio_root=''
    error = -1
    return
  endif
 
  ;
  ; Computing SIO files for all the SMAP data
  ;
  error= 1
  if printflag then print, 'Computing SMAP Level-1 SIO data...
  ans_read_smap,   ff[0], deffile,sio_root, $
    /detail, $
    jsstart      = jsstart    , $
    error        = error      , $
    quiet        = quiet
    
  if error lt 0 then begin
    print, 'Cannot read SMAP data'
    print, 'Exiting ans_smap2sio.pro'
    error= -1
    return
  endif
  ;
  ; Loop through SIO files for full processing
  ;For each sio_root entry (HH) there could also 
  ;be VV and HV files that need to also be processed
  ; Computing Level 1 NetCDF files and KMZ
  ;
  n= n_elements(sio_root)
  
  for i=0, n-1 do begin
    if strlen( sio_root[i] ) gt 0 then begin
      root=                getwrd( sio_root[i], 0, /last, del='/' )
      datafile=            sio_root[i]          + '_image.dat'
      infofile=            sio_root[i]          + '_ancil.txt'
      kmzfile=             df.sio_kmz_dir       + root + '_sio.kmz'
      netcdffile_level_1=  df.ncdf1_output_dir  + root + '_sar_level1.nc'
      logfile=  df.log_dir + '/' + root + '.log'
      ;
;      ; SIO to NetCDF
;      ;
;      if printflag then print, 'Computing SMAP Level-1 NetCDF data...
;      if df.level1cdf eq 'Y' then begin
;        ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1
;        time1=   dt_tm_tojs(systime(/utc))
;        outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
;        close, 1
;        openw, 1, logfile, /append
;        printf, 1, outstr
;        close, 1
;        print, outstr
;      endif
      ;
      ; SIO to KMZ
      ;
      if df.siokmzfile eq 'Y' then begin
        if printflag then begin
          print, ''
          print, 'Computing KMZ files.'
        endif
        ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=3, $
          /nrcs, quiet= quiet, working_dir=df.working_directory
        time1=   dt_tm_tojs(systime(/utc))
        outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
        close, 1
        openw, 1, logfile, /append
        printf, 1, outstr
        close, 1
        print, outstr
      endif
    endif
  endfor
  
  
;---------------------------------------------------------------
;--------------------- End SMAP processing ---------------
;---------------------------------------------------------------
  
end
