;+---------------------------
; NAME: ANS_TEST_SEN_NOISE
; PURPOSE:
;    The purpose of this program is to read Sentinel data and compare
;    NRCS without and without noise removel
; CALLING SEQUENCE:
;    ans_test_sen_noise, input_dir, deffile, output
; INPUT:
;    input_dir    =  Sentinel data come in directories. This input is the name
;                    of such a directory. The program will look
;                    for directorise of the form S1A*SAFE/
;    deffile       =  Defaults file
; OUTPUT:          
;   output         = Text file with comparisons
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (0 if program completes or -1 if not)
; MODIFICATION HISTORY    
;  2016 Apr 18 Frank Monaldo. Creation
; Copyright (C) 2016, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;
                                               
pro ans_test_sen_noise,  input_dir,    deffile , output, $
                         quiet       = quiet   , $
                         help        = help    , $
                         error       = error          
                    
error=     0
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_TEST_SEN_NOISE
  print, ' PURPOSE:
  print, '    The purpose of this program is to read Sentinel data and compare
  print, '    NRCS without and without noise removel
  print, ' CALLING SEQUENCE:
  print, '    ans_test_sen_noise, input_dir, deffile, output
  print, ' INPUT:
  print, '    input_dir    =  Sentinel data come in directories. This input is the name
  print, '                    of such a directory. The program will look
  print, '                    for directorise of the form S1A*SAFE/
  print, '    deffile       =  Defaults file
  print, ' OUTPUT:          
  print, '   output         = Text file with comparisons
  print, ' KEYWORDS:
  print, '     /quiet       = Do not print intermediate results.
  print, '     /help        = Print out help
  print, '     error        = error (0 if program completes or -1 if not)
  error= -1
  return
endif

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
error=1
time0 = dt_tm_tojs(systime(/utc))
search_str = 'S1A*SAFE'
;
; Read defaults file
;
ans_read_defaults, deffile, df, quiet=quiet, error=error
if error lt 0 then return
;
; Determine if input directory exists
; 

ff= file_search( input_dir + search_str, /test_directory, count=count)


if count eq 0 then begin
  print, 'Could not find Sentinel directories in: ' + input_dir
  print, 'Exiting ans_test_sen+_noise.pro '
  error= -1
  return
endif

ndir =  count

close, 1 
openw,  1, output
printf, 1, 'Directory , Polarization, Corenme, Mean Image, Mean Image-Noise, Fraction Noise

for d = 0, ndir -1 do begin
  ;
  ; Create a new working directory
  ;
  working_directory= df.working_directory + '/sen_' +  $
                      dt_tm_fromjs(dt_tm_tojs(systime(/utc)),format='Y$-0n$-d$-h$-m$-s$' ) + $
                      '-' + string(randomu(seed))
  working_directory= strcompress( working_directory, /remove_all)
  file_mkdir, working_directory
  ;
  ; Copy Sentinel directory to working directory.   Note the movement of Sentinel directory to 
  ; and already processed directory must occur at a different level. It is not done in this program
  ;
  if printflag then begin
    print, 'Copying ' + ff[d]             + '/'
    print, 'to      ' + working_directory + '/'
  endif
  file_copy,   ff[d], working_directory  + '/', /recursive
  ;
  ; Move files to already processed directory if move2dir set
  ;
  res= file_search(df.already_processed, /test_directory, count=count)
  if count eq 0 then file_mkdir, move2dir
  if printflag then begin
    print, 'Moving ' + ff[d]
    print, 'to     ' + df.already_processed + '/'
  endif
  file_move,   ff[d], df.already_processed  + '/', /overwrite, /ALLOW_SAME
  ;
  ; Find the s*.xml file and the base_directory for processing, ff= full path to product.xml file
  ; 
  time1 = dt_tm_tojs(systime(/utc))

  if printflag then print, 'Time to copy files to working directory='+ string (time1-time0) +' s'

  ;
  ; Locating leader file 
  ;
  xfiles=   file_search( working_directory, 's*xml', count=data_setmax)
  if data_setmax eq 0 then begin
    print, 'Cannot file xml data in: ' + working_directory
    print, 'Exiting ans_test_sen_noise.pro'
    error = -1
    return
  endif
  ;
  ; Loop through data sets.  
  ;
  for data_set= 0, data_setmax-1 do begin
    base_directory = working_directory
    ;
    ; Fill information into lf structure, Read mapping grid information
    ; and read geo reference dagta
    ;
    corename  = getwrd( xfiles[data_set], 0, /last, del='/' )
    len       = strlen( corename )
    corename  = strmid( corename, 0, len-4 )
    ;
    ; Read and calibrate image file
    ;q
    imgfile   = file_search( base_directory, corename + '.tiff', count= count)
    if count eq 0 then begin
      print, 'Cannot find image file ' + corename + '.tiff'
      error = -1
      return
    endif
    calfile   = file_search( base_directory, 'calibration*' + corename + '*', count= count)
    if count eq 0 then begin
      print, 'Cannot find calibration file ' + 'calibration*' + corename + '*'
      error = -1
      return
    endif
    noisefile = file_search( base_directory, 'noise*'       + corename + '*', count= count)
    if count eq 0 then begin
      print, 'Cannot find noise file ' + 'noise*' + corename + '*'
      error = -1
      return
    endif    
    sen_img2sigma      , imgfile, calfile,            img0, quiet=quiet  
    sen_img2sigma_noise, imgfile, calfile, noisefile, img1, quiet=quiet  
    m0 = mean(img0)
    m1 = mean(img1)
    fr = 0
    pol = 'NONE'
    pol = strupcase( getwrd(corename, 3, del='-') )
    if m1 gt 0 then fr = (m0-m1)/m1
    tt = getwrd(ff[d], 0, /last, del='/')     + ' , '
    tt = tt + pol                             + ' , '
    tt = tt + corename                        + ' , '
    tt = tt + string(m0, format = '(E15.9)' ) + ' , '
    tt = tt + string(m1, format = '(E15.9)' ) + ' , '
    tt = tt + string(fr, format = '(E15.9)' )
    printf, 1, tt
    print, tt
  endfor
  ;
  ; Delete working directory
  ;
  if printflag then print, 'Deleting ' + working_directory  
  file_delete,  working_directory  + '/', /recursive, quiet=quiet
endfor  ; End loop through directories
close, 1

end
