;-----------------------------------------------------
;+
; NAME: ans_tsar2sio
; PURPOSE:
;   To convert TerraSAR-X image to SIO format
; CALLING SEQUENCE
;   ans_tsar2sio, deffile, sio_out
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters /twobyte, deg, ave_line, and ave_pixel
;                If the parameters are explicitly set. For example, ave_pixel=0 then
;                it will over-ride what ever is in the defaults file
; OUTPUT
;   sio_out=     Root file name(s) for SIO files
;   SIO files, NetCDF level and KMZ files
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
;   error:       -1 if bad
; MODIFICATION HISTORY:
;   2011 Jan 01  Frank Monaldo
;   2011 Feb 04  Frank Monaldo. Add jsstart
;   2011 Feb 22  Frank Monado.  Change .cdf extension to .nc
;   2013 Jan 03  Frank Monaldo. Change call to ans_read_tsar to use defile
;   2013 Sep 30  Frank Monaldo. Correct error in help listing.
; Copyright (C) 2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_tsar2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
sio_root=''

if keyword_set(help)   or (n_params(0) lt 1 )  then begin
  print, ' NAME: ans_tsar2sio
  print, ' PURPOSE:
  print, '   To convert TerraSAR-X image to SIO format
  print, ' CALLING SEQUENCE
  print, '   ans_tsar2sio, deffile, sio_out
  print, ' INPUT
  print, '   deffile=     Full path to defaults file. This option will read a specified file
  print, '                for the parameters /twobyte, deg, ave_line, and ave_pixel
  print, '                If the parameters are explicitly set. For example, ave_pixel=0 then
  print, '                it will over-ride what ever is in the defaults file
  print, ' OUTPUT
  print, '   sio_out=     Root file name(s) for SIO files
  print, '   SIO files, NetCDF level and KMZ files
  print, ' KEYWORDS:
  print, '   /quiet       No screen output
  print, '   /help;       Get help
  print, '   error:       -1 if bad
  return
endif
!quiet=  1 ; set idl to quiet

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))


;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin TSAR to SIO. ' + systime(/utc)
  print, ' '
endif
ans_read_defaults, deffile, df, quiet=quiet
;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir         
res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir            
res= file_search( df.log_dir, /test_directory,     count=count)
if count eq 0 then file_mkdir, df.log_dir
;
;  
;---------------------------------------------------------------
;----------- Start TerraSAR-X conversion to SIO  ---------------
;---------------------------------------------------------------

;
; Look for TerraSAR-X directories in base directory
;
ff= file_search( df.tsar_dir +'/*', /test_directory, count=count) & cc=count
if cc eq 0 then begin
  print, 'Cannot find files in ' +  df.tsar_dir +'/*'
  print, 'Exiting ans_tsar2sio'
  sio_root=''
  return
endif

;
; Computing SIO files for all the TerraSAR-X data
;
error= 1
if printflag then print, 'Computing TerraSAR-X Level-1 SIO data...
ans_read_tsar,  ff[0]        , deffile    , $
                outputroot   = outputroot , $
                jsstart      = jsstart    , $
                error        = error      , $
                quiet        = quiet

if error lt 0 then begin
  print, 'Cannot read TSAR data'
  print, 'Exiting ans_tsar2sio.pro'
  error= -1
  return
endif
sio_root= outputroot
;
; Loop through SIO files for full processing
;
; Computing Level 1 NetCDF files and KMZ
;
n= n_elements(sio_root)

for i=0, n-1 do begin
    if strlen( sio_root[i] ) gt 0 then begin
    root=                getwrd( sio_root[i], 0, /last, del='/' )
    datafile=            sio_root[i]          + '_image.dat'
    infofile=            sio_root[i]          + '_ancil.txt'
    landfile=            sio_root[i]          + '_mask.dat'
    kmzfile=             df.sio_kmz_dir       + root + '_sio.kmz'
    netcdffile_level_1=  df.ncdf1_output_dir  + root + '_sar_level1.nc'
    kmz_file=            df.wind_kmz_dir      + root + '_wind.kmz'
    logfile=  df.log_dir + '/' + root + '.log'
    ;
    ; SIO to NetCDF 
    ;
    if printflag then print, 'Computing TerraSAR-X Level-1 NetCDF data...   
    if df.level1cdf eq 'Y' then begin
      ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
    ;
    ; SIO to KMZ
    ;
    if df.siokmzfile eq 'Y' then begin
      if printflag then begin
         print, ''
         print, 'Computing KMZ files.'
      endif
      ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
                   /nrcs, quiet= quiet, working_dir=df.working_directory
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
  endif
endfor


;---------------------------------------------------------------
;--------------------- End TerraSAR-X processing ---------------
;---------------------------------------------------------------

end
