;-----------------------------------------------------
;+
; NAME: ANS_UAVSAR2SIO
; PURPOSE:
;   To convert UAVSAR to SIO format. Starts with latest file
; CALLING SEQUENCE
;   ans_uavsar2sio, deffile, sio_out
; INPUT
;   deffile=     Full path to defaults file. This option will read a specified file
;                for the parameters /twobyte, deg, ave_line, and ave_pixel
;                If the parameters are explicitly set. For example, ave_pixel=0 then
;                it will over-ride what ever is in the defaults file
; OUTPUT
;   sio_out=     Root file name(s) for SIO files
;   SIO files, NetCDF level and KMZ files
; KEYWORDS:
;   /quiet       No screen output
;   /help;       Get help
;   error:       -1 if bad
; MODIFICATION HISTORY:
;   2020 Jan 14  Frank Monaldo. Adapted from ans_sen2sio
; Copyright (C) 2013-2020, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_uavsar2sio, deffile, sio_root, help= help,  quiet= quiet, error=error

!quiet=1
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
sio_root=''

if keyword_set(help)   or (n_params(0) lt 1 )  then begin
  print, ' NAME: ANS_UAVSAR2SIO
  print, ' PURPOSE:
  print, '   To convert UAVSAR to SIO format. Starts with latest file
  print, ' CALLING SEQUENCE
  print, '   ans_uavsar2sio, deffile, sio_out
  print, ' INPUT
  print, '   deffile=     Full path to defaults file. This option will read a specified file
  print, '                for the parameters /twobyte, deg, ave_line, and ave_pixel
  print, '                If the parameters are explicitly set. For example, ave_pixel=0 then
  print, '                it will over-ride what ever is in the defaults file
  print, ' OUTPUT
  print, '   sio_out=     Root file name(s) for SIO files
  print, '   SIO files, NetCDF level and KMZ files
  print, ' KEYWORDS:
  print, '   /quiet       No screen output
  print, '   /help;       Get help
  print, '   error:       -1 if bad
  error  =  -1
  return
endif
!quiet=  1 ; set idl to quiet

;
; Time to start processing
;

jsstart=  dt_tm_tojs(systime(/utc))


;
; Read defaults file
;
if printflag then begin
  print, ' '
  print, 'Begin UAVSAR to SIO. ' + systime(/utc)
  print, ' '
endif
ans_read_defaults, deffile, df, quiet=quiet
;
; 
; Create directories if they do not exist
;
res= file_search( df.already_processed, /test_directory, count=count)
if count eq 0 then file_mkdir, df.already_processed
res= file_search( df.ncdf1_output_dir, /test_directory,  count=count)
if count eq 0 then file_mkdir, df.ncdf1_output_dir         
res= file_search( df.sio_kmz_dir,      /test_directory,  count=count)
if count eq 0 then file_mkdir, df.sio_kmz_dir            
res= file_search( df.log_dir,          /test_directory,  count=count)
if count eq 0 then file_mkdir, df.log_dir
;
;  
;---------------------------------------------------------------
;----------- Start UAVSAR data conversion to SIO  ---------------
;---------------------------------------------------------------

ff= file_search( df.uavsar_dir +'*ann', count=count) & cc=count
if cc eq 0 then begin
  print, 'Cannot find files in ' +  df.uavsar_dir +'/*ann'
  print, 'Exiting ans_uavsar2sio'
  sio_root=''
  error = -2
  return
endif
jsfile = dblarr(cc)
for i = 0, cc-1 do begin
    ans_read_uavsar_ancil, ff[i] , info, quiet = quiet , error = error
    if error ge 0 then begin
      acquisition_start_date    = getwrd( info.Start_Time_of_Acquisition  , 0 )
      acquisition_start_year    = strmid( acquisition_start_date  ,    7, 4 )
      acquisition_start_month   = getwrd( acquisition_start_date  , 1 , del='-')
      acquisition_start_day     = getwrd( acquisition_start_date  , 0 , del='-')
      acquisition_start_time    = strmid( info.Start_Time_of_Acquisition  , 11, 8 )
      acquisition_start_time_str= acquisition_start_year    + ' ' + $
                                  acquisition_start_month   + ' ' + $ 
                                  acquisition_start_day     + ' ' + $
                                  acquisition_start_time  
      acquisition_start_time_js = dt_tm_tojs(  acquisition_start_time_str )
      
      acquisition_stop_date     = getwrd( info.Stop_Time_of_Acquisition  , 0 )
      acquisition_stop_year     = strmid( acquisition_stop_date   ,    7, 4 )
      acquisition_stop_month    = getwrd( acquisition_stop_date   , 1 , del='-')
      acquisition_stop_day      = getwrd( acquisition_stop_date   , 0 , del='-')
      acquisition_stop_time    = strmid( info.Stop_Time_of_Acquisition  , 11, 8 )
      acquisition_stop_time_str = acquisition_stop_year    + ' ' + $
                                  acquisition_stop_month   + ' ' + $ 
                                  acquisition_stop_day     + ' ' + $
                                  acquisition_stop_time  
      acquisition_stop_time_js  = dt_tm_tojs(  acquisition_stop_time_str )                           
      center_js                 = ( acquisition_start_time_js + acquisition_stop_time_js ) / 2.0
      jsfile[i]                 = center_js
      maxjs                     = max( jsfile, index_max)
    endif
endfor

if printflag then print, 'Selected ancillary data file: ' + ff[index_max]

;
; Computing SIO files for all the Sentinel data
;
error= 1
if printflag then print, 'Computing UAVSAR Level-1 SIO data...
   ans_read_uavsar, deffile,  outputroot , $
                   ancil        = ff[index_max] , $
                   error        = error         , $
                   quiet        = quiet

if error lt 0 then begin
  print, 'Cannot read UAVSAR data'
  print, 'Exiting ans_uavsar2sio.pro'
  error= -1
  return
endif
sio_root= outputroot
;
; Loop through SIO files for full processing
;
; Computing Level 1 NetCDF files and KMZ
;
n= n_elements(sio_root)

for i=0, n-1 do begin
    if strlen( sio_root[i] ) gt 0 then begin
    root=                getwrd( sio_root[i], 0, /last, del='/' )
    datafile=            sio_root[i]          + '_image.dat'
    infofile=            sio_root[i]          + '_ancil.txt'
    kmzfile=             df.sio_kmz_dir       + root + '_sio.kmz'
    netcdffile_level_1=  df.ncdf1_output_dir  + root + '_sar_level1.nc'
    logfile=  df.log_dir + '/' + root + '.log'
    ;
    ; SIO to NetCDF 
    ;
    if printflag then print, 'Computing Sentinel Level-1 NetCDF data...   
    if df.level1cdf eq 'Y' then begin
      ans_sio2l1cdf, datafile, infofile, landfile, netcdffile_level_1
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + netcdffile_level_1
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
    ;
    ; SIO to KMZ
    ;
    if df.siokmzfile eq 'Y' then begin
      if printflag then begin
         print, ''
         print, 'Computing KMZ files.'
      endif
      ans_sio2kmz, datafile, infofile, kmzfile, /plotlook, /incid, /usecongrid, avefactor=8, $
                   /nrcs, quiet= quiet, working_dir=df.working_directory
      time1=   dt_tm_tojs(systime(/utc))
      outstr=    string(time1, format='(I10)') + ' '+ dt_tm_fromjs(time1) + ' ' + kmzfile
      close, 1
      openw, 1, logfile, /append
      printf, 1, outstr
      close, 1
      print, outstr
    endif
  endif
endfor


;---------------------------------------------------------------
;--------------------- End UAVSAR processing ---------------
;---------------------------------------------------------------

end
