;----------------------------------------------------------
;+
; NAME: ANS_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;   ans_write_ancil_sio, filename, lf
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;
; MODIFICATION
;  2007 Aug 22 Frank Monaldo
;  2008 Sep 13 Chris Jackson  -
;              Made changes in the Image Identification
;              and Image Orientation block to ensure that
;              certain lines are always written out
;              Made sure PALSAR corners are written
;  2009 May 11 Frank Monaldo
;              Corrected problem w/ print out of incident
;              angle coefficients
;  2009 Jul 27 Frank Monaldo. Added code to keep the program
;              even if the lf.scansar and lf.s_c_altitude do not exist. This
;              was required because these fields are not filled when reading
;              RSAT2 data
;  2009 Oct 14 Radically re-written for new software version
;  2009 Nov 10 Frank Monaldo. correct bug that wrote a line
;              transformation rather than an element transformation
;  2009 Dec 02 Frank Monaldo
;              Changed: longitude (0to-360) transform coef for element
;                   to: longitude (0to360) transform coef for element
;  2010 Jan 05 Frank Monaldo. Changed to accomodate ans_compute_fit and ans_poly_cal
;  2010 Jan 06 Frank Monaldo. Corrected printout of longitude fit information
;  2010 Jan 07 Frank Monaldo. Fixed s mising "^" character in longitude line output.
;                             and removed an unnecessary space.
;  2010 Jan 11 Frank Monaldo. Upgrade form rsat2_write_ancil_sio.pro
;  2010 Jan 20 Frank Monaldo. Corrected some mispellings.
;  2010 Jan 22 Frank Monaldo. Re-instated inner lats/lons (identical to the outer)
;                             This is required for the ship detection code.
;                             Remove early_azimuth_edge near_range_edge
;  2010 Jan 28 Frank Monaldo. Add answer_version to output
;  2010 Mar 05 Frank Monaldo. Corrected error to output corners.
;  2010 Jun 14 Frank Monaldo. Print help if no parameters given
;  2010 Jul 15 Frank Monaldo. Print hostname
;  2011 Jan 23 Frank Monaldo. Remove hostname printout. Check if lf.month is a number
;                             or name
;  2011 Feb 04 Frank Monaldo. Add jsstart
;  2011 Mar 29 Frank Monaldo. Add nrcs_slope nrcs_bias
;  2011 Apr 21 Frank Monaldo. Change flag for SAR band and polarization mode line
;  2011 May 17 Frank Monaldo. Change *inner image corner* to *inner image true corner*
;  2011 May 23 Frank Monaldo. Add JS start and end time for acquistion start and stop
;                             If equal to -3155673600D (Jan 1, 1990) then no value provided.
;  2011 May 27 Frank Monaldo. Improved formating for jsbegin and jsend output
;  2011 May 29 Frank Monaldo. Add elements/line limits of corner
;  2011 Jun 02 Frank Monaldo. Add NRCS slope bias even if not present
;  2011 Jun 22 Frank Monaldo. Make sure that lf.asc_des is either A/D/U
;  2011 Aug 24 Frank Monaldo. Capitize A in UTC  Acquisition date and time format 2
;                             Add sigma_ne coefficients
;  2015 Dec 23 Chris Jackson. Put in tag test for lf.month and the ability to use the
;                             more updated lf.acquisition_ and acq_date_ tags
;                             Changed print ANSWRS.Version to formatted string
;  2016 Jun 13 Frank Monaldo. Add sen_cpol_remove_noise and sen_xpol_remove_noise
;  2020 Jan 13 Frank Monaldo. Add UAVSAR information
;  2020 Feb 18 Frank Monaldo. Add RCM platform code comments
;  2020 Sep 25 Chris Jackson. Change sen_cpol_remove_noise/sen_xpol_remove_noise attributes to  generic cpol_remove_noise/xpol_remove_noise
; Copyright (C) 2007-2016 Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_write_ancil_sio, filename, lf, $
    quiet=quiet, help=help
    
    
  printflag=1
  if keyword_set(quiet) then printflag=0
  
  if keyword_set(help)  or (n_params(0) lt 1)  then begin
    print, ' NAME: ANS_WRITE_ANCIL_SIO'
    print, ' PURPOSE:'
    print, '   To write out data into ancillary data ascii file. These
    print, '   data fit the SIO format
    print, ' CALLING SEQUENCE:'
    print, '   ans_write_ancil_sio, filename, lf'
    print, ' INPUT:'
    print, '   filename=     output file with ancillary data
    print, '   lf=           leader structure'
    print, '   scaling=      scaling for data in binary file of NRCS, default=1
    print, ' OUTPUT:
    print, '   ASCII file in SIO format
    print, ' KEYWORDS:
    print, '   /quiet=      Do not print intermediate results'
    print, '   /help=       List help.
    return
  endif
  
  
  if printflag then begin
    print, 'Begin ans_write_ancil_sio..'
    print, '
  endif
  ;
  ; Generate time strings in expected SIO format....
  ;
  if tag_test(lf, 'month') then begin
    month= lf.month
    if isnumber(lf.month) eq 0 then begin
      case strupcase(lf.month) of
        'JAN': month= '01'
        'FEB': month= '02'
        'MAR': month= '03'
        'APR': month= '04'
        'MAY': month= '05'
        'JUN': month= '06'
        'JUL': month= '07'
        'AUG': month= '08'
        'SEP': month= '09'
        'OCT': month= '10'
        'NOV': month= '11'
        'DEC': month= '12'
        ELSE:  month= '00'
      endcase
    endif else begin
      lf.month= month2name(lf.month,/upper)
    endelse
    
    datestr=      month + '-' + string(lf.day,       format='(I2.2)') + '-' + string(lf.data_year, format='(I4.4)')
    datestr=      strcompress(datestr, /remove_all)
    timestr=      string(lf.hour,   format='(I2.2)') + ':' +  $
      string(lf.minute, format='(I2.2)') + ':' +  $
      string(lf.second, format='(I2.2)')
    timestr=      strcompress(timestr, /remove_all)
    acq_time=     string(lf.data_year, format='(I4.4)') + ' ' + lf.month + ' ' + string(lf.day,       format='(I2.2)')
    acq_timejs=   dt_tm_tojs( acq_time + ' ' + timestr)
    acq_timestr=  dt_tm_fromjs(acq_timejs,     format='Y$-0n$-0d$ h$:m$:s$')
  endif
  
  
  if tag_test(lf, 'acquisition_date') then begin
    acq_timejs = (double(lf.jsend)+double(lf.jsbegin))/2.0
    datestr = dt_tm_fromjs(acq_timejs,     format='0n$-0d$-Y$')
    timestr = dt_tm_fromjs(acq_timejs,     format='h$:m$:s$')
    acq_timestr=  dt_tm_fromjs(acq_timejs,     format='Y$-0n$-0d$ h$:m$:s$')
  endif
  
  sio_timestrjs=dt_tm_tojs(systime(/utc) )
  sio_timestr=  dt_tm_fromjs(sio_timestrjs,  format='Y$-0n$-0d$ h$:m$:s$')
  processing_start=   'NONE'
  if tag_test(lf, 'jsstart') then processing_start=  dt_tm_fromjs(lf.jsstart,  format='Y$-0n$-0d$ h$:m$:s$')
  wavelength=  299792458.0 / lf.radar_frequency
  case lf.frequency_band of
    'C':   code='2'
    'L':   code='3'
    'X':   code='4'
    ELSE:  code='0'
  endcase
  case lf.pol of
    'VV': code= code+'1'
    'HH': code= code+'2'
    'HV': code= code+'3'
    'VH': code= code+'4'
    ELSE: code= code+'0'
  endcase
  
  element_spacing= lf.range_pixel   * lf.ave_pixel
  line_spacing=    lf.azimuth_pixel * lf.ave_line
  dx=              getwrd(lf.datumshiftparameters,0)
  dy=              getwrd(lf.datumShiftParameters,1)
  dz=              getwrd(lf.datumShiftParameters,2)
  dx2=             0.0
  dy2=             0.0
  dz2=             0.0
  lf.sar_sensor=   strupcase(lf.sar_sensor)
  
  
  close,  1
  openw,  1, filename
  printf, 1, lf.name_dir                                 + ' % original root file or directory name'
  printf, 1, string(lf.answrs_version,  format='(A32)')  + ' % ANSWRS version
  printf, 1, string(datestr,            format='(A24)')  + ' % acquisition date
  printf, 1, string(timestr,            format='(A24)')  + ' % acquisition time (GMT) '
  printf, 1, string(acq_timejs,         format='(I24)')  + ' % julian seconds epoch= 2000 Jan 1 00:00:00'
  printf, 1, string(acq_timestr,        format='(A20)')  + ' UTC % Acquisition date and time format 2 '
  printf, 1, string(lf.jsbegin,         format='(I24)')  + ' % Acquisition start time in julian seconds'
  printf, 1, string(lf.jsend,           format='(I24)')  + ' % Acquisition end   time in julian seconds'
  printf, 1, string(processing_start,   format='(A20)')  + ' UTC % Processing start time'
  printf, 1, string(sio_timestr,        format='(A20)')  + ' UTC % SIO creation date and time'
  printf, 1, string(lf.productType,     format='(A24)')  + ' % Product type'
  printf, 1, string(' ',                format='(A24)')  + ' % RCM SLC     = Single Look Complex'
  printf, 1, string(' ',                format='(A24)')  + ' % RCM GRD     = Ground Range Detected'
  printf, 1, string(' ',                format='(A24)')  + ' % RSAT2 SGF   = SAR Georeferenced Fine product'
  printf, 1, string(' ',                format='(A24)')  + ' % RSAT2 SGX   = SAR Georeferenced Extra-Fine product '
  printf, 1, string(' ',                format='(A24)')  + ' % RSAT2 SGC   = SAR Georeferenced Coarse product'
  printf, 1, string(' ',                format='(A24)')  + ' % RSAT2 SSG   = SAR Systematic Geocorrected product '
  printf, 1, string(' ',                format='(A24)')  + ' % RSAT2 SPG   = SAR Precision Geocorrected product '
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI WS     = Wide Swath'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI WSM    = Wide Swath Medium resolution'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI ISM    = Image Mode Single Look Complex'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI IMM    = Image Mode Medium-resolution'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI GMM    = Global Monitoring Image Mode Product '
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI IMP    = Image Mode Precision Image'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI IMG    = Image Mode Ellipsoid Geocoded Imag'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APS    = Alternating Polarization Mode Single-Look'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APP    = Alternating Polarization Mode Precision Image'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APM    = Alternating Polarization Medium-resolution'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI WSS    = Wide Swath Single Look Complex '
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APH    = Alternating Polarization Mode Cross-polar H Level 0
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APC    = Alternating Polarization Mode Co-polar Level 0 '
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI IM     = Image Mode source packets Level 0'
  printf, 1, string(' ',                format='(A24)')  + ' % ENVI APG    = Alternating Polarization Ellipsoid Geocoded '
  printf, 1, string(' ',                format='(A24)')  + ' % SEASAT STD  = Seasat standard mode
  printf, 1, string(' ',                format='(A24)')  + ' % Sentinel IW = Sentinel Interferometric Wide
  printf, 1, string(' ',                format='(A24)')  + ' % Sentinel EW = Extra Wide
  printf, 1, string(' ',                format='(A24)')  + ' % SMAP L1C HIRES = SMAP SAR 
  printf, 1, string(' ',                format='(A24)')  + ' % UAVSAR      = Uninhabited Aerial Vehicle SAR
  printf, 1, string(lf.proc_fac,        format='(A24)')  + ' % processing agency'
  printf, 1, string(lf.proc_fac,        format='(A24)')  + ' % facility'
  printf, 1, string(lf.software_version,format='(A24)')  + ' % SAR processor version
  printf, 1, string(lf.datatake_id,     format='(A24)')  + ' % scene identifier'
  printf, 1, string(lf.datatake_id,     format='(A24)')  + ' % scene reference number (orbit-frame number) {not defined in ASF header}'
  printf, 1, string(lf.orbit_number,    format='(A24)')  + ' % orbit (revolution) number'
  printf, 1, string(lf.sar_sensor,      format='(A24)')  + ' % platform name'
  printf, 1, string(lf.platform_code,   format='(A24)')  + ' % SAR platform code
  printf, 1, string(' ',                format='(A24)')  + ' % RCM-1 = Radarsat Constellation Mission-1 '
  printf, 1, string(' ',                format='(A24)')  + ' % RCM-2 = Radarsat Constellation Mission-2 '
  printf, 1, string(' ',                format='(A24)')  + ' % RCM-3 = Radarsat Constellation Mission-3 '
  printf, 1, string(' ',                format='(A24)')  + ' % J1    = J-ERS-1 '
  printf, 1, string(' ',                format='(A24)')  + ' % E1/2  = E-ERS-1/2 '
  printf, 1, string(' ',                format='(A24)')  + ' % R1/2  = RadarSat-1/2 '
  printf, 1, string(' ',                format='(A24)')  + ' % L1    = LightSAR-1 '
  printf, 1, string(' ',                format='(A24)')  + ' % N1    = ENVISAT-1 '
  printf, 1, string(' ',                format='(A24)')  + ' % A1    = ALOS-1 '
  printf, 1, string(' ',                format='(A24)')  + ' % T1    = TerraSAR-X'
  printf, 1, string(' ',                format='(A24)')  + ' % C1/2  = CosmoSky-Med-1/2 etc.
  printf, 1, string(' ',                format='(A24)')  + ' % SE    = Seasat
  printf, 1, string(' ',                format='(A24)')  + ' % S1A   = Sentinel-1A
  printf, 1, string(' ',                format='(A24)')  + ' % S1B   = Sentinel-1B
  printf, 1, string(' ',                format='(A24)')  + ' % SMAP  = SMAP
  printf, 1, string(' ',                format='(A24)')  + ' % UAVSAR = Uninhabited Aerial Vehicle SAR
  printf, 1,'
  printf, 1,'  ----------------------
  printf, 1,'  Image Type Information
  printf, 1,'  ----------------------
  if tag_test(lf, 'sar_imaging_mode') then begin
    printf, 1, string(lf.sar_imaging_mode, format='(A24)')  + ' % sensor imaging mode'
  endif else begin
    if lf.scansar eq 1 then begin
      printf, 1, string('SCANSAR BEAM', format='(A24)')  + ' % sensor imaging mode (original pixel size > 30 m)'
    endif else begin
      printf, 1, string('STANDARD MODE', format='(A24)') + ' % sensor imaging mode (original pixel size <= 30 m)'
    endelse
  endelse
  
  printf, 1, string('1',                 format='(A24)')   + ' % calibration flag (-1=antenna corrected only not calibrated|0=not calibrated|1=calibrated)
  printf, 1, string(lf.valuetype,        format='(A24)')   + ' % image pixel value units (note: amplitude==magnitude; intensity==power==squared magnitude)
  printf, 1, string(lf.pol,              format='(A24)')   + ' % polarization
  printf, 1, string(lf.frequency_band,   format='(A24)')   + ' % frequency band
  printf, 1, string(code,                format='(A24)')   + ' % flag for SAR band and polarization mode (21=CVV|22=CHH|23=CHV|24=CVH|)
  printf, 1, string('',                  format='(A24)')   + ' %                                         (31=LVV|32=LHH|33=LHV|34=LVH|)
  printf, 1, string('',                  format='(A24)')   + ' %                                         (41=XVV|42=XHH|43=XHV|44=XVH|)
  if lf.asc_des eq '' then lf.asc_des='U'
  lf.asc_des= strupcase( strmid(lf.asc_des, 0, 1) )
  printf, 1, string(lf.asc_des,          format='(A24)')   + ' % orbital pass flag (A=ascending/D=descending)'
  printf, 1, string(lf.scaling,          format='(F24.6)') + ' % Scaling'
  printf, 1, string(lf.beam_mode,        format='(A24)')   + ' % SAR beam mode code'
  if tag_value(lf, 'cpol_remove_noise' ) ne '' then $
    printf, 1, string(lf.cpol_remove_noise,        format='(A24)')   + ' % Cpol Remove Noise'
  if tag_value(lf, 'xpol_remove_noise' ) ne '' then $
    printf, 1, string(lf.xpol_remove_noise,        format='(A24)')   + ' % Xpol Remove Noise'
  printf, 1, ''
  printf, 1, '  -------------------------
  printf, 1, '  SAR Processor Information
  printf, 1, '  -------------------------
  printf, 1, string(wavelength,          format='(F24.5)') + ' % sensor wavelength (m)
  printf, 1, string(lf.radar_frequency,  format='(F24.3)') + ' % sensor frequency (Hz)
  printf, 1, string(lf.sensor_velocity,  format='(A24)')   + ' % sensor velocity relative to image ground swath (m/s)
  printf, 1, string(lf.integration_time, format='(A24)')   + ' % image dwell or integration time (sec)
  printf, 1, string(lf.nom_az_res,       format='(A24)')   + ' % nominal azimuth resolution (m)
  printf, 1, string(lf.nom_rg_res,       format='(A24)')   + ' % nominal range resolution (m)
  printf, 1, string(lf.rcvr_gain,        format='(A24)')   + ' % receiver gain (in db)
  printf, 1, string(lf.proc_gain,        format='(A24)')   + ' % relative processor gain (in db)
  
  printf, 1, ' '
  printf, 1, '  -----------------------------
  printf, 1, '  Image Orientation Information
  printf, 1, '  -----------------------------
  printf, 1, string(lf.range_type,        format='(A24)')   + ' % map descriptor '
  printf, 1, string(element_spacing,      format='(F24.6)') + ' % element/range spacing (m)'
  printf, 1, string(line_spacing,         format='(F24.6)') + ' % line/azimuth spacing (m)'
  printf, 1, string(lf.range_flag,        format='(I24)')   + ' % range type indicator flag (0=slant|1=ground)
  printf, 1, string(lf.heading,           format='(F24.6)') + ' % heading angle in degrees relative to true North (positive clockwise)'
  printf, 1, string(lf.heading_rel,       format='(F24.6)') + $
    ' % heading angle relative to output image LtoR line (+=counter-clockwise,0=L-R line,direction angle)'
  printf, 1, string(lf.flip,              format='(I24)')   + ' % flip flag (0=not flipped|1=flipped elements|2=flipped lines|3=flipped elements & lines)'
  printf, 1, string(lf.illumination_dir,  format='(F24.2)') +  $
    ' % actual illumination direction angle in degrees from L-R image line, clockwise negative (e.g., +90 = image bottom to top direction)
  ;printf, 1, string(lf.early_azimuth_edge,format='(A24)')   + ' % early azimuth edge {L|R|T|B}
  ;printf, 1, string(lf.near_range_edge,   format='(A24)')   + ' % near range edge {L|R|T|B}
  printf, 1, string(lf.range_swath,       format='(F24.6)') + ' % range swath width (m)'
  printf, 1, string(lf.azimuth_swath,     format='(F24.6)') + ' % azimuth swath width (m)'
  printf, 1, string(lf.s_c_altitude,      format='(F24.6)') + ' % sensor altitude (m) corresponding to image center'
  printf, 1, string(lf.center_incid,      format='(F24.6)') + ' % incidence angle to image center pixel (degrees)'
  printf, 1, string(lf.slant_range_center,format='(F24.6)') + ' % slant range distance to image center pixel (m)
  printf, 1, string(lf.slant_range_near,  format='(F24.6)') + ' % slant range distance to near image edge (m)
  printf, 1, string(lf.ny1,               format='(I24)')   + ' % total number of lines in sio image'
  printf, 1, string(lf.nx1,               format='(I24)')   + ' % total number of elements (per line) in sio image'
  printf, 1, string(lf.ny0,               format='(I24)')   + ' % maximum number of lines in master (original) CEOS image'
  printf, 1, string(lf.nx0,               format='(I24)')   + ' % maximum number of elements in master (original) CEOS image'
  printf, 1, string('0',                  format='(A24)')   + ' % reference corner line number relative to master (original) CEOS image
  printf, 1, string('0',                  format='(A24)')   + ' % reference corner element number relative to master (original) CEOS image
  printf, 1, string(lf.ave_line,          format='(I24)')   + ' % reference line spacing increment relative to master (original) CEOS image'
  printf, 1, string(lf.ave_pixel,         format='(I24)')   + ' % reference element spacing increment relative to master (original) CEOS image'
  printf, 1, string(lf.range_looks,       format='(F24.3)') + ' % Number of range   looks in original image
  printf, 1, string(lf.azimuth_looks,     format='(F24.3)') + ' % Number of azimuth looks in original image
  printf, 1,' '
  printf, 1,'  ---------------------------- '
  printf, 1,'  Image Positional Information '
  printf, 1,'  ---------------------------- '
  printf, 1, string(lf.ellipsoidName,     format='(A24)')   + ' % reference ellipsoid (datum) '
  printf, 1, string(lf.semiMajorAxis,     format='(F24.3)') + ' % ref. ellipsoid semi-major axis in meters'
  printf, 1, string(lf.semiMinorAxis,     format='(F24.3)') + ' % ref. ellipsoid semi-minor axis in meters'
  printf, 1, string(dx,format='(F10.1)') + ', ' +  string(dy,format='(F10.1)') +', ' +  string(dx,format='(F10.1)')     + $
    ' % datum (ellipsoid) shifts dx, dy, dz in meters'
  printf, 1, string(dx2,format='(F10.1)') + ', ' +  string(dy2,format='(F10.1)') +', ' +  string(dx2,format='(F10.1)')  + $
    ' % aux. datum (ellipsoid) shifts (rotations) 1st, 2nd, 3rd'
  printf, 1, string(lf.ellipsoidscale,    format='(A24)')   + ' % datum (ellipsoid) scale factor '
  printf, 1, string(lf.center_lat,        format='(F24.5)') + ' % center inner image latitude (degrees)'
  printf, 1, string(lf.center_lon,        format='(F24.5)') + ' % center inner image longitude (degrees)'
  ;
  printf, 1, string(ans_poly_cal(     0, lf.ny1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % UL outer image corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(     0, lf.ny1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % UL outer image corner longitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1, lf.ny1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % UR outer image corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1, lf.ny1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % UR outer image corner longitude (degrees)'
  printf, 1, string(ans_poly_cal(     0,     0,  lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % LL outer image corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(     0,     0,  lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % LL outer image corner longitude (degrees)
  printf, 1, string(ans_poly_cal(lf.nx1,     0,  lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % LR outer image corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1,     0,  lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % LR outer image corner longitude (degrees)'
  ;
  printf, 1, string(ans_poly_cal(     0, lf.ny1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % UL inner image true corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(     0, lf.ny1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % UL inner image true corner longitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1, lf.ny1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % UR inner image true corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1, lf.ny1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % UR inner image true corner longitude (degrees)'
  printf, 1, string(ans_poly_cal(     0,      0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % LL inner image true corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(     0,      0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % LL inner image true corner longitude (degrees)
  printf, 1, string(ans_poly_cal(lf.nx1,      0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp), format='(F24.5)') + ' % LR inner image true corner latitude (degrees)'
  printf, 1, string(ans_poly_cal(lf.nx1,      0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp), format='(F24.5)') + ' % LR inner image true corner longitude (degrees)'
  printf, 1, string(lf.ny1,                                                              format='(I24)')   + ' % UL inner image true corner line location'
  printf, 1, string(1,                                                                   format='(I24)')   + ' % UL inner image true corner element location'
  printf, 1, string(lf.ny1,                                                              format='(I24)')   + ' % UR inner image true corner line location'
  printf, 1, string(lf.nx1,                                                              format='(I24)')   + ' % UR inner image true corner element location'
  printf, 1, string(1,                                                                   format='(I24)')   + ' % LL inner image true corner line location'
  printf, 1, string(1,                                                                   format='(I24)')   + ' % LL inner image true corner element location'
  printf, 1, string(1,                                                                   format='(I24)')   + ' % LR inner image true corner line location'
  printf, 1, string(lf.nx1,                                                              format='(I24)')   + ' % LR inner image true corner element location'
  ;
  printf, 1, string(lf.ny1/2,             format='(I24)')   + ' % center inner image line location
  printf, 1, string(lf.nx1/2,             format='(I24)')   + ' % center inner image element location
  printf, 1,' '
  printf, 1,'   ---------------------------------
  printf, 1,'   Polynomial Transform Coefficients
  printf, 1,'   ---------------------------------
  printf, 1, string('Fit',                format='(A24)')   + ' % source for longitude/latitude polynomial transform
  printf, 1, string(lf.deg,               format='(I24)')   + ' % degree of polynomial for longitude and latitude'
  ;
  ; Specify the longitude base
  ;
  tt=  string('1', format='(A24)') + ' % polynomial transform order (in all transforms, longitude data ranges from -180 to 180
  tt1= string('2', format='(A24)') + ' % polynomial transform order (in all transforms, positive longitudes set to [long - 360] so values range from 0 to -360
  if lf.lngswitchflag eq 1 then tt=tt1
  printf, 1, tt
  
  ;-----------------------------------------------------
  ; print out coefficients for converting i,j to latitude
  ;------------------------------------------------------
  nc=     n_elements(lf.lat_coef)
  tt1=    ' % latitude transform coef for element^'
  tt2=    ' line^'
  for i=0, nc - 1 do begin
    tt= string(lf.lat_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.lat_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.lat_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  
  
  ;-----------------------------------------------------
  ; print out coefficient for converting i,j to longitude
  ;------------------------------------------------------
  nc=     n_elements(lf.lon_coef)
  if lf.lngswitchflag eq 1 then begin
    tt1=' % longitude (0to360) transform coef for element^'
  endif else begin
    tt1=' % longitude (-180to180) transform coef for element^'
  endelse
  
  tt2=    ' line^'
  for i=0, nc - 1 do begin
    tt= string(lf.lon_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.lon_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.lon_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  
  
  
  ;-----------------------------------------------------
  ; print out coefficient for converting longitude, latitude to line j
  ;------------------------------------------------------
  nc=     n_elements(lf.j_coef)
  tt1=    ' % line transform coef for longitude^'
  tt2=    ' latitude^'
  for i=0, nc - 1 do begin
    tt= string(lf.j_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.j_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.j_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  
  
  
  ;-----------------------------------------------------
  ; print out coefficient for converting longitude, latitude to line i
  ;------------------------------------------------------
  nc=     n_elements(lf.i_coef)
  tt1=    ' % element transform coef for longitude^'
  tt2=    ' latitude^'
  for i=0, nc - 1 do begin
    tt= string(lf.i_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.i_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.i_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  
  printf, 1,string('Fit',        format='(A24)') +   ' % source for incident angle polynomial transform
  printf, 1,string(lf.deg_incid, format='(I24)') +   ' % degree of polynomial for incident angle fit
  
  ;-----------------------------------------------------
  ; print out coefficient for converting i,j to incidence
  ;------------------------------------------------------
  nc=     n_elements(lf.incid_coef)
  tt1=    ' % incident angle (deg) transform coef for element^'
  tt2=    ' line^'
  for i=0, nc - 1 do begin
    tt= string(lf.incid_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.incid_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.incid_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  
  printf, 1,string('Fit',        format='(A24)')  +   ' % source for radar look angle polynomial transform (0=N, +/-180)
  printf, 1,string(lf.deg_rlook, format='(I24)')  +   ' % degree of polynomial for look angle fit
  
  ;-----------------------------------------------------
  ; print out coefficient for converting i,j to radar look direction
  ;------------------------------------------------------
  nc=     n_elements(lf.rlook_coef)
  tt1=    ' % radar look direction (deg) transform coef for element^'
  tt2=    ' line^'
  for i=0, nc - 1 do begin
    tt= string(lf.rlook_coef[i], format='(E24.12)')
    tt= tt+ tt1 + strcompress(string(lf.rlook_xexp(i), format='(I)'), /remove_all)
    tt= tt+ tt2 + strcompress(string(lf.rlook_yexp(i), format='(I)'), /remove_all)
    printf, 1, tt
  endfor
  ;-----------------------------------------------------
  ; print out coefficients for converting i to sigma_ne in dB
  ;------------------------------------------------------
  ;
  printf, 1,string('Fit',        format='(A24)')  +   ' % source for sigma ne polynomial transform
  printf, 1,string(lf.deg, format='(I24)')        +   ' % degree of polynomial for sigma ne
  if tag_test(lf, 'sigma_ne_coef') then begin
    nc=     n_elements(lf.sigma_ne_coef)
    tt1=    ' % sigma ne (dB) transform coef for element^'
    for i=0, nc -1 do begin
      tt= string(lf.sigma_ne_coef[i], format='(E24.12)')
      tt= tt + tt1 + strcompress(string(i, format='(I)'), /remove_all)
      printf, 1, tt
    endfor
  endif else begin
    for i=0, lf.deg do begin
      tt= string(0.0, format='(E24.12)')
      tt= tt + tt1 + strcompress(string(i, format='(I)'), /remove_all)
      printf, 1, tt
    endfor
  endelse
  ;
  ; Print out NRCS adjustment
  ;
  if tag_test(lf, 'nrcs_slope') then begin
    printf, 1, string(lf.nrcs_slope, format='(E24.12)') + ' % NRCS slope correction'
  endif else begin
    printf, 1, string(1.0, format='(E24.12)')            + ' % NRCS slope correction'
  endelse
  if  tag_test(lf, 'nrcs_bias') then begin
    printf, 1, string(lf.nrcs_bias, format='(E24.12)')  + ' % NRCS bias correction'
  endif else begin
    printf, 1, string(0.0, format='(E24.12)')            + ' % NRCS slope correction'
  endelse
  
  
  close, 1
  if printflag then print, 'Completed writing out ' + filename +'.'
  
end
