;----------------------------------------------------------
;+
; NAME: ANS_WRITE_LEVEL1B_NRCS
; PURPOSE:'
;   The purpose of this program is to write a level1B NRCS file.  This
;   file could be used to generate subsequent level-2 products. This file
;   will allow the averaging the NRCS data to a specific sampling.
; CALLING SEQUENCE:'
;   ans_write_level1b_nrcs, deffile, sio_ancil_file, sio_nrcs_file, sio_land_mask_file, ncdf_file
; INPUT:'
;   deffile            = Defaults file
;   sio_ancil_file     = Ancillary data in SIO format
;   sio_nrcs_file      = NRCS data in SIO format
;   sio_land_mask_file = Land mask in SIO format
; OUTPUT
;   ncdf_file          = NetCDF output
; KEYWORDS:            
;   icemask            =  Add icemask to file. This in an option
;   /quiet             =  Do not print intermediate results'
;   /help              =  List help.
;   error              =  error (<0 is failure)
;                         -1 Print out help
;                         -2 Cannot read defaults file
;                         -3 Cannot read SIO ancillary data file
;                         -4 Cannot read SIO NRCS file
;                         -5 Cannot read SIO land mask file
; MODIFICATION
;   2016 Jun 08 Frank Monaldo. Creation
; Copyright (C) 6, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro ans_write_level1b_nrcs, deffile, sio_ancil_file, sio_nrcs_file, sio_land_mask_file, ncdf_file,  $
                           icemaske = icemask  , $
                           quiet    = quiet    , $
                           help     = help     , $
                           error    = error

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ANS_WRITE_LEVEL1B_NRCS
  print, ' PURPOSE:'
  print, '   The purpose of this program is to write a level1B NRCS file.  This
  print, '   file could be used to generate subsequent level-2 products. This file
  print, '   will allow the averaging the NRCS data to a specific sampling.
  print, ' CALLING SEQUENCE:'
  print, '   ans_write_level1b_nrcs, deffile, sio_ancil_file, sio_nrcs_file, sio_land_mask_file, ncdf_file
  print, ' INPUT:'
  print, '   deffile            = Defaults file
  print, '   sio_ancil_file     = Ancillary data in SIO format
  print, '   sio_nrcs_file      = NRCS data in SIO format
  print, '   sio_land_mask_file = Land mask in SIO format
  print, ' OUTPUT
  print, '   ncdf_file          = NetCDF output
  print, ' KEYWORDS:            
  print, '   /icemask           =  Add icemask to file. This in an option
  print, '   /quiet             =  Do not print intermediate results'
  print, '   /help              =  List help.
  print, '   error              =  error (<0 is failure)
  error = -1
  return
endif
time1  =   dt_tm_tojs( systime() ) 
error  =   0
!quiet =   1
maxbyte=   255B
;
; Parcing input parameters
;
printflag= 1
if keyword_set(quiet) then printflag=0
;
;=======================================================
;  Read in defaults file
;=======================================================
;
if file_test( deffile) then begin
  ans_read_defaults, deffile, df, quiet=quiet, error = error
  if error lt 0 then begin
    if printflag then print, 'Cannot find defaults file: ' + deffile
    error = -2
    return
  endif 
endif else begin
  if printflag then print, 'Cannot find defaults file: ' + deffile
  error =  -2
  return
endelse
;
;=======================================================
;  Read in ancillary data file file
;=======================================================
;
if file_test( sio_ancil_file ) then begin
  ans_read_sio_info, sio_ancil_file, info, quiet=quiet, error=error
  if error lt 0 then begin
    if printflag then print, 'Cannot read ancillary data file: ' + sio_ancil_file
    error = -3
    return
  endif 
endif else begin
  if printflag then print, 'Cannot find ancillary data file: ' + sio_ancil_file
  error =  -3
  return
endelse
;
;=======================================================
;  Read in NRCS data file
;=======================================================
;
if file_test( sio_nrcs_file ) then begin
  ans_read_sio_data, sio_nrcs_file, nrcs, quiet=quiet, error=error
  if error lt 0 then begin
    if printflag print, 'Cannot read NRCS data file: ' + sio_nrcs_file
    error = -4
    return
  endif 
endif else begin
  if printflag then print, 'Cannot find NRCS data file: ' + sio_nrcs_file
  error =  -4
  return
endelse
;
; 
;=======================================================
;  Read in land mask file data file and rescale to the
;  same size as the NRCS data
;=======================================================
;
if file_test( sio_nrcs_file ) then begin
  ans_read_sio_data, sio_land_mask_file, msk, quiet=quiet, error=error
  if error lt 0 then begin
    if printflag print, 'Cannot read land mask data file: ' + sio_land_mask_file
    error = -5
    return
  endif 
endif else begin
  if printflag then print, 'Cannot find and land RCS data file: ' + sio_land_mask_file
  error =  -5
  return
endelse
;
; Rescale land mask if necessary to match the NRCS file
;
sz_nrcs      = size(nrcs)
sz_mask      = size(msk)
if sz_nrcs[1] ne sz_mask[1] or $
   sz_nrcs[2] ne sz_mask[2] then begin
   msk=  congrid(msk, sz_nrcs[1], nz_nrcs[2] )   
endif





;
;
; Compute longitude, latitude, incident andlge and radar look direction variables
;
i         =  dblarr(info.nx,info.ny)
j         =  dblarr(info.nx,info.ny)
iline     = findgen(info.nx)
jline     = findgen(info.ny)
for ii=0L , info.ny-1 do i(*,ii) = iline
for ii=0L , info.nx-1 do j(ii,*) = jline
longitude = ans_poly_cal(i, j, info.lon_coef  , info.lon_xexp   , info.lon_yexp  )
latitude  = ans_poly_cal(i, j, info.lat_coef  , info.lat_xexp   , info.lat_yexp  )
incid     = ans_poly_cal(i, j, info.incid_coef, info.incid_xexp , info.incid_yexp)
rlook     = ans_poly_cal(i, j, info.rlook_coef, info.rlook_xexp , info.rlook_yexp)
;
; if icemask does not exist, create an icemask filled with zeros
;
if n_elements(icemask) lt 5 then begin
  sz= size(mask)
  icemask = bytarr( sz[1], sz[2] )
endif
;
; Remapping from two to theree dimensions.  The additional dimension is a time layer
;

sz   =   size(sar_wind)
;
; Test for global attributes information
;
finstitution              = 1 & if tag_value( info, 'institution'              ) eq '' then finstitution              = 0
fanswrs_version           = 1 & if tag_value( info, 'answrs_version'           ) eq '' then fanswrs_version           = 0
fanswrs_processing_center = 1 & if tag_value( info, 'answrs_processing_center' ) eq '' then fanswrs_processing_center = 0
fanswrs_product_name      = 1 & if tag_value( info, 'answrs_product_name'      ) eq '' then fanswrs_product_name      = 0
fanswrs_note              = 1 & if tag_value( info, 'answrs_note'              ) eq '' then fanswrs_note              = 0
foriginal_sar_copyright   = 1 & if tag_value( info, 'original_sar_copyright'   ) eq '' then foriginal_sar_copyright   = 0
fplatform_note            = 1 & if tag_value( info, 'platform_note'            ) eq '' then fplatform_note            = 0
fpng_note                 = 1 & if tag_value( info, 'png_note'                 ) eq '' then fpng_note                 = 0
fplatform_name            = 1 & if tag_value( info, 'platform_name'            ) eq '' then fplatform_name            = 0
facq_date_time            = 1 & if tag_value( info, 'acq_date_time'            ) eq '' then facq_date_time            = 0
facq_date_time_js         = 1 & if tag_value( info, 'acq_date_time_js'         ) eq '' then facq_date_time_js         = 0
facq_date_time_begin      = 1 & if tag_value( info, 'acq_date_time_begin'      ) eq '' then facq_date_time_begin      = 0
facq_date_time_end        = 1 & if tag_value( info, 'acq_date_time_end'        ) eq '' then facq_date_time_end        = 0
fsio_create_date_time     = 1 & if tag_value( info, 'sio_create_date_time'     ) eq '' then fsio_create_date_time     = 0
froot_file                = 1 & if tag_value( info, 'root_file'                ) eq '' then froot_file                = 0
fdirfile                  = 1 & if tag_value( info, 'dirfile'                  ) eq '' then fdirfile                  = 0
ficemaskfile              = 1 & if tag_value( info, 'icemaskfile'              ) eq '' then ficemaskfile              = 0
fsensor_frequency_hz      = 1 & if tag_value( info, 'sensor_frequency_hz'      ) eq '' then fsensor_frequency_hz      = 0
fsensor_wavelength_m      = 1 & if tag_value( info, 'sensor_wavelength_m'      ) eq '' then fsensor_wavelength_m      = 0
ffrequency_band           = 1 & if tag_value( info, 'frequency_band'           ) eq '' then ffrequency_band           = 0
fpolarization             = 1 & if tag_value( info, 'polarization'             ) eq '' then fpolarization             = 0
fgmf                      = 1 & if tag_value( info, 'gmf'                      ) eq '' then fgmf                      = 0
falpha                    = 1 & if tag_value( info, 'alpha'                    ) eq '' then falpha                    = 0
forbit_pass_flag          = 1 & if tag_value( info, 'orbit_pass_flag'          ) eq '' then forbit_pass_flag          = 0
fday_night                = 1 & if tag_value( info, 'day_night'                ) eq '' then fday_night                = 0
fsar_beam_code            = 1 & if tag_value( info, 'sar_beam_code'            ) eq '' then fsar_beam_code            = 0
fproduct_type             = 1 & if tag_value( info, 'product_type'             ) eq '' then fproduct_type             = 0
ficefile                  = 1 & if tag_value( info, 'icefile'                  ) eq '' then ficefile                  = 0
ficefile_time_js          = 1 & if tag_value( info, 'icefile_time_js'          ) eq '' then ficefile_time_js          = 0
ficefile_time             = 1 & if tag_value( info, 'icefile_time'             ) eq '' then ficefile_time             = 0
ftitle                    = 1 & if tag_value( info, 'title'                    ) eq '' then ftitle                    = 0
fsource                   = 1 & if tag_value( info, 'source'                   ) eq '' then fsource                   = 0
fhistory                  = 1 & if tag_value( info, 'history'                  ) eq '' then fhistory                  = 0
freference                = 1 & if tag_value( info, 'reference'                ) eq '' then freference                = 0
fcomments                 = 1 & if tag_value( info, 'comments'                 ) eq '' then fcomments                 = 0
fConventions              = 1 & if tag_value( info, 'Conventions'              ) eq '' then fConventions              = 0
flicense                  = 1 & if tag_value( info, 'license'                  ) eq '' then flicense                  = 0
fnaming_authority         = 1 & if tag_value( info, 'naming_authority'         ) eq '' then fnaming_authority         = 0
fcrs                      = 1 & if tag_value( info, 'crs'                      ) eq '' then fcrs                      = 0
fsensor                   = 1 & if tag_value( info, 'sensor'                   ) eq '' then fsensor                   = 0
fgrid_mapping             = 1 & if tag_value( info, 'grid_mapping'             ) eq '' then fgrid_mapping             = 0
fmeta_data_conventions    = 1 & if tag_value( info, 'meta_data_conventions'    ) eq '' then fmeta_data_conventions    = 0
fmeta_link                = 1 & if tag_value( info, 'meta_link'                ) eq '' then fmeta_link                = 0
fkeywords                 = 1 & if tag_value( info, 'keywords'                 ) eq '' then fkeywords                 = 0
fkeywords_vocabulary      = 1 & if tag_value( info, 'keywords_vocabulary'      ) eq '' then fkeywords_vocabulary      = 0
fstandard_name_vocabulary = 1 & if tag_value( info, 'standard_name_vocabulary' ) eq '' then fstandard_name_vocabulary = 0
fcreator_name             = 1 & if tag_value( info, 'creator_name'             ) eq '' then fcreator_name             = 0
fcreator_email            = 1 & if tag_value( info, 'creator_email'            ) eq '' then fcreator_email            = 0
fcreator_url              = 1 & if tag_value( info, 'creator_url'              ) eq '' then fcreator_url              = 0
fcontributor_name         = 1 & if tag_value( info, 'contributor_name'         ) eq '' then fcontributor_name         = 0
fcontributor_role         = 1 & if tag_value( info, 'contributor_role'         ) eq '' then fcontributor_role         = 0
fproject                  = 1 & if tag_value( info, 'project'                  ) eq '' then fproject                  = 0
fpublisher_name           = 1 & if tag_value( info, 'publisher_name'           ) eq '' then fproject                  = 0
fpublisher_email          = 1 & if tag_value( info, 'publisher_email'          ) eq '' then fpublisher_email          = 0
fpublisher_url            = 1 & if tag_value( info, 'publisher_url'            ) eq '' then fpublisher_url            = 0
fprocess_level            = 1 & if tag_value( info, 'process_level'            ) eq '' then fprocess_level            = 0
fcdm_data_type            = 1 & if tag_value( info, 'cdm_data_type'            ) eq '' then fcdm_data_type            = 0
ffeature_type             = 1 & if tag_value( info, 'feature_type'             ) eq '' then ffeature_type             = 0
fnodc_template_version    = 1 & if tag_value( info, 'nodc_template_version'    ) eq '' then fnodc_template_version    = 0
finstrument               = 1 & if tag_value( info, 'instrument'               ) eq '' then finstrument               = 0
fproject_acronym          = 1 & if tag_value( info, 'project_acronym'          ) eq '' then fproject_acronym          = 0
fmodel_source             = 1 & if tag_value( info, 'model_source'             ) eq '' then fmodel_source             = 0
fwind_summary             = 1 & if tag_value( info, 'wind_summary'             ) eq '' then fwind_summary             = 0
fwind_acknowledgement     = 1 & if tag_value( info, 'wind_acknowledgement'     ) eq '' then fwind_acknowledgement     = 0
;
; Additional attriibutes
;
time_format                = 'Y$0n$0d$Th$m$s$'
netcdf_version_id          = '0.0.0'
product_version            = 'UNKNOWN'
time_coverage_start        = '20000000T000000'
time_coverage_end          = '20000000T000000'
platform                   = 'UNKNOWN'
time_coverage_units        = 'days'
time_coverage_duration     = 'P31Y128D'
time_coverage_resolution   = 'P7D'
geospatial_lat_min         = strcompress( string( min(latitude)  , format='(F15.10)') )
geospatial_lat_max         = strcompress( string( max(latitude)  , format='(F15.10)') )
geospatial_lat_units       = 'degrees_north'
geospatial_lat_resolution  = '1'
geospatial_lon_min         = strcompress( string( min(longitude) , format='(F15.10)') )
geospatial_lon_max         = strcompress( string( max(longitude) , format='(F15.10)') )
geospatial_lon_units       = 'degrees_east'
geospatial_lon_resolution  = '1'
geospatial_vertical_min    = '10'
geospatial_vertical_max    = '10'
geospatial_units           = 'meters'
geospatial_resolution      = '1'
source                     = 'UNKNOWN'
model_source               = 'UNKNOWN'
instrument                 = 'UNKNOWN'


help, 'ncdf', /dlm, output=output ; Detemine netcdf_version_id
netcdf_version_id               = getwrd(strcompress(output[1]), 1)
netcdf_version_id               = stress(netcdf_version_id,'D',0,',',ned)
if fanswrs_version then product_version = info.answrs_version
level_2_create_date_timejs      = dt_tm_tojs( systime(/utc))
level_2_create_date_time        = dt_tm_fromjs( level_2_create_date_timejs, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
date_created                    = dt_tm_fromjs( level_2_create_date_timejs, format=time_format)
date_issued                     = date_created
if facq_date_time_begin then begin
  dt   =  getwrd(info.acq_date_time_begin, 0)
  tm   =  getwrd(info.acq_date_time_begin, 1)
  yr   =  getwrd(                      dt, 0, del='-')
  mn   =  getwrd(                      dt, 1, del='-')
  dy   =  getwrd(                      dt, 2, del='-')
  time_coverage_startjs = dt_tm_tojs( yr + ' ' + month2name( fix(mn) ) + ' ' + dy + ' ' + tm )
endif
if facq_date_time_end then begin
  dt   =  getwrd(info.acq_date_time_end, 0)
  tm   =  getwrd(info.acq_date_time_end, 1)
  yr   =  getwrd(                    dt, 0, del='-')
  mn   =  getwrd(                    dt, 1, del='-')
  dy   =  getwrd(                    dt, 2, del='-')
  time_coverage_endjs = dt_tm_tojs( yr + ' ' + month2name( fix(mn) ) + ' ' + dy + ' ' + tm )
endif
time_coverage_start             = dt_tm_fromjs(time_coverage_startjs, format=time_format)
time_coverage_end               = dt_tm_fromjs(time_coverage_endjs  , format=time_format)
if fplatform_name then platform     = tag_value( info, 'platform_name' )
if fmodel_source  then model_source = tag_value( info, 'model_source'  )
if fsource        then source       = tag_value( info, 'source'        )
if fsensor        then instrument   = tag_value( info, 'sensor'        )


;
id= ncdf_create( outputfile, /clobber)
;
; Do not fill the file with default data.
;
ncdf_control, id, /fill


;
; Global Attributes
;
if ftitle                    then NCDF_ATTPUT, id, /GLOBAL, 'title'                                 , info.title
if finstitution              then NCDF_ATTPUT, id, /GLOBAL, 'institution'                           , info.institution
if fsource                   then NCDF_ATTPUT, id, /GLOBAL, 'source'                                , info.source
if fhistory                  then NCDF_ATTPUT, id, /GLOBAL, 'history'                               , info.history
if freference                then NCDF_ATTPUT, id, /GLOBAL, 'reference'                             , info.reference
if fcomments                 then NCDF_ATTPUT, id, /GLOBAL, 'comments'                              , info.comments
if fConventions              then NCDF_ATTPUT, id, /GLOBAL, 'Conventions'                           , info.Conventions
if flicense                  then NCDF_ATTPUT, id, /GLOBAL, 'license'                               , info.license
if fnaming_authority         then NCDF_ATTPUT, id, /GLOBAL, 'naming_authority'                      , info.naming_authority
if fsensor                   then NCDF_ATTPUT, id, /GLOBAL, 'sensor'                                , info.sensor
if fgrid_mapping             then NCDF_ATTPUT, id, /GLOBAL, 'grid_mapping'                          , info.grid_mapping
if fmeta_data_conventions    then NCDF_ATTPUT, id, /GLOBAL, 'meta_data_conventions'                 , info.meta_data_conventions
if fmeta_data_conventions    then NCDF_ATTPUT, id, /GLOBAL, 'Metadata_Conventions'                  , info.meta_data_conventions
if fmeta_link                then NCDF_ATTPUT, id, /GLOBAL, 'meta_link'                             , info.meta_link
if fkeywords                 then NCDF_ATTPUT, id, /GLOBAL, 'keywords'                              , info.keywords
if fkeywords_vocabulary      then NCDF_ATTPUT, id, /GLOBAL, 'keywords_vocabulary'                   , info.keywords_vocabulary
if fstandard_name_vocabulary then NCDF_ATTPUT, id, /GLOBAL, 'standard_name_vocabulary'              , info.standard_name_vocabulary
if fcreator_name             then NCDF_ATTPUT, id, /GLOBAL, 'creator_name'                          , info.creator_name
if fcreator_email            then NCDF_ATTPUT, id, /GLOBAL, 'creator_email'                         , info.creator_email
if fcreator_url              then NCDF_ATTPUT, id, /GLOBAL, 'creator_url'                           , info.creator_url
if fcontributor_name         then NCDF_ATTPUT, id, /GLOBAL, 'contributor_name'                      , info.contributor_name
if fcontributor_role         then NCDF_ATTPUT, id, /GLOBAL, 'contributor_role'                      , info.contributor_role
if fproject                  then NCDF_ATTPUT, id, /GLOBAL, 'project'                               , info.project
if fpublisher_name           then NCDF_ATTPUT, id, /GLOBAL, 'publisher_name'                        , info.publisher_name
if fpublisher_email          then NCDF_ATTPUT, id, /GLOBAL, 'publisher_email'                       , info.publisher_email
if fpublisher_url            then NCDF_ATTPUT, id, /GLOBAL, 'publisher_url'                         , info.publisher_url
if fprocess_level            then NCDF_ATTPUT, id, /GLOBAL, 'process_level'                         , info.process_level
if fprocess_level            then NCDF_ATTPUT, id, /GLOBAL, 'processing_level'                      , info.process_level
if fcdm_data_type            then NCDF_ATTPUT, id, /GLOBAL, 'cdm_data_type'                         , info.cdm_data_type
if ffeature_type             then NCDF_ATTPUT, id, /GLOBAL, 'feature_type'                          , info.feature_type
if fnodc_template_version    then NCDF_ATTPUT, id, /GLOBAL, 'nodc_template_version'                 , info.nodc_template_version
if finstrument               then NCDF_ATTPUT, id, /GLOBAL, 'instrument'                            , info.instrument
if fproject_acronym          then NCDF_ATTPUT, id, /GLOBAL, 'project_acronym'                       , info.project_acronym
if fmodel_source             then NCDF_ATTPUT, id, /GLOBAL, 'model_source'                          , info.model_source
if fwind_summary             then NCDF_ATTPUT, id, /GLOBAL, 'summary'                               , info.wind_summary
if fwind_acknowledgement     then NCDF_ATTPUT, id, /GLOBAL, 'acknowledgement'                       , info.wind_acknowledgement
                                  NCDF_ATTPUT, id, /GLOBAL, 'netcdf_version_id'                     , netcdf_version_id
                                  NCDF_ATTPUT, id, /GLOBAL, 'product_version'                       , product_version
                                  NCDF_ATTPUT, id, /GLOBAL, 'time_coverage_start'                   , time_coverage_start
                                  NCDF_ATTPUT, id, /GLOBAL, 'time_coverage_end'                     , time_coverage_end
                                  NCDF_ATTPUT, id, /GLOBAL, 'platform'                              , platform
                                  NCDF_ATTPUT, id, /GLOBAL, 'time_coverage_units'                   , time_coverage_units
                                  NCDF_ATTPUT, id, /GLOBAL, 'time_coverage_duration'                , time_coverage_duration
                                  NCDF_ATTPUT, id, /GLOBAL, 'time_coverage_resolution'              , time_coverage_resolution
                                  NCDF_ATTPUT, id, /GLOBAL, 'rows'                                  , strcompress(string(info.ny))
                                  NCDF_ATTPUT, id, /GLOBAL, 'columns'                               , strcompress(string(info.nx))
                                  NCDF_ATTPUT, id, /GLOBAL, 'geospatial_vertical_min'               , geospatial_vertical_min
                                  NCDF_ATTPUT, id, /GLOBAL, 'geospatial_vertical_max'               , geospatial_vertical_max
                                  NCDF_ATTPUT, id, /GLOBAL, 'geospatial_units'                      , geospatial_units
                                  NCDF_ATTPUT, id, /GLOBAL, 'geospatial_resolution'                 , geospatial_resolution
;
; ANSWRS (SAROPS) specific global attributes
;
                                  NCDF_ATTPUT, id, /GLOBAL, 'answrs:level_2_windfile'               , getwrd(outputfile, 0, /last, del='/')
                                  NCDF_ATTPUT, id, /GLOBAL, 'answrs:level_2_create_date_time'       , level_2_create_date_time
if fanswrs_version           then NCDF_ATTPUT, id, /GLOBAL, 'answrs:answrs_version'                 , info.answrs_version
if fanswrs_processing_center then NCDF_ATTPUT, id, /GLOBAL, 'answrs:answrs_processing_center'       , info.answrs_processing_center
if fanswrs_product_name      then NCDF_ATTPUT, id, /GLOBAL, 'answrs:answrs_product_name'            , info.answrs_product_name
if fanswrs_note              then NCDF_ATTPUT, id, /GLOBAL, 'answrs:answrs_note'                    , info.answrs_note
if foriginal_sar_copyright   then NCDF_ATTPUT, id, /GLOBAL, 'answrs:original_sar_copyright'         , info.original_sar_copyright
if fplatform_note            then NCDF_ATTPUT, id, /GLOBAL, 'answrs:platform_note'                  , info.platform_note
if fpng_note                 then NCDF_ATTPUT, id, /GLOBAL, 'answrs:png_note'                       , info.png_note
if fplatform_name            then NCDF_ATTPUT, id, /GLOBAL, 'answrs:platform_name'                  , info.platform_name
if facq_date_time            then NCDF_ATTPUT, id, /GLOBAL, 'answrs:acquistion_time'                , info.acq_date_time
if facq_date_time_js         then NCDF_ATTPUT, id, /GLOBAL, 'answrs:acquistion_time_julian_seconds' , info.acq_date_time_js
if facq_date_time_begin      then NCDF_ATTPUT, id, /GLOBAL, 'answrs:acq_date_time_begin'            , info.acq_date_time_begin
if facq_date_time_end        then NCDF_ATTPUT, id, /GLOBAL, 'answrs:acq_date_time_end'              , info.acq_date_time_end
if fsio_create_date_time     then NCDF_ATTPUT, id, /GLOBAL, 'answrs:sio_create_date_time'           , info.sio_create_date_time
if froot_file                then NCDF_ATTPUT, id, /GLOBAL, 'answrs:sar_file_or_directory_name'     , info.root_file
if fdirfile                  then NCDF_ATTPUT, id, /GLOBAL, 'answrs:direction_file_name'            , info.dirfile
if ficemaskfile              then NCDF_ATTPUT, id, /GLOBAL, 'answrs:ice_mask_file_name'             , getwrd(info.icemaskfile, 0, /last, del='/')
if fsensor_frequency_hz      then NCDF_ATTPUT, id, /GLOBAL, 'answrs:radar_frequency_hz'             , info.sensor_frequency_hz
if fsensor_wavelength_m      then NCDF_ATTPUT, id, /GLOBAL, 'answrs:radar_wavelength_m'             , info.sensor_wavelength_m
if ffrequency_band           then NCDF_ATTPUT, id, /GLOBAL, 'answrs:radar_band'                     , info.frequency_band
if fpolarization             then NCDF_ATTPUT, id, /GLOBAL, 'answrs:polarization'                   , info.polarization
if fgmf                      then NCDF_ATTPUT, id, /GLOBAL, 'answrs:geophysical_model_function'     , info.gmf
if falpha                    then NCDF_ATTPUT, id, /GLOBAL, 'answrs:alpha'                          , info.alpha
if forbit_pass_flag          then NCDF_ATTPUT, id, /GLOBAL, 'answrs:orbit_direction'                , info.orbit_pass_flag
if fday_night                then NCDF_ATTPUT, id, /GLOBAL, 'answrs:day_night'                      , info.day_night
if fsar_beam_code            then NCDF_ATTPUT, id, /GLOBAL, 'answrs:sar_beam_code'                  , info.sar_beam_code
if fproduct_type             then NCDF_ATTPUT, id, /GLOBAL, 'answrs:product_type'                   , info.product_type
if ficefile                  then NCDF_ATTPUT, id, /GLOBAL, 'answrs:icefile'                        , info.icefile
if ficefile_time_js          then NCDF_ATTPUT, id, /GLOBAL, 'answrs:icefile_time_js'                , info.icefile_time_js
if ficefile_time             then NCDF_ATTPUT, id, /GLOBAL, 'answrs:icefile_time'                   , info.icefile_time
;

if printflag then begin
  print, 'title                                 : ' + info.title
  print, 'institution                           : ' + info.institution
  print, 'source                                : ' + info.source
  print, 'history                               : ' + info.history
  print, 'reference                             : ' + info.reference
  print, 'comments                              : ' + info.comments
  print, 'Conventions                           : ' + info.Conventions
  print, 'license                               : ' + info.license
  print, 'naming_authority                      : ' + info.naming_authority
  print, 'sensor                                : ' + info.sensor
  print, 'grid_mapping                          : ' + info.grid_mapping
  print, 'meta_data_conventions                 : ' + info.meta_data_conventions
  print, 'meta_link                             : ' + info.meta_link
  print, 'keywords                              : ' + info.keywords
  print, 'keywords_vocabulary                   : ' + info.keywords_vocabulary
  print, 'standard_name_vocabulary              : ' + info.standard_name_vocabulary
  print, 'creator_name                          : ' + info.creator_name
  print, 'creator_email                         : ' + info.creator_email
  print, 'creator_url                           : ' + info.creator_url
  print, 'contributor_name                      : ' + info.contributor_name
  print, 'contributor_role                      : ' + info.contributor_role
  print, 'project                               : ' + info.project
  print, 'publisher_email                       : ' + info.publisher_email
  print, 'publisher_url                         : ' + info.publisher_url
  print, 'process_level                         : ' + info.process_level
  print, 'cdm_data_type                         : ' + info.cdm_data_type
  print, 'feature_type                          : ' + info.feature_type
  print, 'nodc_template_version                 : ' + info.nodc_template_version
  print, 'instrument                            : ' + info.instrument
  print, 'project_acronym                       : ' + info.project_acronym
  print, 'model_source                          : ' + info.model_source
  print, 'netcdf_version_id                     : ' + netcdf_version_id
  print, 'product_version                       : ' + product_version
  print, 'time_coverage_start                   : ' + time_coverage_start
  print, 'time_coverage_end                     : ' + time_coverage_end
  print, 'platform                              : ' + platform
  print, 'time_coverage_units                   : ' + time_coverage_units
  print, 'time_coverage_duration                : ' + time_coverage_duration
  print, 'time_coverage_resolution              : ' + time_coverage_resolution
  print, 'rows                                  : ' + strcompress(string(info.ny))
  print, 'columns                               : ' + strcompress(string(info.nx))
  print, 'geospatial_vertical_min               : ' + geospatial_vertical_min
  print, 'geospatial_vertical_max               : ' + geospatial_vertical_max
  print, 'geospatial_units                      : ' + geospatial_units
  print, 'geospatial_resolution                 : ' + geospatial_resolution
  print, 'answrs:level_2_windfile               : ' + getwrd(outputfile, 0, /last, del='/')
  print, 'answrs:level_2_create_date_time       : ' + level_2_create_date_time
  print, 'answrs:answrs_version                 : ' + info.answrs_version
  print, 'answrs:answrs_processing_center       : ' + info.answrs_processing_center
  print, 'answrs:answrs_product_name            : ' + info.answrs_product_name
  print, 'answrs:answrs_note                    : ' + info.answrs_note
  print, 'answrs:original_sar_copyright         : ' + info.original_sar_copyright
  print, 'answrs:platform_note                  : ' + info.platform_note
  print, 'answrs:png_note                       : ' + info.png_note
  print, 'answrs:platform_name                  : ' + info.platform_name
  print, 'answrs:acquistion_time                : ' + info.acq_date_time
  print, 'answrs:acquistion_time_julian_seconds : ' + info.acq_date_time_js
  print, 'answrs:acq_date_time_begin            : ' + info.acq_date_time_begin
  print, 'answrs:acq_date_time_end              : ' + info.acq_date_time_end
  print, 'answrs:sio_create_date_time           : ' + info.sio_create_date_time
  print, 'answrs:sar_file_or_directory_name     : ' + info.root_file
  print, 'answrs:direction_file_name            : ' + info.dirfile
  print, 'answrs:ice_mask_file_name             : ' + getwrd(info.icemaskfile, 0, /last, del='/')
  print, 'answrs:radar_frequency_hz             : ' + info.sensor_frequency_hz
  print, 'answrs:radar_wavelength_m             : ' + info.sensor_wavelength_m
  print, 'answrs:radar_band                     : ' + info.frequency_band
  print, 'answrs:polarization                   : ' + info.polarization
  print, 'answrs:geophysical_model_function     : ' + info.gmf
  print, 'answrs:alpha                          : ' + info.alpha
  print, 'answrs:orbit_direction                : ' + info.orbit_pass_flag
  print, 'answrs:day_night                      : ' + info.day_night
  print, 'answrs:sar_beam_code                  : ' + info.sar_beam_code
  print, 'answrs:product_type                   : ' + info.product_type
  print, 'answrs:icefile                        : ' + info.icefile
  print, 'answrs:icefile_time_js                : ' + info.icefile_time_js
  print, 'answrs:icefile_time                   : ' + info.icefile_time
endif
;
; Declare dimensions
;
if printflag then print, 'Declaring dimensions for Level-2 netCDF file.'
nfit           =  n_elements(info.lon_coef)
nxid           =  NCDF_DIMDEF(id, 'x'     , info.nx    )
nyid           =  NCDF_DIMDEF(id, 'y'     , info.ny    )
nfitid         =  NCDF_DIMDEF(id, 'xfit'  , nfit       )

;
; Scalar values
;
;
if printflag then print, 'Declaring IDs for scalar quantities for Level-2 netCDF file.'
acquisition_time_id        = NCDF_VARDEF(id, 'acquisition_time'      ,  /double )
nx_id                      = NCDF_VARDEF(id, 'nx'                    ,  /long   )
ny_id                      = NCDF_VARDEF(id, 'ny'                    ,  /long   )
nx0_id                     = NCDF_VARDEF(id, 'nx0'                   ,  /long   )
ny0_id                     = NCDF_VARDEF(id, 'ny0'                   ,  /long   )
nx00_id                    = NCDF_VARDEF(id, 'nx00'                  ,  /long   )
ny00_id                    = NCDF_VARDEF(id, 'ny00'                  ,  /long   )
xn_id                      = NCDF_VARDEF(id, 'xn'                    ,  /float  )
yn_id                      = NCDF_VARDEF(id, 'yn'                    ,  /float  )
line_size_id               = NCDF_VARDEF(id, 'line_size'             ,  /float  )
pixel_size_id              = NCDF_VARDEF(id, 'pixel_size'            ,  /float  )
model_time_js_id           = NCDF_VARDEF(id, 'model_time_js'         ,  /double )
model_time_js_tau_id       = NCDF_VARDEF(id, 'model_time_js_tau'     ,  /double )
upper_left_longitude_id    = NCDF_VARDEF(id, 'upper_left_longitude'  ,  /float  )
upper_right_longitude_id   = NCDF_VARDEF(id, 'upper_right_longitude' ,  /float  )
upper_left_latitude_id     = NCDF_VARDEF(id, 'upper_left_latitude'   ,  /float  )
upper_right_latitude_id    = NCDF_VARDEF(id, 'upper_right_latitude'  ,  /float  )
start_center_longitude_id  = NCDF_VARDEF(id, 'start_center_longitude',  /float  )
start_center_latitude_id   = NCDF_VARDEF(id, 'start_center_latitude' ,  /float  )
scene_center_longitude_id  = NCDF_VARDEF(id, 'scene_center_longitude',  /float  )
scene_center_latitude_id   = NCDF_VARDEF(id, 'scene_center_latitude' ,  /float  )
lower_left_longitude_id    = NCDF_VARDEF(id, 'lower_left_longitude'  ,  /float  )
lower_right_longitude_id   = NCDF_VARDEF(id, 'lower_right_longitude' ,  /float  )
lower_left_latitude_id     = NCDF_VARDEF(id, 'lower_left_latitude'   ,  /float  )
lower_right_latitude_id    = NCDF_VARDEF(id, 'lower_right_latitude'  ,  /float  )
end_center_longitude_id    = NCDF_VARDEF(id, 'end_center_longitude'  ,  /float  )
end_center_latitude_id     = NCDF_VARDEF(id, 'end_center_latitude'   ,  /float  )
northernmost_latitude_id   = NCDF_VARDEF(id, 'northernmost_latitude' ,  /float  )
southernmost_latitude_id   = NCDF_VARDEF(id, 'southernmost_latitude' ,  /float  )
easternmost_longitude_id   = NCDF_VARDEF(id, 'easternmost_longitude' ,  /float  )
westernmost_longitude_id   = NCDF_VARDEF(id, 'westernmost_longitude' ,  /float  )
nrcs_slope_id              = NCDF_VARDEF(id, 'nrcs_slope'            , /float  )
nrcs_bias_id               = NCDF_VARDEF(id, 'nrcs_bias'             , /float  )


;
; Array data ids
;

sigmaid               = NCDF_VARDEF(id, 'sigma'          ,  [nxid, nyid]   ,  /float  )
sar_windid            = NCDF_VARDEF(id, 'sar_wind'       ,  [nxid, nyid]   ,  /float  )
input_dirid           = NCDF_VARDEF(id, 'input_dir'      ,  [nxid, nyid]   ,  /float  )
model_speedid         = NCDF_VARDEF(id, 'model_speed'    ,  [nxid, nyid]   ,  /float  )
maskid                = NCDF_VARDEF(id, 'mask'           ,  [nxid, nyid]   ,  /short  )
longitudeid           = NCDF_VARDEF(id, 'longitude'      ,  [nxid, nyid]   ,  /float  )
latitudeid            = NCDF_VARDEF(id, 'latitude'       ,  [nxid, nyid]   ,  /float  )
rlookid               = NCDF_VARDEF(id, 'rlook'          ,  [nxid, nyid]   ,  /float  )
incidid               = NCDF_VARDEF(id, 'incid'          ,  [nxid, nyid]   ,  /float  )
icemaskid             = NCDF_VARDEF(id, 'icemask'        ,  [nxid, nyid]   ,  /short  )
lon_coefid            = NCDF_VARDEF(id, 'lon_coef'       ,  [nfitid]       ,  /double )
lon_xexpid            = NCDF_VARDEF(id, 'lon_xexp'       ,  [nfitid]       ,  /float  )
lon_yexpid            = NCDF_VARDEF(id, 'lon_yexp'       ,  [nfitid]       ,  /float  )
lat_coefid            = NCDF_VARDEF(id, 'lat_coef'       ,  [nfitid]       ,  /double )
lat_xexpid            = NCDF_VARDEF(id, 'lat_xexp'       ,  [nfitid]       ,  /float  )
lat_yexpid            = NCDF_VARDEF(id, 'lat_yexp'       ,  [nfitid]       ,  /float  )
i_coefid              = NCDF_VARDEF(id, 'i_coef'         ,  [nfitid]       ,  /double )
i_xexpid              = NCDF_VARDEF(id, 'i_xexp'         ,  [nfitid]       ,  /float  )
i_yexpid              = NCDF_VARDEF(id, 'i_yexp'         ,  [nfitid]       ,  /float  )
j_coefid              = NCDF_VARDEF(id, 'j_coef'         ,  [nfitid]       ,  /double )
j_xexpid              = NCDF_VARDEF(id, 'j_xexp'         ,  [nfitid]       ,  /float  )
j_yexpid              = NCDF_VARDEF(id, 'j_yexp'         ,  [nfitid]       ,  /float  )
incid_coefid          = NCDF_VARDEF(id, 'incid_coef'     ,  [nfitid]       ,  /double )
incid_xexpid          = NCDF_VARDEF(id, 'incid_xexp'     ,  [nfitid]       ,  /float  )
incid_yexpid          = NCDF_VARDEF(id, 'incid_yexp'     ,  [nfitid]       ,  /float  )
rlook_coefid          = NCDF_VARDEF(id, 'rlook_coef'     ,  [nfitid]       ,  /double )
rlook_xexpid          = NCDF_VARDEF(id, 'rlook_xexp'     ,  [nfitid]       ,  /float  )
rlook_yexpid          = NCDF_VARDEF(id, 'rlook_yexp'     ,  [nfitid]       ,  /float  )
;
; units
;
NCDF_ATTPUT, id, acquisition_time_id        , 'units', 'seconds since 2000-01-01 00:00:00'
NCDF_ATTPUT, id, nx_id                      , 'units', '1'
NCDF_ATTPUT, id, ny_id                      , 'units', '1'
NCDF_ATTPUT, id, nx0_id                     , 'units', '1'
NCDF_ATTPUT, id, ny0_id                     , 'units', '1'
NCDF_ATTPUT, id, nx00_id                    , 'units', '1'
NCDF_ATTPUT, id, ny00_id                    , 'units', '1'
NCDF_ATTPUT, id, xn_id                      , 'units', '1'
NCDF_ATTPUT, id, yn_id                      , 'units', '1'
NCDF_ATTPUT, id, line_size_id               , 'units', 'm'
NCDF_ATTPUT, id, pixel_size_id              , 'units', 'm'
NCDF_ATTPUT, id, model_time_js_id           , 'units', 'seconds since 2000-01-01 00:00:00'
NCDF_ATTPUT, id, model_time_js_tau_id       , 'units', 'seconds since 2000-01-01 00:00:00'
NCDF_ATTPUT, id, upper_left_longitude_id    , 'units', 'degrees_east'
NCDF_ATTPUT, id, upper_right_longitude_id   , 'units', 'degrees_east'
NCDF_ATTPUT, id, upper_left_latitude_id     , 'units', 'degrees_north'
NCDF_ATTPUT, id, upper_right_latitude_id    , 'units', 'degrees_north'
NCDF_ATTPUT, id, start_center_longitude_id  , 'units', 'degrees_east'
NCDF_ATTPUT, id, start_center_latitude_id   , 'units', 'degrees_north'
NCDF_ATTPUT, id, scene_center_longitude_id  , 'units', 'degrees_east'
NCDF_ATTPUT, id, scene_center_latitude_id   , 'units', 'degrees_north'
NCDF_ATTPUT, id, lower_left_longitude_id    , 'units', 'degrees_east'
NCDF_ATTPUT, id, lower_right_longitude_id   , 'units', 'degrees_east'
NCDF_ATTPUT, id, lower_left_latitude_id     , 'units', 'degrees_north'
NCDF_ATTPUT, id, lower_right_latitude_id    , 'units', 'degrees_north'
NCDF_ATTPUT, id, end_center_longitude_id    , 'units', 'degrees_east'
NCDF_ATTPUT, id, end_center_latitude_id     , 'units', 'degrees_north'
NCDF_ATTPUT, id, northernmost_latitude_id   , 'units', 'degrees_north'
NCDF_ATTPUT, id, southernmost_latitude_id   , 'units', 'degrees_north'
NCDF_ATTPUT, id, easternmost_longitude_id   , 'units', 'degrees_east'
NCDF_ATTPUT, id, westernmost_longitude_id   , 'units', 'degrees_east'
NCDF_ATTPUT, id, nrcs_slope_id              , 'units', '1'
NCDF_ATTPUT, id, nrcs_bias_id               , 'units', '1'
NCDF_ATTPUT, id, sigmaid                    , 'units', '1'
NCDF_ATTPUT, id, sar_windid                 , 'units', 'm s-1'
NCDF_ATTPUT, id, input_dirid                , 'units', 'degrees'
NCDF_ATTPUT, id, model_speedid              , 'units', 'm s-1'
NCDF_ATTPUT, id, maskid                     , 'units', '1'
NCDF_ATTPUT, id, icemaskid                  , 'units', '1'
NCDF_ATTPUT, id, longitudeid                , 'units', 'degrees_east'
NCDF_ATTPUT, id, latitudeid                 , 'units', 'degrees_north'
NCDF_ATTPUT, id, rlookid                    , 'units', 'degrees'
NCDF_ATTPUT, id, incidid                    , 'units', 'degrees'
NCDF_ATTPUT, id, lon_coefid                 , 'units', '1'
NCDF_ATTPUT, id, lon_xexpid                 , 'units', '1'
NCDF_ATTPUT, id, lon_yexpid                 , 'units', '1'
NCDF_ATTPUT, id, lat_coefid                 , 'units', '1'
NCDF_ATTPUT, id, lat_xexpid                 , 'units', '1'
NCDF_ATTPUT, id, lat_yexpid                 , 'units', '1'
NCDF_ATTPUT, id, i_coefid                   , 'units', '1'
NCDF_ATTPUT, id, i_xexpid                   , 'units', '1'
NCDF_ATTPUT, id, i_yexpid                   , 'units', '1'
NCDF_ATTPUT, id, j_coefid                   , 'units', '1'
NCDF_ATTPUT, id, j_xexpid                   , 'units', '1'
NCDF_ATTPUT, id, j_yexpid                   , 'units', '1'
NCDF_ATTPUT, id, incid_coefid               , 'units', '1'
NCDF_ATTPUT, id, incid_xexpid               , 'units', '1'
NCDF_ATTPUT, id, incid_yexpid               , 'units', '1'
NCDF_ATTPUT, id, rlook_coefid               , 'units', '1'
NCDF_ATTPUT, id, rlook_xexpid               , 'units', '1'
NCDF_ATTPUT, id, rlook_yexpid               , 'units', '1'
;
; Long variable names
;
NCDF_ATTPUT, id, acquisition_time_id       , 'long_name'    , 'seconds since 2000-01-01 00:00:00'
NCDF_ATTPUT, id, nx_id                     , 'long_name'    , 'Number of elements in this file'
NCDF_ATTPUT, id, ny_id                     , 'long_name'    , 'Number of lines in this file'
NCDF_ATTPUT, id, nx0_id                    , 'long_name'    , 'Number of elements in SIO file'
NCDF_ATTPUT, id, ny0_id                    , 'long_name'    , 'Number of lines in SIO file'
NCDF_ATTPUT, id, nx00_id                   , 'long_name'    , 'Number of elements in original SAR file'
NCDF_ATTPUT, id, ny00_id                   , 'long_name'    , 'Number of lines in original SAR file'
NCDF_ATTPUT, id, xn_id                     , 'long_name'    , '1'
NCDF_ATTPUT, id, yn_id                     , 'long_name'    , '1'
NCDF_ATTPUT, id, line_size_id              , 'long_name'    , 'Line size'
NCDF_ATTPUT, id, pixel_size_id             , 'long_name'    , 'Pixel size'
NCDF_ATTPUT, id, model_time_js_id          , 'long_name'    , 'Model time julian seconds since 2000-01-01T00:00:00Z'
NCDF_ATTPUT, id, model_time_js_tau_id      , 'long_name'    , 'Model time plus tau julian seconds since 2000-01-01T00:00:00Z'
NCDF_ATTPUT, id, upper_left_longitude_id   , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, upper_right_longitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, upper_left_latitude_id    , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, upper_right_latitude_id   , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, start_center_longitude_id , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, start_center_latitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, scene_center_longitude_id , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, scene_center_latitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, lower_left_longitude_id   , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, lower_right_longitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, lower_left_latitude_id    , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, lower_right_latitude_id   , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, end_center_longitude_id   , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, end_center_latitude_id    , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, northernmost_latitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, southernmost_latitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, easternmost_longitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, westernmost_longitude_id  , 'long_name'    , 'degrees'
NCDF_ATTPUT, id, nrcs_slope_id             , 'long_name'    , 'NRCS slope correction'
NCDF_ATTPUT, id, nrcs_bias_id              , 'long_name'    , 'NRCS bias correction'
NCDF_ATTPUT, id, acquisition_time_id       , 'long_name'    , 'Acqusition time in Julian seconds since 2000-01-01T00:00:00Z'
NCDF_ATTPUT, id, sigmaid                   , 'long_name'    , 'Normalized Radar Cross Section.'
NCDF_ATTPUT, id, sar_windid                , 'long_name'    , 'SAR-derived wind speed at 10-m height neutral stability'
NCDF_ATTPUT, id, input_dirid               , 'long_name'    , 'Interpolated directions used for wind inversion'
NCDF_ATTPUT, id, model_speedid             , 'long_name'    , 'Interpolated model wind speed (=1 for non model directions)'
NCDF_ATTPUT, id, maskid                    , 'long_name'    , 'Interpolated land mask distance from shore line. Positive values land / Negative value water'
NCDF_ATTPUT, id, longitudeid               , 'long_name'    , 'Longitude array in decimal degrees'
NCDF_ATTPUT, id, latitudeid                , 'long_name'    , 'Latitude array in decimal degrees
NCDF_ATTPUT, id, rlookid                   , 'long_name'    , 'Radar look direction array in decimal degrees from North'
NCDF_ATTPUT, id, incidid                   , 'long_name'    , 'Incident angle array in degrees from nadir'
NCDF_ATTPUT, id, icemaskid                 , 'long_name'    , 'Ice mask 0=no_data 1=water 2=land 3=sea_ice 4=snow'
NCDF_ATTPUT, id, lon_coefid                , 'long_name'    , 'Coefficients to compute longitude in degs from pixel/lines'
NCDF_ATTPUT, id, lon_xexpid                , 'long_name'    , 'Longitude pixel exponents'
NCDF_ATTPUT, id, lon_yexpid                , 'long_name'    , 'Longitude line exponents'
NCDF_ATTPUT, id, lat_coefid                , 'long_name'    , 'Coefficients to compute latitude in degs from pixel/lines'
NCDF_ATTPUT, id, lat_xexpid                , 'long_name'    , 'Latitude pixel exponents'
NCDF_ATTPUT, id, lat_yexpid                , 'long_name'    , 'Latitude line exponents'
NCDF_ATTPUT, id, i_coefid                  , 'long_name'    , 'Coefficients to compute pixel from longitue/latitude'
NCDF_ATTPUT, id, i_xexpid                  , 'long_name'    , 'Pixel longitude exponents'
NCDF_ATTPUT, id, i_yexpid                  , 'long_name'    , 'Pixel latitude exponents'
NCDF_ATTPUT, id, j_coefid                  , 'long_name'    , 'Coefficients to compute line from longitue/latitude'
NCDF_ATTPUT, id, j_xexpid                  , 'long_name'    , 'Line longitude exponents'
NCDF_ATTPUT, id, j_yexpid                  , 'long_name'    , 'Line latitude exponents'
NCDF_ATTPUT, id, incid_coefid              , 'long_name'    , 'Coefficients to compute incid in degs from pixel/lines'
NCDF_ATTPUT, id, incid_xexpid              , 'long_name'    , 'Incid pixel exponents'
NCDF_ATTPUT, id, incid_yexpid              , 'long_name'    , 'Incid line exponents'
NCDF_ATTPUT, id, rlook_coefid              , 'long_name'    , 'Coefficients to compute radar look direction in degs from pixel/lines'
NCDF_ATTPUT, id, rlook_xexpid              , 'long_name'    , 'Radar look direction pixel exponents'
NCDF_ATTPUT, id, rlook_yexpid              , 'long_name'    , 'Radar look direction line exponents'

;
; Flags
;
NCDF_ATTPUT, id, icemaskid               , 'flag_values'    , [0, 1, 2, 3, 4]
NCDF_ATTPUT, id, icemaskid               , 'flag_meanings'  , 'no_data water land sea_ice snow'
NCDF_ATTPUT, id, maskid                  , 'flag_values'    , [-1, 0, 1]
NCDF_ATTPUT, id, maskid                  , 'flag_meanings'  ,'water shore land'
;
; Standard Names etc
NCDF_ATTPUT, id, sar_windid              , 'standard_name'  , 'wind_speed'
NCDF_ATTPUT, id, model_speedid           , 'standard_name'  , 'wind_speed'
NCDF_ATTPUT, id, longitudeid             , 'standard_name'  , 'longitude'
NCDF_ATTPUT, id, latitudeid              , 'standard_name'  , 'latitude'
;
; Scale Factors
;
NCDF_ATTPUT, id, sigmaid                 , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, sar_windid              , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, model_speedid           , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, input_dirid             , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, rlookid                 , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, incidid                 , 'scale_factor'  , float(1.0) , /float
NCDF_ATTPUT, id, icemaskid               , 'scale_factor'  , fix(1)     , /short
;
; Add Offset
;
NCDF_ATTPUT, id, sigmaid                 , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, sar_windid              , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, model_speedid           , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, input_dirid             , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, rlookid                 , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, incidid                 , 'add_offset'    , float(0.0) , /float
NCDF_ATTPUT, id, icemaskid               , 'add_offset'    , fix(0)     , /short
;
; Fill Value
;
NCDF_ATTPUT, id, sigmaid                 , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, sar_windid              , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, model_speedid           , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, input_dirid             , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, rlookid                 , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, incidid                 , '_FillValue'    , float(-999.) , /float
NCDF_ATTPUT, id, icemaskid               , '_FillValue'    , fix(0)       , /short
;
;
; Valid Min
;
NCDF_ATTPUT, id, sigmaid                 , 'valid_min'     , float(0.0    ) , /float
NCDF_ATTPUT, id, sar_windid              , 'valid_min'     , float(0.0    ) , /float
NCDF_ATTPUT, id, model_speedid           , 'valid_min'     , float(0.0    ) , /float
NCDF_ATTPUT, id, input_dirid             , 'valid_min'     , float(-180.0 ) , /float
NCDF_ATTPUT, id, rlookid                 , 'valid_min'     , float(-180.0 ) , /float
NCDF_ATTPUT, id, incidid                 , 'valid_min'     , float(0.0    ) , /float
NCDF_ATTPUT, id, icemaskid               , 'valid_min'     , fix(0)         , /short
;                                          '
; Valid Max
;
NCDF_ATTPUT, id, sigmaid                 , 'valid_max'     , float( 100.0  ) , /float
NCDF_ATTPUT, id, sar_windid              , 'valid_max'     , float( 100.0  ) , /float
NCDF_ATTPUT, id, model_speedid           , 'valid_max'     , float( 100.0  ) , /float
NCDF_ATTPUT, id, input_dirid             , 'valid_max'     , float( 360.0  ) , /float
NCDF_ATTPUT, id, rlookid                 , 'valid_max'     , float( 360.0  ) , /float
NCDF_ATTPUT, id, incidid                 , 'valid_max'     , float( 90.0   ) , /float
NCDF_ATTPUT, id, icemaskid               , 'valid_max'     , fix(5)          , /short
;
; Grid mapping
;
NCDF_ATTPUT, id, sigmaid                 , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, sar_windid              , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, model_speedid           , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, incidid                 , 'grid_mappig'  , 'NONE'
NCDF_ATTPUT, id, icemaskid               , 'grid_mappig'  , 'NONE'
;
; Source
;
NCDF_ATTPUT, id, sigmaid                 , 'source'       ,  source
NCDF_ATTPUT, id, sar_windid              , 'source'       ,  source
NCDF_ATTPUT, id, model_speedid           , 'source'       ,  model_source
NCDF_ATTPUT, id, input_dirid             , 'source'       ,  model_source
NCDF_ATTPUT, id, rlookid                 , 'source'       ,  source
NCDF_ATTPUT, id, incidid                 , 'source'       ,  source
NCDF_ATTPUT, id, icemaskid               , 'grid_mappig'  ,  'IMS National Ice Center'
;
; Reference
;
NCDF_ATTPUT, id, sigmaid                 , 'reference'    , 'NONE'
NCDF_ATTPUT, id, sar_windid              , 'reference'    , 'NONE'
NCDF_ATTPUT, id, model_speedid           , 'reference'    , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'reference'    , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'reference'    , 'NONE'
NCDF_ATTPUT, id, incidid                 , 'reference'    , 'NONE'
NCDF_ATTPUT, id, icemaskid               , 'reference'    , 'NONE'
;
;
; ancillary_variables
;
NCDF_ATTPUT, id, sigmaid                 , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, sar_windid              , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, model_speedid           , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, incidid                 , 'ancillary_variables' , 'NONE'
NCDF_ATTPUT, id, icemaskid               , 'ancillary_variables' , 'NONE'
;
; platform
;
NCDF_ATTPUT, id, sigmaid                 , 'platform'     , platform
NCDF_ATTPUT, id, sar_windid              , 'platform'     , platform
NCDF_ATTPUT, id, model_speedid           , 'platform'     , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'platform'     , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'platform'     , platform
NCDF_ATTPUT, id, incidid                 , 'platform'     , platform
NCDF_ATTPUT, id, icemaskid               , 'platform'     , 'NONE'
;
; instrument
;
NCDF_ATTPUT, id, sigmaid                 , 'instrument'   , instrument
NCDF_ATTPUT, id, sar_windid              , 'instrument'   , instrument
NCDF_ATTPUT, id, model_speedid           , 'instrument'   , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'instrument'   , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'instrument'   , instrument
NCDF_ATTPUT, id, incidid                 , 'instrument'   , instrument
NCDF_ATTPUT, id, icemaskid               , 'instrument'   , 'NONE'
;
; Comment
;
NCDF_ATTPUT, id, sigmaid                 , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, sar_windid              , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, model_speedid           , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, input_dirid             , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, rlookid                 , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, incidid                 , 'Comment'      , 'NONE'
NCDF_ATTPUT, id, icemaskid               , 'Comment'      , 'NONE'
;

NCDF_ATTPUT, id, acquisition_time_id     , 'standard_name'  , 'time'
NCDF_ATTPUT, id, acquisition_time_id     , 'calendar'       , 'gregorian'

;
NCDF_ATTPUT, id, sar_windid              , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, sar_windid              , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, sigmaid                 , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, sar_windid              , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, input_dirid             , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, model_speedid           , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, maskid                  , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, rlookid                 , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, incidid                 , 'coordinates'   , 'longitude latitude'
NCDF_ATTPUT, id, icemaskid               , 'coordinates'   , 'longitude latitude'

;
; Exit control mode and switch to data model
;
ncdf_control, id, /endef
;
; Write the scalar data
;
NCDF_VARPUT, id, acquisition_time_id       , info.acq_date_time_js
NCDF_VARPUT, id, nx_id                     , info.nx
NCDF_VARPUT, id, ny_id                     , info.ny
NCDF_VARPUT, id, nx0_id                    , info.nx0
NCDF_VARPUT, id, ny0_id                    , info.ny0
NCDF_VARPUT, id, nx00_id                   , info.nx00
NCDF_VARPUT, id, ny00_id                   , info.ny00
NCDF_VARPUT, id, xn_id                     , info.xn
NCDF_VARPUT, id, yn_id                     , info.yn
NCDF_VARPUT, id, line_size_id              , info.line_size
NCDF_VARPUT, id, pixel_size_id             , info.pixel_size
NCDF_VARPUT, id, model_time_js_id          , double(info.model_time_js)
NCDF_VARPUT, id, model_time_js_tau_id      , double(info.model_time_js_tau)
NCDF_VARPUT, id, upper_left_longitude_id   , float(info.upper_left_longitude  )
NCDF_VARPUT, id, upper_right_longitude_id  , float(info.upper_right_longitude )
NCDF_VARPUT, id, upper_left_latitude_id    , float(info.upper_left_latitude   )
NCDF_VARPUT, id, upper_right_latitude_id   , float(info.upper_right_latitude  )
NCDF_VARPUT, id, start_center_longitude_id , float(info.start_center_longitude)
NCDF_VARPUT, id, start_center_latitude_id  , float(info.start_center_latitude )
NCDF_VARPUT, id, scene_center_longitude_id , float(info.scene_center_longitude)
NCDF_VARPUT, id, scene_center_latitude_id  , float(info.scene_center_latitude )
NCDF_VARPUT, id, lower_left_longitude_id   , float(info.lower_left_longitude  )
NCDF_VARPUT, id, lower_right_longitude_id  , float(info.lower_right_longitude )
NCDF_VARPUT, id, lower_left_latitude_id    , float(info.lower_left_latitude   )
NCDF_VARPUT, id, lower_right_latitude_id   , float(info.lower_right_latitude  )
NCDF_VARPUT, id, end_center_longitude_id   , float(info.end_center_longitude  )
NCDF_VARPUT, id, end_center_latitude_id    , float(info.end_center_latitude   )
NCDF_VARPUT, id, northernmost_latitude_id  , float(info.northernmost_latitude )
NCDF_VARPUT, id, southernmost_latitude_id  , float(info.southernmost_latitude )
NCDF_VARPUT, id, easternmost_longitude_id  , float(info.easternmost_longitude )
NCDF_VARPUT, id, westernmost_longitude_id  , float(info.westernmost_longitude )
NCDF_VARPUT, id, nrcs_slope_id             , float(info.nrcs_slope            )
NCDF_VARPUT, id, nrcs_bias_id              , float(info.nrcs_bias             )

; Write the array data
;
if printflag then print, 'Writing array data netCDF file.'

NCDF_VARPUT, id, sigmaid            ,  sigma
NCDF_VARPUT, id, sar_windid         ,  sar_wind
NCDF_VARPUT, id, input_dirid        ,  input_dir
NCDF_VARPUT, id, model_speedid      ,  model_speed
NCDF_VARPUT, id, maskid             ,  mask
NCDF_VARPUT, id, icemaskid          ,  icemask
NCDF_VARPUT, id, longitudeid        ,  longitude
NCDF_VARPUT, id, latitudeid         ,  latitude
NCDF_VARPUT, id, rlookid            ,  rlook
NCDF_VARPUT, id, incidid            ,  incid
;
NCDF_VARPUT, id, lon_coefid         ,  info.lon_coef
NCDF_VARPUT, id, lon_xexpid         ,  info.lon_xexp
NCDF_VARPUT, id, lon_yexpid         ,  info.lon_yexp
NCDF_VARPUT, id, lat_coefid         ,  info.lat_coef
NCDF_VARPUT, id, lat_xexpid         ,  info.lat_xexp
NCDF_VARPUT, id, lat_yexpid         ,  info.lat_yexp
NCDF_VARPUT, id, i_coefid           ,  info.i_coef
NCDF_VARPUT, id, i_xexpid           ,  info.i_xexp
NCDF_VARPUT, id, i_yexpid           ,  info.i_yexp
NCDF_VARPUT, id, j_coefid           ,  info.j_coef
NCDF_VARPUT, id, j_xexpid           ,  info.j_xexp
NCDF_VARPUT, id, j_yexpid           ,  info.j_yexp
NCDF_VARPUT, id, incid_coefid       ,  info.incid_coef
NCDF_VARPUT, id, incid_xexpid       ,  info.incid_xexp
NCDF_VARPUT, id, incid_yexpid       ,  info.incid_yexp
NCDF_VARPUT, id, rlook_coefid       ,  info.rlook_coef
NCDF_VARPUT, id, rlook_xexpid       ,  info.rlook_xexp
NCDF_VARPUT, id, rlook_yexpid       ,  info.rlook_yexp

ncdf_close, id

if printflag then print, 'Finished writing out file '+ outputfile

end
