;----------------------------------------------------------
;+
; NAME: ANS_WRITE_NRCS_SIO'
; PURPOSE:'
;    To generate an flat NRCS file in ISO format
; CALLING SEQUENCE:'
;    ans_write_nrcs_sio, image, filename, scaling'
; INPUT:'
;   image=        floating-point NRCS image'
;   filename=     filename for output file
; OUTPUT:
;   scaling=      Scaling to convert to linear NRCS (default=1)
;                 nrcs= image * scaling
; KEYWORDS:
;    /quiet=      Do not print intermediate results'
;    /help=       List help.'
;    /twobyte=    Sotre as scaled two-byte'
; MODIFICATION
;  2007 Aug 22 Frank Monaldo 
;  2008 Sep 13 Chris Jackon  - 
;              Made changes in the Image Identification 
;              and Image Orientation block to ensure that
;              certain lines are always written out
;  2009 May 01 Made sure PALSAR corners are written
;              Frank Monaldo 
;              Corrected problem w/ print out of incident
;              angle coefficeints
;  2009 Jul 27 Frank Monaldo. Added code to keep the program
;              even if the lf.scansar and lf.s_c_altitude do not exist. This
;              was required because these fields are not filled when reading
;              RSAT2 data.
;  2009 Oct 14 Frank Monaldo, Radical modification for new vesion of ANSWRS 2.0
;  2010 Feb 23 Frank Monaldo. Add help and image statistics.
;  2020 Jul 21 Frank Monaldo. Improve printout.
;  2010 Dec 29 Frank Monaldo  Pass on silent info
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ans_write_nrcs_sio, image, filename, scaling, $
                             twobyte=twobyte, quiet=quiet, help=help
                             
                             
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 )then begin
  print, ' NAME: ANS_WRITE_NRCS_SIO'
  print, ' PURPOSE:'
  print, '    To generate an flat NRCS file in ISO format
  print, ' CALLING SEQUENCE:'
  print, '    ans_write_nrcs_sio, image, filename, scaling'
  print, ' INPUT:'
  print, '   image=        floating-point NRCS image'
  print, '   filename=     filename for output file
  print, ' OUTPUT:
  print, '   scaling=      Scaling to convert to linear NRCS (default=1)
  print, '                 nrcs= image * scaling
  print, ' KEYWORDS:
  print, '    /quiet=      Do not print intermediate results'
  print, '    /help=       List help.'
  print, '    /twobyte=    Sotre as scaled two-byte'
  return
endif

sz=   size(image)
nx=   sz[1]
ny=   sz[2]
;
; Eliminate zeros
;
w=where(image lt 0, count)
if count gt 0 then image[w]=0.0


if printflag then print, 'Opening up NRCS file'

if printflag then begin
  if n_elements(twobyte) gt 0 then begin
    print, '  to save scaled two-byte data.'
  endif else begin
    print, '  to save four-byte floating point data.'
  endelse
endif

; ------------------------------------------
; Scale to unsigned two bytes if instructed.
; ------------------------------------------

scaling=1
if n_elements(twobyte) gt 0 then begin
  if printflag then print, 'Scaling to unsigned two byte integer...'
  maximum=          max(image)
  im=               uintarr(nx,ny)
  scaling= 64000./maximum
  if n_elements(image) le 200000000 then begin
    im=image * scaling
  endif else begin
    ; Convert line by line for large images
    for j=0L, ny-1 do begin
      im[*,j] = image[*,j]  
    endfor
  endelse
  image=im
  scaling= 1/ scaling  
endif




if printflag then print, 'Writing out SIO NRCS data to: ' + filename + '.'

magic_number= -16678914L
nx=            long(nx)
ny=            long(ny)
data_type=     3L
if  n_elements(twobyte) gt 0 then data_type=1L
byte_size=     4L
if keyword_set(twobyte) then byte_size=2L
;
; Need to fill in the header data here.
; Note the ny, nx This was necessary for Chris Wackerman ship dectection code
; to properly interpret the data 
;
if printflag then begin
  ;
  print, 'Statistics of saved image NRCS data'
  nyy=   ny/10
  nxx=   nx/10
  print, 'Compute stats for first ' + string(nyy) + ' lines.'
  im=   image[0:nxx, 0:nyy]
  print, 'Mean=    ', mean(im)
  print, 'Std=     ', stdev(im)
  w=where(im gt 0, count)
  if count gt 0 then begin
     print, 'Mean > 0=    ', mean(im[w])
     print, 'Std  > 0=   ' , stdev(im[w])
  endif
endif

if printflag then print, 'Writing ' + filename + '.'

close,  1
openw,  1, filename
writeu, 1, magic_number, ny, nx, data_type, byte_size
writeu, 1, image
close,  1

if printflag then print, 'Finished writing ' + filename + '.'


end
