; -----------------------------------------------------
; +
; NAME: 
;   ASF_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a two/three dimensional
;   polynomial fit to go from pixel location to incidence angle.
; CALLING SEQUENCE:
;    asf_incid2pix_fit, lf
; INPUTS:
;   lf:             Input lf struture
; OUTPUTS:
;    lf.incid_coef= Array of incidence angle coefficients a function of 
;                    i,j.  The output of the fit is in degrees.
; KEYWORDS:
;    /quiet=        Suppress printed messages. 
;    /help=         Display help
; MODIFICATION HISTORY:
;    2010 Jan 05 Frank Monaldo Radically re-written to use new
;                ans_compute_fit routine
;    2010 Jun 14 Frank Monaldo. Print help on no input
;    2010 Nov 06 Frank Monaldo. Rotangle is projection PS-SMM/I
;    2010 Dec 29 Frank Monaldo. Pass through silent
; Copyright (C) 1998-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro  asf_incid2pix_fit, lf, quiet=quiet, help=help


printflag=1
if keyword_set(quiet)  then printflag=0

quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '   ASF_INCID2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a two/three dimensional
  print, '   polynomial fit to go from pixel location to incidence angle.
  print, ' CALLING SEQUENCE:
  print, '    asf_incid2pix_fit, lf
  print, ' INPUTS:
  print, '   lf:             Input lf struture
  print, ' OUTPUTS:
  print, '    lf.incid_coef= Array of incidence angle coefficients a function of 
  print, '                    i,j.  The output of the fit is in degrees.
  print, ' KEYWORDS:
  print, '    /quiet=        Suppress printed messages. 
  print, '    /help=         Display help
  return
endif
if printflag then begin
  print, 'Computing a fit to incid_angle as a function of image index...'
endif
rad2deg=   180./!pi
re=        6378.00 ; The Earth's radius in kilometers
;

grid=     30
nxx=      lf.nx1 / grid
n=        findgen(lf.nx1/grid)
i1=       ans_poly_cal( lf.near_start_lon, lf.near_start_lat, lf.i_coef, lf.i_xexp, lf.i_yexp)
j1=       ans_poly_cal( lf.near_start_lon, lf.near_start_lat, lf.j_coef, lf.j_xexp, lf.j_yexp)
i2=       ans_poly_cal( lf.near_end_lon,   lf.near_end_lat,   lf.i_coef, lf.i_xexp, lf.i_yexp)
j2=       ans_poly_cal( lf.near_end_lon,   lf.near_end_lat,   lf.j_coef, lf.j_xexp, lf.j_yexp)
rotang=   atan(j2-j1,i2-i1)
;if lf.scansar eq 1 and lf.projection eq 'PS-SMM/I' then rotang=rotang + !pi/2.
iinside=  findgen(nxx)*cos(rotang)*grid + i1
jinside=  findgen(nxx)*sin(rotang)*grid + j1
;
; We wish to find equations of the form form
;  theta=  sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n


;  The image pt is the near range pixel point.
;
;  A is the angle between a vector from the s/c to the image pt and
;                         a vector from the s/c to the Earth center
;  B is the angle between a vector from the image pt to the s/c and
;                         a vector from the image pt to the Earth center  
;  C is the angle between a vector from the Earth center to the s/c and
;                         a vector from the earth center to the image pt
;  Some form of the following relations are used for the computations:
;
;  alpha^2 = beta^2  + gamma^2 - 2 * beta  * gamma * cos( A )
;  beta^2  = alpha^2 + gamma^2 - 2 * alpha * gamma * cos( B )
;  gamma^2 = alpha^2 + betas^2 - 2 * alpha * beta  * cos( c )
;
;

alpha=  re
alpha2= alpha * alpha
beta=   lf.s_c_altitude + re
beta2=  beta * beta
gamma=  lf.sl_rng_1st_pix
gamma2= gamma * gamma
cosB=   (gamma2 + alpha2 - beta2  ) / 2. /  alpha  / gamma
B=      acos(cosB)
cosA=   (gamma2 + beta2  - alpha2 ) / 2. /  beta   / gamma
A=      acos(cosA)
cosC=   (alpha2 + beta2  - gamma2 ) / 2. /  alpha  / beta
C=      acos(cosC)
count_total=-1L
rotangle=rotang-!pi/2.
incid_angle= dblarr(nxx,nxx)
inext=       dblarr(nxx,nxx)
jnext=       dblarr(nxx,nxx)
for na=0L, nxx-1L do begin      ; Loop in azimuth
  lonstart= ans_poly_cal(iinside[na], jinside[na], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  latstart= ans_poly_cal(iinside[na], jinside[na], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)  
  for nr=0L, nxx-1  do begin   ; Loop in range  
    inext[na,nr]=       double(nr*cos(rotangle)*grid+iinside[na])
    jnext[na,nr]=       double(nr*sin(rotangle)*grid+jinside[na]) 
    lonnext=            ans_poly_cal(inext[na,nr], jnext[na,nr], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
    latnext=            ans_poly_cal(inext[na,nr], jnext[na,nr], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
    ll2rb, lonstart, latstart, lonnext, latnext, dist, azi 
    ;  Compute an smallc
    gamma2i=            alpha2 + beta2 - 2 * alpha * beta * cos( c + dist )
    cosBi=              ( alpha2 + gamma2i -  beta2 ) /( 2.0 * alpha * sqrt( gamma2i) )
    incid_angle[na,nr]= (!pi - acos(cosBi))*rad2deg
  endfor
endfor

inext=       reform(inext,       n_elements(inext)       )
jnext=       reform(jnext,       n_elements(jnext)       )
incid_angle= reform(incid_angle, n_elements(incid_angle) )

if printflag then print, 'i,j => incidence'
ans_compute_fit,  inext,   jnext,  incid_angle, lf.deg, incid_coef, incid_xexp, incid_yexp, quiet=quiet

temp_lf= create_struct( lf, 'incid_coef',  incid_coef, $
                             'incid_xexp', incid_xexp, $
                             'incid_yexp', incid_yexp )


lf=      0
lf=      temp_lf
end
