; -----------------------------------------------------
; +
; NAME: 
;    ASF_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a two/three dimensional
;   polynomial fit to go from lon/lat to pixel location and visa versa.
;   This program depends on knowing the pixel to lon/lat relationship
;   from the asf_ps_ll program or the as_ac_ll program.
; CALLING SEQUENCE:
;   asf_lonlat2pix_fit, lf                     
; INPUTS:
;   lf.deg=              Degree of fit
;   lf.nx0=              Number of horizontal elements in the ximage
;   lf.ny0=              Number of vertical elements in the image.
;   lf.ave_pixel=        The number of horizontal pixels averaged.
;   lf.ave_line=         The number of vertical lines averaged.
;   lf.corner_ne=        The north-east corner array in meters.
;   lf.corner_ll=        The latitude and longitude of corners.
;   lf.projection=       Leader file string that specifies the projection
;   lf.asc_des=          "DESCENDING" or "ASCENDING";   
;   lf.working_direcory= Working directory
; OUTPUTS:
;    lf.i_coef:         Array of coefficients for horizontal pixel fit
;    lf.j_coef=         Array of coefficients for vertical pixel  fit
;    lf.lon_coef=       Array of longitude coefficients
;    lf.lat_coef=       Array of latitude coefficients
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
; MODIFICATION HISTORY:
;   2010 Jan 05 Frank Monaldo. Radicaly re-written to use new ans_compute_fit routine
;   2010 Jan 25 Frank Monaldo. Corrected mispelling of lf.i_coef
;   2010 Jun 14 Frank Monaldo. Print help on no input
;   2010 Jul 15 Frank Monaldo. Add ability to ingest geographical shifts
;   2010 Dec 29 Frank MOnaldo. Pass on quiet
; Copyright (C) 1998-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro asf_lonlat2pix_fit, lf, quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
if keyword_set(help)  or (n_params(0) lt 1)  then begin  
  print, ' NAME: 
  print, '    ASF_LONLAT2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a two/three dimensional
  print, '   polynomial fit to go from lon/lat to pixel location and visa versa.
  print, '   This program depends on knowing the pixel to lon/lat relationship
  print, '   from the asf_ps_ll program or the as_ac_ll program.
  print, ' CALLING SEQUENCE:
  print, '   asf_lonlat2pix_fit, lf                     
  print, ' INPUTS:
  print, '   lf.deg=              Degree of fit
  print, '   lf.nx0=              Number of horizontal elements in the image
  print, '   lf.ny0=              Number of vertical elements in the image.
  print, '   lf.ave_pixel=        The number of horizontal pixels averaged.
  print, '   lf.ave_line=         The number of vertical lines averaged.
  print, '   lf.corner_ne=        The north-east corner array in meters.
  print, '   lf.corner_ll=        The latitude and longitude of corners.
  print, '   lf.projection=       Leader file string that specifies the projection
  print, '   lf.asc_des=          "DESCENDING" or "ASCENDING";   
  print, '   lf.working_direcory= Working directory
  print, ' OUTPUTS:
  print, '    lf.i_coef:          Array of coefficients for horizontal pixel fit
  print, '    lf.j_coef=          Array of coefficients for vertical pixel  fit
  print, '    lf.lon_coef=        Array of longitude coefficients
  print, '    lf.lat_coef=        Array of latitude coefficients
  print, ' KEYWORDS:
  print, '    /quiet=    Suppress printed messages. 
  print, '    /help=     Display help
  return
endif

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

;case strcompress(lf.projection,/remove_all) of
;'PS-SMM/I': begin
;            ;if lf.lngswitchflag eq 0 then begin
;            ;    asf_ps_ll, i,j, lng, lat, df, lf, /quiet 
;            ;endif else begin
;            ;    asf_ps_ll, i,j, lng, lat, df, lf, /lngswitch, /quiet
;            ;endelse
;            end
;ELSE:       begin
            ;
            ; Use ERIM-developed code for i,j <=> lat,lon conversion
            ; 
            ;
            s=    getfile(lf.working_directory +'/outloc_file.txt')
            ns=   n_elements(s)
            lng=  dblarr(ns)
            lat=  dblarr(ns)
            i=    dblarr(ns)
            j=    dblarr(ns)
            for is=0, ns - 1 do begin
              i[is]=    double( getwrd( s[is], 0, del=',' ) )
              j[is]=    double( getwrd( s[is], 1, del=',' ) )
              lat[is]=  double( getwrd( s[is], 2, del=',' ) )
              lng[is]=  double( getwrd( s[is], 3, del=',' ) )
              if lf.lngswitchflag eq 1 and lng[is] lt 0 then lng[is]=  360.00+lng[is]
            endfor
            if lf.sar_sensor eq 'ERS-2' then begin
              ;
              ; Shift i index by 192. Map index 192 to 0
              ;
              i=i-192.0
            endif
            i= (i - 1) / double(lf.ave_pixel)
            j= (j - 1) / double(lf.ave_line) 
            j= lf.ny1 - 1 - j
;            end	
;endcase

;
; If a range bering correction is available then apply it
;
if tag_test(lf, 'range_shift') and tag_test(lf, 'bering_shift') then begin
  if lf.range_shift ne 0 then begin
    if printflag then begin
      print, 'Appling range bering correction...'
      print, 'Range shift='+ string(lf.range_shift) + ' radians or ' + $
                             string(lf.range_shift*6378000.0) + ' m'
      print, 'Bering shift='+ string(lf.bering_shift) + ' degrees'
    endif
    n= n_elements(lng)
    if printflag then hlp, lng, lat
    iistep= n/10
    for ii=0L, n-1 do begin
      tt= 'Adjusting location ' 
      tt= tt + string(lng[ii], format='(F10.5)') + string(lat[ii], format='(F10.5)') 
      rb2ll, lng[ii], lat[ii], lf.range_shift, lf.bering_shift,  lo, la
      lng[ii]=lo
      lat[ii]=la
      tt= tt + ' to ' + string(lng[ii], format='(F10.5)') + string(lat[ii], format='(F10.5)')
      if printflag and ii mod iistep eq 0 then print, tt
    endfor
    if lf.lngswitchflag eq 0  and max(lng) gt 180 then begin
      w=where( lng gt 180, count)
      if count gt 0 then lng[w]= lng[w] - 360
    endif
    if lf.lngswitchflag eq 1  and min(lng) lt 0 then begin
      w=where( lng lt 0, count)
      if count gt 0 then lng[w]= lng[w] + 360
    endif
    if printflag then hlp, lng, lat
  endif
endif


if printflag then print, 'Computing a fit to i/j as function of longitude and latitude

;
; We wish to find equations of the form form
;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;
if printflag then print, 'i,j => longitude'
ans_compute_fit,   i,   j,  lng, lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
if printflag then print, 'i,j => latitude'
ans_compute_fit,   i,   j,  lat, lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet
if printflag then print, 'longitude, lattiude => i'
ans_compute_fit, lng, lat,    i, lf.deg,   i_coef,   i_xexp,   i_yexp, quiet=quiet
if printflag then print, 'longitude, lattiude => j'
ans_compute_fit, lng, lat,    j, lf.deg,   j_coef,   j_xexp,   j_yexp, quiet=quiet


;
; fill in corner_ll for ASF standard mode data
;
if total(lf.corner_ll) eq 0 then begin
  lf.corner_ll[0]= ans_poly_cal(       0, lf.ny1-1, lat_coef, lat_xexp, lat_yexp)
  lf.corner_ll[1]= ans_poly_cal(       0, lf.ny1-1, lon_coef, lon_xexp, lon_yexp)         
  lf.corner_ll[2]= ans_poly_cal(lf.nx1-1, lf.ny1-1, lat_coef, lat_xexp, lat_yexp)
  lf.corner_ll[3]= ans_poly_cal(lf.nx1-1, lf.ny1-1, lon_coef, lon_xexp, lon_yexp)
  lf.corner_ll[2]= ans_poly_cal(lf.nx1-1,        0, lat_coef, lat_xexp, lat_yexp)
  lf.corner_ll[3]= ans_poly_cal(lf.nx1-1,        0, lon_coef, lon_xexp, lon_yexp)
  lf.corner_ll[2]= ans_poly_cal(       0,        0, lat_coef, lat_xexp, lat_yexp)
  lf.corner_ll[3]= ans_poly_cal(       0,        0, lon_coef, lon_xexp, lon_yexp)
endif

temp_lf= create_struct( lf, 'lon_coef', lon_coef,  $
                            'lon_xexp', lon_xexp,  $
                            'lon_yexp', lon_yexp,  $
                            'lat_coef', lat_coef,  $
                            'lat_xexp', lat_xexp,  $
                            'lat_yexp', lat_yexp,  $
                            'i_coef',   i_coef,    $
                            'i_xexp',   i_xexp,    $
                            'i_yexp',   i_yexp,    $
                            'j_coef',   j_coef,    $
                            'j_xexp',   j_xexp,    $
                            'j_yexp',   j_yexp      )
lf=  0
lf= temp_lf
;if printflag then begin
;  print, ' orig i  orig j lon in     lon out      diff     lat in  lat out     diff'
;  for k=0, ns-1 do begin
;    lonout=ans_poly_cal(i(k),j(k), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
;    latout=ans_poly_cal(i(k),j(k), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
;    print, string( i(k),              format='(F7.0)') +  ' ' + $
;           string( j(k),              format='(F7.0)') +  ' ' + $
;           string( lng(k),            format='(F9.4)') +  ' ' + $
;           string( lonout,            format='(F9.4)') +  ' ' + $
;           string( lng(k) - lonout,   format='(F9.4)') +  ' ' + $
;           string( lat(k),            format='(F9.4)') +  ' ' + $
;           string( latout,            format='(F9.4)') +  ' ' + $ 
;           string( lat(k) - latout,  format='(F9.4)')
;  endfor
;  print, ' orig lon  orig lat   orig i  iout  diff orig j   j out   diff'
;  for k=0, ns-1 do begin
;    iout=ans_poly_cal(lng(k),lat(k), lf.i_coef, lf.i_xexp, lf.i_yexp)
;    jout=ans_poly_cal(lng(k),lat(k), lf.j_coef, lf.j_xexp, lf.j_yexp)
;    print, string( lng(k),      format='(F9.4)')  +  ' ' + $
;           string( lat(k),      format='(F9.4)')  +  ' ' + $
;           string( i(k),        format='(F9.1)')  +  ' ' + $
;           string( iout,        format='(F9.1)')  +  ' ' + $
;           string( i(k)-iout,   format='(F10.3)') +  ' ' + $
;           string( j(k),        format='(F9.1)')  +  ' ' + $
;           string( jout,        format='(F9.1)')  +  ' ' + $
;           string( j(k)-jout,   format='(F10.3)') 
;  endfor
;endif

end

