;-----------------------------------------------------
;+
; NAME: ASF_NOMINAL_CAL
; PURPOSE:
;   The purpose of this program is read data from the
;   radiometric data record and generate the appropriate
;   calibration arrays
; CALLING SEQUENCE:
; asf_nominal_cal,  radiometric_data_rec, a1, a2, a3,  noise, /quiet, /help
; INPUT:
;  radiometric_data_rec= byte array from the radiometric projection data
; OUTPUT:
;  a1=    Array with the a1 calibration factor
;  a2=    Array with the a2 calibration factor
;  a3=    Array with the a3 calibration factor
;  noise= Array with the noise calibration factor
;
; Note that sigma= a2 * [ dn^2 - a1 * noise ] + a3
;
; MODIFICATION HISTORY:
; 1998 Nov 11 Frank Monaldo  
; 2010 Feb 28 Frank Monaldo. Re-write for ASNWRS 2.0
; 2010 Mar 14 Frank Monaldo. Change a1_factor to a1 etc
; Copyright (C) 1998-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_nominal_cal,  radiometric_data_rec, a1, a2, a3, noise, $
                      help=help, quiet=quiet
		   
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ASF_NOMINAL_CAL
  print, ' PURPOSE:
  print, '   The purpose of this program is read data from the
  print, '   radiometric data record and generate the appropriate
  print, '   calibration arrays
  print, ' CALLING SEQUENCE:
  print, ' asf_nominal_cal,  radiometric_data_rec, a1, a2, a3, noise, /quiet, /help
  print, ' INPUT:
  print, '  radiometric_data_rec= byte array from the radiometric projection data
  print, ' OUTPUT:
  print, '  a1=    Array with the a1 calibration factor
  print, '  a2=    Array with the a2 calibration factor
  print, '  a3=    Array with the a3 calibration factor
  print, '  noise= Array with the noise calibration factor
  print, '
  print, ' Note that  sigma= a2 * [ dn^2 - a1 * noise ] + a3
  print, '
  return
endif
printflag=1
if keyword_set(quiet)   then printflag=0 


;
; Reading data from record
; 
a1=            float(    string(radiometric_data_rec[  84:  99] ) )
a2=            float(    string(radiometric_data_rec[ 101: 115] ) )
a3=            float(    string(radiometric_data_rec[ 116: 131] ) )
n_samp=        long(     string(radiometric_data_rec[  60:  67] ) )
lookup_tab=    fltarr(n_samp)  
for i=0L, n_samp - 1L do begin 
  b0= i * 16L + 136L 
  b1= b0 + 15L
  lookup_tab[i]= float( string(radiometric_data_rec[b0:b1] ) )
endfor

; Fill in values of 1 with the mean values of the
; other elements of the array.
;
w=      where(lookup_tab eq 1, count)

if count gt 0 then begin
  ww= where(lookup_tab ne 1, count2)
  if count2 gt 0 then begin
    mu=mean(lookup_tab[ww])
    lookup_tab[w]= mu
  endif 
endif

noise=lookup_tab

end
