;--------------------------------------------------------------
;+
; NAME: ASF_PS_LL.PRO
;
; PURPOSE:  The purpose of this program is to convert the x/y
;           coordinates in a polar-stereographic image to latitude and
;           longitude
; CALLING SEQUENCE:
;
; asf_ps_ll, df, lf, x,y, lon, lat, xsamp, ysamp, corner_ne, corner_ll
;
; INPUT:
;              Pixel positions after averaging to make smaller image.
;   x:         horizontal pixel position in polar-stereographic image.
;   y:         vertical   pixel position in polar-stereographic image.
;   ave_pixel: The number of horizontal pixels averaged.
;   ave_line:  The number of vertical lines averaged.
;   xsamp:     Size of x dimension line of tape image  before averaging (meters)
;   ysamp:     Size of y dimension line of tape image  before averaging  (meters)
;   corner_ne: The north-east corner array in meters.
;   corner_ll: The latitude and longitude of corners.
; OUTPUT
;   lon:       Returned longitude in degrees.
;   lat:       Returned latitude  in degrees.
; KEYWORDS:
;   /quiet:   Print no intermediate output
;   /help:    Help. 
;   /lngswitch: Output 0 to 360 longitude, else -180 to 180
; MODIFICATION HISTORY
;   
;    1999 Feb 8  Frank Monaldo 
;    2010 Nov 06 Frank Monaldo Update
;
; Copyright (C) 1998, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_ps_ll, x,y, lon, lat, df, lf, $
		    quiet=quiet, help=help, lngswitch=lngswitch
;

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ASF_PS_LL.PRO
  print, '
  print, ' PURPOSE:  The purpose of this program is to convert the x/y
  print, '           coordinates in a polar-stereographic image to latitude and
  print, '           longitude
  print, ' CALLING SEQUENCE:
  print, '
  print, ' asf_ps_ll, x,y, lon, lat, ave_pixel, ave_line, xsamp, ysamp, corner_ne, $
  print, '           corner_ll, /quiet
  print, '
  print, ' INPUT:
  print, '              Pixel positions after averaging to make smaller image.
  print, '   x:         horizontal pixel position in polar-stereographic image.
  print, '   y:         vertical   pixel position in polar-stereographic image.
  print, '   ave_pixel: The number of horizontal pixels averaged.
  print, '   ave_line:  The number of vertical lines averaged.
  print, '   xsamp:     Size of x dimension line of tape image  before averaging (meters)
  print, '   ysamp:     Size of y dimension line of tape image  before averaging  (meters)
  print, '   corner_ne: The north-east corner array in meters.
  print, '   corner_ll: The latitude and longitude of corners.
  print, ' OUTPUT
  print, '   lon:       Returned longitude in degrees.
  print, '   lat:       Returned latitude  in degrees.
  print, ' KEYWORDS:
  print, '   /quiet:    Print no intermediate output
  print, '   /help:     Help.   
  print, '   /lngswitch:   Output 0 to 360 longitude, else -180 to 180
  return
endif 
printflag=1
if keyword_set(quiet)  then printflag = 0
if printflag eq 1 then begin
  print, ' '
  print, 'Intermediate results from asf_ps_ll.'
endif
lngswitchflag=0
if keyword_set(lngswitch)  then lngswitchflag = lngswitch
;
; constants
;
re=           double(6378140.)   ; Equatorial radius of the Earth in meters.
f=            1/double(298.257)  ; Flattening 
e2=           2*f - f^2          ; Earth's eccentricity squared.
e=            sqrt(e2)           ; Earth's eccentricity.
e4=           e2 * e2            ; Earth's eccentricity^4.
e6=           e4 * e2            ; Earth's eccentricity^6.
r2d=          180./double(!pi)   ; radians to degrees conversion
d2r=          1/r2d              ; degrees to radians conversion
slat=         double(70.0)* d2r  ; Standard latitude
slon=         double(-45)* d2r   ; Standard longitude
sn=           1.0
if lf.corner_ll(0) lt 0 then sn=-1
slat=         sn * slat
;
; References:
; Earth geodetic and geophysical data http://ssd.jpl.nasa.gov/phys_props_earth.html
; Map projection equations, NSWC/DL-7R-3624 
;
xsign=1 
if lf.corner_ne(6)-lf.corner_ne(0) lt 0 then xsign=-1
ysign=1 
if lf.corner_ne(3)-lf.corner_ne(1) lt 0 then ysign=-1
xprime=   float(x) * xsign * float(df.ave_pixel) * lf.pixel_spacing + lf.corner_ne(0) 
yprime=   float(y) * ysign * float(df.ave_line)  * lf.line_spacing + lf.corner_ne(1)

rho=      sqrt( xprime * xprime + yprime * yprime )
cos_slat= cos(slat)
sin_slat= sin(slat)
cm=       cos_slat /sqrt( 1.0 - e2 * ( sqrt( sin_slat ) ) )
t=        tan( !pi / 4 - slat /2.)  / $
             ( ( 1.0 - e * sin_slat ) /(1.0 + e * sin_slat ) )^(e/2.0)
t=        rho * t / (re * cm )
chi=      !pi/2.0 - 2.0 * atan(t)
lat=      chi  + ( e2/2.0 + e4*5.0/24.0 + e6/12.0     ) * sin(2.0*chi)
lat=      lat  + (          e4*7.0/48.0 + e6*9.0/240.0) * sin(4.0*chi)
lat=      lat  + (                        e6*7.0/120.0) * sin(6.0*chi)
lat=      lat * sn * r2d
lon=      atan(sn*yprime,-sn*xprime)*r2d +sn*slon*r2d
lon=      lon * sn
if lf.lngswitchflag eq 1 then begin
  w=where(lon lt 0, count)
  if count gt 0 then lon(w)=360+lon(w)
endif
end




