;-----------------------------------------------------
;+
; NAME: ASF_R_LOOK
; PURPOSE:
;   The purpose of this program is to compute the
;   radar look direction in degrees with respect to 
;   North.  I.e. 90= east
; CALLING SEQUENCE:
;    asf_compute_r_look, lf
; INPUT:
;    lf:             Input structure
; OUTPUT:
;    lf.rlook_coef=      Radar look direction wrt N coefficients   
;    lf.rlook_xexp=      Radar look direction wrt N coefficients  
;    lf.rlook_yexp=      Radar look direction wrt N coefficients  
; KEYWORDS:
;    /quiet=         Print no messages while processing.
;    /help=          Get help.
; MODIFICATION HISTORY:
;
;    2010 Mar 23 Frank Monaldo. Compute look angle from longitue and latitude values
;    2010 Jun 06 Frank Monaldo. Do not need to flip if descending
;    2010 Jun 14 Frank Monaldo. Print help with no input.
;    2010 Dec 29 Frank Monaldo. Pass through silent
; Copyright (C) 1998-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_r_look, lf, help=help, quiet=quiet
		   
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ASF_R_LOOK
  print, ' PURPOSE:
  print, '   The purpose of this program is to compute the
  print, '   radar look direction in degrees with respect to 
  print, '   North.  I.e. 90= east
  print, ' CALLING SEQUENCE:
  print, '    asf_compute_r_look, lf
  print, ' INPUT:
  print, '    lf:             Input structure
  print, ' OUTPUT:
  print, '    lf.rlook_coef=      Radar look direction wrt N coefficients   
  print, '    lf.rlook_xexp=      Radar look direction wrt N coefficients  
  print, '    lf.rlook_yexp=      Radar look direction wrt N coefficients                
  print, ' KEYWORDS:
  print, '    /quiet=         Print no messages while processing.
  print, '    /help=          Get help.
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 

quiet= printflag eq 0
;
; Computing radar look direction from incident angles.
;
look=        dblarr(9)
ii=          dblarr(9)
jj=          dblarr(9)
increment=   4D
ii=          [ 4., lf.nx1/2., lf.nx1-5.,        4., lf.nx1/2., lf.nx1-5.,        0., lf.nx1/2., lf.nx1-5.]
jj=          [ 4.,        4.,        4., lf.ny1/2., lf.ny1/2., lf.ny1/2., lf.ny1-5., lf.ny1-5., lf.ny1-5.]
for i=0, 8 do begin
  lon1= ans_poly_cal( ii[i],             jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat1= ans_poly_cal( ii[i],             jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lon2= ans_poly_cal( ii[i] + increment, jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat2= ans_poly_cal( ii[i] + increment, jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  ll2rb, lon1, lat1, lon2, lat2, dist, azi
  if azi gt 180 then azi= azi - 360.0
  look[i] = double(azi)
  if lf.scansar eq 1 and lf.projection eq 'PS-SMM/I' then look[i]=look[i]+90.0
endfor

;
; If the pass is descending flip the look direction
;
;if strupcase(strmid(lf.asc_des,0,1)) eq 'D' then begin
;  print, 'Pass is descending. Flipping look direction'
;  print, 'Before'
;  hlp, look
;  look= look + 180.0  
;  print, 'After'
;  hlp, look
;endif
;
; Keep values between +/- 180
;
w=where(look gt 180, count)
if count gt 0 then look[w]= look[w]- 360

;
; Compute the i, j to incident angle coefficients.
;
if printflag then print, 'Computing i,j => look angle conversion for ' + lf.sar_sensor
ans_compute_fit, ii,  jj,  look, lf.deg, rlook_coef, rlook_xexp, rlook_yexp, quiet=quiet

if printflag then begin
  print, ' i    j   look '
  for i=0,8 do print, string(ii[i],format='(I5)') +  ' ' + string(jj[i],format='(I5)') + ' ' +string(look[i])
endif

temp_lf= create_struct( lf, 'rlook_coef', rlook_coef, $
                            'rlook_xexp', rlook_xexp, $
                            'rlook_yexp', rlook_yexp  )
                            

lf=0
lf= temp_lf

if printflag then begin
  print, 'rlook coefs', rlook_coef
endif
end
