;-----------------------------------------------------
;+
; NAME: ASF_READ_DATA_FILE.PRO
; PURPOSE:
;   The purpose of this program is read data from the
;   datafile. 
; CALLING SEQUENCE:
;   asf_read_data_file, datafile, image, lf
; INPUT:
;   datafile= string containing data file name.
;   asf_import= Full executable to convert to calibrate
; OUTPUT:
;   image=    image array
; KEYWORD
;   /quiet    Print no intermediate output.
;   /help     Get help.
;   /nominal  Do nominal calibration
;   asf_import= full path to asf_import command (necessary)
;   position_loc= full path to position_locator command (necessary) 
; MODIFICATION HISTORY:
;   1999 Apr 08 Frank Monaldo  
;   2000 Oct 30 Nathaniel Winstead 
;   2002 Nov 08 Frank Monaldo. Returns floating point NRCS array, not dB
;   2004 Oct 06 Frank Monaldo. Modified program to convert to linear NRCS
;               before averaging and to use asf_import command if set.
;   2009 Dec 08 Frank Monaldo. Rewritten for ANSWRS 2.0
;   2009 Dec 20 Frank Monaldo. Added asf_import and postion locator keyword. Removed line to congrid, when
;               no averaging was being performed.
;   2010 Jan 02 Frank Monaldo. Corrected for the fact that the line length for ERS-2 data is 7775
;   2010 Mar 01 Frank Monaldo. Use nominal calibration from model file
;   2016 Jun 13 Chris Jackson. Remove some non ascii characters in comment lines (near line 186 ERS history) 
; Copyright (C) 1998-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

   
pro asf_read_data_file, datafile, image, lf, asf_import=asf_import, nomnal=nominal, $
                        position_loc=position_loc, help=help, quiet=quiet
    
    
if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: ASF_READ_DATA_FILE.PRO
  print, ' PURPOSE:
  print, '   The purpose of this program is read data from the
  print, '   datafile. 
  print, ' CALLING SEQUENCE:
  print, '   asf_read_data_file, datafile, image, lf
  print, ' INPUT:
  print, '   datafile= string containing data file name.
  print, '   asf_import= Full executable to convert to calibrate
  print, ' OUTPUT:
  print, '   image=    image array
  print, ' KEYWORD
  print, '   /quiet    Print no intermediate output.
  print, '   /help     Get help.
  print, '   /nominal  Do nominal calibration
  print, '   asf_import= full path to asf_import command (necessary)
  print, '   position_loc= full path to position_locator command (necessary) 
  return
endif

printflag=1
if keyword_set(quiet) then printflag=0

if printflag then begin
  print, datafile, lf.n_pixel, lf.n_line, lf.ave_pixel, lf.ave_line
endif

if not keyword_set(nominal) then begin
  newdata= datafile + '.img'
  cmd= asf_import + ' ' + ' -sigma '  + datafile + ' ' +  newdata  
  if printflag then print, cmd 
  spawn, cmd
  filename=newdata + '.img'
endif

; Kludge for ERS-2 data
if lf.sar_sensor eq 'ERS-2' then begin
  result=   file_info(filename)
  filesize=  result.size
  lf.n_line=   8192
  lf.n_pixel= filesize / lf.n_line / 4
  lf.nx0=      lf.n_pixel
  lf.nx1=      lf.nx0 / lf.ave_pixel
  if printflag then begin
    print, 'This is ERS-2 data'
    print, 'New image size in lines and pixels...
    print,  lf.n_line, lf.n_pixel
  endif
endif
if printflag then begin
  print, datafile, lf.n_pixel, lf.n_line, lf.ave_pixel, lf.ave_line
endif
n_pixel2= long(lf.n_pixel /lf.ave_pixel)
n_line2=  long(lf.n_line  /lf.ave_line)
image=    fltarr(n_pixel2, n_line2)



result=   file_info(filename)
filesize= result.size
if printflag then print, 'Reading file '+ filename + ' Size=' + strcompress(string(filesize)) + ' bytes'
close, 1
openr, 1, filename

record_length=lf.n_pixel


if printflag then print, 'Declare the image line to be a float array.' + $
             strcompress(lf.n_pixel) + ' elements long.'

image_line=   fltarr(lf.n_pixel)
if keyword_set(nominal) then image_line=bytarr(lf.n_pixel)

new_n_pixel=  n_pixel2 * lf.ave_pixel
sub_image=    fltarr(new_n_pixel, lf.ave_line)

; 
; Read in image
;
bytesread=0L
if lf.sar_sensor ne 'ERS-2' then begin
  for i=0, n_line2-2 do begin
    for ii=0, lf.ave_line - 1 do begin
       image_line=   fltarr(lf.n_pixel)
       readu, 1, image_line 
       if not endian() then image_line=swap_endian(image_line)
       sub_image[*,ii]= float(image_line[0:new_n_pixel-1])
    endfor 
    image[*,i]=    rebin( sub_image, n_pixel2, 1 )
    if (i*lf.ave_line) mod 1000 eq 0 and printflag then begin 
       print, '[used asf_import command]', i*lf.ave_line, max(image_line)
    endif
  endfor
endif else begin
  if printflag then print, 'ERS-2 [used asf_import command]', lf.n_pixel, lf.n_line
  image=fltarr(lf.n_pixel, lf.n_line)
  readu, 1, image
  if not endian() then begin
    if printflag then print, 'Swapping endian in image.'
    image=swap_endian(image)
  endif
  close,1
  if printflag then print, 'Averaging down using congrid, ', n_pixel2, n_line2
  if lf.ave_line gt 1 or lf.ave_pixel gt 1 then begin
    if printflag then print, 'Averaging down using congrid, ', n_pixel2, n_line2
    image= congrid(image, n_pixel2, n_line2)
  endif
endelse
close,1


;
; I need to rotate and flip. It turns out that x increases with line
; and y increases with pixel.  This is opposite the assumption
; built into the asf_read_data_file function.  Only do this
; in scansar mode.
;
lf.projection=strcompress(lf.projection,/remove_all)
if lf.projection eq 'GROUNDRANGE' then lf.projection = 'GROUND RANGE'
if lf.scansar eq 1 and lf.projection eq 'PS-SMM/I' then begin
  if printflag eq 1 then print,'scansar =1 and PS-SMM/I so rotating and flipping.'
  image=reverse(rot(image,90),2)
endif
if lf.scansar eq 1 and lf.projection ne 'PS-SMM/I' then begin 
  if printflag eq 1 then print, 'scansar =1 and not PS-SMM/I so flipping.'
  image=reverse(image,2)		
endif
;
; Perform nominal calibration
;
if keyword_set(nominal) then begin
   adj=1
   if lf.sar_sensor ne 'ERS-2' then begin
     date=    ydn2date(lf.data_year, lf.dn, format='Y$ n$ d$') + ' ' + $
              string(lf.hour,   format='(I2.2)') + ':' +  $
              string(lf.minute, format='(I2.2)') + ':' +  $
              string(lf.second, format='(I2.2)')
     js2=     dt_tm_tojs(date)
     js1=     dt_tm_tojs('1995 Apr 21')
     ;
     ;     Launch - 1/ 1/2001 y = -0.0018x   + 0.0023
     ;     1/ 2/2001 - 2/26/2003 y = -0.002247x + 0.9296
     ;     2/27/2003 - current y = -0.0018x   - 0.3323 + 3
     ;        where:
     ;        x = days since launch
     ;        y = gain adjustment (in db)
     ;
     jsa=     dt_tm_tojs('2001 Jan 01 00:00:00')
     jsb=     dt_tm_tojs('2003 Feb 26 00:00:00')
     slope=   -0.002247 & bias=  0.0023
     if  (js2 gt jsa) and (js2 lt jsb) then begin
       slope=   -0.0018 & bias=   0.9296
     endif
     if  (js2 ge jsb) then begin
       slope=   -0.0018 & bias=   3 - 0.3323 
     endif
     x=       (js2-j1) /86400. ; days 
     y=       slope * x + bias
     adj=     1.0
     adj=     10^(y/10.0)
     adj=     1.0 /adj
   endif
   sz=      size(image)
   nx=      sz[1]
   ny=      sz[2]
   nnoise=  interpol(noise, nn)
   image=   image * image
   for i=0L, ny-1L do begin
     image[*,i]= image[*,i] - lf.a1 * nnoise
   endfor
   image= adj * a2 * image + a3 
endif
end
