;-----------------------------------------------------
;+
; NAME: ASF_READ_FACILITY
;
; PURPOSE:
;        The purpose of this program is read data from the
;        facility  record, this is an updated version of the
;        program that reads the state vector information.
; CALLING SEQUENCE:
;
; asf_read_facility, lf, /help, /quiet
;
; INPUT:
;    lf.facility_data_rec= byte array from the facility_data_rec
;
; OUTPUT:
;    lf.data_take_id:                   data take id
;    lf.image_id:                       image id
;    lf.data_year:                      year of data
;    lf.center_gmt:                     time of data
;       (THESE POSITIONS REFER TO VALID IMAGE AREA)
;    lf.center_lat, center_lon:         center position of image
;    lf.near_start_lat, lf.near_start_lon: near early position.
;    lf.near_end_lat,   lf.near_end_lon:   near late  position.
;    lf.far_start_lat,  lf.far_start_lon:  far  early position.
;    lf.far_end_lat,    lf.far_end_lon:    far  late  position.
;    lf.total pixels,   lf.total_lines:
;    lf.s_c_altitude:                   space craft altitude in km
;    lf.looks_azimuth,  lf.looks_range:    effective number of looks (azi/rng)
;    lf.sl_rng_1st_pix:                 slant range to 1st pixel km
;    lf.azi_res, lf.rng res:               resolution azimuth and range, meters
;    lf.azimuth_pixel, lf.range_pixel:     sampling in range and azimuth, meters 
;    lf.nadir_radius:                   radius of the Earth at nadir km
;    lf.image_radius:                   radius of the Earth at image center km  
;    lf.pos_vector:                     A 3-element array of x,y,z,coordinates (m)
;    lf.vel_vector:                     A 3-element array of xdot,ydot,zdot (m/s)                                   
;
; MODIFICATION HISTORY:
;    1999 feb 19  Frank Monaldo  
;    2000 Oct 30  Nathaniel Winstead  added output to data structure lf
;    2006 Sep 01  Frank Monaldo : Put nominal values in or
;                                 spacecraft altitude and slant range to
;                                 nearest pixel if those values equal 0
;    2010 Jun 14 Frank Monaldo Print help on no inpt
; Copyright (C) 1998, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_read_facility, lf, help=help, quiet=quiet

		   
if keyword_set(help)  or (n_params(0) lt 1) then begin
  print, ' '
  print, ' NAME: ASF_READ_FACILITY'
  print, ''
  print, ' PURPOSE:'
  print, '        The purpose of this program is read data from the'
  print, '        facility  record'
  print, ' CALLING SEQUENCE:
  print, ''
  print, ' asf_read_facility, lf, /help, /quiet'
  print, '
  print, ' INPUT:'
  print, '    lf.facility_data_rec= byte array from the facility_data_rec'
  print, ' '
  print, ' OUTPUT: Added the following variables to data structure lf'
  print, '    lf.data_take_id:                   data take id'
  print, '    lf.image_id:                       image id'
  print, '    lf.data_year:                      year of data'
  print, '    lf.center_gmt:                     time of data'
  print, '    lf.center_lat, center_lon:         center position of image'
  print, '    lf.near_start_lat, lf.near_start_lon: near early position.'
  print, '    lf.near_end_lat,   lf.near_end_lon:   near late  position.'
  print, '    lf.far_start_lat,  lf.far_start_lon:  far  early position.'
  print, '    lf.far_end_lat,    lf.far_end_lon:    far  late  position.'
  print, '    lf.s_c_altitude:                   space craft altitude in km
  print, '    lf.looks_azimuth,  lf.looks_range:    effective number of looks (azi/rng)'
  print, '    lf.azi_res, lf.rng_res:               resolution azimuth and range, meters'
  print, '    lf.azimuth_pixel, lf.range_pixel:     sampling in range and azimuth, meters'
  print, '    lf.nadir_radius:                   radius of the Earth at nadir, km'
  print, '    lf.image_radius:                   radius of the Earth at image center, km' 
  print, '    lf.pos_vector:                     A 3-element array of x,y,z,coordinates (km)
  print, '    lf.vel_vector:                     A 3-element array of xdot,ydot,zdot (m/s)                
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
;
; Determine System Arch
;
arch=!version.arch
;
; Reading data from record
;
lf.datatake_id=           string(lf.facility_data_rec( 20: 33) )
lf.image_id=              string(lf.facility_data_rec( 34: 44) )
lf.data_year=      float( string(lf.facility_data_rec(100:104) ) )
lf.center_gmt=            string(lf.facility_data_rec(105:121) )
lf.center_gmt=     repchr(lf.center_gmt,'T',' ')  ; fix bug from ASF
lf.center_lat=     float( string(lf.facility_data_rec(122:138) ) )
lf.center_lon=     float( string(lf.facility_data_rec(139:155) ) )
lf.near_start_lat= float( string(lf.facility_data_rec(156:172) ) )
lf.near_start_lon= float( string(lf.facility_data_rec(173:189) ) )
lf.near_end_lat=   float( string(lf.facility_data_rec(190:206) ) )
lf.near_end_lon=   float( string(lf.facility_data_rec(207:223) ) )
lf.far_start_lat=  float( string(lf.facility_data_rec(224:240) ) )
lf.far_start_lon=  float( string(lf.facility_data_rec(241:257) ) )
lf.far_end_lat=    float( string(lf.facility_data_rec(258:274) ) )
lf.far_end_lon=    float( string(lf.facility_data_rec(275:291) ) )
lf.total_pixels=   float( string(lf.facility_data_rec(344:352) ) )
lf.total_lines=    float( string(lf.facility_data_rec(353:361) ) )
lf.s_c_altitude=   float( string(lf.facility_data_rec(557:573) ) )
lf.pos_vector(0)=  float( string(lf.facility_data_rec(574:596) ) )
lf.pos_vector(1)=  float( string(lf.facility_data_rec(597:619) ) )
lf.pos_vector(2)=  float( string(lf.facility_data_rec(620:642) ) )
lf.pos_vector=     lf.pos_vector * 1000.00
lf.vel_vector(0)=  float( string(lf.facility_data_rec(643:665) ) )
lf.vel_vector(1)=  float( string(lf.facility_data_rec(666:688) ) )
lf.vel_vector(2)=  float( string(lf.facility_data_rec(689:711) ) )
lf.nadir_radius=   float( string(lf.facility_data_rec(832:848) ) )
lf.image_radius=   float( string(lf.facility_data_rec(849:865) ) )
lf.looks_azimuth=  float( string(lf.facility_data_rec(896:912) ) )
lf.looks_range=    float( string(lf.facility_data_rec(913:929) ) )
lf.sl_rng_1st_pix= float( string(lf.facility_data_rec(1085:1101) ) )
lf.azi_res=        float( string(lf.facility_data_rec(1238:1254) ) )
lf.rng_res=        float( string(lf.facility_data_rec(1255:1271) ) )
lf.azimuth_pixel=  float( string(lf.facility_data_rec(1272:1288) ) )
lf.range_pixel=    float( string(lf.facility_data_rec(1289:1306) ) ) 

;
; Hard wired nominal s/c altitude and slant range if values are zero
; This is to compensate for missing data in new Gatineau data.
;
if lf.s_c_altitude eq 0 then begin
  lf.s_c_altitude=801.9966
  if lf.number_of_beams eq 1 then lf.s_c_altitude=801.4624
  if printflag then print, 's_c_altitude forced to:',  lf.s_c_altitude
endif
if lf.sl_rng_1st_pix eq 0 then begin
  lf.sl_rng_1st_pix=849.0674
  if lf.number_of_beams eq 1 then lf.lf.sl_rng_1st_pix=875.4624
  if printflag then print, 'lf.sl_rng_1st_pix forced to:',  lf.lf.sl_rng_1st_pix
endif


if printflag then  begin
  print, ' '
  print, 'Output of asf_read_facility.pro'
  print, 'datatake_id    ',lf.datatake_id
  print, 'image_id       ',lf.image_id
  print, 'data_year      ',lf.data_year
  print, 'center_gmt     ',lf.center_gmt
  print, 'center_lat     ',lf.center_lat
  print, 'center_lon     ',lf.center_lon
  print, 'near_start_lat ',lf.near_start_lat
  print, 'near_start_lon ',lf.near_start_lon
  print, 'near_end_lat   ',lf.near_end_lat
  print, 'near_end_lon   ',lf.near_end_lon
  print, 'far_start_lat  ',lf.far_start_lat
  print, 'far_start_lon  ',lf.far_start_lon
  print, 'far_end_lat    ',lf.far_end_lat
  print, 'far_end_lon    ',lf.far_end_lon
  print, 's_c_altitude   ',lf.s_c_altitude
  print, 'looks_azimuth  ',lf.looks_azimuth
  print, 'looks_range    ',lf.looks_range
  print, 'sl_rng_1st_pix ',lf.sl_rng_1st_pix
  print, 'azi_res        ',lf.azi_res
  print, 'rng_res        ',lf.rng_res
  print, 'azimuth_pixel  ',lf.azimuth_pixel
  print, 'range_pixel    ',lf.range_pixel
  print, 'nadir_radius   ',lf.nadir_radius
  print, 'image_radius   ',lf.image_radius
  print, 'position vector',lf.pos_vector
  print, 'velocity vector',lf.vel_vector
  print, ' '
endif

end
