;-----------------------------------------------------
;+
; NAME: ASF_READ_LEADER

; PURPOSE:
;        The purpose of this program is read the records from the
;        leader file
; CALLING SEQUENCE:
;
;  asf_read_leader, filename, good_leader, lf, /quiet, /help
;                  
; OUTPUT:
;   good_leader: 1 if data record is good
;   	        -1: problem with data set summary record
;               -2: problem with calibration record
;               -3: problem with max facility data record
;               -4: other problem
;
;   	lf: data structure containing the following:
;   	    1. File Descriptor Record (FDR) - 720 Bytes 
;   	    2. Map Projection Data Record - 1620 Bytes 
;             Note: This data record appears only in RADARSAT ScanSAR Data 
;   	    3. Platform Position Record - 1024 Bytes 
;   	    4. Attitude Data Record - 1024 Bytes  
;   	    5. Radiometric Data Record - 4232 Bytes 
;   	    6. Data Quality Summary Record - 1620 Bytes 
;           	(Calibrated/Uncalibrated flag at field 79) 
;   	    7. Signal Data Histogram Record- 4628 Bytes 
;   	    8. Processed Data Histogram Record - 4628 Bytes 
;   	    9. Range Spectra Record - 5120 Bytes 
;   	    10. Facility Data Record - 1717 Bytes
;           11. defaults file - string
;           12. corner_ll: fltarr(8)
;           13. corner_ne: fltarr(8)
;           14. defaults: string
;           15. asc_des: string
;           16. plat_lat: float( string(data_set_summary_rec( 452: 459)))
;           17. plat_long: float( string(data_set_summary_rec(460: 467)))
;           18. number_of_beams: fix(string(data_set_summary_rec(1766:1767)))
;           19. beam_1_id: string(data_set_summary_rec(1768:1771))
 
; MODIFICATION HISTORY:
;   1998 Jul 21 Frank Monaldo  
;   2000 Oct 27 Nathaniel Winstead : created a data structure
;               called lf (leader file) to pass arguments around
;               more efficiently.
;   2004 Mar 10 Frank Monaldo. Add place holders for ESA and GAT processing information
;   2005 Dec 01 Frank Monaldo. Add radar_band and radar_frequency to lf structure
;   2009 Dec 21 Frank Monaldo. Add nx0,ny0,nx1, ny1, ave_pixel,
;                              ave_line, name_dir, deg, working_direcory to lf structure
;   2010 Mar 01 Frank Monaldo. Read calibration coefficients
;   2010 Mar 19 Frank Monaldo. Fill in ground range if no map projection value
;   2010 Jun 14 Frank Monaldo. Print help if no input.
;   2011 Feb 05 Frank Monaldo. Add error catching if file too short
;   2011 Mar 29 Frank Monaldo. Add nrcs_slope and nrcs_bias to lf structure
;   2011 May 23 Frank Monaldo. Add jsbegin and jsend to structure
;   2011 Jun 22 Frank Monaldo. Add U for unknown ascending/descending
; Copyright (C) 1998-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  line
;-

pro asf_read_leader, filename, good_leader, lf,$
    quiet=quiet, help=help


if keyword_set(help)  or (n_params(0) lt 1) then begin
  print,' NAME: ASF_READ_LEADER
  print,'
  print,' PURPOSE:
  print,'        The purpose of this program is read the records from the
  print,'        leader file
  print,' CALLING SEQUENCE:
  print,'
  print,'   asf_read_leader, filename, good_leader, lf, /quiet, /help
  print,'                  
  print,'
  print,'   1. File Descriptor Record (FDR) - 720 Bytes 
  print,'   2. Data Set Summary Record - 4096 Bytes 
  print,'   3. Map Projection Data Record - 1620 Bytes 
  print,'      Note: This data record appears only in RADARSAT ScanSAR Data 
  print,'   4. Platform Position Record - 1024 Bytes 
  print,'   5. Attitude Data Record - 1024 Bytes  
  print,'   6. Radiometric Data Record - 4232 Bytes 
  print,'   7. Data Quality Summary Record - 1620 Bytes 
  print,'      (Calibrated/Uncalibrated flag at field 79) 
  print,'   8. Signal Data Histogram Record- 4628 Bytes 
  print,'   9. Processed Data Histogram Record - 4628 Bytes 
  print,'  10. Range Spectra Record - 5120 Bytes 
  print,'  11. Facility Data Record - 1717 Bytes 
  print,'  12. corner_ll: fltarr(8)
  print,'  13. corner_ne: fltarr(8)
  print,'  14. defaults: string
  print,'  15. asc_des: string
  print,'  16. plat_lat: float( string(data_set_summary_rec( 452: 459)))
  print,'  17. plat_long: float( string(data_set_summary_rec(460: 467)))
  print,'  18. number_of_beams: fix(string(data_set_summary_rec(1766:1767)))
  print,'  19. beam_1_id: string(data_set_summary_rec(1768:1771))) 
  return
endif
wait, 5 ; Wait 5 seconds to make sure the leader sile is completely
         ; copied into the appropriate directory.
good_leader=1

catch, error_status  
;This statement begins the error handler:  

if error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      good_leader= -4
      return
   endif
   ; handle the error by extending a:  
   catch, /cancel  
endif  


printflag=1
gatflag=0
if keyword_set(quiet)   then printflag=0 
if keyword_set(gat)     then gatflag=1
if printflag ne 0 then begin
  print, 'Reading leader records from: '+ filename
endif
;
; Determine System Arch
;

arch=!version.arch
close, 1
file_descriptor_rec= bytarr(720)
openr, 1, filename
readu, 1, file_descriptor_rec
close, 1

;
; Checking record lengths before proceeding.
;
; Data Set Summary
;
n_data_set=long( string( file_descriptor_rec[180:185] ) ) 
l_data_set=long( string( file_descriptor_rec[186:191] ) ) 
ndata_set_summary= n_data_set * l_data_set
;
;  Map Projection
;
n_map_proj=long( string( file_descriptor_rec[192:197] ) ) 
l_map_proj=long( string( file_descriptor_rec[198:203] ) ) 
nmap_projection_data= n_map_proj * l_map_proj
;
; Platform Position
;
n_plat_pos=long( string( file_descriptor_rec[204:209] ) ) 
l_plat_pos=long( string( file_descriptor_rec[210:215] ) )  
nplatform_position= n_plat_pos * l_plat_pos
;
; Attitude Data
;
n_att_data=long( string( file_descriptor_rec[216:221] ) ) 
l_att_data=long( string( file_descriptor_rec[222:227] ) ) 
nattitude_data= n_att_data * l_att_data
;
; Radiometric Data
;
n_radi_data=long( string( file_descriptor_rec[228:233] ) ) 
l_radi_data=long( string( file_descriptor_rec[234:239] ) ) 
nradiometric_data= n_radi_data * l_radi_data
;
; Data Quality Summary
;
n_qual_sum=long( string( file_descriptor_rec[253:257] ) ) 
l_qual_sum=long( string( file_descriptor_rec[258:263] ) ) 
ndata_quality_summary= n_qual_sum * l_qual_sum
;
; Signal Data Histogram
;
n_data_hist=long( string( file_descriptor_rec[264:269] ) ) 
l_data_hist=long( string( file_descriptor_rec[270:275] ) ) 
nsignal_data_histogram= n_data_hist * l_data_hist
;
; Processed Data Histogram
;
n_proc_hist=0L
l_proc_hist=0L
if n_data_hist eq 2 then begin
  n_data_hist=1L
  n_proc_hist=1L
  l_proc_hist=l_data_hist
endif
nsignal_data_histogram=    n_data_hist * l_data_hist
nprocessed_data_histogram= n_proc_hist * l_proc_hist
;
; Range Spectra
;
n_rang_spec=long( string( file_descriptor_rec[276:281] ) ) 
l_rang_spec=long( string( file_descriptor_rec[282:287] ) ) 
nrange_spectra= n_rang_spec * l_rang_spec
;
;
; Facility Data Record
;
n_fac_data=long( string( file_descriptor_rec(420:425) ) ) 
l_fac_data=long( string( file_descriptor_rec(426:431) ) ) 
nfacility_data= n_fac_data * l_fac_data
;
if printflag ne 0 then  begin
  print, 'Record                  No of Records  Length   Total bytes'
  print, '-------------------------------------------------------------'
  print, 'data set summary....', n_data_set,  l_data_set,  ndata_set_summary
  print, 'map projection data ', n_map_proj,  l_map_proj,  nmap_projection_data
  print, 'platform position...', n_plat_pos,  l_plat_pos,  nplatform_position 
  print, 'attitude data.......', n_att_data,  l_att_data,  nattitude_data
  print, 'radiometric data....', n_radi_data, l_radi_data, nradiometric_data
  print, 'data quality summary', n_qual_sum,  l_qual_sum,  ndata_quality_summary
  print, 'signal data hist....', n_data_hist, l_data_hist, nsignal_data_histogram
  print, 'proc. data hist.....', n_proc_hist, l_proc_hist, nprocessed_data_histogram
  print, 'range spectra.......', n_rang_spec, l_rang_spec, nrange_spectra
  print, 'facility data.......', n_fac_data,  l_fac_data,  nfacility_data
endif
lf={file_descriptor_rec:          bytarr(720),                         $
    data_set_summary_rec:         bytarr(ndata_set_summary>1),         $
    map_projection_data_rec:      bytarr(nmap_projection_data>1),      $
    platform_position_rec:        bytarr(nplatform_position>1),        $
    attitude_data_rec:            bytarr(nattitude_data>1),            $
    radiometric_data_rec:         bytarr(nradiometric_data>1),         $
    data_quality_summary_rec:     bytarr(ndata_quality_summary>1),     $
    signal_data_histogram_rec:    bytarr(nsignal_data_histogram>1),    $
    processed_data_histogram_rec: bytarr(nprocessed_data_histogram>1), $
    range_spectra_rec:            bytarr(nrange_spectra>1),            $
    facility_data_rec:            bytarr(nfacility_data>1),            $
    defaultsfile:                 '',                                  $
    corner_ll:                    fltarr(8),                           $
    corner_ne:                    fltarr(8),                           $
    defaults:                     'defaults.txt',                      $
    asc_des:                      'U',                                 $
    plat_lat:                     0.,                                  $
    plat_long:                    0.,                                  $
    number_of_beams:              0,                                   $
    beam_1_id:                    '',                                  $
    cal_status:                   '',                                  $
    cal_comment:                  '',                                  $
    n_pixel:                      0L,                                  $
    n_line:                       0L,                                  $
    nx0:                          0L,                                  $
    ny0:                          0L,                                  $
    nx1:                          0L,                                  $
    ny1:                          0L,                                  $   
    name_dir:                     '',                                  $
    ave_pixel:                    1L,                                  $
    ave_line:                     1L,                                  $
    deg:                          2,                                   $
    working_directory:            '',                                  $
    isc_dist:                     0.,                                  $
    geo_alt:                      0.,                                  $
    semi_major:                   0.,                                  $
    semi_minor:                   0.,                                  $
    projection:                   '',                                  $
    pixel_spacing:                0.,                                  $
    line_spacing:                 0.,                                  $
    pos_vector:                   fltarr(3),                           $
    vel_vector:                   fltarr(3),                           $
    datatake_id:                  '',                                  $
    image_id:                     '',                                  $ 
    data_year:                    0.,                                  $
    center_gmt:                   '',                                  $
    center_lat:                   0.,                                  $
    center_lon:                   0.,                                  $
    near_start_lat:               0.,                                  $
    near_start_lon:               0.,                                  $
    near_end_lat:                 0.,                                  $
    near_end_lon:                 0.,                                  $
    far_start_lat:                0.,                                  $
    far_start_lon:                0.,                                  $ 
    far_end_lat:                  0.,                                  $
    far_end_lon:                  0.,                                  $
    total_pixels:                 0.,                                  $
    total_lines:                  0.,                                  $
    s_c_altitude:                 0.,                                  $
    nadir_radius:                 0.,                                  $
    image_radius:                 0.,                                  $
    looks_azimuth:                0.,                                  $
    looks_range:                  0.,                                  $
    sl_rng_1st_pix:               0.,                                  $
    azi_res:                      0.,                                  $
    rng_res:                      0.,                                  $
    azimuth_pixel:                0.,                                  $
    range_pixel:                  0.,                                  $
    lngswitchflag:                0,                                   $
    scansar:                      1,                                   $
    dn:                           0.,                                  $
    hour:                         0.,                                  $
    minute:                       0.,                                  $
    second:                       0.,                                  $
    date:                         '',                                  $
    month:                        '',                                  $
    day:                          '',                                  $
    product_id:                   '',                                  $
    look:                         '',                                  $
    ellip_major:                  0.0,                                 $
    ellip_minor:                  0.0,                                 $
    pix_spacing:                  0.0,                                 $
    eph_orb_data:                 0.0,                                 $
    srgr_coef:                    fltarr(6),                           $
    n_samp:                       0L,                                  $
    samp_inc:                     0L,                                  $
    proc_fac:                    'ASF',                                $
    polarization:                'HH',                                 $  
    range_spacing:                0.0,                                 $ ; Fill ESA leader
    azimuth_spacing:              0.0,                                 $
    product:                      '',                                  $   
    product_stage:                '',                                  $
    software_ver:                 '',                                  $
    ref_doc:                      '',                                  $
    acquistion_station:           '',                                  $
    proc_cent:                    '',                                  $  
    proc_time:                    '',                                  $
    sensing_start:                '',                                  $
    sensing_stop:                 '',                                  $
    radar_frequency:              5.3E9,                               $   ; HZ
    radar_band:                   'C',                                 $
    sar_sensor:                   'RADARSAT-1',                        $
    leaderfile:                   filename,                            $
    nrcs_slope:                   1.00,                                $   ; Scaling to correct NRCS
    nrcs_bias:                    0.00,                                $
    jsbegin:                      -3155673600D,                        $   ; Default setting for start data
    jsend:                        -3155673600D                         }   ; Default setting for end  data


lf.file_descriptor_rec=file_descriptor_rec
;
; Redeclare arrays
;
lf.defaults='defaults.txt'
data_set_summary_rec=              bytarr(ndata_set_summary)
if nmap_projection_data gt 0 then begin
  lf.map_projection_data_rec=      bytarr(nmap_projection_data)
endif
if nplatform_position gt 0 then begin
  lf.platform_position_rec=        bytarr(nplatform_position)
endif
if nattitude_data gt 0 then begin
  lf.attitude_data_rec=            bytarr(nattitude_data)
endif
if nradiometric_data gt 0 then begin
  lf.radiometric_data_rec=         bytarr(nradiometric_data)
endif
if ndata_quality_summary gt 0 then begin
  lf.data_quality_summary_rec=     bytarr(ndata_quality_summary)
endif
if nsignal_data_histogram gt 0 then begin
  lf.signal_data_histogram_rec=  bytarr(nsignal_data_histogram)
endif
if nprocessed_data_histogram gt 0 then begin 
  lf.processed_data_histogram_rec= bytarr(nprocessed_data_histogram)
endif 
if nrange_spectra gt 0 then begin
  lf.range_spectra_rec=            bytarr(nrange_spectra)
endif 
if nfacility_data gt 0 then begin
  lf.facility_data_rec=            bytarr(nfacility_data)
endif
close, 1
openr, 1, filename
file_descriptor_rec= bytarr(720)
readu, 1, file_descriptor_rec
;
data_set_summary_rec=         bytarr(ndata_set_summary>1)
map_projection_data_rec=      bytarr(nmap_projection_data>1)
platform_position_rec=        bytarr(nplatform_position>1)
attitude_data_rec=            bytarr(nattitude_data>1)
radiometric_data_rec=         bytarr(nradiometric_data>1)
data_quality_summary_rec=     bytarr(ndata_quality_summary>1)
signal_data_histogram_rec=    bytarr(nsignal_data_histogram>1)
processed_data_histogram_rec= bytarr(nprocessed_data_histogram>1)
range_spectra_rec=            bytarr(nrange_spectra>1)
facility_data_rec=            bytarr(nfacility_data>1)
;
if ndata_set_summary         gt 0 then readu, 1, data_set_summary_rec
lf.data_set_summary_rec=data_set_summary_rec    
if nmap_projection_data      gt 0 then readu, 1, map_projection_data_rec
lf.map_projection_data_rec=map_projection_data_rec
if nplatform_position        gt 0 then readu, 1, platform_position_rec
lf.platform_position_rec=platform_position_rec
if nattitude_data            gt 0 then readu, 1, attitude_data_rec
lf.attitude_data_rec=attitude_data_rec
if nradiometric_data         gt 0 then readu, 1, radiometric_data_rec
lf.radiometric_data_rec=radiometric_data_rec
if ndata_quality_summary     gt 0 then readu, 1, data_quality_summary_rec
lf.data_quality_summary_rec=data_quality_summary_rec
if nsignal_data_histogram    gt 0 then readu, 1, signal_data_histogram_rec
lf.signal_data_histogram_rec=signal_data_histogram_rec
if nprocessed_data_histogram gt 0 then readu, 1, processed_data_histogram_rec
lf.processed_data_histogram_rec=processed_data_histogram_rec
if nrange_spectra            gt 0 then readu, 1, range_spectra_rec
lf.range_spectra_rec=range_spectra_rec
if nfacility_data            gt 0 then readu, 1, facility_data_rec
lf.facility_data_rec=facility_data_rec

close, 1
asf_read_file_des, lf,  quiet=quiet

if max( lf.data_set_summary_rec ) gt 0 then begin               
  asf_read_data_sum, lf, quiet=quiet
endif  else begin
  if printflag eq 1 then print, 'NO DATA SET SUMMARY RECORD'
  good_leader=-1
  return
endelse	
;
; Read DATA QUALITY SUMMARY record.
;     
if max( lf.data_quality_summary_rec ) gt 0 then begin               
   asf_read_data_qual, lf, quiet=quiet
endif  else begin
   if printflag eq 1 then print, 'NO DATA QUALITY SUMMARY'
   good_leader=-2
   return
endelse
;
; Read FACILITY DATA record.
;     
if max( lf.facility_data_rec ) gt 0 then begin               
  asf_read_facility, lf, quiet=quiet
  lf.n_pixel= lf.total_pixels
  lf.n_line=  lf.total_lines 		
  lf.nx0=     lf.n_pixel
  lf.ny0=     lf.n_line
endif  else begin
  if printflag eq 1 then print, 'NO FACILITY DATA RECORD'
  good_leader=-3
  return
endelse	
;
; Read MAP PROJECTION record
;

lf.scansar=1
if lf.number_of_beams eq 1 then lf.scansar=0
if max(  lf.map_projection_data_rec ) gt 0 then begin
   asf_read_map_proj, lf, quiet=quiet  
   if strlen(lf.projection) eq 0 then lf.projection='GROUND RANGE'
endif else begin
   ;    
   ; If there is no map projection data record, then I have a standard 
   ; mode image 
   ;
   lf.line_spacing=  lf.azimuth_pixel
   lf.pixel_spacing= lf.range_pixel
   if printflag then print, 'NO MAP PROJECTION DATA RECORD'
   lf.projection='GROUND RANGE'
endelse	

a1=    0.
a2=    0.
a3=    0.
noise= fltarr(256)
if max(  lf.radiometric_data_rec ) gt 0 then begin
   asf_nominal_cal,  radiometric_data_rec, a1, a2, a3,  noise
endif 
temp_lf= create_struct(lf, 'a1', a1, 'a2', a2, 'a3', a3, 'noise', noise )
lf=      0
lf=      temp_lf



; Skip in data in Lambert projection       
if strcompress(lf.projection,/remove_all) eq 'LAMBERT' then $
good_leader=-4            
;
; 
; Check as see if image crosses 180 meridian.  If so change the
; longitude range to 0 to 360.
;
lf.lngswitchflag = 0
asf_check4_180, lf, quiet=quiet
if lf.lngswitchflag eq 1 then  begin
   if printflag then print,'Changing longitude range...'
   if lf.corner_ll[1]   lt 0 then lf.corner_ll[1]=   360+lf.corner_ll[1]
   if lf.corner_ll[3]   lt 0 then lf.corner_ll[3]=   360+lf.corner_ll[3]
   if lf.corner_ll[5]   lt 0 then lf.corner_ll[5]=   360+lf.corner_ll[5]
   if lf.corner_ll[7]   lt 0 then lf.corner_ll[7]=   360+lf.corner_ll[7]
   if lf.near_start_lon lt 0 then lf.near_start_lon= 360+lf.near_start_lon
   if lf.near_end_lon   lt 0 then lf.near_end_lon=   360+lf.near_end_lon
   if lf.far_start_lon  lt 0 then lf.far_start_lon=  360+lf.far_start_lon
   if lf.far_end_lon    lt 0 then lf.far_end_lon=    360+lf.far_end_lon
   if lf.plat_long      lt 0 then lf.plat_long=      360+lf.plat_long
   if lf.center_lon     lt 0 then lf.center_lon=     360+lf.center_lon
endif  
;
;=====================================================================
; --------  Completed the reading of leader information.--------------
;=====================================================================
;
lf.center_gmt=     repchr(lf.center_gmt,'T',' ')  ; fix bug from ASF
lf.dn=      float(strmid(lf.center_gmt,0,3))
lf.hour=    float(strmid(lf.center_gmt,4,2))
lf.minute=  float(strmid(lf.center_gmt,7,2))
lf.second=  float(strmid(lf.center_gmt,10,6))
lf.date=    ydn2date(lf.data_year, lf.dn)
lf.month=   strlowcase( getwrd( lf.date,1))
lf.day=     getwrd( lf.date,2)

end
