;-----------------------------------------------------
;+
; NAME: ASF_READ_MAP_PROJ
;
; PURPOSE:
;        The purpose of this program is read data from the
;        map projection record
; CALLING SEQUENCE:
;
; asf_read_map_proj, lf, /help
;
; INPUT:
;    lf.map_projection_data_rec= byte array from the map_projection_rec
;
; OUTPUT: add the following variables to lf structure
;    lf.n_pixel=      Number of pixels per line 
;    lf.n_line=       Number of lines per processed band 
;    lf.pixel_spacing=Nominal inter-pixeldistance, meters 
;    lf.line_spacing= Nominal inter-line distance, meters
;    lf.isc_dist=     Distance of platform at input scene centre from the 
;                  geocentre, meters  
;    lf.geo_alt=      Geodetic platform altitude, meters
;    lf.semi_major=   Ellipsoid semi-major axis,meters (kilo)  
;    lf.semi_minor=   Ellipsoid semi-minor axis,meters (kilo)
;    lf.projection=   Map projection alphanumeric description 
;    lf.corner_ne=    this is an 8-element array with the positions
;                  of the corners in meters
;    lf.corner_ll=    this is an 8-element array with the latitude
;                  and longitude corners of the image
; MODIFICATION HISTORY:
;     1998 Jul 21 Frank Monaldo  
;     2000 Oct 30 Nathaniel Winstead  rewrote to handle lf structure
;     2009 Dec 31 Frank Monaldo. Converted to [ ] for variables.
;     2010 Jun 14 Frank Monaldo. Print help with no input.
;     2010 Nov 06 Frank Monaldo. Fill in projection structure element
; Copyright (C) 1998-2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_read_map_proj, lf, help=help, quiet=quiet
		   
if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ''
  print, ' NAME: ASF_READ_MAP_PROJ
  print, '
  print, ' PURPOSE:
  print, '        The purpose of this program is read data from the
  print, '        map projection record
  print, ' CALLING SEQUENCE:
  print, '
  print, ' asf_read_map_proj, lf, quiet=quiet, help=help
  print, '
  print, ' INPUT:
  print, ' lf.map_projection_data_rec= byte array from the map_projection_rec
  print, '
  print, ' OUTPUT:
  print, '    lf.n_pixel=      Number of pixels per line 
  print, '    lf.n_line=       Number of lines per processed band 
  print, '    lf.pixel_spacing=Nominal inter-pixeldistance, meters 
  print, '    lf.line_spacing= Nominal inter-line distance, meters
  print, '    lf.isc_dist=     Distance of platform at input scene centre from the 
  print, '                  geocentre, meters  
  print, '    lf.geo_alt=      Geodetic platform altitude, meters
  print, '    lf.semi_major=   Ellipsoid semi-major axis,meters  (kilo) 
  print, '    lf.semi_minor=   Ellipsoid semi-minor axis,meters  (kilo)
  print, '    lf.projection=   Map projection alphanumeric description     
  print, '    lf.corner_ne=    this is an 8-element array with the 
  print, '                  position of the corners in meters
  print, '    lf.corner_ll=    this is an 8-element array with the latitude
  print, '                  and longitude corners of the image
  print, '
  return
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
;
; Determine System Arch
;
arch=!version.arch

;
; Reading data from record
; 

for i=0, 7 do begin
  first_byte= 16 * i + 944
  last_byte=  first_byte + 15
  lf.corner_ne(i)= float(string(lf.map_projection_data_rec[first_byte: last_byte]) )
endfor
for i=0, 7 do begin
  first_byte= 16 * i + 1072
  last_byte=  first_byte + 15
  lf.corner_ll(i)= float(string(lf.map_projection_data_rec[first_byte: last_byte]) )
endfor
lf.projection=  string(lf.map_projection_data_rec[412:420])
lf.projection=  strcompress( lf.projection, /remove_all )
if printflag  then  begin
  print, ''
  print, 'Output of asf_read_map_proj.pro'
  print, 'n_pixel=',       lf.n_pixel
  print, 'n_line=',        lf.n_line
  print, 'pixel_spacing=', lf.pixel_spacing, ' m'
  print, 'line_spacing=',  lf.line_spacing,  ' m'
  print, 'isc_dist=',      lf.isc_dist/1000.,' km'
  print, 'geo_alt=',       lf.geo_alt/1000., ' km'
  print, 'semi_major=',    lf.semi_major,    ' km'
  print, 'semi_minor=',    lf.semi_minor,    ' km'
  print, 'projection=',    lf.projection
  print,  'corner_ne'
  print, lf.corner_ne
  print, 'corner_ll'
  print, lf.corner_ll
endif
end
