;----------------------------------------------------------
;+
; NAME: ASF_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out ASF data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;   asf_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
; MODIFICATION
;   2009 Dec 21 Frank Monaldo. Re-write for ANSWRS 2.0
;   2010 Jan 08 Frank Monaldo. Change r_look_coef rlook_coef 
;   2010 Jan 13 Frank Monaldo. Re-write to use ans_write_ancil_sio
;   2010 Jan 22 Frank Monaldo. Re-compute illumination and direction and relative heading
;                              in accordance with Wackermans description
;   2010 Jun 04 Frank Monaldo. Corrected Radarsat-1 lf.sar_senor entry.
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2010 Dec 29 Frank Monaldo. pass on quiet command
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro asf_write_ancil_sio, filename, lf, scaling, $
                           quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0

quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ASF_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out ASF data into ancillary data ascii file. These
  print, '   data fit the SIO format
  print, ' CALLING SEQUENCE:'
  print, '   asf_write_ancil_sio, filename, lf,  scaling'
  print, ' INPUT:'
  print, '   filename=     output file with ancillary data
  print, '   lf=           leader structure'
  print, '   scaling=      scaling for data in binary file of NRCS, default=1
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  return
endif


producttype=           lf.beam_1_ID
software_version=      lf.software_ver
orbit_number=          lf.datatake_id
beam_mode=             lf.beam_1_ID
valuetype=             'intensity'
frequency_band=        'C'
sensor_velocity=       sqrt( total(lf.vel_vector^2) )
integration_time=      '-99999.99'
nom_az_res=            string(lf.azi_res, format='(F6.2)')      
nom_rg_res=            string(lf.rng_res, format='(F6.2)')     
rcvr_gain=             '-99999.99'       
proc_gain=             '-99999.99'
range_swath=           lf.range_pixel   * lf.ave_pixel * lf.nx1
azimuth_swath=         lf.azimuth_pixel * lf.ave_line  * lf.ny1
center_incid=          ans_poly_cal(lf.nx1/2,lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
slant_range_center=    -99999.99
slant_range_near=      -99999.99
ellipsoidname=         ' '
semimajoraxis=         -99999.99
semiminoraxis=         -99999.99
datumshiftparameters=  [0.0, 0.0, 0.0]
ellipsoidscale=        '1'
deg_incid=             lf.deg
deg_rlook=             lf.deg
azimuth_looks=         lf.looks_azimuth
range_looks=           lf.looks_range
range_type=            lf.projection
range_flag=            1
element_spacing=       lf.range_pixel   * lf.ave_pixel
lf.line_spacing=       lf.azimuth_pixel * lf.ave_line
range_swath=           lf.range_pixel   * lf.ave_pixel * lf.nx1
azimuth_swath=         lf.azimuth_pixel * lf.ave_line  * lf.ny1 
slant_range_center=    '-99999.99'
slant_range_near=      lf.SL_RNG_1ST_PIX * 1000.   
semimajoraxis=         float(lf.semi_major) *1000
semiminoraxis=         float(lf.semi_minor) *1000
platform_code=         'R1'
scaling=               scaling
pol=                   lf.polarization
if lf.sar_sensor eq 'ERS-2' then platform_code='E2'
if strupcase( strmid(lf.sar_sensor,0,1) ) eq 'R' then lf.sar_sensor='RADARSAT-1'


;
; compute heading of center pixel
;
lo1= ans_poly_cal(lf.nx1/2,        lf.ny1/2,   lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la1= ans_poly_cal(lf.nx1/2,        lf.ny1/2,   lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
lo2= ans_poly_cal(lf.nx1/2,        lf.ny1/2+1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la2= ans_poly_cal(lf.nx1/2,        lf.ny1/2+1, lf.lat_coef, lf.lon_xexp, lf.lat_yexp)
ll2rb, lo1, la1, lo2, la2, r, b

if b gt 180 then b=b-360

heading=      b
flip=         0
illumination_dir=    0
near_range_edge=    'L'
early_azimuth_edge= 'B'
heading_rel=        90 
lf.s_c_altitude = float(lf.s_c_altitude) *1000.



if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf,                                             $ 
                          'platform_code',         platform_code,         $ 
                          'producttype',           producttype,           $ 
                          'software_version',      software_version,      $ 
                          'orbit_number',          orbit_number,          $ 
                          'beam_mode',             beam_mode,             $ 
                          'valuetype',             valuetype,             $ 
                          'frequency_band',        frequency_band,        $ 
                          'heading',               heading,               $
                          'heading_rel',           heading_rel,           $
                          'sensor_velocity',       sensor_velocity,       $ 
                          'integration_time',      integration_time,      $ 
                          'nom_az_res',            nom_az_res,            $ 
                          'nom_rg_res',            nom_rg_res,            $ 
                          'rcvr_gain',             rcvr_gain,             $ 
                          'proc_gain',             proc_gain,             $ 
                          'range_swath',           range_swath,           $ 
                          'azimuth_swath',         azimuth_swath,         $ 
                          'slant_range_center',    slant_range_center,    $ 
                          'slant_range_near',      slant_range_near,      $ 
                          'ellipsoidname',         ellipsoidname,         $ 
                          'semimajoraxis',         semimajoraxis,         $ 
                          'semiminoraxis',         semiminoraxis,         $ 
                          'datumshiftparameters',  datumshiftparameters,  $ 
                          'ellipsoidscale',        ellipsoidscale,        $ 
                          'deg_incid',             deg_incid,             $ 
                          'deg_rlook',             deg_rlook,             $ 
                          'azimuth_looks',         azimuth_looks,         $ 
                          'range_looks',           range_looks,           $ 
                          'range_type',            range_type,            $ 
                          'range_flag',            range_flag,            $ 
                          'element_spacing',       element_spacing,       $ 
                          'scaling',               scaling,               $ 
                          'pol',                   pol,                   $
                          'flip',                  flip,                  $
                          'illumination_dir',      illumination_dir,      $
                          'near_range_edge',       near_range_edge,       $
                          'early_azimuth_edge',    early_azimuth_edge,    $
                          'center_incid',          center_incid           )         
                          
  lf=      0                                    
  lf=      temp_lf
endif else begin  
  lf.platform_code=         platform_code         
  lf.producttype=           producttype           
  lf.software_version=      software_version      
  lf.orbit_number=          orbit_number          
  lf.beam_mode=             beam_mode             
  lf.heading=               heading               
  lf.heading_rel=           heading_rel           
  lf.valuetype=             valuetype             
  lf.frequency_band=        frequency_band        
  lf.sensor_velocity=       sensor_velocity       
  lf.integration_time=      integration_time      
  lf.nom_az_res=            nom_az_res            
  lf.nom_rg_res=            nom_rg_res            
  lf.rcvr_gain=             rcvr_gain             
  lf.proc_gain=             proc_gain             
  lf.range_swath=           range_swath           
  lf.azimuth_swath=         azimuth_swath         
  lf.slant_range_center=    slant_range_center    
  lf.slant_range_near=      slant_range_near      
  lf.ellipsoidname=         ellipsoidname         
  lf.semimajoraxis=         semimajoraxis         
  lf.semiminoraxis=         semiminoraxis         
  lf.datumshiftparameters=  datumshiftparameters  
  lf.ellipsoidscale=        ellipsoidscale        
  lf.deg_incid=             deg_incid             
  lf.deg_rlook=             deg_rlook             
  lf.azimuth_looks=         azimuth_looks         
  lf.range_looks=           range_looks           
  lf.range_type=            range_type            
  lf.range_flag=            range_flag            
  lf.element_spacing=       element_spacing            
  lf.center_incid=          center_incid          
  lf.scaling=               scaling               
  lf.pol=                   pol
  lf.flip=                  flip
  lf.illumination_dir=      illumination_dir 
  lf.near_range_edge=       near_range_edge  
  lf.early_azimuth_edge=    early_azimuth_edge
endelse


;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 
  
 end
