;-----------------------------------------------------
;+
; NAME: COSMOS_DN2SIGMA
; PURPOSE:
;   The purpose of this program is to convert the RADARSAT digital numbers
;   to radar cross section.  This version of the program was written to 
;   convert data as process by Gatineau. This program also averages down the
;   image by the amount  specified with data in the lf structure
; CALLING SEQUENCE:
;   cosmos_dn2sigma, image,  lf
; INPUT:
;     image=       Array containing the digital number image   
;     lf=          Leader information structure           
; OUTPUT:
;     image=       A floating point array of sigma.
; KEYWORDS=
;     /quiet=      Do not print intermediate results.
;     /help=       List help.
;
; MODIFICATION HISTORY:
;  2011 Jan 18 Frank Monaldo
;  2012 Jan 21 Frank Monaldo.  Apply the new
;                              cailbration information from
;                              Horstmann and ESA
;  2013 Dec 30 Frank Monaldo. Took out sqrt of calibration constant
; Copyright (C) 2011-2012, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro cosmos_dn2sigma, image, lf, help=help, quiet=quiet	   


if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print,' NAME: COSMOS_DN2SIGMA
  print,' PURPOSE:
  print,'   The purpose of this program is to convert the RADARSAT digital numbers
  print,'   to radar cross section.  This version of the program was written to 
  print,'   convert data as process by Gatineau. This program also averages down the
  print,'   image by the amount  specified with data in the lf structure
  print,' CALLING SEQUENCE:
  print,'   cosmos_dn2sigma, image,  lf
  print,' INPUT:
  print,'     image=       Array containing the digital number image   
  print,'     lf=          Leader information structure           
  print,' OUTPUT:
  print,'     image=       A floating point array of sigma.
  print,' KEYWORDS=
  print,'     /quiet=      Do not print intermediate results.
  print,'     /help=       List help.
  print,'
  return
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

if printflag then print, 'Converting DN to sigma for COSMO SkyMed Dat
sz= size(image)
nx= sz[1]
ny= sz[2]

;
;  Promote to float-point value.
;
; hlp, image
image=  float(image)

if printflag then begin
  ;
  ; Computing stats based on sub sample
  ;
  print, ''
  print, 'Cosmo-SkyMed Calibration'
  n=   n_elements(image)
  n2=  n / 1000
  ii=  indgen(1000) * n2
  print, 'Input image data.'
  print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
  print, 'SD   of image=                    ' + strcompress(string(stdev( image[ii] )))
  print, '-------------------'
endif

;
; Square digital number
;
image = image * image

if printflag then begin
  n=   n_elements(image)
  n2=  n / 1000
  ii=  indgen(1000) * n2
  print, 'After squaring image'
  print, 'image * image'
  print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
  print, 'SD   of image=                    ' + strcompress(string(stdev( double(image[ii]) )))
  print, '-------------------'
endif

if lf.range_spreading_loss_compensation_geometry ne 'NONE' then begin
  if printflag then print, 'Applying spreading loss correction.'
  correction=  float(lf.reference_slant_range)^(2.0*float(lf.reference_slant_range_exponent))
  image= image * correction
  if printflag then begin
    n=   n_elements(image)
    n2=  n / 1000
    ii=  indgen(1000) * n2
    print, 'After application of spreading loss.'
    print, 'image= image * reference_slant_range^(2.0*reference_slant_range_exponent)'
    print, 'Input image data.'
    print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
    print, 'SD   of image=                    ' + strcompress(string(stdev( double(image[ii]) )))
    print, 'lf.reference_slant_range=         ' + strcompress(string(lf.reference_slant_range))
    print, 'lf.reference_slant_range_exponent=' + strcompress(string(lf.reference_slant_range_exponent))
    print, 'Spreading loss correction=        ' + strcompress(string(correction))
    print, '-------------------'
  endif
endif

if lf.incidence_angle_compensation_geometry ne 'NONE' then begin
  if printflag then print, 'Applying incidence angle correction.'
  correction=   float( sin( !pi / 180.0 * float(lf.reference_incidence_angle) ) ) 
  image=  image * correction
  if printflag then begin
    n=   n_elements(image)
    n2=  n / 1000
    ii=  indgen(1000) * n2
    print, 'After application of angle compensation.'
    print, 'image= image * sin(reference_incidence_angle)'
    print, 'Input image data.'
    print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
    print, 'SD   of image=                    ' + strcompress(string(stdev( double(image[ii]) )))
    print, 'lf.reference_incidence_angle=     ' + strcompress(string(lf.reference_incidence_angle))
    print, 'Correction=                       ' + strcompress(string(correction))
    print, '-------------------'
  endif
endif

if printflag then print, 'Applying rescaling factor.'
correction=      1.0 / float(lf.rescaling_factor)^2
image=  image *  correction
if printflag then begin
  n=   n_elements(image)
  n2=  n / 1000
  ii=  indgen(1000) * n2
  print, 'After application of scaling factor.'
  print, 'image= image/rescaling_factor
  print, 'Input image data.'
  print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
  print, 'SD   of image=                    ' + strcompress(string(stdev( image[ii] )))
  print, 'lf.rescaling_factor=              ' + strcompress(string(lf.rescaling_factor))
  print, 'Correction=                       ' + strcompress(string(correction ))  
  print, '-------------------'
endif


if lf.calibration_constant_compensation_flag eq 0 then begin
  ; 
  ; Assuming the calibration constants are evenly distributed
  ; in range
  ;
  if printflag then print, 'Applying calibration factor.'
  n=        n_elements(lf.calibration_constant)
  ; n1=       nx / 4
  b0=       0
  for b=1, n do begin
    b1= b0 + (n-1)
    if b eq n then b1= nx-1
    image[b0:b1,*] = image[b0:b1,*] / float( lf.calibration_constant[b-1] )
    b0= b1 + 1
  endfor
  if printflag then begin
    n=   n_elements(image)
    n2=  n / 1000
    ii=  indgen(1000) * n2
    print, 'After application of calibration factor.'
    print, 'Input image data.'
    print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
    print, 'SD   of image=                    ' + strcompress(strcompress(string(stdev( image[ii] ))))
    print, 'lf.lf.calibration_constant=       ' + strcompress(string(lf.calibration_constant))
    print, '-------------------'
  endif
endif
if printflag then begin
  ;
  ; Computing stats based on sub sample
  ;
  n=   n_elements(image)
  n2=  n / 1000
  ii=  indgen(1000) * n2
  print, 'NRCS image data.'
  print, 'Mean of image=                    ' + strcompress(string(mean(  image[ii] )))
  print, 'SD   of image=                    ' + strcompress(string(stdev( image[ii] )))
endif
  
end
