; '-----------------------------------------------------
; +
; NAME: 
;    COSMOS_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a 
;   polynomial fit to go from i and j to 
;   incident ang.
; CALLING SEQUENCE:
;    cosmos_incid2pix_fit, lf   
; INPUTS:
;    lf         Structure with lat/lngs and ave_pixel, ave_line
;               and degree of fit
; OUTPUTS:
;   Added to structure     
;   lf.incid_coef=  Array of latitude coefficients 
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
; MODIFICATION HISTORY:
; 2011 Jan 19 Frank Monaldo.
; 2011 Jan 24 Frank Monaldo. Make sure that quiet is passed to sub routines.
; 2011 Jan 24 Frank Monaldo. Correct spellling of NEAR-FAR
; Copyright (C) 2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro cosmos_incid2pix_fit, lf, quiet=quiet, help=help

printflag=1

if keyword_set(quiet) then printflag=0


if keyword_set(help) or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '    COSMOS_INCID2PIX_FIT
  print, ' PURPOSE 
  print, '   The purpose of this program is compute a 
  print, '   polynomial fit to go from i and j to 
  print, '   incident ang.
  print, ' CALLING SEQUENCE:
  print, '    cosmos_incid2pix_fit, lf   
  print, ' INPUTS:
  print, '    lf         Structure with lat/lngs and ave_pixel, ave_line
  print, '               and degree of fit
  print, ' OUTPUTS:
  print, '   Added to structure     
  print, '   lf.incid_coef=  Array of latitude coefficients 
  print, ' KEYWORDS:
  print, '    /quiet=    Suppress printed messages. 
  print, '    /help=     Display help
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

if printflag then  print, 'Computing a fit to lon and lat for line of data...'




nx=           lf.nx
ny=           lf.ny
incid_near=   lf.near_incidence_angle
incid_far=    lf.far_incidence_angle
look_near=    lf.near_look_angle
look_far=     lf.far_look_angle
re=           lf.centre_earth_radius
rh=           lf.satellite_height
beta=         re + rh
c_near=       incid_near - look_near
c_far=        incid_far  - look_far
c_cent=       (c_far + c_near) / 2.0
c_cent_r=     c_cent * !pi / 180.0
if printflag then begin
  print, 'incid_near=   '+ string(incid_near   )
  print, 'incid_far=    '+ string(incid_far    )
  print, 'look_near=    '+ string(look_near    )
  print, 'look_far=     '+ string(look_far     )
  print, 're=           '+ string(re           )
  print, 'rh=           '+ string(rh           )
  print, 'beta=         '+ string(beta         )
  print, 'c_near=       '+ string(c_near       )
  print, 'c_far=        '+ string(c_far        )
  print, 'c_cent=       '+ string(c_cent       )
  print, 'c_cent_r=     '+ string(c_cent_r     )
endif
;
; Compute slant range
;
rs2=          re^2 + beta^2 - 2 * re * beta * cos(c_cent_r)
if printflag then print,  'rs2=          '+ string(rs2     )
rs=           sqrt( rs2 )
if printflag then print,  'rs=           '+ string(rs      )
b_r=          asin( beta / rs * sin(c_cent_r) )
if printflag then print,  'b_r=          '+ string(b_r     )
b=            b_r * 180.00 /!pi
if printflag then print,  'b =            '+ string(b      )
incid_cent=   b
if printflag then print,  'incid_cent=    '+ string(incid_cent      )
i_near=       double(0)
i_far=        double(nx-1)
j_early=      double(0)
j_late=       double(ny-1)
i_cent=       double( (nx-1) ) / 2.0
j_cent=       double( (ny-1) ) / 2.0
if lf.columns_order ne 'NEAR-FAR' then begin
  if printflag then print, 'Column order ' + lf.columns_order
  i_near=     double(nx-1)
  i_far=      double(0)
endif
if lf.lines_order ne 'EARLY-LATE' then begin
  if printflag then print, 'Line order ' + lf.lines_order
  j_early=     double(ny-1)
  j_late=      double(0)
endif


iid= [i_near,        i_cent,      i_far,     i_near,     i_cent,    i_far,      i_near,    i_cent,      i_far ]
jjd= [j_early,       j_early,   j_early,     j_cent,     j_cent,   j_cent,      j_late,    j_late,     j_late ]
inc= [incid_near, incid_cent, incid_far, incid_near, incid_cent, incid_far, incid_near, incid_cent, incid_far ]      
; 
; Compute the relationship between i,j and incident angle
;
; Scale indices appropriately
;
iid=  iid / lf.ave_pixel
jjd=  jjd / lf.ave_line     
;
; We wish to find equations of the form form
;  inc=  sum_m=0^deg sum_n=0^deg A(m,n) * i^m *j^n
;
;
; 
; Compute the relationship between i,j and incident angle
;
ans_compute_fit, iid, jjd,  inc,   lf.deg, incid_coef,   incid_xexp,   incid_yexp, quiet=quiet



ns= n_elements(iid)

if printflag then begin
   ;
   ; Check incidene conversions
   ;
   print, 'Incidence conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( inc[i] )
     incidp=   ans_poly_cal( iid[i], jjd[i], incid_coef, incid_xexp, incid_yexp )
     mu= mu+  (inc[i]-incidp)
     sd= sd + (inc[i]-incidp)^2
     output= output + string( incidp )
     print, output
   endfor
   print, 'Incidence', mu/ns, sqrt(sd/ns)
endif

;
; Add to structure
;

temp_lf= create_struct(lf, 'incid_coef', incid_coef, $
                           'incid_xexp', incid_xexp, $
                           'incid_yexp', incid_yexp  )

lf=               0
lf=               temp_lf


end

