; '-----------------------------------------------------
; +
; NAME: 
;    COSMOS_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a 
;   polynomial fit to go from lon/lat to pixel location and visa versa.
;   This program readings information from the lf structure
; CALLING SEQUENCE:
;    cosmos_lonlat2pix_fit, lf   
; INPUTS:
;    lf         Structure with lat/lngs and ave_pixel, ave_line
;               and degree of fir
; OUTPUTS:
;   Added to structure     
;   lf.i_coef:    Array of coefficients for horizontal pixel fit
;   lf.j_coef=    Array of coefficients for vertical pixel  fit
;   lf.lon_coef=  Array of longitude coefficients
;   lf.lat_coef=  Array of latitude coefficients 
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
; MODIFICATION HISTORY:
;   2010 Oct 04 Frank Monaldo
;   2011 Jan 10 Frank Monaldo. Add pass through for silent flag
;   2011 Jan 23 Frank Monaldo. Change case of variables
;   2011 Jan 24 Frank Monaldo. Make sure that quiet flag is passed to subs.
;                              Use ellipsoid rather than sphere
;   2011 Feb 21 Frank Monaldo. Added along track points in fit.
; Copyright (C) 2003-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro cosmos_lonlat2pix_fit, lf, quiet=quiet, help=help 

printflag=1

if keyword_set(quiet) then printflag=0


if keyword_set(help) or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '    COSMOS_LONLAT2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a 
  print, '   polynomial fit to go from lon/lat to pixel location and visa versa.
  print, '   This program readings information from the lf structure
  print, ' CALLING SEQUENCE:
  print, '    cosmos_lonlat2pix_fit, lf   
  print, ' INPUTS:
  print, '    lf         Structure with lat/lngs and ave_pixel, ave_line
  print, '               and degree of fir
  print, ' OUTPUTS:
  print, '   Added to structure     
  print, '   lf.i_coef:    Array of coefficients for horizontal pixel fit
  print, '   lf.j_coef=    Array of coefficients for vertical pixel  fit
  print, '   lf.lon_coef=  Array of longitude coefficients
  print, '   lf.lat_coef=  Array of latitude coefficients 
  print, ' KEYWORDS:
  print, '    /quiet=    Suppress printed messages. 
  print, '    /help=     Display help
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

if printflag then  print, 'Computing a fit to lon and lat for line of data...'



lat=[ lf.scene_centre_geodetic_coordinates[0],  $  
      lf.bottom_left_geodetic_coordinates[0],   $
      lf.bottom_right_geodetic_coordinates[0],  $
      lf.top_left_geodetic_coordinates[0],      $
      lf.top_right_geodetic_coordinates[0]      ]

lng=[ lf.scene_centre_geodetic_coordinates[1],  $  
      lf.bottom_left_geodetic_coordinates[1],   $
      lf.bottom_right_geodetic_coordinates[1],  $
      lf.top_left_geodetic_coordinates[1],      $
      lf.top_right_geodetic_coordinates[1]      ]
      
iid=  double([(lf.nx-1)/2, 0, lf.nx-1,       0, lf.nx-1])
jjd=  double([(lf.ny-1)/2, 0,       0, lf.ny-1, lf.ny-1])
      
      
;------------------------------------------------------------------------------
; Compute longitudes along first range line, assuming that the
; range line falls on a great circle route
;------------------------------------------------------------------------------
;
j= double(0)

di= double(lf.nx) / 5.0
for ii=1,4 do begin
  i=  double(ii * di)
  ell_ll2rb, lng[1], lat[1], lng[2], lat[2], r, b
  ell_rb2ll, lng[1], lat[1], i * lf.column_spacing, b, lo, la
  lat= [lat, la]  & lng= [lng, lo] & iid= [iid, i ] & jjd= [jjd, j ]
endfor
;
;------------------------------------------------------------------------------
; Compute longitudes along last range line, assuming that the
; range line falls on a great circle route
;------------------------------------------------------------------------------
;
j= double(lf.ny-1)
di= double(lf.nx) / 5.0

for ii=1,4 do begin
  i=  double(ii * di)
  ell_ll2rb, lng[3], lat[3], lng[4], lat[4], r, b
  ell_rb2ll, lng[3], lat[3], i * lf.column_spacing, b, lo, la
  lat= [lat, la]  & lng= [lng, lo] & iid= [iid, i ] & jjd= [jjd, j ]
endfor


      
;------------------------------------------------------------------------------
; Compute longitudes along first azimuth line, assuming that the
; azimuth line falls on a great circle route
;------------------------------------------------------------------------------
;
i= double(0)

dj= double(lf.ny) / 5.0
for jj=1,4 do begin
  j=  double(jj * dj)
  ell_ll2rb, lng[1], lat[1], lng[3], lat[3], r, b
  ell_rb2ll, lng[1], lat[1], j * lf.line_spacing, b, lo, la
  lat= [lat, la]  & lng= [lng, lo] & iid= [iid, i ] & jjd= [jjd, j ]
endfor


;------------------------------------------------------------------------------
; Compute longitudes along last azimuth line, assuming that the
; azimuth line falls on a great circle route
;------------------------------------------------------------------------------
;
i= double(lf.nx-1.0D)

dj= double(lf.ny) / 5.0
for jj=1,4 do begin
  j=  double(jj * dj)
  ell_ll2rb, lng[2], lat[2], lng[4], lat[4], r, b
  ell_rb2ll, lng[2], lat[2], j * lf.line_spacing, b, lo, la
  lat= [lat, la]  & lng= [lng, lo] & iid= [iid, i ] & jjd= [jjd, j ]
endfor

 ;;
 ;;------------------------------------------------------------------------------
 ;; Do a linear fit for azimuth variation
 ;;------------------------------------------------------------------------------
 ;;
 ;j1=     0.0D
 ;j2=     (lf.ny-1.0D)
 ;lat1=   lf.bottom_left_geodetic_coordinates[0]  
 ;lng1=   lf.bottom_left_geodetic_coordinates[1]
 ;lat2=   lf.top_left_geodetic_coordinates[0]  
 ;lng2=   lf.top_left_geodetic_coordinates[1]
 ;m_lat=  (lat2-lat1)/(j2-j1)
 ;b_lat=  lat1 - m_lat * j1
 ;m_lng=  (lng2-lng1)/(j2-j1)
 ;b_lng=  lng1 - m_lng * j1
 ;j3=     j2/2.0
 ;i3=     0.0D
 ;la=     m_lat * j3 + b_lat
 ;lo=     m_lng * j3 + b_lng
 ;lat=    [lat, la]  & lng= [lng, lo] & iid= [iid, i3 ] & jjd= [jjd, j3 ]
 ;
 ;
 ;j1=     0.0D
 ;j2=     (lf.ny-1.0D)
 ;lat1=   lf.bottom_right_geodetic_coordinates[0]  
 ;lng1=   lf.bottom_right_geodetic_coordinates[1]
 ;lat2=   lf.top_right_geodetic_coordinates[0]  
 ;lng2=   lf.top_right_geodetic_coordinates[1]
 ;m_lat=  (lat2-lat1)/(j2-j1)
 ;b_lat=  lat1 - m_lat * j1
 ;m_lng=  (lng2-lng1)/(j2-j1)
 ;b_lng=  lng1 - m_lng * j1
 ;j3=     j2/2.0
 ;i3=     lf.nx-1.0D
 ;la=     m_lat * j3 + b_lat
 ;lo=     m_lng * j3 + b_lng
 ;lat=    [lat, la]  & lng= [lng, lo] & iid= [iid, i3 ] & jjd= [jjd, j3 ]
 
 
 ;
; Scale indices appropriately
;
iid=  iid / lf.ave_pixel
jjd=  jjd / lf.ave_line     
;
; We wish to find equations of the form form
;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;
if printflag then print, 'Computing lng/lat => i fit'
ans_compute_fit, lng, lat,  iid,   lf.deg, i_coef,   i_xexp,   i_yexp, quiet=quiet 
if printflag then print, 'Computing lng/lat => j fit'
ans_compute_fit, lng, lat,  jjd,   lf.deg, j_coef,   j_xexp,   j_yexp, quiet=quiet

;       
; longitude latitude regression.  Longitude and latitude as a function of pixel position
;

if printflag then print, 'Computing i/j => lon fit'
ans_compute_fit, iid, jjd, lng, lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet 
if printflag then print, 'Computing i/j => lat fit'
ans_compute_fit, iid, jjd, lat, lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet

;
; Check and see if we can go back and forth

ns= n_elements(iid)

if printflag then begin
   ;
   ; Check latitude conversions
   ;
   print, 'Latitude conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lat[i] )
     latp=   ans_poly_cal( iid[i], jjd[i], lat_coef, lat_xexp, lat_yexp )
     mu= mu+  (lat[i]-latp)
     sd= sd + (lat[i]-latp)^2
     output= output + string( latp )
     print, output
   endfor
   print, 'Latitude', mu/ns, sqrt(sd/ns)
   ;
   ; Check longitude conversions
   ;
   print, 'Longitude conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lng[i] )
     lngp=   ans_poly_cal( iid[i], jjd[i], lon_coef, lon_xexp, lon_yexp )
     mu= mu+  (lng[i]-lngp)
     sd= sd + (lng[i]-lngp)^2
     output= output + string( lngp )
     print,   output
   endfor
   print, 'Longitude',mu/ns, sqrt(sd/ns)
   ;
   ; Check i conversions
   ;
   print, 'i conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(iid[i])
     ip=     ans_poly_cal( lng[i], lat[i], i_coef,   i_xexp,   i_yexp )
     mu= mu+  (iid[i]-ip)
     sd= sd + (iid[i]-ip)^2
     output= output + string(ip ) + string( iid[i] - ip )
     print,  output
   endfor
   print, 'i',mu/ns, sqrt(sd/ns),mu/ns*lf.column_spacing, sqrt(sd/ns)*lf.column_spacing
   ;
   ; Check j conversions
   ;
   print, 'j conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(jjd[i])
     jp=     ans_poly_cal( lng[i], lat[i], j_coef,   j_xexp,   j_yexp )
     mu= mu+  (jjd[i]-jp)
     sd= sd + (jjd[i]-jp)^2
     output= output + string(jp ) + string( jjd[i] - jp ) 
     print,  output
  endfor
  print, 'j',mu/ns, sqrt(sd/ns), mu/ns*lf.line_spacing ,sqrt(sd/ns)*lf.line_spacing
endif

;
; Add to structure
;

temp_lf= create_struct(lf, 'lon_coef', lon_coef, $
                           'lon_xexp', lon_xexp, $
                           'lon_yexp', lon_yexp, $
                           'lat_coef', lat_coef, $
                           'lat_xexp', lat_xexp, $
                           'lat_yexp', lat_yexp, $
                           'i_coef',   i_coef,   $
                           'i_xexp',   i_xexp,   $
                           'i_yexp',   i_yexp,   $
                           'j_coef',   j_coef,   $
                           'j_xexp',   j_xexp,   $
                           'j_yexp',   j_yexp    )

lf=               0
lf=               temp_lf


end

