; '-----------------------------------------------------
; +
; NAME: 
;    COSMOS_RLOOK2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a 
;   polynomial fit to go from i and j to 
;   radar look angle
; CALLING SEQUENCE:
;    cosmos_rlook2pix_fit, lf   
; INPUTS:
;    lf         Structure with lat/lngs and ave_pixel, ave_line
;               and degree of fit. Assume that the
;               lon_coef and lat_coef are included in 
; OUTPUTS:
;   Added to structure     
;   lf.rlook_coef=  Array of latitude coefficients 
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
; MODIFICATION HISTORY:
;    2011 Jan 19 Frank Monaldo
;    2011 Jan 24 Frank Monaldo. Make sure that quiet is passed to subs
; Copyright (C) 2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro cosmos_rlook2pix_fit, lf, quiet=quiet, help=help

if keyword_set(help) or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '    COSMOS_RLOOK2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a 
  print, '   polynomial fit to go from i and j to 
  print, '   radar look angle
  print, ' CALLING SEQUENCE:
  print, '    cosmos_rlook2pix_fit, lf   
  print, ' INPUTS:
  print, '    lf         Structure with lat/lngs and ave_pixel, ave_line
  print, '               and degree of fit. Assume that the
  print, '               lon_coef and lat_coef are included in 
  print, ' OUTPUTS:
  print, '   Added to structure     
  print, '   lf.rlook_coef=  Array of latitude coefficients 
  print, ' KEYWORDS:
  print, '    /quiet=    Suppress printed messages. 
  print, '    /help=     Display help
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

if printflag then  print, 'Computing a fit from i/j and rlook'


nx=    lf.nx
ny=    lf.ny
ii=    double( [0, nx/3.0, nx/2, nx *2.0/3.0, nx-1 ] ) / lf.ave_pixel
jj=    double( [0, ny/3.0, ny/2, ny *2.0/3.0, ny-1 ] ) / lf.ave_line
nn=    n_elements(ii) 
if lf.columns_order ne 'NEAR-FAR' then begin
  if printflag then print, 'Column order ' + lf.columns_order
endif
if lf.lines_order ne 'EARLY-LATE' then begin
  if printflag then print, 'Line order ' + lf.lines_order
endif
rlook= dblarr(nn*nn)
iid=   rlook
jjd=   rlook
ind=   -1
for i=0, nn-1 do begin
  for j=0, nn-1 do begin
    ind=       ind + 1
    di=        10
    if lf.columns_order ne 'NEAR-FAR' then di=-di
    iid[ind]=  ii[i] 
    jjd[ind]=  jj[j]
    rnd1=  randomn(seed)
    rnd2=  randomn(seed)
    lo1=  ans_poly_cal( ii[i],    jj[j], lf.lon_coef, lf.lon_xexp, lf.lon_yexp) 
    la1=  ans_poly_cal( ii[i],    jj[j], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
    lo2=  ans_poly_cal( ii[i]+di, jj[j], lf.lon_coef, lf.lon_xexp, lf.lon_yexp) 
    la2=  ans_poly_cal( ii[i]+di, jj[j], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
    ll2rb, lo1, la1, lo2, la2, range, bearing
    rlook[ind] = double(bearing) 
    if printflag then begin   
      print, string( i,          format='(I6)'    ) + ' ' + $
             string( j,          format='(I6)'    ) + ' ' + $
             string( iid[ind],   format='(I6)'    ) + ' ' + $
             string( jjd[ind],   format='(I6)'    ) + ' ' + $
             string( lo1,        format='(F10.5)' ) + ' ' + $
             string( la1,        format='(F10.5)' ) + ' ' + $
             string( lo2,        format='(F10.5)' ) + ' ' + $
             string( la2,        format='(F10.5)' ) + ' ' + $
             string( rlook[ind], format='(F10.5)' )
    endif
  endfor
endfor
   

;
; We wisoriunth to find equations of the form form
;  rlook=  sum_m=0^deg sum_n=0^deg A(m,n) * i^m *j^n
;
;
; 
; Compute the relationship between i,j and incident angle
;
ans_compute_fit, iid, jjd,  rlook,   lf.deg, rlook_coef,   rlook_xexp,   rlook_yexp, quiet=quiet



ns= n_elements(iid)

if printflag then begin
   ;
   ; Check incidene conversions
   ;
   print, 'Rlook conversions:'
   mu=0
   sd=0.
   for i=0, ns-1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( rlook[i] )
     rlookp=   ans_poly_cal( iid[i], jjd[i], rlook_coef, rlook_xexp, rlook_yexp )
     mu= mu + (rlook[i]-rlookp)
     sd= sd + (rlook[i]-rlookp)^2
     output= output + string( rlookp )
     print, output
   endfor
   print, 'Rlook:', mu/ns, sqrt(sd/ns)
endif

;
; Add to structure
;

temp_lf= create_struct(lf, 'rlook_coef', rlook_coef, $
                           'rlook_xexp', rlook_xexp, $
                           'rlook_yexp', rlook_yexp  )

lf=               0
lf=               temp_lf


end

