; -----------------------------------------------------
; +
; NAME: 
;   ENV_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a  
;   polynomial fit to go from pixel location to incidence angle.  This
;   works with ASAR data from ENVISAT.
; CALLING SEQUENCE:
;    env_incid2pix_fit, lf
; INPUTS:
;    lf:  leader structure
; OUTPUTS
;    Coefficients added to structure
;    lf.incid_coef: Array of incidence angle coefficients a function of 
;                  i,j.  The output of the fit is in degrees.
; KEYWORDS:
;    /quiet=     Suppress printed messages. 
;    /help=      Display help
;    /diagnostic Print diagnostic results
; MODIFICATION HISTORY:
;  2003 Mar 29 Frank Monaldo  
;  2004 Jun 12 Hardwired a degree of 2 for polynomial fit
;  2009 Nov 25 Changed for ANSWRS2.0
;  2010 jan 06 Change to use ans_compute_ans
;  2010 Jun 14 Print help on no input.
;  2010 Dec 31 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2003-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro  env_incid2pix_fit, lf, quiet=quiet, help=help,diagnostic=diagnostic


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '   ENV_INCID2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a  
  print, '   polynomial fit to go from pixel location to incidence angle.  This
  print, '   works with ASAR data from ENVISAT.
  print, ' CALLING SEQUENCE:
  print, '    env_incid2pix_fit, lf
  print, ' INPUTS:
  print, '    lf:  leader structure
  print, ' OUTPUTS
  print, '    Coefficients added to structure
  print, '    lf.incid_coef: Array of incidence angle coefficients a function of 
  print, '                  i,j.  The output of the fit is in degrees.
  print, ' KEYWORDS:
  print, '    /quiet=     Suppress printed messages. 
  print, '    /help=      Display help
  print, '    /diagnostic Print diagnostic results
  return
endif

if printflag then  print, 'Computing fit of incidence vs i,j...'

for i=0, lf.num_dsd-1 do begin
  if lf.ds_name[i] eq 'MDS1' then i0=i
endfor

sz= size(lf.first_angles)

ns=0L

;
; If only the i=0 values filled, assume that the remaining incident angles
; are independent of j. We then fudge the 2D values
;
only_first_line=0
if total(lf.first_angles[1,*]) eq 0 then only_first_line=1
if only_first_line then begin
  for i=0, sz[1]-1 do begin
    lf.line_num[i]=             i+1
    for j=0, sz[2]-1 do begin
      lf.first_angles[i,j]=       lf.first_angles[0,j]
      lf.first_samp_number[i,j]=  lf.first_samp_number[0,j]
    endfor
  endfor
endif


for i=0, sz[1]-1 do begin
  for j=0, sz[2]-1 do begin
    if lf.first_angles[i,j] ne 0 then begin
      ns=ns+1L
      if ns eq 1 then begin
        incid= [lf.first_angles[i,j]]
        ii=    [lf.first_samp_number[i,j]-1]
        jj=    [lf.line_num[i]-1]
      endif else begin
        incid= [incid, lf.first_angles[i,j]]
        ii=    [ii,    lf.first_samp_number[i,j]]
        jj=    [jj,    lf.line_num[i]-1]
      endelse
    endif
  endfor
endfor

iid=  double(ii) / lf.ave_pixel
jjd=  double(jj) / lf.ave_line     

;
; We wish to find equations of the form form
;  theta=  sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n
;
df_deg=  2
rdeg=   (df_deg+1)^2-1
;
;   
; theta-regression.  incidence position as a function of longitude and latitude 
;
ans_compute_fit, iid,  jjd, incid, lf.deg, incid_coef, incid_xexp, incid_yexp, quiet=quiet


fcheck=finite(incid_coef)
w=where(fcheck eq 0, fcount)
if fcount gt 0 then begin
   print, 'There is a problem with the incident angle fit'
   print, 'Input incident angles'
   print, incid
   stop
endif

if keyword_set(diagnostic) then begin
   print, 'Incidence angle conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=lon(jjd[i]+0.5)
     output=          string(i1, format='(I7)')
     output= output + string(j1, format='(I7)')
     output= output + string( incid[i] )
     incidp=   ans_poly_cal( iid[i], jjd[i], incid_coef, incid_xexp, inicd_yexp )
     mu= mu+  (incid[i]-incidp)
     sd= sd + (incid[i]-incidp)^2
     output= output + string( incidp )
     print, output
   endfor
   print, 'Fractional error in incidence angle', mu/ns, sqrt(sd/ns)
   stop
endif	 


;
; Add to structure
;

temp_lf= create_struct( lf, 'incid_coef', incid_coef, $
                            'incid_xexp', incid_xexp, $
                            'incid_yexp', incid_yexp  )
lf=      0
lf=      temp_lf


end
