; '-----------------------------------------------------
; +
; NAME: 
;    ENV_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a 
;   polynomial fit to go from lon/lat to pixel location and visa versa.
;   This program readings information from the lf structure
; CALLING SEQUENCE:
;    env_lonlat2pix_fit, lf   
; INPUTS:
;    lf         Structure with lat/lngs and ave_pixel, ave_line
;               and degree of fir
; OUTPUTS:
;   Added to structure     
;   lf.i_coef:    Array of coefficients for horizontal pixel fit
;   lf.j_coef=    Array of coefficients for vertical pixel  fit
;   lf.lon_coef=  Array of longitude coefficients
;   lf.lat_coef=  Array of latitude coefficients 
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
;    /diagnostic Display comparison results
;    error=     errpr
; MODIFICATION HISTORY:
;  2003 Apr 01 Frank Monaldo  
;  2009 Nov 25 Frank Monaldo. Rewrite for ANSWRS 2.0
;              Frank Monaldo. Removed the need for the df structure and included
;              Frank Monaldo. coefficients in structure - Frank Monaldo
;  2010 Jan 06 Frank Monaldo. Change to use ans_compute_fit
;  2010 Jan 08 Frank Monaldo. Change fix to long
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2010 Dec 30 Frank Monaldo. Pass on quiet flag
;  2011 Nov 08 Frank Monaldo. Error flag
; Copyright (C) 2003-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro env_lonlat2pix_fit, lf, quiet=quiet, help=help,diagnostic=diagnostic, error=error

printflag=1

error= 0
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1) then begin  
  print, ' NAME: 
  print, '    ENV_LONLAT2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a 
  print, '   polynomial fit to go from lon/lat to pixel location and visa versa.
  print, '   This program readings information from the lf structure
  print, ' CALLING SEQUENCE:
  print, '    env_lonlat2pix_fit, lf   
  print, ' INPUTS:
  print, '    lf         Structure with lat/lngs and ave_pixel, ave_line
  print, '               and degree of fir
  print, ' OUTPUTS:
  print, '   Added to structure     
  print, '   lf.i_coef:    Array of coefficients for horizontal pixel fit
  print, '   lf.j_coef=    Array of coefficients for vertical pixel  fit
  print, '   lf.lon_coef=  Array of longitude coefficients
  print, '   lf.lat_coef=  Array of latitude coefficients 
  print, ' KEYWORDS:
  print, '    /quiet=    Suppress printed messages. 
  print, '    /help=     Display help
  return
endif

if printflag then  print, 'Computing a fit to lon and lat for line of data...'

for i=0, lf.num_dsd-1 do begin
  if lf.ds_name[i] eq 'MDS1' then i0=i
endfor


sz= size(lf.first_lats)

ns=0L

for i=0, sz[1]-1 do begin
  for j=0, sz[2]-1 do begin
    ;Only consider non_zero values.
    if lf.first_lats[i,j]  ne 0 and $
       lf.first_longs[i,j] ne 0 then begin
      ns=ns+1L
      if ns eq 1 then begin
        lat= [lf.first_lats[i,j]]
        lng= [lf.first_longs[i,j]]
        ii=  [lf.first_samp_number[i,j]]
        jj=  [lf.line_num[i]-lf.line_num[0]]
      endif else begin
        lat= [lat, lf.first_lats[i,j]]
        lng= [lng, lf.first_longs[i,j]]
        ii=  [ii,  lf.first_samp_number[i,j]]
        jj=  [jj,  lf.line_num[i]-lf.line_num[0]]
      endelse
    endif
  endfor
endfor

iid=  double(ii) / lf.ave_pixel
jjd=  double(jj) / lf.ave_line     
;
; We wish to find equations of the form form
;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;
 
ans_compute_fit, lng, lat,  iid,   lf.deg, i_coef,   i_xexp,   i_yexp, quiet=quiet
ans_compute_fit, lng, lat,  jjd,   lf.deg, j_coef,   j_xexp,   j_yexp, quiet=quiet


;       
; longitude latitude regression.  Longitude and latitude as a function of pixel position
;


ans_compute_fit, iid, jjd, lng, lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
ans_compute_fit, iid, jjd, lat, lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet

;
; Check for reasonableness
;
lo0=ans_poly_cal(        0,        0, lon_coef, lon_xexp, lon_yexp )
lo1=ans_poly_cal( max(iid), max(jjd), lon_coef, lon_xexp, lon_yexp )
if printflag then print, lo0, lo1

if lo0 gt  400 or $
   lo0 lt -200 or $
   lo1 gt  400 or $
   lo1 lt -200 then begin
   lon_coef= lon_coef*0.
   print, 'Bad lon fit ', lo0, lo1
   error=-1
endif




;
; Check and see if we can go back and forth

if keyword_set(diagnostic) then begin
   ;
   ; Check latitude conversions
   ;
   print, 'Latitude conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lat[i] )
     latp=   ans_poly_cal( iid[i], jjd[i], lat_coef, lat_xexp, lat_yexp )
     mu= mu+  (lat[i]-latp)
     sd= sd + (lat[i]-latp)^2
     output= output + string( latp )
     print, output
   endfor
   stop
   print, 'Latitude', mu/ns, sqrt(sd/ns)
   ;
   ; Check longitude conversions
   ;
   print, 'Longitude conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lng[i] )
     lngp=   ans_poly_cal( iid[i], jjd[i], lon_coef, lon_xexp, lon_yexp )
     mu= mu+  (lng[i]-lngp)
     sd= sd + (lng[i]-lngp)^2
     output= output + string( lngp )
     print,   output
   endfor
   print, 'Longitude',mu/ns, sqrt(sd/ns)
   stop
   ;
   ; Check i conversions
   ;
   print, 'i conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(iid[i])
     ip=     ans_poly_cal( lng[i], lat[i], i_coef,   i_xexp,   i_yexp )
     mu= mu+  (iid[i]-ip)
     sd= sd + (iid[i]-ip)^2
     output= output + string(ip )
     print,  output
   endfor
   print, 'i',mu/ns, sqrt(sd/ns)
   stop
   ;
   ; Check j conversions
   ;
   print, 'j conversions:'
   mu=0
   sd=0.
   for i=0, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(jjd[i])
     jp=     ans_poly_cal( lng[i], lat[i], j_coef,   j_xexp,   j_yexp )
     mu= mu+  (jjd[i]-jp)
     sd= sd + (jjd[i]-jp)^2
     output= output + string(jp )
     print,  output
  endfor
  print, 'j',mu/ns, sqrt(sd/ns)
  stop
endif

;
; Add to structure
;

temp_lf= create_struct(lf, 'lon_coef', lon_coef, $
                           'lon_xexp', lon_xexp, $
                           'lon_yexp', lon_yexp, $
                           'lat_coef', lat_coef, $
                           'lat_xexp', lat_xexp, $
                           'lat_yexp', lat_yexp, $
                           'i_coef',   i_coef,   $
                           'i_xexp',   i_xexp,   $
                           'i_yexp',   i_yexp,   $
                           'j_coef',   j_coef,   $
                           'j_xexp',   j_xexp,   $
                           'j_yexp',   j_yexp    )

lf=               0
lf=               temp_lf

error=1
end

