;----------------------------------------------------------
;+
; NAME: env_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out Envisat data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;    rsat2_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   
; MODIFICATION
; 2007 Aug 22 Frank Monaldo 
; 2008 Sep 13 Chris Jackson  - 
;             Made changes in the Image Identification 
;             and Image Orientation block to ensure that
;             certain lines are always written out
;             Made sure PALSAR corners are written
; 2009 May 01 Frank Monaldo Corrected problem w/ print out of incident
;             angle coefficents
; 2009 Jul 27 Frank Monaldo . Added code to keep the program
;             even if the lf.scansar and lf.s_c_altitude do not exist. This
;             was required because these fields are not filled when reading
;             RSAT2 data
; 2009 Oct 14 Frank Monaldo. Radically re-written for new software version  
; 2009 Nov 10 Frank Monaldo. Correct bug that wrote a line 
;                            transformation rather than an element transformation
;                             Re-written for Envisat
; 2009 Dec 02 Frank Monaldo. Changed: longitude (0to-360) transform coef for element
;                            to: longitude (0to360) transform coef for element
; 2010 Jan 06 Frank Monaldo. Change to use ans_poly_cal
; 2010 Jan 08 Frank Monaldo. Corrected output line for longitude coefficients
; 2010 Jan 13 Frank Monaldo. Re-write to use the generalized ans_write_ancil_sio 
;                            program.
; 2010 Jan 22 Frank Monaldo. Computing illumination direction and relative heading 
;                            according to instructins from C. C. Wackerman
; 2010 Jan 15 Frank Monaldo  Compute heading using lon/lat information
; 2010 Jun 14 Frank Monaldo  Print help on no input.
; 2010 Dec 30 Frank Monaldo  Pass on quiet flag
; 2013 Nov 01 Chris Jackson  Put 'ERS' string into lf_sar_sensor when processing 
;                            ERS data in Envsisat N1 format 
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_write_ancil_sio, filename, lf, scaling, quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: env_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out Envisat data into ancillary data ascii file. These
  print, '   data fit the SIO format
  print, ' CALLING SEQUENCE:'
  print, '    rsat2_write_ancil_sio, filename, lf,  scaling'
  print, ' INPUT:'
  print, '   filename=     output file with ancillary data
  print, '   lf=           leader structure'
  print, '   scaling=      scaling for data in binary file of NRCS, default=1
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  return
endif


idstr = getwrd(lf.product, 0, /last, del='.')
if (idstr eq 'E1') then lf.sar_sensor= 'ERS1'
if (idstr eq 'E2') then lf.sar_sensor= 'ERS2'

name_dir=             lf.product
lf.producttype=       getwrd(lf.product, 1, del='_')
software_version=     lf.software_ver
orbit_number=         string(lf.abs_orbit)
valuetype=            'intensity'
platform_code=        'N1' 
pol=                  lf.polarization
frequency_band=       'C'
scaling=              scaling
lf.asc_des=           strupcase( strmid( lf.asc_des, 0,1) )
range_type=           'GROUND RANGE'
range_flag=           1
element_spacing=      lf.range_pixel   * lf.ave_pixel
lf.line_spacing=      lf.azimuth_pixel * lf.ave_line
beam_mode=            lf.SPH_DESCRIPTOR
flip=                  0
;
; Compute heading
;

lo1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
lo2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
ll2rb, lo1, la1, lo2, la2, r, b
if b gt 180 then b= b-360.0
heading= b



flip=               0
illumination_dir=   0
heading_rel=        90
near_range_edge=   'L'
early_azimuth_edge='B'


sensor_velocity=       sqrt(lf.x_velocity^2 + lf.y_velocity^2 + lf.z_velocity^2)
sensor_velocity=       string(sensor_velocity, format='(F15.6)' ) 
integration_time=      '-99999.99'
nom_az_res=            '-99999.99'      
nom_rg_res=            '-99999.99'      
rcvr_gain=             '-99999.99'       
proc_gain=             '-99999.99'
range_swath=           lf.range_pixel   * lf.ave_pixel * lf.nx1
azimuth_swath=         lf.azimuth_pixel * lf.ave_line  * lf.ny1
center_incid=          ans_poly_cal(lf.nx1/2,lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
slant_range_center=    -99999.99
slant_range_near=      -99999.99
ellipsoidname=         ' '
semimajoraxis=         -99999.99
semiminoraxis=         -99999.99
datumshiftparameters=  [0.0, 0.0, 0.0]
ellipsoidscale=        '1'
deg_incid=             lf.deg
deg_rlook=             lf.deg
 
if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf,                                             $
                          'name_dir',             name_dir,               $
                          'frequency_band',       frequency_band,         $
                          'ellipsoidName',        ellipsoidName,          $
                          'ellipsoidscale',       ellipsoidscale,         $
                          'software_version',     software_version,       $
                          'platform_code',        platform_code,          $
                          'valuetype',            valuetype,              $
                          'orbit_number',         orbit_number,           $
                          'beam_mode',            beam_mode,              $
                          'sensor_velocity',      sensor_velocity,        $
                          'pol',                  pol,                    $
                          'scaling',              scaling,                $
                          'heading',              heading,                $
                          'heading_rel',          heading_rel,            $
                          'integration_time',     integration_time,       $
                          'nom_az_res',           nom_az_res,             $
                          'nom_rg_res',           nom_rg_res,             $
                          'rcvr_gain',            rcvr_gain,              $
                          'proc_gain',            proc_gain,              $
                          'flip',                 flip,                   $
                          'range_type',           range_type,             $
                          'range_flag',           range_flag,             $
                          'element_spacing',      element_spacing,        $
                          'range_swath',          range_swath,            $
                          'azimuth_swath',        azimuth_swath,          $
                          'center_incid',         center_incid,           $
                          'slant_range_center',   slant_range_center,     $
                          'slant_range_near',     slant_range_near,       $
                          'semimajoraxis',        semimajoraxis,          $
                          'semiminoraxis',        semiminoraxis,          $
                          'datumshiftparameters', datumshiftparameters,   $
                          'deg_rlook',            deg_rlook,              $
                          'deg_incid',            deg_incid,              $
                          'near_range_edge',      near_range_edge,        $
                          'early_azimuth_edge',   early_azimuth_edge,     $
                          'illumination_dir',     illumination_dir        )
                          
  lf=      0                                    
  lf=      temp_lf
endif else begin  
  lf.name_dir=             name_dir
  lf.frequency_band=       frequency_band        
  lf.ellipsoidName=        ellipsoidName         
  lf.ellipsoidscale=       ellipsoidscale 
  lf.software_version=     software_version      
  lf.platform_code=        platform_code         
  lf.valuetype=            valuetype             
  lf.orbit_number=         orbit_number
  lf.beam_mode=            beam_mode 
  lf.sensor_velocity=      sensor_velocity 
  lf.pol=                  pol                   
  lf.scaling=              scaling               
  lf.heading=              heading               
  lf.heading_rel=          heading_rel           
  lf.sensor_velocity=      sensor_velocity       
  lf.integration_time=     integration_time      
  lf.nom_az_res=           nom_az_res            
  lf.nom_rg_res=           nom_rg_res            
  lf.rcvr_gain=            rcvr_gain             
  lf.proc_gain=            proc_gain             
  lf.flip=                 flip                  
  lf.range_type=           range_type            
  lf.range_flag=           range_flag            
  lf.element_spacing=      element_spacing       
  lf.range_swath=          range_swath           
  lf.azimuth_swath=        azimuth_swath         
  lf.center_incid=         center_incid          
  lf.slant_range_center=   slant_range_center    
  lf.slant_range_near=     slant_range_near      
  lf.semimajoraxis=        semimajoraxis         
  lf.semiminoraxis=        semiminoraxis         
  lf.datumshiftparameters= datumshiftparameters    
  lf.deg_rlook=            deg_rlook             
  lf.deg_incid=            deg_incid             
  lf.near_range_edge=      near_range_edge       
  lf.early_azimuth_edge=   early_azimuth_edge    
  lf.illumination_dir=     illumination_dir       
endelse


;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet= quiet
 
  
  
  
 end
