;-----------------------------------------------------
;+
; NAME: ENV_WSM_DN2SIGMA
; PURPOSE:
;   The purpose of this program is to convert the ENVISAT digital numbers
;   to radar cross section. The NRCS values are average down if 
;   lf.ave_line or lf.ave_pixel > 1
; CALLING SEQUENCE:
;  env_wsm_dn2sigma, lf, image 
; INPUT: 
;   lf=          Leader information structure           
;   image=       Input digital number array
; OUTPUT:
;   image=       A floating point array of sigma.
; KEYWORDS=
;   /quiet=      Do not print intermediate results.
;   /help=       List help.
;
; MODIFICATION HISTORY:
;   2003 Mar 29 Frank Monaldo 
;   2005 Jul 26 If array too large do not compute statistics
;   2006 Jul 19 Do not compute use the where function for
;               cases when the generated array would be too
;               large
;   2006 Oct 15 Compute stats from smaller array if array too large,
;               Zero out non-finite values
;   2009 Nov 25 Re-written for ANSWRS 2.0
;   2010 Jan 06 Frank Monaldo. Change to ans_poly_cal 
;   2010 Jan 08 Frank Monaldo. Change call to ans_poly_cal to make sure that
;                              the first two input variable have the same size
;   2010 Jan 01 Frank Monaldo. Changed to increase speed of computation of stats
; Copyright (C) 2003-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_wsm_dn2sigma, lf, image, help=help, quiet=quiet	   

if keyword_set(help) or (n_params(0) lt 1) then begin
  print, ' NAME: ENV_WSM_DN2SIGMA
  print, ' PURPOSE:
  print, '   The purpose of this program is to convert the ENVISAT digital numbers
  print, '   to radar cross section. The NRCS values are average down if 
  print, '   lf.ave_line or lf.ave_pixel > 1
  print, ' CALLING SEQUENCE:
  print, '  env_wsm_dn2sigma, lf, image 
  print, ' INPUT: 
  print, '   lf=          Leader information structure           
  print, '   image=       Input digital number array
  print, ' OUTPUT:
  print, '   image=       A floating point array of sigma.
  print, ' KEYWORDS=
  print, '   /quiet=      Do not print intermediate results.
  print, '   /help=       List help.
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 
arch=!version.arch

sz=    size(image)
nx=    sz[1]
ny=    sz[2]
nyy=   ny / 10
image= float(image)

if printflag then begin 
  print, '---------------------------------------------------------'
  print, 'ENVISAT CALIBRATION: Converting digital number to sigma.'
  print, '---------------------------------------------------------'
endif

if printflag and ny lt 10000 then begin
  print, 'Image values BEFORE squaring:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 2000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw Envisat data points from file=', mean(image[w])
    print, 'SD   of raw Envisat data points from file=', stdev(image[w])
  endif 
  if count gt 0 and count ge 2000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Statistics for first 4,000,000 nonzero values.'
    print, 'Mean of raw Envisat data points from file=', mean( image[w[0:1999999]])
    print, 'SD   of raw Envisat data points from file=', stdev(image[w[0:1999999]])
  endif 
endif
;
if printflag then print,'Squaring image line by line...'
for i=0L, ny-1 do begin
  if i mod nyy eq 0 and printflag then print, 'Squaring azimuth line ', i
  image[*,i]= image[*,i] * image[*,i]
endfor

if printflag  and ny lt 10000 then begin
  print, ''
  print, 'Image values AFTER squaring:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 2000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of squared Envisat data points=', mean(image[w])
    print, 'SD   of squared Envisat data points=', stdev(image[w])
  endif 
  if count gt 0 and count ge 2000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Statistics for first 4,000,000 nonzero values.'
    print, 'Mean of squared Envisat data points from file=',mean( image[w[0:1999999]])
    print, 'SD   of squared Envisat data points from file=',stdev(image[w[0:1999999]])
  endif
endif

;
; Check if we want to a apply calibration correction
; 
if lf.ext_cal_fact[0] le 1 then begin
  sfv0= getwrd(lf.software_ver,0,del='/') 
  ;
  ; Data from ESA processor begins with 'ASAR'
  ;
  if strupcase(sfv0) eq 'ASAR' then begin
    ;
    ; numeric version number
    ;
    sfv1= getwrd(lf.software_ver,1,del='/')
      if isnumber(sfv1) ne 0 then begin
        sfv11= float(sfv1)
        if sfv11 eq 3.02 or  lf.ext_cal_fact[0] le 1 then lf.ext_cal_fact[0]= 2.0*7726806
        if sfv11 eq 3.03 and lf.ext_cal_fact[0] le 1 then lf.ext_cal_fact[0]= 7726806.0
        if sfv11 ge 3.05 and lf.ext_cal_fact[0] le 1 then lf.ext_cal_fact[0]= 6729766.5
      endif  ; Endif is a number
  endif      ; Endif first part of software version ='ASAR'
endif        ; Endif lf.ext_cal_fact =< 1

scaling= 1.0
if lf.ext_cal_fact[0] gt 0 then scaling= 1.0 / lf.ext_cal_fact[0]
if printflag then begin
  print, 'Using lf.ext_cal_fact[0]=                ',lf.ext_cal_fact[0] 
  print, 'Original calibration in file=            ',lf.ext_cal_fact[2]
  print, 'Scaling factor= 1.0 / lf.ext_cal_fact[0]=',scaling
endif

for j=0L, ny-1 do begin  ; Loop through azimuth index
  line=image[*,j]
  ;
  ; Find values in the line of the image that are >0 and finite
  w=where(line gt 0 and finite(line) eq 1, count)
  if j mod nyy eq 0 and printflag then  begin
     print, 'Calibrating line ' + string(j, for='(I6)') + ' of ' + string(ny, for='(I6)')
  endif
  ;
  ; Continue processing line only if there
  ; are values >0 in line
  ;
  if count gt 0 then begin
    ;
    ; Compute the incidence angle along the line and
    ; convert to radians.
    ;
    ; Forcing, xtemp and ytemp to have the same number of elements
    ;
    xtemp=     double(w)/lf.ave_pixel
    ytemp=     xtemp * 0.0 + double(j)/lf.ave_line
    incid=     ans_poly_cal( xtemp,  ytemp, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
    incid=     incid * !pi / 180.  
    line[w]=   line[w] * sin(incid) * scaling
    image[w,j]=line[w]
  endif
  w=where(finite(image[*,j]) ne 1, count)
  if count gt 0 then image[w,j]=0.0
endfor ; End azimuth line loop

if printflag then begin
  print, 'External calibration factor=' +string(lf.ext_cal_fact[0])
  print, 'Last line incidence angles:   Min= '+  $
          string(min(incid)*180/!pi) + ' Max=' +string(max(incid)*180/!pi) 
  sz= size(image)
  if sz[4] lt 4e6 then begin
    print, 'Mean of NRCS=' + strcompress(string(mean(image)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(image))))
    w= where(image gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(image[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(image[w]))))
    endif
  endif else begin
    print, 'Number of array elements = ' +string(sz[4]) + ' Too large. To compute full statistics.'
    print, 'Computing statistics from first 500 lines'
    subimage= image[*,0:499]
    print, 'Mean of NRCS=' + strcompress(string(mean(subimage)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(subimage))))
    w= where(subimage gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(subimage[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(subimage[w]))))
    endif
  endelse
endif 

;
; Average down using information contained in defaults structure
;


if (lf.ave_line gt 1) or (lf.ave_pixel gt 1) then  begin
  if printflag then print, "Averaging calibrated NRCS...'
  if printflag then print, 'Rebinning image...'
  nx1=    (long(nx)  / long(lf.ave_pixel) )
  ny1=    (long(ny)  / long(lf.ave_line) )
  nx2=    nx1 * long(lf.ave_pixel)
  ny2=    ny1 * long(lf.ave_line )
  if nx gt 40000 or ny gt 40000 then begin
    ;
    ; kuldge for very big image
    ;
    print, 'Very large image', nx,ny
    print, 'Averaging by using the temporary command.
    nx_old=long(nx)
    ny_old=long(ny)
    nx= nx/2
    ny= ny/2
    if nx * 2L lt nx_old then nx_old=nx_old-1L
    if ny * 2L lt ny_old then ny_old=ny_old-1L
    image=  rebin( temporary(image[0:nx_old-1L, 0:ny_old-1L]), nx, ny)
    nx1=    (long(nx)  / long(lf.ave_pixel) / 2 )
    ny1=    (long(ny)  / long(lf.ave_line)  / 2)
    nx2=    nx1 * long(lf.ave_pixel) * 2
    ny2=    ny1 * long(lf.ave_line ) * 2
    image=  rebin( image[0:nx2-1,0:ny2-1], nx1, ny1)
  endif else begin
    image=  rebin( image[0:nx2-1,0:ny2-1], nx1, ny1)
  endelse
endif
lf.nx1= long(lf.nx0 /lf.ave_pixel)
lf.ny1= long(lf.ny0 /lf.ave_line )

if printflag then begin
  ;
  print, 'Statistics of output image NRCS data'
  nyy=   lf.ny1/10
  print, 'Compute stats for first ' + string(nyy) + ' lines.'
  im=   image[*, 0:nyy]
  print, 'Mean=    ', mean(im)
  print, 'Std=     ', stdev(im)
  w=where(im gt 0, count)
  if count gt 0 then begin
     print, 'Mean > 0=    ', mean( im[w] )
     print, 'Std  > 0=   ' , stdev(im[w] )
  endif
endif
end
