;-----------------------------------------------------
;+
; NAME: ENV_WSM_GEOLOC_READ.PRO
; PURPOSE:
;   The purpose of this program is to read geolocation information
;   Warning run the programs:
;     env_wsm_main_product_header_read
;     env_wsm_specific_product_header_read
;     so that the lf structure is filled with the necessary information
;     This program will also convert to a 0-360 longitude representation
;     if the data cross the International Date Line.
; CALLING SEQUENCE:
;     env_wsm_geoloc_read, filename, lf,  /help, /quiet
; INPUT:
;  filename:  Filename of ENVISAT file in wide swath medium
;             resolution mode.
;  lf:        Structure with leader information
; OUTPUT:
;  More data filled into the lf strucuture
; KEYWODS
;  /quiet:    Do not print out intermediate results
;  /help:     Print out help
; MODIFICATION HISTORY:F
;   2003 Mar 25 Frank Monaldo  
;   2003 May 03 Modified to deal with discontinuities lf.line_num
;   2007 Jun 05 Fill in corner_ll values.
;   2007 Mar 09 Add provision of for x86
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2010 Jul 21 Frank Monaldo. Improve printout
;   2010 Dec 31 Frank Monaldo. Suppress printout
;   2011 Feb 07 Frank Monaldo. Add error catching
; Copyright (C) 2003-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_wsm_geoloc_read, filename, lf, help=help, quiet=quiet, error=error
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ENV_WSM_GEOLOC_READ.PRO
  print, ' PURPOSE:
  print, '   The purpose of this program is to read the SAR image
  print, '   Warning run the programs:
  print, '     env_wsm_main_product_header_read
  print, '     env_wsm_specific_product_header_read
  print, '     so that the lf structure is filled with the necessary information
  print, '     This program will also convert to a 0-360 longitude representation
  print, '     if the data cross the International Date Line.
  print, ' CALLING SEQUENCE:
  print, '     env_wsm_geoloc_read, filename, lf,  /help, /quiet
  print, ' INPUT:
  print, '  filename:  Filename of ENVISAT file in wide swath medium
  print, '             resolution mode.
  print, '  lf:        Structure with leader information
  print, ' OUTPUT:
  print, '  More data filled into the lf strucuture
  print, ' KEYWODS
  print, '  /quiet:    Do not print out intermediate results
  print, '  /help:     Print out help
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
;
; Determine System Arch
;
arch=strcompress(!version.arch, /remove_all)

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  




;
; Reading geolocation from from file.
;

i0=-1
for i=0, lf.num_dsd-1 do begin
  if strupcase(lf.ds_name[i]) eq 'GEOLOCATIONGRIDADS' then i0=i
endfor
if i0 lt 0 then begin
   print,'Could not figure out where the geolocation data start...'
   print,'Exiting'
   return
endif
offset=   lf.ds_offset(i0)

if printflag then print, 'Opening '+ filename + ' to read geolocation data...'
close, 1
openr, 1, filename
point_lun, 1, offset ; Skip into file
glh=  bytarr(lf.dsr_size[i0])
first_longs= lonarr(lf.num_dsr[i0],11)
first_lats=  lonarr(lf.num_dsr[i0],11)
last_longs=  lonarr(lf.num_dsr[i0],11)
last_lats=   lonarr(lf.num_dsr[i0],11)

if printflag then print, i0,lf.num_dsr[i0]
for i=0L, lf.num_dsr[i0]-1 do begin
  ;
  ; Read in geolocation record
  ;
  readu, 1, glh
  ;
  lf.first_line_zero_doppler_time[i]= string( glh[   0:  11] )
  lf.line_num[i]=                     ulong(  glh[  13:  16], 0 )                  
  lf.num_lines[i]=                    ulong(  glh[  17:  20], 0 )
  lf.sub_sat_track[i]=                float(  glh[  21:  24] ,0 )
  for j=0L, 10 do begin
    j1= j * 4 + 25  &  j2= j1+ 3 
    lf.first_samp_number[i,j]=         ulong( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
    j1= j * 4 + 69  &  j2= j1 + 3 
    lf.first_slant_range_times[i,j]=   float( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
    j1= j * 4 + 113 &  j2= j1 + 3 
    lf.first_angles[i,j]=              float( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
    j1= j * 4 + 157 &  j2= j1  + 3 
    first_lats[i,j]=                    long( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
    j1= j * 4 + 201  &  j2= j1 + 3 
    first_longs[i,j]=                   long( glh[ j1: j2], 0)
  endfor
  lf.last_line_zero_doppler_time[i]=  string( glh[267: 278] )
  for j=0L, 10 do begin
    j1= j * 4 + 279  &  j2= j1 + 3 
    lf.last_samp_number[i,j]=           ulong( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
     j1= j * 4 + 323  & j2= j1 + 3 
     lf.last_slant_range_times[i,j]=    float( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin 
    j1= j * 4 + 367  & j2= j1 + 3
    lf.last_angles[i,j]=               float( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
    j1= j * 4 + 411  & j2= j1 + 3 
    last_lats[i,j]=                      long( glh[ j1: j2], 0)
  endfor
  for j=0L, 10 do begin
     j1= j * 4 + 455 & j2= j1 + 3 
     last_longs[i,j]=                    long( glh[ j1: j2], 0)
  endfor
endfor ; Finishing looping through geolocation records

close, 1
if printflag then begin
  print, 'lf.first_angles      ', format='(20A, $)' &  hlp,lf.first_angles
  print, 'lf.last_angles       ', format='(20A, $)' &  hlp,lf.last_angles
  print, 'lf.first_lats        ', format='(20A, $)' &  hlp,lf.first_lats
  print, 'lf.last_lats         ', format='(20A, $)' &  hlp,lf.last_lats
endif

if not endian() then begin ; If we have Intel internal machine representations
 lf.first_line_zero_doppler_time=swap_endian(lf.first_line_zero_doppler_time)
 lf.line_num=                      swap_endian(lf.line_num)		   
 lf.num_lines=                     swap_endian(lf.num_lines)
 lf.sub_sat_track=                 swap_endian(lf.sub_sat_track)
 lf.first_samp_number=             swap_endian(lf.first_samp_number)
 lf.first_slant_range_times=       swap_endian(lf.first_slant_range_times)
 lf.first_angles=                  swap_endian(lf.first_angles)
 first_lats=                       swap_endian(first_lats)
 first_longs=                      swap_endian(first_longs)
 lf.last_line_zero_doppler_time=   swap_endian(lf.last_line_zero_doppler_time)
 lf.last_samp_number=              swap_endian(lf.last_samp_number)
 lf.last_slant_range_times=        swap_endian(lf.last_slant_range_times)
 lf.last_angles=                   swap_endian(lf.last_angles)
 last_lats=                        swap_endian(last_lats)
 last_longs=                       swap_endian(last_longs)
endif


;
; Convert to lat/long values to float and write into structure
;
sz= size(first_lats)
for i=0, sz(1)-1 do begin
  for j=0, sz(2)-1 do begin
    lf.first_lats[i,j]=   float(first_lats[i,j] ) / 1e6
    lf.first_longs[i,j]=  float(first_longs[i,j]) / 1e6
    lf.last_lats[i,j]=    float(last_lats[i,j]  ) / 1e6
    lf.last_longs[i,j]=   float(last_longs[i,j] ) / 1e6
  endfor
endfor
if printflag then begin  
  print, 'Data from geoloc record...'
  print, 'lf.first_lats      ', format='(20A, $)' &  hlp, lf.first_lats
  print, 'lf.first_longs     ', format='(20A, $)' &  hlp, lf.first_longs
  print, 'lf.last_lats       ', format='(20A, $)' &  hlp, lf.last_lats
  print, 'lf.last_longs      ', format='(20A, $)' &  hlp, lf.last_longs
  w=where(lf.first_lats ne 0, count)
  print, 'Number of first lats ne 0: ' + string(count)
  hlp, lf.sub_sat_track
  hlp, lf.first_angles, lf.last_angles
  w=where(lf.first_angles ne 0, count)
  print, 'Number of angles lats ne 0: ' + string(count)
endif

;
; For purposes of continuity, if the data cross the International Date
; Line, we wish to switch to 0 to 360 longitude representation. 
; 
lf.lngswitchflag= 0 ; Set to zero.  This means that the
                    ; assumed longitude representation is 0 to 360
		    
minlng=min([lf.first_near_long, lf.first_mid_long, lf.first_far_long, $
            lf.last_near_long,  lf.last_mid_long,  lf.last_far_long])
	    
w1= where( lf.first_longs ne 0, count1 )
w2= where( lf.last_longs  ne 0, count2 )
if count1 gt 0 then minlng= min( [minlng, lf.first_longs[w1]] )
if count2 gt 0 then minlng= min( [minlng, lf.last_longs[w2] ] )

maxlng=max([lf.first_near_long, lf.first_mid_long, lf.first_far_long, $
            lf.last_near_long,  lf.last_mid_long,  lf.last_far_long])
	    
w1= where( lf.first_longs ne 0, count1 )
w2= where( lf.last_longs  ne 0, count2 )
if count1 gt 0 then maxlng= max( [minlng, lf.first_longs[w1]] )
if count2 gt 0 then maxlng= max( [minlng, lf.last_longs[w2] ] )

if abs( maxlng ) gt 90 and $
   abs( minlng ) gt 90 then begin
   sgnmin= sign( minlng )
   sgnmax= sign( maxlng )
   if sgnmin lt 0 and sgnmax gt 0 then begin
     ;
     ; The data cross the International Date Line
     ; Convert all the longitudes to a 0-360
     ; coordinate system.
     ;
     lf.lngswitchflag=1
     ;
     ;
     if lf.first_near_long lt 0 then lf.first_near_long= lf.first_near_long + 360.
     if lf.first_mid_long  lt 0 then lf.first_mid_long=  lf.first_mid_long  + 360.
     if lf.first_far_long  lt 0 then lf.first_far_long=  lf.first_far_long  + 360.
     if lf.last_near_long  lt 0 then lf.last_near_long=  lf.last_near_long  + 360.
     if lf.last_mid_long   lt 0 then lf.last_mid_long=   lf.last_mid_long   + 360.
     if lf.last_far_long   lt 0 then lf.last_far_long=   lf.last_far_long   + 360.
     w= where( lf.first_longs lt 0, count )
     if count gt 0 then lf.first_longs[w]= lf.first_longs[w] + 360.
     w= where( lf.last_longs  lt 0, count )
     if count gt 0 then lf.last_longs[w]=  lf.last_longs[w] + 360.
     ;
     if printflag then begin
       print, ''
       print, '--------------------------------------------'
       print, 'The data cross the International Date line. '
       print, 'Switch to 0 360 longitude range.'
       hlp,  lf.first_longs[w1]
       hlp,  lf.last_longs[w2]
       print, '--------------------------------------------'
       print, ''
     endif
   endif 
endif

lf.center_lon=mean( [lf.first_near_long, lf.first_far_long,  lf.last_near_long,  lf.last_far_long]  )
lf.center_lat=mean( [lf.first_near_lat,  lf.first_far_lat,   lf.last_near_lat,   lf.last_far_lat ]  )

;
; Fill in ASF style leader information.  This will insure that 
; the proper values will be printed out in the text annotation 
; file.
;
lf.near_start_lat= lf.first_near_lat
lf.near_start_lon= lf.first_near_long
lf.near_end_lat=   lf.last_near_lat
lf.near_end_lon=   lf.last_near_long
lf.far_start_lat=  lf.first_far_lat
lf.far_start_lon=  lf.first_far_long
lf.far_end_lat=    lf.last_far_lat
lf.far_end_lon=    lf.last_far_long
;
; Fill in corner_ll structure
;
lf.corner_ll[0]= lf.first_near_lat
lf.corner_ll[1]= lf.first_near_long
lf.corner_ll[2]= lf.last_near_lat
lf.corner_ll[3]= lf.last_near_long
lf.corner_ll[4]= lf.first_far_lat
lf.corner_ll[5]= lf.first_far_long
lf.corner_ll[6]= lf.last_far_lat
lf.corner_ll[7]= lf.last_far_long

;lf.lngswitchflag = 0
;asf_check4_180, lf
;if lf.lngswitchflag eq 1 then  begin
;   if printflag eq 1 then print,'Changing longitude range...'
;   if lf.corner_ll[1]    lt 0 then lf.corner_ll[1]=   360+lf.corner_ll[1]
;   if lf.corner_ll[3]    lt 0 then lf.corner_ll[3]=   360+lf.corner_ll[3]
;   if lf.corner_ll[5]    lt 0 then lf.corner_ll[5]=   360+lf.corner_ll[5]
;   if lf.corner_ll[7]    lt 0 then lf.corner_ll[7]=   360+lf.corner_ll[7]
;   if lf.near_start_lon  lt 0 then lf.near_start_lon= 360+lf.near_start_lon
;   if lf.near_end_lon    lt 0 then lf.near_end_lon=   360+lf.near_end_lon
;   if lf.far_start_lon   lt 0 then lf.far_start_lon=  360+lf.far_start_lon
;   if lf.far_end_lon     lt 0 then lf.far_end_lon=    360+lf.far_end_lon
;   if lf.plat_long       lt 0 then lf.plat_long=      360+lf.plat_long
;   if lf.center_lon      lt 0 then lf.center_lon=     360+lf.center_lon
;   if lf.first_near_long lt 0 then lf.first_near_long=360+lf.first_near_long
;   if lf.last_near_long  lt 0 then lf.last_near_long= 360+lf.last_near_long 
;   if lf.first_far_long  lt 0 then lf.first_far_long= 360+lf.first_far_long 
;   if lf.last_far_long   lt 0 then lf.last_far_long=  360+lf.last_far_long
;   numb= n_elements(lf.first_longs)
;   for ii=0, numb-1 do begin
;     if lf.first_longs[ii] lt 0 then lf.first_longs[ii]= 360+lf.first_longs[ii] 
;     if lf.last_longs[ii]  lt 0 then lf.last_longs[ii]=  360+lf.last_longs[ii] 
;   endfor
;endif



;
; In May of 2003, we discovered some ENVISAT data where the indices did
; increase monotonically.  At first the values increased, then they
; restarted at 1.  For example, the line_numbers be:
; 1, 501, 1001, 1501, ... 5001, 1,  501, 1001, ...
; We believe this is caused by ESA tacking one image upon
; another.  The next part of this code is to detect this situcaoint
w=where(lf.line_num gt 0, count)
if count gt 2 then begin
 dline=  lf.line_num[w[1]] - lf.line_num[w[0]]
 for i=2, count-1 do begin
   ;
   ; Must convert to floating point to subtract unsigned  values
   ;
   new_dline= float(lf.line_num[w[i]]) - float(lf.line_num[w[i-1]])
   ;
   if new_dline lt 0 then begin ; Discontinuity. Begin fix
     for j=i, count-1 do begin
       lf.line_num[w[j]]= dline + lf.line_num[w[j-1]] 
     endfor ; End loop to correct discontinuity.  (j) 	 
   endif    ; End discontinuity check
  endfor    ; End loop looking for discontinuity. (i)
endif       ; End check of count > 2

end
