;-----------------------------------------------------
;+
; NAME: ENV_WSM_MAIN_PRODUCT_HEADER_READ.PRO
; PURPOSE:
;   The purpose of this program is read the main product
;   header information into leader structure
; CALLING SEQUENCE:
;   env_wsm_main_product_header_read, filename, lf, /help, /quiet
; INPUT:
;  filename:  Filename of ENVISAT file in wide swath medium
;             resolution mode.
;  lf:        Structure to be filled
;             This structure can be generated using program
;             env_wsm_create_lf
; OUTPUT:
;  lf.product:             Product file name
;  lf.productType:         Product typer
;  lf.proc_stage:          Processing Stage Flag
;                          N = Near Real Time, 
;                          T = test product, 
;                          V= fully validated product, 
;                          S = special product. 
;  lf.ref_doc:             Reference Document Describing Product
;  lf.acquistion_station:  Acquisition Station ID
;  lf.proc_cent:           Processing Center ID
;  lf.proc_time:           UTC Time of Processing
;  lf.software_ver:        Software Version
;  lf.sensing_start:       UTC start time of data sensing
;  lf.sensing_stop:        UTC stop  time of data sensing
;  lf.abs_orbit:           Start absolute orbit number
;  lf.state_vector_time:   UTC of ENVISAT state vector.  
;  lf.x_position:          X Position in Earth-Fixed reference (m)
;  lf.y_position:          Y Position in Earth-Fixed reference (m)
;  lf.z_position:          Z Position in Earth-Fixed reference (m)
;  lf.s_c_altitude         Altitude above Earth (m)
;  lf.x_velocity:          X velocity in Earth fixed reference (m/s) 
;  lf.y_velocity:          Y velocity in Earth fixed reference (m/s)
;  lf.z_velocity:          Z velocity in Earth fixed reference (m/s)                                   
;  lf.vector_source:       Source of orbit vectors
;  lf.jsbegin              Begin sensing in julian seconds
;  lf.jsend                End  sensing in julian seconds
;
; MODIFICATION HISTORY:
;  2003 Mar 25  Frank Monaldo.
;  2010 Jan 13  Frank Monaldo.  Make lf.proc_fac=lf.proc_cent
;  2010 Jun 14  Frank Monaldo. Print help with no input
;  2011 Feb 07  Frank Monaldo. Add error return
;  2011 Feb 07  Frank Monaldo. Add error catching
;  2011 May 27  Frank Monaldo. Add jsbegin and jsend
;  2011 Oct 17  Frank Monaldo. Add s_c_altitude
;  2011 Oct 31  Frank Monaldo. Correct for bad stop times. 
;  2013 Feb 01  Frank Monaldo. Print out better error messages
; Copyright (C) 2002-2012, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_wsm_main_product_header_read, filename, lf, error=error, $
                    help=help, quiet=quiet
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ENV_WSM_MAIN_PRODUCT_HEADER_READ.PRO
  print, ' PURPOSE:
  print, '   The purpose of this program is read the main product
  print, '   header information into leader structure
  print, ' CALLING SEQUENCE:
  print, '   env_wsm_main_product_header_read, filename, lf, /help, /quiet
  print, ' INPUT:
  print, '  filename:  Filename of ENVISAT file in wide swath medium
  print, '             resolution mode.
  print, '  lf:        Structure to be filled.
  print, '             This structure can be generated using program
  print, '             env_wsm_create_lf
  print, ' OUTPUT:
  print, '  lf.product:             Product file name
  print, '  lf.productTypre:        Product Typer
  print, '  lf.scansar:             1 if in SCANSAR mode
  print, '  lf.proc_stage:          Processing Stage Flag
  print, '                          N = Near Real Time, 
  print, '                          T = test product, 
  print, '                          V= fully validated product, 
  print, '                          S = special product. 
  print, '  lf.ref_doc:             Reference Document Describing Product
  print, '  lf.acquistion_station:  Acquisition Station ID
  print, '  lf.proc_cent:           Processing Center ID
  print, '  lf.proc_time:           UTC Time of Processing
  print, '  lf.software_ver:        Software Version
  print, '  lf.sensing_start:       UTC start time of data sensing
  print, '  lf.sensing_stop:        UTC stop  time of data sensing
  print, '  lf.abs_orbit:           Start absolute orbit number
  print, '  lf.state_vector_time:   UTC of ENVISAT state vector.  
  print, '  lf.x_position:          X Position in Earth-Fixed reference (m)
  print, '  lf.y_position:          Y Position in Earth-Fixed reference (m)
  print, '  lf.z_position:          Z Position in Earth-Fixed reference (m)
  print, '  lf.s_c_altitude         Altitude above Earth (m)
  print, '  lf.x_velocity:          X velocity in Earth fixed reference (m/s) 
  print, '  lf.y_velocity:          Y velocity in Earth fixed reference (m/s)
  print, '  lf.z_velocity:          Z velocity in Earth fixed reference (m/s)                                  
  print, '  lf.vector_source:       Source of orbit vectors
  print, '  lf.jsbegin              Begin sensing in julian seconds
  print, '  lf.jsend                End  sensing in julian seconds
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
;
; Determine System Arch
;
arch=!version.arch

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', STRMESSAGE(abs(error_status))
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  
earthRadius=   6367           ; radius in km
equatorRadius= 6378.137       ; equatorial radius in km
polarRadius=   6356.7523142   ; polar radius in km   
;
; Reading main product header record from file.
;
if printflag then print, 'Opening '+ filename + ' to read main product header'
if file_test(filename) then begin
  close, 1
  openr, 1, filename
  mph=  bytarr(1247)
  readu, 1, mph
  close, 1
endif else begin
  if printflag then print, 'Cannot find file: ' + filename
  error=-1
  return
endelse
;
;  Reading information from mph record
;
linefeed= string( byte( 10 ) )
lf.product=               string( mph[    0:   72] )
lf.product=               getwrd( lf.product, 0, /last, del='=')
lf.product=               strtrm2( lf.product, 2, linefeed)
lf.product=               strcompress( lf.product )
lf.product=               strtrm2( lf.product, 2, '"')
lf.product=               strcompress( lf.product )
lf.datatake_id=           lf.product
lf.productType=           lf.productType
;
; SCANSAR
; 

res=strpos(lf.product, 'WSM')
if res eq -1 then lf.scansar=0
;
lf.product_stage=         string( mph[   73:   85] )
lf.product_stage=         getwrd( lf.product_stage, 0, /last, del='=')
lf.product_stage=         strtrm2( lf.product_stage, 2, linefeed)
lf.product_stage=         strcompress( lf.product_stage )
lf.product_stage=         strtrm2( lf.product_stage, 2, '"')
lf.product_stage=         strcompress( lf.product_stage )
;
lf.ref_doc=               string( mph[   86:  119] )
lf.ref_doc=               getwrd( lf.ref_doc, 0, /last, del='=')
lf.ref_doc=               strtrm2( lf.ref_doc, 2, linefeed)
lf.ref_doc=               strcompress( lf.ref_doc )
lf.ref_doc=               strtrm2( lf.ref_doc, 2, '"')
lf.ref_doc=               strcompress( lf.ref_doc )
;
lf.acquistion_station=    string( mph[  161:  203] )
lf.acquistion_station=    getwrd( lf.acquistion_station, 0, /last, del='=')
lf.acquistion_station=    strtrm2( lf.acquistion_station, 2, linefeed)
lf.acquistion_station=    strcompress( lf.acquistion_station )
lf.acquistion_station=    strtrm2( lf.acquistion_station, 2, '"')
lf.acquistion_station=    strcompress( lf.acquistion_station )
;
lf.proc_cent=             string( mph[  204:  224] )
lf.proc_cent=             getwrd( lf.proc_cent, 0, /last, del='=')
lf.proc_cent=             strtrm2( lf.proc_cent, 2, linefeed)
lf.proc_cent=             strcompress( lf.proc_cent )
lf.proc_cent=             strtrm2( lf.proc_cent, 2, '"')
lf.proc_cent=             strcompress( lf.proc_cent )
lf.proc_fac=              lf.proc_cent
;
lf.proc_time=             string( mph[  225:  264] )
lf.proc_time=             getwrd( lf.proc_time, 0, /last, del='=')
lf.proc_time=             strtrm2( lf.proc_time, 2, linefeed)
lf.proc_time=             strcompress( lf.proc_time )
lf.proc_time=             strtrm2( lf.proc_time, 2, '"')
lf.proc_time=             strcompress( lf.proc_time )
;
lf.software_ver=          string( mph[  265:  294] )
lf.software_ver=          getwrd( lf.software_ver, 0, /last, del='=')
lf.software_ver=          strtrm2( lf.software_ver, 2, linefeed)
lf.software_ver=          strcompress( lf.software_ver )
lf.software_ver=          strtrm2( lf.software_ver, 2, '"')
lf.software_ver=          strcompress( lf.software_ver )
;
lf.sensing_start=         string( mph[  336:  379] )
lf.sensing_start=         getwrd( lf.sensing_start, 0, /last, del='=')
lf.sensing_start=         strtrm2( lf.sensing_start, 2, linefeed)
lf.sensing_start=         strcompress( lf.sensing_start )
lf.sensing_start=         strtrm2( lf.sensing_start, 2, '"')
lf.sensing_start=         strcompress( lf.sensing_start )
lf.jsbegin=               dt_tm_tojs( lf.sensing_start )
;
lf.sensing_stop=          string( mph[  380:  422] )
lf.sensing_stop=          getwrd( lf.sensing_stop, 0, /last, del='=')
lf.sensing_stop=          strtrm2( lf.sensing_stop, 2, linefeed)
lf.sensing_stop=          strcompress( lf.sensing_stop )
lf.sensing_stop=          strtrm2( lf.sensing_stop, 2, '"')
lf.sensing_stop=          strcompress( lf.sensing_stop )
lf.jsend=                 dt_tm_tojs( lf.sensing_stop )

;
; Sometimes data from MDA has a unreasonable sensing stop time
; If this number is unreasonable, we set the sensing stop time
; to the sensing end time
;

if ( lf.jsend - lf.jsbegin) gt 3600 then begin ; one hour is the max reasonable
  lf.jsend= lf.jsbegin
  lf.sensing_start= dt_tm_fromjs(double(lf.jsend),   format='Y$-0n$-0d$ h$:m$:s$') 
  lf.sensing_stop=  dt_tm_fromjs(double(lf.jsbegin), format='Y$-0n$-0d$ h$:m$:s$') 
endif


center_js_time=           (lf.jsend - lf.jsbegin)/ 2.0 + lf.jsbegin

			   
lf.data_year=             fix(dt_tm_fromjs(center_js_time,format='Y$'))
lf.center_gmt=            dt_tm_fromjs(center_js_time,format='doy$:h$:m$:s$f$')
lf.month=                 strlowcase(dt_tm_fromjs(center_js_time,format='n$'))			  
lf.day=                   fix(dt_tm_fromjs(center_js_time,format='d$'))
lf.minute=                fix(dt_tm_fromjs(center_js_time,format='m$'))
lf.hour=                  fix(dt_tm_fromjs(center_js_time,format='h$'))
lf.second=                float(dt_tm_fromjs(center_js_time,format='s$f$')) 
temp=                     string( mph[  500:  516] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
lf.abs_orbit=             float(temp)

;
lf.state_vector_time=     string( mph[  517:  564] )
lf.state_vector_time=     getwrd( lf.state_vector_time, 0, /last, del='=')
lf.state_vector_time=     strtrm2( lf.state_vector_time, 2, linefeed)
lf.state_vector_time=     strcompress( lf.state_vector_time )
lf.state_vector_time=     strtrm2( lf.state_vector_time, 2, '"')
lf.state_vector_time=     strcompress( lf.state_vector_time )
;
temp=                     string( mph[  587:  613] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 3)
lf.x_position=            double(temp)
;
temp=                     string( mph[  614:  640] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 3)
lf.y_position=            double(temp)
;
temp=                     string( mph[  641:  667] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp  )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp  )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 3)
lf.z_position=            double(temp)
;
; Compute space craft altitude
;
rsat=     sqrt(lf.x_position^2 + lf.y_position^2 + lf.z_position^2)
sin_lat=  lf.z_position / rsat
sin_lat2= sin_lat * sin_lat
cos_lat=  sqrt(lf.x_position^2 + lf.y_position^2) / rsat
cos_lat2= cos_lat * cos_lat
a=        equatorradius
b=        polarradius
rearth=   a* sqrt(b^4/a^4 * sin_lat2 + cos_lat2)
rearth=   rearth / sqrt( 1 - (a - b^2/a^2) * sin_lat2 )
rearth=   rearth * 1000.00  ; Convert from kilometers to meters
lf.s_c_altitude = rsat - rearth

;
temp=                     string( mph[  668:  696] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 5)
lf.x_velocity=            double(temp)
;
temp=                     string( mph[  697:  725] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 5)
lf.y_velocity=            double(temp)
;
temp=                     string( mph[  726:  754] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 5)
lf.z_velocity=            double(temp)
;
lf.vector_source=         string( mph[  755:  773] )
lf.vector_source=         getwrd( lf.vector_source, 0, /last, del='=')
lf.vector_source=         strtrm2( lf.vector_source, 2, linefeed)
lf.vector_source=         strcompress( lf.vector_source  )
lf.vector_source=         strtrm2( lf.vector_source, 2, '"')
lf.vector_source=         strcompress( lf.vector_source  )
;
;
temp=                     string( mph[ 1066: 1103] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 7)
lf.tot_size=              long(temp)
;
temp=                     string( mph[ 1104: 1131] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 7)
lf.sph_size=              long(temp)
;
temp=                     string( mph[ 1132: 1151] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
lf.num_dsd=               long(temp)
;
temp=                     string( mph[ 1152: 1179] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
p=                        strpos(temp,'<')
if p gt -1 then  temp= strep(temp,'D',p, 7)
lf.dsd_size=              long(temp)
;
temp=                     string( mph[ 1180: 1205] )
temp=                     getwrd( temp, 0, /last, del='=')
temp=                     strcompress( temp )
temp=                     strtrm2( temp, 2, '"')
temp=                     strcompress( temp )
lf.num_data_sets=         long(temp)
;
if printflag then  begin
  print, ' '
  print, 'lf.product=           ', lf.product  	   
  print, 'lf.datatake_id=       ', lf.datatake_id
  print, 'lf.SCANSAR=           ', lf.scansar
  print, 'lf.product_stage=     ', lf.product_stage	
  print, 'lf.ref_doc=	         ', lf.ref_doc	
  print, 'lf.acquistion_station=', lf.acquistion_station
  print, 'lf.proc_cent=         ', lf.proc_cent	
  print, 'lf.proc_time=         ', lf.proc_time	
  print, 'lf.software_ver =     ', lf.software_ver
  print, 'lf.sensing_start=     ', lf.sensing_start	
  print, 'lf.sensing_stop=      ', lf.sensing_stop	
  print, 'lf.jsbegin=           ', lf.jsbegin
  print, 'lf.jsend=             ', lf.jsend
  print, 'Start acquistion time=', dt_tm_fromjs(lf.jsbegin)
  print, 'End   acquistion time=', dt_tm_fromjs(lf.jsend  )
  print, 'lf.abs_orbit=         ', lf.abs_orbit	
  print, 'lf.state_vector_time= ', lf.state_vector_time
  print, 'lf.x_position (m)=    ', lf.x_position	
  print, 'lf.y_position (m)=    ', lf.y_position	
  print, 'lf.z_position (m)=    ', lf.z_position	
  print, 'lf.s_c_altitude (m)=  ', lf.s_c_altitude
  print, 'lf.x_velocity (m/s)=  ', lf.x_velocity	
  print, 'lf.y_velocity (m/s)=  ', lf.y_velocity	
  print, 'lf.z_velocity (m/s)=  ', lf.z_velocity	
  print, 'lf.vector_source=     ', lf.vector_source
  print, 'lf.tot_size=          ', lf.tot_size
  print, 'lf.sph_size=          ', lf.sph_size	
  print, 'lf.num_dsd=           ', lf.num_dsd
  print, 'lf.dsd_size=          ', lf.dsd_size
  print, 'lf.num_data_sets=     ', lf.num_data_sets
  print, ' '
endif
end
