;-----------------------------------------------------
;+
; NAME: ENV_WSM_MDS_READ.PRO
; PURPOSE:
;   The purpose of this program is to read the SAR image
;   Warning run the programs:
;     env_wsm_main_product_header_read
;     env_wsm_specific_product_header_read
;     so that the lf structure is filled with the necessary information
; CALLING SEQUENCE:
;     env_wsm_mds_read, filename, lf, img, /help, /quiet
; INPUT:
;  filename:  Filename of ENVISAT file in wide swath medium
;             resolution mode.
;  lf:        Structure with leader information
; OUTPUT:
;  img:       Two dimensional image dat of the appropriate data type
; KEYWORDS:
;  /quiet:    Do print out information about progress
;  /help:     Print out help
;  data_set:  Data set to read 0 or 1
;  data_present_flag:  0 is returned if no data present
; MODIFICATION HISTORY:
;  2003 Mar 25 Frank Monaldo  
;  2004 Jul 30 If processed at CSTARS with Vexcel processor
;              flip the image vertically
;  2004 Sep 01 The Vexcel processor took out the flip, and so did we.
;  2004 Nov 01 remove averaging here and put it after calibration.
;  2009 Jan 09 assigning lf.nx0 properly
;  2009 Nov 25 Change the progress index in reading so that progress is indicated in 10% increments.
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2011 Feb 07 Frank Monaldo. Add error catching
; Copyright (C) 2003-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_wsm_mds_read, filename, lf, img, $
       help=help, quiet=quiet, data_set=data_set,$
       data_present_flag=data_present_flag, error=error
		   
if keyword_set(help)  or (n_params(0) lt 1 )then begin
  print, ' NAME: ENV_WSM_MDS_READ.PRO
  print, ' PURPOSE:
  print, '   The purpose of this program is to read the SAR image
  print, '   Warning run the programs:
  print, '     env_wsm_main_product_header_read
  print, '     env_wsm_specific_product_header_read
  print, '     so that the lf structure is filled with the necessary information
  print, ' CALLING SEQUENCE:
  print, '     env_wsm_mds_read, filename, lf, img, /help, /quiet
  print, ' INPUT:
  print, '  filename:  Filename of ENVISAT file in wide swath medium
  print, '             resolution mode.
  print, '  lf:        Structure with leader information
  print, ' OUTPUT:
  print, '  img:       Two dimensional image dat of the appropriate data type
  print, ' KEYWORDS:
  print, '  /quiet:    Do print out information about progress
  print, '  /help:     Print out help
  print, '  data_set:  Data set to read 0 or 1
  print, '  data_present_flag:  0 is returned if no data present
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  


if keyword_set(data_set) then begin
  data_set=data_set
endif else begin 
  data_set=0
endelse
if printflag then print, $
 'Reading MDS data ' +strcompress(string(data_set+1))
;
; Reading MDS 
;

i0=-1
for i=0, lf.num_dsd-1 do begin
  dsname= strupcase(lf.ds_name[i])
  length= strlen(dsname)
  if length ge 4 then begin
     dsname=strmid(dsname,0.4)
     if dsname eq 'MDS1' and data_set eq 0 then i0=i
     if dsname eq 'MDS2' and data_set eq 1 then i0=i
  endif
endfor
if i0 lt 0 then begin
   print,'Could not figure out where the image data start...'
   print,'Exiting'
   data_present_flag=0
   return
endif
offset=   lf.ds_offset[i0]
ny=       lf.num_dsr[i0]
nx=       lf.dsr_size[i0]
if ny eq 0 or nx eq 0 then begin
  ;
  ; The data have zero size
  ;
  data_present_flag=0
  return
endif
zero_dop= bytarr(12)
line_num= ulonarr(ny)
qflag=    bytarr(ny)
q=        byte(0)
lnum=     long(0)
case lf.data_type of
'UWORD':  begin
          nx=     (nx-17L)/2L
          img=    uintarr(nx,ny)
	  rec=    uintarr(nx)
	  reclen= 2 * nx
	  end
ELSE:     begin
          nx=     (nx-17L)/2L
          img=    uintarr(nx,ny)
	  rec=    uintarr(nx)
	  reclen= 2 * nx
	  end
endcase
; Recomputing the the number of pixel elements, Added 2009 Jan 9
lf.nx0= nx
lf.ny0= ny
if printflag then print, 'Opening '+ filename + ' to read MDS data...'
close, 1
openr, 1, filename
point_lun, 1, offset ; Skip into file
read_flag= 1
istep= ny/10
for i=0L,ny-1 do begin
  if printflag then begin
    if i mod istep eq 0 then begin
      print, 'Reading line: '+ string(i, format='(I6)') + ' of ' + $
              string(ny, format='(I6)') + ' file offset:' + strcompress(offset)
    endif
  endif
  current_byte= offset + i * lf.dsr_size[i0] + 12
  if current_byte le lf.tot_size and read_flag then begin
    readu, 1, zero_dop
  endif else begin
    read_flag=0
  endelse
  current_byte=current_byte+1
  if current_byte le lf.tot_size and read_flag then begin
    readu, 1, q
  endif else begin
    read_flag=0
  endelse
  current_byte=current_byte + 4
  if current_byte le lf.tot_size and read_flag then begin
    readu, 1, lnum
  endif else begin
    read_flag=0
  endelse
  current_byte= current_byte + reclen
  if current_byte le lf.tot_size and read_flag then begin
    readu, 1, rec
  endif else begin
    read_flag=0
  endelse 
  if read_flag then begin
    qflag[i]= q
    line_num[i]=lnum
    img[*,i]= rec
  endif
endfor
if not endian() then begin     ; If we have Intel internal machine representations
  byteorder, line_num, /lswap      ; reverse four-byte order in each 4-byte int
  byteorder, img                   ; reverse bytes in each 2-byte int
endif
close, 1

;
if printflag then begin
  print, 'Raw digital number read from Envisat files.'
  w=     where(img ne 0,count)
  if count gt 0 and count lt 2000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw Envisat data points from file=', mean(img[w])
    print, 'SD of raw Envisat data points from file=', stdev(img[w])
  endif 
endif
  
;
; If processed at CSTARS/Vexcel processor the flip the image
;

res=strpos(strupcase(lf.software_ver), 'FOCUS')
if res gt -1 then begin
  ;  print, 'Envisat image processed at CSTARS.'
  ;  print, 'Flipping image vertically'
  ;  img=reverse(img,2)
endif

end
