;-----------------------------------------------------
;+
; NAME: ENV_WSM_MPP_READ.PRO
; PURPOSE:
;   The purpose of this program is to read main processing
;   parameters information
;   Warning run the programs:
;     env_wsm_main_product_header_read
;     env_wsm_specific_product_header_read
;     so that the lf structure is filled with the necessary information
; CALLING SEQUENCE:
;     env_wsm_mpp_read, filename, lf,  /help, /quiet
; INPUT:
;  filename:  Filename of ENVISAT file in wide swath medium
;             resolution mode.
;  lf:        Structure with leader information
; OUTPUT:
;  More data filled into the lf strucuture
; KEYWODS
;  /quiet:    Do not print out intermediate results
;  /help:     Print out help
; MODIFICATION HISTORY:
;   2003 Mar 25 Frank Monaldo  
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Feb 07 Frank Monaldo. Add error catching
; Copyright (C) 2002-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro env_wsm_mpp_read, filename, lf, help=help, quiet=quiet, error=error
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: ENV_WSM_MPP_READ.PRO
  print, ' PURPOSE:
  print, '   The purpose of this program is to read main processing
  print, '   parameters information
  print, '   Warning run the programs:
  print, '     env_wsm_main_product_header_read
  print, '     env_wsm_specific_product_header_read
  print, '     so that the lf structure is filled with the necessary information
  print, ' CALLING SEQUENCE:
  print, '     env_wsm_mpp_read, filename, lf,  /help, /quiet
  print, ' INPUT:
  print, '  filename:  Filename of ENVISAT file in wide swath medium
  print, '             resolution mode.
  print, '  lf:        Structure with leader information
  print, ' OUTPUT:
  print, '  More data filled into the lf strucuture
  print, ' KEYWODS
  print, '  /quiet:    Do not print out intermediate results
  print, '  /help:     Print out help
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  


;
; Reading geolocation data from file.
;

i0=  -1
for i=0, lf.num_dsd-1 do begin
  if strupcase(lf.ds_name[i]) eq 'MAINPROCESSINGPARAMSADS' then i0=i
endfor
if i0 lt 0 then begin
   print,'Could not figure out where the main procesing parameter data start...'
   print,'Exiting'
   return
endif
offset=   lf.ds_offset[i0]

if printflag then print, 'Opening '+ filename + $
    ' to read main processing parameter data...'
close, 1
openr, 1, filename
point_lun, 1, offset ; Skip into file
mpp=  bytarr(lf.dsr_size[i0])
for i=0L, lf.num_dsr[i0]-1 do begin
  ;
  ; Read in main processing parameter record
  ;
  readu, 1, mpp
  ;
  lf.srgr=   mpp[123]
  for j=0, 1 do begin
    j1= j * 8  + 1377
    j2= j1     +    3
    lf.proc_scaling_fact[j]= float( mpp[j1   : j2  ], 0)
    lf.ext_cal_fact[j]     = float( mpp[j1+4 : j2+4], 0)
  endfor
endfor      
close, 1

if not endian() then begin ; If we have Intel internal machine representations
  lf.proc_scaling_fact= swap_endian(lf.proc_scaling_fact)
  lf.ext_cal_fact=      swap_endian(lf.ext_cal_fact)
endif

; Copy the first two elements of the array to the next two elements.
; This allows us to keep track of the original scale factors should
; we change the last ones.

lf.ext_cal_fact[2]=  lf.ext_cal_fact[0]
lf.ext_cal_fact[3]=  lf.ext_cal_fact[1]
if printflag then begin
  print, 'lf.srgr=                  ',lf.srgr
  print, 'lf.proc_scaling_fact(0)=  ',lf.proc_scaling_fact[0]
  print, 'lf.proc_scaling_fact(1)=  ',lf.proc_scaling_fact[1] 
  print, 'lf.ext_cal_fact(0)=       ',lf.ext_cal_fact[0]
  print, 'lf.ext_cal_fact(1)=       ',lf.ext_cal_fact[1]
endif
  
if lf.srgr ne 1 then lf.projection='SLANT RANGE'
end
