;-----------------------------------------------------
;+
; NAME: GAT_DN2SIGMA
; PURPOSE:
;   The purpose of this program is to convert the RADARSAT digital numbers
;   to radar cross section.  This version of the program was written to 
;   convert data as process by Gatineau. This program also averages down the
;   image by the amount  specified with data in the lf structure
; CALLING SEQUENCE:
;   gat_dn2sigma, image,  lf
; INPUT:
;     image=       Array containing the digital number image   
;     lf=          Leader information structure           
; OUTPUT:
;     image=       A floating point array of sigma.
; KEYWORDS=
;     /quiet=      Do not print intermediate results.
;     /help=       List help.
;
; MODIFICATION HISTORY:
;   2001 Mar 01 Frank Monaldo
;   2009 Sep 18 Chris Jackson. Modified the image squaring (Line 103) and finite tests (Line 141)
;             To run line by line instead of on the whole image
;             Commeneted out image statistics calculation and printing
;   2009 Dec 11 Frank Monaldo Rewrite for ANSWRS 2.0
;   2009 Dec 27 Frank Monaldo Add diagnostic output
;   2010 Jan 06 Frank Monaldo. Change to use ans_poly_cal
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Jan 02 Frank Monaldo. Pass on quiet flag
;   2013 Jan 30 Frank Monaldo. For ITU-SPF-RSAT processing facility
;                              will work, but not sure about calibration
; Copyright (C) 1998-2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro gat_dn2sigma, image, lf, help=help, quiet=quiet	   


if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: GAT_DN2SIGMA
  print, ' PURPOSE:
  print, '   The purpose of this program is to convert the RADARSAT digital numbers
  print, '   to radar cross section.  This version of the program was written to 
  print, '   convert data as process by Gatineau. This program also averages down the
  print, '   image by the amount  specified with data in the lf structure
  print, ' CALLING SEQUENCE:
  print, '   gat_dn2sigma, image,  lf
  print, ' INPUT:
  print, '     image=       Array containing the digital number image   
  print, '     lf=          Leader information structure           
  print, ' OUTPUT:
  print, '     image=       A floating point array of sigma.
  print, ' KEYWORDS=
  print, '     /quiet=      Do not print intermediate results.
  print, '     /help=       List help.
  return
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

if printflag then print, 'Converting DN to sigma for Gatineau-format CEOS data.'

sz= size(image)
nx= sz(1)
ny= sz(2)

nrange     = lf.n_samp
sample_inc = lf.samp_inc
if nrange lt 2 then begin
  nrange     = lf.n_pixel
  sample_inc = 1
endif
;
; The first thing we must do is convert the digital number to beta in
;
;  beta=     10 * log[ (DN^2 + A3) / A2 ] in dB units
;  sigma_db= beta + 10 log (sin (incidence)) in dB units
;  sigma =   10^(sigma_db/10)
;
; Resample the lookup table across the image
;
a22=interpolate(lf.lookup_tab, findgen(sample_inc*nrange)/sample_inc)
;
; Fit into horizontal dimension of the image
;
a2=     fltarr(nx)
na22=   n_elements(a22)
for i=0, nx-1 do begin
  ii= i * lf.ave_pixel
  if ii gt na22-1 then ii=na22-1
  a2[i]= a22[ii]
endfor

if printflag then begin
  print, 'Gatineau format calibration.'
  print, 'Mean  original digital number:', mean(image)
  print, 'Stdev original digital number:', stdev(image)
endif  

image= float(image) ; promote to float

incid= fltarr(nx)
nyy= fix(ny/10)
if printflag then print, 'Converting digital number to sigma.'
offset = lf.offset
a2new  = a2
if lf.proc_fac eq 'ITU-SPF_RSAT' or lf.proc_fac eq 'ITU-SPF-RSAT'  then begin
  offset = 0.0
  a2new  = abs( a2)
endif
for j=0L, ny-1 do begin  ; Loop through azimuth
  line= image[*,j]
  mean_line1=mean(line)
  line= line * line         ; square values in the image
  mean_line2=mean(line)
  ;
  ; Find values in the line of the image that are >0
  w=where(line gt 0, count)
  ;
  ; Continue processing line only if there
  ; are values >0 in line
  ;
  if count gt 0 then begin
    ;
    ; Compute the incidence angle along the line and
    ; convert to radians.
    ;
    xtemp=      dindgen(nx)/lf.ave_pixel
    ytemp=      xtemp * 0.0 + double(j)/lf.ave_line
    incid=      ans_poly_cal( xtemp, ytemp, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
    incid=      incid * !pi / 180. 
    line[w]=    10.0 * alog10( (line[w] + offset) / a2new[w] )
    line[w]=    line[w] + 10.0 * alog10( sin( incid[w] ) )
    line[w]=    10^(line[w]/10.0)
    image[w,j]= line[w]
  endif

  if j mod nyy eq 0 and printflag then  begin
     print, string(j, format='(I6)') + ' of ' + $
            string(ny, format='(I6)'), mean_line1,mean_line2, mean(line)
  endif
endfor ; End; azimuth line loop


if printflag then begin
  print, 'Gatineau format calibration.'
  print, 'Offset=          ' + strcompress( string(lf.offset)               )
  print, 'a2 max=          ' + strcompress( string(max(a2))                 )
  print, 'a2 min=          ' + strcompress( string(min(a2))                 )
  print, 'incid min (rads)=' + strcompress( string(min(incid))              )
  print, 'incid max (degs)=' + strcompress( string(max(incid))              )
  print, 'incid min (degs)=' + strcompress( string(min(incid * 180. /!pi )) )
  print, 'incid max (rads)=' + strcompress( string(max(incid * 180. /!pi )) )
  print, 'Mean  NRCS:      ' + strcompress( mean(image)                     )
  print, 'Stdev NRCS:      ' + strcompress( stdev(image)                    )
endif  

for j=0L, ny-1 do begin  ; Loop through azimuth
  line=image[*,j]
  w=where(finite(line) ne 1,count )
  if count gt 0 then line[w]=0
  image[*,j]=line
endfor ; End; azimuth line loop

;
; Average down the image. We average at this point so we can
; average cross section not digital numnber
;

nx2 = long( nx )/ lf.ave_pixel
ny2=  long( ny) / lf.ave_line
if lf.ave_pixel gt 1 or lf.ave_line gt 1 then begin
  if printflag then begin
    print, 'Averaging down the image from:'
    print, '  Pixels: ' + string(nx, format-'(I6)') + ' to ' +  string(nx2, format='(I6)')
    print, '  Lines:  ' + string(ny, format-'(I6)') + ' to ' +  string(ny2, format='(I6)')
  endif
  nxx=   nx2 * lf.ave_pixel-1L
  nyy=   ny2 * lf.ave_line-1L
  image= rebin(image[0:nxx,0:nyy], nx2, ny2)
endif else begin
  if printflag then print, 'Print no averaging of this image.'
endelse

end
