; -----------------------------------------------------
; +
; NAME: 
;   GAT_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute 
;   polynomial fit to go from pixel location to incidence angle. 
; CALLING SEQUENCE:
;    gat_incid2pix_fit, lf
; INPUTS:
;    lf:    leader information structure 
; OUTPUTS:
;    lf.incid_coef=     Array of incidence angle coefficients a function of 
;                     i,j.  The output of the fit is in degrees.
; KEYWORDS:
;    /quiet=        Suppress printed messages. 
;    /help=         Display help
;    /diagnosic     Print out fit comparisons.
; MODIFICATION HISTORY:
;   2001 Mar 01
;   2006 Sep 01 Frank Monaldo Put nominal values in or
;                             spacecraft altitude and slant range to
;                             nearest pixel if those values equal 0
;   2009 Dec 11 Frank Monaldo Rewrite for ANSWRS 2.0  
;   2010 Jan 06 Change to use ans_poly_cal
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Jun 01 Frank Monaldo. Pass on quiet flag
;   2013 Jan 26 Frank Monaldo.  If lf.n_samp lt 2 use lf.n_pixel
; Copyright (C) 2001-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro  gat_incid2pix_fit,  lf, diagnostic=diagnostic, quiet=quiet, help=help

!quiet=    1
printflag= 1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0


if keyword_set(help)  or (n_params(0) lt 1 ) then begin  
  print, ' NAME: 
  print, '   GAT_INCID2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute 
  print, '   polynomial fit to go from pixel location to incidence angle. 
  print, ' CALLING SEQUENCE:
  print, '    gat_incid2pix_fit, lf
  print, ' INPUTS:
  print, '    lf:    leader information structure 
  print, ' OUTPUTS:
  print, '    lf.incid_coef=     Array of incidence angle coefficients a function of 
  print, '                     i,j.  The output of the fit is in degrees.
  print, ' KEYWORDS:
  print, '    /quiet=        Suppress printed messages. 
  print, '    /help=         Display help
  print, '    /diagnosic     Print out fit comparisons.
  return
endif
if printflag  then begin
  print, 'Program; gat_incid2pix_fit
  print, 'Computing a fit to incid_angle as a function of image index...'
endif
rad2deg=   180./!pi
deg2rad=   1.  / rad2deg

;
; Since the Gatineau data is in range/azimuth coordinates, the incidence
; angle should really be just a function of the j-index of the image
; array.
; 
; Compute the Earth radius at the particular latitude and longitude
; in meters
if printflag then begin
  print, 'Input parameters from leader file:
  print, '  lf.plat_lat=         ',lf.plat_lat
  print, '  lf.ellip_minor=      ',lf.ellip_minor
  print, '  lf.ellip_major=      ',lf.ellip_major
  print, '  lf.eph_orb_data=     ',lf.eph_orb_data
  print, '  lf.s_c_altitude=     ',lf.s_c_altitude
endif

;
; Hard wired nominal s/c altitude and slant range if values are zero
; This is to compensate for missing data in new Gatineau data.
;




tanangle2=tan(lf.plat_lat*deg2rad)^2
re= lf.ellip_minor * sqrt( 1. + tanangle2 )
re= re/sqrt(lf.ellip_minor^2/lf.ellip_major^2+tanangle2)*1e3

lf.s_c_altitude=lf.eph_orb_data-re

error_flag=0
if lf.s_c_altitude le 0 then begin
  lf.s_c_altitude=801.9966*1e3
  if lf.number_of_beams eq 1 then lf.s_c_altitude=801.4624*1e3
  if printflag then print, 's_c_altitude forced to:',  lf.s_c_altitude 
  error_flag=1
endif
if lf.sl_rng_1st_pix le 0 then begin
  lf.sl_rng_1st_pix=849.0674*1e3
  if lf.number_of_beams eq 1 then lf.sl_rng_1st_pix=875.4624*1e3
  if printflag then print, 'lf.sl_rng_1st_pix forced to:',  lf.sl_rng_1st_pix
endif

nrange     = lf.n_samp
sample_inc = lf.samp_inc
if nrange lt 2 then begin
  nrange     = lf.n_pixel
  sample_inc = 1
endif

drg= sample_inc * lf.pix_spacing 

rg=  findgen(nrange)* drg
rg2= rg*rg
rg3= rg2*rg
rg4= rg2*rg2
rg5= rg4*rg
rs=  lf.srgr_coeff[0]      + $
     lf.srgr_coeff[1]* rg  + $
     lf.srgr_coeff[2]* rg2 + $
     lf.srgr_coeff[3]* rg3 + $
     lf.srgr_coeff[4]* rg4 + $
     lf.srgr_coeff[5]* rg5 

if total(lf.srgr_coeff) eq 0 then stop
incidence= acos( (lf.s_c_altitude^2 - rs^2 + 2*re*lf.s_c_altitude )/ 2 /rs /re)
incidence= incidence*rad2deg
if printflag then begin
  print, 'Incidence'
  hlp, incidence
endif
if error_flag then begin
  ;
  ; If I had to guess about altitude make sure that the
  ; minimum incidence is 18 degrees or larger
  ;
  min_incidence=min(incidence)
  if min_incidence lt 18 then begin
     incidence=(18-min_incidence) + incidence
  endif
  if printflag then begin
    print, 'Incidence'
    hlp, incidence
  endif
endif


irmax=       sample_inc*nrange

nincidence=interpolate(incidence,findgen(long(sample_inc * nrange))/sample_inc) 
;
incid=        dblarr(100)
ii2=          dblarr(100)  
jj2=          dblarr(100)  
irstep=       long(irmax)     / 10
ijstep=       long(lf.n_line) / 10
count_total=  -1L

for ir=0L, 9 do begin         ; Loop in azimuth
  ii= double(ir * irstep) / lf.ave_pixel
  for ij=0L, 9 do begin ; Loop in range  
    jj=  double(ij * ijstep ) / lf.ave_line 
    count_total=         count_total + 1L
    ii2[count_total]=    ii               ; Used for diagnostic
    jj2[count_total]=    jj               ; Used for diagnostic
    incid[count_total]=  nincidence[ir* irstep]
  endfor
endfor 

if printflag then print, 'Computing fit of i.j to incidence.'

ans_compute_fit, ii2, jj2, incid, lf.deg, incid_coef, incid_xexp, incid_yexp, quiet= quiet

temp_lf= create_struct( lf, 'incid_coef', incid_coef, $
                            'incid_xexp', incid_xexp, $
                            'incid_yexp', incid_yexp  )
lf=      0
lf=      temp_lf

if printflag then begin
  print, 'Incidence Coefficients'
  print, incid_coef
  hlp, incid
endif

end
