;+
; NAME: 
;   GAT_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a two/three dimensional
;   polynomial fit to go from lon/lat to pixel location and visa versa.
; CALLING SEQUENCE:
;    gat_lonlat2pix_fit, lf                       
; INPUTS:
;   lf.longitude=  Array of longitudes
;   lf.latitude=   Array of latitudes
;   lf.i_index=    Array of column indices   
;   lf.j_index=    Array of row indices
; OUTPUTS:
;   lf.i_coef:    Array of coefficients for horizontal pixel fit
;   lf.j_coef=    Array of coefficients for vertical pixel  fit
;   lf.lon_coef=  Array of longitude coefficients
;   lf.lat_coef=  Array of latitude coefficients
; KEYWORDS:
;    /quiet=    Suppress printed messages. 
;    /help=     Display help
;    /diagnostic   Print comparisions of fits
; MODIFICATION HISTORY:
;  2001 Mar 01 Frank Monaldo  
;  2009 Dec 11 Frank Monaldo  Rewrite of ANSWRS2.0
;  2010 Jan 06 Frank Monaldo. Change to use ans_compute_fit
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2011 Jan 01 Frank Monaldo. Pass on quiet flag.
; Copyright (C) 2001-2009 Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro gat_lonlat2pix_fit,lf, diagnostic=diagnostic, quiet=quiet, help=help

!quiet=   1
printflag=1
if keyword_set(quiet) then printflag=0
quiet=  printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin  
  print,' NAME: 
  print,'   GAT_LONLAT2PIX_FIT
  print,' PURPOSE:
  print,'   The purpose of this program is compute a two/three dimensional
  print,'   polynomial fit to go from lon/lat to pixel location and visa versa.
  print,' CALLING SEQUENCE:
  print,'    gat_lonlat2pix_fit, lf                       
  print,' INPUTS:
  print,'   lf.longitude=  Array of longitudes
  print,'   lf.latitude=   Array of latitudes
  print,'   lf.i_index=    Array of column indices   
  print,'   lf.j_index=    Array of row indices
  print,' OUTPUTS:
  print,'   lf.i_coef:     Array of coefficients for horizontal pixel fit
  print,'   lf.j_coef=     Array of coefficients for vertical pixel  fit
  print,'   lf.lon_coef=   Array of longitude coefficients
  print,'   lf.lat_coef=   Array of latitude coefficients
  print,' KEYWORDS:
  print,'    /quiet=       Suppress printed messages. 
  print,'    /help=        Display help
  print,'    /diagnostic   Print comparisions of fits
  return
endif

if printflag then print, 'Computing a fit to lon and lat for line of data...'

   
;
; At this point we can change longitude latitude coordinates
;
;
; We wish to find equations of the form form
;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n


ans_compute_fit, lf.longitude, lf.latitude, lf.i_index/ double(lf.ave_pixel), lf.deg, i_coef, i_xexp, i_yexp, quiet=quiet
ans_compute_fit, lf.longitude, lf.latitude, lf.j_index/ double(lf.ave_line),  lf.deg, j_coef, j_xexp, j_yexp, quiet=quiet

if keyword_set(diagnostic) then begin
  print, ' '
  print, '--------------------------------------------------------------------------'
  print, 'Longitude   Latitude   i (in)   j(in)  i(out)  j(out)    idiff    jdiff
  print, '--------------------------------------------------------------------------'
  for i=0lL, ns-1 do begin
    in=    lf.i_index[i]/ double(lf.ave_pixel)
    jn=    lf.j_index[i]/ double(lf.ave_pixel)
    ip=    ans_poly_cal(lf.longitude[i], lf.latitude[i], i_coef, i_xexp, i_yexp)
    jp=    ans_poly_cal(lf.longitude[i], lf.latitude[i], j_coef, j_xexp, j_yexp)
    print, string(lf.longitude[i], format='(F9.3)' ) + $  
           string(lf.latitude[i],  format='(F9.3)' ) + $ 
           string(in,              format='(F9.2)' ) + $
           string(in,              format='(F9.2)' ) + $
           string(ip,              format='(F9.2)' ) + $
           string(jp,              format='(F9.2)' ) + $
           string(ip-in,           format='(F8.3)' ) + $
           string(jp-jn,           format='(F8.3)' ) 
  endfor
  stop
endif
;       
; longitude latitude regression.  Longitude and latitude as a function of pixel position
;
if printflag then print, 'Computing i/j => longitude fit.'
xtemp= lf.i_index/ double(lf.ave_pixel)
ytemp= lf.j_index/ double(lf.ave_line)
ans_compute_fit,xtemp, ytemp, lf.longitude, lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
if printflag then print, 'Computing i/j => latitude fit.'
ans_compute_fit, xtemp, ytemp, lf.latitude,  lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet

if keyword_set(diagnostic) then begin
  print, ' '
  print, '--------------------------------------------------------------------------'
  print, '        i        j  lng(in)  lat(in) long(out)  lat(out)  londiff  latdiff
  print, '--------------------------------------------------------------------------'
  for i=0lL, ns-1 do begin
    in=    lf.i_index[i]/ double(lf.ave_pixel)
    jn=    lf.j_index[i]/ double(lf.ave_pixel)
    lngp=  ans_poly_cal(in, jn, lon_coef, lon_xexp, lon_yexp)
    latp=  ans_poly_cal(in, jn, lat_coef, lat_xexp, lat_yexp)
    print, string(in,                     format='(F9.2)'  ) + $
           string(jn,                     format='(F9.2)'  ) + $
           string(lf.longitude[i],        format='(F9.3)'  ) + $  
           string(lf.latitude[i],         format='(F9.3)'  ) + $ 
           string(lngp,                   format='(F9.3)'  ) + $  
           string(latp,                   format='(F9.3)'  ) + $
           string(lf.longitude[i] - lngp, format='(F11.6)' ) + $  
           string(lf.latitude[i]  - latp, format='(F11.6)' ) 
           
  endfor
  stop
endif
temp_lf= create_struct(lf, 'lon_coef', lon_coef, $
                           'lon_xexp', lon_xexp, $
                           'lon_yexp', lon_yexp, $
                           'lat_coef', lat_coef, $
                           'lat_xexp', lat_xexp, $
                           'lat_yexp', lat_yexp, $
                           'i_coef',   i_coef,   $
                           'i_xexp',   i_xexp,   $
                           'i_yexp',   i_yexp,   $
                           'j_coef',   j_coef,   $
                           'j_xexp',   j_xexp,   $
                           'j_yexp',   j_yexp    )
lf=  0
lf=  temp_lf

end

