;-----------------------------------------------------
;+
; NAME: GAT_READ_DATA_FILE.PRO
; PURPOSE:
;      The purpose of this program is to read the 
;      Gatineau data file once through to obtain
;      useful header information and information about
;      longitude and latitude in the image.  This progam
;      stores i,j,lat,lon in "outloc_file.txt" 
; CALLING SEQUENCE:
;   gat_read_data_file, datafile, image, lf
; INPUT:
;    datafile= string containing data file name.
;    lf=       leader info structure
; OUTPUT:
;    lf=       updated leader info structure
;    image=    returned image
; KEYWORD
;    /quiet    Print no intermediate output.
;    /help     Get help.
;    error=    error
; MODIFICATION HISTORY:
;   2001 Mar 01 Frank Monaldo 
;   2004 Jul 30 Add provision for reversing bytes
;               for two-byte integer data when
;               program is running on an x86 arch.
;   2009 Dec 11 Frank Monaldo  Total rewrite for ANSWRS 2.0
;   2009 Dec 27 Corrected problem so that wide swath data
;               are read as single bytes
;   2009 Dec 30 Removed computation of center lat and lon.
;               Will do that in the ans_read_gat_rsat
;               program
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Feb 05 Frank Monaldo. Add error
; Copyright (C) 2001-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
 
pro gat_read_data_file, datafile, image, lf, help=help, quiet=quiet, error=error

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: GAT_READ_DATA_FILE.PRO
  print, ' PURPOSE:
  print, '      The purpose of this program is to read the 
  print, '      Gatineau data file once through to obtain
  print, '      useful header information and information about
  print, '      longitude and latitude in the image.  This progam
  print, '      stores i,j,lat,lon in "outloc_file.txt" 
  print, ' CALLING SEQUENCE:
  print, '   gat_read_data_file, datafile, image, lf
  print, ' INPUT:
  print, '    datafile= string containing data file name.
  print, '    lf=       leader info structure
  print, ' OUTPUT:
  print, '    lf=       updated leader info structure
  print, '    image=    returned image
  print, ' KEYWORD
  print, '    /quiet    Print no intermediate output.
  print, '    /help     Get help.
  print, '    error=    error
  return
endif

error=0
printflag=1
if keyword_set(quiet) then printflag=  0 
quiet= printflag eq 0
catch, error_status  
;This statement begins the error handler
if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  or $
      error_status eq -174 then begin
      error= -1
      return
   endif
   ; handle the error by extending a:  
   catch, /cancel  
endif  
;
; Check to see if the data are compressed.
;

close, 1
openr, 1, datafile
;
; Declarations
; 
re=             6378. * 1000. ; Earth radius in meters
alt=            800   * 1000. ; Space craft altitude in meters
rad2deg=        180. / !pi    ; Conversion from radians to degrees
headerlength=   16252
lheaderlength=  192
header=         bytarr(headerlength)
lineheader=     bytarr(lheaderlength)
if not keyword_set(noheader) then readu,1, header
readu,1, lineheader

eflag=endian()  ; 0= little endian (x86)
                ; 1= big endian'

if eflag then begin
  length= long(         lineheader[8:11],  0, 1)
endif else begin
  length= long( reverse(lineheader[8:11]), 0, 1)
endelse
;
; Determine image size
;
f=fstat(1)
lf.n_line= (f.size- headerlength)/ length
lf.n_pixel= length-lheaderlength
close, 1

if printflag then begin
  print, 'Reading file:    ' + datafile
  print, 'File size:       ' + string(f.size)
  print, 'lheaderlength:   ' + string(lheaderlength)
  print, 'Number of lines: ' + string(lf.n_line)
  print, 'Numner of pixels:' + string(lf.n_pixel)
endif



openr, 1, datafile

;
; Read first line of information,  no image data included in this line.
;
readu, 1, header
;
twobyte=  0

if lf.scansar then begin ; 
; 
;  The image is one byte
; 
;  SCANSAR data stored as a single byte
  image=       bytarr(lf.n_pixel, lf.n_line)   
  image_line=  bytarr(lf.n_pixel)  
endif else begin
 lf.n_pixel=  lf.n_pixel/ 2
 if printflag then begin
   print, 'Number of pixels w/two byte samples:' + string(lf.n_pixel)
 endif 
 image=       uindgen(lf.n_pixel, lf.n_line)   
 image_line=  uindgen(lf.n_pixel)    
 twobyte=     1
endelse

nsamp= 10L
; 
; Read in image
;

iarray= lindgen(nsamp) * lf.n_line /nsamp
 
lon1=  fltarr( nsamp )
lon2=  fltarr( nsamp )
lon3=  fltarr( nsamp )
lat1=  fltarr( nsamp )
lat2=  fltarr( nsamp )
lat3=  fltarr( nsamp )
sr1=   fltarr( nsamp ) 
sr2=   fltarr( nsamp )
sr3=   fltarr( nsamp )
head=  fltarr( nsamp )
ii=0
for i=0L, lf.n_line-1L do begin 
  readu, 1, lineheader, image_line
  if twobyte and not eflag then begin
    image[*,i]= swap_endian(image_line)
  endif else begin
    image[*,i]= image_line
  endelse
  w=where( i eq iarray, count)
  if count gt 0 then begin
    if printflag then print, string(i, format='(I6)') + ' of '+ string(lf.n_line, format='(I6)') + string(mean(image[*,i]))
    if not eflag  then begin
      length=     long( reverse(lineheader[   8:   11]), 0, 1)
      line_num=   long( reverse(lineheader[  12:   15]), 0, 1)
      rec_num=    long( reverse(lineheader[  15:   19]), 0, 1)
      sr_first=   long( reverse(lineheader[  64:   67]), 0, 1)
      sr_mid=     long( reverse(lineheader[  68:   71]), 0, 1)
      sr_last=    long( reverse(lineheader[  72:   75]), 0, 1)
      lat_first=  long( reverse(lineheader[ 132:  135]), 0, 1)
      lat_mid=    long( reverse(lineheader[ 136:  139]), 0, 1)
      lat_last=   long( reverse(lineheader[ 140:  143]), 0, 1)
      long_first= long( reverse(lineheader[ 144:  147]), 0, 1)
      long_mid=   long( reverse(lineheader[ 148:  151]), 0, 1)
      long_last=  long( reverse(lineheader[ 152:  155]), 0, 1)
      heading=    long( reverse(lineheader[ 180:  183]), 0, 1)
    endif else begin
      length=     long(         lineheader[   8:   11],  0, 1)
      line_num=   long(         lineheader[  12:   15],  0, 1)
      rec_num=    long(         lineheader[  15:   19],  0, 1)
      sr_first=   long(         lineheader[  64:   67],  0, 1)
      sr_mid=     long(         lineheader[  68:   71],  0, 1)
      sr_last=    long(         lineheader[  72:   75],  0, 1)
      lat_first=  long(         lineheader[ 132:  135],  0, 1)
      lat_mid=    long(         lineheader[ 136:  139],  0, 1)
      lat_last=   long(         lineheader[ 140:  143],  0, 1)
      long_first= long(         lineheader[ 144:  147],  0, 1)
      long_mid=   long(         lineheader[ 148:  151],  0, 1)
      long_last=  long(         lineheader[ 152:  155],  0, 1)
      heading=    long(         lineheader[ 180:  183],  0, 1)
    endelse
    if ii lt nsamp then begin
      lon1[ii]=  long_first / 1e6
      lon2[ii]=  long_mid   / 1e6
      lon3[ii]=  long_last  / 1e6
      lat1[ii]=  lat_first  / 1e6
      lat2[ii]=  lat_mid    / 1e6
      lat3[ii]=  lat_last   / 1e6
      sr1[ii]=   sr_first
      sr2[ii]=   sr_mid
      sr3[ii]=   sr_last   
      head[ii]=  heading
      ii=        ii + 1
    endif
    if i eq 0 then begin
         lf.corner_ll[0]=     lat_first  / 1e6
	 lf.near_start_lat=   lf.corner_ll[0]
	 lf.corner_ll[1]=     long_first / 1e6
	 lf.near_start_lon=   lf.corner_ll[1]
	 lf.corner_ll[2]=     lat_last   / 1e6
	 lf.far_start_lat=    lf.corner_ll[2]
	 lf.corner_ll[3]=     long_last  / 1e6
	 lf.far_start_lon=    lf.corner_ll[3]
     endif
     if i eq lf.n_line-1 then begin
         lf.corner_ll[4]=     lat_first   / 1e6
	 lf.near_end_lat=     lf.corner_ll[4]
	 lf.corner_ll[5]=     long_first  / 1e6
	 lf.near_end_lon=     lf.corner_ll[5]
	 lf.corner_ll[6]=     lat_first  / 1e6
	 lf.far_end_lat=      lf.corner_ll[6]
	 lf.corner_ll[7]=     long_last / 1e6
	 lf.far_end_lon=      lf.corner_ll[7]	 
     endif
   endif   ; End check for a sample
endfor
close, 1

longitude= [lon1,   lon2,   lon3  ]
latitude=  [lat1,   lat2,   lat3  ]
j_index=   [iarray, iarray, iarray]
j1=        iarray * 0 
j2=        j1  + lf.n_pixel / 2L
j3=        j1  + lf.n_pixel - 1L
i_index=   [j1,     j2,     j3    ]
sr=        [sr1,    sr2,    sr3   ]
w= where(longitude eq 0 or latitude eq 0, count)
if count gt 0 then begin
  longitude=  longitude[w]
  latitude=   latitude[w] 
  i_index=    i_index[w]  / lf.ave_pixel     
  j_index=    j_index[w]  / lf.ave_line
  sr=         sr[w]        
endif
n=n_elements(longitude)
if printflag then begin
  for i=0L, n-1 do begin        
    print, string( i_index[i],   format='(I6)')    + ' ' +$
           string( j_index[i],   format='(I6)')    + ' ' +$
           string( longitude[i], format='(F11.2)') + ' ' +$
           string( latitude[i],  format='(F11.2)') 
  endfor
endif
;
; 
; Check as see if image crosses 180 meridian.  If so change the
; longitude range to 0 to 360.
;
lf.lngswitchflag = 0
asf_check4_180, lf, quiet=quiet
if lf.lngswitchflag then  begin
  print,'Changing longitude range...'
  if lf.corner_ll[1]   lt 0 then lf.corner_ll[1]=   360+lf.corner_ll[1]
  if lf.corner_ll[3]   lt 0 then lf.corner_ll[3]=   360+lf.corner_ll[3]
  if lf.corner_ll[5]   lt 0 then lf.corner_ll[5]=   360+lf.corner_ll[5]
  if lf.corner_ll[7]   lt 0 then lf.corner_ll[7]=   360+lf.corner_ll[7]
  if lf.near_start_lon lt 0 then lf.near_start_lon= 360+lf.near_start_lon
  if lf.near_end_lon   lt 0 then lf.near_end_lon=   360+lf.near_end_lon
  if lf.far_start_lon  lt 0 then lf.far_start_lon=  360+lf.far_start_lon
  if lf.far_end_lon    lt 0 then lf.far_end_lon=    360+lf.far_end_lon
  if lf.plat_long      lt 0 then lf.plat_long=      360+lf.plat_long
  if lf.center_lon     lt 0 then lf.center_lon=     360+lf.center_lon
endif  

temp_lf= create_struct( lf, 'longitude', longitude, $
                            'latitude',  latitude,  $
                            'i_index',   i_index,   $
                            'j_index',   j_index,   $
                            'sr',        sr         )
lf=      0
lf=      temp_lf

;
; This an gross approximation of the center_lon center_lat
; useful later for generating a filename for the output data.
;


end


