;-----------------------------------------------------
;+
; NAME: GAT_READ_LEADER
; PURPOSE:
;        The purpose of this program is read the records from the
;        leader file in Gatineau format. Note for Gatineau data what
;        we call leader file information is in the trailer file
; CALLING SEQUENCE:
;  gat_read_leader, leaderfile, good_leader, lf            
; OUTPUT:
;   good_leader: 1 if data record is good
;   	        -1: problem with data set summary record
;               -2: problem with calibration record
;               -3: problem with max facility data record
;               -4: other pronlem
;   lf:          Updated leader structure
; KEYWORDS:
;  /quiet:       Do not print intermediate output.
;  /help         Print out help. 
; MODIFICATION HISTORY:
;   2001 Mar 01 Frank Monaldo. Cnvert to read Gatineau format
;   2009 Dec 14 Frank Monaldo. Added 
;                              product_type
;                              ellipsoid
;                              orbit_num
;                              software_version
;                              range_resolution
;                              azimuth_resolution
;                              wavelength
;                              range_looks
;                              azimuth_looks
;                              ave_line
;                              ave_pixel
;                              to lf structure
;   2010 Jan 01 Added platform heading to lf structure
;   2010 Jan 01 Frank Monaldo. Print help on no  input.
;   2011 Mar 29 Frank Monaldo Add ncrs_slope and nrcs_bias to structure
;   2011 May 23 Frank Monaldo. Add jsbegin and jsend to structure
; Copyright (C) 2001-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro gat_read_leader, leaderfile, good_leader, lf,$
    quiet=quiet, help=help


if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: GAT_READ_LEADER
  print, ' PURPOSE:
  print, '        The purpose of this program is read the records from the
  print, '        leader file in Gatineau format. Note for Gatineau data what
  print, '        we call leader file information is in the trailer file
  print, ' CALLING SEQUENCE:
  print, '  gat_read_leader, leaderfile, good_leader, lf            
  print, ' OUTPUT:
  print, '   good_leader: 1 if data record is good
  print, '   	        -1: problem with data set summary record
  print, '               -2: problem with calibration record
  print, '               -3: problem with max facility data record
  print, '   lf:          Updated leader structure
  print, ' KEYWORDS:
  print, '  /quiet:       Do not print intermediate output.
  print, '  /help         Print out help. 
return
endif
good_leader=1

printflag=1
if keyword_set(quiet)   then printflag=0 

if printflag eq 1 then print, 'Reading leader records from: '+ leaderfile
catch, error_status  
;This statement begins the error handler
if error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      good_leader=-4
      return
   endif
   ; handle the error by extending a:  
   catch, /cancel  
endif  
;
; Read if the file descriptor record
;
close, 1
file_descriptor_rec= bytarr(720)
openr, 1, leaderfile
readu, 1, file_descriptor_rec
close, 1

; Checking record lengths before proceeding.
;
; Data Set Summary
;
n_data_set=long( string( file_descriptor_rec[180:185] ) ) 
l_data_set=long( string( file_descriptor_rec[186:191] ) ) 
ndata_set_summary= n_data_set * l_data_set
;
;  Map Projection
;
n_map_proj=long( string( file_descriptor_rec[192:197] ) ) 
l_map_proj=long( string( file_descriptor_rec[198:203] ) ) 
nmap_projection_data= n_map_proj * l_map_proj
;
; Platform Position
;
n_plat_pos=long( string( file_descriptor_rec[204:209] ) ) 
l_plat_pos=long( string( file_descriptor_rec[210:215] ) ) 
nplatform_position= n_plat_pos * l_plat_pos
nplatform_position= n_plat_pos * l_plat_pos
;nplatform_position= 9860L

;
; Attitude Data
;
n_att_data=long( string( file_descriptor_rec[216:221] ) ) 
l_att_data=long( string( file_descriptor_rec[222:227] ) ) 
nattitude_data= n_att_data * l_att_data
;
; Radiometric Data
;
n_radi_data=long( string( file_descriptor_rec[228:233] ) ) 
l_radi_data=long( string( file_descriptor_rec[234:239] ) ) 
nradiometric_data= n_radi_data * l_radi_data
;
; Radiometric Compensation
;
n_radi_comp=long( string( file_descriptor_rec[240:245] ) ) 
l_radi_comp=long( string( file_descriptor_rec[246:251] ) ) 
nradiometric_comp= n_radi_comp * l_radi_comp
;
;
; Data Quality Summary
;
n_qual_sum=long( string( file_descriptor_rec[253:257] ) ) 
l_qual_sum=long( string( file_descriptor_rec[258:263] ) ) 
ndata_quality_summary= n_qual_sum * l_qual_sum
;
; Signal Data Histogram
;
n_data_hist=long( string( file_descriptor_rec[264:269] ) ) 
l_data_hist=long( string( file_descriptor_rec[270:275] ) ) 
nsignal_data_histogram= n_data_hist * l_data_hist
;
; Processed Data Histogram
;
n_proc_hist=0L
l_proc_hist=0L
if n_data_hist eq 2 then begin
  n_data_hist=1L
  n_proc_hist=1L
  l_proc_hist=l_data_hist
endif
nsignal_data_histogram=    n_data_hist * l_data_hist
nprocessed_data_histogram= n_proc_hist * l_proc_hist
;
; Range Spectra
;
n_rang_spec=long( string( file_descriptor_rec[276:281] ) ) 
l_rang_spec=long( string( file_descriptor_rec[282:287] ) ) 
nrange_spectra= n_rang_spec * l_rang_spec
; 
; Detailed Processing Parameter Record
;
n_detail_proc=long( string( file_descriptor_rec[324:329] ) ) 
l_detail_proc=long( string( file_descriptor_rec[330:335] ) ) 
ndetail_proc= n_detail_proc * l_detail_proc
;
;
; Facility Data Record
;
n_fac_data=long( string( file_descriptor_rec[420:425] ) ) 
l_fac_data=long( string( file_descriptor_rec[426:431] ) ) 
nfacility_data= n_fac_data * l_fac_data
;
if printflag ne 0 then  begin
  print, 'Record                 No of Records  Length         Total bytes'
  print, '-------------------------------------------------------------'
  print, 'data set summary....', n_data_set,    l_data_set,    ndata_set_summary
  print, 'map projection data ', n_map_proj,    l_map_proj,    nmap_projection_data
  print, 'platform position...', n_plat_pos,    l_plat_pos,    nplatform_position 
  print, 'attitude data.......', n_att_data,    l_att_data,    nattitude_data
  print, 'radiometric data....', n_radi_data,   l_radi_data,   nradiometric_data
  print, 'radiometric comp....', n_radi_comp,   l_radi_comp,   nradiometric_comp  
  print, 'data quality summary', n_qual_sum,    l_qual_sum,    ndata_quality_summary
  print, 'signal data hist....', n_data_hist,   l_data_hist,   nsignal_data_histogram
  print, 'proc data hist......', n_proc_hist,   l_proc_hist,   nprocessed_data_histogram
  print, 'detail proc  param..', n_detail_proc, l_detail_proc, ndetail_proc
  print, 'range spectra.......', n_rang_spec,   l_rang_spec,   nrange_spectra
  print, 'facility data.......', n_fac_data,    l_fac_data,    nfacility_data
endif
; 
; Define structure
;
lf={file_descriptor_rec:          bytarr(720),                         $
    data_set_summary_rec:         bytarr(ndata_set_summary>1),         $
    map_projection_data_rec:      bytarr(nmap_projection_data>1),      $
    platform_position_rec:        bytarr(nplatform_position>1),        $
    attitude_data_rec:            bytarr(nattitude_data>1),            $
    radiometric_data_rec:         bytarr(nradiometric_data>1),         $
    radiometric_comp_rec:         bytarr(nradiometric_comp>1),         $
    data_quality_summary_rec:     bytarr(ndata_quality_summary>1),     $
    signal_data_histogram_rec:    bytarr(nsignal_data_histogram>1),    $
    processed_data_histogram_rec: bytarr(nprocessed_data_histogram>1), $
    range_spectra_rec:            bytarr(nrange_spectra>1),            $
    detail_proc_rec:              bytarr(ndetail_proc>1),              $
    facility_data_rec:            bytarr(nfacility_data>1),            $
    product_type: '',                         $
    defaultsfile: '', 	    	    	        $
    corner_ll: fltarr(8),     	    	    	$
    corner_ne: fltarr(8),     	    	    	$
    defaults: 'defaults.txt',	    	    	$
    asc_des: '',    	    	    	    	$
    plat_lat: 0.,  	    	    	    	$
    plat_long: 0., 	    	    	    	$
    plat_head: 0.0,                            $
    number_of_beams: 0,    	    	    	$
    beam_1_id: '',  	    	    	    	$
    cal_status: '', 	    	    	    	$
    cal_comment: '', 	    	    	    	$
    n_pixel: 0L,    	    	    	    	$
    n_line: 0L,     	    	    	    	$
    isc_dist: 0.,   	    	    	    	$
    geo_alt: 0.,    	    	    	    	$
    semi_major: 0., 	    	    	    	$
    semi_minor: 0., 	    	    	    	$
    projection: '', 	    	    	    	$
    pixel_spacing: 0.,	    	    	    	$
    line_spacing: 0.,	    	    	    	$
    pos_vector: fltarr(3),  	    	    	$
    vel_vector: fltarr(3),  	    	    	$
    datatake_id: '',	    	    	    	$
    image_id: '',   	    	    	    	$
    data_year: 0.,  	    	    	    	$
    center_gmt: '', 	    	    	    	$
    center_lat: 0., 	    	    	    	$
    center_lon: 0., 	    	    	    	$
    near_start_lat: 0.,     	    	    	$
    near_start_lon: 0.,     	    	    	$
    near_end_lat: 0.,	    	    	    	$
    near_end_lon:0.,	    	    	    	$
    far_start_lat: 0.,	    	    	    	$
    far_start_lon: 0.,	    	    	    	$
    far_end_lat: 0.,	    	    	    	$
    far_end_lon: 0.,	    	    	    	$
    total_pixels: 0.,	    	    	    	$
    total_lines: 0.,	    	    	    	$
    s_c_altitude: 0.,	    	    	    	$
    nadir_radius: 0.,	    	    	    	$
    image_radius: 0.,	    	    	    	$
    looks_azimuth: 0.,	    	    	    	$
    looks_range: 0.,	    	    	    	$
    sl_rng_1st_pix: 0.,     	    	    	$
    azimuth_pixel: 0.,	    	    	    	$
    range_pixel: 0.,	    	    	    	$
    lngswitchflag: 0,  	    	    	    	$
    scansar: 1,     	    	    	    	$
    dn: 0., 	    	    	    	    	$
    hour: 0.,	    	    	    	    	$
    minute: 0.,     	    	    	    	$
    second: 0.,     	    	    	    	$
    date: '',	    	    	    	    	$
    month: '',	    	    	    	    	$
    day: '',	    	    	    	    	$
    product_id: '',                            $
    look: '',          	    	    	    	$
    ellip_major:0.0,                           $
    ellip_minor:0.0,                           $
    pix_spacing:0.0,                           $
    eph_orb_data:0.0,                          $
    srgr_coeff:dblarr(6),                      $
    n_samp:0L,                                 $
    samp_inc:0L,                               $
    lookup_tab:fltarr(512),                    $
    noise_scale:0.0,                           $
    offset:0.0,                                $
    proc_fac:'GAT',                            $
    polarization:'HH',                         $
    leaderfile:'',                             $
    rootfile:'',                               $
    range_spacing:0.0,                         $
    azimuth_spacing:0.0,                       $
    product:'',                                $   
    product_stage:'',                          $
    software_ver:'',                           $
    ref_doc:'',                                $
    acquistion_station: '',                    $
    proc_cent:'',                              $  
    proc_time:'',                              $
    sensing_start:'',                          $
    sensing_stop:'',                           $
    radar_frequency:5.3e9,                     $
    radar_band:'C',                            $
    sar_sensor:'RADARSAT-1',                   $
    nx0:0L,                                    $
    ny0:0L,                                    $
    nx1:0L,                                    $
    ny1:0L,                                    $
    ave_pixel:1L,                              $
    ave_line:1L,                               $
    name_dir:'',                               $
    deg:2,                                     $
    software_version:'',                       $
    scene_identifier:'',                       $
    ellipsoid:'',                              $
    orbit_num:'',                              $
    range_resolution:'',                       $
    azimuth_resolution:'',                     $
    wavelength:'',                             $
    range_looks:'',                            $
    azimuth_looks:'',                          $
    nrcs_slope: 1.0,                           $
    nrcs_bias:  0.0,                           $
    jsbegin:   -3155673600D,                   $   ; Default setting for start data
    jsend:     -3155673600D                    }   ; Default setting for end  data

lf.file_descriptor_rec=        file_descriptor_rec
lf.product_type=        string(file_descriptor_rec[48:63])
lf.software_version=    string(file_descriptor_rec[32:43])
lf.scene_identifier=    lf.product_type
;
;    
;
; Redeclare arrays
;
lf.data_set_summary_rec=         bytarr(ndata_set_summary>1)
lf.map_projection_data_rec=      bytarr(nmap_projection_data>1)
lf.platform_position_rec=        bytarr(nplatform_position>1)
lf.attitude_data_rec=            bytarr(nattitude_data>1)
lf.radiometric_data_rec=         bytarr(nradiometric_data>1)
lf.radiometric_comp_rec=         bytarr(nradiometric_comp>1)
lf.data_quality_summary_rec=     bytarr(ndata_quality_summary>1)
lf.signal_data_histogram_rec=    bytarr(nsignal_data_histogram>1)
lf.processed_data_histogram_rec= bytarr(nprocessed_data_histogram>1)
lf.range_spectra_rec=            bytarr(nrange_spectra>1)
lf.detail_proc_rec=              bytarr(ndetail_proc>1)
lf.facility_data_rec=            bytarr(nfacility_data>1)

close, 1
openr, 1, leaderfile
file_descriptor_rec= bytarr(720)
;
readu, 1, file_descriptor_rec
;
;
data_set_summary_rec=            bytarr(ndata_set_summary>1)
map_projection_data_rec=         bytarr(nmap_projection_data>1)
platform_position_rec=           bytarr(nplatform_position>1)
attitude_data_rec=               bytarr(nattitude_data>1)
radiometric_data_rec=            bytarr(nradiometric_data>1)
radiometric_comp_rec=            bytarr(nradiometric_data>1)
data_quality_summary_rec=        bytarr(ndata_quality_summary>1)
signal_data_histogram_rec=       bytarr(nsignal_data_histogram>1)
processed_data_histogram_rec=    bytarr(nprocessed_data_histogram>1)
range_spectra_rec=               bytarr(nrange_spectra>1)
detail_proc_rec=                 bytarr(ndetail_proc>1)
facility_data_rec=               bytarr(nfacility_data>1)

totalbytes=720L
print, ''
if ndata_set_summary         gt 0 then begin
   readu, 1, data_set_summary_rec
   totalbytes=totalbytes+ndata_set_summary
   if printflag eq 1 then begin
     print, 'Read data_set_summary_rec         ' + $
             string(ndata_set_summary,  format='(I6.6)')+  ' '+$
	     string(totalbytes,  format='(I6.6)') 
   endif
endif
if ndata_quality_summary     gt 0 then begin
   readu, 1, data_quality_summary_rec
   totalbytes=totalbytes+ndata_quality_summary
   if printflag eq 1 then begin
     print, 'Read data_quality_summary_rec     ' + $
             string(ndata_quality_summary,  format='(I6.6)')+  ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif
if nsignal_data_histogram    gt 0 then begin
  readu, 1, signal_data_histogram_rec
  totalbytes=totalbytes+nsignal_data_histogram
   if printflag eq 1 then begin
     print, 'Read signal_data_histogram_rec    ' + $
             string(nsignal_data_histogram,  format='(I6.6)')+ ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif
if nprocessed_data_histogram gt 0 then begin
  readu, 1, processed_data_histogram_rec
  totalbytes=totalbytes+nprocessed_data_histogram
   if printflag eq 1 then begin
     print, 'Read processed_data_histogram_rec ' + $
             string(nprocessed_data_histogram,  format='(I6.6)')+ ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif  
if ndetail_proc              gt 0 then begin
   readu, 1, detail_proc_rec
   totalbytes=totalbytes+ndetail_proc
   if printflag eq 1 then begin
     print, 'Read detail_proc_rec              ' + $
             string(ndetail_proc,  format='(I6.6)')+ ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif  
if nplatform_position        gt 0 then begin
   readu, 1, platform_position_rec
   totalbytes=totalbytes+nplatform_position 
   if printflag eq 1 then begin
     print, 'Read platform_position_rec        ' + $
             string(nplatform_position ,  format='(I6.6)')+ ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif  
if nattitude_data            gt 0 then begin
  readu, 1, attitude_data_rec
  totalbytes=totalbytes+nattitude_data
   if printflag eq 1 then begin
     print, 'Read attitude_data_rec            ' + $
             string(nattitude_data,  format='(I6.6)')+ ' '+$
	     string(totalbytes,  format='(I6.6)')
   endif
endif  
if nradiometric_data         gt 0 then begin
  readu, 1, radiometric_data_rec
  totalbytes=totalbytes+nradiometric_data
  if printflag eq 1 then begin
     print, 'Read radiometric_data_rec         ' + $
             string(nradiometric_data,  format='(I6.6)')+' '+ $
	     string(totalbytes,  format='(I6.6)')
   endif
endif  
if nradiometric_comp         gt 0 then begin
  readu, 1, radiometric_comp_rec
  totalbytes=totalbytes+nradiometric_comp
  if printflag eq 1 then begin
     print, 'Read radiometric_comp_rec         ' + $
             string(nradiometric_comp,  format='(I6.6)')+' '+ $
	     string(totalbytes,  format='(I6.6)')
   endif
endif    
if nmap_projection_data      gt 0 then begin
  readu, 1, map_projection_data_rec
  totalbytes=totalbytes+nmap_projection_data
  if printflag eq 1 then begin
     print, 'Read map_projection_data_rec      ' + $
             string(nmap_projection_data,  format='(I6.6)')+' '+ $
	     string(totalbytes,  format='(I6.6)')
   endif
endif   
if nrange_spectra            gt 0 then begin
  readu, 1, range_spectra_rec
  totalbytes=totalbytes+nrange_spectra
  if printflag eq 1 then begin
     print, 'Read range_spectra_rec            ' + $
             string(nrange_spectra,  format='(I6.6)')+' '+ $
	     string(totalbytes,  format='(I6.6)')
   endif
endif   
  
if nfacility_data            gt 0 then begin
  readu, 1, facility_data_rec
  totalbytes=totalbytes+nfacility_data
  if printflag eq 1 then begin
     print, 'Read facility_data_rec            ' + $
             string(nfacility_data,  format='(I6.6)')+' '+ $
	     string(totalbytes,  format='(I6.6)')
   endif
endif   
close, 1

lf.data_set_summary_rec=        data_set_summary_rec    
lf.map_projection_data_rec=     map_projection_data_rec
lf.platform_position_rec=       platform_position_rec
lf.attitude_data_rec=           attitude_data_rec
lf.radiometric_data_rec=        radiometric_data_rec
lf.radiometric_comp_rec=        radiometric_comp_rec
lf.data_quality_summary_rec=    data_quality_summary_rec
lf.signal_data_histogram_rec=   signal_data_histogram_rec
lf.processed_data_histogram_rec=processed_data_histogram_rec
lf.detail_proc_rec=             detail_proc_rec
lf.range_spectra_rec=           range_spectra_rec
lf.facility_data_rec=           facility_data_rec



gat_read_file_des, lf, quiet=quiet

if max( lf.data_set_summary_rec ) gt 0 then begin               
  gat_read_data_sum, lf, quiet=quiet
endif  else begin
  good_leader=-1
endelse	
;
; Read DATA QUALITY SUMMARY record.
;     
if max( lf.data_quality_summary_rec ) gt 0 then begin               
   gat_read_data_qual, lf, quiet=quiet
endif  else begin
  good_leader=-2
endelse
;
;
; Read FACILITY DATA record.
;     
if max( lf.facility_data_rec ) gt 0 then begin               
 asf_read_facility, lf, quiet=quiet
 lf.n_pixel= lf.total_pixels
 lf.n_line=  lf.total_lines 		
endif  else begin
 good_leader=-3
endelse	
if printflag then begin
  print, 'According to the facility data record:'
  print, 'Number of pixels:  ' + string(lf.n_pixel, format='(I6)')   
  print, 'Number of lines:  '  + string(lf.n_line,  format='(I6)') 
endif

;
; Read SIGNAL DATA HISTOGRAM
;
    
if max( signal_data_histogram_rec ) gt 0 then begin               
 gat_read_signal_data_his, lf, quiet=quiet
endif

;
; Read MAP PROJECTION RECORD
;
if max(  lf.map_projection_data_rec ) gt 0 then begin
  asf_read_map_proj, lf, quiet=quiet   
endif else begin
  ;    
  ; If there is no map projection data record, then I have a standard 
  ; mode image 
  ;
  ;lf.scansar=0   ; Set the scansar flag to standard mode
  if lf.azimuth_pixel gt 0 then lf.line_spacing=  lf.azimuth_pixel
  if lf.range_pixel   gt 0 then lf.pixel_spacing= lf.range_pixel
  if printflag then print, 'NO MAP PROJECTION DATA RECORD'
  lf.projection='GROUND RANGE'
endelse	
;
; Skip in data in Lambert projection       
;
if strcompress(lf.projection,/remove_all) eq 'LAMBERT' then good_leader=-4         

;
; DETAILED PROCESSING RECORD
;
    
if max( detail_proc_rec ) gt 0 then begin               
 gat_read_detail_proc, lf, quiet=quiet
endif 

;
; RADIOMETRIC DATA RECORD
;
if max( lf.radiometric_data_rec ) gt 0 then begin               
 gat_read_rad_data, lf, quiet=quiet
endif 
;
;=====================================================================
; --------  Completed the reading of leader information.--------------
;=====================================================================
    ;
lf.center_gmt=     repchr(lf.center_gmt,'T',' ')  ; fix bug from ASF
lf.dn=      float(strmid(lf.center_gmt,0,3))
lf.hour=    float(strmid(lf.center_gmt,4,2))
lf.minute=  float(strmid(lf.center_gmt,7,2))
lf.second=  float(strmid(lf.center_gmt,10,6))
lf.date=    ydn2date(lf.data_year, lf.dn)
lf.month=   strlowcase( getwrd( lf.date,1))
lf.day=     getwrd( lf.date,2)

end
