;----------------------------------------------------------
;+
; NAME: GAT_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out Radarsat-1 data from Gatineau into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;   gat_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=       Do not print intermediate results'
;   /help=        List help.
;   
; MODIFICATION HISTORY:
; 2007 Aug 22 Frank Monaldo 
; 2008 Sep 13 Chris Jackson  - 
;             Made changes in the Image Identification 
;             and Image Orientation block to ensure that
;             certain lines are always written out
;             Made sure PALSAR corners are written
; 2009 May 01 Frank Monaldo 
;             Corrected problem w/ print out of incident
;             angle coefficients
; 2009 Jul 27 Frank Monaldo. Added code to keep the program
;             even if the lf.scansar and lf.s_c_altitude do not exist. This
;             was required because these fields are not filled when reading
;             RSAT data
; 2009 Oct 14 Frank Monaldo. Radically re-written for new software version 
; 2009 Nov 10 Frank Monaldo. Correct bug that wrote a line 
;                            transformation rather than an element transformation
; 2009 Nov 20 Frank Monaldo. Rewrite for ANSWRS 2.0
; 2009 Dec 02 Frank Monaldo. Changed: longitude (0to-360) transform coef for element
;                            to: longitude (0to360) transform coef for element
; 2009 Dec 14 Frank Monaldo. Conversion to Gatineau format from PALSAR
; 2010 Jan 07 Frank Monaldo. Corrected output line for longitude coefficients
; 2010 Jan 14 Frank Monaldo. Convert to use ans_write_ancil_sio
; 2010 Jan 22 Frank Monaldo. Re-compute illumination and direction and relative heading
;                           in accordance with Wackermans description
; 2010 Jun 14 Frank Monaldo. Print help on no input.
; 2011 Jan 02 Frank Monaldo. Pass on quit flag.
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro gat_write_ancil_sio, filename, lf, scaling, $
                           quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1 ) then begin
  print, ' NAME: GAT_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out Radarsat-1 data from Gatineau into ancillary data ascii file. These
  print, '   data fit the SIO format
  print, ' CALLING SEQUENCE:'
  print, '   gat_write_ancil_sio, filename, lf,  scaling'
  print, ' INPUT:'
  print, '   filename=     output file with ancillary data
  print, '   lf=           leader structure'
  print, '   scaling=      scaling for data in binary file of NRCS, default=1
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  return
endif


producttype=          lf.product_type
lf.datatake_id=       lf.scene_identifier
orbit_number=         lf.orbit_num
platform_code=        'R1'          
valuetype=            'intensity'
illumination_dir=     0
heading_rel=          90
near_range_edge=      'R'
early_azimuth_edge=   'T'
pol=                  'HH'
frequency_band=       'C'
scaling=              scaling
sensor_velocity=       sqrt( total( lf.vel_vector^2 ) )
sensor_velocity=       string(sensor_velocity, format='(F15.6)' ) 
integration_time=      '-99999.99'
nom_az_res=            '-99999.99'      
nom_rg_res=            '-99999.99'      
rcvr_gain=             '-99999.99'       
proc_gain=             '-99999.99'
lf.range_pixel=        lf.pixel_spacing  
lf.azimuth_pixel=      lf.line_spacing   
range_swath=           lf.pixel_spacing   * lf.ave_pixel * lf.nx1  
azimuth_swath=         lf.line_spacing    * lf.ave_line  * lf.ny1 
slant_range_near=      lf.sl_rng_1st_pix
slant_range_center=    -9999.99
flip=                  0
beam_mode=             lf.beam_1_ID
center_incid=          ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp) 
center_lon=            ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp)
center_lat=            ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp)
ellipsoidname=         lf.ellipsoid
semimajoraxis=         float(lf.ellip_major) *1000.
semiminoraxis=         float(lf.ellip_minor) *1000.
datumshiftparameters=  [0.0, 0.0, 0.0 ]
ellipsoidscale=        '1'
sensor_velocity=       sqrt( total( lf.vel_vector * lf.vel_vector) )
deg_incid=             lf.deg
deg_rlook=             lf.deg
range_flag=            1
range_type=            'GROUND RANGE'
lf.asc_des=            strcompress(strmid(lf.asc_des, 0, 1))

lo1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
lo2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
ll2rb, lo1, la1, lo2, la2, r, b
if b gt 180 then b= b-360.0
heading= b



if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf,                                           $
                          'producttype',          producttype,          $ 
                          'orbit_number',         orbit_number,         $ 
                          'platform_code',        platform_code,        $ 
                          'valuetype',            valuetype,            $ 
                          'illumination_dir',     illumination_dir,     $ 
                          'near_range_edge',      near_range_edge,      $ 
                          'early_azimuth_edge',   early_azimuth_edge,   $ 
                          'pol',                  pol,                  $ 
                          'frequency_band',       frequency_band,       $ 
                          'scaling',              scaling,              $ 
                          'range_swath',          range_swath,          $ 
                          'azimuth_swath',        azimuth_swath,        $ 
                          'slant_range_near',     slant_range_near,     $ 
                          'slant_range_center',   slant_range_center,   $ 
                          'flip',                 flip,                 $ 
                          'beam_mode',            beam_mode,            $ 
                          'ellipsoidname',        ellipsoidname,        $ 
                          'semimajoraxis',        semimajoraxis,        $ 
                          'semiminoraxis',        semiminoraxis,        $ 
                          'datumshiftparameters', datumshiftparameters, $ 
                          'ellipsoidscale',       ellipsoidscale,       $ 
                          'sensor_velocity',      sensor_velocity,      $ 
                          'deg_incid',            deg_incid,            $ 
                          'deg_rlook',            deg_rlook,            $
                          'integration_time',     integration_time,     $
                          'nom_az_res',           nom_az_res,           $      
                          'nom_rg_res',           nom_rg_res,           $      
                          'rcvr_gain',            rcvr_gain,            $      
                          'proc_gain',            proc_gain,            $
                          'range_type',           range_type,           $
                          'range_flag',           range_flag,           $
                          'heading',              heading,              $
                          'heading_rel',          heading_rel,          $
                          'center_incid',         center_incid          ) 
                          
  lf=      0                                    
  lf=      temp_lf       
endif else begin  
  lf.producttype=          producttype             
  lf.orbit_number=         orbit_number        
  lf.platform_code=        platform_code       
  lf.valuetype=            valuetype           
  lf.illumination_dir=     illumination_dir    
  lf.near_range_edge=      near_range_edge     
  lf.early_azimuth_edge=   early_azimuth_edge  
  lf.pol=                  pol                 
  lf.frequency_band=       frequency_band      
  lf.scaling=              scaling             
  lf.range_swath=          range_swath         
  lf.azimuth_swath=        azimuth_swath       
  lf.slant_range_near=     slant_range_near    
  lf.slant_range_center=   slant_range_center  
  lf.flip=                 flip                
  lf.beam_mode=            beam_mode            
  lf.ellipsoidname=        ellipsoidname       
  lf.semimajoraxis=        semimajoraxis       
  lf.semiminoraxis=        semiminoraxis       
  lf.datumshiftparameters= datumshiftparameters 
  lf.ellipsoidscale=       ellipsoidscale      
  lf.sensor_velocity=      sensor_velocity     
  lf.deg_incid=            deg_incid           
  lf.deg_rlook=            deg_rlook 
  lf.deg_rlook=            deg_rlook
  lf.integration_time=     integration_time 
  lf.nom_az_res=           nom_az_res      
  lf.nom_rg_res=           nom_rg_res  
  lf.rcvr_gain=            rcvr_gain      
  lf.proc_gain=            proc_gain      
  lf.range_type=           range_type 
  lf.range_flag=           range_flag    
  lf.heading=              heading
  lf.heading_rel=          heading_rel
  lf.center_incid=         center_incid
endelse

;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 
  
 end
