;+---------------------------
; NAME: READ_SENTINEL
; PURPOSE:
;    The purpose of this program is to read Sentinel data and
;    create a SIO formatted binary image file, ASCII file, and
;    binary land mask or jpg file
; CALLING SEQUENCE:
;    ans_read_sen, input_dir
; INPUT:
;    input_dir  =  Sentinel data come in directories. This input is the name
;                  of such a directory. A directory might look like
;                  S1A_WV_SLC__1SSV_20120101T040741_20120101T040828_001771_000001_55D4.SAFE/
;    deffile   =   Defaults file
; OUTPUT:
;    Binary files with image and land mask data and an ASCII file with ancillary data. The
;    convention for file names is of the form:
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_image.dat (image file)
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_ancil.txt (ancillary data file)
;        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_mask.dat  (land mask)
;    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
;    second, julian seconds, center longitude, center latitude, polarization[swathnumber], frequency band,
;    file type and extension. The data
;    files include the SIO 20-byte header.  The
; KEYWORDS:
;     /quiet       = Do not print intermediate results.
;     /help        = Print out help
;     error        = error (0 if program completes or -1 if not)
;     outputroot   = Root name of SIO files
;
; MODIFICATION HISTORY
;  2013 Sep 30 Frank Monaldo. Created for ANSWRS 2.0
;  2014 May 15 Frank Monakdo. Added quiet sen_img2sigma call
;  2014 Sep 26 Frank Monaldo. Respect sio_cross_pol
;  2014 Sep 29 Frank Monaldo. Corrected a bug if outputroot variable does
;                             not exist.
;  2014 Sep 30 Frank Monaldo. Removed land mask generation.  This is now ans_sat2sio
; Copyright (C) 1999-2013, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;

pro make_s1_quicklook,  input_dir,    $
    quiet       = quiet           , $
    help        = help            , $
    outputroot  = outputroot      , $
    error       = error
    
  error=     0
  if keyword_set(help) or (n_params(0) lt 1) then begin
    print, ' NAME: ANS_READ_SEN
    print, ' PURPOSE:
    print, '    The purpose of this program is to read Sentinel data and
    print, '    create a SIO formatted binary image file, ASCII file, and
    print, '    binary land mask
    print, ' CALLING SEQUENCE:
    print, '    ans_read_sen, input_dir
    print, ' INPUT:
    print, '    input_dir  =  Sentinel data come in directories. This input is the name
    print, '                  of such a directory. A directory might look like
    print, '                  S1A_WV_SLC__1SSV_20120101T040741_20120101T040828_001771_000001_55D4.SAFE/
    print, '
    print, ' OUTPUT:
    print, '    Binary files with image and land mask data and an ASCII file with ancillary data. The
    print, '    convention for file names is of the form:
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_image.dat (image file)
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_ancil.txt (ancillary data file)
    print, '        S1A_STS_2009_10_11_14_02_35_0308584955_129.12W_33.43N_HH[001]_C_mask.dat  (land mask)
    print, '    The file name is delimited by _ and the entries are sensor, ground stationyear, month, day, hour, minute
    print, '    second, julian seconds, center longitude, center latitude, polarization[swathnumber], frequency band,
    print, '    file type and extension. The data
    print, '    files include the SIO 20-byte header.  The
    print, ' KEYWORDS:
    print, '     /quiet       = Do not print intermediate results.
    print, '     /help        = Print out help
    print, '     error        = error (0 if program completes or -1 if not)
    print, '     outputroot   = Root name of SIO files
    print, '
    error= -1
    return
  endif
  
  !quiet=1
  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  error=1
  time0 = dt_tm_tojs(systime(/utc))
  ;
  
  ; Determine if input directory exists
  ;
  
  ff= file_search( input_dir, /test_directory, count=count)
  
  if count eq 0 then begin
    print, 'Could not find Sentinel directory: ' + input_dir
    print, 'Exiting read_sentinel.pro '
    error= -1
    return
  endif
  
  ;
  ; Find the s*.xml file and the base_directory for processing, ff= full path to product.xml file
  ;
  ;
  
  time1 = dt_tm_tojs(systime(/utc))
  
  
  ;
  ; Locating leader file
  ;
  xfiles=   file_search( input_dir, 's*xml', count=data_setmax)
  if data_setmax eq 0 then begin
    print, 'Cannot file xml data in: ' + input_dir
    print, 'Exiting ans_read_sen.pro'
    error = -1
    return
  endif
  ;
  ;
  ;Find either the HH or the VV  ;

  ;
  root_file = getwrd( input_dir, 0, /last, del='/')
  corename  = root_file
  corename  = strlowcase(strmid(corename,0,3)+'-'+strmid(corename,4,2)+'-'+strmid(corename,7,3)+'-'+strmid(corename,15,1)+strmid(corename,15,1))
  base_directory = input_dir
  len       = strlen(root_file)
 
  ;
  ; Set polflag to decide whether to create SIO files
  ; of cross-pol data
  ;
  
  ;
  ; Read and calibrate image file
  ;
  imgfile       = file_search( base_directory, corename + '*.tiff', count= count)
  if count eq 0 then begin
    print, 'Cannot find image file ' + corename + '*.tiff'
    error = -1
    return
  endif
  calfile         = file_search( base_directory, 'calibration*' + corename + '*', count= count)
  if count eq 0 then begin
    print, 'Cannot find calibration file ' + 'calibration*' + corename + '*'
    error = -1
    return
  endif
  
  sen_img2sigma, imgfile, calfile, img, quiet=quiet
;
; Adjust NRCS if necessary
;
;im = image(sqrt(img), min_value = 0, max_value = 1,/buffer)
im = image(sqrt(img), min_value = 0.01, max_value = 0.32,/buffer)
im.Save, strmid(root_file, 0, len-4 )+'png'
  
  
end
