;-----------------------------------------------------
;+
; NAME: PSAR_DN2SIGMA'
; PURPOSE:'
;   The purpose of this program is to convert the PALSAR digital numbers'
;   to radar cross section. '
; CALLING SEQUENCE:'
;  psar_dn2sigma, image,lf' 
; INPUT:'
;     image=       Array containing the digital number image '
;     lf=          Leader information structure  '         
; OUTPUT:'
;     image=       A floating point array of sigma.'
; KEYWORDS:
;     /quiet=      Do not print intermediate results.
;     /help=       List help.
; MODIFICATION HISTORY:
;  2005 Nov 30 Frank Monaldo 
;  2009 Nov 19 Rewrite for ANSWRS 2.0
;  2010 Jan 01 Frank Monaldo. Print help on no input
;  2011 Jan 01 Frank Monaldo. Pass on quiet flag.
; Copyright (C) 2006-2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro psar_dn2sigma, image,lf,  help=help, quiet=quiet	   

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: PSAR_DN2SIGMA'
  print, ' PURPOSE:'
  print, '   The purpose of this program is to convert the PALSAR digital numbers'
  print, '   to radar cross section. '
  print, ' CALLING SEQUENCE:'
  print, '  psar_dn2sigma, image,lf' 
  print, ' INPUT:'
  print, '     image=       Array containing the digital number image '
  print, '     lf=          Leader information structure  '         
  print, ' OUTPUT:'
  print, '     image=       A floating point array of sigma.'
  print, ' KEYWORDS:
  print, '     /quiet=      Do not print intermediate results.
  print, '     /help=       List help.
  return
endif
printflag= 1 
if keyword_set(quiet)          then printflag=    0 
quiet= printflag eq 0

ave_pixel= lf.ave_pixel
ave_line=  lf.ave_line

sz=    size(image)
nx=    sz[1]
ny=    sz[2]
nyy=   1000
;
; Converting from integer to float
;
if printflag then begin
  print, 'Image size= ' + strcompress( string(nx)),  ', ' + strcompress(string(ny))
  print, 'Converting from integer to float..
endif
image= float(image)

if printflag then begin 
  print, '---------------------------------------------------------'
  print, 'PALSAR CALIBRATION: Converting digital number to sigma.'
  print, '---------------------------------------------------------'
endif

if ny lt 10000 then begin
  w=     where(image ne 0,count)
  if printflag then print, 'Image values before squaring. Data of lower' 
  if count gt 0 and count lt 4000000 and printflag then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw PALSAR data points from file=', mean(image[w])
    print, 'SD   of raw PALSAR data points from file=', stdev(image[w])
  endif 
endif else begin
  nx1= min([499, nx])
  ny1= min([499, ny])
  image2= image[0:nx1-1,0:ny1-1]
  w=     where(image2 ne 0 and finite(image2) eq 1,count)
  if printflag then print, 'Image values before squaring. Data of lower' + $
         strcompress(string(nx1+1))                    + 'x' +$
         strcompress(string(ny1+1))                    + ' corner '
  if count gt 0 and printflag then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw PALSAR data points from file=', mean(image2[w])
    print, 'SD   of raw PALSAR data points from file=', stdev(image2[w])
  endif else begin
    if printflag then print, 'There are no no-zero points in this corner
  endelse
endelse

if printflag then print,'Squaring image line by line...'
jstep = ny / 10
for j=0L, ny-1 do begin
  if j mod jstep eq 0 and printflag then print, 'Squaring azimuth line ', j
  image[*,j]= image[*,j] * image[*,j]
endfor

if printflag and ny lt 10000 then begin
  print, 'Image values after squaring:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw PALSAR data points from file=', mean(image[w])
    print, 'SD   of raw PALSAR data points from file=', stdev(image[w])
  endif 
endif else begin
  nx1= min([499, nx])
  ny1= min([499, ny])
  if printflag then print, 'Image values after squaring. Data of lower'  + $
         strcompress(string(nx1+1))                    + 'x' +$
         strcompress(string(ny1+1))                    + ' corner '
  if count gt 0 and printflag then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of raw PALSAR data points from file=', mean(image2[w])
    print, 'SD   of raw PALSAR data points from file=', stdev(image2[w])
  endif else begin
    if printflag then print, 'There are no no-zero points in this corner
  endelse
endelse
  

if printflag then print, 'Using lf.ext_cal_fact[0]=                ',lf.ext_cal_fact[0] 

nyy=ny/10
for j=0L, ny-1 do begin    ; Loop through azimuth index
  line=image[*,j]
  ;
  ;Find values in the line of the image that are >0 and finite
  w=where(line gt 0 and finite(line) eq 1, count)
  if j mod nyy eq 0 and printflag then  begin
     if printflag then print, 'Calibrating line ' + string(j, for='(I6)') + ' of ' + string(ny, for='(I6)')
  endif
  ;
  ; Continue processing line only if there
  ; are values >0 in line
  ;
  if count gt 0 then begin
    image[w,j]=10.0 * alog10( line[w] ) + lf.ext_cal_fact[w]
    image[w,j]=10^(image[w,j] / 10.0)
  endif
  w=where(finite(image[*,j]) ne 1, count)
  if count gt 0 then image[w,j]=0.0
endfor   ; End azimuth line loop

if printflag then begin
  print, 'Computing statistics from unaveraged array
  print, 'External calibration factor=' +string(lf.ext_cal_fact[0])
  sz= size(image)
  if sz[4] lt 4e6 then begin
    print, 'Mean of NRCS=' + strcompress(string(mean(image)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(image))))
    w= where(image gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(image[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(image[w]))))
    endif
  endif else begin
    print, 'Number of array elements = ' +string(sz[4]) + ' Too large. To compute full statistics.'
    print, 'Computing statistics from first 500 lines'
    subimage= image[*,0:499]
    print, 'Mean of NRCS=' + strcompress(string(mean(subimage)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(subimage))))
    w= where(subimage gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(subimage[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(subimage[w]))))
    endif
  endelse
endif 


if printflag then print, 'Averaging calibrated NRCS...'
;
;Rebinning 
;
;
sz= size(image)
nx=     sz[1]
ny=     sz[2]
lf.nx0= nx
lf.ny0= ny
if ave_pixel gt 1 or ave_line gt 1 then begin
  nx1=  long(nx)  / long( ave_pixel ) 
  ny1=  long(ny)  / long( ave_line  ) 
  nx2= nx1 * long( ave_pixel)
  ny2= ny1 * long( ave_line )
  output='Rebinning image from ' + string(nx) + ',' +  string(ny) + ' to '
  output=output +  string(nx1) + ', ' +  string(ny1)
  lf.nx1= nx1
  lf.ny1= ny1
  if printflag then print, strcompress(output)
  image=rebin( image[0:nx2-1,0:ny2-1], nx1, ny1)
endif
if printflag then begin
  print, 'Computing statistics from averages array
  print, 'External calibration factor=' +string(lf.ext_cal_fact[0])
  sz= size(image)
  if sz[4] lt 4e6 then begin
    print, 'Mean of NRCS=' + strcompress(string(mean(image)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(image))))
    w= where(image gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(image[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(image[w]))))
    endif
  endif else begin
    print, 'Number of array elements = ' +string(sz[4]) + ' Too large. To compute full statistics.'
    print, 'Computing statistics from first 500 lines'
    ny1= min( [499, sz(2)-1 ] )
    subimage= image[*,0:499]
    print, 'Mean of NRCS=' + strcompress(string(mean(subimage)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(subimage))))
    w= where(subimage gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(subimage[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(subimage[w]))))
    endif
  endelse
endif 

end
