;-----------------------------------------------------
;+
; NAME: psar_read_image_file'
; PURPOSE:'
;   This program reads the content of the PALSAR image file'
; CALLING SEQUENCE:'
;   psar_read_image_file, imagefile, image, lf
; INPUT:'
;   imagefile= Image file'
;   lf=        Leader file structure
; OUTPUT: following two variables added to lf structure:'
;   image=  SAR image
;   lf=     Updated lf structure
; KEYWORDS
;   /quiet       Do not print to screen'
;   /pause       Pause to show each record'
;   /force  Assume level 1.5 processed data'
;   error= error  (1 if good, -1 if bad)
; MODIFICATION HISTORY:
; 2006 Nov 28 Frank Monaldo  
; 2007 May 11 Added provision to generate the
;             polynomial relationship between
;             slant range and incident angle, if
;             such a relationship was not provided
;             in the leader file
; 2009 Nov 19 Large modeifications for ANSWRS 2.0
; 2010 Jan 05 Change to accommodate ans_compute_fit routine
; 2010 Jun 14 Frank Monaldo. Print help on no input
; 2011 Jan 01 Frank Monaldo. Pass on quiet flag
; 2011 Feb 08 Frank Monaldo. Return error=-1 if cannot read data
; Copyright (C) 2006-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  psar_read_image_file, imagefile, image, lf,   $
                           help=help, quiet=quiet, $
                           error= error,           $
                           pause=pause, force=force
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: psar_read_image_file'
  print, ' PURPOSE:'
  print, '   This program reads the content of the PALSAR image file'
  print, ' CALLING SEQUENCE:'
  print, '   psar_read_image_file, imagefile, image, lf
  print, ' INPUT:'
  print, '   imagefile= Image file'
  print, '   lf=        Leader file structure
  print, ' OUTPUT: following two variables added to lf structure:'
  print, '   image=  SAR image
  print, '   lf=     Updated lf structure
  print, ' KEYWORDS
  print, '   /quiet       Do not print to screen'
  print, '   /pause       Pause to show each record'
  print, '   /force  Assume level 1.5 processed data'
  print, '   error= error  (1 if good, -1 if bad)
  print, ' 
  error= -1
  return           
endif
printflag=1
if keyword_set(quiet)   then printflag=0 
quiet=  printflag eq 0


error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  


ave_pixel= lf.ave_pixel
ave_line=  lf.ave_line
;;
; Determine System Arch
;

eflag=endian()  ; 0= little endian (x86)
                ; 1= big endian'

;
; Compute radius of curvature
;
;
; Convert angles to radians.
;
phi_rad=   lf.sensor_platform_latitude_deg   * !pi / 180.0
sin_phi=   sin(phi_rad)
e=         sqrt( 1 - lf.semi_minor_axis_km^2 / lf.semi_major_axis_km^2 )
r_curv=    lf.semi_major_axis_km / sqrt( 1 - e * e  * sin_phi * sin_phi)





file_descriptor_rec=      bytarr(720)


; facility_related_rec to be declared...

; Open file
;
if printflag then begin
  print, '-------------------------------------------------
  print, ''
  print, 'PSAR_READ_IMAGE_FILE PROGRAM
  print, ''
  print, '-------------------------------------------------
endif
res=file_info(imagefile)
if printflag then print, 'File '+ imagefile + ' Size=' +strcompress(string(res.size)) + 'bytes'

close, 1
openr, 1, imagefile

;
;------------------------------------------
; Read file descriptor record
; -----------------------------------------
;
readu, 1, file_descriptor_rec
if printflag then print, 'Reading file descriptor record'
if eflag then begin
  record_sequence_number                     = long(         file_descriptor_rec[0:3],  0 )
endif else begin
  record_sequence_number                     = long( reverse(file_descriptor_rec[0:3]), 0 )
endelse                                                                                 
first_record_subtype_code                    =           fix(byte(file_descriptor_rec[4]))
record_type_code                             =           fix(byte(file_descriptor_rec[5]))
second_record_subtype_code                   =           fix(byte(file_descriptor_rec[6]))
third_record_subtype_code                    =           fix(byte(file_descriptor_rec[7]))
if eflag then begin
  length_of_this_record                      = long(         file_descriptor_rec[8:11], 0    ) 
endif else begin
  length_of_this_record                      = long( reverse(file_descriptor_rec[8:11]), 0   )
endelse
ascii_flag                                   =     string(   file_descriptor_rec[ 12: 13]   )
format_control_document_id                   =     string(   file_descriptor_rec[ 16: 27]   )
format_control_document_revision_level       =     string(   file_descriptor_rec[ 28: 29]   )
file_design_descriptor_revision_letter       =     string(   file_descriptor_rec[ 30: 31]   )
logical_volume_generating_facility           =     string(   file_descriptor_rec[ 32: 43]   )
file_number                                  = long( string( file_descriptor_rec[ 44: 47]   ))
referenced_file_name                         =     string(   file_descriptor_rec[ 48: 63]   )
processing_level_from_file_name              =     strmid( referenced_file_name, 7, 1 )
referenced_sequence_and_location_type_flag   =     string(   file_descriptor_rec[ 64: 67]   )
sequence_number_location                     = long( string( file_descriptor_rec[ 68: 75]   ))
sequence_number_field_length                 = long( string( file_descriptor_rec[ 76: 79]   ))
record_code_and_location_type_flag           =      string(  file_descriptor_rec[ 80: 83]   )
record_code_location                         = long( string( file_descriptor_rec[ 84: 91]   ))
record_code_field_length                     = long( string( file_descriptor_rec[ 92: 95]   ))
record_length_and_location_type_flag         =      string(  file_descriptor_rec[ 96: 99]   )
record_length_location                       = long( string( file_descriptor_rec[101:107] ))
record_length_field_length                   = long( string( file_descriptor_rec[108:111] ))
number_of_sar_data_records                   = long( string( file_descriptor_rec[180:185] ))
sar_data_record_length                       = long( string( file_descriptor_rec[186:191] ))
number_of_bits_per_sample                    = long( string( file_descriptor_rec[216:219] ))
number_of_samples_per_pixel                  = long( string( file_descriptor_rec[220:223] ))
number_of_bytes_per_pixel                    = long( string( file_descriptor_rec[224:227] ))
number_of_sar_channnels_in_this_tile         = long( string( file_descriptor_rec[232:235] ))
number_of_lines_per_data_set                 = long( string( file_descriptor_rec[236:243] ))
number_of_left_border_pixels                 = long( string( file_descriptor_rec[244:255] ))
number_of_right_border_pixels                = long( string( file_descriptor_rec[256:259] ))
number_of_top_border_pixels                  = long( string( file_descriptor_rec[260:263] ))
number_of_bottom_border_pixels               = long( string( file_descriptor_rec[264:267] ))
interleaving_factor                          =       string( file_descriptor_rec[268:271] )
number_of_physical_records_per_line          = long( string( file_descriptor_rec[272:273] ))
number_of_physical_records_per_multi         = long( string( file_descriptor_rec[274:275] ))
number_of_bytes_of_prefix_data_per_line      = long( string( file_descriptor_rec[276:279] ))
number_of_bytes_of_pixel_data_per_record     = long( string( file_descriptor_rec[280:287] )) 
number_of_bytes_of_suffix_data_per_line      = long( string( file_descriptor_rec[288:291] ))
pref_suf_flag                                =       string( file_descriptor_rec[292:295] )
sample_data_line_number_locator              =       string( file_descriptor_rec[296:303] )
sar_channel_number_locator                   =       string( file_descriptor_rec[304:311] )
time_of_sar_data_line_locator                =       string( file_descriptor_rec[312:319] )
left_fill_count_locator                      =       string( file_descriptor_rec[320:327] )
right_fill_count_locator                     =       string( file_descriptor_rec[328:335] )
pad_pixels_present_indicator                 =       string( file_descriptor_rec[336:339] )
sar_data_line_quality_code_locator           =       string( file_descriptor_rec[368:375] )
calibration_information_field_locator        =       string( file_descriptor_rec[376:383] )
gain_values_field_locator                    =       string( file_descriptor_rec[384:391] )
bais_values_field_locator                    =       string( file_descriptor_rec[392:399] )
sar_data_format_type                         =       string( file_descriptor_rec[400:427] )
sar_data_format_type_code                    =       string( file_descriptor_rec[428:431] )
number_of_left_fill_bits_in_pixel            = long( string( file_descriptor_rec[432:435] ))
number_of_right_fill_bits_in_pixel           = long( string( file_descriptor_rec[436:439] ))
maximum_data_range_of_pixel                  = 0L
if string( file_descriptor_rec[440:447] ) ne '        ' then $
         maximum_data_range_of_pixel                  = long( string( file_descriptor_rec[440:447] ))
if printflag then begin
  print, 'record_sequence_number                       = ' + string(record_sequence_number)                   
  print, 'first_record_subtype_code                    = ' + string(first_record_subtype_code)                
  print, 'record_type_code                             = ' + string(record_type_code)                         
  print, 'second_record_subtype_code                   = ' + string(second_record_subtype_code)               
  print, 'third_record_subtype_code                    = ' + string(third_record_subtype_code)                
  print, 'length_of_this_record                        = ' + string(length_of_this_record)                    
  print, 'ascii_flag                                   = ' + ascii_flag                                       
  print, 'format_control_document_id                   = ' + format_control_document_id                       
  print, 'format_control_document_revision_level       = ' + format_control_document_revision_level           
  print, 'file_design_descriptor_revision_letter       = ' + file_design_descriptor_revision_letter           
  print, 'logical_volume_generating_facility           = ' + logical_volume_generating_facility               
  print, 'file_number                                  = ' + string(file_number)                                
  print, 'referenced_file_name                         = ' + referenced_file_name           
  print, 'processing_level_from_file_name              = ' + processing_level_from_file_name
  print,   'B= level 1.1 and C= level 1.5'
  print, 'referenced_sequence_and_location_type_flag   = ' + referenced_sequence_and_location_type_flag       
  print, 'sequence_number_location                     = ' + string(sequence_number_location)                  
  print, 'sequence_number_field_length                 = ' + string(sequence_number_field_length)              
  print, 'record_code_and_location_type_flag           = ' + record_code_and_location_type_flag               
  print, 'record_code_location                         = ' + string(record_code_location)                     
  print, 'record_code_field_length                     = ' + string(record_code_field_length)                 
  print, 'record_length_and_location_type_flag         = ' + string(record_length_and_location_type_flag)     
  print, 'record_length_location                       = ' + string(record_length_location)                   
  print, 'record_length_field_length                   = ' + string(record_length_field_length)               
  print, 'number_of_sar_data_records                   = ' + string(number_of_sar_data_records)               
  print, 'sar_data_record_length                       = ' + string(sar_data_record_length)                   
  print, 'number_of_bits_per_sample                    = ' + string(number_of_bits_per_sample)                
  print, 'number_of_samples_per_pixel                  = ' + string(number_of_samples_per_pixel)              
  print, 'number_of_bytes_per_pixel                    = ' + string(number_of_bytes_per_pixel)                
  print, 'number_of_sar_channnels_in_this_tile         = ' + string(number_of_sar_channnels_in_this_tile)     
  print, 'number_of_lines_per_data_set                 = ' + string(number_of_lines_per_data_set)             
  print, 'number_of_left_border_pixels                 = ' + string(number_of_left_border_pixels)             
  print, 'number_of_right_border_pixels                = ' + string(number_of_right_border_pixels)            
  print, 'number_of_top_border_pixels                  = ' + string(number_of_top_border_pixels)              
  print, 'number_of_bottom_border_pixels               = ' + string(number_of_bottom_border_pixels)           
  print, 'interleaving_factor                          = ' + string(interleaving_factor)                      
  print, 'number_of_physical_records_per_line          = ' + string(number_of_physical_records_per_line)      
  print, 'number_of_physical_records_per_multi         = ' + string(number_of_physical_records_per_multi)     
  print, 'number_of_bytes_of_prefix_data_per_line      = ' + string(number_of_bytes_of_prefix_data_per_line)  
  print, 'number_of_bytes_of_pixel_data_per_record     = ' + string(number_of_bytes_of_pixel_data_per_record) 
  print, 'number_of_bytes_of_suffix_data_per_line      = ' + string(number_of_bytes_of_suffix_data_per_line)  
  print, 'pref_suf_flag                                = ' + pref_suf_flag                                    
  print, 'sample_data_line_number_locator              = ' + sample_data_line_number_locator                  
  print, 'sar_channel_number_locator                   = ' + sar_channel_number_locator                       
  print, 'time_of_sar_data_line_locator                = ' + time_of_sar_data_line_locator                    
  print, 'left_fill_count_locator                      = ' + left_fill_count_locator                          
  print, 'right_fill_count_locator                     = ' + right_fill_count_locator                         
  print, 'pad_pixels_present_indicator                 = ' + pad_pixels_present_indicator                     
  print, 'sar_data_line_quality_code_locator           = ' + sar_data_line_quality_code_locator               
  print, 'calibration_information_field_locator        = ' + calibration_information_field_locator            
  print, 'gain_values_field_locator                    = ' + gain_values_field_locator                        
  print, 'bais_values_field_locator                    = ' + bais_values_field_locator                        
  print, 'sar_data_format_type                         = ' + sar_data_format_type                             
  print, 'sar_data_format_type_code                    = ' + sar_data_format_type_code                        
  print, 'number_of_left_fill_bits_in_pixel            = ' + string(number_of_left_fill_bits_in_pixel)        
  print, 'number_of_right_fill_bits_in_pixel           = ' + string(number_of_right_fill_bits_in_pixel)       
  print, 'maximum_data_range_of_pixel                  = ' + string(maximum_data_range_of_pixel)              
  print, 'FINISHED READING FILE DESCRIPTOR REOCORD.'
endif 
if keyword_set(pause) then stop
parray= ['H','V']

if (processing_level_from_file_name ne 'C' or processing_level_from_file_name eq 'B') and not keyword_set(force) then begin 
  ;----------------------
  ; SIGNAL DATA RECORD
  ;----------------------
  signal_data_rec=          bytarr(sar_data_record_length)
  for i=0, number_of_lines_per_data_set-1 do begin
    readu, 1,signal_data_rec
    if eflag then begin
      record_sequence_number                     = long(         signal_data_rec[0:3],  0 )
    endif else begin
      record_sequence_number                     = long( reverse(signal_data_rec[0:3]), 0 )
    endelse
    first_record_subtype_code                    =           fix(byte(signal_data_rec[4]))
    record_type_code                             =           fix(byte(signal_data_rec[5]))
    second_record_subtype_code                   =           fix(byte(signal_data_rec[6]))
    third_record_subtype_code                    =           fix(byte(signal_data_rec[7]))
    if eflag then begin
      length_of_this_record                      = long(         signal_data_rec[8:11], 0    ) 
    endif else begin
      length_of_this_record                      = long( reverse(signal_data_rec[8:11]), 0   )
    endelse
    if printflag and (i mod 2000 eq 0 ) then begin
      print, 'record_sequence_number                       = ' + string(record_sequence_number)                   
      print, 'first_record_subtype_code                    = ' + string(first_record_subtype_code)                
      print, 'record_type_code                             = ' + string(record_type_code)                         
      print, 'second_record_subtype_code                   = ' + string(second_record_subtype_code)               
      print, 'third_record_subtype_code                    = ' + string(third_record_subtype_code)                
      print, 'length_of_this_record                        = ' + string(length_of_this_record)    
    endif
    if keyword_set(pause) then stop
  endfor
endif

if processing_level_from_file_name eq 'C'  or processing_level_from_file_name eq 'B' or keyword_set(force) then begin 
  ;----------------------
  ; PROCESSED DATA REC DATA RECORD
  ;------------------------
  if printflag then begin
    print, 'Reading -- Processed Data Record -- data record'
  endif
  header=             bytarr(192)
  if strcompress(sar_data_format_type_code, /remove_all) eq 'IU2'  or strcompress(sar_data_format_type_code, /remove_all)  eq 'C*8' then begin
    ; Data format eq two-byte unsigned integer
    read_data=1
    image                = uindgen(number_of_bytes_of_pixel_data_per_record / 2, number_of_sar_data_records)
    image_line           = uindgen(number_of_bytes_of_pixel_data_per_record / 2)
    sensor_acq_year      = ulindgen(number_of_sar_data_records)
    sensor_acq_day       = ulindgen(number_of_sar_data_records)
    sensor_acq_ms        = ulindgen(number_of_sar_data_records)
    first_latitude       = dblarr(number_of_sar_data_records)
    mid_latitude         = dblarr(number_of_sar_data_records)
    last_latitude        = dblarr(number_of_sar_data_records)
    first_longitude      = dblarr(number_of_sar_data_records)
    mid_longitude        = dblarr(number_of_sar_data_records)
    last_longitude       = dblarr(number_of_sar_data_records)
    slant_range_first_km = dblarr(number_of_sar_data_records)
    slant_range_mid_km   = dblarr(number_of_sar_data_records)
    slant_range_last_km  = dblarr(number_of_sar_data_records)
    incident_first       = dblarr(number_of_sar_data_records)
    incident_mid         = dblarr(number_of_sar_data_records)
    incident_lat         = dblarr(number_of_sar_data_records)
  endif
  if printflag then begin
    hlp, header, number_of_lines_per_data_set, image_line
    print, (192L + n_elements(image_line) *2L) * number_of_lines_per_data_set
  endif
  istep =  number_of_lines_per_data_set /10L
  for i=0, number_of_lines_per_data_set-1 do begin
    if printflag and i mod istep eq 0 then print, 'Reading image... Line: ' + string( i, format='(I6.6)')
    readu, 1, header, image_line
    if eflag then begin
      record_sequence_number   = long(         header[0:3],  0 )
    endif else begin           
      record_sequence_number   = long( reverse(header[0:3]), 0 )
    endelse                    
    first_record_subtype_code  =      fix(byte(header[4]))
    record_type_code           =      fix(byte(header[5]))
    second_record_subtype_code =      fix(byte(header[6]))
    third_record_subtype_code  =      fix(byte(header[7]))
    if eflag then begin        
      length_of_this_record    = long(         header(8:11), 0    ) 
    endif else begin           
      length_of_this_record    = long( reverse(header(8:11)), 0   )
    endelse
    if eflag then begin
      sensor_acq_year[i]      = ulong( header[  36: 39], 0 )
      sensor_acq_day[i]       = ulong( header[  40: 43], 0 )
      sensor_acq_ms[i]        = ulong( header[  44: 47], 0 )  
      sar_channel             = fix(   header[  50: 51], 0 )
      transmit_pol            = fix(   header[  52: 53], 0 )
      receive_pol             = fix(   header[  54: 55], 0 )
      lf.polarization         = parray(transmit_pol) +  parray(receive_pol)
      slant_range_first_km[i] = long(  header[  64: 67], 0 ) / double(1e3)
      slant_range_mid_km[i]   = long(  header[  68: 71], 0 ) / double(1e3)
      slant_range_last_km[i]  = long(  header[  72: 75], 0 ) / double(1e3)
      first_latitude[i]       = long(  header[132:135 ], 0 ) / double(1e6)
      mid_latitude[i]         = long(  header[136:139 ], 0 ) / double(1e6)
      last_latitude[i]        = long(  header[140:143 ], 0 ) / double(1e6)
      first_longitude[i]      = long(  header[144:147 ], 0 ) / double(1e6)
      mid_longitude[i]        = long(  header[148:151 ], 0 ) / double(1e6)
      last_longitude[i]       = long(  header[152:155 ], 0 ) / double(1e6)
    endif else begin
      sensor_acq_year[i]      = ulong( reverse(header[ 36: 39]), 0 )
      sensor_acq_day[i]       = ulong( reverse(header[ 40: 43]), 0 )
      sensor_acq_ms[i]        = ulong( reverse(header[ 44: 47]), 0 ) 
      sar_channel             = fix(   reverse(header[ 50: 51]), 0 )
      transmit_pol            = fix(   reverse(header[ 52: 53]), 0 )   
      receive_pol             = fix(   reverse(header[ 54: 55]), 0 )
      lf.polarization         = parray(transmit_pol) +  parray(receive_pol)
      slant_range_first_km[i] = long(  reverse(header[ 64: 67]), 0 ) / double(1e3)
      slant_range_mid_km[i]   = long(  reverse(header[ 68: 71]), 0 ) / double(1e3)
      slant_range_last_km[i]  = long(  reverse(header[ 72: 75]), 0 ) / double(1e3)
      first_latitude[i]       = long(  reverse(header[132:135]), 0 ) / double(1e6)
      mid_latitude[i]         = long(  reverse(header[136:139]), 0 ) / double(1e6)
      last_latitude[i]        = long(  reverse(header[140:143]), 0 ) / double(1e6)
      first_longitude[i]      = long(  reverse(header[144:147]), 0 ) / double(1e6)
      mid_longitude[i]        = long(  reverse(header[148:151]), 0 ) / double(1e6)
      first_latitude[i]       = long(  reverse(header[152:155]), 0 ) / double(1e6)
      image_line              = swap_endian(image_line)
    endelse               
    image[*,i]= image_line
    if printflag and (i mod 2000 eq 0 ) then begin
      print, 'i                                            = ' + string(i)
      print, 'sensor_acq_year                              = ' + string(sensor_acq_year[i])
      print, 'sensor_acq_day                               = ' + string(sensor_acq_day[i])       
      print, 'sensor_acq_ms                                = ' + string(sensor_acq_ms[i])        
      print, 'record_sequence_number                       = ' + string(record_sequence_number)    
      print, 'first_record_subtype_code                    = ' + string(first_record_subtype_code) 
      print, 'record_type_code                             = ' + string(record_type_code)          
      print, 'second_record_subtype_code                   = ' + string(second_record_subtype_code)
      print, 'third_record_subtype_code                    = ' + string(third_record_subtype_code) 
      print, 'length_of_this_record                        = ' + string(length_of_this_record)    
      print, 'first_latitude[i]                            = ' + string(first_latitude[i])
      print, 'mid_latitude[i]                              = ' + string(mid_latitude[i])  
      print, 'last_latitude[i]                             = ' + string(last_latitude[i]) 
      print, 'first_longitude[i]                           = ' + string(first_longitude[i])
      print, 'mid_longitude[i]                             = ' + string(mid_longitude[i]) 
      print, 'first_latitude[i]                            = ' + string(first_latitude[i])
      print, 'slant_range_first_km[i]                      = ' + string(slant_range_first_km[i])
      print, 'slant_range_mid_km [i]                       = ' + string(slant_range_mid_km [i])
      print, 'slant_range_last_km[i]                       = ' + string(slant_range_last_km[i])
      print, 'mean(image_line)                             = ' + string(mean(image_line)) 
      if keyword_set(pause) then stop
    endif
  endfor  ; End reading through lines
  sz=     size(image)
  nx=     sz[1]
  ny=     sz[2]
  lf.nx0= nx
  lf.ny0= ny
  lf.nx1= nx / ave_pixel
  lf.ny1= ny / ave_line
  ;
  ; Go from incident angle polynomial as a function of slant range to index
  ;
  nn=n_elements(slant_range_first_km)
  count=0
  incident=[0.0D]
  ii=      [0.0D]
  jj=      [0.0D]
  
  iii=0
  if printflag then  print, 'Computed incident angle and slant ranges.'
  if total( lf.incid_coefsr[0] ) eq 0 then begin
    ;
    ; The leader file did not provide a polynomial relationship between
    ; slant trand and incident angel.  We are therefor going to compute one
    ;
    inctemp= findgen(32) + 18
    rslant=  fltarr(32)
    for k=0, 31 do begin
      inct= inctemp[k]
      slant_range_incid, lf.semi_major_axis_km, lf.semi_minor_axis_km, $
                         lf.geodetic_altitude_km, lf.sensor_platform_latitude_deg, $
                         inct, r_s
      rslant(k)= r_s
    endfor
    result = POLY_FIT( rslant, inctemp, 5)
    lf.incid_coefsr[0:5]= result
    ; 
    ; Finished computing a polynomial relationship between slant range and
    ; local incident angle
    ;
  endif
  for j=0, nn-1, 200 do begin
    iii=iii+1
    for i=0, 2 do begin
      case i of
      0: ii1=0
      1: ii1=nx/2
      2: ii1=nx-1
      endcase
      count=count+1
      jj= [jj,j]
      ii= [ii,ii1]
      case i of
      0: sr= slant_range_first_km[j]
      1: sr= slant_range_mid_km[j]
      2: sr= slant_range_last_km[j]
      endcase
      if sr gt 0 then begin 
         incid= lf.incid_coefsr[0]                     +  $
                lf.incid_coefsr[1] * sr                + $
                lf.incid_coefsr[2] * sr * sr           + $
                lf.incid_coefsr[3] * sr * sr * sr      + $
                lf.incid_coefsr[4] * sr * sr * sr * sr + $
                lf.incid_coefsr[5] * sr * sr * sr * sr * sr
         if (iii mod 5) eq 0 and printflag then begin 
           slant_range_incid, lf.semi_major_axis_km, lf.semi_minor_axis_km, $
                              lf.geodetic_altitude_km, lf.sensor_platform_latitude_deg, $
                              incid* 180. /!pi, r_s
           print, incid* 180. /!pi, sr, r_s
         endif
      endif else begin  ; If sr le 0 then compute sr and incid angle in a different way
         ;
         print, 'No obvious way to compute incident angle'
         stop
         ;
      endelse
      incident= [incident, incid]
    endfor 
  endfor
  ;
  ;Comparing image width from slant range data and geoographical data
  ;
  if printflag then begin
    print, ''
    print, 'Comparisons of swath width and incident angle computations'
    print, '----------------------------------------------------------'
    print, 'Swath computed from the number of pixels time the size of each pixel'
    print, strcompress( string( number_of_bytes_of_pixel_data_per_record / 2 ) ) + $
           ' pixels * '                                                          + $
           strcompress( string( lf.pixel_spacing ) )                             + $
           ' = '                                                                 + $
           strcompress( string( number_of_bytes_of_pixel_data_per_record * lf.pixel_spacing / 2. ) ) + $
           ' meters '
    print, 'SWATH WIDTH=' + $
           strcompress( string( number_of_bytes_of_pixel_data_per_record * lf.pixel_spacing / 2. ) ) + $
           ' meters '
    print, ''
    print, 'Swath width computed from slant range data.'
    rs1=  slant_range_first_km[0]
    rs2=  slant_range_last_km[0]
    print, 'First slant range='  + strcompress(string(rs1)) + ' km'
    print, 'Last  slant range='  + strcompress(string(rs2)) + ' km'
    sin_phi= sin(lf.sensor_platform_latitude_deg * !pi / 180.0)
    ec=      sqrt( 1 -  lf.semi_minor_axis_km * lf.semi_minor_axis_km  $
                     /  lf.semi_major_axis_km / lf.semi_major_axis_km )
    re=      lf.semi_major_axis_km / sqrt( 1 - ec * ec  * sin_phi * sin_phi)
    cos_phi= -(rs1^2 - (re + lf.geodetic_altitude_km)^2 - re^2) / 2. / (re + lf.geodetic_altitude_km) / re
    phi1=    acos(cos_phi)
    cos_phi= -(rs2^2 - (re + lf.geodetic_altitude_km)^2 - re^2)/ 2. /  (re + lf.geodetic_altitude_km) / re
    phi2=    acos(cos_phi)
    print,   'Incident angle 1= ' + strcompress(string(phi1*180/!pi))
    print,   'Incident angle 2= ' + strcompress(string(phi2*180/!pi))
    dphi=abs(phi1-phi2)
    print,   'SWATH WIDTH=' +strcompress(string(dphi*re*1000.0)) + ' meters'
    print, ''
    print,   'Swath width computed from geographic data'
    lo1=ans_poly_cal(0                                           , 0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
    lo2=ans_poly_cal(number_of_bytes_of_pixel_data_per_record / 2, 0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
    la1=ans_poly_cal(0                                           , 0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp )
    la2=ans_poly_cal(number_of_bytes_of_pixel_data_per_record / 2, 0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp )
    print, 'Long/lat of first point first line = ' +  strcompress(string(lo1)) +  ' ' +  strcompress(string(la1))  
    print, 'Long/lat of last  point first line = ' +  strcompress(string(lo2)) +  ' ' +  strcompress(string(la2))
    ll2rb,  lo1,la1,lo2,la2, range,ber
    print, 'SWATH WIDTH=' +  strcompress(string(range*re*1000.)) + ' meters'
  endif
  n=         n_elements(incident)
  incident=  incident[1:n-1]
  ii=        ii[1:n-1]
  jj=        jj[1:n-1]
  w=where(incident gt 0, count)
  if count gt 0 then begin
    incident[w]=incident[w]
    ii[w]=   ii[w]
    jj[w]=   jj[w]
    ii=      ii / ave_pixel
    jj=      jj / ave_line
  endif else begin
    print, 'No incidence angles values.'
    stop
  endelse
  ;
  ; Convert to degrees
  ;
  incident= 180.0 / !pi *  incident
  ;  
  ; Perform fit
  ;
  if printflag then print, 'i,j => incident angle'
  ans_compute_fit, ii,  jj,  incident, lf.deg, incid_coef, incid_xexp, incid_yexp, quiet=quiet
  lf.incid_coef= incid_coef
  lf.incid_xexp= incid_xexp
  lf.incid_yexp= incid_yexp
  
  ;
  ; Rescale other conversions
  ;
  lf.i_coef=       lf.i_coef / ave_pixel
  lf.j_coef=       lf.j_coef / ave_line
  lf.lon_coef[1]=  lf.lon_coef[1]* ave_pixel
  lf.lon_coef[2]=  lf.lon_coef[2]* ave_line
  lf.lon_coef[3]=  lf.lon_coef[3]* ave_line * ave_pixel
  lf.lat_coef[1]=  lf.lat_coef[1]* ave_pixel  
  lf.lat_coef[2]=  lf.lat_coef[2]* ave_line   
  lf.lat_coef[3]=  lf.lat_coef[3]* ave_line * ave_pixel
  ;
  ; Note that the longitudes go from -180 to 180
  ;
endif   

close,1 ; Close image file

lf.incid_coef=incid_coef


if printflag then begin
  print, 'CHECKING CONVERSIONS'
  PRINT, '----------------------'
  lat0= ans_poly_cal(0,        0,          lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp  )
  lat1= ans_poly_cal(lf.nx1-1, 0,          lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp  )
  lat2= ans_poly_cal(lf.nx1-1, lf.ny1-1,   lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp  )
  lat3= ans_poly_cal(0,        lf.ny1-1,   lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp  )
  lon0= ans_poly_cal(0,        0,          lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp  )
  lon1= ans_poly_cal(lf.nx1-1, 0,          lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp  )
  lon2= ans_poly_cal(lf.nx1-1, lf.ny1-1,   lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp  )
  lon3= ans_poly_cal(0,        lf.ny1-1,   lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp  )
  inc0= ans_poly_cal(0,        0,          lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
  inc1= ans_poly_cal(lf.nx1-1, 0,          lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
  inc2= ans_poly_cal(lf.nx1-1, lf.ny1-1,   lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
  inc3= ans_poly_cal(0,        lf.ny1-1,   lf.incid_coef, lf.incid_xexp, lf.incid_yexp) 
  i0=   ans_poly_cal(lon0,     lat0,       lf.i_coef,     lf.i_xexp,     lf.i_yexp    )
  i1=   ans_poly_cal(lon1,     lat1,       lf.i_coef,     lf.i_xexp,     lf.i_yexp    )
  i2=   ans_poly_cal(lon2,     lat2,       lf.i_coef,     lf.i_xexp,     lf.i_yexp    )
  i3=   ans_poly_cal(lon3,     lat3,       lf.i_coef,     lf.i_xexp,     lf.i_yexp    ) 
  j0=   ans_poly_cal(lon0,     lat0,       lf.j_coef,     lf.j_xexp,     lf.j_yexp    )
  j1=   ans_poly_cal(lon1,     lat1,       lf.j_coef,     lf.j_xexp,     lf.j_yexp    )
  j2=   ans_poly_cal(lon2,     lat2,       lf.j_coef,     lf.j_xexp,     lf.j_yexp    )
  j3=   ans_poly_cal(lon3,     lat3,       lf.j_coef,     lf.j_xexp,     lf.j_yexp    )   
  print, 'i/j to lon/lat'
  print, string(0,         format='(I8)')   + string(0,        format='(I8)') + $
         string(lon0,      format='(F12.6)')  + string(lat0,   format='(F12.6)') 
  print, string(lf.nx1-1,  format='(I8)')   + string(0,        format='(I8)') + $
         string(lon1,      format='(F12.6)')  + string(lat1,   format='(F12.6)') 
  print, string(lf.nx1-1,  format='(I8)')   + string(lf.ny1-1, format='(I8)') + $
         string(lon2,      format='(F12.6)')  + string(lat2,   format='(F12.6)') 
  print, string(0,         format='(I8)')   + string(lf.ny1-1, format='(I8)') + $
         string(lon3,      format='(F12.6)')  + string(lat3,   format='(F12.6)') 
  print, 'lon/lat to i,j'
  print, string(lon0,      format='(F12.6)')  + string(lat0,   format='(F12.6)') + $
         string(i0,        format='(I8)')   + string(j0,       format='(I8)')      
  print, string(lon1,      format='(F12.6)')  + string(lat1,   format='(F12.6)') + $
         string(i1,        format='(I8)')   + string(j1,       format='(I8)')
  print, string(lon2,      format='(F12.6)')   + string(lat2,  format='(F12.6)') + $
         string(i2,        format='(I8)')   + string(j2,       format='(I8)')      
  print, string(lon3,      format='(F12.6)')  + string(lat3,   format='(F12.6)') + $
         string(i3,        format='(I8)')   + string(j3,       format='(I8)')       
  print, 'i,j to incidence'
  print, string(0,         format='(I8)')   + string(0,        format='(I8)') + $
         string(inc0,      format='(F12.6)')  
  print, string(lf.nx1-1,  format='(I8)')   + string(0,        format='(I8)') + $
         string(inc1,      format='(F12.6)') 
  print, string(lf.nx1-1,  format='(I8)')   + string(lf.ny1-1, format='(I8)') + $
         string(inc2,      format='(F12.6)') 
  print, string(0,         format='(I8)')   + string(lf.ny1-1, format='(I8)') + $
         string(inc3,      format='(F12.6)') 
endif 


if printflag then print, "Finished reading image file.'

end







