;-----------------------------------------------------
;+
; NAME: psar_read_leader'
; PURPOSE: 
;   This program reads a PSAR leader file and saves the results into a 
;   structure. The structure needs to have been created using the 
;   psar_create_lf program.
; CALLING SEQUENCE:'
;   psar_read_leader, leaderfile, lf, good_leader
; OUTPUT: '
;   lf=         Update of leader file data structure'
;   good_leader= 1 if good 0 else
; KEYWORDS:'
;   /help   Print out help'
;   /quiet  Do not print to screen'
;  
; MODIFICATION HISTORY:
; 2006 Nov 27 Frank Monaldo  
; 2008 Mar 31 FMM Add number_of_sar_channels 
; 2010 Jan 08 Frank Monaldo. Added  ellipsoid_designator to structure
; 2010 Jan 20 Frank Monaldo. Change radar frequency in lf structure to
;                            units of Hz.
;                            Correct spelling of incidence_angle_at_scence_center
;                            Added distance_to_scene_center_m to structure
; 2010 Jun 14 Frank Monaldo. Print help on no input.
; 2011 Jan 01 Frank Monaldo. Pass of quiet flag
; 2011 Feb 08 Frank Monaldo. Add error catch for reading
; 2011 Mar 30 Frank Monaldo. Add nrcs_slope and nrcs_bias to lf structure
; 2011 Apr 06 Frank Monaldo. Recompute lon/lat to i/j conversion if cross international date line
; 2014 Mar 28 Frank Monaldo. Change to square brackets to index arrays for a few remaining arrays
; 2014 Apr 05 Frank Monaldo. Use the latitude and longitude information in the facility data record
;                            rather than the map projection record.
; 2014 Apr 07 Frank Monlado. Remarked out use of other map_projection longitude and latitude
;                            coefficients
; 2014 Apr 08 Frank Monaldo. Corrected print out of results of i/j comparisons
; 2014 Apr 09 Frank Monaldo. Correct error in offset on lon,lat => i,j
; 2014 Apr 14 Frank Monaldo. If the FDR does not contain the latitude and longitude information then
;                            revert to similar those less precise information in the map projection
;                            record.
; 2017 Aug 04 Chris Jackson. Modified test and added redundent calulcation for assigning values to lf.cent_lon and lf.center_lat                            
; Copyright (C) 2004-2017, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  psar_read_leader, leaderfile, lf, good_leader, help=help, quiet=quiet
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: psar_read_leader'
  print, ' PURPOSE:'
  print, '   This program reads a PSAR leader file and saves the results into a 
  print, '   structure. The structure needs to have been created using the 
  print, '   psar_create_lf program.
  print, ' CALLING SEQUENCE:'
  print, '   psar_read_leader, leaderfile, lf, good_leader
  print, ' INPUT:'
  print, '   leaderfile= PALSAR leader file'
  print, ' OUTPUT: '
  print, '   lf=         Update of leader file data structure'
  print, '   good_leader= 1 if good 0 else
  print, ' KEYWORDS:'
  print, '   /help   Print out help'
  print, '   /quiet  Do not print to screen'
  return           
endif
good_leader=0
printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin 
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      good_leader=-1
      return
   endif
   catch, /cancel  
endif  

eflag=endian()  ; 0= little endian (x86)
                ; 1= big endian

file_descriptor_rec=      bytarr(720)

; facility_related_rec to be declared...

; Open file
;
if not file_test(leaderfile) then begin
  if printflag then print, 'Cannot find leader file: ' + leaderfile
  goodleader=0
endif
close, 1
openr, 1, leaderfile

;
;------------------------------------------
; Read file descriptor record
; -----------------------------------------
;
readu, 1, file_descriptor_rec
if printflag then print, 'Reading file descriptor record'
if eflag then begin
  record_sequence_number                     = long(         file_descriptor_rec[0:3], 0 )
endif else begin
  record_sequence_number                     = long( reverse(file_descriptor_rec[0:3]), 0 )
endelse
first_record_subtype_code                    =               byte(file_descriptor_rec[4])
record_type_code                             =               byte(file_descriptor_rec[5])
second_record_subtype_code                   =               byte(file_descriptor_rec[6])
third_record_subtype_code                    =               byte(file_descriptor_rec[7])
if eflag then begin
  length_of_this_record                      = long(         file_descriptor_rec[8:11], 0    ) 
endif else begin
  length_of_this_record                      = long( reverse(file_descriptor_rec[8:11]), 0   )
endelse
ascii_flag                                   =     string(   file_descriptor_rec[ 12: 13] )
continuation_flag                            =     string(   file_descriptor_rec[ 14: 15] )
format_control_document_id                   =     string(   file_descriptor_rec[ 16: 27] )
format_control_document_revision_level       =     string(   file_descriptor_rec[ 28: 29] )
file_design_descriptor_revision_letter       =     string(   file_descriptor_rec[ 30: 31] )
logical_volume_generating_facility           =     string(   file_descriptor_rec[ 32: 43] )
file_number                                  = long( string( file_descriptor_rec[ 44: 47] ))
referenced_file_name                         =     string(   file_descriptor_rec[ 48: 63] )
referenced_sequence_and_location_type_flag   =     string(   file_descriptor_rec[ 64: 67] )
sequence_number_location                     = long( string( file_descriptor_rec[ 68: 75] ))
sequence_number_field_length                 = long( string( file_descriptor_rec[ 76: 79] ))
record_code_and_location_type_flag           =      string(  file_descriptor_rec[ 80: 83] )
record_code_location                         = long( string( file_descriptor_rec[ 84: 91] ))
record_code_field_length                     = long( string( file_descriptor_rec[ 92: 95] ))
record_length_and_location_type_flag         =      string(  file_descriptor_rec[ 96: 99] )
record_length_location                       = long( string( file_descriptor_rec[101:107] ))
record_length_field_length                   = long( string( file_descriptor_rec[108:111] ))
number_of_data_set_summary_records           = long( string( file_descriptor_rec[180:185] ))
data_set_summary_record_length               = long( string( file_descriptor_rec[186:191] ))
number_of_map_projection_data_records        = long( string( file_descriptor_rec[192:197] ))
map_projection_record_length                 = long( string( file_descriptor_rec[198:203] ))
number_of_platform_position_records          = long( string( file_descriptor_rec[204:209] ))
platform_position_record_length              = long( string( file_descriptor_rec[210:215] ))
number_of_attitude_data_records              = long( string( file_descriptor_rec[216:221] ))
attitude_data_record_length                  = long( string( file_descriptor_rec[222:227] ))
number_of_radiometric_records                = long( string( file_descriptor_rec[228:233] ))
radiometric_record_length                    = long( string( file_descriptor_rec[234:239] ))
number_of_radiometric_compensation_records   = long( string( file_descriptor_rec[240:245] ))
radiometric_compensation_record_length       = long( string( file_descriptor_rec[246:251] ))
number_of_data_quality_summary_records       = long( string( file_descriptor_rec[252:257] ))
data_quality_summary_record_length           = long( string( file_descriptor_rec[258:263] ))
number_of_data_histograms                    = long( string( file_descriptor_rec[264:269] ))
data_histogram_record_length                 = long( string( file_descriptor_rec[270:275] ))
number_of_range_spectra_records              = long( string( file_descriptor_rec[276:281] ))
range_spectra_record_length                  = long( string( file_descriptor_rec[282:287] ))
number_of_dem_descriptor_records             = long( string( file_descriptor_rec[288:293] ))
dem_descriptor_record_length                 = long( string( file_descriptor_rec[294:299] ))
number_of_radar_par_update_records           = long( string( file_descriptor_rec[300:305] ))
radar_par_update_record_length               = long( string( file_descriptor_rec[306:311] ))
number_of_annotation_data_records            = long( string( file_descriptor_rec[312:317] ))
annotation_data_record_length                = long( string( file_descriptor_rec[318:323] ))
number_of_det_processing_records             = long( string( file_descriptor_rec[324:329] ))
det_processing_record_length                 = long( string( file_descriptor_rec[330:335] ))
number_of_calibration_records                = long( string( file_descriptor_rec[336:341] ))
calibration_record_length                    = long( string( file_descriptor_rec[342:347] ))
number_of_gcp_records                        = long( string( file_descriptor_rec[348:353] ))
gcp_record_length                            = long( string( file_descriptor_rec[354:359] ))
number_of_facility_data_records              = lonarr(11)
facility_data_record_length                  = lonarr(11)           
number_of_facility_data_records[0]           = long( string( file_descriptor_rec[420:425] ))
facility_data_record_length[0]               = long( string( file_descriptor_rec[426:433] ))
number_of_facility_data_records[1]           = long( string( file_descriptor_rec[434:439] ))
facility_data_record_length[1]               = long( string( file_descriptor_rec[440:447] ))
number_of_facility_data_records[2]           = long( string( file_descriptor_rec[448:453] ))
facility_data_record_length[2]               = long( string( file_descriptor_rec[454:461] ))
number_of_facility_data_records[3]           = long( string( file_descriptor_rec[462:467] ))
facility_data_record_length[3]               = long( string( file_descriptor_rec[468:475] ))
number_of_facility_data_records[4]           = long( string( file_descriptor_rec[476:481] ))
facility_data_record_length[4]               = long( string( file_descriptor_rec[482:489] ))
number_of_facility_data_records[5]           = long( string( file_descriptor_rec[490:495] ))
facility_data_record_length[5]               = long( string( file_descriptor_rec[496:503] ))
number_of_facility_data_records[6]           = long( string( file_descriptor_rec[504:509] ))
facility_data_record_length[6]               = long( string( file_descriptor_rec[510:517] ))
number_of_facility_data_records[7]           = long( string( file_descriptor_rec[518:523] ))
facility_data_record_length[7]               = long( string( file_descriptor_rec[524:531] ))
number_of_facility_data_records[8]           = long( string( file_descriptor_rec[532:537] ))
facility_data_record_length[8]               = long( string( file_descriptor_rec[538:545] ))
number_of_facility_data_records[9]           = long( string( file_descriptor_rec[546:551] ))
facility_data_record_length[9]               = long( string( file_descriptor_rec[552:559] ))
number_of_facility_data_records[10]          = long( string( file_descriptor_rec[560:565] ))
facility_data_record_length[10]              = long( string( file_descriptor_rec[566:573] ))
;


if printflag then begin
  print, 'LEADER FILE: = ' + leaderfile
  print, 'record_sequence_number                       =' + string(record_sequence_number)
  print, 'first_record_subtype_code                    =' + string(first_record_subtype_code, format='(I4)') 
  print, 'record_type_code                             =' + string(record_type_code, format='(I4)')          
  print, 'second_record_subtype_code                   =' + string(second_record_subtype_code, format='(I4)')
  print, 'third_record_subtype_code                    =' + string(third_record_subtype_code, format='(I4)') 
  print, 'length_of_this_record                        =' + string(length_of_this_record, format='(I4)')     
  print, 'ascii_flag                                   =' + ascii_flag                                   
  print, 'continuation_flag                            =' + continuation_flag                            
  print, 'format_control_document_id                   =' + format_control_document_id                   
  print, 'format_control_document_revision_level       =' + format_control_document_revision_level       
  print, 'file_design_descriptor_revision_letter       =' + file_design_descriptor_revision_letter       
  print, 'logical_volume_generating_facility           =' + logical_volume_generating_facility           
  print, 'file_number                                  =' + string(file_number)                          
  print, 'referenced_file_name                         =' + referenced_file_name                         
  print, 'referenced_sequence_and_location_type_flag   =' + referenced_sequence_and_location_type_flag   
  print, 'sequence_number_location                     =' + string(sequence_number_location)                   
  print, 'sequence_number_field_length                 =' + string(sequence_number_field_length)               
  print, 'record_code_and_location_type_flag           =' + record_code_and_location_type_flag                 
  print, 'record_code_location                         =' + string(record_code_location)                       
  print, 'record_code_field_length                     =' + string(record_code_field_length)                   
  print, 'record_length_and_location_type_flag         =' + string(record_length_and_location_type_flag)       
  print, 'record_length_location                       =' + string(record_length_location)                     
  print, 'record_length_field_length                   =' + string(record_length_field_length)                 
  print, 'number_of_data_set_summary_records           =' + string(number_of_data_set_summary_records)         
  print, 'data_set_summary_record_length               =' + string(data_set_summary_record_length)             
  print, 'number_of_map_projection_data_records        =' + string(number_of_map_projection_data_records)      
  print, 'map_projection_record_length                 =' + string(map_projection_record_length)               
  print, 'number_of_platform_position_records          =' + string(number_of_platform_position_records)        
  print, 'platform_position_record_length              =' + string(platform_position_record_length)            
  print, 'number_of_attitude_data_records              =' + string(number_of_attitude_data_records)            
  print, 'attitude_data_record_length                  =' + string(attitude_data_record_length)                
  print, 'number_of_radiometric_records                =' + string(number_of_radiometric_records)              
  print, 'radiometric_record_length                    =' + string(radiometric_record_length)                  
  print, 'number_of_radiometric_compensation_records   =' + string(number_of_radiometric_compensation_records) 
  print, 'radiometric_compensation_record_length       =' + string(radiometric_compensation_record_length)     
  print, 'number_of_data_quality_summary_records       =' + string(number_of_data_quality_summary_records)     
  print, 'data_quality_summary_record_length           =' + string(data_quality_summary_record_length)           
  print, 'number_of_data_histograms                    =' + string(number_of_data_histograms)                  
  print, 'data_histogram_record_length                 =' + string(data_histogram_record_length)              
  print, 'number_of_range_spectra_records              =' + string(number_of_range_spectra_records)            
  print, 'range_spectra_record_length                  =' + string(range_spectra_record_length)                
  print, 'number_of_dem_descriptor_records             =' + string(number_of_dem_descriptor_records)           
  print, 'dem_descriptor_record_length                 =' + string(dem_descriptor_record_length)                          
  print, 'number_of_radar_par_update_records           =' + string(number_of_radar_par_update_records)         
  print, 'radar_par_update_record_length               =' + string(radar_par_update_record_length)             
  print, 'number_of_annotation_data_records            =' + string(number_of_annotation_data_records)         
  print, 'annotation_data_record_length                =' + string(annotation_data_record_length)              
  print, 'number_of_det_processing_records             =' + string(number_of_det_processing_records)           
  print, 'det_processing_record_length                 =' + string(det_processing_record_length)               
  print, 'number_of_calibration_records                =' + string(number_of_calibration_records)              
  print, 'calibration_record_length                    =' + string(calibration_record_length)                  
  print, 'number_of_gcp_records                        =' + string(number_of_gcp_records)                      
  print, 'gcp_record_length                            =' + string(gcp_record_length)     
  for i= 0, 10 do begin
    print, 'number_of_facility_data_records['+ string(i,format='(I2)') +']' + string(number_of_facility_data_records[i] )
    print, 'facility_data_record_length['+ string(i,format='(I2)') +']'     + string(facility_data_record_length[i]     )
  endfor
  print, 'FINISHED READING THE FILE DESCRIPTOR RECORD'
  if keyword_set(pause) then stop
endif 
;
; Declare other records based on file descriptor information
;
if number_of_data_set_summary_records         gt 0 then data_set_summary_rec        = bytarr( data_set_summary_record_length        )
if number_of_map_projection_data_records      gt 0 then map_projection_rec          = bytarr( map_projection_record_length          )
if number_of_platform_position_records        gt 0 then platform_position_rec       = bytarr( platform_position_record_length       )
if number_of_attitude_data_records            gt 0 then attitude_data_rec           = bytarr( attitude_data_record_length           )
if number_of_radiometric_records              gt 0 then radiometric_rec             = bytarr( radiometric_record_length             )
if number_of_radiometric_compensation_records gt 0 then radiometric_compensation_rec= bytarr( radiometric_compensation_record_length)
if number_of_data_quality_summary_records     gt 0 then data_quality_summary_rec    = bytarr( data_quality_summary_record_length    )
if number_of_data_histograms                  gt 0 then data_histogram_rec          = bytarr( data_histogram_record_length          )
if number_of_range_spectra_records            gt 0 then range_spectra_rec           = bytarr( range_spectra_record_length           )
if number_of_dem_descriptor_records           gt 0 then dem_descriptor_rec          = bytarr( dem_descriptor_record_length          )
if number_of_radar_par_update_records         gt 0 then radar_par_update_rec        = bytarr( radar_par_update_record_length        )
if number_of_annotation_data_records          gt 0 then annotation_data_rec         = bytarr( annotation_data_record_length         )
if number_of_det_processing_records           gt 0 then det_processing_rec          = bytarr( det_processing_record_length          )
if number_of_calibration_records              gt 0 then calibration_rec             = bytarr( calibration_record_length             )
if number_of_gcp_records                      gt 0 then gcp_rec                     = bytarr( gcp_record_length                     )
if number_of_facility_data_records[0 ]        gt 0 then facility_data_record0       = bytarr( facility_data_record_length[0 ]       ) 
if number_of_facility_data_records[1 ]        gt 0 then facility_data_record1       = bytarr( facility_data_record_length[1 ]       )
if number_of_facility_data_records[2 ]        gt 0 then facility_data_record2       = bytarr( facility_data_record_length[2 ]       )
if number_of_facility_data_records[3 ]        gt 0 then facility_data_record3       = bytarr( facility_data_record_length[3 ]       )
if number_of_facility_data_records[4 ]        gt 0 then facility_data_record4       = bytarr( facility_data_record_length[4 ]       )
if number_of_facility_data_records[5 ]        gt 0 then facility_data_record5       = bytarr( facility_data_record_length[5 ]       )
if number_of_facility_data_records[6 ]        gt 0 then facility_data_record6       = bytarr( facility_data_record_length[6 ]       )
if number_of_facility_data_records[7 ]        gt 0 then facility_data_record7       = bytarr( facility_data_record_length[7 ]       )
if number_of_facility_data_records[8 ]        gt 0 then facility_data_record8       = bytarr( facility_data_record_length[8 ]       )
if number_of_facility_data_records[9 ]        gt 0 then facility_data_record9       = bytarr( facility_data_record_length[9 ]       )
if number_of_facility_data_records[10]        gt 0 then facility_data_record10      = bytarr( facility_data_record_length[10]       )
;
;---------------------------------------------------------------
; Data set summary record 
; --------------------------------------------------------------
if number_of_data_set_summary_records         gt 0 then begin
  for i=0L, number_of_data_set_summary_records-1 do begin
    readu, 1, data_set_summary_rec
    if eflag then begin
      record_sequence_number                     = string(long(        data_set_summary_rec[0:3],  0))
    endif else begin
      record_sequence_number                     = string(long(reverse(data_set_summary_rec[0:3]), 0))
    endelse
    first_record_subtype_code                    = string(fix(byte(data_set_summary_rec[4])))
    record_type_code                             = string(fix(byte(data_set_summary_rec[5])))
    second_record_subtype_code                   = string(fix(byte(data_set_summary_rec[6])))
    third_record_subtype_code                    = string(fix(byte(data_set_summary_rec[7])))
    if eflag then begin
      length_of_this_record                      = string(long(        data_set_summary_rec[8:11], 0 ))
    endif else begin
      length_of_this_record                      = string(long(reverse(data_set_summary_rec[8:11]),0 ))
    endelse
    data_set_summary_record_sequence_number      = string(  data_set_summary_rec[  12:  14] )
    scene_identifier                             = string(  data_set_summary_rec[  20:  51] )
    input_scene_center_time                      = string(  data_set_summary_rec[  68:  99] )
    processed_scene_center_latitude_deg          = string(  data_set_summary_rec[ 116: 131] )
    processed_scene_center_longitude_deg         = string(  data_set_summary_rec[ 132: 147] )
    processed_scene_center_true_heading_deg      = string(  data_set_summary_rec[ 148: 163] )
    ellipsoid_designator                         = string(  data_set_summary_rec[ 164: 179] )
    ellipsoid_semi_major_axis_km                 = string(  data_set_summary_rec[ 180: 195] )
    ellipsoid_semi_minor_axis_km                 = string(  data_set_summary_rec[ 196: 211] )
    earth_mass                                   = string(  data_set_summary_rec[ 212: 227] )
    gravitational_constant                       = string(  data_set_summary_rec[ 228: 243] )
    ellipsoid_j2_parameter                       = string(  data_set_summary_rec[ 244: 259] )
    ellipsoid_j3_parameter                       = string(  data_set_summary_rec[ 260: 275] )
    ellipsoid_j4_parameter                       = string(  data_set_summary_rec[ 276: 291] )
    average_terraom_height                       = string(  data_set_summary_rec[ 308: 321] )
    scene_center_line_number                     = string(  data_set_summary_rec[ 324: 331] )
    scene_center_pixel_number                    = string(  data_set_summary_rec[ 332: 339] )
    processed_scence_length_km                   = string(  data_set_summary_rec[ 340: 355] )
    processed_scence_width_km                    = string(  data_set_summary_rec[ 356: 371] )
    number_of_sar_channels                       = string(  data_set_summary_rec[ 388: 391] )
    sensor_platform_mission_identifier           = string(  data_set_summary_rec[ 396: 411] )
    sensor_id_mode_of_operation                  = string(  data_set_summary_rec[ 412: 443] )
    orbit_number                                 = string(  data_set_summary_rec[ 444: 451] )
    sensor_platform_latitude_deg                 = string(  data_set_summary_rec[ 452: 459] )
    sensor_platform_longitude_deg                = string(  data_set_summary_rec[ 460: 467] )
    sensor_platform_heading_deg                  = string(  data_set_summary_rec[ 468: 475] )
    sensor_clock_angle                           = string(  data_set_summary_rec[ 476: 483] )
    incidence_angle_at_scene_center              = string(  data_set_summary_rec[ 484: 491] )
    radar_wavelength_m                           = string(  data_set_summary_rec[ 500: 515] )
    motion_compensation_indicator                = string(  data_set_summary_rec[ 516: 517] )
    range_pulse_code_specifier                   = string(  data_set_summary_rec[ 518: 533] )
    range_pulse_amplitude_coefficient1           = string(  data_set_summary_rec[ 534: 549] )
    range_pulse_amplitude_coefficient2           = string(  data_set_summary_rec[ 550: 565] )
    range_pulse_amplitude_coefficient3           = string(  data_set_summary_rec[ 566: 581] )
    range_pulse_amplitude_coefficient4           = string(  data_set_summary_rec[ 582: 597] )
    range_pulse_amplitude_coefficient5           = string(  data_set_summary_rec[ 598: 613] )
    down_linked_data_chirp_extraction            = string(  data_set_summary_rec[ 694: 701] )
    sampling_rate                                = string(  data_set_summary_rec[ 710: 725] )
    range_gate_at_early_edge                     = string(  data_set_summary_rec[ 726: 741] )
    range_pulse_length                           = string(  data_set_summary_rec[ 742: 757] )
    base_band_conversion_flag                    = string(  data_set_summary_rec[ 758: 761] )
    range_compressed_flag                        = string(  data_set_summary_rec[ 762: 765] )
    receiver_gain_like_polarization              = string(  data_set_summary_rec[ 766: 781] )
    receiver_gain_cross_polarization             = string(  data_set_summary_rec[ 782: 797] )
    quantization_bits_per_channel                = string(  data_set_summary_rec[ 798: 805] )
    quantizer_descriptor                         = string(  data_set_summary_rec[ 806: 817] )
    dc_bias_i_component                          = string(  data_set_summary_rec[ 818: 833] ) 
    dc_bias_q_component                          = string(  data_set_summary_rec[ 834: 849] ) 
    gain_imbalance_i_q                           = string(  data_set_summary_rec[ 850: 865] ) 
    antenna_electronic_boresight_deg             = string(  data_set_summary_rec[ 898: 913] ) 
    antenna_mechanical_boresight_deg             = string(  data_set_summary_rec[ 914: 929] ) 
    echo_tracker_on_off                          = string(  data_set_summary_rec[ 930: 933] ) 
    nominal_prf_mhz                              = string(  data_set_summary_rec[ 934: 949] ) 
    effective_two_way_elevation_3db_beam_width   = string(  data_set_summary_rec[ 950: 965] ) 
    effective_two_way_azimuth_3db_beam_width     = string(  data_set_summary_rec[ 966: 981] ) 
    satellite_encoded_binary_time_code           = string(  data_set_summary_rec[ 982: 997] ) 
    satellite_clock_time                         = string(  data_set_summary_rec[ 998:1029] )
    satellite_clock_increment_nsec               = string(  data_set_summary_rec[1030:1045] )
    processing_facility_identifier               = string(  data_set_summary_rec[1046:1063] )
    processing_system_identifier                 = string(  data_set_summary_rec[1062:1069] )
    processing_version_identifier                = string(  data_set_summary_rec[1071:1077] )
    processing_facility_process_code             = string(  data_set_summary_rec[1078:1093] )
    product_level_code                           = string(  data_set_summary_rec[1094:1109] )
    product_type_specifier                       = string(  data_set_summary_rec[1110:1141] )
    processing_algorithm_identifier              = string(  data_set_summary_rec[1142:1173] )
    nominal_effective_number_azimuth_looks       = string(  data_set_summary_rec[1174:1189] )
    nominal_effective_number_range_looks         = string(  data_set_summary_rec[1190:1205] )
    bandwidth_per_look_azimuth_hz                = string(  data_set_summary_rec[1206:1221] )
    bandwidth_per_look_range_hz                  = string(  data_set_summary_rec[1222:1237] )
    total_processor_bandwidth_azimuth_hz         = string(  data_set_summary_rec[1238:1253] )
    total_processor_bandwidth_range_hz           = string(  data_set_summary_rec[1254:1269] )
    weighting_function_designator_azimuth        = string(  data_set_summary_rec[1170:1301] )
    weighting_function_designator_range          = string(  data_set_summary_rec[1302:1333] )
    data_input_source                            = string(  data_set_summary_rec[1334:1349] )
    nominal_ground_range_resolution_m            = string(  data_set_summary_rec[1350:1365] )
    nominal_ground_azimuth_resolution_m          = string(  data_set_summary_rec[1366:1381] )
    along_track_doppler_frequency_constant_term  = string(  data_set_summary_rec[1414:1429] )
    along_track_doppler_frequency_linear_term    = string(  data_set_summary_rec[1430:1445] )
    along_track_doppler_frequency_quadratic_term = string(  data_set_summary_rec[1446:1461] )
    cross_track_doppler_frequency_constant_term  = string(  data_set_summary_rec[1478:1496] )
    cross_track_doppler_frequency_linear_term    = string(  data_set_summary_rec[1494:1509] )
    cross_track_doppler_frequency_quadratic_term = string(  data_set_summary_rec[1510:1525] )
    time_direction_along_pixel_direction         = string(  data_set_summary_rec[1526:1533] )
    time_direction_along_line_direction          = string(  data_set_summary_rec[1534:1541] )
    line_content_indicator                       = string(  data_set_summary_rec[1670:1677] )
    clutter_lock_applied                         = string(  data_set_summary_rec[1678:1681] )
    auto_focusing_applied                        = string(  data_set_summary_rec[1682:1685] )
    line_spacing_m                               = string(  data_set_summary_rec[1687:1701] )
    pixel_spacing_m                              = string(  data_set_summary_rec[1702:1717] )
    processor_range_compression_designator       = string(  data_set_summary_rec[1718:1733] )
    doppler_center_frequency_constant_term       = string(  data_set_summary_rec[1734:1749] )
    doppler_center_frequency_linear_term         = string(  data_set_summary_rec[1750:1765] )
    calibration_data_indicator                   = string(  data_set_summary_rec[1766:1769] )
    start_line_number_of_calibration_upper       = string(  data_set_summary_rec[1770:1777] )
    stop_line_number_of_calibration_upper        = string(  data_set_summary_rec[1778:1785] )
    start_line_number_of_calibration_bottom      = string(  data_set_summary_rec[1786:1793] )
    stop_line_number_of_calibration_bottom       = string(  data_set_summary_rec[1794:1801] )
    prf_switching_indicator                      = string(  data_set_summary_rec[1802:1805] )
    line_locator_of_prf_switching                = string(  data_set_summary_rec[1806:1813] )
    direction_of_beam_center                     = string(  data_set_summary_rec[1814:1829] )
    yaw_steerting_mode_flag                      = string(  data_set_summary_rec[1830:1833] )
    parameter_table_number                       = string(  data_set_summary_rec[1834:1837] )
    nominal_off_nadir_angle                      = string(  data_set_summary_rec[1838:1853] )
    antenna_beam_number                          = string(  data_set_summary_rec[1854:1857] )
    incidence_angle_constant_term                = string(  data_set_summary_rec[1886:1905] )
    incidence_angle_linear_term                  = string(  data_set_summary_rec[1906:1925] )
    incidence_angle_quadratic_term               = string(  data_set_summary_rec[1926:1945] )
    incidence_angle_cubic_term                   = string(  data_set_summary_rec[1946:1965] )
    incidence_angle_fourth_term                  = string(  data_set_summary_rec[1966:1985] )
    incidence_angle_fifth_term                   = string(  data_set_summary_rec[1986:2005] )
    number_of_annotation_points                  = string(  data_set_summary_rec[2006:2013] )
    if printflag then begin
      print, ''
      print, '------------------------'
      print, 'DATA SET SUMMARY RECORD:'
      print, '------------------------'
      print, 'record_sequence_number                       = ' + record_sequence_number                       
      print, 'length_of_this_record                        = ' + length_of_this_record
      print, 'first_record_subtype_code                    = ' + first_record_subtype_code                    
      print, 'record_type_code                             = ' + record_type_code                             
      print, 'second_record_subtype_code                   = ' + second_record_subtype_code                   
      print, 'third_record_subtype_code                    = ' + third_record_subtype_code                    
      print, 'length_of_this_record                        = ' + length_of_this_record                        
      print, 'data_set_summary_record_sequence_number      = ' + data_set_summary_record_sequence_number      
      print, 'scene_identifier                             = ' + scene_identifier                             
      print, 'input_scene_center_time                      = ' + input_scene_center_time                      
      print, 'processed_scene_center_latitude_deg          = ' + processed_scene_center_latitude_deg          
      print, 'processed_scene_center_longitude_deg         = ' + processed_scene_center_longitude_deg         
      print, 'processed_scene_center_true_heading_deg      = ' + processed_scene_center_true_heading_deg      
      print, 'ellipsoid_designator                         = ' + ellipsoid_designator                         
      print, 'ellipsoid_semi_major_axis_km                 = ' + ellipsoid_semi_major_axis_km                 
      print, 'ellipsoid_semi_minor_axis_km                 = ' + ellipsoid_semi_minor_axis_km                 
      print, 'earth_mass                                   = ' + earth_mass                                   
      print, 'gravitational_constant                       = ' + gravitational_constant                       
      print, 'ellipsoid_j2_parameter                       = ' + ellipsoid_j2_parameter                       
      print, 'ellipsoid_j3_parameter                       = ' + ellipsoid_j3_parameter                       
      print, 'ellipsoid_j4_parameter                       = ' + ellipsoid_j4_parameter                       
      print, 'average_terraom_height                       = ' + average_terraom_height                       
      print, 'scene_center_line_number                     = ' + scene_center_line_number                     
      print, 'scene_center_pixel_number                    = ' + scene_center_pixel_number                    
      print, 'processed_scence_length_km                   = ' + processed_scence_length_km                   
      print, 'processed_scence_width_km                    = ' + processed_scence_width_km                    
      print, 'number_of_sar_channels                       = ' + number_of_sar_channels                       
      print, 'sensor_platform_mission_identifier           = ' + sensor_platform_mission_identifier           
      print, 'sensor_id_mode_of_operation                  = ' + sensor_id_mode_of_operation                  
      print, 'orbit_number                                 = ' + orbit_number                                 
      print, 'sensor_platform_latitude_deg                 = ' + sensor_platform_latitude_deg                 
      print, 'sensor_platform_longitude_deg                = ' + sensor_platform_longitude_deg                
      print, 'sensor_platform_heading_deg                  = ' + sensor_platform_heading_deg                  
      print, 'sensor_clock_angle                           = ' + sensor_clock_angle                           
      print, 'incidence_angle_at_scene_center              = ' + incidence_angle_at_scene_center             
      print, 'radar_wavelength_m                           = ' + radar_wavelength_m                           
      print, 'motion_compensation_indicator                = ' + motion_compensation_indicator                
      print, 'range_pulse_code_specifier                   = ' + range_pulse_code_specifier                   
      print, 'range_pulse_amplitude_coefficient1           = ' + range_pulse_amplitude_coefficient1           
      print, 'range_pulse_amplitude_coefficient2           = ' + range_pulse_amplitude_coefficient2           
      print, 'range_pulse_amplitude_coefficient3           = ' + range_pulse_amplitude_coefficient3           
      print, 'range_pulse_amplitude_coefficient4           = ' + range_pulse_amplitude_coefficient4           
      print, 'range_pulse_amplitude_coefficient5           = ' + range_pulse_amplitude_coefficient5           
      print, 'down_linked_data_chirp_extraction            = ' + down_linked_data_chirp_extraction            
      print, 'sampling_rate                                = ' + sampling_rate                                
      print, 'range_gate_at_early_edge                     = ' + range_gate_at_early_edge                     
      print, 'range_pulse_length                           = ' + range_pulse_length                           
      print, 'base_band_conversion_flag                    = ' + base_band_conversion_flag                    
      print, 'range_compressed_flag                        = ' + range_compressed_flag                        
      print, 'receiver_gain_like_polarization              = ' + receiver_gain_like_polarization              
      print, 'receiver_gain_cross_polarization             = ' + receiver_gain_cross_polarization             
      print, 'quantization_bits_per_channel                = ' + quantization_bits_per_channel                
      print, 'quantizer_descriptor                         = ' + quantizer_descriptor                         
      print, 'dc_bias_i_component                          = ' + dc_bias_i_component                          
      print, 'dc_bias_q_component                          = ' + dc_bias_q_component                          
      print, 'gain_imbalance_i_q                           = ' + gain_imbalance_i_q                           
      print, 'antenna_electronic_boresight_deg             = ' + antenna_electronic_boresight_deg             
      print, 'antenna_mechanical_boresight_deg             = ' + antenna_mechanical_boresight_deg             
      print, 'echo_tracker_on_off                          = ' + echo_tracker_on_off                          
      print, 'nominal_prf_mhz                              = ' + nominal_prf_mhz                              
      print, 'effective_two_way_elevation_3db_beam_width   = ' + effective_two_way_elevation_3db_beam_width   
      print, 'effective_two_way_azimuth_3db_beam_width     = ' + effective_two_way_azimuth_3db_beam_width     
      print, 'satellite_encoded_binary_time_code           = ' + satellite_encoded_binary_time_code           
      print, 'satellite_clock_time                         = ' + satellite_clock_time                         
      print, 'satellite_clock_increment_nsec               = ' + satellite_clock_increment_nsec               
      print, 'processing_facility_identifier               = ' + processing_facility_identifier               
      print, 'processing_system_identifier                 = ' + processing_system_identifier                 
      print, 'processing_version_identifier                = ' + processing_version_identifier                
      print, 'processing_facility_process_code             = ' + processing_facility_process_code             
      print, 'product_level_code                           = ' + product_level_code                           
      print, 'product_type_specifier                       = ' + product_type_specifier                       
      print, 'processing_algorithm_identifier              = ' + processing_algorithm_identifier              
      print, 'nominal_effective_number_azimuth_looks       = ' + nominal_effective_number_azimuth_looks       
      print, 'nominal_effective_number_range_looks         = ' + nominal_effective_number_range_looks         
      print, 'bandwidth_per_look_azimuth_hz                = ' + bandwidth_per_look_azimuth_hz                
      print, 'bandwidth_per_look_range_hz                  = ' + bandwidth_per_look_range_hz                  
      print, 'total_processor_bandwidth_azimuth_hz         = ' + total_processor_bandwidth_azimuth_hz         
      print, 'total_processor_bandwidth_range_hz           = ' + total_processor_bandwidth_range_hz           
      print, 'weighting_function_designator_azimuth        = ' + weighting_function_designator_azimuth        
      print, 'weighting_function_designator_range          = ' + weighting_function_designator_range          
      print, 'data_input_source                            = ' + data_input_source                            
      print, 'nominal_ground_range_resolution_m            = ' + nominal_ground_range_resolution_m            
      print, 'nominal_ground_azimuth_resolution_m          = ' + nominal_ground_azimuth_resolution_m          
      print, 'along_track_doppler_frequency_constant_term  = ' + along_track_doppler_frequency_constant_term  
      print, 'along_track_doppler_frequency_linear_term    = ' + along_track_doppler_frequency_linear_term    
      print, 'along_track_doppler_frequency_quadratic_term = ' + along_track_doppler_frequency_quadratic_term 
      print, 'cross_track_doppler_frequency_constant_term  = ' + cross_track_doppler_frequency_constant_term  
      print, 'cross_track_doppler_frequency_linear_term    = ' + cross_track_doppler_frequency_linear_term    
      print, 'cross_track_doppler_frequency_quadratic_term = ' + cross_track_doppler_frequency_quadratic_term 
      print, 'time_direction_along_pixel_direction         = ' + time_direction_along_pixel_direction         
      print, 'time_direction_along_line_direction          = ' + time_direction_along_line_direction          
      print, 'line_content_indicator                       = ' + line_content_indicator                       
      print, 'clutter_lock_applied                         = ' + clutter_lock_applied                         
      print, 'auto_focusing_applied                        = ' + auto_focusing_applied                        
      print, 'line_spacing_m                               = ' + line_spacing_m                               
      print, 'pixel_spacing_m                              = ' + pixel_spacing_m                              
      print, 'processor_range_compression_designator       = ' + processor_range_compression_designator       
      print, 'doppler_center_frequency_constant_term       = ' + doppler_center_frequency_constant_term       
      print, 'doppler_center_frequency_linear_term         = ' + doppler_center_frequency_linear_term         
      print, 'calibration_data_indicator                   = ' + calibration_data_indicator                   
      print, 'start_line_number_of_calibration_upper       = ' + start_line_number_of_calibration_upper       
      print, 'stop_line_number_of_calibration_upper        = ' + stop_line_number_of_calibration_upper        
      print, 'start_line_number_of_calibration_bottom      = ' + start_line_number_of_calibration_bottom      
      print, 'stop_line_number_of_calibration_bottom       = ' + stop_line_number_of_calibration_bottom       
      print, 'prf_switching_indicator                      = ' + prf_switching_indicator                      
      print, 'line_locator_of_prf_switching                = ' + line_locator_of_prf_switching                
      print, 'direction_of_beam_center                     = ' + direction_of_beam_center                     
      print, 'yaw_steerting_mode_flag                      = ' + yaw_steerting_mode_flag                      
      print, 'parameter_table_number                       = ' + parameter_table_number                       
      print, 'nominal_off_nadir_angle                      = ' + nominal_off_nadir_angle                      
      print, 'antenna_beam_number                          = ' + antenna_beam_number                          
      print, 'incidence_angle_constant_term                = ' + incidence_angle_constant_term                
      print, 'incidence_angle_linear_term                  = ' + incidence_angle_linear_term                  
      print, 'incidence_angle_quadratic_term               = ' + incidence_angle_quadratic_term               
      print, 'incidence_angle_cubic_term                   = ' + incidence_angle_cubic_term                   
      print, 'incidence_angle_fourth_term                  = ' + incidence_angle_fourth_term                   
      print, 'incidence_angle_fifth_term                   = ' + incidence_angle_fifth_term                   
      print, 'number_of_annotation_points                  = ' + number_of_annotation_points                  
      print, 'FINISHED DATA SUMMARY RECORD'
      if keyword_set(pause) then stop
    endif
  endfor 
  if isnumber( sensor_platform_heading_deg) then lf.sensor_heading   = sensor_platform_heading_deg
  lf.number_of_sar_channels           = number_of_sar_channels 
  lf.product_type                     = processing_facility_identifier
  lf.scene_identifier                 = scene_identifier
  lf.asc_des                          = time_direction_along_line_direction
  lf.antenna_beam_number              = antenna_beam_number 
  lf.receiver_gain_like_polarization  = receiver_gain_like_polarization
  lf.ellipsoid_designator            = ellipsoid_designator
endif else begin 
  if printflag then print, 'THERE IS NO DATA SUMMARY RECORD'
  if keyword_set(pause) then stop
endelse
;
; Sample Range Gate Computations
;

;
;------------------------------------------
; Map Projection Record
;------------------------------------------
;
if number_of_map_projection_data_records      gt 0 then begin
  for i=0L, number_of_map_projection_data_records-1  do  begin     
    readu, 1, map_projection_rec
    if eflag then begin
      record_sequence_number                     = string(long(        map_projection_rec[0:3],  0))
    endif else begin
      record_sequence_number                     = string(long(reverse(map_projection_rec[0:3]), 0))
    endelse
    first_record_subtype_code                    = string(fix(byte(map_projection_rec[4])))
    record_type_code                             = string(fix(byte(map_projection_rec[5])))
    second_record_subtype_code                   = string(fix(byte(map_projection_rec[6])))
    third_record_subtype_code                    = string(fix(byte(map_projection_rec[7])))
    if eflag then begin
      length_of_this_record                      = string(long(        map_projection_rec(8:11), 0 ))
    endif else begin
      length_of_this_record                      = string(long(reverse(map_projection_rec(8:11)),0 ))
    endelse
    map_projection_descriptor                    = string(  map_projection_rec[  28:  59] )
    number_of_pixels_per_line                    = string(  map_projection_rec[  60:  75] )
    number_of_lines                              = string(  map_projection_rec[  76:  91] )
    nominal_inter_pixel_distance_m               = string(  map_projection_rec[  92: 107] )
    nominal_inter_line_distance_m                = string(  map_projection_rec[ 108: 123] )
    orientation_of_output_scence_center          = string(  map_projection_rec[ 124: 139] )
    platform_inclination_deg                     = string(  map_projection_rec[ 140: 155] ) 
    ascending_node_deg                           = string(  map_projection_rec[ 156: 171] )
    distance_to_scene_center_m                   = string(  map_projection_rec[ 172: 187] )
    geodetic_altitude_m                          = string(  map_projection_rec[ 188: 203] )
    ground_speed_nadir_mps                       = string(  map_projection_rec[ 204: 219] )                 
    platform_heading_m                           = string(  map_projection_rec[ 220: 235] )
    reference_elliposid                          = string(  map_projection_rec[ 236: 267] )
    semi_major_axis_ellipsoid_m                  = string(  map_projection_rec[ 268: 283] )
    semi_minor_axis_ellipsoid_m                  = string(  map_projection_rec[ 284: 299] )
    datum_shift_parameter_referenced_to_green    = string(  map_projection_rec[ 300: 315] )
    datum_shift_parameter_perpend_to_green       = string(  map_projection_rec[ 316: 331] )
    datum_shift_parameter_direct_rot             = string(  map_projection_rec[ 332: 347] )
    datum_shift_parameter_direct_rot1            = string(  map_projection_rec[ 348: 363] )
    datum_shift_parameter_direct_rot2            = string(  map_projection_rec[ 364: 378] )
    datum_shift_parameter_direct_rot3            = string(  map_projection_rec[ 379: 395] )
    scale_factor_ellipsoid                       = string(  map_projection_rec[ 396: 411] )
    map_projection                               = string(  map_projection_rec[ 412: 443] )
    utm_descriptor_first_default                 = string(  map_projection_rec[ 444: 475] )
    signature_of_utm_zone                        = string(  map_projection_rec[ 476: 479] )
    map_origin_easting_m                         = string(  map_projection_rec[ 480: 495] )
    map_origin_northing_m                        = string(  map_projection_rec[ 496: 511] )
    projection_center_longitude_deg              = string(  map_projection_rec[ 512: 527] )
    projection_center_latitude_deg               = string(  map_projection_rec[ 528: 543] )
    first_standard_parallel                      = string(  map_projection_rec[ 544: 559] )
    second_standard_parallel                     = string(  map_projection_rec[ 560: 575] )
    scale_factor                                 = string(  map_projection_rec[ 576: 591] )
    utm_descriptor_second_default                = string(  map_projection_rec[ 592: 623] ) 
    projection2_center_longitude_deg             = string(  map_projection_rec[ 624: 639] )
    projection2_center_latitude_deg              = string(  map_projection_rec[ 640: 655] )
    scale_factor2                                = string(  map_projection_rec[ 656: 671] )
    projection_descriptor                        = string(  map_projection_rec[ 672: 703] )
    map_origin_easting2_m                        = string(  map_projection_rec[ 703: 719] )
    map_origin_northing2_m                       = string(  map_projection_rec[ 720: 735] )
    projection_center_longitude2_deg             = string(  map_projection_rec[ 736: 751] )
    projection_center_latitude2_deg              = string(  map_projection_rec[ 752: 767] )
    first_standard_parallel2                     = string(  map_projection_rec[ 768: 783] )
    second_standard_parallel2                    = string(  map_projection_rec[ 784: 799] )
    top_left_corner_northing_km                  = string(  map_projection_rec[ 944: 959] )
    top_left_corner_easting_km                   = string(  map_projection_rec[ 960: 975] )
    top_right_corner_northing_km                 = string(  map_projection_rec[ 976: 991] )
    top_right_corner_easting_km                  = string(  map_projection_rec[ 992:1007] )
    bottom_right_corner_northing_km              = string(  map_projection_rec[1008:1023] )
    bottom_right_corner_easting_km               = string(  map_projection_rec[1024:1039] )
    bottom_left_corner_northing_km               = string(  map_projection_rec[1040:1055] )
    bottom_left_corner_easting_km                = string(  map_projection_rec[1056:1071] )
    top_left_corner_latitude_deg                 = string(  map_projection_rec[1072:1087] )
    top_left_corner_longitude_deg                = string(  map_projection_rec[1088:1103] )
    top_right_corner_latitude_deg                = string(  map_projection_rec[1104:1119] )
    top_right_corner_longitude_deg               = string(  map_projection_rec[1120:1135] )
    bottom_right_corner_latitude_deg             = string(  map_projection_rec[1136:1151] )
    bottom_right_corner_longitude_deg            = string(  map_projection_rec[1152:1167] )
    bottom_left_corner_latitude_deg              = string(  map_projection_rec[1168:1183] )
    bottom_left_corner_longitude_deg             = string(  map_projection_rec[1184:1199] )
    top_left_corner_terrain_height               = string(  map_projection_rec[1200:1215] )
    top_right_corner_terrain_height              = string(  map_projection_rec[1216:1231] )
    bottom_right_corner_terrain_height           = string(  map_projection_rec[1232:1247] )
    bottom_left_corner_terrain_height            = string(  map_projection_rec[1248:1263] )
    a11                                          = string(  map_projection_rec[1264:1283] )
    a12                                          = string(  map_projection_rec[1284:1303] )
    a13                                          = string(  map_projection_rec[1304:1323] )
    a14                                          = string(  map_projection_rec[1324:1343] ) 
    a21                                          = string(  map_projection_rec[1344:1363] )
    a22                                          = string(  map_projection_rec[1364:1383] )
    a23                                          = string(  map_projection_rec[1384:1403] )
    a24                                          = string(  map_projection_rec[1404:1423] )
    b11                                          = string(  map_projection_rec[1424:1443] )
    b12                                          = string(  map_projection_rec[1444:1463] )
    b13                                          = string(  map_projection_rec[1464:1483] )
    b14                                          = string(  map_projection_rec[1484:1503] ) 
    b21                                          = string(  map_projection_rec[1504:1523] )
    b22                                          = string(  map_projection_rec[1524:1543] )
    b23                                          = string(  map_projection_rec[1544:1563] )
    b24                                          = string(  map_projection_rec[1564:1583] ) 
    if printflag then begin 
      print, ''
      print, '----------------------'
      print, 'MAP PROJECTION RECORD:'
      print, '----------------------'
      print, 'record_sequence_number                       = ' + record_sequence_number                       
      print, 'length_of_this_record                        = ' + length_of_this_record
      print, 'first_record_subtype_code                    = ' + first_record_subtype_code                    
      print, 'record_type_code                             = ' + record_type_code                             
      print, 'second_record_subtype_code                   = ' + second_record_subtype_code                   
      print, 'third_record_subtype_code                    = ' + third_record_subtype_code                    
      print, 'length_of_this_record                        = ' + length_of_this_record                        
      print, 'map_projection_descriptor                    = ' + map_projection_descriptor                    
      print, 'number_of_pixels_per_line                    = ' + number_of_pixels_per_line                    
      print, 'number_of_lines                              = ' + number_of_lines                              
      print, 'nominal_inter_pixel_distance_m               = ' + nominal_inter_pixel_distance_m               
      print, 'nominal_inter_line_distance_m                = ' + nominal_inter_line_distance_m                
      print, 'orientation_of_output_scence_center          = ' + orientation_of_output_scence_center          
      print, 'platform_inclination_deg                     = ' + platform_inclination_deg                     
      print, 'ascending_node_deg                           = ' + ascending_node_deg                           
      print, 'distance_to_scene_center_m                   = ' + distance_to_scene_center_m                   
      print, 'geodetic_altitude_m                          = ' + geodetic_altitude_m                          
      print, 'ground_speed_nadir_mps                       = ' + ground_speed_nadir_mps                       
      print, 'platform_heading_m                           = ' + platform_heading_m                           
      print, 'reference_elliposid                          = ' + reference_elliposid                          
      print, 'semi_major_axis_ellipsoid_m                  = ' + semi_major_axis_ellipsoid_m                  
      print, 'semi_minor_axis_ellipsoid_m                  = ' + semi_minor_axis_ellipsoid_m                  
      print, 'datum_shift_parameter_referenced_to_green    = ' + datum_shift_parameter_referenced_to_green    
      print, 'datum_shift_parameter_perpend_to_green       = ' + datum_shift_parameter_perpend_to_green        
      print, 'datum_shift_parameter_direct_rot             = ' + datum_shift_parameter_direct_rot              
      print, 'datum_shift_parameter_direct_rot1            = ' + datum_shift_parameter_direct_rot1             
      print, 'datum_shift_parameter_direct_rot2            = ' + datum_shift_parameter_direct_rot2             
      print, 'datum_shift_parameter_direct_rot3            = ' + datum_shift_parameter_direct_rot3             
      print, 'scale_factor_ellipsoid                       = ' + scale_factor_ellipsoid                       
      print, 'map_projection                               = ' + map_projection                               
      print, 'utm_descriptor_first_default                 = ' + utm_descriptor_first_default                 
      print, 'signature_of_utm_zone                        = ' + signature_of_utm_zone                        
      print, 'map_origin_easting_m                         = ' + map_origin_easting_m                         
      print, 'map_origin_northing_m                        = ' + map_origin_northing_m                        
      print, 'projection_center_longitude_deg              = ' + projection_center_longitude_deg              
      print, 'projection_center_latitude_deg               = ' + projection_center_latitude_deg               
      print, 'first_standard_parallel                      = ' + first_standard_parallel                      
      print, 'second_standard_parallel                     = ' + second_standard_parallel                     
      print, 'scale_factor                                 = ' + scale_factor                                 
      print, 'utm_descriptor_second_default                = ' + utm_descriptor_second_default                
      print, 'projection2_center_longitude_deg             = ' + projection2_center_longitude_deg             
      print, 'projection2_center_latitude_deg              = ' + projection2_center_latitude_deg              
      print, 'scale_factor2                                = ' + scale_factor2                                
      print, 'projection_descriptor                        = ' + projection_descriptor                        
      print, 'map_origin_easting2_m                        = ' + map_origin_easting2_m                        
      print, 'map_origin_northing2_m                       = ' + map_origin_northing2_m                       
      print, 'projection_center_longitude2_deg             = ' + projection_center_longitude2_deg             
      print, 'projection_center_latitude2_deg              = ' + projection_center_latitude2_deg              
      print, 'first_standard_parallel2                     = ' + first_standard_parallel2                     
      print, 'second_standard_parallel2                    = ' + second_standard_parallel2                    
      print, 'top_left_corner_northing_km                  = ' + top_left_corner_northing_km                  
      print, 'top_left_corner_easting_km                   = ' + top_left_corner_easting_km                   
      print, 'top_right_corner_northing_km                 = ' + top_right_corner_northing_km                 
      print, 'top_right_corner_easting_km                  = ' + top_right_corner_easting_km                  
      print, 'bottom_right_corner_northing_km              = ' + bottom_right_corner_northing_km              
      print, 'bottom_right_corner_easting_km               = ' + bottom_right_corner_easting_km               
      print, 'bottom_left_corner_northing_km               = ' + bottom_left_corner_northing_km               
      print, 'bottom_left_corner_easting_km                = ' + bottom_left_corner_easting_km                
      print, 'top_left_corner_latitude_deg                 = ' + top_left_corner_latitude_deg                 
      print, 'top_left_corner_longitude_deg                = ' + top_left_corner_longitude_deg                
      print, 'top_right_corner_latitude_deg                = ' + top_right_corner_latitude_deg                
      print, 'top_right_corner_longitude_deg               = ' + top_right_corner_longitude_deg               
      print, 'bottom_right_corner_latitude_deg             = ' + bottom_right_corner_latitude_deg             
      print, 'bottom_right_corner_longitude_deg            = ' + bottom_right_corner_longitude_deg            
      print, 'bottom_left_corner_latitude_deg              = ' + bottom_left_corner_latitude_deg              
      print, 'bottom_left_corner_longitude_deg             = ' + bottom_left_corner_longitude_deg             
      print, 'top_left_corner_terrain_height               = ' + top_left_corner_terrain_height               
      print, 'top_right_corner_terrain_height              = ' + top_right_corner_terrain_height              
      print, 'bottom_right_corner_terrain_height           = ' + bottom_right_corner_terrain_height           
      print, 'bottom_left_corner_terrain_height            = ' + bottom_left_corner_terrain_height            
      print, 'a11                                          = ' + a11                                          
      print, 'a12                                          = ' + a12                                          
      print, 'a13                                          = ' + a13                                          
      print, 'a14                                          = ' + a14                                          
      print, 'a21                                          = ' + a21                                          
      print, 'a22                                          = ' + a22                                          
      print, 'a23                                          = ' + a23                                          
      print, 'a24                                          = ' + a24                                          
      print, 'b11                                          = ' + b11                                          
      print, 'b12                                          = ' + b12                                          
      print, 'b13                                          = ' + b13                                          
      print, 'b14                                          = ' + b14                                          
      print, 'b21                                          = ' + b21                                          
      print, 'b22                                          = ' + b22                                          
      print, 'b23                                          = ' + b23                                          
      print, 'b24                                          = ' + b24    
      print, 'FINSIHED READING MAP PROJECTION RECORD'
      if keyword_set(pause) then stop
   endif
  endfor
endif else begin 
  if printflag then print, 'THERE IS NO MAP PROJECTION RECORD'
  if keyword_set(pause) then stop
endelse

;
;------------------------------------------
; Platform Position Record
;------------------------------------------
;
if number_of_platform_position_records        gt 0 then begin
  for i=0L, number_of_platform_position_records-1 do begin
    readu, 1, platform_position_rec
    if eflag then begin
      record_sequence_number                     = string(long(        platform_position_rec[0:3],  0))
    endif else begin
      record_sequence_number                     = string(long(reverse(platform_position_rec[0:3]), 0))
    endelse                                                                             
    first_record_subtype_code                    = string(fix(byte(platform_position_rec[4])))
    record_type_code                             = string(fix(byte(platform_position_rec[5])))
    second_record_subtype_code                   = string(fix(byte(platform_position_rec[6])))
    third_record_subtype_code                    = string(fix(byte(platform_position_rec[7])))
    if eflag then begin
      length_of_this_record                      = string(long(        platform_position_rec[8:11], 0 ))
    endif else begin
      length_of_this_record                      = string(long(reverse(platform_position_rec[8:11]),0 ))
    endelse
    orbital_elements_designator                  = string(  platform_position_rec[ 12: 43] )
    first_orbit_element_x_m                      = string(  platform_position_rec[ 44: 59] )
    second_orbit_element_y_m                     = string(  platform_position_rec[ 60: 75] )
    third_orbit_element_z_m                      = string(  platform_position_rec[ 76: 91] )
    fourth_orbit_element_x_m_s                   = string(  platform_position_rec[ 92:107] )
    fifth_orbit_element_y_m_s                    = string(  platform_position_rec[108:123] )
    sixth_orbit_element_z_m_z                    = string(  platform_position_rec[124:139] )
    number_of_data_points                        = string(  platform_position_rec[140:143] )
    year_of_data_point                           = string(  platform_position_rec[144:147] )
    month_of_data_point                          = string(  platform_position_rec[148:151] )
    day_of_data_point                            = string(  platform_position_rec[152:155] )
    day_in_the_year                              = string(  platform_position_rec[156:159] )
    seconds_of_the_day                           = string(  platform_position_rec[160:181] )
    time_interval_between_data_points            = string(  platform_position_rec[182:203] )
    reference_coordinate_system                  = string(  platform_position_rec[204:267] )
    greenwich_mean_hour_angle                    = string(  platform_position_rec[268:289] )
    along_track_position_error_m                 = string(  platform_position_rec[290:305] )
    across_track_position_error_m                = string(  platform_position_rec[306:321] )
    radial_postion_error_m                       = string(  platform_position_rec[322:337] )
    along_track_velocity_error_m_s               = string(  platform_position_rec[338:353] )
    across_track_velocityerror_m_s               = string(  platform_position_rec[354:369] )
    radial_velocity_error_m_s                    = string(  platform_position_rec[370:385] )
    ; 29 data points are not read here
    if printflag then begin
      print, ''
      print, '-------------------------'
      print, 'PLATFORM POSITION RECORD:'
      print, '-------------------------' 
      print, 'record_sequence_number             =' +  record_sequence_number             
      print, 'length_of_this_record              =' +  length_of_this_record              
      print, 'first_record_subtype_code          =' +  first_record_subtype_code          
      print, 'record_type_code                   =' +  record_type_code                   
      print, 'second_record_subtype_code         =' +  second_record_subtype_code         
      print, 'third_record_subtype_code          =' +  third_record_subtype_code          
      print, 'length_of_this_record              =' +  length_of_this_record            
      print, 'orbital_elements_designator        =' +  orbital_elements_designator        
      print, 'first_orbit_element_x_m            =' +  first_orbit_element_x_m            
      print, 'second_orbit_element_y_m           =' +  second_orbit_element_y_m           
      print, 'third_orbit_element_z_m            =' +  third_orbit_element_z_m            
      print, 'fourth_orbit_element_x_m_s         =' +  fourth_orbit_element_x_m_s         
      print, 'fifth_orbit_element_y_m_s          =' +  fifth_orbit_element_y_m_s          
      print, 'sixth_orbit_element_z_m_z          =' +  sixth_orbit_element_z_m_z          
      print, 'number_of_data_points              =' +  number_of_data_points              
      print, 'year_of_data_point                 =' +  year_of_data_point                 
      print, 'month_of_data_point                =' +  month_of_data_point                
      print, 'day_of_data_point                  =' +  day_of_data_point                  
      print, 'day_in_the_year                    =' +  day_in_the_year                    
      print, 'seconds_of_the_day                 =' +  seconds_of_the_day                 
      print, 'time_interval_between_data_points  =' +  time_interval_between_data_points  
      print, 'reference_coordinate_system        =' +  reference_coordinate_system        
      print, 'greenwich_mean_hour_angle          =' +  greenwich_mean_hour_angle          
      print, 'along_track_position_error_m       =' +  along_track_position_error_m       
      print, 'across_track_position_error_m      =' +  across_track_position_error_m      
      print, 'radial_postion_error_m             =' +  radial_postion_error_m             
      print, 'along_track_velocity_error_m_s     =' +  along_track_velocity_error_m_s     
      print, 'across_track_velocityerror_m_s     =' +  across_track_velocityerror_m_s     
      print, 'radial_velocity_error_m_s          =' +  radial_velocity_error_m_s          
      print, 'FINISHED READING PLATFORM POSITION RECORD'
      if keyword_set(pause) then stop
    endif
  endfor
endif else begin 
  if printflag then print, 'THERE IS NO PLATFORM POSITION RECORD'
  if keyword_set(pause) then stop
endelse
;
;---------------------------------------------------------
; Attitude Data Record
;----------------------------------------------------------  
; 
if number_of_attitude_data_records            gt 0 then begin
  for i=0L, number_of_attitude_data_records-1 do  begin
    readu, 1, attitude_data_rec
    if eflag then begin
      record_sequence_number                     = string(long(        attitude_data_rec[0:3],  0))
    endif else begin
      record_sequence_number                     = string(long(reverse(attitude_data_rec[0:3]), 0))
    endelse
    first_record_subtype_code                    = string(fix(byte(attitude_data_rec[4])))
    record_type_code                             = string(fix(byte(attitude_data_rec[5])))
    second_record_subtype_code                   = string(fix(byte(attitude_data_rec[6])))
    third_record_subtype_code                    = string(fix(byte(attitude_data_rec[7])))
    if eflag then begin
      length_of_this_record                      = string(long(        attitude_data_rec[8:11], 0 ))
    endif else begin
      length_of_this_record                      = string(long(reverse(attitude_data_rec[8:11]),0 ))
    endelse
    number_of_data_points                        = string(attitude_data_rec[ 12: 15] )
    day_of_the_year                              = string(attitude_data_rec[ 16: 19] )
    millisec_of_the_day                          = string(attitude_data_rec[ 20: 27] )
    pitch_data_quality_flag                      = string(attitude_data_rec[ 28: 31] )
    roll_data_quality_flag                       = string(attitude_data_rec[ 32: 35] )
    yaw_data_quality_flag                        = string(attitude_data_rec[ 36: 39] )
    pitch_deg                                    = string(attitude_data_rec[ 40: 53] )
    roll_deg                                     = string(attitude_data_rec[ 54: 67] )
    yaw_deg                                      = string(attitude_data_rec[ 68: 81] )
    pitch_rate_data_quality_flag                 = string(attitude_data_rec[ 82: 85] )
    roll_rate_data_data_quality_flag             = string(attitude_data_rec[ 86: 89] )
    yaw_rate_data_data_quality_flag              = string(attitude_data_rec[ 90: 93] )
    pitch_rate_data_deg_s                        = string(attitude_data_rec[ 94:107] )
    roll_rate_data_deg_s                         = string(attitude_data_rec[108:121] )
    yaw_rate_data_deg_s                          = string(attitude_data_rec[122:135] )
    ; 
    ; Other attitude data points not read here
    ;
    if printflag then begin
      print, ''
      print, '---------------------'
      print, 'ATTITUDE DATA RECORD:'
      print, '---------------------'                  
      print, 'record_sequence_number             =' +  record_sequence_number                
      print, 'length_of_this_record              =' +  length_of_this_record              
      print, 'first_record_subtype_code          =' +  first_record_subtype_code             
      print, 'record_type_code                   =' +  record_type_code                      
      print, 'second_record_subtype_code         =' +  second_record_subtype_code            
      print, 'third_record_subtype_code          =' +  third_record_subtype_code             
      print, 'length_of_this_record              =' +  length_of_this_record               
      print, 'number_of_data_points              =' +  number_of_data_points                 
      print, 'day_of_the_year                    =' +  day_of_the_year                       
      print, 'millisec_of_the_day                =' +  millisec_of_the_day                   
      print, 'pitch_data_quality_flag            =' +  pitch_data_quality_flag               
      print, 'roll_data_quality_flag             =' +  roll_data_quality_flag                
      print, 'yaw_data_quality_flag              =' +  yaw_data_quality_flag                 
      print, 'pitch_deg                          =' +  pitch_deg                             
      print, 'roll_deg                           =' +  roll_deg                              
      print, 'yaw_deg                            =' +  yaw_deg                               
      print, 'pitch_rate_data_quality_flag       =' +  pitch_rate_data_quality_flag          
      print, 'roll_rate_data_data_quality_flag   =' +  roll_rate_data_data_quality_flag      
      print, 'yaw_rate_data_data_quality_flag    =' +  yaw_rate_data_data_quality_flag       
      print, 'pitch_rate_data_deg_s              =' +  pitch_rate_data_deg_s                 
      print, 'roll_rate_data_deg_s               =' +  roll_rate_data_deg_s                  
      print, 'yaw_rate_data_deg_s                =' +  yaw_rate_data_deg_s                   
      print, 'FINISHED READING ATTITUDE DATA RECORD'
      if keyword_set(pause) then stop
    endif
  endfor
endif else begin 
  if printflag then print, 'THERE IS NO ATTITUDE POSITION RECORD'
  if keyword_set(pause) then stop
endelse
;
;
;---------------------------------------------------------
; Radiometric Data Record
;----------------------------------------------------------  
; 
if number_of_radiometric_records              gt 0 then begin
  for i=0L, number_of_radiometric_records-1 do begin
    readu, 1, radiometric_rec
    if eflag then begin
      record_sequence_number                     = string(long(        radiometric_rec[0:3],  0))
    endif else begin
      record_sequence_number                     = string(long(reverse(radiometric_rec[0:3]), 0))
    endelse
    first_record_subtype_code                    = string(fix(byte(radiometric_rec[4])))
    record_type_code                             = string(fix(byte(radiometric_rec[5])))
    second_record_subtype_code                   = string(fix(byte(radiometric_rec[6])))
    third_record_subtype_code                    = string(fix(byte(radiometric_rec[7])))
    if eflag then begin
      length_of_this_record                      = string(long(        radiometric_rec[8:11], 0 ))
    endif else begin
      length_of_this_record                      = string(long(reverse(radiometric_rec[8:11]),0 ))
    endelse   
    radiometric_data_record_sequence_number      = string(  radiometric_rec[ 12: 15] )
    number_of_radiometric_data_record_fields     = string(  radiometric_rec[ 16: 19] )
    calibration_factor                           = string(  radiometric_rec[ 20: 35] )
    ; Level 1:   sigma0 = 10 * log_10 <I^2+Q^2) + calibration_factor - 69.5
    ; Level 1.5  sigma0 = 10 * log_10 <DN^2>    + calibration_factor 
    real_part_transmission_distortion_11         = string(  radiometric_rec[ 36: 51] )
    imag_part_transmission_distortion_11         = string(  radiometric_rec[ 52: 67] )
    real_part_transmission_distortion_12         = string(  radiometric_rec[ 68: 83] )
    imag_part_transmission_distortion_12         = string(  radiometric_rec[ 84: 99] )
    real_part_transmission_distortion_21         = string(  radiometric_rec[100:115] )
    imag_part_transmission_distortion_21         = string(  radiometric_rec[116:131] )
    real_part_transmission_distortion_22         = string(  radiometric_rec[132:147] )
    imag_part_transmission_distortion_22         = string(  radiometric_rec[148:163] )
    real_part_reception_distortion_11            = string(  radiometric_rec[164:179] )
    imag_part_reception_distortion_11            = string(  radiometric_rec[180:195] )
    real_part_reception_distortion_12            = string(  radiometric_rec[196:211] )
    imag_part_reception_distortion_12            = string(  radiometric_rec[212:227] )
    real_part_reception_distortion_21            = string(  radiometric_rec[228:243] )
    imag_part_reception_distortion_21            = string(  radiometric_rec[244:259] )
    real_part_reception_distortion_22            = string(  radiometric_rec[260:275] )
    imag_part_reception_distortion_22            = string(  radiometric_rec[276:291] )
    if printflag then begin
      print, ''
      print, '------------------------'
      print, 'RADIOMETRIC DATA RECORD:'
      print, '------------------------'
      print, 'record_sequence_number                   =' + record_sequence_number                   
      print, 'length_of_this_record                    =' + length_of_this_record                    
      print, 'first_record_subtype_code                =' + first_record_subtype_code                
      print, 'record_type_code                         =' + record_type_code                         
      print, 'second_record_subtype_code               =' + second_record_subtype_code               
      print, 'third_record_subtype_code                =' + third_record_subtype_code                
      print, 'length_of_this_record                    =' + length_of_this_record                    
      print, 'radiometric_data_record_sequence_number  =' + radiometric_data_record_sequence_number  
      print, 'number_of_radiometric_data_record_fields =' + number_of_radiometric_data_record_fields 
      print, 'calibration_factor                       =' + calibration_factor                        
      print, 'real_part_transmission_distortion_11     =' + real_part_transmission_distortion_11     
      print, 'imag_part_transmission_distortion_11     =' + imag_part_transmission_distortion_11     
      print, 'real_part_transmission_distortion_12     =' + real_part_transmission_distortion_12     
      print, 'imag_part_transmission_distortion_12     =' + imag_part_transmission_distortion_12     
      print, 'real_part_transmission_distortion_21     =' + real_part_transmission_distortion_21     
      print, 'imag_part_transmission_distortion_21     =' + imag_part_transmission_distortion_21     
      print, 'real_part_transmission_distortion_22     =' + real_part_transmission_distortion_22     
      print, 'imag_part_transmission_distortion_22     =' + imag_part_transmission_distortion_22     
      print, 'real_part_reception_distortion_11        =' + real_part_reception_distortion_11        
      print, 'imag_part_reception_distortion_11        =' + imag_part_reception_distortion_11        
      print, 'real_part_reception_distortion_12        =' + real_part_reception_distortion_12        
      print, 'imag_part_reception_distortion_12        =' + imag_part_reception_distortion_12        
      print, 'real_part_reception_distortion_21        =' + real_part_reception_distortion_21        
      print, 'imag_part_reception_distortion_21        =' + imag_part_reception_distortion_21        
      print, 'real_part_reception_distortion_22        =' + real_part_reception_distortion_22        
      print, 'imag_part_reception_distortion_22        =' + imag_part_reception_distortion_22        
      print, 'FINISHED READING RADIOMETRIC DATA  RECORD'
      if keyword_set(pause) then stop
    endif    
  endfor
endif else begin 
  if printflag then print, 'THERE IS NO RADIOMETRIC DATA RECORD'
  if keyword_set(pause) then stop
endelse
;
  
if number_of_radiometric_compensation_records gt 0 then $
  for i=0L, number_of_radiometric_compensation_records-1 do   readu, 1, radiometric_compensation_rec
  
  
  
if number_of_data_quality_summary_records     gt 0 then begin
  for i=0L, number_of_data_quality_summary_records-1 do begin    
    readu, 1, data_quality_summary_rec
    if eflag then begin
      record_sequence_number                        = string(long(       data_quality_summary_rec[0:3],  0)) 
    endif else begin                               
      record_sequence_number                        = string(long(reverse(data_quality_summary_rec[0:3]), 0))
    endelse                                        
    first_record_subtype_code                       = string(fix(byte(data_quality_summary_rec[4])))
    record_type_code                                = string(fix(byte(data_quality_summary_rec[5])))
    second_record_subtype_code                      = string(fix(byte(data_quality_summary_rec[6])))
    third_record_subtype_code                       = string(fix(byte(data_quality_summary_rec[7])))
    if eflag then begin                            
      length_of_this_record                         = string(long(        data_quality_summary_rec[8:11], 0 ))
    endif else begin                               
      length_of_this_record                         = string(long(reverse(data_quality_summary_rec[8:11]),0 ))
    endelse                                        
    data_summary_quality_record_sequence_number     = string(   data_quality_summary_rec[ 12: 15] )
    sar_channel_indicator                           = string(   data_quality_summary_rec[ 16: 19] )
    date_of_last_calibration_update                 = string(   data_quality_summary_rec[ 20: 25] )
    number_of_channels                              = string(   data_quality_summary_rec[ 26: 29] )
    nominal_integrated_side_lobe_ratio              = string(   data_quality_summary_rec[ 30: 45] )
    nominal_peak_side_lobe_to_main_ratio            = string(   data_quality_summary_rec[ 46: 61] )
    nominal_azimuth_ambiguity                       = string(   data_quality_summary_rec[ 62: 77] )
    nominal_range_ambiguity                         = string(   data_quality_summary_rec[ 78: 93] )
    estimate_of_snr                                 = string(   data_quality_summary_rec[ 94:109] )
    actual_bit_error_rate                           = string(   data_quality_summary_rec[110:125] )
    nominal_slant_range_resolution_m                = string(   data_quality_summary_rec[126:141] )
    nominal_slant_azimuth_resolution_m              = string(   data_quality_summary_rec[142:157] )
    nominal_radiometric_resolution_db               = string(   data_quality_summary_rec[158:173] )
    instantaneous_radiometric_resolution_db         = string(   data_quality_summary_rec[174:189] )
    nominal_absolute_rad_cal_uncertainty_db         = string(   data_quality_summary_rec[190:205] )
    nominal_absolute_rad_cal_uncertainty2_db        = string(   data_quality_summary_rec[206:221] )
    nominal_relative_rad_cal_uncertainty_db         = string(   data_quality_summary_rec[222:237] )
    nominal_relative_rad_cal_uncertainty2_db        = string(   data_quality_summary_rec[238:253] )
    ;
    ; Same calibration for other channels 
    ;
    nominal_absolute_location_error_along_track_m   = string(   data_quality_summary_rec[734:749] )
    nominal_absolute_location_error_across_track_m  = string(   data_quality_summary_rec[750:765] )
    nominal_geometric_distortion_in_line_direction  = string(   data_quality_summary_rec[766:781] )
    nominal_geometric_distortion_in_pixel_direction = string(   data_quality_summary_rec[782:797] )
    nominal_geometric_distance_skew                 = string(   data_quality_summary_rec[798:813] )
    scene_orientation_error                         = string(   data_quality_summary_rec[814:829] )
    along_track_rel_misregistration_of_sar_channel  = string(   data_quality_summary_rec[830:845] )
    across_track_rel_misregistration_of_sar_channel = string(   data_quality_summary_rec[846:861] )
    ;
    ; Misregistration errors for other channels.
    ;
    if printflag then begin
      print, ''
      print, '-----------------------------'
      print, 'DATA QUALITY SUMMARY RECORDS:'
      print, '-----------------------------'
      print, 'record_sequence_number                         =' + record_sequence_number
      print, 'length_of_this_record                          =' + length_of_this_record                    
      print, 'first_record_subtype_code                      =' + first_record_subtype_code                   
      print, 'record_type_code                               =' + record_type_code                            
      print, 'second_record_subtype_code                     =' + second_record_subtype_code                  
      print, 'third_record_subtype_code                      =' + third_record_subtype_code                   
      print, 'length_of_this_record                          =' + length_of_this_record                       
      print, 'data_summary_quality_record_sequence_number    =' + data_summary_quality_record_sequence_number 
      print, 'sar_channel_indicator                          =' + sar_channel_indicator                       
      print, 'date_of_last_calibration_update                =' + date_of_last_calibration_update             
      print, 'number_of_channels                             =' + number_of_channels                          
      print, 'nominal_integrated_side_lobe_ratio             =' + nominal_integrated_side_lobe_ratio          
      print, 'nominal_peak_side_lobe_to_main_ratio           =' + nominal_peak_side_lobe_to_main_ratio        
      print, 'nominal_azimuth_ambiguity                      =' + nominal_azimuth_ambiguity                   
      print, 'nominal_range_ambiguity                        =' + nominal_range_ambiguity                     
      print, 'estimate_of_snr                                =' + estimate_of_snr                             
      print, 'actual_bit_error_rate                          =' + actual_bit_error_rate                       
      print, 'nominal_slant_range_resolution_m               =' + nominal_slant_range_resolution_m            
      print, 'nominal_slant_azimuth_resolution_m             =' + nominal_slant_azimuth_resolution_m          
      print, 'nominal_radiometric_resolution_db              =' + nominal_radiometric_resolution_db           
      print, 'instantaneous_radiometric_resolution_db        =' + instantaneous_radiometric_resolution_db     
      print, 'nominal_absolute_rad_cal_uncertainty_db        =' + nominal_absolute_rad_cal_uncertainty_db     
      print, 'nominal_absolute_rad_cal_uncertainty2_db       =' + nominal_absolute_rad_cal_uncertainty2_db    
      print, 'nominal_relative_rad_cal_uncertainty_db        =' + nominal_relative_rad_cal_uncertainty_db     
      print, 'nominal_relative_rad_cal_uncertainty2_db       =' + nominal_relative_rad_cal_uncertainty2_db    
      print, 'nominal_absolute_location_error_along_track_m  =' + nominal_absolute_location_error_along_track_m  
      print, 'nominal_absolute_location_error_across_track_m =' + nominal_absolute_location_error_across_track_m 
      print, 'nominal_geometric_distortion_in_line_direction =' + nominal_geometric_distortion_in_line_direction 
      print, 'nominal_geometric_distortion_in_pixel_direction=' + nominal_geometric_distortion_in_pixel_direction
      print, 'nominal_geometric_distance_skew                =' + nominal_geometric_distance_skew                
      print, 'scene_orientation_error                        =' + scene_orientation_error                       
      print, 'along_track_rel_misregistration_of_sar_channel =' + along_track_rel_misregistration_of_sar_channel 
      print, 'across_track_rel_misregistration_of_sar_channel=' + across_track_rel_misregistration_of_sar_channel      
      print, 'FINSIHED READING DATA QUALITY SUMMARY RECORD'
      if keyword_set(pause) then stop  
    endif
  endfor
  lf.nominal_slant_range_resolution_m               =       nominal_slant_range_resolution_m
  lf.nominal_slant_azimuth_resolution_m             =       nominal_slant_azimuth_resolution_m
endif else begin 
  if printflag then print, 'THERE IS NO DATA QUALITY SUMMARY RECORD'
  if keyword_set(pause) then stop
endelse
;
if number_of_data_histograms                  gt 0 then $
  for i=0L, number_of_data_histograms-1 do                    readu, 1, data_histogram_rec
if number_of_range_spectra_records            gt 0 then $
  for i=0L, number_of_range_spectra_records-1 do              readu, 1, range_spectra_rec
if number_of_dem_descriptor_records           gt 0 then $
  for i=0L,  number_of_dem_descriptor_records-1 do            readu, 1, dem_descriptor_rec
if number_of_radar_par_update_records         gt 0 then $
  for i=0l, number_of_radar_par_update_records-1 do           readu, 1, radar_par_update_rec
if number_of_annotation_data_records          gt 0 then $
  for i=0L, number_of_annotation_data_records-1 do            readu, 1, annotation_data_rec
if number_of_det_processing_records           gt 0 then $
  for i=0lL, number_of_det_processing_records-1 do            readu, 1, det_processing_rec
if number_of_calibration_records              gt 0 then $
  for i=0L, number_of_calibration_records-1 do                readu, 1, calibration_rec
if number_of_gcp_records                      gt 0 then $
  for i=0L, number_of_gcp_records-1 do                        readu, 1, gcp_rec
;
;--------------------------------------------------------------------
; Facility Related Document Record 
; -------------------------------------------------------------------
if printflag then begin
    print, '------------------------------------------
    print, ' FACILITY DATA RECORD
    print, '------------------------------------------
endif
flag_data_in_facility_data_record = 0
for i=0, 10 do begin
  ;
  ; Loop through facility data record.
  ;
  if printflag then print, 'Facility Data Record         =' + string(i) + ' ' + string(facility_data_record_length[i])
  if number_of_facility_data_records[i] gt 0 then begin
      fdr  =  bytarr(facility_data_record_length[i])
      readu, 1, fdr
      if printflag then begin
        if eflag then begin
          record_sequence_number     = long(          fdr[0:3],   0)
          length_of_this_record      = long(          fdr[8:11],  0)
        endif else begin          
          record_sequence_number    =  long(reverse(  fdr[0:3]),  0)
          length_of_this_record      = long(reverse(  fdr[8:11]), 0)
        endelse
        first_record_subtype_code   = fix(fdr[4])
        record_type_code            = fix(fdr[5])
        second_record_subtype_code  = fix(fdr[6])
        third_record_subtype_code   = fix(fdr[7])
        facility_data_record_sequence_number= string( fdr[12:16] )
        if printflag then begin
          print, '  record_sequence_number                =', strcompress( string(record_sequence_number               ))
          print, '  first_record_subtype_code             =', strcompress( string(first_record_subtype_code            ))
          print, '  record_type_code                      =', strcompress( string(record_type_code                     ))
          print, '  second_record_subtype_code            =', strcompress( string(second_record_subtype_code           ))
          print, '  third_record_subtype_code             =', strcompress( string(third_record_subtype_code            ))
          print, '  length_of_this_record                 =', strcompress( string(length_of_this_record                ))
          print, '  facility_data_record_sequence_number  =', strcompress( string(facility_data_record_sequence_number ))
        endif
      endif
  endif

  if i eq 10 and long(  string(facility_data_record_length[i]) ) gt 3104 then begin
   ;
   ; Fill in coefficients
   ; 
   p_exp = [4, 4, 4, 4, 4, $
            3, 3, 3, 3, 3, $
            2, 2, 2, 2, 2, $
            1, 1, 1, 1, 1, $
            0, 0, 0, 0, 0 ]
   l_exp = [ 4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0 ]
   phi_coef= fltarr(25)
   lam_coef= fltarr(25)
   coef    = strarr(50)
   j0      = 1024L
   dj      = 20L

   for j=0, 49 do begin
     j1 = j0 + j * dj
     j2=  j1 + dj -1
     coef[j]= string( fdr[ j1: j2] )
   endfor
   phi_coef = double( coef[0:24] )
   lam_coef = double( coef[25:49] )
   p0  = string( fdr[2024:2043] )
   l0  = string( fdr[2044:2063] )
   phi_exp = [4, 4, 4, 4, 4, $
            3, 3, 3, 3, 3, $
            2, 2, 2, 2, 2, $
            1, 1, 1, 1, 1, $
            0, 0, 0, 0, 0 ]
   lam_exp = [ 4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0, $
             4, 3, 2, 1, 0 ]
   j0      = 2064L
   dj      = 20L
    for j=0, 49 do begin
      j1 = j0 + j * dj
      j2=  j1 + dj -1
      coef[j]= string( fdr[ j1: j2] )
    endfor
    phi0  = string( fdr[3064:3083] )
    lam0  = string( fdr[3084:3103] )   
    ;
    ; Set flag so we know that lat/long data in facility data record
    ;
    flag_data_in_facility_data_record = 1
  endif
endfor
;
; Close leaderfile
;
close,1
if flag_data_in_facility_data_record then begin
  ; 
  ; Compute new coefficients
  ;
  nx    =  2L * long( scene_center_pixel_number )
  ny    =  2L * long( scene_center_line_number  )
  nsamp =  100L
  nxx   =  nx / nsamp
  nyy   =  ny / nsamp
  dx    =  double( nsamp    )
  dy    =  double( nsamp    )
  xx    =  dblarr( nxx, nyy )
  xxx   =  xx
  yy    =  dblarr( nxx, nyy )
  yyy   =  yy
  zz    =  dblarr( nxx, nyy )
  xline =  dindgen(nxx) * dx - double(p0)
  yline =  dindgen(nyy) * dy - double(l0)
  xline1=  dindgen(nxx) * dx
  yline1=  dindgen(nyy) * dy 
  for i=0L, nyy-1L do begin 
     xx[*,i]  = xline
     xxx[*,i] = xline1
  endfor
  for i=0l, nxx-1L do begin 
     yy[i,*]  = yline
     yyy[i,*] = yline1 
  endfor  
  xx2   =  xx  * xx
  xx3   =  xx2 * xx
  xx4   =  xx3 * xx
  yy2   =  yy  * yy
  yy3   =  yy2 * yy
  yy4   =  yy3 * yy
  lam   =  lam_coef[0]  * xx4 * yy4 + lam_coef[1]  * xx4 * yy3 + $
           lam_coef[2]  * xx4 * yy2 + lam_coef[3]  * xx4 * yy  + $
           lam_coef[4]  * xx4                                  + $
           lam_coef[5]  * xx3 * yy4 + lam_coef[6]  * xx3 * yy3 + $
           lam_coef[7]  * xx3 * yy2 + lam_coef[8]  * xx3 * yy  + $
           lam_coef[9]  * xx3      + $
           lam_coef[10] * xx2 * yy4 + lam_coef[11] * xx2 * yy3 + $
           lam_coef[12] * xx2 * yy2 + lam_coef[13] * xx2 * yy  + $
           lam_coef[14] * xx2                                  + $
           lam_coef[15] * xx  * yy4 + lam_coef[16] * xx  * yy3 + $
           lam_coef[17] * xx  * yy2 + lam_coef[18] * xx  * yy  + $
           lam_coef[19] * xx                                   + $
           lam_coef[20] * yy4       + lam_coef[21] * yy3       + $
           lam_coef[22] * yy2       + lam_coef[23] * yy        + $
           lam_coef[24]
  phi   =  phi_coef[0]  * xx4 * yy4 + phi_coef[1]  * xx4 * yy3 + $
           phi_coef[2]  * xx4 * yy2 + phi_coef[3]  * xx4 * yy  + $
           phi_coef[4]  * xx4                                  + $
           phi_coef[5]  * xx3 * yy4 + phi_coef[6]  * xx3 * yy3 + $
           phi_coef[7]  * xx3 * yy2 + phi_coef[8]  * xx3 * yy  + $
           phi_coef[9]  * xx3      + $
           phi_coef[10] * xx2 * yy4 + phi_coef[11] * xx2 * yy3 + $
           phi_coef[12] * xx2 * yy2 + phi_coef[13] * xx2 * yy  + $
           phi_coef[14] * xx2                                  + $
           phi_coef[15] * xx  * yy4 + phi_coef[16] * xx  * yy3 + $
           phi_coef[17] * xx  * yy2 + phi_coef[18] * xx  * yy  + $
           phi_coef[19] * xx                                   + $
           phi_coef[20] * yy4       + phi_coef[21] * yy3       + $
           phi_coef[22] * yy2       + phi_coef[23] * yy        + $
           phi_coef[24]
  
  
  
  xx    =  reform(xx ,  nxx * nyy )
  yy    =  reform(yy ,  nxx * nyy )
  xxx   =  reform(xxx,  nxx * nyy )
  yyy   =  reform(yyy,  nxx * nyy )
  lam   =  reform(lam,  nxx * nyy )
  phi   =  reform(phi,  nxx * nyy )
  
  deg   =  2
  
  ans_compute_fit, xxx, yyy,  lam,  deg, lon_coef, lon_xexp, lon_yexp , /quiet
  ans_compute_fit, xxx, yyy,  phi,  deg, lat_coef, lat_xexp, lat_yexp , /quiet
  ans_compute_fit, lam, phi,  xxx,  deg, i_coef  , i_xexp  , i_yexp   , /quiet
  ans_compute_fit, lam, phi,  yyy,  deg, j_coef  , j_xexp  , j_yexp   , /quiet
  
  lf.lon_coef  = lon_coef
  lf.lat_coef  = lat_coef
  lf.i_coef    = i_coef  
  lf.j_coef    = j_coef  
  lf.lon_xexp  = lon_xexp
  lf.lat_xexp  = lat_xexp
  lf.i_xexp    = i_xexp  
  lf.j_xexp    = j_xexp  
  lf.lon_yexp  = lon_yexp 
  lf.lat_yexp  = lat_yexp
  lf.i_yexp    = i_yexp  
  lf.j_yexp    = j_yexp  
  
  
  if printflag then begin
    lamfit   =  ans_poly_cal(xxx, yyy, lf.lon_coef, lf.lon_xexp,  lf.lon_yexp )
    phifit   =  ans_poly_cal(xxx, yyy, lf.lat_coef, lf.lat_xexp,  lf.lat_yexp )
    iifit    =  ans_poly_cal(lam, phi, lf.i_coef,   lf.i_xexp,    lf.i_yexp   )
    jjfit    =  ans_poly_cal(lam, phi, lf.j_coef,   lf.j_xexp,    lf.j_yexp   )
    cosang   =  cos(mean(phi) *!pi/180.)
    lamdiff  =  lam - lamfit
    phidiff  =  phi - phifit
    iidiff   =  xxx - iifit
    jjdiff  =   yyy - jjfit
    if printflag then begin
      mu = mean (     lamdiff  )
      sd = stdev(     lamdiff  )
      mx = max(   abs(lamdiff) )
      mn = min(   abs(lamdiff) )
      print, 'Longitude'
      print, 'Mean diff     = ' + string( mu ) + ' ' + string( mu *  100000.0 * cosang , format='(F14.9)') + ' m'
      print, 'Variance diff = ' + string( sd ) + ' ' + string( sd *  100000.0 * cosang , format='(F14.9)') + ' m'
      print, 'Max diff      = ' + string( mx ) + ' ' + string( mx *  100000.0 * cosang , format='(F14.9)') + ' m'
      print, 'Min diff      = ' + string( mn)  + ' ' + string( mn *  100000.0 * cosang , format='(F14.9)') + ' m'
      mu = mean (     phidiff  )
      sd = stdev(     phidiff  )
      mx = max(   abs(phidiff) )
      mn = min(   abs(phidiff) )
      print, 'Latitude
      print, 'Mean diff     = ' + string( mu ) + ' ' + string( mu * 100000.0 , format='(F14.9)') + ' m' 
      print, 'Variance diff = ' + string( sd ) + ' ' + string( sd * 100000.0 , format='(F14.9)') + ' m' 
      print, 'Max diff      = ' + string( mx ) + ' ' + string( mx * 100000.0 , format='(F14.9)') + ' m'
      print, 'Min diff      = ' + string( mn ) + ' ' + string( mn * 100000.0 , format='(F14.9)') + ' m'
      mu = mean (     iidiff  )
      sd = stdev(     iidiff  )
      mx = max(   abs(iidiff) )
      mn = min(   abs(iidiff) )
      print, 'Pixel'
      print, 'Mean diff     = ' + string( mu ) + ' ' + string( mu , format='(F14.9)') + ' pixels' 
      print, 'Variance diff = ' + string( sd ) + ' ' + string( sd , format='(F14.9)') + ' pixels' 
      print, 'Max diff      = ' + string( mx ) + ' ' + string( mx , format='(F14.9)') + ' pixels'
      print, 'Min diff      = ' + string( mn ) + ' ' + string( mn , format='(F14.9)') + ' pixels'
      mu = mean (     jjdiff  )
      sd = stdev(     jjdiff  )
      mx = max(   abs(jjdiff) )
      mn = min(   abs(jjdiff) )
      print, 'Line'
      print, 'Mean diff     = ' + string( mu ) + ' ' + string( mu , format='(F14.9)') + ' pixels' 
      print, 'Variance diff = ' + string( sd ) + ' ' + string( sd , format='(F14.9)') + ' pixels' 
      print, 'Max diff      = ' + string( mx ) + ' ' + string( mx , format='(F14.9)') + ' pixels'
      print, 'Min diff      = ' + string( mn ) + ' ' + string( mn , format='(F14.9)') + ' pixels'
      print, 'Corner comparisons'
      print, string('i', format='(A6)') + string('j', format='(A6)') +  $
             string('Long 2nd-order', format='(A20)') + string('Lat 2nd-order', format='(A20)') + $
             string('Long 4th-order', format='(A20)') + string('Lat 4th-order', format='(A20)') + '   m'
      px = double([0, nx-1, nx-1,     0])
      li = double([0,    0, ny-1,  ny-1])
      lo = ans_poly_cal(px, li, lf.lon_coef, lf.lon_xexp,  lf.lon_yexp )
      la = ans_poly_cal(px, li, lf.lat_coef, lf.lat_xexp,  lf.lat_yexp )
      px= px - double(p0)
      li= li - double(l0)
      px2= px  * px
      px3= px2 * px
      px4= px3 * px
      li2= li  * li
      li3= li2 * li
      li4= li3 * li   
      for k =0, 3 do begin          
         lo1   =  lam_coef[0]  * px4 * li4 + lam_coef[1]  * px4 * li3 + $
                  lam_coef[2]  * px4 * li2 + lam_coef[3]  * px4 * li  + $
                  lam_coef[4]  * px4                                  + $
                  lam_coef[5]  * px3 * li4 + lam_coef[6]  * px3 * li3 + $
                  lam_coef[7]  * px3 * li2 + lam_coef[8]  * px3 * li  + $
                  lam_coef[9]  * px3      + $
                  lam_coef[10] * px2 * li4 + lam_coef[11] * px2 * li3 + $
                  lam_coef[12] * px2 * li2 + lam_coef[13] * px2 * li  + $
                  lam_coef[14] * px2                                  + $
                  lam_coef[15] * px  * li4 + lam_coef[16] * px  * li3 + $
                  lam_coef[17] * px  * li2 + lam_coef[18] * px  * li  + $
                  lam_coef[19] * px                                   + $
                  lam_coef[20] * li4       + lam_coef[21] * li3       + $
                  lam_coef[22] * li2       + lam_coef[23] * li        + $
                  lam_coef[24]
         la1   =  phi_coef[0]  * px4 * li4 + phi_coef[1]  * px4 * li3 + $
                  phi_coef[2]  * px4 * li2 + phi_coef[3]  * px4 * li  + $
                  phi_coef[4]  * px4                                  + $
                  phi_coef[5]  * px3 * li4 + phi_coef[6]  * px3 * li3 + $
                  phi_coef[7]  * px3 * li2 + phi_coef[8]  * px3 * li  + $
                  phi_coef[9]  * px3      + $
                  phi_coef[10] * px2 * li4 + phi_coef[11] * px2 * li3 + $
                  phi_coef[12] * px2 * li2 + phi_coef[13] * px2 * li  + $
                  phi_coef[14] * px2                                  + $
                  phi_coef[15] * px  * li4 + phi_coef[16] * px  * li3 + $
                  phi_coef[17] * px  * li2 + phi_coef[18] * px  * li  + $
                  phi_coef[19] * px                                   + $
                  phi_coef[20] * li4       + phi_coef[21] * li3       + $
                  phi_coef[22] * li2       + phi_coef[23] * li        + $
                  phi_coef[24]
         ll2rb, lo[k], la[k], lo1[k], la1[k], r, b
         r    =   6378000. * r
         outstr =          string( px[k] + double(p0),  format='(I6)'   ) 
         outstr = outstr + string( li[k] + double(l0),  format='(I6)'   ) 
         outstr = outstr + string( lo[k],  format='(F20.9)') 
         outstr = outstr + string( la[k],  format='(F20.9)') 
         outstr = outstr + string( lo1[k], format='(F20.9)') 
         outstr = outstr + string( la1[k], format='(F20.9)') + string(r)
         print, outstr
      endfor
    endif
  endif
endif

; 
; Assign to leader structure
;
lf.product=         scene_identifier
lf.product_stage=   product_level_code
lf.proc_fac=        processing_facility_identifier
lf.software_ver=    processing_version_identifier
;
; Parse time
;
monthstr=[ ' ','jan', 'feb', 'mar', 'apr', 'may', 'jun', $
               'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
center_gmt=                        input_scene_center_time
lf.data_year=            fix( strmid(center_gmt,  0,  4) )
lf.month=      monthstr( fix( strmid(center_gmt,  4,  2) ) )
lf.day=                  fix( strmid(center_gmt,  6,  2) ) 
lf.hour=                 fix( strmid(center_gmt,  8,  2) ) 
lf.minute=               fix( strmid(center_gmt, 10,  2) )              
lf.second=             float( strmid(center_gmt, 12,  5) ) / 1000.0
lf.abs_orbit=          float(       orbit_number                         ) 
lf.x_position=        double(       first_orbit_element_x_m              )  
lf.y_position=        double(       second_orbit_element_y_m             )
lf.z_position=        double(       third_orbit_element_z_m              )
lf.x_velocity=        double(       fourth_orbit_element_x_m_s           )
lf.y_velocity=        double(       fifth_orbit_element_y_m_s            )
lf.z_velocity=        double(       sixth_orbit_element_z_m_z            )
if isnumber(processed_scene_center_latitude_deg ) gt 0 then lf.center_lat=  float( processed_scene_center_latitude_deg  ) 
if isnumber(processed_scene_center_longitude_deg) gt 0 then lf.center_lon=  float( processed_scene_center_longitude_deg ) 
lf.ext_cal_fact=      float(        calibration_factor                   )
if n_elements(map_projection) gt 0 then lf.projection=  map_projection
lf.proc_fac=                        'ALOS'
lf.polarization=     getwrd(sensor_id_mode_of_operation,2)
if lf.polarization eq '-H' then lf.polarization='HH'
if lf.polarization eq '-V' then lf.polarization='VV'
lf.orbit_number=      long(         orbit_number                         )
lf.leaderfile=        getwrd( leaderfile, 0, /last, del='/')
lf.pixel_spacing=     float(        pixel_spacing_m                      )
lf.line_spacing=      float(        line_spacing_m                       )
lf.range_pixel=       float(        pixel_spacing_m                      )
lf.azimuth_pixel=     float(        line_spacing_m                       )
lf.range_spacing=     float(        pixel_spacing_m                      )
lf.azimuth_spacing=   float(        line_spacing_m                       )
lf.radar_frequency=   float(        radar_wavelength_m                   )
lf.radar_frequency=   3E8 / lf.radar_frequency ; Hz
lf.radar_band=                       'L'                
lf.range_looks=       float(   nominal_effective_number_range_looks      )
lf.azimuth_looks=     float(   nominal_effective_number_azimuth_looks    )
if n_elements(top_left_corner_latitude_deg     ) gt 0 then lf.corner_ll[0] = float( top_left_corner_latitude_deg      )
if n_elements(top_left_corner_longitude_deg    ) gt 0 then lf.corner_ll[1] = float( top_left_corner_longitude_deg     )
if n_elements(top_right_corner_latitude_deg    ) gt 0 then lf.corner_ll[2] = float( top_right_corner_latitude_deg     )
if n_elements(top_right_corner_longitude_deg   ) gt 0 then lf.corner_ll[3] = float( top_right_corner_longitude_deg    )
if n_elements(bottom_right_corner_latitude_deg ) gt 0 then lf.corner_ll[4] = float( bottom_right_corner_latitude_deg  )
if n_elements(bottom_right_corner_longitude_deg) gt 0 then lf.corner_ll[5] = float( bottom_right_corner_longitude_deg )
if n_elements(bottom_left_corner_latitude_deg  ) gt 0 then lf.corner_ll[6] = float( bottom_left_corner_latitude_deg   )
if n_elements(bottom_left_corner_longitude_deg ) gt 0 then lf.corner_ll[7] = float( bottom_left_corner_longitude_deg  )

if incidence_angle_constant_term ne '                    ' then begin
  ;
  ; Only fill if the strings are not empty
  ;
  lf.incid_coefsr[0] =   double( incidence_angle_constant_term        )
  lf.incid_coefsr[1] =   double( incidence_angle_linear_term          )
  lf.incid_coefsr[2] =   double( incidence_angle_quadratic_term       )
  lf.incid_coefsr[3] =   double( incidence_angle_cubic_term           )
  lf.incid_coefsr[4] =   double( incidence_angle_fourth_term          )
  lf.incid_coefsr[5] =   double( incidence_angle_fifth_term           )
endif
              ;
              ; theta(rads)= a0 + a1 R = a2 R^2 + a3 R^3 + a4 R^4 + a5 R^5
              ; where R=slantrange in km;
;
; Remarked out. Now using the facility data record.
if flag_data_in_facility_data_record eq 0 then begin
  ;
  ; The facility data record does not contain the latitude and longitude information
  ; we revert to map_projection data
  ;
  if n_elements(a11) gt 0 then begin
    lf.lon_coef=  [ double(a11), double(a12), double(a13), double(a14), 0., 0.] 
    lf.lat_coef=  [ double(a21), double(a22), double(a23), double(a24), 0., 0.] 
    ;lf.lon_xexp=  [   0,   1,   0,   1, 2, 0]    
    ;lf.lat_xexp=  [   0,   1,   0,   1, 2, 0]
    ;lf.lon_yexp=  [   0,   0,   1,   1, 0, 2]
    ;lf.lat_yexp=  [   0,   0,   1,   1, 0, 2]
    
    lf.lon_xexp=  [   0,   0,   1,   1, 0, 2]    
    lf.lat_xexp=  [   0,   0,   1,   1, 0, 2]
    lf.lon_yexp=  [   0,   1,   0,   1, 2, 0]
    lf.lat_yexp=  [   0,   1,   0,   1, 2, 0]
    
    lf.j_coef=    [ double(b11), double(b12), double(b13), double(b14), 0., 0.] 
    lf.i_coef=    [ double(b21), double(b22), double(b23), double(b24), 0., 0.]
    lf.i_xexp=    [   0,   1,   0,   1, 2, 0]
    lf.j_xexp=    [   0,   1,   0,   1, 2, 0]
    lf.i_yexp=    [   0,   0,   1,   1, 0, 2]
    lf.j_yexp=    [   0,   0,   1,   1, 0, 2]
  endif
endif 


good_leader=1
asf_check4_180, lf, quiet=quiet
if lf.lngswitchflag and lf.lon_coef[0] lt 0 then begin
  ;
  ; convert the longitude coefficents for 0-360
  ;
  lf.lon_coef[0]= 360.0 + lf.lon_coef[0]
  lf.lon_coef[1]=   0.0 + lf.lon_coef[1]
  lf.lon_coef[2]=   0.0 + lf.lon_coef[2]
  lf.lon_coef[3]=   0.0 + lf.lon_coef[4]
  ; 
  ; Recompute the longitude/latitude to i,j
  ;
  if printflag then print, 'Recompute lon/lat to i/j conversion..
  ii=  dblarr(6400) ; 80 * 80
  jj=  ii
  lo=  ii
  la=  ii
  st=  double(10)
  count=-1L
  for i=0, 79L do begin
    iii= i * st
    for j=0, 79L do begin
      count=     count + 1L
      ii[count]= iii
      jj[count]= j * st
      lo[count]= ans_poly_cal( ii[count], jj[count], lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
      la[count]= ans_poly_cal( ii[count], jj[count], lf.lat_coef, lf.lat_xexp, lf.lat_yexp )
    endfor
  endfor
  ;Note that this is hardwired for PSAR
  ans_compute_fit, lo, la, ii, 2, coef, xexp, yexp
  lf.i_coef=  coef
  lf.i_xexp=  xexp
  lf.i_yexp=  yexp
  ans_compute_fit, lo, la, jj, 2, coef, xexp, yexp
  lf.j_coef=  coef
  lf.j_xexp=  xexp
  lf.j_yexp=  yexp
endif

lf.semi_major_axis_km=                 ellipsoid_semi_major_axis_km      
lf.semi_minor_axis_km=                 ellipsoid_semi_minor_axis_km
if isnumber(sensor_platform_latitude_deg) gt 0 then begin
  lf.sensor_platform_latitude_deg=       sensor_platform_latitude_deg
  geodetic_altitude_km=                  geodetic_altitude_m / 1000. 
  lf.geodetic_altitude_km=               geodetic_altitude_km 
endif
lf.incidence_angle_at_scene_center=    incidence_angle_at_scene_center
if n_elements(distance_to_scene_center_m) then lf.distance_to_scene_center_m=         distance_to_scene_center_m
lf.scene_center_line_number=           scene_center_line_number
lf.scene_center_pixel_number=          scene_center_pixel_number
if printflag then begin
  print, lf.semi_major_axis_km, lf.semi_minor_axis_km, lf.sensor_platform_latitude_deg, lf.geodetic_altitude_km
  print, lf.incidence_angle_at_scene_center,lf.scene_center_line_number, lf.scene_center_pixel_number
endif
lf.center_gmt= day_in_the_year                        +':' + $
               string(lf.hour,   format='(I2.2)')     +':' + $
               string(lf.minute, format='(I2.2)')     +':' + $
               string(lf.second, format='(F6.3)')
lf.center_gmt= strcompress(lf.center_gmt,/remove_all)
lf.proc_cent=          'ALOS'

if (lf.center_lon eq 0) then lf.center_lon = ans_poly_cal( lf.scene_center_pixel_number, lf.scene_center_line_number, lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
if (lf.center_lat eq 0) then lf.center_lat = ans_poly_cal( lf.scene_center_pixel_number, lf.scene_center_line_number, lf.lat_coef, lf.lat_xexp, lf.lat_yexp )


delta_t= range_gate_at_early_edge / 1e6;                 ; Convert to seconds
slant_range_computed_km= delta_t / 2.0 * 3E8             ; slant range in
                                                         ; m. Divide by 2 to
                                                         ; account for round-trip
                                                         ; travel time
slant_range_computed_km= slant_range_computed_km / 1e3   ; Convert to km

ellipsoid_semi_minor_axis_km= double(ellipsoid_semi_minor_axis_km)
ellipsoid_semi_major_axis_km= double(ellipsoid_semi_major_axis_km)
ec=sqrt( 1 - ellipsoid_semi_minor_axis_km * ellipsoid_semi_minor_axis_km  $ 
           / ellipsoid_semi_major_axis_km / ellipsoid_semi_major_axis_km )

phi_rad = 0.0
sin_phi = 0.0
r_curv  = 0.0
if isnumber(processed_scene_center_latitude_deg) then begin
  phi_rad=        !pi /180.0 * processed_scene_center_latitude_deg
  sin_phi=        sin(phi_rad)
  r_curv=         ellipsoid_semi_major_axis_km / sqrt( 1 - ec * ec  * sin_phi * sin_phi)
endif
incid_computed=0
if n_elements(geodetic_altitude) gt 0 then begin
  incid_computed= slant_range_computed_km * slant_range_computed_km - $
                  2.0 * r_curv * geodetic_altitude_km               - $
                  geodetic_altitude_km  * geodetic_altitude_km 
  incid_computed= incid_computed / 2.0 / slant_range_computed_km / r_curv               
  incid_computed= !pi - acos( incid_computed )               
  incid_computed= incid_computed * 180.0 / !pi 
endif
lf.r_curv=      r_curv*1000.
lf.slant_range_computed_km= slant_range_computed_km

if printflag and n_elements(geodetic_altitude) gt 0 then begin
  print, 'Incident Angle Computations'
  print, '-----------------------------------------'
  print, 'range_gate_at_early_edge (musec)=        ',  range_gate_at_early_edge
  print, 'range_gate_at_early_edge (sec)=          ',  delta_t
  print, 'slant_range_computed_km (km)=            ',  slant_range_computed_km
  print, 'ellipsoid_semi_minor_axis_km (km)=       ',  ellipsoid_semi_minor_axis_km
  print, 'ellipsoid_semi_major_axis_km (km)=       ',  ellipsoid_semi_major_axis_km
  print, 'eccentricity=                            ',  ec
  print, 'processed_scene_center_latitude_deg (deg)',  processed_scene_center_latitude_deg
  print, 'geodetic_altitude_km (km)=               ',  geodetic_altitude_km
  print, 'phi_rad (rads)=                          ',  phi_rad
  print, 'sin_phi=                                 ',  sin_phi
  print, 'radius of curvature (km)=                ',  r_curv
  print, 'near range incidence (deg)=              ',  incid_computed
  print, '-----------------------------------------'
endif
end







