;----------------------------------------------------------
;+
; NAME: PSAR_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out PALSAR data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;    psar_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   
; MODIFICATION
;   2007 Aug 22 Frank Monaldo 
;   2008 Sep 13 Chris Jackson  - 
;               Made changes in the Image Identification 
;               and Image Orientation block to ensure that
;               certain lines are always written out
;               Made sure PALSAR corners are written
;   2009 May 01 Frank Monaldo 
;               Corrected problem w/ print out of incident
;               angle coefficients
;   2009 Jul 27 Frank Monaldo. Added code to keep the program
;               even if the lf.scansar and lf.s_c_altitude do not exist. This
;               was required because these fields are not filled when reading
;               RSAT2 data
;   2009 Oct 14 Radically re-written for new software version 
;   2009 Nov 10 Frank Monaldo, Correct bug that wrote a line 
;                              transformation rather than an element transformation
;   2009 Nov 20 Frank Monaldo. Rewrite for ANSWRS 2.0
;   2009 Dec 02 Frank Monaldo. Changed: longitude (0to-360) transform coef for element
;                              to: longitude (0to360) transform coef for element
;   2009 Dec 23 Frank Monaldo. Set lf.asc_des to D or A
;   2009 Jan 05 Frank Monaldo. Change to accommodate ans_compute_fit
;                              and ans_poly_cal
;   2010 Jan 07 Frank Monaldo. Correct the output line for longitude coefficients
;                              Add A/D for ascending/descending pass
;   2010 Jan 08 Frank Monaldo. Use ellipsoid_designator for the reference ellipsoid
;   2010 Jan 12 Frank Monaldo. Write to general ans_write_ancil_sio
;   2010 Jan 21 Frank Monaldo. Need to compute center incidence and slant range to center
;                              of image.
;   2010 Jan 22 Frank Monaldo. Computing illumination direction and relative heading 
;                              according to instructins from C. C. Wackerman
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Jan 01 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro psar_write_ancil_sio, filename, lf, scaling, quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help) or (n_params(0) lt 1 )  then begin
  print, ' NAME: PSAR_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out PALSAR data into ancillary data ascii file. These
  print, '   data fit the SIO format
  print, ' CALLING SEQUENCE:'
  print, '    psar_write_ancil_sio, filename, lf,  scaling'
  print, ' INPUT:'
  print, '   filename=     output file with ancillary data
  print, '   lf=           leader structure'
  print, '   scaling=      scaling for data in binary file of NRCS, default=1
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  return
endif
;
; Compute and fill lf
;
productType=        lf.product_type
frequency_band=     'L'
ellipsoidName=      lf.ellipsoid_designator
software_version=   lf.software_ver
datatake_id=        lf.scene_identifier
platform_code=      'A1'
lf.scansar=         0
if lf.range_pixel gt 30 or lf.azimuth_pixel gt 25 then lf.scansar=1
valuetype=         'intensity'
pol=                lf.polarization 
scaling=            scaling
heading=            lf.sensor_heading
heading_rel=        0.0
sensor_velocity=    sqrt(lf.x_velocity^2 + lf.y_velocity^2 + lf.z_velocity^2)
sensor_velocity=    string(sensor_velocity, format='(F15.6)' ) 
integration_time=   '-9999.99'
if isnumber(lf.nominal_slant_azimuth_resolution_m) eq 0 then lf.nominal_slant_azimuth_resolution_m='0'
nom_az_res=         string(lf.nominal_slant_azimuth_resolution_m)
center_incid=       ans_poly_cal(lf.nx1-1, lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
if isnumber(lf.nominal_slant_range_resolution_m)   eq 0 then lf.nominal_slant_range_resolution_m=  '0'
nom_rg_res=         float(lf.nominal_slant_range_resolution_m)
nom_rg_res=         string(asin( !pi / 180. * center_incid) * nom_rg_res)
rcvr_gain=          string(lf.receiver_gain_like_polarization)       
proc_gain=          '-9999.99'
lf.asc_des=         strupcase( strmid( lf.asc_des, 0, 1))
beam_mode=          lf.antenna_beam_number
flip=               0

if lf.asc_des eq 'A' then begin
  illumination_dir=   90
  near_range_edge=    'L'
  early_azimuth_edge= 'B'
  heading_rel=        180
endif else begin
  illumination_dir=   -90
  near_range_edge='R'
  early_azimuth_edge= 'T'
  heading_rel=        0  
endelse

range_type=1
if float(lf.product_stage) lt 1.5 then range_type=0 
range_flag=range_type


element_spacing=     lf.range_pixel   * lf.ave_pixel 
lf.line_spacing=     lf.azimuth_pixel * lf.ave_line
range_swath=         lf.range_pixel   * lf.ave_pixel *  lf.nx1
azimuth_swath=       lf.azimuth_pixel * lf.ave_line  * lf.ny1
center_lat=          ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
center_lon=          ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
semimajoraxis=       double(lf.semi_major_axis_km)   * 1000.0
semiminoraxis=       double(lf.semi_minor_axis_km)   * 1000.0
lf.s_c_altitude=     double(lf.geodetic_altitude_km) * 1000.0
;
; Computing slant_range_center.
;
center_incid_rad=    double(center_incid) * !dpi /180.
major=               double(semimajoraxis)
minor=               double(semiminoraxis)
cos_phi=             cos(center_lat* !dpi/180.)
sin_phi=             sin(center_lat* !dpi/180.)
re=                  (major^2*cos_phi)^2 + (minor^2*sin_phi)^2
re=                  re / ( (minor * cos_phi)^2 + (minor*sin_phi)^2 )
re=                  sqrt( re )
ra=                  lf.s_c_altitude
C=                   !pi - center_incid_rad
sin_c=               sin(C)
rea=                 re+ra
B=                   asin( sin_c * re /rea )
A=                   !pi - B - C
sin_a=               sin(A)
slant_range_center=  sin_a / sin_c * rea
slant_range_near=    double(lf.slant_range_computed_km) * 1000.0
datumshiftparameters= [0.0, 0.0, 0.0]
ellipsoidscale=      '1'
deg_incid=           lf.deg
deg_rlook=           lf.deg

if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf,                                             $
                          'productType',          productType,            $
                          'frequency_band',       frequency_band,         $
                          'ellipsoidName',        ellipsoidName,          $
                          'software_version',     software_version,       $
                          'platform_code',        platform_code,          $
                          'valuetype',            valuetype,              $
                          'pol',                  pol,                    $
                          'scaling',              scaling,                $
                          'heading',              heading,                $
                          'heading_rel',          heading_rel,            $
                          'sensor_velocity',      sensor_velocity,        $
                          'integration_time',     integration_time,       $
                          'nom_az_res',           nom_az_res,             $
                          'nom_rg_res',           nom_rg_res,             $
                          'rcvr_gain',            rcvr_gain,              $
                          'proc_gain',            proc_gain,              $
                          'beam_mode',            beam_mode,              $
                          'flip',                 flip,                   $
                          'range_type',           range_type,             $
                          'range_flag',           range_flag,             $
                          'element_spacing',      element_spacing,        $
                          'range_swath',          range_swath,            $
                          'azimuth_swath',        azimuth_swath,          $
                          'center_incid',         center_incid,           $
                          'slant_range_center',   slant_range_center,     $
                          'slant_range_near',     slant_range_near,       $
                          'semimajoraxis',        semimajoraxis,          $
                          'semiminoraxis',        semiminoraxis,          $
                          'datumshiftparameters', datumshiftparameters,   $
                          'ellipsoidscale',       ellipsoidscale,         $
                          'deg_rlook',            deg_rlook,              $
                          'deg_incid',            deg_incid,              $
                          'near_range_edge',      near_range_edge,        $
                          'early_azimuth_edge',   early_azimuth_edge,     $
                          'illumination_dir',     illumination_dir        )
                          
  lf=      0                                    
  lf=      temp_lf
endif else begin  
  lf.productType=           productType 
  lf.frequency_band=        frequency_band 
  lf.ellipsoidName=         ellipsoidName 
  lf.software_version=      software_version    
  lf.platform_code=         platform_code               
  lf.valuetype=             valuetype          
  lf.pol=                   pol                
  lf.scaling=               scaling            
  lf.heading=               heading            
  lf.heading_rel=           heading_rel        
  lf.sensor_velocity=       sensor_velocity    
  lf.integration_time=      integration_time   
  lf.nom_az_res=            nom_az_res         
  lf.nom_rg_res=            nom_rg_res               
  lf.rcvr_gain=             rcvr_gain          
  lf.proc_gain=             proc_gain                 
  lf.beam_mode=             beam_mode          
  lf.flip=                  flip               
  lf.range_type=            range_type    
  lf.range_flag=            range_flag
  lf.element_spacing=       element_spacing      
  lf.range_swath=           range_swath        
  lf.azimuth_swath=         azimuth_swath     
  lf.center_incid=          center_incid       
  lf.slant_range_center=    slant_range_center 
  lf.slant_range_near=      slant_range_near   
  lf.semimajoraxis=         semimajoraxis      
  lf.semiminoraxis=         semiminoraxis      
  lf.datumshiftparameters=  datumshiftparameters 
  lf.deg_rlook=             deg_rlook          
  lf.deg_incid=             deg_incid          
  lf.near_range_edge=       near_range_edge   
  lf.early_azimuth_edge=    early_azimuth_edge 
  lf.illumination_dir=      illumination_dir 
  lf.ellipsoidscale=        ellipsoidscale
endelse


;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 



 end
