;-----------------------------------------------------
;+
; NAME: RCM_DN2SIGMA
; PURPOSE:
;    The purpose of this program is to convert the RCM digital numbers
;    to radar cross section. We assume the digital numbner has been squared
; CALLING SEQUENCE:
;    rcm_dn2sigma, img, offset, lutsigma
;  INPUT:
;    img           =   2D array containing the digital number from the images
;                        already squared (if real).    
;    offset        =   Offset
;    lutsigma      =   1D array of gains 
; OUTPUT:     
;    img           =   A floating point array of sigma.
; KEYWORDS=   
;    /quiet        =   Do not print intermediate results.
;    /help         =   List help.
;    /complexdata  =   Set if data are complex
;    error         =   <0 bad 
; MODIFICATION HISTORY:
;  2018 Jul 23  Frank Monaldo .
;  2020 Mar 15  Frank Monaldo.  Update to work with newly released RCM data examples
; Copyright (C) 2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_dn2sigma, img, offset, lutsigma, help=help, quiet=quiet, complexdata=complexdata, error=error

error=     1
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: RCM_DN2SIGMA
  print, ' PURPOSE:
  print, '    The purpose of this program is to convert the RCM digital numbers
  print, '    to radar cross section. We assume the digital numbner has been squared
  print, ' CALLING SEQUENCE:
  print, '    rcm_dn2sigma, img, offset, lutsigma
  print, '  INPUT:
  print, '    img           =   2D array containing the digital number from the images
  print, '                        already squared (if real).    
  print, '    offset        =   Offset
  print, '    lutsigma      =   1D array of gains 
  print, ' OUTPUT:     
  print, '    img           =   A floating point array of sigma.
  print, ' KEYWORDS=   
  print, '    /quiet        =   Do not print intermediate results.
  print, '    /help         =   List help.
  print, '    /complexdata  =   Set if data are complex
  print, '    error         =   <0 bad 
  error = -1
  return
endif

printflag=   1
complexd=    0
if keyword_set(complexdata) then complexd=1
if keyword_set(quiet)       then printflag=0 

sz    =  size(img)
nx    =  sz[1]
ny    =  sz[2]
if complexd eq 0 then img   =  float(img)
nx2   =  n_elements(lutsigma)
ratio =  float(nx2) / float(nx)

if printflag then begin 
  print, '---------------------------------------------------------'
  print, 'RCM CALIBRATION: Converting digital number to sigma.'
  print, '---------------------------------------------------------'
  print, ' '
  print, 'Adding bias to image of ' + string( offset )
  print, 'Complex data flag= ' +string(complexd)
  help, img
endif

;
; Note that the image was squared in the rcm_read_image program if the data
; are real
;

if complexd eq 0 then img=  img + float(offset)


if nx ne nx2 then begin
  if printflag then print, 'Resizing the gains array...'
  lutsigma2=interpol(lutsigma, findgen(nx2), findgen(nx) * ratio )
endif else begin
  lutsigma2= lutsigma
endelse


if printflag  then begin
  print, 'Before Calibration'
  hlp, img
endif

w= where(lutsigma2 ne 0 )

if complexd then begin
 if printflag then print, 'Original data  complex'
 for j=0L, ny-1 do begin
    if printflag and (j mod 1000) eq 0 then begin 
       print, 'Applying gains on '+ strcompress(string(j)) + ' of ' +  strcompress(string(ny))
    endif
    img[w,j]= img[w,j] / lutsigma2[w] / lutsigma2[w]
   endfor
   print, 'Calibrated complex imagery'
endif else begin
  if printflag then print, 'Original data not complex'
  for j=0L, ny-1 do begin
    if printflag and (j mod 1000) eq 0 then begin 
      print, 'Applying gains on '+ strcompress(string(j)) + ' of ' +  strcompress(string(ny))
    endif
    img[w,j]= (img[w,j] + offset) / lutsigma2[w]
   endfor
endelse

if printflag  then begin
  print, 'After Calibration'
  hlp, img, lutsigma2 
endif


end
