;-----------------------------------------------------
;+
; NAME: RCM_READ_IMAGE.PRO
; PURPOSE:
;  Read main RCM IFF img file and square the output
; CALLING SEQUENCE
;   rcm_read_image, filename, ave_pixel, ave_line, lf, img, /help, /quiet, /NOSQU
; INPUT
;  filename   : img file in TIFF format
;  ave_line   : Number of lines to average
;  ave_pixel  : Number of pixels to average
;  lf         : Structure with leader file information
; OUTPUT      
;  img      : Averaged img in the digital values in the file
; KEYWORDS:
;  /help       Get help
;  /quiet      Do not print out intermediate output
;  /nosqu      Do not square output
;  error       <0 if bad
; MODIFICATION HISTORY:
;  2018 Jul 23 Frank Monaldo.  Catch error on reading TIFF file
;  2020 Feb 07 Chris Jackson.  Update the query_tiff call to return tinfo and ginfo variable.  
;                              Default (tif) layer to 0.  
;  2020 Feb 21 Frank Monaldo.  Change variable img to avoid conflict with img function
; Copyright (C) 2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_read_image, filename, ave_pixel, ave_line, lf, img, help=help, quiet=quiet, $
                      nosqu=nosqu, error=error 
printflag=1
if keyword_set(quiet) then printflag=0
error    = 1

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RCM_READ_IMAGE.PRO
  print, ' PURPOSE:
  print, '  Read main RCM IFF img file and square the output
  print, ' CALLING SEQUENCE
  print, '   rcm_read_image, filename, ave_pixel, ave_line, lf, img, /help, /quiet, /NOSQU
  print, ' INPUT
  print, '  filename   : img file in TIFF format
  print, '  ave_line   : Number of lines to average
  print, '  ave_pixel  : Number of pixels to average
  print, '  lf         : Structure with leader file information
  print, ' OUTPUT      
  print, '  img      : Averaged img in the digital values in the file
  print, ' KEYWORDS:
  print, '  /help       Get help
  print, '  /quiet      Do not print out intermediate output
  print, '  /nosqu      Do not square output
  print, '  error       <0 if bad
  error = -1
  return    
endif       

printflag=1
if printflag then begin
  print, 'Reading img data from TIFF file' + filename
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  if printflag then begin
    print, 'Cannot find file: ' + filename
    print, 'Exiting: rcm_read_img'
  endif 
  error = -2
  return
endif
;
;
nx=    long(lf.nx0)
ny=    long(lf.ny0)
nx2=   nx / long(ave_pixel)
ny2=   ny / long(ave_line)
if printflag then begin
  print, 'Original img size: ' + string(nx, format='(I7)') + $
                                   string(ny, format='(I7)')
endif

nx3=       nx2 * ave_pixel
sub_img= uintarr(nx3, ave_line)

img=     fltarr( long(nx2), long(ny2) )
;
; Check for validity of TIFF file
;
res=  query_tiff(filename, tinfo, geotiff=ginfo)
if res eq 0 then begin
  if printflag then begin
    print, filename + ' not a valid tiff file'
    print, 'Exiting: rcm_read_img'
  endif 
  error = -3
  return
endif

;Default the tiff layer to zero.  Layer > 1 means there are multiple imgs
;in the tiff file this can be check in the values retunrd in tifno 
layer=0
if printflag then print, 'tinfo.channels=' , tinfo.channels
if tinfo.channels eq 2  then begin
  ;
  ; full high-resolution complex data
  ;
  if printflag then print, 'Reading complex RCM data file: ' + filename
  sub_img= read_tiff(filename, image_index=layer )
  realp=     reform( sub_img[0, *, *] )
  imagp=     reform( sub_img[1, *, *] )
  img  =     complex( realp, imagp )
  print, 'Just read img in rmc_read_img'
endif else begin
  ;
  if printflag then print, 'Reading RCM data file: ' + filename
  for j=0L, ny2-1 do begin
    jj=        j * ave_line
    sub_img= read_tiff(filename, sub_rect=[0, jj, nx3, ave_line], image_index=layer )
    if strupcase(lf.datatype) ne 'COMPLEX' then begin
      if keyword_set(nosqu) then begin
        img[*,j]=     rebin(sub_img, nx2, 1)
        mu=  mean(sub_img)
        lab= 'Original data'
      endif else begin
        sub_img2=     float(sub_img) * float(sub_img)
        img[*,j]=     rebin(sub_img2, nx2, 1)
        mu=mean(sub_img2)
        lab= 'Original data squared.'
      endelse
      if printflag and (j mod 1000) eq 0 then begin
        percent_done= float(j) / ny2 * 100.0 
        print, 'Reading file. ' + string( percent_done, format='(F5.1)') + $
                '% done. '      + lab + ' Mean=' + string(mu)
      endif
    endif else begin
      sub_img2=     float(sub_img[0,*,*])^2 + float(sub_img[1,*,*])^2
      if keyword_set(nosqu) then sub_img2=sqrt(sub_img2)
      sub_img2=     reform(sub_img2)
      sub_img2=     float(rebin(sub_img2, nx2, 1))
      img[*,j]=     sub_img2
    endelse  
  endfor
endelse
if printflag then print, 'Finished reading file: ' + filename
end
