;-----------------------------------------------------
;+
; NAME: RCM_READ_LUTSIGMA_XML
; PURPOSE:
;   Read main RCM look up table XML
; CALLING SEQUENCE
;  rcm_read_lutsigma_xml, file, info, ii, lutsigma
; INPUT
;  file          : LUT Sigma xml file                                 
; OUTPUT
;  info:         : information structure
;  ii:           : column (range) array
;  lutsigma      : lutsigma
; KEYWORDS:
;  /help         : Print out help
;  /quiet        : Do not print out intermediate results
;  error         : <0 bad
; MODIFICATION HISTORY:
; 2018 Jul 18 Frank Monaldo  
; Copyright (C) 2007-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_read_lutsigma_xml, file, info, ii, lutsigma     , $
                        help    =  help                 , $
                        quiet   =  quiet                , $
                        error   =  error
                        
error = 1
printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RCM_READ_LUTSIGMA_XML
  print, ' PURPOSE:
  print, '   Read main RCM look up table XML
  print, ' CALLING SEQUENCE
  print, '  rcm_read_lutsigma_xml, file, info, ii, lutsigma
  print, ' INPUT
  print, '  file          : LUT Sigma xml file                                 
  print, ' OUTPUT
  print, '  info:         : information structure
  print, '  ii:           : column (range) array
  print, '  lutsigma      : lutsigma
  print, ' KEYWORDS:
  print, '  /help         : Print out help
  print, '  /quiet        : Do not print out intermediate results
  print, '  error         : <0 bad
  print, ' error = -1
  return    
endif  

if printflag then begin
  print, 'Reading RCM LUT Sigmag'
endif

res= file_search(file, count=count)
if not file_test(file) then begin
  print, 'Cannot find file: ' + file
  print, 'Exiting: rcm_read_lutsigma_xml' 
  error = -2
  return
endif

if printflag then print, 'Reading file: ' + file
s= getfile(file)
n= n_elements(s)

info = { sarCalibrationType    :  'Sigma Nought' , $
         pixelFirstLutValue    :  long(0)        , $
         stepSize              :  long(0)        , $
         offset                :  double(0)      , $
         numberOfValues        :  long(0)        , $
         nx                    :  long(0)          }
         
;
; Fill in information structure
;
index= intarr(n)


;
; Find 
;

for k = 0, n-1 do begin
  if strpos( s[k], 'pixelFirstLutValue') ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'pixelFirstLutValue','',ned)   ; Remove pixelFirstLutValue
    info.pixelFirstLutValue     = long( ss )
    if printflag then print, 'info.pixelFirstLutValue     =  ' +  strcompress( string(info.pixelFirstLutValue), /remove_all)
  endif
  if strpos( s[k], 'stepSize')             ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'stepSize','',ned)   ; Remove stepSize
    info.stepSize     = long( ss )
    if printflag then print, 'info.stepSize               =  ' +  strcompress( string(info.stepSize), /remove_all)
  endif
  if strpos( s[k], 'numberOfValues')       ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'numberOfValues','',ned)   ; Remove numberOfValues
    info.numberOfValues = long(ss)
    if printflag then print, 'info.numberOfValues         =  ' +  strcompress( string(info.numberOfValues), /remove_all)
  endif    
  if strpos( s[k], 'offset')       ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'offset','',ned)   ; Remove offset
    info.offset = long(ss)
    if printflag then print, 'info.offset                 =  ' +  strcompress( string(info.offset), /remove_all)
  endif    
  if strpos( s[k], 'gains')     ge 0 then begin
    ss          = s[k]
    ss          = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = getwrd(ss, 1, del='"' )
   
    ss          = s[k]     
    nx          = info.numberOfValues
    ii          = indgen( nx,  /long) * info.stepSize + info.pixelFirstLutValue
    lutsigma = dblarr( nx )
    head        =  '<gains>'
    tail        =  '</gains>'
    if printflag then begin
      print, 'Head                        =  ' + head
      print, 'Tail                        =  ' + tail
    endif
    for i = 0, nx - 1 do begin
      ss       = s[k]
      ss       = stress(ss, 'D' , 0, head, '' , ned )
      ss       = stress(ss, 'D' , 0, tail, '' , ned )
      for i = 0, nx - 1 do begin
         lutsigma[ i ] = float( getwrd( ss, i ) )
      endfor
    endfor  ; Looping through noise level values
  endif     ; endif    strpos( s[k], 'gains')     ge 0
endfor      ; Looping through lines with calibration corrections.


end




