;-----------------------------------------------------
;+
; NAME: RCM_READ_NOISE_XML
; PURPOSE:
;   Read main RCM Noise Level XML
; CALLING SEQUENCE
;  rcm_read_noise_xml, file, info, ii, noise_level
; INPUT
;  file          : noise level xml file                                 
; OUTPUT
;  info:         : information structure
;  ii:           : column (range) array
;  noise_level   : noise_level
; KEYWORDS:
;  /help         : Print out help
;  /quiet        : Do not print out intermediate results
;  error         : <0 bad
; MODIFICATION HISTORY:
; 2018 Jul 18 Frank Monaldo  
; Copyright (C) 2007-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_read_noise_xml, file, info, ii, noise_level      , $
                        help    =  help                 , $
                        quiet   =  quiet                , $
                        error   =  error
                        
error = 1
printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RCM_READ_NOISE_XML
  print, ' PURPOSE:
  print, '   Read main RCM Noise Level XML
  print, ' CALLING SEQUENCE
  print, '  rcm_read_noise_xml, file, info, ii, noise_level
  print, ' INPUT
  print, '  file          : noise level xml file                                 
  print, ' OUTPUT
  print, '  info:         : information structure
  print, '  ii:           : column (range) array
  print, '  noise_level   : noise_level
  print, ' KEYWORDS:
  print, '  /help         : Print out help
  print, '  /quiet        : Do not print out intermediate results
  print, '  error         : <0 bad
  error = -1
  return    
endif  

if printflag then begin
  print, 'Reading RCM noise level'
endif

res= file_search(file, count=count)
if not file_test(file) then begin
  print, 'Cannot find file: ' + file
  print, 'Exiting: rcm_read_noise_xml' 
  error = -2
  return
endif

if printflag then print, 'Reading file: ' + file
s= getfile(file)
n= n_elements(s)

info = { sarCalibrationType    :  'Sigma Nought' , $
         pixelFirstNoiseValue  :  long(0)        , $
         stepSize              :  long(0)        , $
         numberOfValues        :  long(0)        , $
         noiseLevelValuesUnits :  'dB'           , $
         nx                    :  long(0)          }
         
;
; Fill in information structure
;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<referenceNoiseLevel>')
w1  =  where(index ge 0,count1)
index = index * 0
for i=0, n-1 do index[i]= strpos(s[i], '</referenceNoiseLevel>')
w2  =  where(index ge 0,count2)

if count1 eq 0   or $
   count1 ne count2 then begin
   if printflag then begin
     print, 'Cannot not find even pairs of referenceNoiseLevel'
     print, 'Exiting: rcm_read_noise_xml'
  endif 
  error = -3
  return
end

k0 = -1
for k = 0, count1 -1 do begin
  index = index * 0
  for i=w1[k], w2[k] do begin
     index = strpos( s[i], 'Sigma Nought') 
  endfor
  w  = where( index ge 0, count)
  k0 = k
endfor

if k0 eq -1          or   $
   count1 ne count2 then begin
   if printflag then begin
     print, 'Cannot not find Sigma Nought corrections'
     print, 'Exiting: rcm_read_noise_xml'
  endif 
  error = -4
  return
endif

;
; Find 
;

for k = w1[k0], w2[k0] do begin
  if strpos( s[k], 'pixelFirstNoiseValue') ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'pixelFirstNoiseValue','',ned)   ; Remove pixelFirstNoiseValue
    info.pixelFirstNoiseValue     = long( ss )
    if printflag then print, 'info.pixelFirstNoiseValue   =  ' +  strcompress( string(info.pixelFirstNoiseValue), /remove_all)
  endif
  if strpos( s[k], 'stepSize')             ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'stepSize','',ned)   ; Remove stepSize
    info.stepSize     = long( ss )
    if printflag then print, 'info.stepSize               =  ' +  strcompress( string(info.stepSize), /remove_all)
  endif
  if strpos( s[k], 'numberOfValues')       ge 0 then begin
    ss = s[k]
    ss = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = stress(ss,'D',0,'numberOfValues','',ned)   ; Remove numberOfValues
    info.numberOfValues = long(ss)
    if printflag then print, 'info.numberOfValues         =  ' +  strcompress( string(info.numberOfValues), /remove_all)
  endif    
  if strpos( s[k], 'noiseLevelValues')     ge 0 then begin
    ss          = s[k]
    ss          = repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ''  ) &  ss= repchr( ss, '/', ' ' )
    ss = getwrd(ss, 1, del='"' )
    if printflag then print, 'info.noiseLevelValuesUnits  =  ' +  strcompress( info.noiseLevelValuesUnits, /remove_all)
    ss          = s[k]     
    nx          = info.numberOfValues
    ii          = indgen( nx,  /long) * info.stepSize + info.pixelFirstNoiseValue
    noise_level = fltarr( nx )
    head        =  '<noiseLevelValues units="' + strcompress( info.noiseLevelValuesUnits, /remove_all )+'">'
    tail        =  '</noiseLevelValues>'
    if printflag then begin
      print, 'Head                        =  ' + head
      print, 'Tail                        =  ' + tail
    endif
    for i = 0, nx - 1 do begin
      ss       = s[k]
      ss       = stress(ss, 'D' , 0, head, '' , ned )
      ss       = stress(ss, 'D' , 0, tail, '' , ned )
      for i = 0, nx - 1 do begin
         noise_level[ i ] = float( getwrd( ss, i ) )
      endfor
    endfor  ; Looping through noise level values
  endif     ; endif    strpos( s[k], 'noiseLevelValues')     ge 0
endfor      ; Looping through lines with Sigma Nought corrections.


end




