;-----------------------------------------------------
;+
; NAME: RCM_READ_PRODUCT
; PURPOSE:
;  Read main RCM product.xml file
; CALLING SEQUENCE
;  rcm_read_product, filename, lf, ii, jj, incidence, longitude, latitude 
; INPUT
;  filename:       product.XML file                                 
; OUTPUT
;  lf:             information structure
;  ii:             column array
;  jj:             row array
;  incidence:      incidence angle array
;  latitude:       latitude  array
;  longitude:      longitude array
;  outputfile:     Name of file to print the data to.
; KEYWORDS:
;  /help           Print out help
;  /quiet          Do not print out intermediate results
;  error           <0 bad
; MODIFICATION HISTORY:
;   2018 Jul 04 Frank Monaldo
;   2018 Jul 20 Frank Monaldo. Cleaning up
;   2020 Feb 13 Frank Monaldo. Create lf.pol element to be used to eventuall write ou
;                                 
; Copyright (C) 2007-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_read_product, filename, lf, ii, jj, incidence, longitude, latitude,  $
                        outputfile   =   outputfile,  $
                        help         =   help      ,  $
                        quiet        =   quiet

re        = 6378e3 ; Earth radius in meters
printflag = 1
if keyword_set(quiet) then printflag=0
error     = 1

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RCM_READ_PRODUCT
  print, ' PURPOSE:
  print, '  Read main RCM product.xml file
  print, ' CALLING SEQUENCE
  print, '  rcm_read_product, filename, lf, ii, jj, incidence, longitude, latitude 
  print, ' INPUT
  print, '  filename:       product.XML file                                 
  print, ' OUTPUT
  print, '  lf:             information structure
  print, '  ii:             column array
  print, '  jj:             row array
  print, '  incidence:      incidence angle array
  print, '  latitude:       latitude  array
  print, '  longitude:      longitude array
  print, '  outputfile:     Name of file to print the data to.
  print, ' KEYWORDS:
  print, '  /help           Print out help
  print, '  /quiet          Do not print out intermediate results
  print, '  error           <0 bad
  error = -1
  return    
endif       

if printflag then begin
  print, 'Reading general header information with rcm_read_product'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: rcm_read_product' 
  return
endif

if printflag then print, 'Reading file: ' + filename
s= getfile(filename)
n= n_elements(s)
;
;
; Creating leader 
lf={ product                         :  ''                , $        
     productType                     :  ''                , $
     sar_sensor                      :  ''                , $
     documentIdentifier              :  ''                , $  
     acquistion_station              :  ''                , $
     beam_mode                       :  ''                , $
     beamMode                        :  ''                , $
     beamModeDefinitionId            :  ''                , $ 
     beamModeVersion                 :  ''                , $ 
     beamModeMnemonic                :  ''                , $ 
     proc_cent                       :  ''                , $  
     proc_fac                        :  ''                , $
     proc_time                       :  ''                , $  
     software_ver                    :  ''                , $
     sensing_start                   :  ''                , $
     sensing_stop                    :  ''                , $
     data_year                       :  0                 , $
     center_gmt                      :  ''                , $
     month                           :  ''                , $
     day                             :  0                 , $
     hour                            :  0                 , $
     minute                          :  0                 , $
     second                          :  0.0               , $
     abs_orbit                       :  0.0               , $ 
     state_vector_time               :  ''                , $
     x_position                      :  0.0D              , $
     y_position                      :  0.0D              , $         
     z_position                      :  0.0D              , $
     x_velocity                      :  0.0D              , $         
     y_velocity                      :  0.0D              , $
     z_velocity                      :  0.0D              , $
     first_near_lat                  :  0.0D              , $
     near_start_lat                  :  0.0D              , $ ; fill in ASF style leader
     first_near_long                 :  0.0D              , $
     near_start_lon                  :  0.0D              , $ ; fill in ASF style leader
     first_mid_lat                   :  0.0D              , $
     first_mid_long                  :  0.0D              , $
     first_far_lat                   :  0.0D              , $ 
     far_start_lat                   :  0.0D              , $ ; fill in ASF style leader
     first_far_long                  :  0.0D              , $
     far_start_lon                   :  0.0D              , $ ; fill in ASF style leader
     last_near_lat                   :  0.0D              , $
     near_end_lat                    :  0.0D              , $ ; fill in ASF style leader
     last_near_long                  :  0.0D              , $
     near_end_lon                    :  0.0D              , $ ; fill in ASF style leader
     last_mid_lat                    :  0.0D              , $
     last_mid_long                   :  0.0D              , $
     last_far_lat                    :  0.0D              , $
     far_end_lat                     :  0.0D              , $ ; fill in ASF style leader
     last_far_long                   :  0.0D              , $
     far_end_lon                     :  0.0D              , $ ; fill in ASF style leader
     center_lon                      :  0.0               , $ ; fill in ASF style leader
     center_lat                      :  0.0               , $ ; fill in ASF style leader
     azimuth_looks                   :  0.0               , $
     range_looks                     :  0.0               , $
     range_spacing                   :  0.0               , $
     azimuth_spacing                 :  0.0               , $
     ny0                             :  0L                , $
     nx0                             :  0L                , $
     ny1                             :  0L                , $
     nx1                             :  0L                , $     
     lngswitchflag                   :  0                 , $
     datatake_id                     :  ''                , $   ; fill in ASF style leader
     projection                      :  ''                , $   ; fill in ASF style leader
     polarization                    :  strarr(5)         , $   ; fill in ASF style leader
     pol                             :  ''                , $
     pixel_spacing                   :   0.00             , $   ; fill in GAT style leader    
     line_spacing                    :   0.00             , $   ; fill in GAT style header  
     range_pixel                     :   0.00             , $   ; fill in ASF style header
     azimuth_pixel                   :   0.00             , $
     radar_frequency                 :   '9.65e+9'        , $   ; HZ
     radar_band                      :  'C'               , $
     asc_des                         :  ''                , $
     datatype                        :  ''                , $
     bitspersample                   :  16                , $
     lineTimeOrdering                :  ''                , $
     pixelTimeOrdering               :  ''                , $
     s_c_altitude_m                  :  0.0D              , $
     slant_range_near_m              :  0.0D              , $
     incidence_near_deg              :  0.0D              , $
     incidence_far_deg               :  0.0D              , $
     imagefiles                      :  strarr(5)         , $
     noisefiles                      :  strarr(5)         , $
     antennaPointing                 :  'RIGHT'           , $
     groundToSlantRangeCoefficients  :  ''                , $
     ellipsoidName                   :  ''                , $
     semiMajorAxis                   :  ''                , $
     semiMinorAxis                   :  ''                , $
     datumShiftParameters            :  ''                , $
     software_version                :  ''                , $
     incidenceAngleNearRange         :  ''                , $
     incidenceAngleFarRange          :  ''                , $
     slantRangeNearEdge              :  ''                , $
     satelliteHeight                 :  ''                , $
     inputDatasetId                  :  ''                , $
     nrcs_slope_array                :  [1.0,1.0,1.0,1.0] , $
     nrcs_bias_array                 :  [0.0,0.0,0.0,0.0] , $
     nrcs_slope                      :  1.0               , $
     nrcs_bias                       :  0.0               , $
     jsbegin                         :  -3155673600D      , $   ; Default setting for start data
     jsend                           :  -3155673600D        }   ; Default setting for end  data
                    
monthstr=['', 'jan', 'feb', 'mar', 'apr', 'may', 'jun', $
              'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
              
if printflag then print, 'Parsing:      ' + filename
;
; General parameters
;
n= n_elements(s)

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'productId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.product     = getwrd(ss, 1)
  lf.datatake_id = lf.product
endif
if printflag then print, 'product                = ' + lf.product
if printflag then print, 'datatake_id            = ' + lf.datatake_id

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'productType')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.productType= getwrd(ss, 1)
  if lf.productType eq 'GRD' then lf.projection='GROUND RANGE'
endif

if printflag then print, 'productType            = ' + lf.productType
if printflag then print, 'projection             = ' + lf.projection

;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'dataType')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.datatype= getwrd(ss, 1)
endif
if printflag then print, 'dataType               = ' + lf.dataType

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'inputDatasetId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.inputDatasetId= getwrd(ss, 1)
endif else begin
  lf.inputDatasetId = lf.product
endelse
if printflag then print, 'inputDatasetId         = ' + lf.inputDatasetId  


index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'processingFacility')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= stress(ss,'R',0,'<processingFacility>' ,'',ned)
  ss= stress(ss,'R',0,'</processingFacility>','',ned)
  lf.proc_cent= strtrim(ss,1)
  lf.proc_fac = strtrim(ss,1)
endif
if printflag then print, 'proc_cent              = ' + lf.proc_cent

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'processingTime')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= stress(ss,'R',0,'<processingTime>','',ned)
  ss= stress(ss,'R',0,'</processingTime>','',ned)
  lf.proc_time= ss
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'softwareVersion')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= stress(ss,'R',0,'<softwareVersion>','',ned)
  ss= stress(ss,'R',0,'</softwareVersion>','',ned)
  lf.software_ver= strtrim(ss,1)
endif
if printflag then print, 'softwareVersion        = ' + lf.software_ver


;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'beamMode')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= stress(ss,'R',0,'<beamMode>','',ned)
  ss= stress(ss,'R',0,'</beamMode>','',ned)
  ss= strtrim(ss,1)
  lf.beam_mode= ss
  lf.beamMode= ss
endif
if printflag then print, 'beam_mode              = ' + lf.beam_mode
if printflag then print, 'beamMode               = ' + lf.beamMode

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'beamModeMnemonic')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.beamModeMnemonic= getwrd(ss, 1)
endif
if printflag then print, 'beamModeMnemonic       = ' + lf.beamModeMnemonic

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'beamModeDefinitionId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.beamModeDefinitionId= getwrd(ss, 1)
endif
if printflag then print, 'beamModeDefinitionId   = ' + lf.beamModeDefinitionId

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'beamModeVersion')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.beamModeVersion= getwrd(ss, 1)
endif
if printflag then print, 'beamModeVersion        = ' + lf.beamModeVersion

;
for i=0, n-1 do index[i]= strpos(s[i], 'documentIdentifier')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.documentIdentifier= getwrd(ss, 1)
endif
if printflag then print, 'documentIdentifier     = ' + lf.documentIdentifier
;

for i=0, n-1 do index[i]= strpos(s[i], 'satellite')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.sar_sensor= getwrd(ss, 1)
endif
if printflag then print, 'sar_sensor             = ' + lf.sar_sensor


;
for i=0, n-1 do index[i]= strpos(s[i], 'polarizations')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'polarizations','',ned)
  for j=0, 4 do lf.polarization[j] = strcompress(getwrd(ss, j), /remove_all)
endif
if printflag then begin
  for i=0, 4 do print, 'polarization           = ' + string( i, format='(I2)') + ' ' + lf.polarization[i]
endif

for i=0, n-1 do index[i]= strpos(s[i], 'ellipsoidName')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'ellipsoidName',' /',ned)
  lf.ellipsoidName= strcompress(getwrd(ss, 1, del='/'))
endif
if printflag then print, 'ellipsoidName          = ' + lf.ellipsoidName

for i=0, n-1 do index[i]= strpos(s[i], 'numberOfRangeLooks')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'numberOfRangeLooks','',ned)
  lf.range_looks= float(ss)
endif
if printflag then print, 'range_looks            = ' + strcompress(string(lf.range_looks), /remove_all)


for i=0, n-1 do index[i]= strpos(s[i], 'numberOfAzimuthLooks')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'numberOfAzimuthLooks','',ned)
  lf.azimuth_looks= float(ss)
endif
if printflag then print, 'azimuth_looks          = ' +  strcompress(string(lf.azimuth_looks), /remove_all)


for i=0, n-1 do index[i]= strpos(s[i], 'groundToSlantRangeCoefficients')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'groundToSlantRangeCoefficients',' /',ned)
  lf.groundToSlantRangeCoefficients= getwrd(ss, 1, del='/')
endif
if printflag then print, 'groundToSlantRangeCoefficients = ' +  lf.groundToSlantRangeCoefficients
;
for i=0, n-1 do index[i]= strpos(s[i], 'semiMajorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'semiMajorAxis','/',ned)
  lf.semiMajorAxis= getwrd(ss, 2) 
endif
if printflag then print, 'semiMajorAxis          = ' +  lf.semiMajorAxis
;
for i=0, n-1 do index[i]= strpos(s[i], 'semiMinorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'semiMinorAxis','/',ned)
  lf.semiMinorAxis= getwrd(ss, 2) 
endif
if printflag then print, 'semiMajorAxis          = ' +  lf.semiMajorAxis


for i=0, n-1 do index[i]= strpos(s[i], 'datumShiftParameters')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'datumShiftParameters','/',ned)
  ss= stress(ss,'R',0,'units="m"','/',ned)
  lf.datumShiftParameters= getwrd(ss, 2,del= '/') 
endif
if printflag then print, 'datumShiftParameters   = ' +  lf.datumShiftParameters
;
for i=0, n-1 do index[i]= strpos(s[i], 'radarCenterFrequency')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.radar_frequency= float(getwrd(ss, 2 ))
endif
if printflag then print, 'radarCenterFrequency   = ' +  strcompress( string(lf.radar_frequency), /remove_all )

;
; Note need to read the incidenceAngles.xml file in the calibration directory
;
for i=0, n-1 do index[i]= strpos(s[i], 'incAngNearRng')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.incidenceAngleNearRange= getwrd(ss, 2 )
endif
if printflag then print, 'incidenceAngleNearRange= ' +  strcompress( string(lf.incidenceAngleNearRange), /remove_all )
;
for i=0, n-1 do index[i]= strpos(s[i], 'incAngFarRng')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.incidenceAngleFarRange= getwrd(ss, 2 )
endif
if printflag then print, 'incidenceAngleFarRange = ' +  strcompress( string(lf.incidenceAngleFarRange), /remove_all )

;
for i=0, n-1 do index[i]= strpos(s[i], 'slantRangeNearEdge')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.slantRangeNearEdge= getwrd(ss, 2 )
endif
if printflag then print, 'slantRangeNearEdge     = ' +  strcompress( lf.slantRangeNearEdge, /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'satelliteHeight')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.satelliteHeight= getwrd(ss, 2 ) 
  s_c_altitude_m    = lf.satelliteHeight
endif
if printflag then print, 'satelliteHeight        = ' +  strcompress( lf.satelliteHeight, /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'passDirection')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.asc_des=  strupcase(getwrd(ss, 1))
endif
if printflag then print, 'asc_des                = ' +  strcompress( lf.asc_des, /remove_all )


for i=0, n-1 do index[i]= strpos(s[i], 'xPosition')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.x_position=  double(getwrd(ss, 2))
endif
if printflag then print, 'x_position             = ' +  strcompress( string(lf.x_position), /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'yPosition')
w=where(index ge 0,count) 
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.y_position=  double(getwrd(ss, 2))
endif
if printflag then print, 'y_position             = ' +  strcompress( string(lf.y_position), /remove_all )


for i=0, n-1 do index[i]= strpos(s[i], 'zPosition')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.z_position=  double(getwrd(ss, 2))
endif
if printflag then print, 'z_position             = ' +  strcompress( string(lf.z_position), /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'xVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.x_velocity=  double(getwrd(ss, 3))
endif
if printflag then print, 'x_velocity             = ' +  strcompress( string(lf.x_velocity), /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'yVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.y_velocity=  double(getwrd(ss, 3))
endif
if printflag then print, 'y_velocity             = ' +  strcompress( string(lf.y_velocity), /remove_all )

for i=0, n-1 do index[i]= strpos(s[i], 'zVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.z_velocity=  double(getwrd(ss, 3))
endif
if printflag then print, 'z_velocity             = ' +  strcompress( string(lf.z_velocity), /remove_all )


for i=0, n-1 do index[i]= strpos(s[i], 'rawDataStartTime')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.sensing_start=  strupcase(getwrd(ss, 1))
  lf.center_gmt=     lf.sensing_start
  lf.data_year=           fix( strmid(lf.center_gmt,  0,  4) )
  lf.month=     monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
  lf.day=                 fix( strmid(lf.center_gmt,  8,  2) )
  lf.hour=                fix( strmid(lf.center_gmt, 11,  2) )
  lf.minute=              fix( strmid(lf.center_gmt, 14,  2) )
  lf.second=              fix( strmid(lf.center_gmt, 17,  2) )
endif

if printflag then print, 'center_gmt             = ' +  strcompress( string(lf.center_gmt), /remove_all )
if printflag then print, 'data_year              = ' +  strcompress( string(lf.data_year ), /remove_all )
if printflag then print, 'month                  = ' +  strcompress( string(lf.month     ), /remove_all )
if printflag then print, 'day                    = ' +  strcompress( string(lf.day       ), /remove_all )
if printflag then print, 'hour                   = ' +  strcompress( string(lf.hour      ), /remove_all )
if printflag then print, 'minute                 = ' +  strcompress( string(lf.minute    ), /remove_all )
if printflag then print, 'second                 = ' +  strcompress( string(lf.second    ), /remove_all )


; Time stamp
for i=0, n-1 do index[i]= strpos(s[i], 'timeStamp')
w=where(index ge 0,count)
timejs=     dblarr(count)

if count gt 0 then begin
  for i=0L, count-1 do begin
    ss= s[w[i]]
    ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
    ss= strupcase(getwrd(ss, 1))
    year= strmid(ss,0,4)
    mon=  monthstr( fix( strmid(ss,  5,  2) ) )
    day=  strmid(ss,  8,  2)  
    ss=   year + ' ' + mon + ' ' + day + ' ' + strmid(ss,11,11)
    timejs[i]=  dt_tm_tojs(ss)
  endfor
endif

w= where(timejs gt 0, cc )
if cc gt 0 then begin
  lf.jsbegin=   min( timejs) 
  lf.jsend=     max( timejs) 
  jscen=       (lf.jsbegin + lf.jsend)/ 2.0 
endif
;print, dt_tm_fromjs(lf.jsbegin) 
;print, dt_tm_fromjs(lf.jsend  ) 
;print, dt_tm_fromjs(   jscen  ) 

lf.center_gmt=     dt_tm_fromjs(jscen,for='Y$-0n$-d$Th$:m$:s$')
lf.data_year=           fix( strmid(lf.center_gmt,  0,  4) )
lf.month=     monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
lf.day=                 fix( strmid(lf.center_gmt,  8,  2) )
lf.hour=                fix( strmid(lf.center_gmt, 11,  2) )
lf.minute=              fix( strmid(lf.center_gmt, 14,  2) )
lf.second=              fix( strmid(lf.center_gmt, 17,  2) )




for i=0, n-1 do index[i]= strpos(s[i], 'antennaPointing')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.antennaPointing=  strupcase(getwrd(ss, 1))
endif
if printflag then print, 'antennaPointing        = ' +  strcompress( string(lf.antennaPointing    ), /remove_all )
;
;
for i=0, n-1 do index[i]= strpos(s[i], 'numLines')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.ny0=  strupcase(getwrd(ss, 1))
endif
if printflag then print, 'numLines               = ' +  strcompress( string(lf.ny0    ), /remove_all )
;
for i=0, n-1 do index[i]= strpos(s[i], 'samplesPerLine')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.nx0=  strupcase(getwrd(ss, 1))
endif
if printflag then print, 'samplesPerLine         = ' +  strcompress( string(lf.nx0    ), /remove_all )



for i=0L, n-1 do index[i]= strpos(s[i], 'rasterAttributes')
w=where(index ge 0, count)
if count ge 1 then begin
  s1=s[w[0]:w[1]]
  n1= n_elements(s1)
  for i=0, n1-1 do begin
    ss= s1[i] & ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
    ;print, ss
    if strpos(ss, 'dataType')               ge 0 then dataType=               getwrd(ss, 1)
    if strpos(ss, 'bitsPerSample')          ge 0 then bitsPerSample=          getwrd(ss, 2)
    if strpos(ss, 'numberOfLine')           ge 0 then numberOfLines=          getwrd(ss, 1)
    ;
    ; Note that the space at the end of the search string is necessary
    ;
    if strpos(ss, 'sampledPixelSpacing ')   ge 0 then sampledPixelSpacing=    getwrd(ss, 2)
    if strpos(ss, 'sampledLineSpacing ')    ge 0 then sampledLineSpacing=     getwrd(ss, 2)
    if strpos(ss, 'lineTimeOrdering')       ge 0 then lineTimeOrdering=       getwrd(ss, 1)	
    if strpos(ss, 'pixelTimeOrdering')      ge 0 then pixelTimeOrdering=      getwrd(ss, 1)
  endfor
  lf.range_pixel=       sampledPixelSpacing
  lf.azimuth_pixel=     sampledLineSpacing
  lf.pixel_spacing=     lf.range_pixel
  lf.line_spacing=      lf.azimuth_pixel
  lf.pixelTimeOrdering= strupcase(pixelTimeOrdering         ) 
  lf.lineTimeOrdering=  strupcase(lineTimeOrdering)
  if printflag then begin
    print, 'range_pixel            = ' +  strcompress(  lf.range_pixel       , /remove_all )
    print, 'azimuth_pixel          = ' +  strcompress(  lf.azimuth_pixel     , /remove_all )
    print, 'ny0                    = ' +  strcompress(  lf.ny0               , /remove_all )
    print, 'nx0                    = ' +  strcompress(  lf.nx0               , /remove_all )
    print, 'pixelTimeOrdering      = ' +  strcompress(  lf.pixelTimeOrdering , /remove_all )
    print, 'lineTimeOrdering       = ' +  strcompress(  lf.lineTimeOrdering  , /remove_all )
  endif 
endif
; ----------------------------------------------------
;      GENERAL GET THE SIZE OF THE ARRAY
; ---------------------------------------------------     
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'geolocationGrid')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=  w[0]
  iend=    w[1]
  index2=  index[w[0]:w[1]]
  s2=      s[w[0]:w[1]]
  n=       n_elements(s2)
  index2= intarr(n) & istart=-1 & iend=-1
  for i=0L, n-1 do index2[i]= strpos(s2[i], 'numberOfGridPoints')
  w=where(index2 ge 0,count)
  if count gt 0 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s2[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'azimuth':   nazimuth= long(getwrd(ss,1))
      'range':     nrange=   long(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif
; ----------------------------------------------------
;                   GENERAL GRID REFERNCE TIMES
; ---------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'gridReferenceTime')
w=where(index ge 0,count)
if count ge 2 then begin
  tReferenceTimeUTC=''
  tauReferenceTime= 0.0
  if count ge 2 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'tReferenceTimeUTC':    tReferenceTimeUTC=         getwrd(ss,1)
      'tauReferenceTime':     tauReferenceTime=   double(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif
; ----------------------------------------------------
;                   FILL IN ARRAY
; ---------------------------------------------------
n=      n_elements(s)      
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<imageTiePoint>')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</imageTiePoint>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)
ii=        lonarr(count)
jj=        lonarr(count)
longitude= dblarr(count)
latitude=  dblarr(count)
height=    dblarr(count)
incidence= dblarr(count)

if count gt 0 then begin
  for cc=0L, count - 1L do begin
    istart=w1[cc]
    iend=  w2[cc]
    for i=istart, iend do begin
      ss=  s[i]
      ss=  repchr( ss, '<', ' ' )
      ss=  repchr( ss, '>', ' ' )
      ss=  repchr( ss, '/', ' ' )
      key= getwrd( ss, 0)
      case key of
      'line':        jj[cc]=         long(    float(getwrd(ss, 1 )))
      'pixel':       ii[cc]=         long(    float(getwrd(ss, 1 )))                                                                  
      'latitude':    latitude[cc]=   double(        getwrd(ss, 2 ))  ; degrees                                                               
      'longitude':   longitude[cc]=  double(        getwrd(ss, 2 ))  ; degrees                                                                 
      'height':      height[cc]=     double(        getwrd(ss, 2 ))  ; degrees   
      ELSE: ; No operation 
      endcase
    endfor    ; End loop i          ; Loop through i, s[i] of lines between imageTiePoint tages
    if printflag then begin
        out =         string(       ii[cc], format='(F10.4)') 
        out = out +   string(       jj[cc], format='(F10.4)')
        out = out +   string( latitude[cc], format='(F12.5)') 
        out = out +   string(longitude[cc], format='(F12.5)') 
        print, out
    endif
  endfor      ; End loop cc         ; Loop through number of imageTiePoints
endif         ; End if count gt 0   ; Check to see if there are any imageTiePoints

;-----------------------------------------------------
; Read information to determine the incidence angle
;----------------------------------------------------

slant_range_near_m                               = lf.slantrangenearedge 
if printflag then begin
  print, 'incidence_near_deg     = ' + strcompress( lf.incidenceanglenearrange , /remove_all)
  print, 'incidence_far_deg      = ' + strcompress( lf.incidenceanglefarrange  , /remove_all)
  print, 'slant_range_near_m     = ' + strcompress( lf.slantrangenearedge      , /remove_all)
endif 
;
; Pixel time direction determines whether the first pixel is near range or far range. 
; Hence the calculation must be inverted
;
if printflag then print,'PixelTimeOrdering      = ' +  strcompress( lf.pixelTimeOrdering)
if strupcase(lf.pixelTimeOrdering) eq 'INCREASING' then begin
  ;
  ; near range is first pixel
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0L, n-1 do begin
    c=            c0 + ii[k] *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor 
endif else begin
  ;
  ; far range is first pixel
  ; 
  ; First compute the slant range at far incidence
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0, n-1 do begin
    c=            c0 + (lf.nx0 -ii[k]) *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor
endelse


;---------------------------------------
; Get filenames
; ---------------------------------------

n=      n_elements(s)      
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<sceneAttributes>')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</sceneAttributes>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)


if count gt 0 then begin
  istart=w1[0]
  iend=  w2[0]
  for i=istart, iend do begin
    ss=  s[i]
    ss=  repchr( ss, '<', ' ' )
    ss=  repchr( ss, '>', ' ' )
    ss=  repchr( ss, '/', ' ' )
    key= getwrd( ss, 0)
    if key eq 'ipdf' then begin
      polkey=getwrd( ss, 1, del='"')
      for k=0, 4 do begin
        if polkey eq lf.polarization[k] then begin
           lf.imagefiles[k]= getwrd(ss, 4)
           lf.noisefiles[k]= 'noiseLevels_' + lf.polarization[k] + '.xml'
           if printflag then begin
              print, lf.polarization[k] + '                     = ' + lf.imagefiles[k]
              print, lf.polarization[k] + '                     = ' + lf.noisefiles[k]
           endif
        endif
      endfor
    endif
  endfor   ; Looping through indice includeing sceneAttributes tags
endif      ; Check count of sceneAttributes tags


if keyword_set(outputfile) then begin
  close,  1
  openw,  1,  outputfile
  printf, 1, 'product=               ' + lf.product       
  printf, 1, 'documentidentifier=    ' + lf.documentidentifier
  printf, 1, 'sar_sensor=            ' + lf.sar_sensor
  printf, 1, 'polariztion=           ' + lf.polarization
  printf, 1, 'radar_frequency=       ' + lf.radar_frequency
  printf, 1, 'asc_des=               ' + lf.asc_des
  printf, 1, 'center_gmt=            ' + lf.center_gmt
  printf, 1, 'data_year=             ' + strcompress(string(lf.data_year))
  printf, 1, 'month=                 ' + lf.month
  printf, 1, 'day=                   ' + strcompress(string(lf.day))
  printf, 1, 'hour=                  ' + strcompress(string(lf.hour))
  printf, 1, 'minute=                ' + strcompress(string(lf.minute))
  printf, 1, 'second=                ' + strcompress(string(lf.second))
  printf, 1, 'antennapointing=       ' + strcompress(string(lf.antennapointing))
  printf, 1, 'range_pixel=           ' + strcompress(string(lf.range_pixel))
  printf, 1, 'azimuth_pixel=         ' + strcompress(string(lf.azimuth_pixel))
  printf, 1, 'numberoflines=         ' + strcompress(string(lf.nx0))
  printf, 1, 'numberofsamplesperline=' + strcompress(string(lf.ny0))
  printf, 1, 'pixeltimeordering=     ' + lf.pixeltimeordering
  printf, 1, 'linetimeordering=      ' + lf.linetimeordering
  n=n_elements(ii)
  printf, 1,  'i  j longitude, latitude'
  for i=0L, n-1 do begin
    printf, 1, string(double(ii[i]),        format='(E24.16E3)') + ' ' + $
               string(double(jj[i]),        format='(E24.16E3)') + ' ' + $
               string(double(longitude[i]), format='(E24.16E3)') + ' ' + $
               string(double(latitude[i]),  format='(E24.16E3)')
  endfor
  printf,1, 'i, incidence'
  n=n_elements(incidence)
  for i=0L, n-1 do begin
    printf, 1, string(double(ii[i]),           format='(E24.16E3)') + ' ' + $
               string(double(incidence[i]),    format='(E24.16E3)') 
  endfor
  close, 1
  if printflag then print, 'Stored file='+outputfile
endif

end
