;-----------------------------------------------------
;+
; NAME: RCM_READ_SIGMA_LUT
; PURPOSE:
;   Read RCM Sigma look up table
; CALLING SEQUENCE
;  rcm_read_sigma_lut, file, info, ii, sigma_lut
; INPUT
;  file          : sigma lut xml file                                 
; OUTPUT
;  info:         : information structure
;  ii:           : column (range) array
;  noise_level   : noise_level
; KEYWORDS:
;  /help         : Print out help
;  /quiet        : Do not print out intermediate results
;  error         : <0 bad
; MODIFICATION HISTORY:
; 2018 Jul 20 Frank Monaldo  
; Copyright (C) 2007-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rcm_read_sigma_lut, file, info, ii, sigma_lut      , $
                         help    =  help                 , $
                         quiet   =  quiet                , $
                         error   =  error
                        
error = 1
printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RCM_READ_SIGMA_LUT
  print, ' PURPOSE:
  print, '   Read RCM Sigma look up table
  print, ' CALLING SEQUENCE
  print, '  rcm_read_sigma_lut, file, info, ii, sigma_lut
  print, ' INPUT
  print, '  file          : sigma lut xml file                                 
  print, ' OUTPUT
  print, '  info:         : information structure
  print, '  ii:           : column (range) array
  print, '  noise_level   : noise_level
  print, ' KEYWORDS:
  print, '  /help         : Print out help
  print, '  /quiet        : Do not print out intermediate results
  print, '  error         : <0 bad
  error = -1
  return    
endif  

if printflag then begin
  print, 'Reading RCM Sigma Lut'
endif

res= file_search(file, count=count)
if not file_test(file) then begin
  print, 'Cannot find file: ' + file
  print, 'Exiting: rcm_read_sigma_lut' 
  error = -2
  return
endif

if printflag then print, 'Reading file: ' + file
s= getfile(file)
n= n_elements(s)


info = { pixelFirstLutValue    :  long(0)        , $
         stepSize              :  long(0)        , $
         numberOfValues        :  long(0)        , $
         offset                :  long(0)         }
         
         
         
;
; Fill in information structure
;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<pixelFirstLutValue>')
w    =  where(index ge 0,count)
if count gt 0 then begin
  ss                     = s[w[0]]
  ss                     = stress(ss, 'R' , 0, '<pixelFirstLutValue>', '' , ned )
  ss                     = stress(ss, 'R' , 0,'</pixelFirstLutValue>', '' , ned )
  lf.pixelFirstLutValue   = long( ss )
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<stepSize>')
w    =  where(index ge 0,count)
if count gt 0 then begin
  ss          = s[w[0]]
  ss          = stress(ss, 'R' , 0, '<stepSize>', '' , ned )
  ss          = stress(ss, 'R' , 0,'</stepSize>', '' , ned )
  lf.stepSize = long( ss )
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<numberOfValues>')
w    =  where(index ge 0,count)
if count gt 0 then begin
  ss                = s[w[0]]
  ss                = stress(ss, 'R' , 0, '<numberOfValues>', '' , ned )
  ss                = stress(ss, 'R' , 0,'</numberOfValues>', '' , ned )
  lf.numberOfValues = long( ss )
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<offset>')
w    =  where(index ge 0,count)
if count gt 0 then begin
  ss          = s[w[0]]
  ss          = stress(ss, 'R' , 0, '<offset>', '' , ned )
  ss          = stress(ss, 'R' , 0,'</offset>', '' , ned )
  lf.offset   = long( ss )
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], '<gains>')
w    =  where(index ge 0,count)


if count eq 0 then begin
   if printflag then begin
     print, 'Cannot not find array gains'
     print, 'Exiting: rcm_read_sigma_lut'
  endif 
  error = -3
  return
endif

sigma_lut = fltarr(lf.numberOfValues)
ss = s[w[0]]
ss = stress(ss, 'R' , 0, '<gains>' , '' , ned )
ss = stress(ss, 'R' , 0, '</gains>', '' , ned )
for i = 0, lf.numberOfValues - 1 do sigma_lut[ i ] = float( getwrd( ss, i ) )


end




