;-----------------------------------------------------
;+
; NAME: RIS_READ_IMAGE.PRO
; PURPOSE:
;  Read RISAT TIFF image file
; CALLING SEQUENCE
;   ris_read_image, filename, img
; INPUT
;   filename   : Image file in TIFF format
; OUTPUT      
;   img        : Averaged image in the digital values in the file
; KEYWORDS:
;   ave_line   : Number of lines to average
;   ave_pixel  : Number of pixels to average
;   outfile    : Output png file that displays the image
;   /display   : Display image on X-window display
;   /reducedis : Ratio to reduce the display size (default=10)
;   /help      : Get help
;   /quiet     : Do not print out intermediate output
;   error      : <0 if bad
;
; MODIFICATION HISTORY:
;  2014 Dec 01 Frank Monaldo. Creation based on rsat2_read_image
; Copyright (C) 2008-2014, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro ris_read_image, filename, img         , $
                    ave_pixel = ave_pixel , $ 
                    ave_line  = ave_line  , $
                    help      = help      , $
                    quiet     = quiet     , $
                    outfile   = outfile   , $
                    display   = display   , $
                    reducedis = resduce   , $
                    error     = error 
printflag=1
if keyword_set(quiet) then printflag=0
error    = 1

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RIS_READ_IMAGE.PRO
  print, ' PURPOSE:
  print, '  Read RISAT TIFF image file
  print, ' CALLING SEQUENCE
  print, '   ris_read_image, filename, img
  print, ' INPUT
  print, '   filename   : Image file in TIFF format
  print, ' OUTPUT      
  print, '   img        : Averaged image in the digital values in the file
  print, ' KEYWORDS:
  print, '   ave_line   : Number of lines to average
  print, '   ave_pixel  : Number of pixels to average
  print, '   outfile    : Output png file that displays the image
  print, '   /display   : Display image on X-window display
  print, '  /reducedis : Ratio to reduce the display size (default=10)
  print, '   /help      : Get help
  print, '   /quiet     : Do not print out intermediate output
  print, '   error      : <0 if bad
  print, '
  error = -1
  return    
endif       

if printflag then print, 'Reading RISAT image data from TIFF file'

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: ris_read_image' 
  img = 0
  error = -1
  return
endif
;
;
if n_elements(ave_pixel) eq 0 then ave_pixel = 1
if n_elements(ave_line)  eq 0 then ave_line  = 1
if n_elements(reducedis) eq 0 then reducedis = 10

res=  query_tiff(filename, info)
if res ne 1 then begin
  error = -2
  if printflag then print, 'TIFF file no valid: ' + filename
  img = 0
  return
endif


nx=    long(info.dimensions[0])
ny=    long(info.dimensions[1])
nx2=   nx / long(ave_pixel)
ny2=   ny / long(ave_line)
if printflag then begin
  print, 'Original image size: ' + string(nx,  format='(I7)') + $
                                   string(ny,  format='(I7)')
  if ave_pixel ne 1 or ave_line ne 1 then begin
    print, 'New image size     : ' + string(nx2, format='(I7)') + $
                                     string(ny2, format='(I7)')
  endif
endif

nx3       = nx2 * ave_pixel
sub_image = uintarr(nx3, ave_line)
img       = fltarr( long(nx2), long(ny2) )

  ;
if printflag then print, 'Reading RISAT data file: ' + filename
for j=0L, ny2-1 do begin
  jj        = j * ave_line
  sub_image = read_tiff(filename, sub_rect=[0, jj, nx3, ave_line] )
  img[*,j]  = rebin(sub_image, nx2, 1)
  mu        =  mean(sub_image)
  lab       = 'Original data'
  if printflag and (j mod 1000) eq 0 then begin
      percent_done= float(j) / ny2 * 100.0 
      print, 'Reading file. ' + string( percent_done, format='(F5.1)') + $
              '% done. '      + lab + ' Mean=' + string(mu)
  endif
endfor
if printflag then print, 'Finished reading file: ' + filename

buffer= 0
if keyword_set(display) or keyword_set(outfile)then begin
  img2  = congrid( smooth(img,[reducedis,reducedis]), $
                   nx2/reducedis, ny2/reducedis       )
endif
if keyword_set(display) then tvscl, ls(img2)
if keyword_set(outfile) then begin
  im = image( ls(img2), /buffer)
  im.save, outfile
endif

end
