;----------------------------------------------------------
;+
; NAME: RSAT2_COMPUTE_RLOOK'
; PURPOSE:'
;   Compute the radar look direction for RSAT2 dat
; CALLING SEQUENCE:'
;   rsat2_compute_rlook, lf,  rlook_coef'
; INPUT:'
;   lf=           leader structure'
; OUTPUT:
;   lf=           Fit values in struture
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.'
; MODIFICATION
;  2009 Oct 21 Frank Monaldo 
;  2009 Dec 05 Frank Monaldo 
;              Added 1 pixel in range direction and azimuth before
;              Now only add in the range direction
;  2009 Dec 06 Frank Monaldo
;              It turns out I made the above correction for pixelTimeOrdering
;              INCREASING. Now the correctio is made for INCREASING and 
;              DECREASING
;  2009 Dec 23 Frank MonaldoYet another correction for pixelTimeOrdering 
;              DECREASING. Now the index is decrements I-1) instead of
;              (i+1)
;  2010 Jan 05 Modify to use ans_compute_fit
;  2010 Jun 14 Frank Monaldo. Print help on no input.   
;  2010 Dec 30 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_compute_rlook, lf,  rlook_coef, quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0

quiet=  printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 )  then begin
  print, 'NAME: RSAT2_COMPUTE_RLOOK'
  print, 'PURPOSE:'
  print, '  Compute the radar look direction for RSAT2 dat
  print, 'CALLING SEQUENCE:'
  print, '  rsat2_compute_rlook, lf,  rlook_coef'
  print, 'INPUT:'
  print, '  lf=           leader structure'
  print, 'OUTPUT:
  print, '  lf=           Fit values in struture
  print, 'KEYWORDS:
  print, '  /quiet=      Do not print intermediate results'
  print, '  /help=       List help.'
  return
endif


look=        dblarr(9)
ii=          dblarr(9)
jj=          dblarr(9)
increment=   4D
first_char=  strmid(lf.pixelTimeOrdering, 0, 1)
first_char=  strupcase(first_char)
ii= [ 4., lf.nx1/2., lf.nx1-5.,        4., lf.nx1/2., lf.nx1-5.,        0., lf.nx1/2., lf.nx1-5.]
jj= [ 4.,        4.,        4., lf.ny1/2., lf.ny1/2., lf.ny1/2., lf.ny1-5., lf.ny1-5., lf.ny1-5.]
for i=0, 8 do begin
  lon1= ans_poly_cal( ii[i],             jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat1= ans_poly_cal( ii[i],             jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  lon2= ans_poly_cal( ii[i] + increment, jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
  lat2= ans_poly_cal( ii[i] + increment, jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
  ll2rb, lon1, lat1, lon2, lat2, dist, azi
  if azi gt 180 then azi= azi - 360.0
  look[i] = double(azi) 
endfor

if first_char eq 'D' then look= look + 180.0
w=where(look gt 180, count)
if count gt 0 then look[w]= look[w]- 360



if printflag then print, 'i,j => look angle'
ans_compute_fit, ii,  jj,  look, lf.deg, rlook_coef, rlook_xexp, rlook_yexp, quiet=quiet

if printflag then begin
  print, ' i    j   look '
  for i=0,8 do print, string(ii[i],format='(I5)') +  ' ' + string(jj[i],format='(I5)') + ' ' +string(look[i])
endif

temp_lf= create_struct( lf, 'rlook_coef', rlook_coef, $
                            'rlook_xexp', rlook_xexp, $
                            'rlook_yexp', rlook_yexp  )
                            

                            
lf= 0
lf= temp_lf

end
