;-----------------------------------------------------
;+
; NAME: RSAT2_DN2SIGMA
; PURPOSE:
;    The purpose of this program is to convert the Radarsat-2 digital numbers
;    to radar cross section. We assume the digital numbner has been squared
; CALLING SEQUENCE:
;    rsat2_dn2sigma, image, bvalue, avalue
; INPUT:
;    image=   Array containing the digital number from the images
;             already sqaured.    
;    bvalue=  Offset
;    avalue=  Array of gains 
; OUTPUT:     
;    image=   A floating point array of sigma.
; KEYWORDS=   
;     /quiet= Do not print intermediate results.
;     /help=  List help.
;     /complexdata = set if data are complex
;
; MODIFICATION HISTORY:
;  2008 May 15  Frank Monaldo 
;  2009 May 06  Frank Monaldo. divided by A^2 rather than A for complex data
;  2010 Jun 14 Frank Monaldo. Print help on no input.
; Copyright (C) 2008, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_dn2sigma, image, bvalue, avalue, help=help, quiet=quiet, complexdata=complexdata

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: RSAT2_DN2SIGMA
  print, ' PURPOSE:
  print, '    The purpose of this program is to convert the Radarsat-2 digital numbers
  print, '    to radar cross section. We assume the digital numbner has been squared
  print, ' CALLING SEQUENCE:
  print, '    rsat2_dn2sigma, image, bvalue, avalue
  print, ' INPUT:
  print, '    image=   Array containing the digital number from the images
  print, '             already sqaured.    
  print, '    bvalue=  Offset
  print, '    avalue=  Array of gains
  print, ' OUTPUT:     
  print, '    image=   A floating point array of sigma.
  print, ' KEYWORDS=   
  print, '     /quiet= Do not print intermediate results.
  print, '     /help=  List help.
  print, '     /complexdata = set if data are complex
  return
endif

printflag=   1
complexd=    0
if keyword_set(complexdata) then complexd=1


if keyword_set(quiet)   then printflag=0 

sz=    size(image)
nx=    sz[1]
ny=    sz[2]
image= float(image)
nx2=   n_elements(avalue)
ratio= float(nx2) / float(nx)

if printflag then begin 
  print, '---------------------------------------------------------'
  print, 'RADARSAT-2 CALIBRATION: Converting digital number to sigma.'
  print, '---------------------------------------------------------'
  print, ' '
  print, 'Adding bias to image of ' + string( bvalue)
endif

;
; Note that the image was squared in the rsat2_read_image program
;

if complexd eq 0 then image=  image + float(bvalue)


avalue2=  avalue
if nx ne nx2 then begin
  if printflag then print, 'Resizing the Avalue array...'
  avalue2=interpol(avalue, findgen(nx2), findgen(nx) * ratio )
endif

if printflag  and ny lt 20000 then begin
  print, ''
  print, 'Image values AFTER squaring:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of squared Radarsat-2 data points=', mean(image[w])
    print, 'SD   of squared Radarsat-2 data points=', stdev(image[w])
  endif 
  if count gt 0 and count ge 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Statistics for first 4,000,000 nonzero values.'
    print, 'Mean of squared Radarsat-2 data points from file=', mean(image[w[0:3999999]])
    print, 'SD   of squared Radarsat-2 data points from file=', stdev(image[w[0:3999999]])
  endif
endif

w= where(avalue2 ne 0 )
if complexd then begin
 if printflag then print, 'Original data  complex'
 for j=0L, ny-1 do begin
    if printflag and (j mod 1000) eq 0 then begin 
       print, 'Applying gains on '+ strcompress(string(j)) + ' of ' +  strcompress(string(ny))
    endif
    image[w,j]= image[w,j] / avalue2[w] / avalue2[w]
   endfor
endif else begin
  if printflag then print, 'Original data not complex'
  for j=0L, ny-1 do begin
    if printflag and (j mod 1000) eq 0 then begin 
      print, 'Applying gains on '+ strcompress(string(j)) + ' of ' +  strcompress(string(ny))
    endif
    image[w,j]= image[w,j] / avalue2[w]
   endfor
endelse

if printflag  and ny lt 20000 then begin
  print, ''
  print, 'Image values AFTER calibration:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of Radarsat-2 sigma=', mean(image[w])
    print, 'SD   of Radarsat-2 sigma=', stdev(image[w])
  endif 
  if count gt 0 and count ge 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Statistics for first 4,000,000 nonzero values.'
    print, 'Mean of Radarsat-2 sigma=', mean(image[w[0:3999999]])
    print, 'SD   of Radarsat-2 sigma=', stdev(image[w[0:3999999]])
  endif
endif
end
