; '-----------------------------------------------------
; +
; NAME: 
;   RSAT2_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a two/three dimensional
;   polynomial fit to go from pixel location to incidence angle.  This
;   works with Radarsat-2 data
; CALLING SEQUENCE:
;   rsat2_incid2pix_fit, lf, ii, jj, incidence
; INPUTS:
;   lf:         Leader structure
;   ii:         columns
;   jj:         Rows
;   incidence:  Incident angle
; OUTPUTS
;   lf:         Fits place in lf structire 
; KEYWORDS:
;    /quiet=       Suppress printed messages. 
;    /help=        Display help
;    /diagnostic   Display comparison results
; MODIFICATION HISTORY:
;    2008 May 05 Frank Monaldo  
;                Hardwired a degree of 2 for polynomial fit
;                Removed requirement for defaults structure. The
;                parameters ave_line and ave_pixel are passed in
;                explicitly.
;    2010 Jan 05 Use new ans_compute_fit
;    2010 Jan 08 Frank Monaldo. Change fix to long
;    2010 Jun 14 Frank Monaldo. Print help on no input.
;    2010 Dec 30 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2003-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro  rsat2_incid2pix_fit, lf, ii, jj, incidence, incid_coef, $
                          quiet=       quiet,     $
                          help=        help,      $
                          diagnostic=  diagnostic

if keyword_set(help)  or (n_params(0) lt 1 ) then begin  
  print, ' +
  print, ' NAME: 
  print, '   RSAT2_INCID2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a two/three dimensional
  print, '   polynomial fit to go from pixel location to incidence angle.  This
  print, '   works with Radarsat-2 data
  print, ' CALLING SEQUENCE:
  print, '   rsat2_incid2pix_fit, lf, ii, jj, incidence
  print, ' INPUTS:
  print, '   lf:         Leader structure
  print, '   ii:         columns
  print, '   jj:         Rows
  print, '   incidence:  Incident angle
  print, ' OUTPUTS
  print, '   lf:         Fits place in lf structire 
  print, ' KEYWORDS:
  print, '    /quiet=       Suppress printed messages. 
  print, '    /help=        Display help
  print, '    /diagnostic   Display comparison results
  return
endif

;
; Parcing input parameters
;
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0



if printflag then  print, 'Computing fit of incidence vs i,j for Radarsat-2...'

w=where(incidence gt 0, count)
if count eq 0 then begin
  print, 'No nonzero incident angles'
  print, 'Exiting rsat2_incid2pix_fit'
  return
endif
incid= incidence[w]
iid=   ii[w]/ lf.ave_pixel
jjd=   jj[w]/ lf.ave_line  


if printflag then print, 'i,j => incident angle'
ans_compute_fit, iid,  jjd,  incid, lf.deg, incid_coef, incid_xexp, incid_yexp, quiet=quiet

temp_lf= create_struct( lf, 'incid_coef', incid_coef,  $
                            'incid_xexp', incid_xexp,  $
                            'incid_yexp', incid_yexp   )

lf=  0
lf=  temp_lf


fcheck=finite(incid_coef)
w=where(fcheck eq 0, fcount)
if fcount gt 0 then begin
   print, 'There is a problem with the incident angle fit'
   print, 'Input incident angles'
   print, incid
   stop
endif

if keyword_set(diagnostic) then begin
   print, 'Incidence angle conversions:'
   mu=0
   sd=0.
   for i=0L, ns -1 do begin
     i1=long(iid[j]+0.5)
     j1=long(jjd[j]+0.5)
     output=          string(i1, format='(I7)')
     output= output + string(j1, format='(I7)')
     output= output + string( incid[j] )
     incidp= ans_poly_cal( iid[j], jjd[j], lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
     mu=     mu+  (incid[j]-incidp)
     sd=     sd + (incid[j]-incidp)^2
     output= output + string( incidp )
     print,  output
   endfor
   print, 'Fractional error in incidence angle', mu/ns, sqrt(sd/ns)
   stop
endif	    
end
