; '-----------------------------------------------------
; +
; NAME: 
;    RSAT2_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a two (or specified) degfre
;   polynomial fit to go from lon/lat to pixel location and visa versa.
;   This program readings information from the lf structure
; CALLING SEQUENCE:
;    rsat2_lonlat2pix_fit, lf, ii, jj, longitude, latitude       
; INPUTS:        
;    lf=           Leaderfile structure
;    ii=           Array of column indices
;    jj=           Array of row indices
;    longitude=    Array of longitudes
;    latitude=     Array of latitude
; OUTPUTS:        
;    lf:           Fit coefs etc. placed into lf structure
; KEYWORDS:
;    /quiet=       Suppress printed messages. 
;    /help=        Display help
;    /diagnostic   Display comparison results
; MODIFICATION HISTORY:
;    2005 May 03 Frank Monaldo 
;    2009 Oct 14 Frank Monaldo
;                Modified so that a defaults structure is not needed. The 
;                defaults structure used to provide the deg, ave_line and ave_pixel
;                parameters. The now are explicited set
;    2010 Jan 05 Frank Monaldo Change for ans_compute_fit and ans_poly_cal_routines
;    2010 Jan 08 Frank Monaldo. Change fix to long
;    2010 Jun 14 Frank Monaldo. Print help on no input.
;    2010 Dec 30 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2005-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-
;-----------------------------------------------------
;
pro rsat2_lonlat2pix_fit,lf, ii, jj, longitude, latitude, $
                          quiet=       quiet,     $
                          help=        help,      $
                          diagnostic=  diagnostic


if keyword_set(help)  or (n_params(0) lt 1 ) then begin  
  print, ' NAME: 
  print, '    RSAT2_LONLAT2PIX_FIT
  print, ' PURPOSE:
  print, '   The purpose of this program is compute a two (or specified) degfre
  print, '   polynomial fit to go from lon/lat to pixel location and visa versa.
  print, '   This program readings information from the lf structure
  print, ' CALLING SEQUENCE:
  print, '    rsat2_lonlat2pix_fit, lf, ii, jj, longitude, latitude 
  print, ' INPUTS:        
  print, '    lf=           Leaderfile structure
  print, '    ii=           Array of column indices
  print, '    jj=           Array of row indices
  print, '    longitude=    Array of longitudes
  print, '    latitude=     Array of latitude
  print, ' OUTPUTS:        
  print, '    lf:           Fit coefs etc. placed into lf structure
  print, ' KEYWORDS:
  print, '    /quiet=       Suppress printed messages. 
  print, '    /help=        Display help
  print, '    /diagnostic   Display comparison results
  return
endif

;
; Parcing input parameters
;
printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if printflag then  print, 'Computing a fit to lon and lat for line of data...'

lf.ny1= lf.ny0 / lf.ave_line
lf.nx1= lf.nx0 / lf.ave_pixel

w=  where(latitude and longitude ne 0, count)
if count eq 0 then begin
  print, 'There are not nonzero longitudes and latitudes...'
  print, 'Exisitng rsat2_lonlat2pixfit'
  return
endif 
lat= double(latitude[w])
lng= double(longitude[w])
iid= double(ii[w])   / lf.ave_pixel ; reduce indices by averaging factor
jjd= double(jj[w])   / lf.ave_line  
  
;
; We wish to find equations of the form form
;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
;


if printflag then print, 'i,j => longitude'

ans_compute_fit,   iid,  jjd,  lng, lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
if printflag then print, 'i,j => latitude'
ans_compute_fit,   iid, jjd,  lat, lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet
if printflag then print, 'longitude, lattiude => i'
ans_compute_fit,  lng,  lat,  iid, lf.deg,   i_coef,   i_xexp,   i_yexp, quiet=quiet
if printflag then print, 'longitude, lattiude => j'
ans_compute_fit,  lng, lat,   jjd, lf.deg,   j_coef,   j_xexp,   j_yexp, quiet=quiet

temp_lf= create_struct( lf, 'lon_coef', lon_coef,  $
                            'lon_xexp', lon_xexp,  $
                            'lon_yexp', lon_yexp,  $
                            'lat_coef', lat_coef,  $
                            'lat_xexp', lat_xexp,  $
                            'lat_yexp', lat_yexp,  $
                            'i_coef',   i_coef,    $
                            'i_xexp',   i_xexp,    $
                            'i_yexp',   i_yexp,    $
                            'j_coef',   j_coef,    $
                            'j_xexp',   j_xexp,    $
                            'j_yexp',   j_yexp      )

lf=  0
lf=  temp_lf
; Check and see if we can go back and forth

if keyword_set(diagnostic) then begin
   ;
   ; Check latitude conversions
   ;
   print, 'Latitude conversions:'
   mu=0
   sd=0.
   for i=0L, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lat[i] )
     latp=   ans_poly_cal( iid[i], jjd[i], lat_coef, lat_xexp, lat_yexp )
     mu= mu+  (lat[i]-latp)
     sd= sd + (lat[i]-latp)^2
     output= output + string( latp )
     print, output
   endfor
   print, 'Latitude', mu/ns, sqrt(sd/ns)
   stop
   ;
   ; Check longitude conversions
   ;
   print, 'Longitude conversions:'
   mu=0
   sd=0.
   for i=0L, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output=          string(i1, format='(I5)')
     output= output + string(j1, format='(I5)')
     output= output + string( lng[i] )
     lngp=   ans_poly_cal( iid[i], jjd[i], lon_coef, lon_xexp, lon_yexp )
     mu= mu+  (lng[i]-lngp)
     sd= sd + (lng[i]-lngp)^2
     output= output + string( lngp )
     print,   output
   endfor
   print, 'Longitude',mu/ns, sqrt(sd/ns)
   stop
   ;
   ; Check i conversions
   ;
   print, 'i conversions:'
   mu=0
   sd=0.
   for i=0L, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(iid[i])
     ip=     ans_poly_cal( lng[i], lat[i], i_coef, i_xexp, j_yexp )
     mu= mu+  (iid[i]-ip)
     sd= sd + (iid[i]-ip)^2
     output= output + string(ip )
     print,  output
   endfor
   print, 'i',mu/ns, sqrt(sd/ns)
   stop
   ;
   ; Check j conversions
   ;
   print, 'j conversions:'
   mu=0
   sd=0.
   for i=0L, ns -1 do begin
     i1=long(iid[i]+0.5)
     j1=long(jjd[i]+0.5)
     output= string(lat[i]) + ' ' + string(lng[i])
     output= output + string(jjd[i])
     jp=     ans_poly_cal( lng[i], lat[i], j_coef, j_coef, j_xexp, j_yexp )
     mu= mu+  (jjd[i]-jp)
     sd= sd + (jjd[i]-jp)^2
     output= output + string(jp )
     print,  output
  endfor
  print, 'j',mu/ns, sqrt(sd/ns)
endif
end

