;-----------------------------------------------------
;+
; NAME: RSAT2_READ_IMAGE.PRO
; PURPOSE:
;  Read main Radarsat-2 TIFF image file and square the output
; CALLING SEQUENCE
;   rsat2_read_image, filename, ave_pixel, ave_line, lf, image, /help, /quiet, /NOSQU
; INPUT
;  filename   : Image file in TIFF format
;  ave_line   : Number of lines to average
;  ave_pixel  : Number of pixels to average
;  lf         : Structure with leader file information
; OUTPUT      
;  image      : Averaged image in the digital values in the file
; KEYWORDS:
;  /help       Get help
;  /quiet      Do not print out intermediate output
;  /nosqu      Do not square output
;  error       <0 if bad
;
; MODIFICATION HISTORY:
;  2008 May 05 Frank Monaldo  
;  2008 Sep 25 Frank Monaldo. Modifications to read complex data-FMM
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2014 Apr 15 Frank Monaldo. Catch error on reading TIFF file
; Copyright (C) 2008, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_read_image, filename, ave_pixel, ave_line, lf, image, help=help, quiet=quiet, $
                      nosqu=nosqu, error=error 
printflag=1
if keyword_set(quiet) then printflag=0
error    = 1

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print,  ' NAME: RSAT2_READ_IMAGE.PRO
  print,  ' PURPOSE:
  print,  '  Read main Radarsat-2 TIFF image file and square the output
  print,  ' CALLING SEQUENCE
  print,  '   rsat2_read_image, filename, ave_pixel, ave_line, lf, image, /help, /quiet, /NOSQU
  print,  ' INPUT
  print,  '  filename   : Image file in TIFF format
  print,  '  ave_line   : Number of lines to average
  print,  '  ave_pixel  : Number of pixels to average
  print,  '  lf         : Structure with leader file information
  print,  ' OUTPUT      
  print,  '  image      : Averaged image in the digital values in the file
  print,  ' KEYWORDS:
  print,  '  /help       Get help
  print,  '  /quiet      Do not print out intermediate output
  print,  '  /nosqu      Do not square output
  print,  '  error       <0 if bad
  error = -1
  return    
endif       

if printflag then begin
  print, 'Reading image data from TIFF file'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: rsat2_read_image' 
  return
endif
;
;
nx=    long(lf.nx0)
ny=    long(lf.ny0)
nx2=   nx / long(ave_pixel)
ny2=   ny / long(ave_line)
if printflag then begin
  print, 'Original image size: ' + string(nx, format='(I7)') + $
                                   string(ny, format='(I7)')
endif

nx3=       nx2 * ave_pixel
sub_image= uintarr(nx3, ave_line)

image=     fltarr( long(nx2), long(ny2) )
;
; Check for validity of TIFF file
;
res=  query_tiff(filename)
if res eq 0 then begin
  error = -2
  return
endif
if strupcase( lf.datatype ) eq 'COMPLEX'  and $
  ave_line                  eq 1          and $
  ave_pixel                 eq 1          and $
  keyword_set(nosqu) then begin
  ;
  ; full high-resolution complex data
  ;
  if printflag then print, 'Reading complex Radarsat-2 data file: ' + filename
  sub_image= read_tiff(filename, image_index=layer )
  realp=     reform( sub_image[0, *, *] )
  imagp=     reform( sub_image[1, *, *] )
  image=     complex( realp, imagp )
endif else begin
  ;
  if printflag then print, 'Reading Radarsat-2 data file: ' + filename
  for j=0L, ny2-1 do begin
    jj=        j * ave_line
    sub_image= read_tiff(filename, sub_rect=[0, jj, nx3, ave_line], image_index=layer )
    if strupcase(lf.datatype) ne 'COMPLEX' then begin
      if keyword_set(nosqu) then begin
        image[*,j]=     rebin(sub_image, nx2, 1)
        mu=  mean(sub_image)
        lab= 'Original data'
      endif else begin
        sub_image2=     float(sub_image) * float(sub_image)
        image[*,j]=     rebin(sub_image2, nx2, 1)
        mu=mean(sub_image2)
        lab= 'Original data squared.'
      endelse
      if printflag and (j mod 1000) eq 0 then begin
        percent_done= float(j) / ny2 * 100.0 
        print, 'Reading file. ' + string( percent_done, format='(F5.1)') + $
                '% done. '      + lab + ' Mean=' + string(mu)
      endif
    endif else begin
      sub_image2=     float(sub_image[0,*,*])^2 + float(sub_image[1,*,*])^2
      if keyword_set(nosqu) then sub_image2=sqrt(sub_image2)
      sub_image2=     reform(sub_image2)
      sub_image2=     float(rebin(sub_image2, nx2, 1))
      image[*,j]=     sub_image2
    endelse  
  endfor
endelse
if printflag then print, 'Finished reading file: ' + filename
end
