;-----------------------------------------------------
;+
; NAME: RSAT2_READ_PRODUCT
; PURPOSE:
;  Read main Radarsat-2 PRODUCT.xml file
; CALLING SEQUENCE
;  rsat2_read_product, filename, lf, ii, jj, incidence, longitude, latitude, sigma_ne
; INPUT
;  filename:       PRODUCT.XML file                                 
; OUTPUT
;  lf:             information structure
;  ii:             column array
;  jj:             row array
;  incidence:      incidence angle array
;  latitude:       latitude  array
;  longitude:      longitude array
;  outputfile:     Name of file to print the data to.
;  sigma_ne:       Noise equivalent sigma array
; MODIFICATION HISTORY:
; 2008 May 04 Frank Monaldo  
; 2008 Sep 25 Frank Monaldo. Add data_type
; 2010 Jan 13 Frank Monaldo. Fill in position/velocity in lf structure
; 2010 Jan 20 Frank Monaldo. Corrected reading of velocitties
; 2010 Jun 14 Frank Monaldo. Print help on no input.
; 2011 Mar 30 Frank Monaldo. Add nrcs_slope and nrs_bias to structure
; 2011 May 23 Frank Monaldo. Add jsbegin and jsend to structure
; 2011 May 27 Frank Monaldo. Fill in start and top time. Compute center time
; 2011 Aug 23 Frank Monaldo. Read sigma_ne
; 2020 May 27 Frank Monaldo. Better error handling
; Copyright (C) 2007-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_read_product, filename,lf, ii, jj, incidence, longitude, latitude, sigma_ne, $
                        outputfile=outputfile, help=help, quiet=quiet, error =error

re=    6378e3 ; Earth radius in meters
printflag=1
if keyword_set(quiet) then printflag=0
error = 1

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print,' NAME: RSAT2_READ_PRODUCT
  print,' PURPOSE:
  print,'  Read main Radarsat-2 PRODUCT.xml file
  print,' CALLING SEQUENCE
  print,'  rsat2_read_product, filename, lf, ii, jj, incidence, longitude, latitude
  print,' INPUT
  print,'  filename:       PRODUCT.XML file                                 
  print,' OUTPUT
  print,'  lf:             information structure
  print,'  ii:             column array
  print,'  jj:             row array
  print,'  incidence:      incidence angle array
  print,'  latitude:       latitude  array
  print,'  longitude:      longitude array
  print,'  outputfile:     Name of file to print the data to.
  print,'  sigma_ne:       Noise equivalent sigma array
  error = -1
  return    
endif       

if printflag then begin
  print, 'Reading general header information with rsat2_read_product'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: rsat2_read_product' 
  return
endif

if printflag then print, 'Reading file: ' + filename
s= getfile(filename)
n= n_elements(s)
;
;
; Creating leader 
lf={ product:                  '',               $        
     product_stage:            '',               $
     productType:              '',               $
     sar_sensor:               '',               $
     documentIdentifier:       '',               $  
     acquistion_station:       '',               $
     beam_mode:                '',               $
     proc_cent:                '',               $  
     proc_time:                '',               $  
     software_ver :            '',               $
     sensing_start:            '',               $
     sensing_stop:             '',               $
     data_year:                0,                $
     center_gmt:               '',               $
     month:                    '',               $
     day:                      0,                $
     hour:                     0,                $
     minute:                   0,                $
     second:                   0.0,              $
     abs_orbit:                0.0,              $ 
     state_vector_time:        '',               $
     x_position:               0.0D,             $
     y_position:               0.0D,             $         
     z_position:               0.0D,             $
     x_velocity:               0.0D,             $         
     y_velocity:               0.0D,             $
     z_velocity:               0.0D,             $
     vector_source:               '',            $
     tot_size:                 0L,               $
     sph_size:                 0L,               $
     num_dsd:                  0L,               $
     dsd_size:                 0L,               $
     num_data_sets:            0L,               $
     sph_descriptor:           '',               $  
     first_line_time:          '',               $
     last_line_time:           '',               $
     first_near_lat:           0.0D,             $
     near_start_lat:           0.0D,             $ ; fill in ASF style leader
     first_near_long:          0.0D,             $
     near_start_lon:           0.0D,             $ ; fill in ASF style leader
     first_mid_lat:            0.0D,             $
     first_mid_long:           0.0D,             $
     first_far_lat:            0.0D,             $ 
     far_start_lat:            0.0D,             $ ; fill in ASF style leader
     first_far_long:           0.0D,             $
     far_start_lon:            0.0D,             $ ; fill in ASF style leader
     last_near_lat:            0.0D,             $
     near_end_lat:             0.0D,             $ ; fill in ASF style leader
     last_near_long:           0.0D,             $
     near_end_lon:             0.0D,             $ ; fill in ASF style leader
     last_mid_lat:             0.0D,             $
     last_mid_long:            0.0D,             $
     last_far_lat:             0.0D,             $
     far_end_lat:              0.0D,             $ ; fill in ASF style leader
     last_far_long:            0.0D,             $
     far_end_lon:              0.0D,             $ ; fill in ASF style leader
     center_lon:               0.0,              $ ; fill in ASF style leader
     center_lat:               0.0,              $ ; fill in ASF style leader
     pass:                     '',               $
     algorithm:                '',               $
     sample_type:              '',               $
     mds1_tx_rx_polar:         '',               $
     mds2_tx_rx_polar:         '',               $
     compression:              '',               $
     azimuth_looks:            0.0,              $
     range_looks:              0.0,              $
     range_spacing:            0.0,              $
     azimuth_spacing:          0.0,              $
     line_length:              0L,               $
     data_type:                '',               $
     first_line_zero_doppler_time: strarr(500),  $
     line_num:                 ulonarr(500),     $
     num_lines:                ulonarr(500),     $
     sub_sat_track:            fltarr(500),      $
     first_samp_number:        ulonarr(500,11),  $
     first_slant_range_times:  fltarr(500,11),   $
     first_angles:             fltarr(500,11),   $
     first_lats:               fltarr(500,11),   $
     first_longs:              fltarr(500,11),   $ 
     last_line_zero_doppler_time:  strarr(500),  $
     last_samp_number:         ulonarr(500,11),  $
     last_slant_range_times:   fltarr(500,11),   $
     last_angles:              fltarr(500,11),   $
     last_lats:                fltarr(500,11),   $
     last_longs:               fltarr(500,11),   $
     ds_name:                  strarr(500),      $
     ds_type:                  strarr(500),      $
     ds_offset:                lonarr(500),      $
     ds_size:                  lonarr(500),      $
     num_dsr:                  lonarr(500),      $
     dsr_size:                 lonarr(500),      $
     srgr:                     byte(0),          $
     proc_scaling_fact:        fltarr(2),        $
     ext_cal_fact:             fltarr(4),        $ 
     ny0:                      0L,               $
     nx0:                      0L,               $
     ny1:                      0L,               $
     nx1:                      0L,               $
     lngswitchflag:            0,                $
     proc_fac:                 'XXXX',           $
     datatake_id:              '',               $   ; fill in ASF style leader
     projection:               'GROUND RANGE',   $   ; fill in ASF style leader
     polarization:             'V',              $   ; fill in ASF style leader
     leaderfile:                '',              $
     rootfile:                  '',              $   
     pixel_spacing:             0.00,            $   ; fill in GAT style leader    
     line_spacing:              0.00,            $   ; fill in GAT style header  
     range_pixel:               0.00,            $   ; fill in ASF style header
     azimuth_pixel:             0.00,            $
     radar_frequency:           '9.65e+9',       $   ; HZ
     radar_band:               'C',              $
     layer:                     0,               $
     corner_ll:                fltarr(8),        $
     asc_des:                  '',               $
     datatype:                 '',               $
     bitspersample:            16,               $
     lineTimeOrdering:         '',               $
     pixelTimeOrdering:        '',               $
     s_c_altitude_m:           0.0D,             $
     slant_range_near_m:       0.0D,             $
     incidence_near_deg:       0.0D,             $
     incidence_far_deg:        0.0D,             $
     imagefiles:               strarr(5),        $
     antennaPointing:          'RIGHT',          $
     groundToSlantRangeCoefficients: '',         $
     ellipsoidName:            '',               $
     semiMajorAxis:            '',               $
     semiMinorAxis:            '',               $
     datumShiftParameters:     '',               $
     software_version:         '',               $
     incidenceAngleNearRange:  '',               $
     incidenceAngleFarRange:   '',               $
     slantRangeNearEdge:       '',               $
     satelliteHeight:          '',               $
     inputDatasetId:           '',               $
     nrcs_slope:               1.0,              $
     nrcs_bias:                0.0,              $
     jsbegin:   -3155673600D,                    $   ; Default setting for start data
     jsend:     -3155673600D                     }   ; Default setting for end  data

monthstr=['', 'jan', 'feb', 'mar', 'apr', 'may', 'jun', $
              'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
              
if printflag then print, 'Parsing:      ' + filename
;
; General parameters
;
n= n_elements(s)

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'productId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.product= getwrd(ss, 1)
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'productType')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.productType= getwrd(ss, 1)
endif
;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'dataType')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.datatype= getwrd(ss, 1)
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'inputDatasetId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.inputDatasetId= getwrd(ss, 1)
endif
;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'beamModeMnemonic')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.beam_mode= getwrd(ss, 1)
endif

;
index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'imageId')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.datatake_id= getwrd(ss, 1)
endif

index= intarr(n)
for i=0, n-1 do index[i]= strpos(s[i], 'processingFacility')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.proc_fac= getwrd(ss, 1)
endif
;
for i=0, n-1 do index[i]= strpos(s[i], 'documentIdentifier')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.documentIdentifier= getwrd(ss, 1)
endif
;

for i=0, n-1 do index[i]= strpos(s[i], 'satellite')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.sar_sensor= getwrd(ss, 1)
endif
;
for i=0, n-1 do index[i]= strpos(s[i], 'softwareVersion')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.software_version= getwrd(ss, 1)
endif
;
for i=0, n-1 do index[i]= strpos(s[i], 'polarizations')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'polarizations',' /',ned)
  lf.polarization= strcompress(getwrd(ss, 1, del='/'))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'ellipsoidName')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'ellipsoidName',' /',ned)
  lf.ellipsoidName= strcompress(getwrd(ss, 1, del='/'))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'numberOfRangeLooks')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'numberOfRangeLooks','/',ned)
  lf.range_looks= float(strcompress(getwrd(ss, 1, del='/')))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'numberOfAzimuthLooks')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'numberOfAzimuthLooks','/',ned)
  lf.azimuth_looks= float(strcompress(getwrd(ss, 1, del='/')))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'groundToSlantRangeCoefficients')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'groundToSlantRangeCoefficients',' /',ned)
  lf.groundToSlantRangeCoefficients= strcompress(getwrd(ss, 1, del='/'))
endif

;
for i=0, n-1 do index[i]= strpos(s[i], 'semiMajorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'semiMajorAxis','/',ned)
  lf.semiMajorAxis= getwrd(ss, 2) 
endif
;
for i=0, n-1 do index[i]= strpos(s[i], 'semiMinorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'semiMinorAxis','/',ned)
  lf.semiMinorAxis= getwrd(ss, 2) 
endif

for i=0, n-1 do index[i]= strpos(s[i], 'datumShiftParameters')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  ss= stress(ss,'R',0,'datumShiftParameters','/',ned)
  ss= stress(ss,'R',0,'units="m"','/',ned)
  lf.datumShiftParameters= getwrd(ss, 2,del= '/') 
endif
;
for i=0, n-1 do index[i]= strpos(s[i], 'radarCenterFrequency')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.radar_frequency= float(getwrd(ss, 2 ))
endif


;
for i=0, n-1 do index[i]= strpos(s[i], 'incidenceAngleNearRange')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.incidenceAngleNearRange= getwrd(ss, 2 )
endif

;
for i=0, n-1 do index[i]= strpos(s[i], 'incidenceAngleFarRange')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.incidenceAngleFarRange= getwrd(ss, 2 )
endif

;
for i=0, n-1 do index[i]= strpos(s[i], 'slantRangeNearEdge')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.slantRangeNearEdge= getwrd(ss, 2 )
endif

;
for i=0, n-1 do index[i]= strpos(s[i], 'satelliteHeight')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' )
  lf.satelliteHeight= getwrd(ss, 2 ) 
endif

for i=0, n-1 do index[i]= strpos(s[i], 'passDirection')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.asc_des=  strupcase(getwrd(ss, 1))
endif


for i=0, n-1 do index[i]= strpos(s[i], 'xPosition')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.x_position=  double(getwrd(ss, 2))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'yPosition')
w=where(index ge 0,count) 
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.y_position=  double(getwrd(ss, 2))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'zPosition')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.z_position=  double(getwrd(ss, 2))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'xVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.x_velocity=  double(getwrd(ss, 3))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'yVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.y_velocity=  double(getwrd(ss, 3))
endif

for i=0, n-1 do index[i]= strpos(s[i], 'zVelocity')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.z_velocity=  double(getwrd(ss, 3))
endif


for i=0, n-1 do index[i]= strpos(s[i], 'rawDataStartTime')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.sensing_start=  strupcase(getwrd(ss, 1))
  lf.center_gmt=     lf.sensing_start
  lf.data_year=           fix( strmid(lf.center_gmt,  0,  4) )
  lf.month=     monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
  lf.day=                 fix( strmid(lf.center_gmt,  8,  2) )
  lf.hour=                fix( strmid(lf.center_gmt, 11,  2) )
  lf.minute=              fix( strmid(lf.center_gmt, 14,  2) )
  data_year=              fix( strmid(lf.center_gmt,  0,  4) )
  lf.month=     monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
  lf.day=                 fix( strmid(lf.center_gmt,  8,  2) )
  lf.hour=                fix( strmid(lf.center_gmt, 11,  2) )
  lf.minute=              fix( strmid(lf.center_gmt, 14,  2) )
  lf.second=              fix( strmid(lf.center_gmt, 17,  8) )
endif

; Time stamp
for i=0, n-1 do index[i]= strpos(s[i], 'timeStamp')
w=where(index ge 0,count)
timejs=     dblarr(count)

if count gt 0 then begin
  for i=0L, count-1 do begin
    ss= s[w[i]]
    ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
    ss= strupcase(getwrd(ss, 1))
    year= strmid(ss,0,4)
    mon=  monthstr( fix( strmid(ss,  5,  2) ) )
    day=  strmid(ss,  8,  2)  
    ss=   year + ' ' + mon + ' ' + day + ' ' + strmid(ss,11,11)
    timejs[i]=  dt_tm_tojs(ss)
  endfor
endif

w= where(timejs gt 0, cc )
if cc gt 0 then begin
  lf.jsbegin=   min( timejs) 
  lf.jsend=     max( timejs) 
  jscen=       (lf.jsbegin + lf.jsend)/ 2.0 
endif
;print, dt_tm_fromjs(lf.jsbegin) 
;print, dt_tm_fromjs(lf.jsend  ) 
;print, dt_tm_fromjs(   jscen  ) 

lf.center_gmt=     dt_tm_fromjs(jscen,for='Y$-0n$-d$Th$:m$:s$')
lf.data_year=           fix( strmid(lf.center_gmt,  0,  4) )
lf.month=     monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
lf.day=                 fix( strmid(lf.center_gmt,  8,  2) )
lf.hour=                fix( strmid(lf.center_gmt, 11,  2) )
lf.minute=              fix( strmid(lf.center_gmt, 14,  2) )
lf.second=              fix( strmid(lf.center_gmt, 17,  2) )




for i=0, n-1 do index[i]= strpos(s[i], 'antennaPointing')
w=where(index ge 0,count)
if count gt 0 then begin
  ss= s[w[0]]
  ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
  lf.antennaPointing=  strupcase(getwrd(ss, 1))
endif
for i=0L, n-1 do index[i]= strpos(s[i], 'rasterAttributes')
w=where(index ge 0, count)
if count ge 1 then begin
  s1=s[w[0]:w[1]]
  n1= n_elements(s1)
  for i=0, n1-1 do begin
    ss= s1[i] & ss= repchr( ss, '<', ' ' ) & ss= repchr( ss, '>', ' ' ) &  ss= repchr( ss, '/', ' ' )
    if strpos(ss, 'dataType')               ge 0 then dataType=               getwrd(ss, 1)
    if strpos(ss, 'bitsPerSample')          ge 0 then bitsPerSample=          getwrd(ss, 2)
    if strpos(ss, 'numberOfSamplesPerLine') ge 0 then numberOfSamplesPerLine= getwrd(ss, 1)
    if strpos(ss, 'numberOfLine')           ge 0 then numberOfLines=          getwrd(ss, 1)			
    if strpos(ss, 'sampledPixelSpacing')    ge 0 then sampledPixelSpacing=    getwrd(ss, 2)		
    if strpos(ss, 'sampledLineSpacing')     ge 0 then sampledLineSpacing=     getwrd(ss, 2)
    if strpos(ss, 'lineTimeOrdering')       ge 0 then lineTimeOrdering=       getwrd(ss, 1)	
    if strpos(ss, 'pixelTimeOrdering')      ge 0 then pixelTimeOrdering=      getwrd(ss, 1)
  endfor
  lf.range_pixel=       sampledPixelSpacing
  lf.azimuth_pixel=     sampledLineSpacing
  lf.ny0=               numberOfLines
  lf.nx0=               numberOfSamplesPerLine
  lf.pixelTimeOrdering= strupcase(pixelTimeOrdering         ) 
  lf.lineTimeOrdering=  strupcase(lineTimeOrdering)
  if printflag then begin
    print, 'lf.range_pixel       ', lf.range_pixel      
    print, 'lf.azimuth_pixel     ', lf.azimuth_pixel    
    print, 'lf.ny0               ', lf.ny0              
    print, 'lf.nx0               ', lf.nx0              
    print, 'lf.pixelTimeOrdering ', lf.pixelTimeOrdering
    print, 'lf.lineTimeOrdering  ', lf.lineTimeOrdering 
  endif
endif
    
; ----------------------------------------------------
;                   GENERAL GET THE SIZE OF THE ARRAY
; ---------------------------------------------------
       
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'geolocationGrid')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=  w[0]
  iend=    w[1]
  index2=  index[w[0]:w[1]]
  s2=      s[w[0]:w[1]]
  n=       n_elements(s2)
  index2= intarr(n) & istart=-1 & iend=-1
  for i=0L, n-1 do index2[i]= strpos(s2[i], 'numberOfGridPoints')
  w=where(index2 ge 0,count)
  if count gt 0 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s2[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'azimuth':   nazimuth= long(getwrd(ss,1))
      'range':     nrange=   long(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif
; ----------------------------------------------------
;                   GENERAL GRID REFERNCE TIMES
; ---------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'gridReferenceTime')
w=where(index ge 0,count)
if count ge 2 then begin
  tReferenceTimeUTC=''
  tauReferenceTime= 0.0
  if count ge 2 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'tReferenceTimeUTC':    tReferenceTimeUTC=         getwrd(ss,1)
      'tauReferenceTime':     tauReferenceTime=   double(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif


; ----------------------------------------------------
;                   FILL IN ARRAY
; ---------------------------------------------------
n=      n_elements(s)      
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<imageTiePoint>')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</imageTiePoint>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)
ii=        lonarr(count)
jj=        lonarr(count)
longitude= dblarr(count)
latitude=  dblarr(count)
height=    dblarr(count)
incidence= dblarr(count)
if count gt 0 then begin
  for cc=0L, count - 1L do begin
    istart=w1[cc]
    iend=  w2[cc]
    if printflag and (i mod 100) eq 0 then begin
      print, 'Parsing line: ' + string(cc, format='(I8)') + 'for ii, jj, longitude latitude, and height...'
    endif
    for i=istart, iend do begin
      ss=  s[i]
      ss=  repchr( ss, '<', ' ' )
      ss=  repchr( ss, '>', ' ' )
      ss=  repchr( ss, '/', ' ' )
      key= getwrd( ss, 0)
      case key of
      'line':        jj[cc]=         long(    float(getwrd(ss, 1 )))
      'pixel':       ii[cc]=         long(    float(getwrd(ss, 1 )))                                                                  
      'latitude':    latitude[cc]=   double(        getwrd(ss, 2 ))  ; degrees                                                               
      'longitude':   longitude[cc]=  double(        getwrd(ss, 2 ))  ; degrees                                                                 
      'height':      height[cc]=     double(        getwrd(ss, 2 ))  ; degrees    
      ELSE: ; No operation 
      endcase
    endfor    ; End loop i          ; Loop through i, s[i] of lines between imageTiePoint tages
  endfor      ; End loop cc         ; Loop through number of imageTiePoints
endif         ; End if count gt 0   ; Check to see if there are any imageTiePoints

;-----------------------------------------------------
; Read information to determine the incidence angle
;----------------------------------------------------

n=      n_elements(s)      
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<sarProcessingInformation>')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</sarProcessingInformation>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)
istart=             w1[0]
iend=               w2[0]
incidence_near_deg= 0
incidence_far_deg=  0
slant_range_near_m= 0
s_c_altitude_m=     lf.satelliteHeight
for i=istart, iend do begin
  ss=  s[i]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  key= getwrd( ss, 0)
  case key of
  'incidenceAngleNearRange': incidence_near_deg= double(    float(getwrd(ss, 2 ))) ; degrees
  'incidenceAngleFarRange':  incidence_far_deg=  double(    float(getwrd(ss, 2 ))) ; degrees
  'slantRangeNearEdge':      slant_range_near_m= double(    float(getwrd(ss, 2 ))) ; meters                              
  'satelliteHeight':         s_c_altitude_m=     double(    float(getwrd(ss, 2 ))) ; meters                                                               
  ELSE: ; No operation 
  endcase
endfor    ; End loop i          ; Loop through i, s[i] of lines between imageTiePoint tages
if printflag then begin
  print, 'incidence_near_deg:', incidence_near_deg
  print, 'incidence_far_deg:',  incidence_far_deg
  print, 'slant_range_near_m:', slant_range_near_m
  print, 's_c_altitude_m:',     s_c_altitude_m
endif 
;
; Pixel time direction determines whether the first pixel is near range or far range. 
; Hence the calculation must be inverted
;
if printflag then print,'PixelTimeOrdering ' + lf.pixelTimeOrdering
if strupcase(lf.pixelTimeOrdering) eq 'INCREASING' then begin
  ;
  ; near range is first pixel
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0L, n-1 do begin
    c=            c0 + ii[k] *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor 
endif else begin
  ;
  ; far range is first pixel
  ; 
  ; First compute the slant range at far incidence
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0, n-1 do begin
    c=            c0 + (lf.nx0 -ii[k]) *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor
endelse
;---------------------------------------
; Read noise equivalent sigma_ne array
;---------------------------------------
sigma_ne= 0
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<referenceNoiseLevel incidenceAngleCorrection="Sigma Nought">')
w1=where(index ge 0, count)

if count gt 0 then begin ; There is an array of sigma_ne as a function of range index
  istart  =w1[0]
  i=      istart
  while i lt n do begin
    if strpos(s[i], '</referenceNoiseLevel>') ge 0 then begin
      iend=i
      i=n-1
    endif
    i= i+1
  endwhile
  pixelfirstnoisevalue=     -1
  stepsize=                  0
  numberofnoiseLevelvalues=  0
  i1=  istart
  if iend le n-1 then begin ; We have found sigma_ne info
    for i=istart, iend do begin
       ss=  s[i]
       ss=  repchr( ss, '<', ' ' )
       ss=  repchr( ss, '>', ' ' )
       ss=  repchr( ss, '/', ' ' )
       key= getwrd( ss, 0)
       case key of
      'pixelFirstNoiseValue':      pixelfirstnoisevalue=     long( getwrd(ss, 1 ))
      'stepSize':                  stepsize=                 long( getwrd(ss, 1 )) 
      'numberOfNoiseLevelValues':  numberofnoiseLevelvalues= long( getwrd(ss, 1 ))                            
      'noiseLevelValues':          i1=i  
      ELSE:                        ; No operation
      endcase      
    endfor
    if printflag then begin
      print, 'Sigma Noise Level Values'
      print, 'pixelfirstnoisevalue=     ', pixelfirstnoisevalue     
      print, 'stepsize=                 ', stepsize                 
      print, 'numberofnoiseLevelvalues= ', numberofnoiseLevelvalues 
    endif
    if i1 ge istart and i1 le iend then begin
      dataline= s[i1]
      dataline=  stress(dataline, 'D', 0, '<noiseLevelValues units="dB">', ned )
      dataline=  stress(dataline, 'D', 0, '</noiseLevelValues>',           ned )
      if pixelfirstnoisevalue        gt -1 and $
         stepsize                    gt  0 and $
         numberofnoiseLevelvalues    gt  0 and $
         long(numberOfSamplesPerLine) gt  0 then begin
         sigma_ne=  fltarr( long(numberOfSamplesPerLine) )
         for i=0,  numberofnoiseLevelvalues-1 do begin
           j=  pixelfirstnoisevalue + i * stepsize                  
           sigma_ne[j]= float( getwrd(dataline, i) )
         endfor
      endif
    endif
  endif  ; End we have found sigma_ne info
endif ; End check for sigma_ne data existence


;---------------------------------------
; Get filenames
; ---------------------------------------

n=      n_elements(s)      
index=  intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<imageAttributes>')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</imageAttributes>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)
polarr= strarr(5)

for i=0, 4 do polarr[i]=getwrd(lf.polarization, i)

if count gt 0 then begin
  istart=w1[0]
  iend=  w2[0]
  for i=istart, iend do begin
    ss=  s[i]
    ss=  repchr( ss, '<', ' ' )
    ss=  repchr( ss, '>', ' ' )
    ss=  repchr( ss, '/', ' ' )
    key= getwrd( ss, 0)
    if key eq 'fullResolutionImageData' then begin
      polkey=getwrd( ss, 1, del='"')
      for k=0, 4 do begin
        if polkey eq polarr[k] then begin
           lf.imagefiles[k]= getwrd(ss, 2)
           if printflag then print, polarr[k] + ' ' + lf.imagefiles[k]
        endif
      endfor
    endif
  endfor
endif  ; Check count of imageAttributes tags
if printflag then begin
  print, 'Image files: ',   lf.imagefiles
  print, 'Polarizations: ', lf.polarization
endif

if keyword_set(outputfile) then begin
  close,  1
  openw,  1,  outputfile
  printf, 1, 'product=               ' + lf.product       
  printf, 1, 'documentidentifier=    ' + lf.documentidentifier
  printf, 1, 'sar_sensor=            ' + lf.sar_sensor
  printf, 1, 'polariztion=           ' + lf.polarization
  printf, 1, 'radar_frequency=       ' + lf.radar_frequency
  printf, 1, 'asc_des=               ' + lf.asc_des
  printf, 1, 'center_gmt=            ' + lf.center_gmt
  printf, 1, 'data_year=             ' + strcompress(string(lf.data_year))
  printf, 1, 'month=                 ' + lf.month
  printf, 1, 'day=                   ' + strcompress(string(lf.day))
  printf, 1, 'hour=                  ' + strcompress(string(lf.hour))
  printf, 1, 'minute=                ' + strcompress(string(lf.minute))
  printf, 1, 'second=                ' + strcompress(string(lf.second))
  printf, 1, 'antennapointing=       ' + strcompress(string(lf.antennapointing))
  printf, 1, 'range_pixel=           ' + strcompress(string(lf.range_pixel))
  printf, 1, 'azimuth_pixel=         ' + strcompress(string(lf.azimuth_pixel))
  printf, 1, 'numberoflines=         ' + strcompress(string(lf.nx0))
  printf, 1, 'numberofsamplesperline=' + strcompress(string(lf.ny0))
  printf, 1, 'pixeltimeordering=     ' + lf.pixeltimeordering
  printf, 1, 'linetimeordering=      ' + lf.linetimeordering
  n=n_elements(ii)
  printf, 1,  'i  j longitude, latitude'
  for i=0L, n-1 do begin
    printf, 1, string(double(ii[i]),        format='(E24.16E3)') + ' ' + $
               string(double(jj[i]),        format='(E24.16E3)') + ' ' + $
               string(double(longitude[i]), format='(E24.16E3)') + ' ' + $
               string(double(latitude[i]),  format='(E24.16E3)')
  endfor
  printf,1, 'i, incidence'
  n=n_elements(incidence)
  for i=0L, n-1 do begin
    printf, 1, string(double(ii[i]),           format='(E24.16E3)') + ' ' + $
               string(double(incidence[i]),    format='(E24.16E3)') 
  endfor
  close, 1
  if printflag then print, 'Stored file='+outputfile
endif

end
