;-----------------------------------------------------
;+
; NAME: RSAT2_READ_PRODUCT_FAST'
; PURPOSE:'
;  Read main Radarsat-2 PRODUCT.xml file. Relies on  IDL 8.7'
; CALLING SEQUENCE'
;  rsat2_read_product_fast , filename, lf, ii, jj, incidence, longitude, latitude, sigma_ne'
; INPUT'
;  filename:       PRODUCT.XML file          '                       
; OUTPUT'
;  lf        :  Information structure'
;  ii        :  Column array'
;  jj        :  Row array'
;  incidence :  Incident angle array'
;  latitude  :  latitude  array'
;  longitude :  longitude array'
;  outputfile:  Name of file to print the data to.'
;  sigma_ne  :  Noise equivalent sigma array'
;  error     :  Less than 0 if does not complete properly'
;                -1 : Return for help'
;                -2 : Two early a version of IDL'
;                -3 : Cannot find product.xml file'
;  /help         Print help'
;  /quiet        Process quietly'
; MODIFICATION HISTORY:
; 2020 May 21 Frank Monaldo.  Rewritten to us IDL 8.7 XML parser
;                             Can deal with different versions of XML file that
;                             do and do not have polarization tags in the 
;                             nosielevel variable
; Copyright (C) 2020, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_read_product_fast, filename,lf, ii, jj, incidence, longitude, latitude, sigma_ne, $
                        outputfile=outputfile, help=help, quiet=quiet, error=error

re      =    6378e3 ; Earth radius in meters
printflag=1
if keyword_set(quiet) then printflag=0
error = 1
if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: RSAT2_READ_PRODUCT_FAST'
  print, ' PURPOSE:'
  print, '  Read main Radarsat-2 PRODUCT.xml file. Relies on  IDL 8.7'
  print, ' CALLING SEQUENCE'
  print, '  rsat2_read_product_fast , filename, lf, ii, jj, incidence, longitude, latitude, sigma_ne'
  print, ' INPUT'
  print, '  filename:       PRODUCT.XML file          '                       
  print, ' OUTPUT'
  print, '  lf        :  Information structure'
  print, '  ii        :  Column array'
  print, '  jj        :  Row array'
  print, '  incidence :  Incident angle array'
  print, '  latitude  :  latitude  array'
  print, '  longitude :  longitude array'
  print, '  outputfile:  Name of file to print the data to.'
  print, '  sigma_ne  :  Noise equivalent sigma array'
  print, '  error     :  Less than 0 if does not complete properly'
  print, '                -1 : Return for help'
  print, '                -2 : Two early a version of IDL'
  print, '                -3 : Cannot find product.xml file'
  print, '  /help         Print help'
  print, '  /quiet        Process quietly'
  error = -1
  return    
endif    

; 
; Version test
; 

v = float(getwrd(!version.release,0, del='.')) + 0.1 * fix(getwrd(!version.release,1, del='.')) 
if v le 8.6 then begin
  if printflag then begin
    print, 'The IDL release is: '+ !version.release
    print, 'It must be greater than 8.6 for this software'
    print, 'The XML parse code is required'
    print, 'Exiting rsat2_read_proudct_fast.pro'
  endif
  error = -2
  return
endif  

res= file_search(filename, count=count)
if count  eq 0 then begin
  if printflag then begin
    print, 'Cannot find file: ' + filename
    print, 'Exiting: rsat2_read_product' 
  endif
  error = -3
  return
endif

re=    6378.0e3 ; Earth radius in meters
;
; Read new way
;
x         = 0
x_hash    = xml_parse(filename)
x_str    = x_hash.ToStruct(/recursive)
;
; Creating leader structure
;
lf={ product:                  '',               $        
     product_stage:            '',               $
     productType:              '',               $
     sar_sensor:               '',               $
     documentIdentifier:       '',               $  
     acquistion_station:       '',               $
     beam_mode:                '',               $
     proc_cent:                '',               $  
     proc_time:                '',               $  
     software_ver :            '',               $
     sensing_start:            '',               $
     sensing_stop:             '',               $
     data_year:                0,                $
     center_gmt:               '',               $
     month:                    '',               $
     day:                      0,                $
     hour:                     0,                $
     minute:                   0,                $
     second:                   0.0,              $
     abs_orbit:                0.0,              $ 
     state_vector_time:        '',               $
     x_position:               0.0D,             $
     y_position:               0.0D,             $         
     z_position:               0.0D,             $
     x_velocity:               0.0D,             $         
     y_velocity:               0.0D,             $
     z_velocity:               0.0D,             $
     vector_source:               '',            $
     tot_size:                 0L,               $
     sph_size:                 0L,               $
     num_dsd:                  0L,               $
     dsd_size:                 0L,               $
     num_data_sets:            0L,               $
     sph_descriptor:           '',               $  
     first_line_time:          '',               $
     last_line_time:           '',               $
     first_near_lat:           0.0D,             $
     near_start_lat:           0.0D,             $ ; fill in ASF style leader
     first_near_long:          0.0D,             $
     near_start_lon:           0.0D,             $ ; fill in ASF style leader
     first_mid_lat:            0.0D,             $
     first_mid_long:           0.0D,             $
     first_far_lat:            0.0D,             $ 
     far_start_lat:            0.0D,             $ ; fill in ASF style leader
     first_far_long:           0.0D,             $
     far_start_lon:            0.0D,             $ ; fill in ASF style leader
     last_near_lat:            0.0D,             $
     near_end_lat:             0.0D,             $ ; fill in ASF style leader
     last_near_long:           0.0D,             $
     near_end_lon:             0.0D,             $ ; fill in ASF style leader
     last_mid_lat:             0.0D,             $
     last_mid_long:            0.0D,             $
     last_far_lat:             0.0D,             $
     far_end_lat:              0.0D,             $ ; fill in ASF style leader
     last_far_long:            0.0D,             $
     far_end_lon:              0.0D,             $ ; fill in ASF style leader
     center_lon:               0.0,              $ ; fill in ASF style leader
     center_lat:               0.0,              $ ; fill in ASF style leader
     pass:                     '',               $
     algorithm:                '',               $
     sample_type:              '',               $
     compression:              '',               $
     azimuth_looks:            0.0,              $
     range_looks:              0.0,              $
     range_spacing:            0.0,              $
     azimuth_spacing:          0.0,              $
     line_length:              0L,               $
     data_type:                '',               $
     first_line_zero_doppler_time: strarr(500),  $
     line_num:                 ulonarr(500),     $
     num_lines:                ulonarr(500),     $
     sub_sat_track:            fltarr(500),      $
     first_samp_number:        ulonarr(500,11),  $
     first_slant_range_times:  fltarr(500,11),   $
     first_angles:             fltarr(500,11),   $
     first_lats:               fltarr(500,11),   $
     first_longs:              fltarr(500,11),   $ 
     last_line_zero_doppler_time:  strarr(500),  $
     last_samp_number:         ulonarr(500,11),  $
     last_slant_range_times:   fltarr(500,11),   $
     last_angles:              fltarr(500,11),   $
     last_lats:                fltarr(500,11),   $
     last_longs:               fltarr(500,11),   $
     ds_name:                  strarr(500),      $
     ds_type:                  strarr(500),      $
     ds_offset:                lonarr(500),      $
     ds_size:                  lonarr(500),      $
     num_dsr:                  lonarr(500),      $
     dsr_size:                 lonarr(500),      $
     srgr:                     byte(0),          $
     proc_scaling_fact:        fltarr(2),        $
     ext_cal_fact:             fltarr(4),        $ 
     ny0:                      0L,               $
     nx0:                      0L,               $
     ny1:                      0L,               $
     nx1:                      0L,               $
     lngswitchflag:            0,                $
     proc_fac:                 'XXXX',           $
     datatake_id:              '',               $   ; fill in ASF style leader
     projection:               'GROUND RANGE',   $   ; fill in ASF style leader
     polarization:             'VV',             $   ; fill in ASF style leader
     leaderfile:                '',              $
     rootfile:                  '',              $   
     pixel_spacing:             0.00,            $   ; fill in GAT style leader    
     line_spacing:              0.00,            $   ; fill in GAT style header  
     range_pixel:               0.00,            $   ; fill in ASF style header
     azimuth_pixel:             0.00,            $
     radar_frequency:           '9.65e+9',       $   ; HZ
     radar_band:               'C',              $
     layer:                     0,               $
     corner_ll:                fltarr(8),        $
     asc_des:                  '',               $
     datatype:                 '',               $
     bitspersample:            16,               $
     lineTimeOrdering:         '',               $
     pixelTimeOrdering:        '',               $
     s_c_altitude_m:           0.0D,             $
     slant_range_near_m:       0.0D,             $
     incidence_near_deg:       0.0D,             $
     incidence_far_deg:        0.0D,             $
     imagefiles:               strarr(5),        $
     antennaPointing:          'RIGHT',          $
     ellipsoidName:            '',               $
     semiMajorAxis:            '',               $
     semiMinorAxis:            '',               $
     datumShiftParameters:     '',               $
     software_version:         '',               $
     incidenceAngleNearRange:  '',               $
     incidenceAngleFarRange:   '',               $
     slantRangeNearEdge:       '',               $
     GROUNDTOSLANTRANGECOEFFICIENTS : '',        $
     satelliteHeight:          '',               $
     inputDatasetId:           '',               $
     nrcs_slope:               1.0,              $
     nrcs_bias:                0.0,              $
     jsbegin:   -3155673600D,                    $   ; Default setting for start data
     jsend:     -3155673600D                     }   ; Default setting for end  data
     
     

monthstr=['', 'jan', 'feb', 'mar', 'apr', 'may', 'jun', $
              'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
              
              
lf.product                 = tag_value(x_str.product,'productid')      
lf.documentidentifier      = tag_value(x_str.product,'documentidentifier')  
;   
; Image Generation Parameters
lf.producttype             = tag_value(x_str.product.imagegenerationparameters.generalprocessinginformation, 'producttype') 
lf.proc_fac                = tag_value(x_str.product.imagegenerationparameters.generalprocessinginformation, 'processingfacility')   
lf.software_version        = tag_value(x_str.product.imagegenerationparameters.generalprocessinginformation, 'softwareversion')
lf.proc_time               = tag_value(x_str.product.imagegenerationparameters.generalprocessinginformation, 'processingTime')
lf.range_looks             = float(tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation,'numberofrangelooks'))
lf.azimuth_looks           = float(tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation,'numberofazimuthlooks'))
lf.incidenceAngleNearRange = tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation.INCIDENCEANGLENEARRANGE,'_text')
lf.incidence_near_deg      = lf.incidenceanglenearrange
lf.incidenceanglefarrange  = tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation.incidenceanglefarrange,'_text')
lf.incidence_far_deg       = lf.incidenceangleFarRange
lf.slantRangeNearEdge      = tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation.slantrangenearedge,'_text')
lf.slant_range_near_m      = lf.slantrangenearedge
lf.satelliteheight         = tag_value(x_str.product.imagegenerationparameters.sarprocessinginformation.satelliteheight,'_text')
lf.s_c_altitude_m          = lf.satelliteheight
;
;
;  Image attributes
lf.datatype                = tag_value(x_str.product.imageattributes.rasterattributes,'datatype')
lf.ellipsoidName           = tag_value(x_str.product.imageattributes.geographicinformation.referenceellipsoidparameters,'ellipsoidName')
lf.semimajoraxis           = tag_value(x_str.product.imageattributes.geographicinformation.referenceellipsoidparameters.semimajoraxis,'_text')
lf.semiminoraxis           = tag_value(x_str.product.imageattributes.geographicinformation.referenceellipsoidparameters.semiminoraxis,'_text')
lf.datumshiftparameters    = tag_value(x_str.product.imageattributes.geographicinformation.referenceellipsoidparameters.datumshiftparameters,'_text')
lf.nx0                     = tag_value(x_str.product.imageattributes.rasterattributes,'numberofsamplesperline')  
lf.ny0                     = tag_value(x_str.product.imageattributes.rasterattributes,'numberOfLines')
lf.range_pixel             = tag_value(x_str.product.imageattributes.rasterattributes.sampledPixelSpacing,'_text')
lf.azimuth_pixel           = tag_value(x_str.product.imageattributes.rasterattributes.sampledLineSpacing,'_text')
lf.pixelTimeOrdering       = tag_value(x_str.product.imageattributes.rasterattributes,'pixelTimeOrdering')
lf.lineTimeOrdering        = tag_value(x_str.product.imageattributes.rasterattributes,'lineTimeOrdering')
imagefiles                 = x_str.product.imageattributes.fullresolutionimagedata

if 'LIST' eq typename(imagefiles) then begin
  nc                         = n_elements(imagefiles)
  for i =0, nc-1 do lf.imagefiles[i] = tag_value(imagefiles[i].tostruct(/recursive), '_text')
endif else begin
    lf.imagefiles[0] = tag_value(imagefiles, '_text')
endelse

;
; Source attributes
;
lf.inputdatasetid          = tag_value(x_str.product.sourceattributes,'inputdatasetid')
lf.beam_mode               = tag_value(x_str.product.sourceattributes,'beammodemnemonic')
lf.datatake_id             = tag_value(x_str.product.sourceattributes,'imageid')
lf.asc_des                 = tag_value(x_str.product.sourceattributes.orbitandattitude.orbitinformation,'passdirection')  
lf.antennaPointing         = tag_value(x_str.product.sourceattributes.radarparameters,'antennaPointing') 
lf.beam_mode               = tag_value(x_str.product.sourceattributes.radarparameters,'polarizations')
lf.polarization            = tag_value(x_str.product.sourceattributes.radarparameters,'polarizations')
lf.sar_sensor              = tag_value(x_str.product.sourceattributes, 'satellite') 
lf.radar_frequency         = float( tag_value(x_str.product.sourceattributes.radarparameters.radarcenterfrequency,'_text'))

;
; ignore groundtoslantrangecoefficients

; State vector  Only first value
;
y  = x_str.product.sourceattributes.ORBITANDATTITUDE.ORBITINFORMATION.statevector  
y1 = y[0].ToStruct(/recursive)
lf.x_position      = double( tag_value(y1.xposition, '_text') )
lf.y_position      = double( tag_value(y1.yposition, '_text') )
lf.z_position      = double( tag_value(y1.zposition, '_text') )
lf.x_velocity      = double( tag_value(y1.xvelocity, '_text') )
lf.y_velocity      = double( tag_value(y1.yvelocity, '_text') )
lf.z_velocity      = double( tag_value(y1.zvelocity, '_text') )

;
; Time
lf.sensing_start   = strcompress(tag_value(x_str.product.sourceattributes,'RAWDATASTARTTIME'))
lf.center_gmt      = lf.sensing_start
lf.data_year       = fix( strmid(lf.center_gmt,  0,  4) )
lf.month           = monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
lf.day             = fix( strmid(lf.center_gmt,  8,  2) )
lf.hour            = fix( strmid(lf.center_gmt, 11,  2) )
lf.minute          = fix( strmid(lf.center_gmt, 14,  2) )
data_year          = fix( strmid(lf.center_gmt,  0,  4) )
lf.month           = monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
lf.day             = fix( strmid(lf.center_gmt,  8,  2) )
lf.hour            = fix( strmid(lf.center_gmt, 11,  2) )
lf.minute          = fix( strmid(lf.center_gmt, 14,  2) )
lf.second          = fix( strmid(lf.center_gmt, 17,  8) )

; Time stamp
y= x_str.product.sourceattributes.ORBITANDATTITUDE.ORBITINFORMATION.statevector 
nstate      = n_elements(y)
timejs      = dblarr(nstate)
for i=0,nstate-1 do begin
  y1        = y[i].ToStruct(/recursive)
  ss        = tag_value(y1, 'timestamp')
  year      = strmid(ss,0,4)
  mon       = monthstr( fix( strmid(ss,  5,  2) ) )
  day       = strmid(ss,  8,  2)  
  ss        = year + ' ' + mon + ' ' + day + ' ' + strmid(ss,11,11)
  timejs[i] = dt_tm_tojs(ss)
endfor
w= where(timejs ne 0, cc )

if cc gt 0 then begin
  lf.jsbegin = min( timejs[w]) 
  lf.jsend   = max( timejs[w]) 
  jscen      = (lf.jsbegin + lf.jsend)/ 2.0 
endif

lf.center_gmt = dt_tm_fromjs(jscen,for='Y$-0n$-d$Th$:m$:s$')
lf.data_year  = fix( strmid(lf.center_gmt,  0,  4) )
lf.month      = monthstr( fix( strmid(lf.center_gmt,  5,  2) ) )
lf.day        = fix( strmid(lf.center_gmt,  8,  2) )
lf.hour       = fix( strmid(lf.center_gmt, 11,  2) )
lf.minute     = fix( strmid(lf.center_gmt, 14,  2) )
lf.second     = fix( strmid(lf.center_gmt, 17,  2) )
;;
; ----------------------------------------------------
;                   Image tie points
; ---------------------------------------------------
       
g         = x_str.product.imageattributes.geographicinformation.geolocationgrid.imagetiepoint       
ng        = n_elements(g)      
jj        = lonarr(ng)
ii        = lonarr(ng)
latitude  = dblarr(ng)  
longitude = dblarr(ng)   
height    = dblarr(ng)   
for i = 0, ng -1 do begin
  gs           = g[i].ToStruct(/recursive)
  jj[i]        = long( float(tag_value(gs.imagecoordinate, 'line')  ) )
  ii[i]        = long( float(tag_value(gs.imagecoordinate, 'pixel') ) )
  latitude[i]  = double( tag_value(gs.geodeticcoordinate.latitude, '_text')  ) ; degrees
  longitude[i] = double( tag_value(gs.geodeticcoordinate.longitude, '_text') ) ; degrees
  height[i]    = double( tag_value(gs.geodeticcoordinate.height, '_text')    ) ; degrees  
endfor
       
;
;
; Pixel time direction determines whether the first pixel is near range or far range. 
; Hence the calculation must be inverted
;

incidence_near_deg = double( lf.incidenceAngleNearRange ) 
incidence_far_deg  = double( lf.incidenceAngleFarRange  ) 
slant_range_near_m = double( lf.slantRangeNearEdge      ) 
s_c_altitude_m     = double( lf.satelliteHeight         ) 

if strupcase(lf.pixelTimeOrdering) eq 'INCREASING' then begin
  ;
  ; near range is first pixel
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0L, n-1 do begin
    c=            c0 + ii[k] *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor 
endif else begin
  ;
  ; far range is first pixel
  ; 
  ; First compute the slant range at far incidence
  ;
  gamma0=    slant_range_near_m
  gamma2=    gamma0^2
  alpha=     double(re)
  alpha2=    alpha^2
  beta=      re + s_c_altitude_m
  beta2=     beta * beta
  c0=        acos( (beta2 + alpha2 - gamma2) / 2.0 / alpha / beta)
  n=         n_elements(ii) 
  incidence= dblarr(n)
  scaling=   lf.range_pixel / re
  for k=0, n-1 do begin
    c=            c0 + (lf.nx0 -ii[k]) *  scaling
    gamma2=       alpha2 + beta2 - 2.0 * alpha * beta * cos(c)
    gamma=        sqrt(gamma2)
    incidence[k]= !dpi - acos( (gamma2 + alpha2 - beta2) / 2.0 / alpha /gamma )
    incidence[k]= incidence[k] * 180. / !pi
  endfor
endelse
 
;---------------------------------------
; Read noise equivalent sigma_ne array
;---------------------------------------

noise                      = x_str.product.sourceattributes.radarparameters.referencenoiselevel
nn                         = n_elements( noise )
polestr                    = strarr(5)
for i= 0, 4 do polestr[i]  = getwrd( lf.polarization, i)
w                          = where(polestr ne '', count)
if count eq 1 then begin
  sigma_ne= fltarr( lf.nx0 )
endif else begin
  sigma_ne= fltarr( count, lf.nx0 )
endelse
for i = 0, count-1 do begin    ; Loop through polarization
  for j = 0, nn-1 do begin     ; Loop through noise1 objects
    noise1                    = noise[j].ToStruct(/recursive)
    incidenceanglecorrection  = tag_value( noise1, '_incidenceanglecorrection' )
    noiselevelvalues_units    = tag_value( noise1.numberofnoiselevelvalues, '_UNITS')
    correction                = strcompress(strupcase( incidenceanglecorrection), /remove_all)
    stepsize                  = long( tag_value( noise1, 'stepsize') )
    pole                      = tag_value( noise1, '_pole')
    numberofnoiselevelvalues  = tag_value( noise1, 'numberofnoiselevelvalues' )
    pixelfirstnoisevalue      = tag_value( noise1, 'pixelfirstnoisevalue' )
    if count eq 1 then pole = polestr[i]
    if printflag then print, pole, ' ', polestr[i], ' ',correction
    ;
    ; files with multiple noise levels per polarization
    ;
    if pole eq polestr[i] and correction eq 'SIGMANOUGHT' then begin
      noiselevelvalues                = tag_value( noise1.noiselevelvalues, '_TEXT' )
      if count eq 1 then begin 
        for k=0, numberofnoiselevelvalues-1 do begin
           el           =  pixelfirstnoisevalue + k * stepsize 
           sigma_ne[el] =  float( getwrd( noiselevelvalues, k) )
        endfor
      endif else begin
        for k=0, numberofnoiselevelvalues-1 do begin
           el           =  pixelfirstnoisevalue + k * stepsize 
           sigma_ne[i, el] =  float( getwrd( noiselevelvalues, k) )
        endfor    
      endelse       ; Only one polarization
    endif           ; Check if polariation and corection match
    ;
    ; files with single noise levels all polarization
    ;
    if pole eq '' and correction eq 'SIGMANOUGHT' then begin
      noiselevelvalues                = tag_value( noise1.noiselevelvalues, '_TEXT' )
      if count eq 1 then begin 
        for k=0, numberofnoiselevelvalues-1 do begin
           el           =  pixelfirstnoisevalue + k * stepsize 
           sigma_ne[el] =  float( getwrd( noiselevelvalues, k) )
        endfor
      endif else begin
        for k=0, numberofnoiselevelvalues-1 do begin
           el           =  pixelfirstnoisevalue + k * stepsize 
           sigma_ne[i, el] =  float( getwrd( noiselevelvalues, k) )
        endfor    
      endelse       ; Only one polarization
    endif           ; Check if polariation and corection match 
  endfor            ; End loop noise1 objects
endfor              ; End loop through polarization
if printflag then hlp, sigma_ne
;
; Interpolate and fill sigma_ne array
;

case count of
1    : begin ; only one dimension
         s = sigma_ne
         w = where(s ne 0, c )
         if c gt 2 then begin     ; If there are more than 2 filled in values
           for j = 0, c-2 do begin
             j1 = w[j]
             j2 = w[j+1]
             slope = (s[j2]- s[j1]) / float(j2-j1)
             bias  = s[j2] - slope * j2
             for k = j1, j2 do begin
               s[k] = slope * float(k) + bias
             endfor
           endfor
           ; 
           ; Deal with edge cases
           ; 
           if w[0] eq 0 then begin
              j1 = w[0]
              j2 = w[0]+1
              slope = (s[j2]- s[j1]) / float(j2-j1)
              bias  = s[j2] - slope * j2
              for k = 0, w[0]-1 do s[k] = slope * float(k) + bias
           endif
           if w[c-1] ne lf.nx0-1 then begin
              j1 = w[c-1]-1
              j2 = w[c-1]
              slope = (s[j2]- s[j1]) / float(j2-j1)
              bias  = s[j2] - slope * j2
              for k = w[c-1], lf.nx0-1 do s[k] = slope * float(k) + bias
           endif
           sigma_ne = s
         endif
       end
else : begin
       for i=0, count-1 do begin
         s = reform(sigma_ne[i,*])
         w = where(s ne 0, c )
         if c gt 2 then begin     ; If there are more than 2 filled in values
           for j = 0, c-2 do begin
             j1 = w[j]
             j2 = w[j+1]
             slope = (s[j2]- s[j1]) / float(j2-j1)
             bias  = s[j2] - slope * j2
             for k = j1, j2 do begin
               s[k] = slope * float(k) + bias
             endfor
           endfor
           ; 
           ; Deal with edge cases
           ; 
           if w[0] eq 0 then begin
              j1 = w[0]
              j2 = w[0]+1
              slope = (s[j2]- s[j1]) / float(j2-j1)
              bias  = s[j2] - slope * j2
              for k = 0, w[0]-1 do s[k] = slope * float(k) + bias
           endif
           if w[c-1] ne lf.nx0-1 then begin
              j1 = w[c-1]-1
              j2 = w[c-1]
              slope = (s[j2]- s[j1]) / float(j2-j1)
              bias  = s[j2] - slope * j2
              for k = w[c-1], lf.nx0-1 do s[k] = slope * float(k) + bias
           endif
           sigma_ne[i,*] = s
         endif
       endfor
       end
endcase
;
; Convert to sigma
; 
w = where( sigma_ne ne 0, /null)
sigma_ne[w] = 10^( sigma_ne[w]/10.00)


if printflag then begin
  print, 'product                =  ' + strcompress(lf.product                )     
  print, 'documentidentifier     =  ' + strcompress(lf.documentidentifier     )
  print, 'sar_sensor             =  ' + strcompress(lf.sar_sensor             )
  print, 'polariztion            =  ' + strcompress(lf.polarization           )
  print, 'radar_frequency        =  ' + strcompress(lf.radar_frequency        )
  print, 'asc_des                =  ' + strcompress(lf.asc_des                )
  print, 'center_gmt             =  ' + strcompress(lf.center_gmt             )
  print, 'data_year              =  ' + strcompress(string(lf.data_year)      )
  print, 'month                  =  ' + strcompress(lf.month                  )
  print, 'day                    =  ' + strcompress(string(lf.day)            )
  print, 'hour                   =  ' + strcompress(string(lf.hour)           )
  print, 'minute                 =  ' + strcompress(string(lf.minute)         )
  print, 'second                 =  ' + strcompress(string(lf.second)         )
  print, 'antennapointing        =  ' + strcompress(string(lf.antennapointing))
  print, 'range_pixel            =  ' + strcompress(string(lf.range_pixel)    )
  print, 'azimuth_pixel          =  ' + strcompress(string(lf.azimuth_pixel)  )
  print, 'numberoflines          =  ' + strcompress(string(lf.nx0)            )
  print, 'numberofsamplesperline =  ' + strcompress(string(lf.ny0)            )
  print, 'pixeltimeordering      =  ' + strcompress(lf.pixeltimeordering      ) 
  print, 'linetimeordering       =  ' + strcompress(lf.linetimeordering       )
  print,  '         i        j   longitude   latitude'
  for i=0L, n-1, n/20 do begin
    print , string(double(ii[i]),        format='(I10.10)') + ' ' + $
            string(double(jj[i]),        format='(I10.10)') + ' ' + $
            string(double(longitude[i]), format='(F10.5)' ) + ' ' + $
            string(double(latitude[i]),  format='(F10.5)' )
  endfor
  print, 'i, incidence'
  n=n_elements(incidence)
  for i=0L, n-1, n/10 do begin
    print, string(double(ii[i]),          format='(I10.10)') + ' ' + $
           string(double(incidence[i]),   format='(F10.5)' )
  endfor
  close, 1
endif

if n_elements(outputfile) gt 0 then begin
  close,  1
  openw,  1,  outputfile
  printf, 1, 'product                =  ' + lf.product       
  printf, 1, 'documentidentifier     =  ' + lf.documentidentifier
  printf, 1, 'sar_sensor             =  ' + lf.sar_sensor
  printf, 1, 'polariztion            =  ' + lf.polarization
  printf, 1, 'radar_frequency        =  ' + lf.radar_frequency
  printf, 1, 'asc_des                =  ' + lf.asc_des
  printf, 1, 'center_gmt             =  ' + lf.center_gmt
  printf, 1, 'data_year              =  ' + strcompress(string(lf.data_year))
  printf, 1, 'month                  =  ' + lf.month
  printf, 1, 'day                    =  ' + strcompress(string(lf.day))
  printf, 1, 'hour                   =  ' + strcompress(string(lf.hour))
  printf, 1, 'minute                 =  ' + strcompress(string(lf.minute))
  printf, 1, 'second                 =  ' + strcompress(string(lf.second))
  printf, 1, 'antennapointing        =  ' + strcompress(string(lf.antennapointing))
  printf, 1, 'range_pixel            =  ' + strcompress(string(lf.range_pixel))
  printf, 1, 'azimuth_pixel          =  ' + strcompress(string(lf.azimuth_pixel))
  printf, 1, 'numberoflines          =  ' + strcompress(string(lf.nx0))
  printf, 1, 'numberofsamplesperline =  ' + strcompress(string(lf.ny0))
  printf, 1, 'pixeltimeordering      =  ' + lf.pixeltimeordering
  printf, 1, 'linetimeordering       =  ' + lf.linetimeordering
  n=n_elements(ii)
  printf, 1,  'i  j longitude, latitude'
  for i=0L, n-1 do begin
    print, string(double(ii[i]),        format='(I10.10)') + ' ' + $
           string(double(jj[i]),        format='(I10.10)') + ' ' + $
           string(double(longitude[i]), format='(E24.16E3)') + ' ' + $
           string(double(latitude[i]),  format='(E24.16E3)')
  endfor
  print, 'i, incidence'
  n=n_elements(incidence)
  for i=0L, n-1 do begin
    print, string(double(ii[i]),           format='(E24.16E3)') + ' ' + $
         string(double(incidence[i]),    format='(E24.16E3)') 
  endfor
  if printflag then print, 'Stored file='+outputfile
endif
;
; fill in clean up
;
lf.pixel_spacing    = lf.range_pixel
lf.line_spacing     = lf.azimuth_pixel
lf.range_spacing    = lf.range_pixel
lf.azimuth_spacing  = lf.azimuth_pixel
lf.PROC_CENT        = lf.proc_fac
lf.software_ver     = lf.software_version
lf.sensing_stop     = dt_tm_fromjs(lf.jsend, format='Y$-0n$-0d$Th$:m$:s$f$Z', decimal=6)
end
