;----------------------------------------------------------
;+
; NAME: RSAT2_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out RSAT2/RCM data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;    rsat2_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   
; MODIFICATION
;  2007 Aug 22 Frank Monaldo 
;  2008 Sep 13 Chris Jackson  - 
;              Made changes in the Image Identification 
;              and Image Orientation block to ensure that
;              certain lines are always written out
;              Made sure PALSAR corners are written
;  2009 May 01 Frank Monaldo 
;              Corrected problem w/ print out of incident
;              angle coefficients
;  2009 Jul 27 Frank Monaldo. Added code to keep the program
;              even if the lf.scansar and lf.s_c_altitude do not exist. This
;              was required because these fields are not filled when reading
;              RSAT2 data
;  2009 Oct 14 Radically re-written for new software version 
;  2009 Nov 10 Frank Monaldo correct bug that wrote a line 
;               transformation rather than an element transformation
;  2009 Dec 02 Frank Monaldo
;              Changed: longitude (0to-360) transform coef for element
;              to: longitude (0to360) transform coef for element 
;  2010 Jan 05 Frank Monaldo. Changed to accomodate ans_compute_fit and ans_poly_cal
;  2010 Jan 06 Frank Monaldo. Corrected printout of longitude fit information
;  2010 Jan 07 Frank Monaldo. Fixed s mising "^" character in longitude line output.
;                             and removed an unnecessary space.
;  2010 Jan 15 Frank Monaldo. Computed heading increasing/decreasing line valuye
;  2010 Jan 11 Frank Monaldo. Change to use ans_write_ancil_sio
;  2010 Jan 22 Frank Monaldo. Re-compute illumination and direction and relative heading
;                             in accordance with Wackermans description
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2010 Dec 30 Frank Monaldo. Pass on quiet flag
;  2020 Fen 18 Frank Monaldo. Properly assign platform code for RCM data.
; Copyright (C) 2007-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro rsat2_write_ancil_sio, filename, lf, scaling, $
                           quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)   or (n_params(0) lt 1 ) then begin
  print,' NAME: RSAT2_WRITE_ANCIL_SIO'
  print,' PURPOSE:'
  print,'   To write out RSAT2/RCM data into ancillary data ascii file. These
  print,'   data fit the SIO format
  print,' CALLING SEQUENCE:'
  print,'    rsat2_write_ancil_sio, filename, lf,  scaling'
  print,' INPUT:'
  print,'   filename=     output file with ancillary data
  print,'   lf=           leader structure'
  print,'   scaling=      scaling for data in binary file of NRCS, default=1
  print,' OUTPUT:
  print,'    ASCII file in SIO format
  print,' KEYWORDS:
  print,'    /quiet=      Do not print intermediate results'
  print,'    /help=       List help.'
  return
endif

;
; Fill in missing lf structure values.

orbit_number=   lf.datatake_id
lf.datatake_id= lf.inputDatasetId
platform_code=  'R2'
if tag_value(lf, 'sar_sensor') ne '' then platform_code=lf.sar_sensor
if platform_code eq 'RADARSAT-2'     then platform_code='R2
if not tag_test(lf,'scansar') then begin
  scansar=0
  if lf.range_pixel gt 30 or lf.azimuth_pixel gt 25 then scansar=1
endif
valuetype=            'intensity'
lf.asc_des=           strupcase( strmid(lf.asc_des,0,1) )
scaling=              scaling
frequency_band=       'C'
sensor_velocity=      sqrt(lf.x_velocity^2 + lf.y_velocity^2 + lf.z_velocity^2)
sensor_velocity=      string(sensor_velocity, format='(F15.6)' ) 
integration_time=     '-9999.9999'
nom_az_res=           '-9999.9999'
nom_rg_res=           '-9999.9999'
proc_gain=            '-9999.9999'
rcvr_gain=            '-9999.9999'
range_type=           'GROUND RANGE'
range_flag=           1
range_coef=            getwrd(lf.groundToSlantRangeCoefficients,1)
if range_coef eq 1 then begin
  range_type='NONGEOCODED'
  range_flag=        0
endif
flip=                  0
;
; Compute heading
;

lo1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la1= ans_poly_cal(lf.nx1/2, lf.ny1/2,   lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
lo2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la2= ans_poly_cal(lf.nx1/2, lf.ny1/2+1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
ll2rb, lo1, la1, lo2, la2, r, b
if b gt 180 then b= b-360.0
heading= b
heading_rel= heading

illumination_dir=0
near_range_edge='L'
early_azimuth_edge='B'
heading_rel=     90

if lf.lineTimeOrdering  eq 'DECREASING' then heading_rel= -90
if lf.pixelTimeOrdering eq 'DECREASING' then heading_rel= 180

s_c_altitude= lf.satelliteheight

center_incid= ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
center_lon=   ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lon_coef,   lf.lon_xexp,   lf.lon_yexp  )
center_lat=   ans_poly_cal(lf.nx1/2, lf.ny1/2, lf.lat_coef,   lf.lat_xexp,   lf.lat_yexp  )
slant_range_center= -9999.99
slant_range_near=   lf.slantRangeNearEdge   
range_swath=   lf.range_pixel     * lf.nx1
azimuth_swath= lf.azimuth_pixel   * lf.ny1

;
; Computing slant_range_center.
;
center_incid_rad=    double(center_incid) * !dpi /180.
major=               double(lf.semimajoraxis)
minor=               double(lf.semiminoraxis)
cos_phi=             cos(center_lat* !dpi/180.)
sin_phi=             sin(center_lat* !dpi/180.)
re=                  (major^2*cos_phi)^2 + (minor^2*sin_phi)^2
re=                  re / ( (minor * cos_phi)^2 + (minor*sin_phi)^2 )
re=                  sqrt( re )
ra=                  s_c_altitude
C=                   !pi - center_incid_rad
sin_c=               sin(C)
rea=                 re+ra
B=                   asin( sin_c * re /rea )
A=                   !pi - B - C
sin_a=               sin(A)
slant_range_center=  sin_a / sin_c * rea
slant_range_center=  string(slant_range_center,format='(E21.15)') 


;
; Add to lf structure
;
if not tag_test(lf, 'scansar') then begin
  temp_lf= create_struct( lf, 'scansar', scansar)
  lf=      0
  lf=      temp_lf
endif

lf.pixel_spacing=   lf.range_pixel
lf.line_spacing=    lf.azimuth_pixel
deg_incid=          lf.deg
deg_rlook=          lf.deg
sar_imaging_mode=   lf.producttype

if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf, 'platform_code',      platform_code,      $
                              'orbit_number',       orbit_number,       $
                              'valuetype',          valuetype,          $
                              'scaling',            scaling,            $
                              'frequency_band',     frequency_band,     $
                              'sensor_velocity',    sensor_velocity,    $
                              'integration_time',   integration_time,   $
                              'nom_az_res',         nom_az_res,         $
                              'nom_rg_res',         nom_rg_res,         $
                              'proc_gain',          proc_gain,          $
                              'rcvr_gain',          rcvr_gain,          $
                              'range_type',         range_type,         $
                              'range_flag',         range_flag,         $
                              'heading',            heading,            $
                              'heading_rel',        heading_rel,        $
                              'flip',               flip,               $
                              'ellipsoidscale',     '1',                $
                              'illumination_dir',   illumination_dir,   $
                              'near_range_edge',    near_range_edge,    $
                              'early_azimuth_edge', early_azimuth_edge, $'
                              's_c_altitude',       s_c_altitude,       $
                              'center_incid',       center_incid,       $
                              'slant_range_center', slant_range_center, $
                              'slant_range_near',   slant_range_near,   $
                              'range_swath',        range_swath,        $
                              'azimuth_swath',      azimuth_swath,      $
                              'deg_incid',          deg_incid,          $
                              'deg_rlook',          deg_rlook,          $
                              'sar_imaging_mode',   sar_imaging_mode    ) 
  lf=      0
  lf=      temp_lf
endif else begin  
  lf.platform_code=      platform_code     
  lf.orbit_number=       orbit_number      
  lf.valuetype=          valuetype         
  lf.scaling=            scaling           
  lf.frequency_band=     frequency_band    
  lf.sensor_velocity=    sensor_velocity   
  lf.integration_time=   integration_time  
  lf.nom_az_res=         nom_az_res        
  lf.nom_rg_res=         nom_rg_res        
  lf.proc_gain=          proc_gain         
  lf.rcvr_gain=          rcvr_gain         
  lf.range_type=         range_type        
  lf.range_flag=         range_flag        
  lf.heading=            heading           
  lf.heading_rel=        heading_rel       
  lf.flip=               flip              
  lf.ellipsoidscale=     '1'               
  lf.illumination_dir=   illumination_dir  
  lf.near_range_edge=    near_range_edge   
  lf.early_azimuth_edge= early_azimuth_edge
  lf.s_c_altitude=       s_c_altitude      
  lf.center_incid=       center_incid      
  lf.slant_range_center= slant_range_center
  lf.slant_range_near=   slant_range_near  
  lf.range_swath=        range_swath       
  lf.azimuth_swath=      azimuth_swath     
  lf.deg_incid=          deg_incid         
  lf.deg_rlook=          deg_rlook         
  lf.sar_imaging_mode=   sar_imaging_mode
endelse


;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 
end
