;-----------------------------------------------------
;+
; NAME: SEN_AZIMUTH_NOISE_IMAGE
; PURPOSE:
;  Read noise XML file of Sentinel-1 SAR data and produce the full
;  2D image of the azimuth noise
; CALLING SEQUENCE
;   sen_azimuth_noise_image, file, img
; NOTE: Requires IDL 8.7 for XML_PARSE
; INPUT
;   file       :  XML file with noise information 
; OUTPUT
;   img        :  Azimuth noise image
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /display    :  Display layout of points
;  nx          :  The size of the array in the range dimension
;  ny          :  The size of the array in the azimuth dimension
;                 By the default, these values are set to the maximum 
;                 index values given in the XML file.  We assume that
;                 nx, ny  >= corresponding values in XML file    
; sid          : A 2D array of swath identification values (0 = no data, # = Swath #)                
;  /checkvalues:  Print point-by-point comparisons   
;  pngimage    :  Optional PNG image output
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2018 Dec 12 Frank Monaldo. 
;  2019 Jan 02 Chris Jackson.  Add option for retrun of swath identification array (SID)
; Copyright (C) 2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_azimuth_noise_image, file,     img      ,  $
                             help        = help        ,  $
                             quiet       = quiet       ,  $
                             display     = display     ,  $
                             nx          = nx          ,  $
                             ny          = ny          ,  $
                             sid         = sid         ,  $
                             pngimage    = pngimage    ,  $
                             checkvalues = checkvalues ,  $
                             error       = error

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_AZIMUTH_NOISE_IMAGE
  print, ' PURPOSE:
  print, '  Read noise XML file of Sentinel-1 SAR data and produce the full
  print, '  2D image of the azimuth noise
  print, ' CALLING SEQUENCE
  print, '   sen_azimuth_noise_image, file, img
  print, ' NOTE: Requires IDL 8.7 for XML_PARSE
  print, ' INPUT
  print, '   file       :  XML file with noise information 
  print, ' OUTPUT
  print, '   img        :  Azimuth noise image
  print, ' KEYWORDS:
  print, '  /help       :  Print out help
  print, '  /quiet      :  Suppress intermediate print output
  print, '  /display    :  Display layout of points
  print, '  nx          :  The size of the array in the range dimension
  print, '  ny          :  The size of the array in the azimuth dimension
  print, '                 By the default, these values are set to the maximum 
  print, '                 index values given in the XML file.  We assume that
  print, '                 nx, ny  >= corresponding values in XML file  
    print, 'sid          : A 2D array of swath identification values (0 = no data, # = Swath #)        
  print, '  /checkvalues:  Print point-by-point comparisons   
  print, '  pngimage    :  Optional PNG image output
  print, '  error=      :  error <0 if cannot read data
  error = -1
  return
endif


error= 1
deg =  3

if printflag then print, 'Reading file : ' + file 

sen_read_noise_xml_fast, file, x, /azimuth, /quiet   ,  $
                               error    = error     
if error lt 0 then begin
  print, 'Cannot read file: ' + file + ' for azimuth information'
  print, 'Exiting sen_azimuth_noise_image'
  error = -2
  return
endif


nblock = x.azimuth.numberofsubswaths ; the number of blocks
firstazimuthline = lonarr(nblock)
firstrangesample = lonarr(nblock)
lastazimuthline  = lonarr(nblock)
lastrangesample  = lonarr(nblock)
swath            = strarr(nblock)


for i = 0, nblock -1 do begin
  z                   = tag_value(x.azimuth, 'SUBSWATH'+ string(i, format='(I3.3)') )
  firstazimuthline[i] = z.firstazimuthline
  firstrangesample[i] = z.firstrangesample
  lastazimuthline[i]  = z.lastazimuthline 
  lastrangesample[i]  = z.lastrangesample 
  swath[i]            = z.swath           
endfor
;
; Find size
;
nxfile  = max( lastrangesample + 1)
nyfile  = max( lastazimuthline + 1 )
if not keyword_set(nx) then nx = nxfile
if not keyword_set(ny) then ny = nyfile
if nx lt nxfile then nx = nxfile
if ny lt nyfile then ny = nyfile
img = fltarr(nx,ny)
sid = intarr(nx,ny)
;
; Find unique swath names
;
idx   = sort(swath)
z1    = swath[idx]
z2    = z1(uniq(z1))
ns    = n_elements(z2)
names = strarr(ns)

for s =0, ns-1 do names[s] =z2[s]

for s = 0, ns-1 do begin
   w         = where( swath eq names[s], count )
   sw        = swath[w]
   firstline = firstazimuthline[w]
   firstrange= firstrangesample[w]
   lastline  = lastazimuthline[w]
   lastrange = lastrangesample[w]
   if keyword_set(display) then begin
     for j=0,n_elements(firstrange)-1 do begin
       if keyword_set(display) then begin
         print, string(firstline[j],firstrange[j],lastline[j],lastrange[j], format='(4I7)')    
       endif
     endfor 
   endif
   if keyword_set(display) then begin
     for j=0, n_elements(firstrange)-1 do begin
       if s eq 0 and j eq 0 then begin
         plot, [firstrange[j], lastrange[j] , lastrange[j] , firstrange[j] , firstrange[j]], $
               [firstline[j] , firstline[j] , lastline[j]  , lastline[j]   , firstline[j] ], $
               /xstyle, xrange = [0,nx] , $
               /ystyle, yrange = [0,ny]
       endif else begin
         oplot, [firstrange[j], lastrange[j] , lastrange[j] , firstrange[j] , firstrange[j]], $
                [firstline[j], firstline[j] , lastline[j]  , lastline[j]   , firstline[j] ]
       endelse
       rm   = ( firstrange[j] + lastrange[j] ) / 2.0
       am   = ( firstline[j]  + lastline[j]  ) / 2.0
       xyouts, rm, am, sw, alignment=0.5
     endfor
   endif
endfor  ; End looping through subswath or block names
;if keyword_set(display) then stop

firstplot=1
for i = 0, nblock -1 do begin
  z                 = tag_value(x.azimuth, 'subswath'+ string(i, format='(i3.3)') )
  line              = z.line
  noiseazimuthlut   = z.noiseazimuthlut
  n                 = z.lastazimuthline - z.firstazimuthline + 1 
  full_line         = findgen(n) + z.firstazimuthline
  full_noise        = fltarr(n)
  for j=0, n_elements(line)-1 do begin
    full_noise( line[j] - z.firstazimuthline ) = noiseazimuthlut[j]
  endfor
  if n_elements(line) ge 2 then begin ; do fit
    for j = 1, n_elements(line)-1 do begin
     ip    = line[j-1]
     in    = line[j]
     if (in-ip) gt 1 then begin
       ;
       ; linear file
       ;
       y2     = noiseazimuthlut[j  ]
       y1     = noiseazimuthlut[j-1]
       x2     = in
       x1     = ip
       slope  = (y2-y1) /(x2-x1)
       bias   =  y2 - slope *  x2
       for k = ip+1, in-1 do full_noise[k-line[0]] = slope * float(k) + bias  
     endif
    endfor
    if z.lastazimuthline gt in then begin
      for k=in, z.lastazimuthline do begin
        full_noise[k-z.firstazimuthline] = slope * float(k) + bias 
      endfor
    endif
  endif else begin
    full_noise = fltarr( n_elements(full_noise) ) + max( noiseazimuthlut)
  endelse
  if keyword_set(display) then begin
    plot, line, noiseazimuthlut, psym=4, /ystyle, yrange=[0,1.5], /xstyle, $
          xrange= [z.firstrangesample, z.lastrangesample]
    oplot,full_line, full_noise
    print, 'Swath : ' + z.swath + ',   Block : ' + string( i, format='(I3.3)' )
  endif
  ;
  ; fill in img 
  ;
  for rg = z.firstrangesample, z.lastrangesample do begin
      img[rg, z.firstazimuthline: z.lastazimuthline] = full_noise
      sid[rg, z.firstazimuthline: z.lastazimuthline] = strmid(z.SWATH,strlen(z.swath)-1,1)+0
  endfor
  ;if i eq 0 then begin
  ;  plot, full_line, full_noise, /xstyle, xrange=[0,12000]
  ;endif else begin
  ;  oplot, full_line, full_noise
  ;endelse
next_block:;
endfor              ; End block loop 
if keyword_set(pngimage) then begin
   if printflag then print, 'Saving : ' + pngimage
   ;im  = image( img , /buffer, title=file)
   ;im.save, pngimage
endif

if keyword_set(checkvalues) then begin
  for i = 0, nblock -1 do begin
    z                   = tag_value(x.azimuth, 'SUBSWATH'+ string(i, format='(I3.3)') )
    firstazimuthline = z.firstazimuthline
    firstrangesample = z.firstrangesample
    lastazimuthline  = z.lastazimuthline 
    lastrangesample  = z.lastrangesample 
    nn               = n_elements( z.noiseazimuthlut )
    outstr           =          string( firstrangesample     , format = '(I7)'   ) + ' '   
    outstr           = outstr + string( firstazimuthline     , format = '(I7)'   ) + ' '   
    outstr           = outstr + string( z.noiseazimuthlut[0] , format = '(F8.5)' ) + ' '  
    outstr           = outstr + string( img[firstrangesample,firstazimuthline ] , format = '(F8.5)' ) + ' '
    print, outstr
    outstr           =          string( lastrangesample     , format = '(I7)'   ) + ' '   
    outstr           = outstr + string( lastazimuthline     , format = '(I7)'   ) + ' '   
    outstr           = outstr + string( z.noiseazimuthlut[nn-1] , format = '(F8.5)' ) + ' '  
    outstr           = outstr + string( img[lastrangesample,lastazimuthline ] , format = '(F8.5)' ) + ' '
    print, outstr
  endfor
endif

end
