;-----------------------------------------------------
;+
; NAME: SEN_FILLIN_LF.PRO
; PURPOSE:
;    Read XML file and defaults file to fill in the lf structure with ancillary
;    information.   Note that the lon, lat, rlook, and incident coefs
;    are computed in this progam.  The lngswitch is set here as well.
; CALLING SEQUENCE
;    sen_fillin_lf, xfile, deffile, root_file, jsstart, lf
; INPUT
;   xfil e     : Sentinel-1 XML file
;   deffile    : Defaults file
;   root_file  : Root directory name
;   jsstart    : Processing start time in Julian seconds
; OUTPUT
;   lf:        : lf structure
; OPTIONS
;   /help      : Get help
;   /quiet     : Suppress intermediat results
;   /display   : Display fits for diagnostics purposes.
;   error      : < 0 bad
; MODIFICATION HISTORY:
;  2013 Oct 08 Frank Monaldo Creation 
;  2014 may 15 Frank Monaldo. Suppress printout if printflag not set
;  2014 Dec 12 Chris Jackson  Fix two minor errors in variables in the section on longitude basis change
;  2015 Mar 09 Frank Monaldo  Reduce the number of fit points printed. 
;  2015 Nov 12 Frank Monaldo. Add ability to adjust NRCS correction for Sentinel 1
;                             based on satellite and polarization
;  2016 Jun 13 Frank Monaldo. Add sen_cpol_remove_noise and sen_xpol_remove_noise
;  2020 Sep 25 Chris Jackson. Change sen_cpol_remove_noise/sen_xpol_remove_noise attributes to output as generic cpol_remove_noise/xpol_remove_noise
; Copyright (C) 2010-2016, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  sen_fillin_lf, xfile, deffile, root_file, jsstart, lf,  $ 
                    help    =     help   , $   
                    quiet   =     quiet  , $
                    display =     display, $
                    error   =     error

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_FILLIN_LF.PRO
  print, ' PURPOSE:
  print, '    Read XML file and defaults file to fill in the lf structure with ancillary
  print, '    information.   Note that the lon, lat, rlook, and incident coefs
  print, '    are computed in this progam.  The lngswitch is set here as well.
  print, ' CALLING SEQUENCE
  print, '    sen_fillin_lf, xfile, deffile, root_file, jsstart, lf
  print, ' INPUT
  print, '   xfil e     : Sentinel-1 XML file
  print, '   deffile    : Defaults file
  print, '   root_file  : Root directory name
  print, '   jsstart    : Processing start time in Julian seconds
  print, ' OUTPUT
  print, '   lf:        : lf structure
  print, ' OPTIONS
  print, '   /help      : Get help
  print, '   /quiet     : Suppress intermediat results
  print, '   /display   : Display fits for diagnostics purposes.
  print, '   error      : < 0 bad
  error = -1
  return
endif

speed_of_light   = 299792458.00 ; m/s 

;
;  Read XML file
; 
sen_read_xml, xfile, x, quiet = quiet, error = error
if error lt 0 then return
;
;  Read defaults file
;
ans_read_defaults, deffile, df, quiet=quiet, error = error
if error lt 0 then return

ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet

;
; Declaring lf structure
;

x.startTime            = repchr( x.startTime, 'T', ' ' )
x.stopTime             = repchr( x.stopTime,  'T', ' ' )
startTime_js           = dt_tm_tojs( x.startTime, /YMD )
stopTime_js            = dt_tm_tojs( x.stopTime,  /YMD )
center_js              = (startTime_js + stopTime_js) /2.0D
acquisition_date       = dt_tm_fromjs( center_js, format='0n$-0d$-Y$' )
acquisition_time       = dt_tm_fromjs( center_js, format='h$:m$:s$' )
acq_date_time_js       = center_js
acq_date_time          = dt_tm_fromjs( center_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
absorbit               = long(x.absoluteOrbitNumber)
processing_start       = dt_tm_fromjs( jsstart, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
current_js             = dt_tm_tojs( systime(/utc) )
sio_create_date_time   = dt_tm_fromjs( current_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
case x.polarisation of
'HH':  flag_sar_band_polarization = '22'
'VV':  flag_sar_band_polarization = '21'
'VH':  flag_sar_band_polarization = '24'
'HV':  flag_sar_band_polarization = '23'
ELSE:  flag_sar_band_polarization = '00'
endcase
orbit_pass_flag        =  strupcase( strmid( x.pass, 0, 1 ) )
sensor_frequency_hz    =  double(x.radarFrequency) 
sensor_wavelength_m    =  speed_of_light / sensor_frequency_hz
map_descriptor         =  'GROUND RANGE'
range_type             =  1
if x.productType ne 'GRD' then begin
  map_descriptor       =  'SLANT RANGE'
  range_type           =  0
endif
range_swath_width_m    =  float(  x.numberOfSamples ) * float( x.rangePixelSpacing    )
azimuth_swath_width_m  =  float(  x.numberOfLines   ) * float( x.azimuthPixelSpacing  )
nx0                    =  long(   x.numberOfSamples )
ny0                    =  long(   x.numberOfLines   )
nx1                    =  nx0 / long(df.ave_pixel)
ny1                    =  ny0 / long(df.ave_line)
cpol_remove_noise  = 'N'
xpol_remove_noise  = 'N'
if tag_value( df, 'sen_cpol_remove_noise') ne '' then cpol_remove_noise = tag_value( df, 'sen_cpol_remove_noise') 
if tag_value( df, 'sen_xpol_remove_noise') ne '' then xpol_remove_noise = tag_value( df, 'sen_xpol_remove_noise') 


lf= {root_file                    :  root_file                       , $
    answrs_version                :  answrs_version                  , $
    acquisition_date              :  acquisition_date                , $ 
    acquisition_time              :  acquisition_time                , $
    acq_date_time_js              :  acq_date_time_js                , $
    acq_date_time                 :  acq_date_time                   , $
    jsbegin                       :  startTime_js                    , $ 
    jsend                         :  stopTime_js                     , $
    processing_start              :  processing_start                , $
    sio_create_date_time          :  sio_create_date_time            , $
    product_type                  :  x.productType                   , $
    processing_agency             : 'ESA'                            , $
    processing_facility           : 'ESA'                            , $
    sar_processor_version         : 'BLANK'                          , $
    scene_identifier              :  x.missionDataTakeId             , $
    scene_reference_number        :  x.imageNumber                   , $
    orbit_number                  :  long(x.absoluteOrbitNumber)     , $
    platform_name                 :  x.missionId                     , $
    sar_platform_code             :  x.missionId                     , $
    sensor_imaging_mode           :  x.mode                          , $
    calibration_flag              :  1                               , $
    image_pixel_value_units       :  'Intensity'                     , $
    polarization                  :  x.polarisation                  , $
    frequency_band                :  'C'                             , $
    flag_sar_band_polarization    :  flag_sar_band_polarization      , $
    orbit_pass_flag               :  orbit_pass_flag                 , $
    scaling                       :  1.000000                        , $
    sar_beam_code                 :  x.mode                          , $
    sensor_wavelength_m           :  sensor_wavelength_m             , $
    sensor_frequency_hz           :  sensor_frequency_hz             , $
    sensor_vel_rel_ground_mps     :  -9999.9999                      , $
    integration_time_s            :  -9999.9999                      , $
    nominal_azimuth_resolution_m  :  -9999.9999                      , $
    nominal_range_resolution_m    :  -9999.9999                      , $
    receiver_gain_db              :  -9999.9999                      , $
    relative_processor_gain_db    :  -9999.9999                      , $
    map_descriptor                :  map_descriptor                  , $
    pixel_spacing_m               :  float(x.rangePixelSpacing)      , $
    line_spacing_m                :  float(x.azimuthPixelSpacing)    , $
    range_type                    :  range_type                      , $
    heading_north_deg             :  x.platformHeading               , $
    heading_image_deg             :  0.0                             , $
    flip_flag                     :  0                               , $
    illumination_dir_deg          :  90.0                            , $
    early_azimuth_edge            :  '-9999.9999'                    , $
    near_range_edge               :  '-9999.9999'                    , $
    range_swath_width_m           :  range_swath_width_m             , $
    azimuth_swath_width_m         :  azimuth_swath_width_m           , $
    sensor_altitude_m             :  -9999.9999                      , $
    incid_angle_center_deg        :  float(x.incidenceAngleMidSwath) , $
    slant_range_center_m          :  -9999.9999                      , $
    slant_range_near_m            :  -9999.9999                      , $
    lines_original_image          :  nx0                             , $
    pixels_original_image         :  ny0                             , $
    lines_sio_image               :  nx1                             , $
    pixels_sio_image              :  ny1                             , $
    nx0                           :  nx0                             , $
    ny0                           :  ny0                             , $
    nx1                           :  nx1                             , $
    ny1                           :  ny1                             , $
    ave_pixel                     :  df.ave_pixel                    , $
    ave_line                      :  df.ave_line                     , $
    looks_range                   :  float(x.range_looks)            , $
    looks_azimuth                 :  float(x.azimuth_looks)          , $
    ref_ellipsoid                 :  x.ellipsoidName                 , $
    ref_semi_major_axis_m         :  x.ellipsoidSemiMajorAxis        , $
    ref_semi_minor_axis_m         :  x.ellipsoidSemiMinorAxis        , $
    datumshiftparameters          :  '0 0  0'                        , $
    datum_scale                   :  1                               , $
    nrcs_slope                    : 1.00                             , $
    nrcs_bias                     : 0.0                              , $
    lngswitchflag                 : 0                                , $
    icefile                       :''                                , $
    icefile_time_js               :''                                , $
    icefile_time                  :''                                , $
    cpol_remove_noise             : cpol_remove_noise                , $
    xpol_remove_noise             : xpol_remove_noise                , $
    deg                           : df.deg                           , $
    deg_incid                     : df.deg                           , $
    deg_rlook                     : df.deg                            }
    

;
; Correction of NRCS
;
satellite = getwrd(xfile, 0, del='-')
if satellite ne 's1b' then satellite = 's1a'
case satellite of 
's1a' : begin
          case lf.polarization of
          'HH': begin
                  lf.nrcs_slope = df.sen1_slope_hh
                  lf.nrcs_bias  = df.sen1_bias_hh
                end
          'VV': begin
                  lf.nrcs_slope = df.sen1_slope_vv
                  lf.nrcs_bias  = df.sen1_bias_vv
                end
          'HV': begin
                  lf.nrcs_slope = df.sen1_slope_hv
                  lf.nrcs_bias  = df.sen1_bias_hv
                end
          'VH': begin
                  lf.nrcs_slope = df.sen1_slope_hv
                  lf.nrcs_bias  = df.sen1_bias_hv
                end
          endcase
        end
's1b' : begin
          case lf.polarization of
          'HH': begin
                  lf.nrcs_slope = df.sen2_slope_hh
                  lf.nrcs_bias  = df.sen2_bias_hh
                end
          'VV': begin             
                  lf.nrcs_slope = df.sen2_slope_vv
                  lf.nrcs_bias  = df.sen2_bias_vv
                end
          'HV': begin
                  lf.nrcs_slope = df.sen2_slope_hv
                  lf.nrcs_bias  = df.sen2_bias_hv
                end
          'VH': begin
                  lf.nrcs_slope = df.sen2_slope_hv
                  lf.nrcs_bias  = df.sen2_bias_hv
                end
          endcase
        end        
endcase
;
;  Compute fit 
;
;  Make sure that we start in the +/- 180 degrees basis
;
w = where( x.longitude gt 180, count)
if count gt 0 then x.longitude[w] = x.longitude[w] - 360.00

lonmax  =   max( x.longitude )
lonmin  =   min( x.longitude );

if lonmax gt 120 and lonmin lt -120 then lf.lngswitchflag = 1;
if lf.lngswitchflag then begin
  ; switch to 0-360 basis   
  w = where( x.longitude lt 0, count)
  if count gt 0 then x.longitude[w] = x.longitude[w] + 360.00
endif

if printflag then print, 'Fitting longitude...'
ans_compute_fit, x.pixel     , x.line     , x.longitude      , df.deg, lon_coef   ,   lon_xexp ,   lon_yexp , quiet = quiet
if printflag then print, 'Fitting latitude...'
ans_compute_fit, x.pixel     , x.line     , x.latitude       , df.deg, lat_coef   ,   lat_xexp ,   lat_yexp , quiet = quiet
if printflag then print, 'Fitting incident Angle...'
ans_compute_fit, x.pixel     , x.line     , x.incidenceAngle , df.deg, incid_coef , incid_xexp , incid_yexp , quiet = quiet
if printflag then print, 'Fitting i...'
ans_compute_fit, x.longitude , x.latitude , x.pixel          , df.deg, i_coef     ,     i_xexp ,     i_yexp , quiet = quiet
if printflag then print, 'Fitting j...'
ans_compute_fit, x.longitude , x.latitude , x.line           , df.deg, j_coef     ,     j_xexp ,     j_yexp , quiet = quiet
;
; Compute array of radar look directions to fit
;


nfit   = 11L
iline  = dindgen(nfit) * (nx1-1)   / nfit
jline  = dindgen(nfit) * (ny1-1)   / nfit
ii     = dblarr( nfit, nfit )
jj     = dblarr( nfit, nfit )
n      = n_elements( ii )
rlook  = dblarr( n )
ii1    = rlook
jj1    = rlook

for i = 0, nfit - 1 do ii[*,i] = iline
for i = 0, nfit - 1 do jj[i,*] = jline
ind = 0
for i=0, nfit - 1 do begin
  for j=0, nfit - 1 do begin
    lng0       =  ans_poly_cal( ii[i,j]      , jj[i,j], lon_coef, lon_xexp, lon_yexp )
    lat0       =  ans_poly_cal( ii[i,j]      , jj[i,j], lat_coef, lat_xexp, lat_yexp )
    lng1       =  ans_poly_cal( ii[i,j] + 1D , jj[i,j], lon_coef, lon_xexp, lon_yexp )
    lat1       =  ans_poly_cal( ii[i,j] + 1D , jj[i,j], lat_coef, lat_xexp, lat_yexp )
    ll2rb, lng0, lat0, lng1, lat1, r, b
    out =  string(i, format='(I3)') + string(j, format='(I3)')
    if j mod 20 eq 0 then begin
      out =  out + string(ii[i,j], format='(I7)') + string(j, format='(I7)')
      out =  out + string( lng0, format='(F8.2)')  + string( lat0, format='(F8.2)') 
      out =  out + string( lng1, format='(F8.2)')  + string( lat1, format='(F8.2)') 
      out =  out + string( r,    format='(F12.9)') + string(   b,  format='(F9.3)') 
      if printflag then print, out
    endif
    rlook[ind] = b
    ii1[ind]   = ii[i,j]
    jj1[ind]   = jj[i,j]
    ind        = ind + 1L 
  endfor
endfor



ans_compute_fit, ii1, jj1, rlook,  df.deg, rlook_coef, rlook_xexp , rlook_yexp , quiet = quiet
; 
; Display fits
;
if keyword_set(display) then begin
  nx     = nx1 / 10
  ny     = ny1 / 10
  if printflag then print, 'Computing fits using array size of ' + string(nx) + ' x ' + string(ny)
  lng    = fltarr(nx,ny)
  lat    = lng
  incid  = lng
  rlk    = lng
  for i = 0, nx - 1L do begin
    ii  = double(i) * 10.00
    for j = 0, ny -1L do begin
       jj = double(j) * 10.00
       lng[i,j]    = ans_poly_cal( ii, jj, lon_coef  , lon_xexp  ,   lon_yexp )
       lat[i,j]    = ans_poly_cal( ii, jj, lat_coef  , lat_xexp  ,   lat_yexp )
       incid[i,j]  = ans_poly_cal( ii, jj, incid_coef, incid_xexp, incid_yexp ) 
       rlk[i,j]    = ans_poly_cal( ii, jj, rlook_coef, rlook_xexp, rlook_yexp ) 
    endfor
  endfor
  im = image( lng,  title= 'Longitude'  )
  stop
  im = image( lat,  title= 'Latitude'   )
  stop
  im = image(incid, title= 'Incidence' )
  stop
  im = image(rlk,   title= 'Radar Look' )
  stop
endif
lf = create_struct( lf,  'lon_coef'       ,  lon_coef    , $
                         'lon_xexp'       ,  lon_xexp    , $
                         'lon_yexp'       ,  lon_yexp    , $
                         'lat_coef'       ,  lat_coef    , $
                         'lat_xexp'       ,  lat_xexp    , $
                         'lat_yexp'       ,  lat_yexp    , $
                         'i_coef'         ,  i_coef      , $
                         'i_xexp'         ,  i_xexp      , $
                         'i_yexp'         ,  i_yexp      , $
                         'j_coef'         ,  j_coef      , $
                         'j_xexp'         ,  j_xexp      , $
                         'j_yexp'         ,  j_yexp      , $
                         'incid_coef'     ,  incid_coef  , $
                         'incid_xexp'     ,  incid_xexp  , $
                         'incid_yexp'     ,  incid_yexp  , $
                         'rlook_coef'     ,  rlook_coef  , $
                         'rlook_xexp'     ,  rlook_xexp  , $
                         'rlook_yexp'     ,  rlook_yexp  )
                         
                         
; compute center geographic position
;
ic         = nx1 / 2
jc         = ny1 / 2 
center_lon = ans_poly_cal( double(ic), double(jc), lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
center_lat = ans_poly_cal( double(ic), double(jc), lf.lat_coef, lf.lat_xexp, lf.lat_yexp)

lf  = create_struct(lf, 'center_lat', center_lat, 'center_lon', center_lon ) 

end








