;;-----------------------------------------------------
;+
; NAME: SEN_IMG2SIGMA
; PURPOSE:
;   Convert Sentinel-1 image and calibration file to calibrated NRCS
;   image
; CALLING SEQUENCE
;   sen_img2sigma, imgfile, calfile, img 
; INPUT
;   imgfile    :  Image file (tiff)
;   calfile    :  Calibration file
; OUTPUT
;   img        :  NRCS image
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /display    :  Display NRCS image
;  /diagnosis  :  Provide statistics and stop and key points.
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo. Creation
;  2013 Sep 17 Frank Monaldo. Add calibration for SLC data
;  2014 May 14 Frank Monaldo. Correct spelling
;  2014 Oct 14 Frank Monaldo. Add diagnosis option
;  2014 Oct 20 Frank Monaldo. Divide by the square of sigmanought
; Copyright (C) 2013-2014, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_img2sigma, imgfile, calfile, img,  help=help, quiet=quiet, error=error, display=display, $
                   diagnosis=diagnosis

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_IMG2SIGMA
  print, ' PURPOSE:
  print, '   Convert Sentinel-1 image and calibration file to calibrated NRCS
  print, '   image
  print, ' CALLING SEQUENCE
  print, '   sen_img2sigma, imgfile, calfile, img 
  print, ' INPUT
  print, '   imgfile    :  Image file (tiff)
  print, '   calfile    :  Calibration file
  print, ' OUTPUT
  print, '   img        :  NRCS image
  print, ' KEYWORDS:
  print, '  /help       :  Print out help
  print, '  /quiet      :  Suppress intermediate print output
  print, '  /display    :  Display NRCS image
  print, '  error=      :  error <0 if cannot read data
  error = -1
  return
endif
;
;Sentinel-1 Product Specification
; Ref: S1-RS-MDA-52-7441
; Issue/Revision: 2/7
; Date: Jan. 30, 2014
; The L1 Calibration ADS provides four calibration LUTs with every product. Applying any of these tables undoes the application output 
; scaling introduced by the processor and scales the image to achieve the desired type of absolute calibration, notably:
;
; Three of these LUTs allow one to convert the digital numbers in the image MDS into beta0 , sigma0 or gamma radiometrically calibrated 
; imagery  depending on which LUT is used  by applying a range dependent gain (and a constant offset in the GRD case) to the image data. 
; Note that in order to achieve the desired calibration the external calibration factor Kext (externalCalibrationScalingFactor), 
; has also been built into these tables. The fourth LUT, the dn LUT allows one to recover the original DN value that the image had before 
; the scaling by the application LUT (therefore the dn LUT does not include the external calibration factor Kext).
;
; In order to convert the DN of a given range pixel i in a Sentinel-1 SAR image to a calibrated value or the original DN, the 
; user has to apply one of the following formulae:
;
; For SLC products: 
;
;  value(i) = |DN_I|^2 / A_i^2
; 
;  For GRD products:
;
;  value(i) = (|DN_I|^2 + b) / A_i^2
;
;  where, depending on the selected LUT:
;
;   value(i) = one of beta0_i , sigma0_i, gamma_i, or original DN_i .
;   A = one of betaNought(i) , sigmaNought(i) , gamma(i) or dn(i)
;   b = constant offset.

error= 0

if printflag then print, 'Creating NRCS array using sen_img2sigma'

if not file_test(imgfile) then begin
  print, 'Cannot find file: ' + imgfile+ ' Exiting sen_img2sigma'
  error = -1
  return
endif

if not file_test(calfile) then begin
  print, 'Cannot find file: ' + calfile+ ' Exiting sen_img2sigma'
  error = -1
  return
endif


sen_read_image, imgfile, img, error = error, quiet = quiet

if keyword_set(diagnosis) then begin
  w=  where(img gt 0,count)
  shortfile = getwrd(imgfile, 0, /last, del='/')
  print, 'Image                : ' + shortfile
  print, 'Image mean           : ' + strcompress(mean(img))
  print, 'Image std dev        : ' + strcompress(stdev(img))
  print, 'Maximum              : ' + strcompress(max(img))
  if count gt 0 then begin
    print, 'For values gt 0
    print, 'Image mean           : ' + strcompress(mean(  img[w] ))
    print, 'Image std dev        : ' + strcompress(stdev( img[w] ))
    print, 'Maximum              : ' + strcompress(max(img))
    h  =  hist(img[w], xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram Image (>0)', xrange=[0, max(xx)] /4 )
    p.save, 'hist'+shortfile+'.png', resolution=128
  endif else begin
    h  =  hist(img, xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram Image', xrange=[0, max(xx)] /4 )
    p.save, 'hist'+shortfile+'.png', resolution=128 
  endelse
endif
if error lt 0 then return
if printflag then hlp, img

sen_read_cal_xml, calfile, x, error = error, quiet =  quiet
if error lt 0 then return
;
; Fit to calibration
;
;   Create 1D from 2D arrays
n=  n_elements(x.pixel)
xx  = lonarr(n)
yy  = lonarr(n)
zz  = fltarr(n)
for i=0, n-1L do begin
  xx[i] = x.pixel[i]
  yy[i] = x.line[i]
  zz[i] = x.sigmaNought[i]
endfor

if keyword_set(diagnosis) then begin
  print, 'SigmaNought Cal      : ' + shortfile
  print, 'Cal mean             : ' + strcompress(mean(  x.sigmaNought ))
  print, 'Cag std dev          : ' + strcompress(stdev( x.sigmaNought ))   
endif

ans_compute_fit, xx, yy, zz, 2, coef, xexp, yexp, quiet=quiet

if keyword_set(diagnosis) then begin
  zzout = ans_poly_cal( xx, yy, coef, xexp, yexp )
  print, 'Difference between input and fit calibration values'
  print, 'Cal fit mean         : ' + strcompress(mean(  x.sigmaNought ))
  print, 'Cal fit std dev      : ' + strcompress(stdev( x.sigmaNought )) 
endif 


sz    = size(img)
nx    = sz[1]
ny    = sz[2]
if printflag then print, 'Squaring the image...        ' + systime(/utc)
img   = float(img) * img
if keyword_set(diagnosis) then begin
  w=  where(img gt 0,count)
  shortfile = getwrd(imgfile, 0, /last, del='/')
  print, 'Image^2              : ' + shortfile
  print, 'Image^2 mean         : ' + strcompress(mean( img  ))
  print, 'Image^2 Std Dev      : ' + strcompress(stdev( img ))
  print, 'Image^2 Maximum      : ' + strcompress(max(  img  ))
  if count gt 0 then begin
    print, 'For values gt 0
    print, 'Image^2 mean         : ' + strcompress(mean(  img[w] ))
    print, 'Image^2 Std Dev      : ' + strcompress(stdev( img[w] ))
    print, 'Image^2 Maximum      : ' + strcompress(max(   img[w] ))
    h  =  hist(img[w], xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram Image^2 (>0)', xrange=[0, max(xx)] /4 )
    p.save, 'hist2'+shortfile+'.png', resolution=128
  endif else begin
    h  =  hist(img, xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram Image^2', xrange=[0, max(xx)] /4 )
    p.save, 'hist2'+shortfile+'.png', resolution=128
  endelse
endif


iline = dindgen(nx)
jline = dindgen(ny)
ii    = dblarr(nx,ny)
jj    = dblarr(nx,ny)
if printflag then print, 'Computing calibration fit... ' + systime(/utc)
for j=0, ny-1L do ii(*, j) = iline
for i=0, nx-1L do jj(i, *) = jline
aa    =  float( ans_poly_cal( ii, jj, coef, xexp, yexp) )

w= where(aa ne 0, count )
if keyword_set(diagnosis) then begin
  hlp, aa
endif

if count gt 0 then begin
 if strpos( imgfile, 'slc') lt 0 then begin
   ; Intensity data
   img[w] = img[w]  / aa[w] /aa[w]
 endif else begin
   ; Single-look complex data
  img[w] = img[w]  / aa[w] / aa[w]
 endelse
endif
w= where( aa eq 0, count)
if count gt 0 then img[w]= 0

if keyword_set(diagnosis) then begin
  w=  where(img gt 0,count)
  shortfile = getwrd(imgfile, 0, /last, del='/')
  print, 'NRCS                 : ' + shortfile
  print, 'NRCS mean            : ' + strcompress(mean(img))
  print, 'NRCS std dev         : ' + strcompress(stdev(img))
  if count gt 0 then begin
    print, 'For values gt 0
    print, 'NRCS mean            : ' + strcompress(mean(  img[w] ))
    print, 'NRCS std dev        : ' + strcompress(stdev( img[w] ))
    h  =  hist(img[w], xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram NRCS (>0)',xrange=[0, max(xx)] /4 )
    p.save, 'nrcs'+shortfile+'.png', resolution=128
  endif else begin
    h  =  hist(img, xx, nbin=500)
    p  =  plot(xx, h, /buffer, xtitle='Histogram NRCS', xrange=[0, max(xx)] /4 )
    p.save, 'nrcs'+shortfile+'.png', resolution=128
  endelse
endif

if keyword_set(display) then begin
  im=  image( ls(img)                 , $
              axis_style    = 1       , $
              xcolor        = 'white' , $
              ycolor        = 'white' , $
              xtext_color   = 'black' , $
              ytext_color   = 'black' , $
              xtitle        = 'Pixel' , $
              ytitle        = 'Line' )
endif

end
