;;-----------------------------------------------------
;+
; NAME: SEN_IMG2SIGMA_NOISE
; PURPOSE:
;   Convert Sentinel-1 image and calibration file to calibrated NRCS
;   image
; CALLING SEQUENCE
;   sen_img2sigma_noise, imgfile, calfile, noisefile, img
; INPUT
;   imgfile   : Image file (tiff)
;   calfile   : Calibration file
;   noisefile : Noise file
; OUTPUT
;   img       : NRCS image
; KEYWORDS:
;  /help      : Print out help
;  /quiet     : Suppress intermediate print output
;  /display   : Display NRCS image
;  /diagnosis : Provide statistics and stop and key points.
;  /keepnoise : Do not remove noise
;  dn2_mean   : Mean DN^2 squared input
;  noise_mean : Mean noise value
;  final_mean : Mean (DN^2 - noise)
;  error      : error <0 if cannot read data
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo. Creation
;  2013 Sep 17 Frank Monaldo. Add calibration for SLC data
;  2014 May 14 Frank Monaldo. Correct spelling
;  2014 Oct 14 Frank Monaldo. Add diagnosis option
;  2014 Oct 20 Frank Monaldo. Divide by the square of sigmanought
;  2016 Apr 14 Frank Monaldo. Added noise removal and noise file to
;                             the call
;  2016 Apr 20 Frank Monaldo. Add the ability return the image mean
;                             and noise statistics
;  2019 Mar 21 Chris Jackson. Correction to deal with NaNs resulting from the noise subtraction.
;  2020 Apr 20 Chris Jackson. Fix the cases where only valid rng_img is available (Pre-March 2018)
;  2020 Sep 29 Chris Jackson  Update calls to sen_range_noise_image, sen_azimuth_noise_image to return the array of beam positions
;  2020 Oct 27 Chris Jackson  Eliminate the call for Beam Balancing.  
; Copyright (C) 2013-2016, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro sen_img2sigma_noise, imgfile, calfile, noisefile, img,  $
  help        =  help                   ,  $
  quiet       =  quiet                  ,  $
  error       =  error                  ,  $
  display     =  display                ,  $
  keepnoise   =  keepnoise              ,  $
  dn2_mean    =  dn2_mean               ,  $
  noise_mean  =  noise_mean             ,  $
  final_mean  =  final_mean             ,  $
  diagnosis   =  diagnosis

  printflag=1
  if keyword_set(quiet) then printflag=0

  if keyword_set(help) or (n_params(0) lt 1 ) then begin
    print, ' NAME: SEN_IMG2SIGMA_NOISE
    print, ' PURPOSE:
    print, '   Convert Sentinel-1 image and calibration file to calibrated NRCS
    print, '   image
    print, ' CALLING SEQUENCE
    print, '   sen_img2sigma_noise, imgfile, calfile, noisefile, img
    print, ' INPUT
    print, '   imgfile   : Image file (tiff)
    print, '   calfile   : Calibration file
    print, '   noisefile : Noise file
    print, ' OUTPUT
    print, '   img       : NRCS image
    print, ' KEYWORDS:
    print, '  /help      : Print out help
    print, '  /quiet     : Suppress intermediate print output
    print, '  /display   : Display NRCS image
    print, '  /diagnosis : Provide statistics and stop and key points.
    print, '  /keepnoise : Do not remove noise
    print, '  dn2_mean   : Mean DN^2 squared input
    print, '  noise_mean : Mean noise value
    print, '  final_mean : Mean (DN^2 - noise)
    print, '  error      : error <0 if cannot read data
    error = -1
    return
  endif
  ;
  ;Sentinel-1 Product Specification
  ; Ref: S1-RS-MDA-52-7441
  ; Issue/Revision: 2/7
  ; Date: Jan. 30, 2014
  ; The L1 Calibration ADS provides four calibration LUTs with every product. Applying any of these tables undoes the application output
  ; scaling introduced by the processor and scales the image to achieve the desired type of absolute calibration, notably:
  ;
  ; Three of these LUTs allow one to convert the digital numbers in the image MDS into beta0 , sigma0 or gamma radiometrically calibrated
  ; imagery � depending on which LUT is used � by applying a range dependent gain (and a constant offset in the GRD case) to the image data.
  ; Note that in order to achieve the desired calibration the external calibration factor Kext (externalCalibrationScalingFactor),
  ; has also been built into these tables. The fourth LUT, the dn LUT allows one to recover the original DN value that the image had before
  ; the scaling by the application LUT (therefore the dn LUT does not include the external calibration factor Kext).
  ;
  ; In order to convert the DN of a given range pixel i in a Sentinel-1 SAR image to a calibrated value or the original DN, the
  ; user has to apply one of the following formulae:
  ;
  ; For SLC products:
  ;
  ;  value(i) = |DN_I|^2 / A_i^2
  ;
  ;  For GRD products:
  ;
  ;  value(i) = (|DN_I|^2 + b) / A_i^2
  ;
  ;  where, depending on the selected LUT:
  ;
  ;   value(i) = one of beta0_i , sigma0_i, gamma_i, or original DN_i .
  ;   A = one of betaNought(i) , sigmaNought(i) , gamma(i) or dn(i)
  ;   b = constant offset.

  error= 0

  if printflag then print, 'Creating NRCS array using sen_img2sigma'

  if not file_test(imgfile) then begin
    print, 'Cannot find image file: ' + imgfile+ ' Exiting sen_img2sigma_noise'
    error = -2
    return
  endif

  if not file_test(calfile) then begin
    print, 'Cannot find file: ' + calfile+ ' Exiting sen_img2sigma_noise'
    error = -3
    return
  endif

  sen_read_image, imgfile, img, error = error, quiet = quiet
  ;
  ;========================================================
  ; Image before squaring
  ;========================================================
  ;
  if keyword_set(diagnosis) then begin
    w=  where(img gt 0,count)
    shortfile = getwrd(imgfile, 0, /last, del='/')
    print, 'Image                : ' + shortfile
    print, 'Image mean           : ' + strcompress(mean(img))
    print, 'Image std dev        : ' + strcompress(stdev(img))
    print, 'Maximum              : ' + strcompress(max(img))
    if count gt 0 then begin
      print, 'For values gt 0
      print, 'Image mean           : ' + strcompress(mean(  img[w] ))
      print, 'Image std dev        : ' + strcompress(stdev( img[w] ))
      print, 'Maximum              : ' + strcompress(max(img))
      h  =  hist(img[w], xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram Image (>0)', xrange=[0, max(xx)] /4 )
      p.save, 'hist'+shortfile+'.png', resolution=128
    endif else begin
      h  =  hist(img, xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram Image', xrange=[0, max(xx)] /4 )
      p.save, 'hist'+shortfile+'.png', resolution=128
    endelse
  endif
  if printflag then hlp, img
  ;
  ;========================================================
  ; Image after squaring
  ;========================================================
  ;
  sz    = size(img)
  nx    = sz[1]
  ny    = sz[2]
  if printflag then print, 'Squaring the image...        ' + systime(/utc)
  img   = float(img) * img
  if keyword_set(diagnosis) then begin
    w=  where(img gt 0,count)
    shortfile = getwrd(imgfile, 0, /last, del='/')
    print, 'Image^2              : ' + shortfile
    print, 'Image^2 mean         : ' + strcompress(mean( img  ))
    print, 'Image^2 Std Dev      : ' + strcompress(stdev( img ))
    print, 'Image^2 Maximum      : ' + strcompress(max(  img  ))
    if count gt 0 then begin
      print, 'For values gt 0
      print, 'Image^2 mean         : ' + strcompress(mean(  img[w] ))
      print, 'Image^2 Std Dev      : ' + strcompress(stdev( img[w] ))
      print, 'Image^2 Maximum      : ' + strcompress(max(   img[w] ))
      h  =  hist(img[w], xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram Image^2 (>0)', xrange=[0, max(xx)] /4 )
      p.save, 'hist2'+shortfile+'.png', resolution=128
    endif else begin
      h  =  hist(img, xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram Image^2', xrange=[0, max(xx)] /4 )
      p.save, 'hist2'+shortfile+'.png', resolution=128
    endelse
  endif

  dn2_mean   = mean(img)
  noise_mean = 0.0
  final_mean = dn2_mean

  ;
  ;========================================================
  ; Get Calibration Values 
  ;========================================================
  ;
  sen_read_cal_xml, calfile  ,  x, error = error, quiet =  quiet
  if error lt 0 then return
  ;
  ; Fit to calibration
  ;
  ;   Create 1D from 2D arrays
  n=  n_elements(x.pixel)
  xx  = lonarr(n)
  yy  = lonarr(n)
  zz  = fltarr(n)
  for i=0, n-1L do begin
    xx[i] = x.pixel[i]
    yy[i] = x.line[i]
    zz[i] = x.sigmaNought[i]
  endfor

  if keyword_set(diagnosis) then begin
    print, 'SigmaNought Cal      : ' + shortfile
    print, 'Cal mean             : ' + strcompress(mean(  x.sigmaNought ))
    print, 'Cag std dev          : ' + strcompress(stdev( x.sigmaNought ))
  endif

  ans_compute_fit, xx, yy, zz, 2, coef, xexp, yexp, quiet=quiet

  if keyword_set(diagnosis) then begin
    zzout = ans_poly_cal( xx, yy, coef, xexp, yexp )
    print, 'Difference between input and fit calibration values'
    print, 'Cal fit mean         : ' + strcompress(mean(  x.sigmaNought ))
    print, 'Cal fit std dev      : ' + strcompress(stdev( x.sigmaNought ))
  endif

  iline = dindgen(nx)
  jline = dindgen(ny)
  ii    = dblarr(nx,ny)
  jj    = dblarr(nx,ny)
  if printflag then print, 'Computing calibration fit... ' + systime(/utc)
  for j=0, ny-1L do ii(*, j) = iline
  for i=0, nx-1L do jj(i, *) = jline
  aa    =  float( ans_poly_cal( ii, jj, coef, xexp, yexp) )

  w= where(aa ne 0, count )
  if keyword_set(diagnosis) then hlp, aa

  ;
  ;========================================================
  ; Get Range and Azimuth Noise 
  ;========================================================
  ;
  if not keyword_set(keepnoise) then begin
    sz   = size(img)
    nx   = sz[1]
    ny   = sz[2]

    sen_range_noise_image, noisefile, rng_img, nx=nx, ny=ny, sid=sid_rng, error=error
    
    if error lt 0 then begin
      if printflag then begin
        print, 'Cannot process range noise in file: ' + noisefile
        print, ' Exiting sen_img2sigma_noise'
      endif
      error = -4
      return
    endif

    ;Put in a sction here to put in the Park beam balancing coefficients (instead of in sen_range_noise)
    ;Apply sub-swath scaling - See Park Paper - just a place holder for now
    ;As we just apply his derived value.  But they seem to improve the results for imagery acquired pre-March 2018
    ;and we should develop a program to calcuate these on a running basis
    ;Check if swath is EW - if the sid contains 5 vlues
;    KN = [1.363,0.991,1.043,0.990,0.932];
;    KP = 1.e-4*[-2.602,-3.553,-2.661,-2.289,-2.106]
;    kp_img = rng_img*0.0+1.0
;    nbeams = max(sid_rng)
;    if (nbeams eq 5) then begin
;      for i = 0, nbeams-1 do begin
;        idx = where(sid_rng eq i+1, cnt) ;
;        rng_img[idx] = temporary(rng_img[idx])*KN[i]
;        kp_img[idx] = kp_img[idx]*KP[i]
;      end
;    endif
;    kp_img = temporary(kp_img)*aa*aa

    ;Default the az_img to 1's
    az_img = rng_img*0.0+1.0

    sen_azimuth_noise_image, noisefile, az_img, nx=nx, ny=ny, sid=sid_az, error=error
    if (error eq -2) then begin
      print, 'Cannot process azimuth noise in file: ' + noisefile
      print, 'The azimuth noise array is not present.  File might predate March 2018'
      error = 0
    endif

    ;
    ;========================================================
    ; Calibrate image from noise-removed image
    ;========================================================
    ;
    if strpos( imgfile, 'slc') lt 0 then begin
      ; Intensity data
      img  = temporary(img) - rng_img*az_img
      ;img  = temporary(img) - (rng_img*az_img + kp_img)
      w      = where( img lt 0, /null)
      img[w]  = 10.0^(-5.0)
      w     = where(finite(img,/NAN),/null)
      img[w] = 10.0^(-5.0)
    endif
    ;noise_mean = mean(x.noiselut)
    final_mean = mean(img)
    if printflag then begin
      print, 'Mean image before noise removal : ' + string( dn2_mean         ,  format ='(E20.12)')
      print, 'Mean image after noise removal  : ' + string( final_mean       ,  format ='(E20.12)')
      ;print, 'Mean noise fit                  : ' + string( mean(aa)         ,  format ='(E20.12)')
      ;print, 'Mean noise LUT                  : ' , string( mean(x.noiselut) ,  format ='(E20.12)')
    endif
  endif    ; End check if keepnoise keyword set

  
  
  w= where(aa ne 0, count )
  if count gt 0 then begin
    if strpos( imgfile, 'slc') lt 0 then begin
      ; Intensity data
      img[w] = img[w]  / aa[w] /aa[w]
    endif else begin
      ; Single-look complex data
      img[w] = img[w]  / aa[w] / aa[w]
    endelse
  endif
  w= where( aa eq 0, count)
  if count gt 0 then img[w]= 0

  if keyword_set(diagnosis) then begin
    w=  where(img gt 0,count)
    shortfile = getwrd(imgfile[0], 0, /last, del='/')
    print, 'NRCS                 : ' + shortfile
    print, 'NRCS mean            : ' + strcompress(mean(img))
    print, 'NRCS std dev         : ' + strcompress(stdev(img))
    if count gt 0 then begin
      print, 'For values gt 0
      print, 'NRCS mean            : ' + strcompress(mean(  img[w] ))
      print, 'NRCS std dev        : ' + strcompress(stdev( img[w] ))
      h  =  hist(img[w], xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram NRCS (>0)',xrange=[0, max(xx)] /4 )
      p.save, 'nrcs'+shortfile+'.png', resolution=128
    endif else begin
      h  =  hist(img, xx, nbin=500)
      p  =  plot(xx, h, /buffer, xtitle='Histogram NRCS', xrange=[0, max(xx)] /4 )
      p.save, 'nrcs'+shortfile+'.png', resolution=128
    endelse
  endif

  if keyword_set(display) then begin
    im=  image( ls(img)                 , $
      axis_style    = 1       , $
      xcolor        = 'white' , $
      ycolor        = 'white' , $
      xtext_color   = 'black' , $
      ytext_color   = 'black' , $
      xtitle        = 'Pixel' , $
      ytitle        = 'Line' )
  endif

end
