;;-----------------------------------------------------
;+
; NAME: SEN_NOISE_SCALE_BALANCE
; PURPOSE:
;  Program will return the value fpr Noise Power Scaling and Interswath Power Balancing
;  described Efficient Thermal Noise Removal for Sentinel-1 TOPSAR Cross-Polarization Channel
;  Jeong-Won Park et al. IEEE TRANSACTIONS ON GEOSCIENCE AND REMOTE SENSING Vol 56, Issue3, March 2018
;  https://doi.org/10.1109/TGRS.2017.2765248
;  The program uses the S1 GRD NRCS image along with ESA suppplied noise vectors
;  to estimate the Kns and Kpb as detailed in the paper
;  All calculations are done pre calibration
;  The program relies on IDL V8.6.1 (or greater) since it ruses the IDL XML reader (in sen_read_noise_xml_fast.pro)
; CALLING SEQUENCE
;   sen_noise_scale_balance, input_dir, kns, kpb
; INPUT
;   input_dir  =  Sentinel data come in directories. This input is the name
;                  of such a directory. A directory might look like
;                  S1A_EW_GRDM_1SDV_20181110T020115_20181110T020219_024519_02B0A7_2729.SAFE/
; OUTPUT
;   kns         :  A 1-D array of the Noise Power Scaleing values (in element per beam position)
;   kpb         :  A 1-D array of the Interswath Power Balancing values (in element per beam position)
; KEYWORDS:
;   /copol      :  Process the co-pol image instead of the cross pol image
;  /help        :  Print out help
;  /quiet       :  Suppress intermediate print output
;  /display     :  Display some diagnostic plots
;  error=       :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2019 Jan 16 Chris Jackson. Creation
;
; Copyright (C) 2013-2018, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro sen_noise_scale_balance, input_dir, kns, kpb,  help = help, quiet=quiet, error=error, display=display

  printflag=1
  if keyword_set(quiet) then printflag=0


  if keyword_set(help) or (n_params(0) lt 1 ) then begin
    print, ' NAME: SEN_NOISE_SCALE_BALANCE
    print, ' PURPOSE:
    print, '  Program will return the value fpr Noise Power Scaling and Interswath Power Balancing
    print, '  described Efficient Thermal Noise Removal for Sentinel-1 TOPSAR Cross-Polarization Channel
    print, '  Jeong-Won Park et al. IEEE TRANSACTIONS ON GEOSCIENCE AND REMOTE SENSING Vol 56, Issue3, March 2018
    print, '  https://doi.org/10.1109/TGRS.2017.2765248
    print, '  The program uses the S1 GRD NRCS image along with ESA suppplied noise vectors
    print, '  to estimate the Kns and Kpb as detailed in the paper
    print, '  All calculations are done pre calibration
    print, '  The program relies on IDL V8.6.1 (or greater) since it ruses the IDL XML reader (in sen_read_noise_xml_fast.pro)
    print, ' CALLING SEQUENCE
    print, '   sen_noise_scale_balance, input_dir, kns, kpb
    print, ' INPUT
    print, '   input_dir  =  Sentinel data come in directories. This input is the name
    print, '                  of such a directory. A directory might look like
    print, '                  S1A_EW_GRDM_1SDV_20181110T020115_20181110T020219_024519_02B0A7_2729.SAFE/
    print, ' OUTPUT
    print, '   kns         :  A 1-D array of the Noise Power Scaleing values (in element per beam position)
    print, '   kpb         :  A 1-D array of the Interswath Power Balancing values (in element per beam position)
    print, ' KEYWORDS:
    print, '  /copol      :  Process the co-pol image instead of the cross pol image
    print, '  /help       :  Print out help
    print, '  /quiet      :  Suppress intermediate print output
    print, '  /display    :  Display contents of x
    print, '  error=      :  error <0 if cannot read data
    error = -1
    return
  endif

  error= 0

  ff= file_search( input_dir, /test_directory, count=count)

  if count eq 0 then begin
    print, 'Could not find Sentinel directory: ' + input_dir
    print, 'Exiting sen_noise_scale_balance '
    error= -1
    return
  endif


  ;----------------------------------------------
  ; Find the data sets
  ;---------------------------------------------

  xfiles=   file_search( input_dir, 's*xml', count=data_setmax)
  if data_setmax eq 0 then begin
    print, 'Cannot file xml data in: ' + input_dir
    print, 'Exiting ans_noise_scale_balance.pro'
    error = -1
    return
  endif
  ;
  ; Loop through data sets.
  ;

  for data_set= 0, data_setmax-1 do begin

    fullname  = getwrd( xfiles[data_set], 0, /last, del='/' )
    ;Temporary substitue for windows
    ;fullname  = getwrd( xfiles[data_set], 0, /last, del='\' )
    corename  = strmid(fullname, 0, strlen(fullname)-4 )
    print, 'Processing: ' + corename

    ;Determine the polarization
    pol =  strupcase(strmid(corename,11,2))
    if keyword_set(copol)  then begin
      if strcmp(pol,'VH') or strcmp(pol,'HV') then continue
    endif else begin
      if (strcmp(pol,'VV') or strcmp(pol,'HH')) then continue
    endelse

    ; Read and calibrate image file
    ;
    imgfile       = file_search( input_dir, corename + '.tiff', count= count)
    if count eq 0 then begin
      print, 'Cannot find image file ' + corename + '.tiff'
      print, 'Exiting ans_noise_scale_balance.pro'
      error = -1
      return
    endif

    sen_read_image, imgfile, img, error = error, quiet = quiet
    sz = size(img);
    ;    nx =
    ;    ny =
    ; Square the digitial number
    img   = float(temporary(img))^2

    noisefile = file_search(input_dir, 'noise*'       + corename + '*', count= count)
    if count eq 0 then begin
      print, 'Cannot find noise file ' + 'noise-' + corename + '*'
      print, 'Exiting ans_noise_scale_balance.pro'
      error = -1
      return
    endif

    sen_azimuth_noise_image, noisefile[0], az_img, sid=sid_az, nx=nx_az, ny=ny_az
    ;Prior to March 2018 there is no azimuth Noise Array (inless we implement Park et al) 
    sen_range_noise_image, noisefile[0], rng_img, sid =sid_rng, nx=nx_rng, ny=ny_rng
    if (isa(az_img, /array) eq 0) then begin
      az_img = img*0.0+1
      az_nx = sz[1]
      az_ny = sz[2]
      sid_az = sid_rng
    endif
    ;Set beam position to zero where the image is zero
    ;sid is the array of beam positions
    sid = sid_az
    sid[where(img eq 0,/null)] = 0
    ;Also do this to az_img and rng_img
    ;az_img[where(img eq 0,/null)] = 0
    ;rng_img[where(img eq 0,/null)] = 0

    ;
    ;========================================================
    ; Calibrate image from noise-removed image
    ;========================================================
    calfile = file_search(input_dir, 'calibration*'       + corename + '*', count= count)
    if count eq 0 then begin
      print, 'Cannot find noise file ' + 'calibration-' + corename + '*'
      print, 'Exiting ans_noise_scale_balance.pro'
      error = -1
      return
    endif
    ;
    sen_read_cal_xml, calfile  ,  x, error = error, quiet =  quiet
    if error lt 0 then return
    ;
    ; Fit to calibration
    ;
    ;   Create 1D from 2D arrays
    n=  n_elements(x.pixel)
    xx  = lonarr(n)
    yy  = lonarr(n)
    zz  = fltarr(n)
    for i=0, n-1L do begin
      xx[i] = x.pixel[i]
      yy[i] = x.line[i]
      zz[i] = x.sigmaNought[i]
    endfor

    ans_compute_fit, xx, yy, zz, 2, coef, xexp, yexp, quiet=quiet
    nx = sz[1];
    ny = sz[2];
    iline = dindgen(nx)
    jline = dindgen(ny)
    ii    = dblarr(nx,ny)
    jj    = dblarr(nx,ny)
    if printflag then print, 'Computing calibration fit... ' + systime(/utc)
    for j=0, ny-1L do ii(*, j) = iline
    for i=0, nx-1L do jj(i, *) = jline
    aa    =  float( ans_poly_cal( ii, jj, coef, xexp, yexp) )
    ;
    wc= where(aa ne 0, count,/null )
    img[wc] = temporary(img[wc])/ aa[wc] /aa[wc]
    rng_img[wc] = temporary(rng_img[wc])/ aa[wc] /aa[wc]

    wc= where( aa eq 0, count,/null)
    img[wc] = 0
    rng_img[wc] = 0



    ;Find which columns contain only a single swath
    n_subswath=3;
    mode='IW"
    if (max(sid) gt 3) then begin
      n_subswath = 5
      mode='EW'
    endif
    col_max = max(sid, dimension=2)
    w0 = where(sid eq 0, /null, count)
    sid[w0] = 32 ; Pick a number bigger than 5
    ;Get the min and the max of each column
    col_min = min(sid, dimension=2)
    ;If there is only 1 beam position in the column min and max will be the same
    col_diff = col_max-col_min
    sid[w0] = 0
    ;Identify the columns where there is only one beam (w_gap) and where there are several (wc_gap)
    w_gap = where(col_diff eq 0,count,complement=wc_gap)
    pdiff = ts_diff(w_gap,1)
    w = where(abs(pdiff) gt 1, count,/null)

    if (count eq  n_subswath-1) then begin
      ;Make an array (2 columns x N subswath) to hold the start and stop pixel locations of each subswath
      ss_pixels=intarr(2,n_subswath)
      ss_pixels[0,0] = w_gap[0]
      ss_pixels[1,0:n_subswath-2] = w_gap[w];
      ss_pixels[0,1:n_subswath-1] = w_gap[w+1]
      ss_pixels[1,n_subswath-1] = n_elements(w_gap)
    endif else begin
      print, 'Wrong Number of Seams ('+string(count, format='(I2)')+') for '+string(n_subswath,format='(I2)')+' subswaths'
      print, 'Exiting ans_noise_scale_balance.pro'
      error = -1
      return
    endelse

    nx_idx = indgen(nx,1)
    ;Need weights - use the range gradient (Do this before zero-ing the rng_img)
    rng_grad=abs(ts_diff(mean(rng_img,dimension=2,/NAN),1))
    rng_grad = mean(rng_img,dimension=2,/NAN)
    ;Zero out the bins with beam overlap
    rng_grad[wc_gap] = 0;
    rng_grad[wc_gap-1] = 0
    rng_grad[wc_gap+1] = 0
    ;rng_grad = temporary(rng_grad)/max(rng_grad[w_gap]) ; May not be necessary
    p2 = plot(nx_idx[w_gap],rng_grad[w_gap],title='Fitting Weigts (Range Noise Gradient)!C'+corename,  ytitle='Gradient',xtitle='Range Bin')

    ;Also do this to az_img and rng_img
    az_img[where(img eq 0,/null)] = 0
    rng_img[where(img eq 0,/null)] = 0


    if keyword_set(display) then begin
      img_mean = mean(img, dimension=2,/NAN)
      noise_mean = mean(rng_img*az_img,dimension=2,/NAN)
      img_noise_mean = mean(img-rng_img*az_img,dimension=2,/NAN)
      p1 = plot(img_mean,color='k',title='Signal(Black) / Noise(Red) / Signal Minus Noise(Blue)!C'+corename, ytitle='NRCS Linear Units',xtitle='Range Bin')
      p1 = plot(noise_mean,color='r',/overplot)
      p1 = plot(img_noise_mean,color='b',/overplot)
      p1.min_value=0
    endif

    ;Find the scale (Kp) by looking over values between 0 and 2 (in stepsize 0.01)
    ; Do the whole image and then collect the stats on each subswath.
    ;We want the Kp which produces the minimum RRS

    fit_param = fltarr(2*n_subswath,201)
    rss = fltarr(n_subswath,201)
    rss_w = fltarr(n_subswath,201)
    ;kk = [0.0:2.0:0.01];  (k 0 to 200)
    ;kk = [0.0:2.0:0.1]; k = 0,20
    kk = [0.8:1.1:0.01];k = 0,30
    for k = 0,30 do begin
      ;Scale the noise and remove it from the NRCS (DN)
      sk_img = img - kk[k]*rng_img*az_img
      sk_mean = mean(sk_img, dimension=2,/NAN)
      print, kk[k]
      for ss = 0, n_subswath-1 do begin ; n_subswath-1 do begin
        ;Do a linear fir on a sub-swath basis
        nx1 = ss_pixels[0,ss]
        nx2 = ss_pixels[1,ss]
        data_x = nx_idx[nx1:nx2]
        data_y = sk_mean[nx1:nx2]
        fit_param[2*ss:2*ss+1,k] = poly_fit(data_x,data_y,1,yfit=yfit, yerror = yerror)
        yresid=sk_mean[nx1:nx2] - yfit
        rss_w[ss,k] = total(rng_grad[nx1:nx2]*yresid^2)
        rss[ss,k] = total(yresid^2)
;        ;                if  keyword_set(display) and ( 10*kk[k] mod 5 eq 0 ) then begin
;        tstr = corename+'!CK='+string(kk[k], format='(f6.2)')+' Beam='+ string(ss+1, format='(I2)')
;        p3 = plot(data_x, data_y,color='grey',sym='+', title=tstr)
;        p3 = plot(data_x, yfit,color='r',/overplot)
;        ;                endif
;  
;  
      endfor
      ;print, 'Here'
    endfor  ;End Loop over K
    ;Now find the min RSS and its associated Kp for each subswath
    min_rss_w = min(rss_w[*,0:k-1],loc_rss_w,dimension=2,max=max_rss_w,/NAN)
    ind_w = array_indices(rss_w, loc_rss_w)
    min_rss = min(rss[*,0:k-1],loc_rss,dimension=2,max=max_rss,/NAN)
    ind = array_indices(rss, loc_rss)

    k_scale = fltarr(n_subswath)
    k_scale_w = k_scale;
    k_scale_w[0:n_subswath-1] = kk[ind_w[1,0:n_subswath-1]]
    k_scale[0:n_subswath-1] = kk[ind[1,0:n_subswath-1]]


    ;Now get K_Balance
    k_bal0 = fltarr(n_subswath)
    k_balf = k_bal0
    new_noise = rng_img *0.0 ;Make a zero array the same size as rng_img (or img)

    print,'Filename,Beam,Kp(Weight),Kp,Ks0, KsF'    ;The initial K_bal of beam 1 is zero
    for ss = 0, n_subswath-1 do begin
      ;Get the range bin at the swath boundary.
      ;For the boundary we will take the mean of the end of beam N-1 and Beam N
      if ss gt 0 then begin
        ibin =  (ss_pixels[1,ss-1]+ss_pixels[0,ss])/2.0
        fidx1 = 2*(ss-1)
        fidx2 = 2*ss
        k_bal0[ss] = poly(ibin,fit_param[fidx1:fidx1+1,ind_w[1,ss-1]])-poly(ibin,fit_param[fidx2:fidx2+1,ind_w[1,ss]])
      endif
      wn = where(sid eq ss+1,/null)
      new_noise[wn] =  k_scale_w[ss]*rng_img[wn]*az_img[wn] + k_bal0[ss]
    endfor

    ;NOw adjust total power of new total noise to be te same as the old total noise
    k_balf = k_bal0 + mean(rng_img - new_noise,/NAN)

    for ss = 0, n_subswath-1 do begin
      wn = where(sid eq ss+1,/null)
      new_noise[wn] =  k_scale_w[ss]*rng_img[wn]*az_img[wn] - k_balf[ss] ; Minus? Plus?
      print,corename,mode,ss+1,k_scale_w[ss],k_scale[ss],k_bal0[ss],k_balf[ss], format='%s,  %s%1d,%10.3f,%10.3f,%10.3f,%10.3f'

      if  keyword_set(display) then begin
        tstr = 'Beam: '+ mode+string(ss+1, format='(I1)')+'!CMin Kp='+string(k_scale_w[ss], format='(f6.2)')+' (Weighted)'
        tstr = tstr+'!C    ='+string(k_scale[ss], format='(f6.2)')+' (UnWeighted)'

        r1 = mean(img - rng_img*az_img,dimension=2,/NAN)
        r2 = mean(img - k_scale_w[ss]*rng_img*az_img,dimension=2,/NAN)
        r3 = mean(img - new_noise,dimension=2,/NAN)
        nx1 = ss_pixels[0,ss]
        nx2 = ss_pixels[1,ss]
        data_x = nx_idx[nx1:nx2]
        if ss eq 0 then begin
          p15 = plot(data_x, r1[nx1:nx2],color='k',title=corename, ytitle='Image and Scaled Noise')
        endif else begin
          p15 = plot(data_x, r1[nx1:nx2],color='k',/overplot)
        endelse
        p15 = plot(data_x, r2[nx1:nx2],color='r',/overplot)
        p15 = plot(data_x, r3[nx1:nx2],color='b',/overplot)
        ;
      endif ; Display
    endfor


    for ss = 0, n_subswath-1 do begin

      ;Plot the RRS value for each subswath
      if  keyword_set(display) then begin
        tstr = 'Beam: '+ mode+string(ss+1, format='(I1)')+'!CMin Kp='+string(k_scale_w[ss], format='(f6.2)')+' (Weighted)'
        tstr = tstr+'!C    ='+string(k_scale[ss], format='(f6.2)')+' (UnWeighted)'
        p4 = plot(kk,rss_w[ss,*]/max_rss_w[ss],color='k',title=tstr, ytitle='Normalized RSS Sum ', xtitle='Scale Factor', POSITION=[.05,.05,.30,.90])
        p4 = plot(kk,rss[ss,*]/max_rss[ss],color='b',/overplot)

        img_mean = mean(img, dimension=2,/NAN)
        noise_mean = mean(k_scale_w[ss]*rng_img*az_img,dimension=2,/NAN)
        ;Refit to get the residuals
        sk_img = img - k_scale_w[ss]*rng_img*az_img
        sk_mean = mean(sk_img, dimension=2,/NAN)
        nx1 = ss_pixels[0,ss]
        nx2 = ss_pixels[1,ss]
        data_x = nx_idx[nx1:nx2]
        data_y = sk_mean[nx1:nx2]
        fit = poly_fit(data_x,data_y,1,yfit=yfit, yerror = yerror)
        ; PLot Image and Noise Mean Profile
        p5 = plot(data_x, img_mean[nx1:nx2],color='k',title=corename, ytitle='Image and Scaled Noise', axis_style=1,/current, POSITION=[.40,.35,.95,.95])
        p5 = plot(data_x, noise_mean[nx1:nx2],color='r',/overplot)
        ;PLot Image Minus Noise Profile and Fit Line
        p6 = plot(data_x, data_y,color='b',/current,POSITION=[.40,.35,.95,.95],axis_style=0,thick=0.5)
        p6 = plot(data_x, yfit,color='grey',thick=2,/overplot,axis_style=0)
        a6 = axis('y',target=p6,location=[max(p6.xrange),0,0],color='b',textpos=1,tickdir=1,title='Image Minus Scales Noise')
        ;Plot Residuals
        yresid=sk_mean[nx1:nx2] - yfit
        p7 = plot(data_x, yresid,sym='+', xtitle='Range Bin', ytitle='Residual', /current, POSITION=[.40,.05,.95,.30])
      endif
      print, 'Here'

    endfor ; for ss



    print, 'Here'

  endfor  ; Loop over Dataset....





  print, 'Here'




end
