;-----------------------------------------------------
;+
; NAME: SEN_RANGE_NOISE_IMAGE
; PURPOSE:
;  Read noise XML file of Sentinel-1 SAR data and produce the full
;  2D image of the range noise
; CALLING SEQUENCE
;   sen_range_noise_image, file, img
; NOTE: Requires IDL 8.7 for XML_PARSE
; INPUT
;   file       :  XML file with noise information
; OUTPUT
;   img        :  range noise image
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /display    :  Display layout of points
;  nx          :  The size of the array in the range dimension
;  ny          :  The size of the array in the azimuth dimension
;                 By the default, these values are set to the maximum
;                 index vlaues given in the XML file.  We assume that
;                 nx, ny  >= corresponding values in XML file
;  sid         : A 2D array of swath identification values (0 = no data, # = Swath #)                  
;  pngimage    :  Optional PNG image output
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2018 Dec 12 Chris Jackson  Initial Version
;  2019 May 23 Chris Jackson  Fix the logic when the wrong number of beam positions are detected
;  2020 Jan 20 Chris Jackson  Fix the beam logic to properly index when n_beans is 1
;  2020 Apr 07 Chris Jackson. Adjust the finding of beam seams to avoid edge points.  Remove extraneous commented out text at the end of the program.
;  2020 Sep 29 Chris Jackson  Properly return nx ny and sid (beam position array)
; Copyright (C) 2018, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro sen_range_noise_image, file     ,  noise_full_image      ,  $
  help     = help    ,  $
  quiet    = quiet   ,  $
  display  = display ,  $
  nx          = nx          ,  $
  ny          = ny          ,  $
  sid         = sid         ,  $
  pngimage = pngimage,  $
  error    = error

  printflag=1
  if keyword_set(quiet) then printflag=0

  if keyword_set(help) or (n_params(0) lt 1 ) then begin
    print, ' NAME: SEN_RANGE_NOISE_IMAGE
    print, ' PURPOSE:
    print, '  Read noise XML file of Sentinel-1 SAR data and produce the full
    print, '  2D image of the range noise
    print, ' CALLING SEQUENCE
    print, '   sen_range_noise_image, file, img
    print, ' NOTE: Requires IDL 8.7 for XML_PARSE
    print, ' INPUT
    print, '   file       :  XML file with noise information
    print, ' OUTPUT
    print, '   img        :  Azimuth noise image
    print, ' KEYWORDS:
    print, '  /help       :  Print out help
    print, '  /quiet      :  Suppress intermediate print output
    print, '  /display    :  Display layout of points
    print, '  nx          :  The size of the array in the range dimension
    print, '  ny          :  The size of the array in the azimuth dimension
    print, '                 By the default, these values are set to the maximum
    print, '                 index values given in the XML file.  We assume that
    print, '                 nx, ny  >= corresponding values in XML file
    print, '  pngimage    :  Optional PNG image output
    print, '  error=      :  error <0 if cannot read data
    error = -1
    return
  endif

  buffer = 0
  error= 1
  if keyword_set(pngimage) then buffer = 1

  if printflag then print, 'Reading file : ' + file

  sen_read_noise_xml_fast, file, x, /quiet   ,  $
    error    = error
  if error lt 0 then begin
    print, 'Cannot read file: ' + file + ' for range information'
    print, 'Exiting sen_range_noise_image'
    error = -2
    return
  endif


  ;Make zero values in the noiseLut (the boundary pixels) Nans
  ;to avoid interpolation errors
  sz   = size(x.noiserangelut)
  noise_npixels = sz[1]
  noise_nlines = sz[2];
  x.noiserangelut[where(x.noiserangelut eq 0,/null)]=!VALUES.F_NAN

  ;Get the full image size
  nxfile = max(x.pixel)+1L  ;Lines and Pixels are indexed 0 to N-1
  nyfile = max(x.line)+1L

  if printflag then begin
    print, 'Full Image size is (Lines x Pixels): ', string(nyfile, format='(I6)'), ' x ', string(nxfile, format='(I6)')
    print, 'Noise LUT size is (Lines x Pixels): ', string(sz[2], format='(I4)'), ' x ', string(sz[1], format='(I4)')
    print, 'Mean Thermal Noise (uncalibrated): '
    print, '     Linear: ',string( mean(x.noiserangelut,/NAN) ,  format ='(E14.6)')
    print, '        dB : ',string (10*alog10(mean(x.noiserangelut,/NAN)), format ='(F8.3)')
    hlp, x.noiserangelut

  endif

  noisefile=file
  if keyword_set(display) then begin
    tstr= strmid(noisefile,strpos(noisefile,'noise-s1'),73)
    im = image(x.noiserangelut,axis_style=1,xtickdir=1,ytickdir=1,margin=[0.1,0,0.05,0],dimensions=[1000,600],title=tstr,rgb_table=33,buffer=buffer)
    c=colorbar(target=im,rgb_table=33,title='Thermal Noise Power (Uncalibrated)',position=[0.2, 0.1,.8,.2])
    save_str=stress(tstr,'R',0,'xml','png')
    if keyword_set(pngimage) then im.Save, strmid(noisefile,0,strpos(noisefile,'noise-s1'))+save_str
    p1 = plot(x.noiserangelut[*,0],title=tstr,buffer=buffer)
    for i=1, noise_nlines-1 do p2 = plot(x.noiserangelut[*,i]+50*(i-1), color='r',overplot=p1)
  endif

  ;Get out information about the RangeNoiseLUT and image size
  n_beams = 1
  if strcmp(x.mode,'IW') then n_beams=3
  if strcmp(x.mode,'EW') then n_beams=5
  image_nlines  = nyfile
  image_npixels = nxfile

  ;Becuase the noiseLUT is irrgularly spaced (in both pixel and lines) and contains discontinuities at the beam seams
  ;interpolation will be done between a per subswath basis

  ;Find the location of the subswath boundary for each line (this is not constant )
  noise_idx = intarr(2*n_beams,noise_nlines); These will be the boundary indicies
  image_bin = intarr(2*n_beams,noise_nlines); These will be range bins at the boundary
  ;n_beams = 1 -> The index will be just 0 and the noise_npixel-1
  ;               and the image bins will be 0 to image_npixels-1
  ;        >1  -> The index will be the start and end (like n_beams = 1) but then
  ;               for each beam you get end pixel and a start pixel of the next beam
  ;               (these are adjacent)

  for i = 0,noise_nlines-1 do begin
    ;Look for adjacent pixels - this represents the boundary
    ;avoid points near the edges
    pdiff = ts_diff(x.pixel[0:noise_npixels-10,i],1)
    w = where(abs(pdiff) eq 1, count)
    w2 = where(x.pixel[*,i] eq max(x.pixel[*,i]))
    if (count eq n_beams-1) then begin
      ;Build an array of swath start and stop
      idx = [0, w2[0]]
      if (w[0] gt 0) then idx=[0,w,w+1,w2[0]]
      noise_idx[0,i]=idx[sort(idx)] ;Sort the array
      image_bin[0,i]=x.pixel[noise_idx[*,i],i]
    endif else begin
      ;There are one too many seams
      ;So go though the array again taking the largest values in each pair
      w3 = where(abs(ts_diff(x.pixel[w,i],1)) gt 1)
      tdx=[w3, n_elements(w)-1]
      idx=[0,w[tdx],w[tdx]+1,w2]
      if (n_elements(idx) eq 2*n_beams) then begin
        noise_idx[0,i]=idx[uniq(idx, sort(idx))] ;Sort the array
        image_bin[0,i]=x.pixel[noise_idx[*,i],i]
      endif else begin
        print, 'Wrong Number of Seams:', count, '   for line number: ', i
        print, 'Using value of previous line for seam'
        noise_idx[0,i] = noise_idx[*,i-1]
        image_bin[0,i] = image_bin[*,i-1]
      endelse
    endelse
  endfor

  if keyword_set(display) then begin
    tstr= strmid(noisefile,strpos(noisefile,'noise-s1'),73)
    w = where(x.noiserangelut gt 0, count)
    p1 = plot(x.pixel[w], x.line[w],symbol='dot', sym_color='b', linestyle='', sym_size=2, axis_style=1,xtickdir=1,ytickdir=1,title=tstr,buffer=buffer)
    for j = 0,2*n_beams-1 do begin
      p1=plot(image_bin[j,*], x.line[0,*], symbol='dot', sym_color='r',sym_size=5,sym_fill_color='r',sym_filled=1,/overplot)
    endfor
    save_str=stress(tstr,'R',0,'.xml','_lines.png')
    if keyword_set(pngimage) then p1.Save, strmid(noisefile,0,strpos(noisefile,'noise-s1'))+save_str
    ;p1.xrange=[2800, 3200]
  endif

  ;Now loop over the subswaths and interpolate each one individually


  ilen = intarr(noise_nlines)
  noise_full_image=fltarr(image_npixels,image_nlines)
  sid = intarr(image_npixels,image_nlines)

  for i = 0,n_beams-1 do begin

    ;Find the largest bin for each beam
    for j = 0,noise_nlines-1 do  ilen[j] = n_elements(x.pixel[noise_idx[2*i,j]:noise_idx[2*i+1,j],j])
    ss_min_idx = min(noise_idx[2*i:2*i+1,*])
    ss_max_idx = max(noise_idx[2*i:2*i+1,*])
    ss_width = ss_max_idx - ss_min_idx +1

    ss_noise = fltarr(ss_width,noise_nlines)
    ss_pixel = lonarr(ss_width,noise_nlines)
    ss_line  = lonarr(ss_width,noise_nlines)

    for j = 0,(noise_nlines-1) do begin  ;Can probably eliminate the loop - just do assignment
      istart = noise_idx[2*i,j] - ss_min_idx
      ;This always starts at zero.......???
      ss_noise[istart:istart+ilen[j]-1,j] = x.noiserangelut[noise_idx[2*i,j]:noise_idx[2*i+1,j],j]
      ss_pixel[istart:istart+ilen[j]-1,j] = x.pixel[noise_idx[2*i,j]:noise_idx[2*i+1,j],j]
      ss_line[istart:istart+ilen[j]-1,j] = x.line[noise_idx[2*i,j]:noise_idx[2*i+1,j],j]
    endfor

    ss_noise[where(ss_noise eq 0,/null)] = !VALUES.F_NAN

    if keyword_set(display) then begin
      tstr= strmid(noisefile,strpos(noisefile,'noise-s1'),73)+'!CBeam #'+strcompress(string(i+1))
      im2 = image(ss_noise,axis_style=1,xtickdir=1,ytickdir=1,margin=[0.1,0,0.05,0],dimensions=[200,800],title=tstr,rgb_table=33,buffer=buffer)
      c=colorbar(target=im,rgb_table=33,title='Thermal Noise Power (Uncalibrated)',position=[0.2, 0.1,.8,.2])
      ;save_str=stress(tstr,'R',0,'xml','png')
      ;      ;im.Save, strmid(noisefile,0,strpos(noisefile,'noise-s1'))+save_str
      p1 = plot(ss_pixel, ss_line,symbol='dot', sym_color='k', linestyle='', sym_size=5,sym_fill_color='k',sym_filled=1, $
        axis_style=1,xtickdir=1,ytickdir=1,title=tstr, buffer=buffer)
      w = where(ss_noise gt 0, count)
      p1 = plot(ss_pixel[w], ss_line[w],symbol='+', sym_color='r', linestyle='', sym_size=0.5,sym_fill_color='r',sym_filled=1,title=tstr,/overplot)
      ;save_str=stress(tstr,'R',0,'.xml','_lines.png')
      ;p1.Save, strmid(noisefile,0,strpos(noisefile,'noise-s1'))+save_str
    endif

    ss_min_bin = min(image_bin[2*i:2*i+1,*])
    ss_max_bin = max(image_bin[2*i:2*i+1,*])
    ss_sz = size(ss_noise)
    ss_full_bin = ss_max_bin - ss_min_bin+1

    ;Interpolate the subswath
    triangulate, ss_pixel, ss_line, tr, bb
    xout = indgen(ss_full_bin)
    yout = indgen(image_nlines)
    noise_ss_image = griddata(ss_pixel-ss_min_bin, ss_line, ss_noise,/linear,Triangles=tr, /grid,xout=xout,yout=yout)

    ;Apply sub-swath scaling - See Park Paper - just a place holder for now
    ;As we just apply his derived value.  But they seem to improve the results
;    if strcmp(x.mode,'EW') then begin
;      print, 'Applying sub-swath scaling
;      KN = [1.363,0.991,1.043,0.990,0.932];
;      noise_ss_image = temporary(noise_ss_image)*KN[i]
;    endif
    ;im3 = image(noise_ss_image,axis_style=1,xtickdir=1,ytickdir=1,margin=[0.1,0,0.05,0],title=tstr,rgb_table=33)
    ;Copy the fully intrpolated subswath into the full array
    for j = 0, noise_nlines-2 do begin
      x1 = image_bin[2*i,j]
      x2 = image_bin[2*i+1,j]
      y1 = x.line[0,j]
      y2 = x.line[0,j+1]
      noise_full_image[x1:x2,y1:y2] = noise_ss_image[x1-ss_min_bin:x2-ss_min_bin,y1:y2];
      sid[x1:x2,y1:y2] = i+1;
    endfor


    ;Extroplate to full in NAN areas

    print, 'Finish Beam #'+strcompress(string(i+1))

  endfor

  ;Get the full image size
  nx = nxfile  ;Lines and Pixels are indexed 0 to N-1
  ny = nyfile 


end
