;;-----------------------------------------------------
;+
; NAME: SEN_READ_CAL_XML
; PURPOSE:
;  Read calibration XML file of Sentinel-1 SAR data
; CALLING SEQUENCE
;   sen_read_cal_xml, file, x 
; INPUT
;   file       :  XML file with calibration information 
; OUTPUT
;   x          :  Structure with calibration information.
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /display    :  Display contents of x
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo. Creation
;  2013 Oct 17 Frank Monaldo.  Note that the caibration of the GRD data requires an constant offset.
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_read_cal_xml, file, x,  help = help, quiet=quiet, error=error, display=display

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_READ_CAL_XML
  print, ' PURPOSE:
  print, '  Read calibration XML file of Sentinel-1 SAR data
  print, ' CALLING SEQUENCE
  print, '   sen_read_cal_xml, file, x 
  print, ' INPUT
  print, '   file       :  XML file with calibration information 
  print, ' OUTPUT
  print, '   x          :  Structure with calibration information.
  print, ' KEYWORDS:
  print, '  /help       :  Print out help
  print, '  /quiet      :  Suppress intermediate print output
  print, '  /display    :  Display contents of x
  print, '  error=      :  error <0 if cannot read data
  error = -1
  return
endif

error= 0

if printflag then print, 'Reading xml information with sen_read_cal_xml'

if not file_test(file) then begin
  print, 'Cannot find file: ' + file + ' Exiting sen_read_cal_xml'
  error = -1
  return
endif

if printflag then print, 'Reading file: ' + file 
s= getfile(file)
n= n_elements(s)

if printflag then print, 'Parsing:      ' + file 
;
; Declaring x structure
;

    
x={ missionId                   :  '' ,  $
    productType                 :  '' ,  $
    polarisation                :  '' ,  $
    mode                        :  '' ,  $
    swath                       :  '' ,  $
    startTime                   :  '' ,  $
    stopTime                    :  '' ,  $
    absoluteOrbitNumber         :  '' ,  $
    missionDataTakeId           :  '' ,  $
    imageNumber                 :  '' ,  $
    absoluteCalibrationConstant :  ''    }

; ----------------------------------------------------
;                   ADSHEADER INFORMATION
; ---------------------------------------------------

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'adsHeader')

w=where(index ge 0,count)


if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i]
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'missionId'                 : x.missionId               = getwrd(ss,1)
    'productType'               : x.productType             = getwrd(ss,1)
    'polarisation'              : x.polarisation            = getwrd(ss,1)
    'mode'                      : x.mode                    = getwrd(ss,1)
    'swath'                     : x.swath                   = getwrd(ss,1)
    'startTime'                 : x.startTime               = getwrd(ss,1)
    'stopTime'                  : x.stopTime                = getwrd(ss,1)
    'absoluteOrbitNumber'       : x.absoluteOrbitNumber     = getwrd(ss,1)
    'missionDataTakeId'         : x.missionDataTakeId       = getwrd(ss,1)
    'imageNumber'               : x.imageNumber             = getwrd(ss,1)
    ELSE                        : ; No operation
    endcase
  endfor
endif
;
; ----------------------------------------------------
;             CALIBRATION INFORMATION
; ----------------------------------------------------
;
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'calibrationInformation')
w=where(index ge 0,count)
if count ge 2 then begin
  istart = w[0]
  iend   = w[1]
  for i=istart, iend do begin
    ss  = s[i] 
    ss  = repchr( ss, '<', ' ' )
    ss  = repchr( ss, '>', ' ' )
    ss  = repchr( ss, '/', ' ' )
    key = getwrd(ss,0)
    if key eq 'absoluteCalibrationConstant'  then x.absoluteCalibrationConstant = getwrd(ss,1)
  endfor
endif
;
; ----------------------------------------------------
;             CALIBRATION VECTOR LIST
; ----------------------------------------------------
;
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'calibrationVectorList')
w=where(index ge 0,count)
if count ge 2 then begin
  istart = w[0]
  iend   = w[1]
  count =  getwrd( s[istart], 1, del='"' )
  ny    =  long( count )
endif else begin
  print, 'Cannot find the calibration vector'
  print, 'Exiting sen_read_cal_xml...;
  error = -1
  return
endelse

for i=0L, n-1 do index[i]= strpos(s[i], '<pixel')
w=where(index ge 0, count)
if count ge 2 then begin
  nx =  getwrd( s[ w[0] ], 1, del='"' )
endif else begin
  print, 'Cannot find the calibration vector'
  print, 'Exiting sen_read_cal_xml...;
  error = -1
  return
endelse


azimuthTime  =  strarr( ny )
line         =  lonarr( nx, ny )
pixel        =  lonarr( nx, ny )
sigmaNought  =  fltarr( nx, ny ) 
betaNought   =  fltarr( nx, ny )
gamma        =  fltarr( nx, ny )
dn           =  fltarr( nx, ny )

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<calibrationVector>')
w0 = where(index ge 0, count0)
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '</calibrationVector>')
w1 = where(index ge 0, count1)
n  = min( [count0, count1] )
;
; Loop though calibration vectors
;
for j=0, n-1 do begin
  ; 
  ; Loop through lines in a particular calibration vector
  for k = w0[j], w1[j] do begin
    ss    = s[k]
    ss    = repchr( ss, '<', ' ' )
    ss    = repchr( ss, '>', ' ' )
    ss    = repchr( ss, '/', ' ' )
    ss    = repchr( ss, '"', ' ' )
    sflag =  -1
    if strpos(ss, 'line'        )  ge 0 then sflag = 0
    if strpos(ss, 'pixel'       )  ge 0 then sflag = 1
    if strpos(ss, 'sigmaNought' )  ge 0 then sflag = 2
    if strpos(ss, 'betaNought'  )  ge 0 then sflag = 3
    if strpos(ss, 'gamma'       )  ge 0 then sflag = 4
    if strpos(ss, 'dn'          )  ge 0 then sflag = 5
    if strpos(ss, 'azimuthTime' )  ge 0 then sflag = 6
    case sflag of
    0:      begin
            ss    = repchr( ss, 'line', ' ' )
            ln    =  long(  getwrd( ss, 1) )
            for i = 0, nx - 1 do line[ *, j ] = ln
            end 
    1:      begin
              for i = 0, nx - 1 do  pixel[ i, j ]      = long(  getwrd( ss, i + 3) ) 
            end 
    2:      begin
              for i = 0, nx - 1 do sigmaNought[ i, j ] = float( getwrd( ss, i + 3) ) 
            end
    3:      begin
              for i = 0, nx - 1 do betaNought[ i, j ]  = float( getwrd( ss, i + 3) ) 
            end 
    4:      begin
              for i = 0, nx - 1 do gamma[ i, j ]       = float( getwrd( ss, i + 3) ) 
            end 
    5:      begin
              for i = 0, nx - 1 do dn[ i, j ]          = float( getwrd( ss, i + 3) ) 
            end 
    6:      begin
            azimuthTime[j]                             =        getwrd( ss, 1 ) 
            end 
    ELSE  : ; No operation
    endcase
  endfor
endfor

x= create_struct(x, 'line'           ,   line          , $
                    'pixel'          ,   pixel         , $
                    'sigmaNought'    ,   sigmaNought   , $
                    'betaNought'     ,   betaNought    , $
                    'gamma'          ,   gamma         , $
                    'dn'             ,   dn            , $
                    'azimuthTime'    ,   azimuthTime    )
                    
if printflag then hlp, line, pixel, sigmaNought, betaNought, gamma, dn

end
