;-----------------------------------------------------
;+
; NAME: SEN_READ_IMAGE'
; PURPOSE:'
;  Read image file of Sentinel-1 SAR data'
; CALLING SEQUENCE'
;   sen_read_image, file, img' 
; INPUT'
;   file       :  TIFF image file'
; OUTPUT'
;   img        :  Image array read for TIFF file.'
; KEYWORDS:'
;  /help       :  Print out help'
;  /quiet      :  Suppress intermediate print output'
;  /display    :  Display image';  
;  nxmin       :  Minimum x-index for SLC file'
;  nymin       :  Minimum y-index SLC file'
;  nxmax       :  Maximum x-index for SLC file'
;  nymax       :  Maximum y-index SLC file' 
;  nlines	     :  Read simultaneously a number of lines from file 
;  /comp       :  Return complex arrays'
;  imgi        :  If /comp set, img will be real part'
;                 and imgi contains the imaginary part'
;  error       :  error <0 if cannot read data'
;                 -1 Exit help'
;                 -2 Cannot find or query file'   
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo.   Creation
;  2013 Oct 17 Frank Monaldo.   For SLC complex data, the absolute
;                               value is returned
;  2014 Sep 05 Frank Monaldo.   Corrected f[i] to file for SLC data
;  2014 Sep 08 Frank Monaldo.   Add ability to read one line at a time
;  2019 Apr 17 Frank Monaldo.   Add ability to read a subset of the
;                               SLC imagery
;  2019 Apr 18 Frank Monaldo.   Added ability to check endian
;  2019 Apr 22 Alejandro Egido. Added ability to read multiple lines simultaneously
;  2019 May 03 Frank Monaldo.   Remove write output image.  This program just
;                               returns the img (real and imaginary)
; Copyright (C) 2013-2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_read_image, file,  img          , $
                    display  =  display , $
                    nxmin    =  nxmin   , $
                    nymin    =  nymin   , $
                    nxmax    =  nxmax   , $
                    nymax    =  nymax   , $
                    nlines   =  nlines  , $
                    imgi     =  imgi    , $
                    comp     =  comp    , $
                    help     =  help    , $
                    quiet    =  quiet   , $ 
                    error    =  error

printflag=1
if keyword_set(quiet) then printflag=0
if n_elements(ave) eq 0 then ave = 1
ave  =  max( [ ave, 1 ] )

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print,' NAME: SEN_READ_IMAGE'
  print,' PURPOSE:'
  print,'  Read image file of Sentinel-1 SAR data'
  print,' CALLING SEQUENCE'
  print,'   sen_read_image, file, img' 
  print,' INPUT'
  print,'   file       :  TIFF image file'
  print,' OUTPUT'
  print,'   img        :  Image array read for TIFF file.'
  print,' KEYWORDS:'
  print,'  /help       :  Print out help'
  print,'  /quiet      :  Suppress intermediate print output'
  print,'  /display    :  Display image';  
  print,'  nxmin       :  Minimum x-index for SLC file'
  print,'  nymin       :  Minimum y-index SLC file'
  print,'  nxmax       :  Maximum x-index for SLC file'
  print,'  nymax       :  Maximum y-index SLC file' 
  print,'  nlines	     :  Read simultaneously a number of lines from file' 
  print,'  /comp       :  Return complex arrays'
  print,'  imgi        :  If /comp set, img will be real part'
  print,'                 and imgi contains the imaginary part'
  print,'  error       :  error <0 if cannot read data'
  print,'                 -1 Exit help'
  print,'                 -2 Cannot find or query file'   
  return
endif

error= 0


if printflag then print, 'Reading image information with sen_read_image'

if not file_test(file) then begin
  if printflag then print, 'Cannot find file: ' + file+ ' Exiting sen_read_image'
  error = -2
  return
endif

if printflag then print, 'Querying file: ' + file 

res   = query_tiff(file, info)
if res eq 0 then begin
  if printflag then begin
     print, 'Error in reading        : ' + file
     print, 'Query_tiff returns value: ' + string(res)
     print, 'Exiting                 : sen_read_image.pro'
  endif
  error = -2
  return
endif
;
; Read first two bytes to determine byter order
;
tiff_byte_order = 1 ; Motorola, 0 = Intel

close, 1
openr, 1, file
b  =  bytarr(2)
readu, 1, b
if string(b) ne 'MM' then tiff_byte_order= 0
close, 1

if printflag then begin
   print, 'Reading file         :  ' + file
   print, 'Number of x-elements : ' + strcompress( string(info.dimensions[0], format='(-I10)'))
   print, 'Number of y-elements : ' + strcompress( string(info.dimensions[1], format='(-I10)')) 
   print, 'TIFF Byte Order      :  ' + string(b) 
   print, 'TIFF Byte Order Flag : ' + strcompress(string(tiff_byte_order))
endif

if strpos(strupcase(file),'SLC') lt 0 then begin
  if printflag then begin
    print, 'Reading a file as real data'
  endif
  img =  read_tiff( file )
endif else begin
  ;
  ; Reading complex data
  ;
  nx         =  info.dimensions[0] - 1
  ny         =  info.dimensions[1] - 1
  if n_elements(nxmax) ne 0 then begin
    nxmax = min( [nx, nxmax] )
  endif else begin
    nxmax = nx
  endelse
  if n_elements(nymax) ne 0 then begin
    nymax = min( [ny, nymax] )
  endif else begin
    nymax = ny
  endelse
  if n_elements(nxmin) ne 0 then begin
    nxmin = max( [ 0L, nxmin] )
  endif else begin
    nxmin = 0L
  endelse
  if n_elements(nymin) ne 0 then begin
    nymin = max( [ 0L, nymin] )
  endif else begin
    nymin = 0L
  endelse

  nBlock = isa( nBlock ) ? nBlock : 1
  
  nnx        =  nxmax - nxmin + 1L
  nny        =  nymax - nymin + 1L
  img        =  0
  imgi       =  0
  if keyword_set(comp) then begin
     img   = intarr(nnx,nny)
     imgi  = intarr(nnx,nny)
     cflag = 1
     if printflag then begin
       print, 'Setting cflag        :  1'
       print, 'Reading data as complex two arrays...'
     endif
  endif else begin
     img   = fltarr(nnx,nny)
     cflag      =  0   ; Return only absolute value
     if printflag then begin
       print, 'Setting cflag        :  0'
       print, 'Reading data as complex into one array...'
     endif
  endelse
  
  jstep      =  nny / 10
  
  ;
  ; Looping
  ;

  for j=nymin, nymax-nlines, nlines do begin
	indx = [j, j+nlines - 1] - nymin
    ln   =  read_tiff( file, sub_rect=[nxmin, j, nnx, nlines] )
    if tiff_byte_order then begin
      re  =  fix(      ishft(ln,-16)     )
      im  =  fix( ishft(ishft(ln,16),-16)) 
    endif else begin
      re  =  swap_endian(fix(       ishft(ln,-16)      ))
      im  =  swap_endian(fix( ishft(ishft(ln, 16),-16 ))) 
    endelse

    if cflag eq 0 then begin
      img[*,indx[0]:indx[1]]   =  abs(complex(re, im ))
    endif else begin
      img[*,indx[0]:indx[1] ] =  re 
      imgi[*,indx[0]:indx[1] ] =  im
    endelse
    if (j mod jstep) eq 0 then begin
       print, 'Reading line: ' + string(j) + ' to' + string(nymax)  + ' ' + string( mean(img[*,j-nymin]) )
    endif
  endfor
  if printflag then begin
     help, img
     print, mean(img)
     if cflag eq 1 then begin
       help, imgi
       print, mean(imgi)
     endif
  endif
endelse

if keyword_set(display) then begin
  if cflag eq 0 then begin
    im  =  image(  img                , $
              axis_style    = 1       , $
              xcolor        = 'white' , $
              ycolor        = 'white' , $
              xtext_color   = 'black' , $
              ytext_color   = 'black' , $
              xtitle        = 'Pixel' , $
              ytitle        = 'Line' )
  endif else begin
    im  =  image(  abs(complex(img,imgi)) , $
              axis_style    = 1       , $
              xcolor        = 'white' , $
              ycolor        = 'white' , $
              xtext_color   = 'black' , $
              ytext_color   = 'black' , $
              xtitle        = 'Pixel' , $
              ytitle        = 'Line' )
  endelse
endif

end
