;;-----------------------------------------------------
;+
; NAME: SEN_READ_NOISE_XM_FAST
; PURPOSE:
;  Read noise XML file of Sentinel-1 SAR data. This version uses IDL XML call
; CALLING SEQUENCE
;   sen_read_noise_xml_fast, file, x 
; NOTE: Require IDL 8.7 for XML_PARSE
; INPUT
;   file       :  XML file with noise information 
; OUTPUT
;   x          :  Structure with noise information.
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /azimuth    ;  Return azimuth noise (range returned by default)
;  For older file version only the range lut is returned
;  /display    :  display noise image
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo. Creation
;  2016 Apr 14 Frank Monaldo. Correct NoiseLut variable name string
;  2016 Apr 15 Frank Monaldo. Correct printout statement  
;  2018 Aug 06 Frank Monaldo. Minor annotation changes
;  2018 Aug 07 Frank Monaldo. Look for new tags to read noise floor data
;                             for files processed after 2018-03-10. 
;                             Old tags are noiseVector      and noiseLut.
;                             New tags are noiseRangeVector and noiseRangeLut.
;  2018 Dec 12 Frank Monaldo. Use IDL XML parser and read range and azimuth noise vectors
;                             in new noise file version. 
;  2020 Apr 10 Chris Jackson. If version is "old" and azimuth keyword is set return error = -2                            
; Copyright (C) 2013-2018, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_read_noise_xml_fast, file, x,  help = help, quiet=quiet, error=error, $
                                   azimuth  =  azimuth  , $
                                   display  =  display

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_READ_NOISE_XM_FAST
  print, ' PURPOSE:
  print, '  Read noise XML file of Sentinel-1 SAR data. This version uses IDL XML call
  print, ' CALLING SEQUENCE
  print, '   sen_read_noise_xml_fast, file, x 
  print, ' NOTE: Require IDL 8.7 for XML_PARSE
  print, ' INPUT
  print, '   file       :  XML file with noise information 
  print, ' OUTPUT
  print, '   x          :  Structure with noise information.
  print, ' KEYWORDS:
  print, '  /help       :  Print out help
  print, '  /quiet      :  Suppress intermediate print output
  print, '  /azimuth    ;  Return azimuth noise (range returned by default)
  print, '  For older file version only the range lut is returned
  print, '  error=      :  error <0 if cannot read data
  print, '  /display    :  display noise image
  error = -1
  return
endif


error= 1

if printflag then print, 'Reading xml information with sen_read_noise_xml_fast'

if not file_test(file) then begin
  print, 'Cannot find file: ' + file + ' Exiting sen_read_noise_fast_xml'
  error = -2
  return
endif

if printflag then print, 'Reading file: ' + file 
x         = 0
x_hash    = xml_parse(file)
x_struct  = x_hash.ToStruct(/recursive)
hdr       = x_struct.noise.adsheader
; ----------------------------------------------------
;                   ADSHEADER INFORMATION
; ---------------------------------------------------
; Header information is the same in the new and old versions of the noise
; file
;
x={ missionId           : hdr.missionId           ,  $
    productType         : hdr.productType         ,  $
    polarisation        : hdr.polarisation        ,  $
    mode                : hdr.mode                ,  $
    swath               : hdr.swath               ,  $
    startTime           : hdr.startTime           ,  $
    stopTime            : hdr.stopTime            ,  $
    absoluteOrbitNumber : hdr.absoluteOrbitNumber ,  $
    missionDataTakeId   : hdr.missionDataTakeId   ,  $
    imageNumber         : hdr.imageNumber            }


;
;
; Checking for noise file version
;
version = 'old'
z= x_struct.noise
;
; If this is not a old version then
; tag_value(z,'noisevectorlist') = ''
;
if n_elements(tag_value(z,'noisevectorlist') ) eq 1  and $
     typename(tag_value(z,'noisevectorlist') ) ne 'ANONYMOUS' then version = 'new'

if version eq 'old' then begin
  ; ----------------------------------------------------
  ;             Noise AZIMUTH VECTOR LIST
  ; ----------------------------------------------------  
  ;There is no azimuth noise information in the "old" (pre-March 2018) files
  ;if requesting azimuth noise - set error = -2 and return
  if keyword_set(azimuth) then begin
    error = -2
    return
  endif
  
  
  ; ----------------------------------------------------
  ;             Noise RANGE VECTOR LIST
  ; ----------------------------------------------------
  y                = x_struct.noise.noisevectorlist.noisevector
  ny               = n_elements(y)
  azimuthtime      = strarr(ny)
  ; 
  ; compute maximum number pxcount
  ;
  pxcount          = lonarr( ny )
  for i = 0,  ny - 1 do begin
    z              = y[i]
    a              = z.tostruct(/recursive)
    pxcount[i]     = a.pixel._count
  endfor
  nx               = max( pxcount )
  noise            = fltarr( nx, ny )
  lines            = lonarr( nx, ny )
  pixels           = lonarr( nx, ny )
  for i = 0,  ny - 1 do begin
    z              = y[i]
    a              = z.tostruct(/recursive)
    azimuthtime[i] = a.azimuthtime
    line           = long(a.line)
    pxstr          = a.pixel._text
    pxcount        = a.pixel._count
    noisestr       = a.noiselut._text
    for j = 0, pxcount-1 do begin
       pixels[j,i] = long(  getwrd( pxstr   , j ) )
       lines[j,i]  = line
       noise[j,i]  = float( getwrd( noisestr, j ) )
    endfor
  endfor 
  range   =  create_struct('line'          ,   lines          , $
                           'pixel'         ,   pixels         , $
                           'noiserangelut' ,   noise          , $
                           'azimuthtime'   ,   azimuthtime   )
  x= create_struct(x, range)
  if keyword_set(display) then begin
    set_plot, 'x'
    tvscl, noise
    stop
  endif
  return
endif
;
; If the azimuth keyword is not selected, read the 
; noise range information
;
if not keyword_set(azimuth) then begin
  ; ----------------------------------------------------
  ;             Noise RANGE VECTOR LIST
  ; ----------------------------------------------------
  y                = x_struct.noise.NOISERANGEVECTORLIST.NOISERANGEVECTOR
  ny               = n_elements(y)
  azimuthtime      = strarr(ny)
  ; 
  ; compute maximum number pxcount
  ;
  pxcount          = lonarr( ny )
  for i = 0,  ny - 1 do begin
    z              = y[i]
    a              = z.tostruct(/recursive)
    pxcount[i]     = a.pixel._count
  endfor
  nx               = max( pxcount )
  noise            = fltarr( nx, ny )
  lines            = lonarr( nx, ny )
  pixels           = lonarr( nx, ny )
  for i = 0,  ny - 1 do begin
    z              = y[i]
    a              = z.tostruct(/recursive)
    azimuthtime[i] = a.azimuthtime
    line           = long(a.line)
    pxstr          = a.pixel._text
    pxcount        = a.pixel._count
    noisestr       = a.NOISERANGELUT._text
    for j = 0, pxcount-1 do begin
       pixels[j,i] = long(  getwrd( pxstr   , j ) )
       lines[j,i]  = line
       noise[j,i]  = float( getwrd( noisestr, j ) )
    endfor
  endfor 
  range   =  create_struct('line'          ,   lines          , $
                           'pixel'         ,   pixels         , $
                           'noiserangelut' ,   noise  , $
                           'azimuthTime'   ,   azimuthTime     )
  x= create_struct(x, range)
  if keyword_set(display) then begin
    set_plot, 'x'
    tvscl, noise
    stop
  endif
  return
endif



; ----------------------------------------------------
;             Noise AZIMUTH VECTOR LIST
; ----------------------------------------------------
z                 =x_struct.noise.NOISEAZIMUTHVECTORLIST

if fix(x_struct.noise.NOISEAZIMUTHVECTORLIST._count) eq 0 then begin
  ;
  ; There is no azimuth information noise
  ;
  error = -2
  return
endif
y                = x_struct.noise.NOISEAZIMUTHVECTORLIST.NOISEAZIMUTHVECTOR
ny               = n_elements(y)
azimuth          = create_struct( 'numberofsubswaths', ny )
for i = 0,  ny - 1 do begin
  z                   = y[i]
  a                   = z.tostruct(/recursive)
  swath               = a.swath
  firstazimuthline    = long( a.firstazimuthline )
  firstrangesample    = long( a.firstrangesample )
  lastazimuthline     = long( a.lastazimuthline  )
  lastrangesample     = long( a.lastrangesample  )  
  line                = a.line
  lnstr               = a.line._text
  lncount             = long(a.line._count)
  noisestr            = a.NOISEAZIMUTHLUT._text
  lines               = lonarr(lncount)
  noise               = fltarr(lncount)
  for j = 0, lncount-1 do begin 
    lines[j]   = long(  getwrd( lnstr   , j ) )
    noise[j]   = float( getwrd( noisestr, j ) )
  endfor
  name                = 'subswath' + string( i, format = '(I3.3)' )
  swathstruct         = create_struct( 'count', lncount ,  'swath'           , swath           , $
                                                           'firstazimuthline', firstazimuthline, $
                                                           'firstrangesample', firstrangesample, $
                                                           'lastazimuthline' , lastazimuthline , $
                                                           'lastrangesample' , lastrangesample , $
                                                           'line'            , lines           , $
                                                           'noiseazimuthlut' , noise           )

  azimuth  = CREATE_STRUCT( azimuth, name, swathstruct) 
endfor
if keyword_set(display) then begin
  for i = 0, ny-1 do begin
    b     = 'subswath' + string( i, format = '(I3.3)' )
    a     = tag_value(azimuth, b)
    lnmax = 0
    ymax  = 0
    lnmax = max( [lnmax, a.line           ] )
    ymax  = max( [ymax , a.noiseazimuthlut] )
  endfor
  for i = 0, ny-1 do begin
    b     = 'subswath' + string( i, format = '(I3.3)' )
    a     = tag_value(azimuth, b)
    if i eq 0 then begin
      plot, a.line, a.noiseazimuthlut, /xstyle, xrange= [0, lnmax], /ystyle, yrange= [0, ymax*1.10]
    endif else begin
      oplot, a.line, a.noiseazimuthlut
    endelse  
    print,a.firstrangesample
  stop
  endfor
endif

                         
x= create_struct(x, 'azimuth', azimuth )







end
