;;-----------------------------------------------------
;+
; NAME: SEN_READ_XML
; PURPOSE:
;  Read main XML file of Sentinel-1 SAR data
; CALLING SEQUENCE
;   sen_read_xml, file, x 
; INPUT
;   file       :  XML file with general information 
; OUTPUT
;   x          :  Structure with general information.
; KEYWORDS:
;  /help       :  Print out help
;  /quiet      :  Suppress intermediate print output
;  /display    :  Display contents of x
;  error=      :  error <0 if cannot read data
; MODIFICATION HISTORY:
;  2013 Sep 30 Frank Monaldo. Creation
; Copyright (C) 2013, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_read_xml, file, x,  help=help, quiet=quiet, error=error, display=display

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: SEN_READ_XML
  print, ' PURPOSE:
  print, '  Read main XML file of Sentinel-1 SAR data
  print, ' CALLING SEQUENCE
  print, '   sen_read_xml, file, x 
  print, ' INPUT
  print, '   file       :  XML file with general information 
  print, ' OUTPUT
  print, '   x          :  Structure with general information.
  print, ' KEYWORDS:
  print, '  /help       :  Print out help
  print, '  /quiet      :  Suppress intermediate print output
  print, '  error=      : error <0 if cannot read data
  print,  ' /display    :  Display contents of x
  error = -1
  return
endif

error= 0

if printflag then print, 'Reading xml information with sen_read_xml'

if not file_test(file) then begin
  print, 'Cannot find file: ' + file+ ' Exiting sen_read_xml''
  error = -1
  return
endif

if printflag then print, 'Reading file: ' + file 
s= getfile(file)
n= n_elements(s)

if printflag then print, 'Parsing:      ' + file 
;
; Declaring x structure
;

    
x={ missionId                 :  '' ,  $
    productType               :  '' ,  $
    polarisation              :  '' ,  $
    mode                      :  '' ,  $
    swath                     :  '' ,  $
    startTime                 :  '' ,  $
    stopTime                  :  '' ,  $
    absoluteOrbitNumber       :  '' ,  $
    missionDataTakeId         :  '' ,  $
    imageNumber               :  '' ,  $
    productQualityIndex       :  '' ,  $
    azimuthTime               :  '' ,  $
    outputDataMean_re         :  '' ,  $
    outputdatamean_im         :  '' ,  $
    outputDataStdDev_re       :  '' ,  $
    outputDataStdDev_im       :  '' ,  $
    pass                      :  '' ,  $
    platformHeading           :  '' ,  $
    projection                :  '' ,  $
    radarFrequency            :  '' ,  $
    productFirstLineUtcTime   :  '' ,  $
    productLastLineUtcTime    :  '' ,  $
    ascendingNodeTime         :  '' ,  $
    anchorTime                :  '' ,  $
    slantRangeTime            :  '' ,  $
    pixelValue                :  '' ,  $
    outputPixels              :  '' ,  $
    rangePixelSpacing         :  '' ,  $
    azimuthPixelSpacing       :  '' ,  $
    azimuthTimeInterval       :  '' ,  $
    numberOfSamples           :  '' ,  $
    numberOfLines             :  '' ,  $
    incidenceAngleMidSwath    :  '' ,  $
    ellipsoidName             :  '' ,  $
    ellipsoidSemiMajorAxis    :  '' ,  $
    ellipsoidSemiMinorAxis    :  '' ,  $
    range_looks               :  '' ,  $
    azimuth_looks             :  ''   }

; ----------------------------------------------------
;                   ADSHEADER INFORMATION
; ---------------------------------------------------

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'adsHeader')

w=where(index ge 0,count)


if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i]
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'missionId'                 : x.missionId               = getwrd(ss,1)
    'productType'               : x.productType             = getwrd(ss,1)
    'polarisation'              : x.polarisation            = getwrd(ss,1)
    'mode'                      : x.mode                    = getwrd(ss,1)
    'swath'                     : x.swath                   = getwrd(ss,1)
    'startTime'                 : x.startTime               = getwrd(ss,1)
    'stopTime'                  : x.stopTime                = getwrd(ss,1)
    'absoluteOrbitNumber'       : x.absoluteOrbitNumber     = getwrd(ss,1)
    'missionDataTakeId'         : x.missionDataTakeId       = getwrd(ss,1)
    'imageNumber'               : x.imageNumber             = getwrd(ss,1)
    ELSE                        : ; No operation
    endcase
  endfor
endif
; ----------------------------------------------------
;             QUALITY INFORMATION
; ---------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'qualityInformation')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss  = s[i] 
    ss  = repchr( ss, '<', ' ' )
    ss  = repchr( ss, '>', ' ' )
    ss  = repchr( ss, '/', ' ' )
    key = getwrd(ss,0)
    case key of 
    'productQualityIndex'     : x.productQualityIndex = getwrd(ss,1)
    'azimuthTime'             : x.azimuthTime         = getwrd(ss,1)
    'outputDataMean'          : begin
                                  for ii=1, 2 do begin
                                    ss1  = s[i+ii]
                                    ss1  = repchr( ss1, '<', ' ' )
                                    ss1  = repchr( ss1, '>', ' ' )
                                    ss1  = repchr( ss1, '/', ' ' )
                                    key1 = getwrd( ss1, 0)
                                    if key1 eq 're' then x.outputDataMean_re= getwrd(ss1,1)
                                    if key1 eq 'im' then x.outputDataMean_im= getwrd(ss1,1)
                                  endfor
                                end
    'outputDataStdDev'        : begin
                                  for ii=1, 2 do begin
                                    ss1  = s[i+ii]
                                    ss1  = repchr( ss1, '<', ' ' )
                                    ss1  = repchr( ss1, '>', ' ' )
                                    ss1  = repchr( ss1, '/', ' ' )
                                    key1 = getwrd( ss1, 0)
                                    if key1 eq 're' then x.outputDataStdDev_re= getwrd(ss1,1)
                                    if key1 eq 'im' then x.outputDataStdDev_im= getwrd(ss1,1)
                                  endfor
                                end
    ELSE                        : ; No operation
    endcase
  endfor
endif
;------------------------------------------------------------------------------
;                      GENERAL ANNOTATION
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'generalAnnotation')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'radarFrequency'            : x.radarFrequency           =   getwrd(ss,1)    
    'pass'                      : x.pass                     =   getwrd(ss,1)
    'platformHeading'           : x.platformHeading          =   getwrd(ss,1)
    'projection'                : x.projection               =   getwrd(ss,1)
    'productFirstLineUtcTime'   : x.productFirstLineUtcTime  =   getwrd(ss,1)
    'productLastLineUtcTime'    : x.productLastLineUtcTime   =   getwrd(ss,1)
    'ascendingNodeTime'         : x.ascendingNodeTime        =   getwrd(ss,1)
    'anchorTime'                : x.anchorTime               =   getwrd(ss,1)
    'slantRangeTime'            : x.slantRangeTime           =   getwrd(ss,1)
    'pixelValue'                : x.pixelValue               =   getwrd(ss,1)
    'outputPixels'              : x.outputPixels             =   getwrd(ss,1)
    'rangePixelSpacing'         : x.rangePixelSpacing        =   getwrd(ss,1)
    'azimuthPixelSpacing'       : x.azimuthPixelSpacing      =   getwrd(ss,1)
    'azimuthTimeInterval'       : x.azimuthTimeInterval      =   getwrd(ss,1)
    'numberOfSamples'           : x.numberOfSamples          =   getwrd(ss,1)
    'numberOfLines'             : x.numberOfLines            =   getwrd(ss,1)
    'incidenceAngleMidSwath'    : x.incidenceAngleMidSwath   =   getwrd(ss,1)
     ELSE                       : ; No operation
    endcase
  endfor
endif 

;------------------------------------------------------------------------------
;                      IMAGE ANNOTATION
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'imageAnnotation')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'productFirstLineUtcTime'   : x.productFirstLineUtcTime  =   getwrd(ss,1)
    'productLastLineUtcTime'    : x.productLastLineUtcTime   =   getwrd(ss,1)
    'ascendingNodeTime'         : x.ascendingNodeTime        =   getwrd(ss,1)
    'anchorTime'                : x.anchorTime               =   getwrd(ss,1)
    'slantRangeTime'            : x.slantRangeTime           =   getwrd(ss,1)
    'pixelValue'                : x.pixelValue               =   getwrd(ss,1)
    'outputPixels'              : x.outputPixels             =   getwrd(ss,1)
    'rangePixelSpacing'         : x.rangePixelSpacing        =   getwrd(ss,1)
    'azimuthPixelSpacing'       : x.azimuthPixelSpacing      =   getwrd(ss,1)
    'azimuthTimeInterval'       : x.azimuthTimeInterval      =   getwrd(ss,1)
    'numberOfSamples'           : x.numberOfSamples          =   getwrd(ss,1)
    'numberOfLines'             : x.numberOfLines            =   getwrd(ss,1)
    'incidenceAngleMidSwath'    : x.incidenceAngleMidSwath   =   getwrd(ss,1)
    'ellipsoidName'             : x.ellipsoidName            =   getwrd(ss,1)           
    'ellipsoidSemiMajorAxis'    : x.ellipsoidSemiMajorAxis   =   getwrd(ss,1)
    'ellipsoidSemiMinorAxis'    : x.ellipsoidSemiMinorAxis   =   getwrd(ss,1)
     ELSE                        : ; No operation
    endcase
  endfor
endif 
;------------------------------------------------------------------------------
;                      LOOKS
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'rangeProcessing')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    if key eq 'numberOfLooks' then x.range_looks  =   getwrd(ss,1)
  endfor
endif 

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'azimuthProcessing')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    if key eq 'numberOfLooks' then x.azimuth_looks  =   getwrd(ss,1)
  endfor
endif 

;------------------------------------------------------------------------------
;                      GEOLOCATION ARRAYS
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<geolocationGridPoint>')
w0=where(index ge 0,count)
for i=0L, n-1 do index[i]= strpos(s[i], '</geolocationGridPoint>')
w1=where(index ge 0,count)
if count gt 0 then begin
  longitude      = fltarr(count)
  latitude       = fltarr(count)
  incidenceAngle = fltarr(count)
  line           = lonarr(count)
  pixel          = lonarr(count)
  line           = line - 1
  for i=0, count-1 do begin      
    flongitude      = 0
    flatitude       = 0
    fincidenceAngle = 0
    fline           = 0
    fpixel          = 0
    for ii=w0[i], w1[i] do begin
      ss              = s[ii]
      ss              = repchr( ss, '<', ' ' )
      ss              = repchr( ss, '>', ' ' )
      ss              = repchr( ss, '/', ' ' )
      key= getwrd(ss,0)
      case key of
      'longitude'      : begin
                           slongitude      = getwrd(ss,1)
                           if isnumber(slongitude)    gt 0 then flongitude       = 1
                         end
      'latitude'       : begin 
                           slatitude        = getwrd(ss,1)
                           if isnumber(slatitude)      gt 0 then flatitude       = 1
                         end                         
      'incidenceAngle' : begin
                           sincidenceAngle  = getwrd(ss,1)
                           if isnumber(incidenceAngle) gt 0 then fincidenceAngle = 1
                         end 
      'line'           : begin
                           sline            = getwrd(ss,1) 
                           if isnumber(sline)          gt 0 then fsline          = 1
                         end
      'pixel'          : begin
                           spixel           = getwrd(ss,1)
                           if isnumber(spixel)         gt 0 then fpixel          = 1  
                         end 
      ELSE             : ; No operation
      endcase      
    endfor
    if flongitude and flatitude and fincidenceAngle and fsline and fpixel then begin
      longitude[i]      =   float( slongitude      )
      latitude[i]       =   float( slatitude       )
      incidenceAngle[i] =   float( sincidenceAngle )
      line[i]           =   long(  sline           )
      pixel[i]          =   long(  spixel          )
    endif
  endfor
endif


if keyword_set(display) then begin 
   print,  ''
   print, 'XML Contents'
   print, '--------------------------------------------------------------'
   print, 'missionId                 = ' , x.missionId              
   print, 'productType               = ' , x.productType            
   print, 'polarisation              = ' , x.polarisation           
   print, 'mode                      = ' , x.mode                   
   print, 'swath                     = ' , x.swath                  
   print, 'startTime                 = ' , x.startTime              
   print, 'stopTime                  = ' , x.stopTime               
   print, 'absoluteOrbitNumber       = ' , x.absoluteOrbitNumber    
   print, 'missionDataTakeId         = ' , x.missionDataTakeId      
   print, 'imageNumber               = ' , x.imageNumber            
   print, 'productQualityIndex       = ' , x.productQualityIndex    
   print, 'azimuthTime               = ' , x.azimuthTime            
   print, 'outputDataMean_re         = ' , x.outputDataMean_re      
   print, 'outputdatamean_im         = ' , x.outputdatamean_im      
   print, 'outputDataStdDev_re       = ' , x.outputDataStdDev_re    
   print, 'outputDataStdDev_im       = ' , x.outputDataStdDev_im    
   print, 'pass                      = ' , x.pass                   
   print, 'platformHeading           = ' , x.platformHeading        
   print, 'projection                = ' , x.projection             
   print, 'radarFrequency            = ' , x.radarFrequency         
   print, 'productFirstLineUtcTime   = ' , x.productFirstLineUtcTime
   print, 'productLastLineUtcTime    = ' , x.productLastLineUtcTime 
   print, 'ascendingNodeTime         = ' , x.ascendingNodeTime      
   print, 'anchorTime                = ' , x.anchorTime             
   print, 'slantRangeTime            = ' , x.slantRangeTime         
   print, 'pixelValue                = ' , x.pixelValue             
   print, 'outputPixels              = ' , x.outputPixels           
   print, 'rangePixelSpacing         = ' , x.rangePixelSpacing      
   print, 'azimuthPixelSpacing       = ' , x.azimuthPixelSpacing    
   print, 'azimuthTimeInterval       = ' , x.azimuthTimeInterval    
   print, 'numberOfSamples           = ' , x.numberOfSamples        
   print, 'numberOfLines             = ' , x.numberOfLines          
   print, 'incidenceAngleMidSwath    = ' , x.incidenceAngleMidSwath 
   print, 'ellipsoidName             = ' , x.ellipsoidName                   
   print, 'ellipsoidSemiMajorAxis    = ' , x.ellipsoidSemiMajorAxis 
   print, 'ellipsoidSemiMinorAxis    = ' , x.ellipsoidSemiMinorAxis 
   print, file
   hlp,  longitude, latitude, incidenceAngle, line, pixel
   print, '--------------------------------------------------------------'
   print, ''
endif

x= create_struct(x, 'longitude'      ,   longitude      , $
                    'latitude'       ,   latitude       , $
                    'incidenceAngle' ,   incidenceAngle , $
                    'line'           ,   line           , $
                    'pixel'          ,   pixel          )
end
