;----------------------------------------------------------
;+
; NAME: SEN_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out Sentinel-1 data into ancillary data ascii file. These
;   data fit the SIO format.
; CALLING SEQUENCE:'
;    sen_write_ancil_sio, filename, lf
; INPUT:'
;   filename    = output file with ancillary data
;   lf          = leader structure'
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   error     =  error ( <0 is bad )
; MODIFICATION HISTORY:
;   2013 Oct 10 Frank Monaldo.
;   2016 Jun 10 Frank Monaldo. Add sen_cpol_remove_noise and sen_xpol_remove_noise
;
; Copyright (C) 2007-2016, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro sen_write_ancil_sio, filename, lf,  quiet=quiet, help=help, error=error


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
error  = 0
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: SEN_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out Sentinel-1 data into ancillary data ascii file. These
  print, '   data fit the SIO format.
  print, ' CALLING SEQUENCE:'
  print, '    sen_write_ancil_sio, filename, lf
  print, ' INPUT:'
  print, '   filename    = output file with ancillary data
  print, '   lf          = leader structure'
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  print, '   error     =  error ( <0 is bad )
  error  = -1
  return
endif

sar_sensor = 'Sentinel-1A'
if strupcase( lf.platform_name ) eq 'S1B' then sar_sensor = 'Sentinel-1B'
range_type='GROUND RANGE'
range_flag= 1
if lf.product_type ne 'GRD' then begin
  range_type = 'SLANT RANGE'
  range_flag  = 1
endif
lf = create_struct( lf, 'name_dir'            ,  lf.root_file                                             ,  $
                        'month'               ,  dt_tm_fromjs( lf.acq_date_time_js, format='0n$'        ) ,  $
                        'day'                 ,  long( dt_tm_fromjs( lf.acq_date_time_js, format='0d$') ) ,  $
                        'data_year'           ,  long( dt_tm_fromjs( lf.acq_date_time_js, format='Y$' ) ) ,  $
                        'hour'                ,  long( dt_tm_fromjs( lf.acq_date_time_js, format='h$' ) ) ,  $
                        'minute'              ,  long( dt_tm_fromjs( lf.acq_date_time_js, format='m$' ) ) ,  $
                        'second'              ,  long( dt_tm_fromjs( lf.acq_date_time_js, format='s$' ) ) ,  $
                        'radar_frequency'     ,  lf.sensor_frequency_hz                                   ,  $
                        'pol'                 ,  lf.polarization                                          ,  $
                        'range_pixel'         ,  lf.pixel_spacing_m                                       ,  $
                        'azimuth_pixel'       ,  lf.line_spacing_m                                        ,  $
                        'productType'         ,  lf.sensor_imaging_mode                                   ,  $
                        'proc_fac'            ,  lf.processing_facility                                   ,  $
                        'software_version'    ,  lf.sar_processor_version                                 ,  $
                        'datatake_id'         ,  lf.scene_reference_number                                ,  $
                        'sar_sensor'          ,  sar_sensor                                               ,  $
                        'platform_code'       ,  lf.platform_name                                         ,  $
                        'sar_imaging_mode'    ,  lf.sensor_imaging_mode                                   ,  $
                        'valuetype'           ,  lf.image_pixel_value_units                               ,  $
                        'asc_des'             ,  lf.orbit_pass_flag                                       ,  $
                        'beam_mode'           ,  lf.sensor_imaging_mode                                   ,  $
                        'sensor_velocity'     ,  '-9999.9999'                                             ,  $
                        'integration_time'    ,  '-9999.9999'                                             ,  $
                        'nom_az_res'          ,  '-9999.9999'                                             ,  $
                        'nom_rg_res'          ,  '-9999.9999'                                             ,  $
                        'rcvr_gain'           ,  string(lf.receiver_gain_db)                              ,  $
                        'proc_gain'           ,  string(lf.relative_processor_gain_db)                    ,  $
                        'range_flag'          ,  range_flag                                               ,  $
                        'heading'             ,  lf.heading_north_deg                                     ,  $
                        'heading_rel'         ,  lf.heading_image_deg                                     ,  $
                        'flip'                ,  lf.flip_flag                                             ,  $
                        'illumination_dir'    ,  lf.illumination_dir_deg                                  ,  $
                        'range_swath'         ,  lf.range_swath_width_m                                   ,  $
                        'azimuth_swath'       ,  lf.azimuth_swath_width_m                                 ,  $
                        's_c_altitude'        ,  lf.sensor_altitude_m                                     ,  $
                        'center_incid'        ,  lf.incid_angle_center_deg                                ,  $
                        'slant_range_center'  ,  lf.slant_range_center_m                                  ,  $
                        'slant_range_near'    ,  lf.slant_range_near_m                                    ,  $
                        'range_looks'         ,  lf.looks_range                                           ,  $
                        'azimuth_looks'       ,  lf.looks_azimuth                                         ,  $
                        'ellipsoidName'       ,  lf.ref_ellipsoid                                         ,  $
                        'semiMajorAxis'       ,  float(lf.ref_semi_major_axis_m)                          ,  $
                        'semiMinorAxis'       ,  float(lf.ref_semi_minor_axis_m)                          ,  $
                        'ellipsoidscale'      ,  lf.datum_scale                                           )



;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 
end
