;-----------------------------------------------------
;+
; NAME: SMAP_FILLIN_LF.PRO
; PURPOSE:
;    Read SMAP HDF5 file and defaults file to fill in the lf structure with ancillary
;    information.   Note that the lon, lat, rlook, and incident coefs
;    are NOT computed in this progam since the SMAP swath will be parsed
;    in ans_read_smap and will computed on a section by section basis
; CALLING SEQUENCE
;    sen_fillin_lf, xfile, deffile, lf
; INPUT
;   xfile     : Sentinel-1 XML file
;   deffile    : Defaults file
; OUTPUT
;   lf:        : lf structure
; OPTIONS
;   jsstart    : Processing start time in Julian seconds
;   /help      : Get help
;   /quiet     : Suppress intermediat results
;   /display   : Display fits for diagnostics purposes.
;   error      : < 0 bad
; MODIFICATION HISTORY:
; 2015 Dec 21 Chris Jackson  Inital Version
; Copyright (C) 2010-2013, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-

pro  smap_fillin_lf, h5file, deffile, lf,  $
    jsstart =     jsstart, $
    help    =     help   , $
    quiet   =     quiet  , $
    display =     display, $
    error   =     error
    
  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  if not keyword_set(jsstart) then jsstart=  dt_tm_tojs(systime(/utc))
  
  if keyword_set(help)  or (n_params(0) lt 1 ) then begin
    print, ' NAME: SEN_FILLIN_LF.PRO
    print, ' PURPOSE:
    print, '    Read SMAP HDF5 file and defaults file to fill in the lf structure with ancillary
    print, '    information.   Note that the lon, lat, rlook, and incident coefs
    print, '    are NOT computed in this progam since the SMAP swath will be parsed
    print, '    in ans_read_smap and will computed on a section by section basis
    print, ' CALLING SEQUENCE
    print, '    sen_fillin_lf, h5file, deffile, lf
    print, ' INPUT
    print, '   h5file     : Sentinel-1 XML file
    print, '   deffile    : Defaults file
    print, ' OUTPUT
    print, '   lf:        : lf structure
    print, ' OPTIONS
    print, '   jsstart    : Processing start time in Julian seconds
    print, '   /help      : Get help
    print, '   /quiet     : Suppress intermediat results
    print, '   /display   : Display fits for diagnostics purposes.
    print, '   error      : < 0 bad
    error = -1
    return
  endif
  
  speed_of_light   = 299792458.00 ; m/s
  ;
  ;
  ;  Read defaults file
  ;
  ans_read_defaults, deffile, df, quiet=quiet, error = error
  if error lt 0 then return
  
  ;ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
  answrs_version = 2.0
  ;
  ;  Read out attributes in the SMAP hdf5 file
  ;
  fileid = 0
  fileid = h5f_open(h5file)
  if (fileid le 0) then begin
    print,  ''
    print, '============================================================
    print, 'H5F_OPEN failed for file: ' + h5file
    print, 'Exiting smap_fillin_lf'
    print, '============================================================
    print, ''
    error = -1
    return
  endif
  
  ;groupid = h5g_open(fileid,'/Metadata/OrbitMeasuredLocation')
  root_file = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/DatasetIdentification'),'fileName'))
  cross_track_sample = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/GridSpatialRepresentation/crossTrack'),'dimensionSize'))
  track_sample = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/GridSpatialRepresentation/track'),'dimensionSize'))
  cross_track_reskm = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/GridSpatialRepresentation/crossTrack'),'resolution'))
  track_reskm = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/GridSpatialRepresentation/track'),'resolution'))
  start_time_str = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/OrbitMeasuredLocation'),'halfOrbitStartDateTime'))
  stop_time_str  = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/OrbitMeasuredLocation'),'halfOrbitStopDateTime'))
  orbitdir_str = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/OrbitMeasuredLocation'),'orbitDirection'))
  revNumber = h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/OrbitMeasuredLocation'),'revNumber'))
  
  nasa_software_ver=h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/ProcessStep'),'SWVersionID'))
  nasa_software_date=h5a_read(h5a_open_name(h5g_open(fileid,'/Metadata/ProcessStep'),'softwareDate'))
  
  h5f_close, fileid 
  ; Determine if its a defaults file of a Level-2 netCDF file
  filebreak, h5file, file=infile ;Puts the filename only into infile
  
 
 
  if (strcmp(root_file,infile,/FOLD_CASE) eq 0) then begin
    if printflag then begin
      print,  ''
      print, '============================================================
      print, 'Warning!! Mis-match between filename as given:' + infile
      print, 'and SMAP embedded filename' + root_file
      print, '============================================================
      print, ''
    endif
  endif
  
  ;
  ; Declaring lf structure
  ;
  startTime            = repchr(start_time_str, 'T', ' ' )
  startTime            = repchr(startTime, 'Z', '' )
  stopTime             = repchr(stop_time_str,  'T', ' ' )
  stopTime            = repchr(stopTime, 'Z', '' )
  startTime_js           = dt_tm_tojs( startTime, /YMD )
  stopTime_js            = dt_tm_tojs( stopTime,  /YMD )
  center_js              = (startTime_js + stopTime_js) /2.0D
  acquisition_date       = dt_tm_fromjs( center_js, format='0n$-0d$-Y$' )
  acquisition_time       = dt_tm_fromjs( center_js, format='h$:m$:s$' )
  acq_date_time_js       = center_js
  acq_date_time          = dt_tm_fromjs( center_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
  absorbit               = long(revNumber)
  processing_start       = dt_tm_fromjs( jsstart, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
  current_js             = dt_tm_tojs( systime(/utc) )
  sio_create_date_time   = dt_tm_fromjs( current_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
  orbit_pass_flag        =  strupcase( strmid(orbitdir_str, 0, 1 ) )
  sensor_frequency_hz    =  1.225e9 ; L-Band 1.225 MHz
  sensor_wavelength_m    =  speed_of_light / sensor_frequency_hz
  map_descriptor         =  'GROUND RANGE'
  range_type             =  1
  range_swath_width_m    =  float(cross_track_sample) * float(strmid(cross_track_reskm,0,1)) *1000.0
  azimuth_swath_width_m  =  float(track_sample) * float(strmid(track_reskm,0,1)) *1000.0
  nx0                    =  long(cross_track_sample)
  ny0                    =  long(track_sample)
  nx1                    =  nx0 / long(df.ave_pixel)
  ny1                    =  ny0 / long(df.ave_line)
  range_pixel            =  float(strmid(cross_track_reskm,0,1))*1000.0 *float(df.ave_pixel)
  azimuth_pixel          =  float(strmid(track_reskm,0,1))*1000.0*float(df.ave_line)
  sar_processor          =  nasa_software_date+'V'+nasa_software_ver 
  
  lf= {name_dir                   :  root_file                       , $
    answrs_version                :  answrs_version                  , $
    acquisition_date              :  acquisition_date                , $
    acquisition_time              :  acquisition_time                , $
    acq_date_time_js              :  acq_date_time_js                , $
    acq_date_time                 :  acq_date_time                   , $
    jsbegin                       :  startTime_js                    , $
    jsend                         :  stopTime_js                     , $
    processing_start              :  processing_start                , $
    sio_create_date_time          :  sio_create_date_time            , $
    producttype                  : 'L1C_S0_HIRES'                   , $
    processing_agency             : 'NASA'                           , $
    proc_fac                      : 'JPL'                            , $
    software_version              :  sar_processor                   , $
    datatake_id                :  0                               , $
    scene_reference_number        :  0                               , $
    orbit_number                  :  absorbit                        , $
    sar_sensor                    :  'SMAP'                          , $
    platform_code                 :  'SMAP'                          , $
    sar_imaging_mode              :  'HIRES'                         , $
    scansar                       :  1                               , $
    calibration_flag              :  1                               , $
    image_pixel_value_units       :  'Intensity'                     , $
    valuetype                     :  'Intensity'                     , $
    polarization                  :  'HH'                            , $
    pol                           :  'HH'                            , $
    radar_frequency               :  1.225e9                         , $
    frequency_band                :  'L'                             , $
    flag_sar_band_polarization    :  '00'                            , $
    asc_des                       :  orbit_pass_flag                 , $
    scaling                       :  1.000000                        , $
    beam_mode                     :  'HIRES'                         , $
    sensor_wavelength_m           :  sensor_wavelength_m             , $
    sensor_frequency_hz           :  sensor_frequency_hz             , $
    sensor_velocity               :  -9999.0                      , $
    integration_time              :  -9999.0                      , $
    nom_az_res                    :  1000.0                       , $
    nom_rg_res                    :  1000.0                       , $
    rcvr_gain                     :  -9999.0                      , $
    proc_gain                     :  -9999.0                      , $
    range_type                    :  map_descriptor                  , $
    pixel_spacing_m               :  float(strmid(cross_track_reskm,0,1))*1000.0 , $
    line_spacing_m                :  float(strmid(track_reskm,0,1))*1000.0 , $
    range_pixel                   :  range_pixel                     , $
    azimuth_pixel                 :  azimuth_pixel                   , $
    range_flag                    :  range_type                      , $
    heading                       :  0.0                             , $
    heading_rel                   :  0.0                             , $
    flip                          :  0                               , $
    illumination_dir              :  90.0                            , $
    early_azimuth_edge            :  '-9999.0'                    , $
    near_range_edge               :  '-9999.0'                    , $
    range_swath                   :  range_swath_width_m             , $
    azimuth_swath                 :  azimuth_swath_width_m           , $
    s_c_altitude                  :  7066.8                          , $
    center_incid                   :  40.0                            , $
    slant_range_center            :  -9999.0                      , $
    slant_range_near              :  -9999.0                      , $
    lines_original_image          :  ny0                             , $
    pixels_original_image         :  nx0                             , $
    lines_sio_image               :  ny1                             , $
    pixels_sio_image              :  nx1                             , $
    nx0                           :  nx0                             , $
    ny0                           :  ny0                             , $
    nx1                           :  nx1                             , $
    ny1                           :  ny1                             , $
    ave_pixel                     :  df.ave_pixel                    , $
    ave_line                      :  df.ave_line                     , $
    range_looks                   :  -9999.0                         , $
    azimuth_looks                 :  -9999.0                         , $
    ellipsoidName                 :  'WGS84'                         , $
    semiMajorAxis                 :  6378.137                          , $
    semiMinorAxis                 :  6356.752                          , $
    datumshiftparameters          :  '0 0  0'                        , $
    datum_scale                   :  1                               , $
    ellipsoidscale                :  1                               , $
    center_lat                    : -9999.0                          , $
    center_lon                    : -9999.0                          , $
    nrcs_slope                    : df.smap_slope                     , $
    nrcs_bias                     : df.smap_bias                      , $
    lngswitchflag                 : 0                                , $
    icefile                       :''                                , $
    icefile_time_js               :''                                , $
    icefile_time                  :''                                , $
    deg                           : df.deg                           , $
    deg_incid                     : df.deg                           , $
    deg_rlook                     : df.deg                            }
    

end








