; -----------------------------------------------------
; +
; NAME:
;   SMAP_INCID2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a
;   polynomial fit to go from pixel location to incidence angle.
;   For SMAP incidence is sent in as a 2D array (Valid angle values are 0 to 90 deg; fill values =-9999)
; CALLING SEQUENCE:
;    smap_incid2pix_fit, incid, lf
; INPUTS:
;    incid   2D array of incidence angles (Valid angle values are 0 to 90 deg; fill values =-9999)
;    lf     leader structure
; OUTPUTS
;    Coefficients added to structure
;    lf.incid_coef: Array of incidence angle coefficients a function of
;                  i,j.  The output of the fit is in degrees.
; KEYWORDS:
;    /quiet=     Suppress printed messages.
;    /help=      Display help
;    /diagnostic Print diagnostic results
; MODIFICATION HISTORY:
;  2015 Dec 23 Chris Jackson.  Initial version.  Based on env_lonlat2pix
; Copyright (C) 2003-2015, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;-----------------------------------------------------
;
pro  smap_incid2pix_fit, incid, lf, quiet=quiet, help=help,diagnostic=diagnostic


  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  
  if keyword_set(help) or (n_params(0) lt 1) then begin
    print, ' NAME:
    print, '   SMAP_INCID2PIX_FIT
    print, ' PURPOSE:
    print, '   The purpose of this program is compute a
    print, '   polynomial fit to go from pixel location to incidence angle.
    print, '   For SMAP incidence is sent in as a 2D array (Valid angle values are 0 to 90 deg)
    print, ' CALLING SEQUENCE:
    print, '    smap_incid2pix_fit, incid, lf
    print, ' INPUTS:
    print, '    incid   2D array of incidence angles (Valid angle values are 0 to 90 deg)
    print, '    lf      leader structure
    print, ' OUTPUTS
    print, '    Coefficients added to structure
    print, '    lf.incid_coef: Array of incidence angle coefficients a function of
    print, '                  i,j.  The output of the fit is in degrees.
    print, ' KEYWORDS:
    print, '    /quiet=     Suppress printed messages.
    print, '    /help=      Display help
    print, '    /diagnostic Print diagnostic results
    return
  endif
  
  if printflag then  print, 'Computing fit of incidence vs i,j...'
  
  sz= size(incid)
  
  if (sz[0] eq 0)then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Incidence Angle Array has zero elements'
      print, 'Exiting smap_incidpix'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  
  ;Set up the ii and jj arrays
  iid = lonarr(sz[1],sz[2])
  for i =0,sz[2]-1L do iid(*,i) = lindgen(sz[1])
  jjd = rotate(iid,1)
  
  ;Now doing a fit on a 1000 x 1000 array is a bit of overkill
  ;For SMAP incidence which is always nearly 40 deg a 100 x 100) should work just as well.
  ;Find a divisor that produces 100 lines - since the # of lines
  ;can vary (between 500 and 1000) and the number of pixels is always
  ;roughly 1000.  This will slightly "over-fit" for partial frames but....
  nskip = 1.0;
  if(floor(sz[2]/100.0) gt 1) then nskip = floor(sz[2]/100.0)
  ldx = lindgen(floor((sz[4]-1L)/float(nskip)))*nskip
  
  
  ;Get the valid incidence angle value to sent into the fit (Values =  -9999 where no data are taken)
  fidx = where(incid[ldx] ge 0,/null);
  
  ;Potentially a large number of point (1K x 1K) could be input.
  ;While the program works its probably better to cur the number down.
  
  ans_compute_fit, iid[ldx[fidx]],  jjd[ldx[fidx]], incid[ldx[fidx]], 2, incid_coef, incid_xexp, incid_yexp,quiet=quiet
  lf.deg_incid=2
  
  ;
  ; theta-regression.  incidence position as a function of longitude and latitude
  
  
  
  fcheck=finite(incid_coef)
  w=where(fcheck eq 0, fcount)
  if fcount gt 0 then begin
    print, 'There is a problem with the incident angle fit'
    print, 'Input incident angles'
    print, incid
    stop
  endif
  
  if keyword_set(diagnostic) then begin
    ns=n_elements(fidx)
    iid = iid[fidx]
    jjd = jjd[fidx]
    
    
    print, 'Incidence angle conversions:'
    mu=0
    sd=0.
    for i=0, ns -1 do begin
      i1=long(iid[i]+0.5)
      j1=lon(jjd[i]+0.5)
      output=          string(i1, format='(I7)')
      output= output + string(j1, format='(I7)')
      output= output + string( incid[i] )
      incidp=   ans_poly_cal( iid[i], jjd[i], incid_coef, incid_xexp, inicd_yexp )
      mu= mu+  (incid[i]-incidp)
      sd= sd + (incid[i]-incidp)^2
      output= output + string( incidp )
      print, output
    endfor
    print, 'Fractional error in incidence angle', mu/ns, sqrt(sd/ns)
    stop
  endif
  
  
  ;
  ; Add to structure
  ;
  
  temp_lf= create_struct( lf, 'incid_coef', incid_coef, $
    'incid_xexp', incid_xexp, $
    'incid_yexp', incid_yexp  )
  lf=      0
  lf=      temp_lf
  
  
end
