; '-----------------------------------------------------
; +
; NAME:
;    SMAP_LONLAT2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a
;   polynomial fit to go from lon/lat to pixel location and visa versa.
;   This program requires the SMAP longitude and latitide arrays
;   and the lf structure (in which to place the results)
;   i/j are assumed to correspong to pixel and row number
; CALLING SEQUENCE:
;    smap_lonlat2pix_fit, lon_array, lat_array, lf
; INPUTS:
;    lon_array  2D Array of longitudes
;    lat_array  2D Array of latitude
;    lf         Structure with lat/lngs and ave_pixel, ave_line
;               and degree of fit
; OUTPUTS:
;   Added to structure
;   lf.i_coef:    Array of coefficients for horizontal pixel fit
;   lf.j_coef=    Array of coefficients for vertical pixel  fit
;   lf.lon_coef=  Array of longitude coefficients
;   lf.lat_coef=  Array of latitude coefficients
; KEYWORDS:
;    /quiet=    Suppress printed messages.
;    /help=     Display help
;    /diagnostic Display comparison results
;    error=     errpr
; MODIFICATION HISTORY:
;  2015 Dec 23 Chris Jackson.  Initial version.  Based on env_lonlat2pix
; Copyright (C) 2003-2015, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;-----------------------------------------------------
;
pro smap_lonlat2pix_fit, lng, lat, lf, quiet=quiet, help=help,diagnostic=diagnostic, error=error

  printflag=1
  
  error= 0
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  
  if keyword_set(help) or (n_params(0) lt 1) then begin
    print, ' NAME:
    print, '    SMAP_LONLAT2PIX_FIT
    print, ' PURPOSE:
    print, '   The purpose of this program is compute a
    print, '   polynomial fit to go from lon/lat to pixel location and visa versa.
    print, '   This program requires the SMAP longitude and latitide arrays
    print, '   and the lf structure (in which to place the results)
    print, '   i/j are assumed to correspong to pixel and row number
    print, ' CALLING SEQUENCE:
    print, '    smap_lonlat2pix_fit, lon_array, lat_array, lf
    print, ' INPUTS:
    print, '    lon_array  2D Array of longitudes
    print, '    lat_array  2D Array of latitude
    print, '    lf         Structure with lat/lngs and ave_pixel, ave_line
    print, '               and degree of fit
    print, ' OUTPUTS:
    print, '   Added to structure
    print, '   lf.i_coef:    Array of coefficients for horizontal pixel fit
    print, '   lf.j_coef=    Array of coefficients for vertical pixel  fit
    print, '   lf.lon_coef=  Array of longitude coefficients
    print, '   lf.lat_coef=  Array of latitude coefficients
    print, ' KEYWORDS:
    print, '    /quiet=    Suppress printed messages.
    print, '    /help=     Display help
    return
  endif
  
  if printflag then  print, 'Computing a fit to lng and lat for line of data...'
  
  sz_lon= size(lng)
  sz_lat = size(lat)
  
  if (sz_lon[1] ne sz_lat[1]) or (sz_lon[2] ne sz_lat[2]) then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Longitude and Latitude Arrays are not the same size'
      print, 'Exiting smap_lonlat2pix'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  if (sz_lon[0] eq 0) or (sz_lat[0]) eq 0 then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Either the Longitude and Latitude Arrays have zero elements'
      print, 'Exiting smap_lonlat2pix'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  
  ;Set up the ii and jj arrays
  iid = lonarr(sz_lon[1],sz_lon[2])
  for i =0,sz_lon[2]-1L do iid(*,i) = lindgen(sz_lon[1])
  jjd = rotate(iid,1)
  
  ;Check to see if longitude (normally -180 to 180) contains
  ;a discontinuity (i.e. crosses the international date line)
  ;If so change basis to 0 - 360
  
  if ( (max(lng) - min(lng)) gt 160.0) then begin
     lf.lngswitchflag = 1
     lng = (lng + 360) MOD 360
  endif
  
  ;Now doing a fit on a 1000 x 1000 array is a bit of overkill
  ;a 200 x 200 (or even 100 x 100) should work just as well. 
  ;Find a divisor that produces 200 lines - since the # of lines
  ;can vary (between 500 and 1000) and the number of pixels is always
  ;roughly 1000.  This will slightly "over-fit" for partial frames but....  
   nskip = 1.0;
  if(floor(sz_lon[2]/200.0) gt 1) then nskip = floor(sz_lon[2]/200.0)
  ldx = lindgen(floor((sz_lon[4]-1L)/float(nskip)))*nskip
  
  ;
  ; We wish to find equations of the form form
  ;  i=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
  ;  j=  sum_m=0^deg sum_n=0^deg A(m,n) *lng^m lat^n
  ;
  lf.deg = 3
  ;ans_compute_fit, lng, lat,  reform(iid,sz_lon[4],1), lf.deg, i_coef,   i_xexp,   i_yexp, quiet=quiet
  ;ans_compute_fit, lng, lat,  reform(jjd,sz_lon[4],1), lf.deg, j_coef,   j_xexp,   j_yexp, quiet=quiet
  ans_compute_fit, lng[ldx], lat[ldx],  iid[ldx], lf.deg, i_coef,   i_xexp,   i_yexp, quiet=quiet
  ans_compute_fit, lng[ldx], lat[ldx],  jjd[ldx], lf.deg, j_coef,   j_xexp,   j_yexp, quiet=quiet
  
  ;
  ; longitude latitude regression.  Longitude and latitude as a function of pixel position
  ;
  ;ans_compute_fit, iid, jjd, reform(lng,sz_lon[4],1), lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
  ;ans_compute_fit, iid, jjd, reform(lat,sz_lon[4],1), lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet
  ans_compute_fit, iid[ldx], jjd[ldx], lng[ldx], lf.deg, lon_coef, lon_xexp, lon_yexp, quiet=quiet
  ans_compute_fit, iid[ldx], jjd[ldx], lat[ldx], lf.deg, lat_coef, lat_xexp, lat_yexp, quiet=quiet  
  ;
  ; Check for reasonableness
  ;
  lo0=ans_poly_cal(        0,        0, lon_coef, lon_xexp, lon_yexp )
  lo1=ans_poly_cal( max(iid), max(jjd), lon_coef, lon_xexp, lon_yexp )
  if printflag then print, lo0, lo1
  
  if lo0 gt  400 or $
    lo0 lt -200 or $
    lo1 gt  400 or $
    lo1 lt -200 then begin
    lon_coef= lon_coef*0.
    print, 'Bad lon fit ', lo0, lo1
    error=-1
  endif
    
  ;
  ; Check and see if we can go back and forth
  
  if keyword_set(diagnostic) then begin
    sz = size(iid)
    ns = sz[4]
    
    ;
    ; Check latitude conversions
    ;
    print, 'Latitude conversions:'
    mu=0
    sd=0.
    for i=0, ns -1L do begin
      i1=long(iid[i]+0.5)
      j1=long(jjd[i]+0.5)
      output=          string(i1, format='(I5)')
      output= output + string(j1, format='(I5)')
      output= output + string( lat[i] )
      latp=   ans_poly_cal( iid[i], jjd[i], lat_coef, lat_xexp, lat_yexp )
      mu= mu+  (lat[i]-latp)
      sd= sd + (lat[i]-latp)^2
      output= output + string( latp )
      print, output
    endfor
    stop
    print, 'Latitude', mu/ns, sqrt(sd/ns)
    ;
    ; Check longitude conversions
    ;
    print, 'Longitude conversions:'
    mu=0
    sd=0.
    for i=0, ns -1L do begin
      i1=long(iid[i]+0.5)
      j1=long(jjd[i]+0.5)
      output=          string(i1, format='(I5)')
      output= output + string(j1, format='(I5)')
      output= output + string( lng[i] )
      lngp=   ans_poly_cal( iid[i], jjd[i], lon_coef, lon_xexp, lon_yexp )
      mu= mu+  (lng[i]-lngp)
      sd= sd + (lng[i]-lngp)^2
      output= output + string( lngp )
      print,   output
    endfor
    print, 'Longitude',mu/ns, sqrt(sd/ns)
    stop
    ;
    ; Check i conversions
    ;
    print, 'i conversions:'
    mu=0
    sd=0.
    for i=0, ns -1L do begin
      i1=long(iid[i]+0.5)
      j1=long(jjd[i]+0.5)
      output= string(lat[i]) + ' ' + string(lng[i])
      output= output + string(iid[i])
      ip=     ans_poly_cal( lng[i], lat[i], i_coef,   i_xexp,   i_yexp )
      mu= mu+  (iid[i]-ip)
      sd= sd + (iid[i]-ip)^2
      output= output + string(ip )
      print,  output
    endfor
    print, 'i',mu/ns, sqrt(sd/ns)
    stop
    ;
    ; Check j conversions
    ;
    print, 'j conversions:'
    mu=0
    sd=0.
    for i=0, ns -1L do begin
      i1=long(iid[i]+0.5)
      j1=long(jjd[i]+0.5)
      output= string(lat[i]) + ' ' + string(lng[i])
      output= output + string(jjd[i])
      jp=     ans_poly_cal( lng[i], lat[i], j_coef,   j_xexp,   j_yexp )
      mu= mu+  (jjd[i]-jp)
      sd= sd + (jjd[i]-jp)^2
      output= output + string(jp )
      print,  output
    endfor
    print, 'j',mu/ns, sqrt(sd/ns)
    stop
  endif
  
  ;
  ; Add to structure
  ;
  
  temp_lf= create_struct(lf, 'lon_coef', lon_coef, $
    'lon_xexp', lon_xexp, $
    'lon_yexp', lon_yexp, $
    'lat_coef', lat_coef, $
    'lat_xexp', lat_xexp, $
    'lat_yexp', lat_yexp, $
    'i_coef',   i_coef,   $
    'i_xexp',   i_xexp,   $
    'i_yexp',   i_yexp,   $
    'j_coef',   j_coef,   $
    'j_xexp',   j_xexp,   $
    'j_yexp',   j_yexp )
    
  lf=               0
  lf=               temp_lf
  
  error=1
end

