; -----------------------------------------------------
; +
; NAME:
;   SMAP_RLOOK2PIX_FIT
; PURPOSE:
;   The purpose of this program is compute a
;   polynomial fit to go from pixel location to radar look (azimuth) angle.
;   For SMAP radar look (azimuth) is sent in as a 2D array (Valid angle values are 0 to 90 deg; fill values =-9999)
; CALLING SEQUENCE:
;    smap_rlook2pix_fit, rlook, lf
; INPUTS:
;    rlook   2D array of radar look (azimuth) angles (Valid angle values are 0 to 90 deg; fill values =-9999)
;    lf     leader structure
; OUTPUTS
;    Coefficients added to structure
;    lf.rlook_coef: Array of radar look (azimuth) angle coefficients a function of
;                  i,j.  The output of the fit is in degrees.
; KEYWORDS:
;    /quiet=     Suppress printed messages.
;    /help=      Display help
;    /diagnostic Print diagnostic results
; MODIFICATION HISTORY:
;  2015 Dec 23 Chris Jackson.  Initial version.  Based on smap_incid2pix_fit
; Copyright (C) 2003-2015, Johns Hopkins University/Applied
; Physics Laboratory. This software may be used, copied,
; or redistributed as long as it is not sold and this
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied
; warranties whatsoever.
;-
;-----------------------------------------------------
;
pro  smap_rlook2pix_fit, rlook, lf, quiet=quiet, help=help,diagnostic=diagnostic


  printflag=1
  if keyword_set(quiet) then printflag=0
  quiet= printflag eq 0
  
  if keyword_set(help) or (n_params(0) lt 1) then begin
    print, ' NAME:
    print, '   SMAP_RLOOK2PIX_FIT
    print, ' PURPOSE:
    print, '   The purpose of this program is compute a
    print, '   polynomial fit to go from pixel location to radar look (azimuth) angle.
    print, '   For SMAP radar look (azimuth) is sent in as a 2D array (Valid angle values are 0 to 90 deg)
    print, ' CALLING SEQUENCE:
    print, '    smap_rlook2pix_fit, rlook, lf
    print, ' INPUTS:
    print, '    rlook   2D array of radar look (azimuth) angles (Valid angle values are 0 to 90 deg)
    print, '    lf      leader structure
    print, ' OUTPUTS
    print, '    Coefficients added to structure
    print, '    lf.rlook_coef: Array of radar look (azimuth) angle coefficients a function of
    print, '                  i,j.  The output of the fit is in degrees.
    print, ' KEYWORDS:
    print, '    /quiet=     Suppress printed messages.
    print, '    /help=      Display help
    print, '    /diagnostic Print diagnostic results
    return
  endif
  
  if printflag then  print, 'Computing fit of radar look (azimuth) vs i,j...'
  
  sz= size(rlook)
  
  if (sz[0] eq 0)then begin
    if (printflag gt 0) then begin
      print,  ''
      print, '============================================================
      print, 'Radar Look (azimuth) Angle Array has zero elements'
      print, 'Exiting smap_rlookpix'
      print, '============================================================
      print, ''
    endif
    error= -1
    return
  endif
  
  
  
  
  ;Set up the ii and jj arrays
  iid = lonarr(sz[1],sz[2])
  for i =0,sz[2]-1L do iid(*,i) = lindgen(sz[1])
  jjd = rotate(iid,1)
  
  ;Now doing a fit on a 1000 x 1000 array is a bit of overkill
  ;Find a divisor that produces 200 lines - since the # of lines
  ;can vary (between 500 and 1000) and the number of pixels is always
  ;roughly 1000.  This will slightly "over-fit" for partial frames but....
  nskip = 1.0;
  if(floor(sz[2]/200.0) gt 1) then nskip = floor(sz[2]/200.0)
  ldx = lindgen(floor((sz[4]-1L)/float(nskip)))*nskip
  
  ;Get the valid radar look (azimuth) angle value to sent into the fit (Values =  -9999 where no data are taken)
  fidx = where(rlook[ldx] ge -180.0,/null);
  
  ;The HDF documentation days that the valid values are 0 to 360 deg but the data show +/- 180
  ;Since azimuth is clockwise from true north this array should always be 0 to 360 .
  ;For the time being just make if 0 - 360 always
  rlook[ldx[fidx]] = (rlook[ldx[fidx]] + 360) MOD 360

  
  ans_compute_fit, iid[ldx[fidx]],  jjd[ldx[fidx]], rlook[ldx[fidx]], 3, rlook_coef, rlook_xexp, rlook_yexp,quiet=quiet
  lf.deg_rlook = 3
  
  ;
  ; theta-regression.  radar look (azimuth) position as a function of longitude and latitude
  
  
  
  fcheck=finite(rlook_coef)
  w=where(fcheck eq 0, fcount)
  if fcount gt 0 then begin
    print, 'There is a problem with the incident angle fit'
    print, 'Input incident angles'
    print, rlook
    stop
  endif
  
  if keyword_set(diagnostic) then begin
    ns=n_elements(fidx)
    iid = iid[fidx]
    jjd = jjd[fidx]
    
    
    print, 'Radar Look (azimuth) angle conversions:'
    mu=0
    sd=0.
    for i=0, ns -1 do begin
      i1=long(iid[i]+0.5)
      j1=lon(jjd[i]+0.5)
      output=          string(i1, format='(I7)')
      output= output + string(j1, format='(I7)')
      output= output + string( rlook[i] )
      rlookp=   ans_poly_cal( iid[i], jjd[i], rlook_coef, rlook_xexp, rlook_yexp )
      mu= mu+  (rlook[i]-rlookp)
      sd= sd + (rlook[i]-rlookp)^2
      output= output + string( rlookp )
      print, output
    endfor
    print, 'Fractional error in Radar Look (azimuth) angle', mu/ns, sqrt(sd/ns)
    stop
  endif
  
  
  ;
  ; Add to structure
  ;
  
  temp_lf= create_struct( lf, 'rlook_coef', rlook_coef, $
    'rlook_xexp', rlook_xexp, $
    'rlook_yexp', rlook_yexp)
    
    
  lf=      0
  lf=      temp_lf
  
  
end
