;-----------------------------------------------------
;+
; NAME: TSAR_DN2SIGMA
; PURPOSE:
;    The purpose of this program is to convert the TerraSAR-X digital numbers
;    to radar cross section. We assume the digital numbner has been squared
; CALLING SEQUENCE:
;  tsar_dn2sigma, image, lf
; INPUT:
;     image=       Array containing the digital number image
;     lf=          Leader information structure           
; OUTPUT:
;     image=       A floating point array of sigma.
; KEYWORDS=
;     /quiet=      Do not print intermediate results.
;     /help=       List help.
;
; MODIFICATION HISTORY:
; 2007 Oct 30 Frank Monaldo 
; 2009 Dec 06 Rewrite for ANSWRS 2.0
; 2010 Jan 06 Frank Monaldo. Change to use ans_compute_fit
; 2010 Jan 08 Frank Monaldo. Reformed inputs to 1d for ans_compute_fit
; 2010 Jun 14 Frank Monaldo. Print help on no input.
; 2012 Mar 21 Frank Monaldo. Change () to [] for arrays
; 2014 Feb 17 Frank Monaldo. Set calibrated sigma values < 0 the zero
; Copyright (C) 2007-2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_dn2sigma, image, lf, help=help, quiet=quiet	   

if keyword_set(help) or (n_params(0) lt 1 )  then begin
  print, ' NAME: TSAR_DN2SIGMA
  print, ' PURPOSE:
  print, '    The purpose of this program is to convert the TerraSAR-X digital numbers
  print, '    to radar cross section. We assume the digital numbner has been squared
  print, ' CALLING SEQUENCE:
  print, '  tsar_dn2sigma, image, lf
  print, ' INPUT:
  print, '     image=       Array containing the digital number image
  print, '     lf=          Leader information structure           
  print, ' OUTPUT:
  print, '     image=       A floating point array of sigma.
  print, ' KEYWORDS=
  print, '     /quiet=      Do not print intermediate results.
  print, '     /help=       List help.
  return
endif
printflag=1
if keyword_set(quiet)   then printflag=0 

sz=    size(image)
nx=    sz(1)
ny=    sz(2)
nyy=   ny / 10
image= float(image)

if printflag then begin 
  print, '---------------------------------------------------------'
  print, 'TERRASAR-X CALIBRATION: Converting digital number to sigma.'
  print, '---------------------------------------------------------'
endif


if printflag  and ny lt 10000 then begin
  print, ''
  print, 'Image values AFTER squaring:'
  w=     where(image ne 0,count)
  if count gt 0 and count lt 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Mean of squared TerraSAR-X data points=', mean(image[w])
    print, 'SD   of squared TerraSAR-X data points=', stdev(image[w])
  endif 
  if count gt 0 and count ge 4000000 then begin
    print, 'There are ' + string(count) + ' non-zero values.'
    print, 'Statistics for first 4,000,000 nonzero values.'
    print, 'Mean of squared TerraSAR-X data points from file=', mean(image[w[0:3999999]])
    print, 'SD   of squared TerraSAR-X data points from file=', stdev(image[w[0:3999999]])
  endif
endif

scaling= lf.ext_cal_fact[lf.layer]

 
if printflag then begin
  print, 'Stored lf.ext_cal_fact=                ' + strcompress(string(lf.ext_cal_fact[lf.layer]), /remove_all)
  print, 'Layer=                                 ' + strcompress(string(lf.layer)                 , /remove_all)
  print, 'Scaling factor= 1/lf.ext_cal_fact=     ' + strcompress(string(scaling)                  , /remove_all)
endif

deg2rad= !pi / 180.00
for j=0L, ny-1L do begin  ; Loop through azimuth index
  line=image[*,j]
  ;
  ; Find values in the line of the image that are >0 and finite
  ;
  w=where(line gt 0 and finite(line) eq 1, count)
  ;
  ; Continue processing line only if there
  ; are values >0 in line
  ;
  if count gt 0 then begin
    ;
    ; Compute the incidence angle along the line and
    ; convert to radians.
    ;
    xtemp=     double(w)
    ytemp=     xtemp * 0. + double(j)
    incid=     ans_poly_cal(xtemp, ytemp, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
    incid=     incid * deg2rad 
    line[w]=   line[w] * sin(incid) * scaling
    image[w,j]=line[w]
    if j mod nyy eq 0 and printflag then  begin
       print, 'Calibrating line ' + string(j, for='(I6)') + ' of ' + string(ny, for='(I6)'), mean(line[w]), mean(sin(incid))
    endif
  endif
  w=where(finite(image[*,j]) ne 1, count)
  if count gt 0 then image[w,j]=0.0
endfor ; End azimuth line loop
;
; Remove values less than zero
;
w= where(image lt 0, count)
if count gt 0 then image[w]=0.0

if printflag then begin
  print, 'External calibration factor=' +string(lf.ext_cal_fact[lf.layer])
  print, ' for layer: ' + string(lf.layer)
  print, 'Last line incidence angles:   Min= ' +  $
          string(min(incid)*180/!pi) + ' Max=' +string(max(incid)*180/!pi) 
  sz= size(image)
  if sz[4] lt 4e6 then begin
    print, 'Mean of NRCS=' + strcompress(string(mean(image)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(image))))
    w= where(image gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(image[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(image[w]))))
    endif
  endif else begin
    print, 'Number of array elements = ' +string(sz[4]) + ' Too large. To compute full statistics.'
    print, 'Computing statistics from first 500 lines'
    subimage= image[*,0:499]
    print, 'Mean of NRCS=' + strcompress(string(mean(subimage)))
    print, 'SD   of NRCS=' + strcompress(string(sqrt(variance(subimage))))
    w= where(subimage gt 0, count )
    if count gt 0 then begin
      print, 'Mean of nonzero NRCS=' + strcompress(string(mean(subimage[w])))
      print, 'SD   of nonzero NRCS=' + strcompress(string(sqrt(variance(subimage[w]))))
    endif
  endelse
endif 


end
