;-----------------------------------------------------
;+
; NAME: TSAR_LONLATINC_FIT
; PURPOSE:
;   Generate a polynomical relationship between i and j (pixel and row) in the the image
;   and longitude, latitude and incident angle
;   The program tsar_read_mapping_grid reads a binary mapping grid file and produces 
;   an array of i and j and the associted t and tau values.  The program
;   tsar_read_georef reals an XML file and returns arrays of associated
;   t, tau, longitude, latitude, and incident angle. What we would like
;   is arrays of associated i, j and longitude, latitude and incidence. This program
;   takes the first i,j,t, tau arrays and computes a polynomial relationship
;   betweem i, j in the TSAR image and t and tau, essentially range and azimuth.
;   The program uses the t, tau, longtiude, latitude, and incident angle arrays
;   to compute polynominal relationships between t and tau and longitude,
;   latitude, and incidence. the relationships are combined so that we can 
;   produce polynomical relationships between i,j and longitude, latitude,
;   and incident angle.
; CALLING SEQUENCE
;   tsar_lonlatinc_fit, i, j, t1, tau1, t2, tau2, longitude, latitude, incidence, lf
; INPUT   
;   i=         pixel     array from the tsar_read_mapping_grid program
;   j=         row       array from the tsar_read_mapping_grid program
;   t1=        t         array from the tsar_read_mapping_grid program
;   tau1=      tau       array from the tsar_read_mapping_grid program
;   t2=        t         array from the tsar_read_georef       program
;   tau2=      tau       array from the tsar_read_georef       program
;   longitude= longitude array from the tsar_read_georef       program
;   latitude=  latitude  array from the tsar_read_georef       program
;   incidence= incidence array from the tsar_read_georef       program
; OUTPUT
;   lf:  lf structure to which the new fit parameters have been added. 
; KEYWORDS:
;   /quiet:       Do not print screen output
;   /help:        Get help
;   /diagnostic   Show diagnostic output
; MODIFICATION HISTORY:
;    2009 Oct 14 Frank Monaldo Rewrite for ANSWRS 2.0
;    2010 Jan 06 Frank Monaldo Added changes to use ans_compute_fit routine
;    2010 Jan 08 Frank Monaldo Reformed inputs to 1d for ans_compute_fit
;    2010 Jun 14 Frank Monaldo. Print help on no input
;    2011 Jan 01 Frank Monaldo. Pass on quiet flag
; Copyright (C) 200-2010, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_lonlatinc_fit, i, j, t1, tau1, t2, tau2, longitude, latitude, incidence, lf, $
                        help=help, quiet=quiet, diagnostic=diagnostic

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: TSAR_LONLATINC_FIT
  print, ' PURPOSE:
  print, '   Generate a polynomical relationship between i and j (pixel and row) in the the image
  print, '   and longitude, latitude and incident angle
  print, '   The program tsar_read_mapping_grid reads a binary mapping grid file and produces 
  print, '   an array of i and j and the associted t and tau values.  The program
  print, '   tsar_read_georef reals an XML file and returns arrays of associated
  print, '   t, tau, longitude, latitude, and incident angle. What we would like
  print, '   is arrays of associated i, j and longitude, latitude and incidence. This program
  print, '   takes the first i,j,t, tau arrays and computes a polynomial relationship
  print, '   betweem i, j in the TSAR image and t and tau, essentially range and azimuth.
  print, '   The program uses the t, tau, longtiude, latitude, and incident angle arrays
  print, '   to compute polynominal relationships between t and tau and longitude,
  print, '   latitude, and incidence. the relationships are combined so that we can 
  print, '   produce polynomical relationships between i,j and longitude, latitude,
  print, '   and incident angle.
  print, ' CALLING SEQUENCE
  print, '   tsar_lonlatinc_fit, i, j, t1, tau1, t2, tau2, longitude, latitude, incidence, lf
  print, ' INPUT   
  print, '   i=         pixel     array from the tsar_read_mapping_grid program
  print, '   j=         row       array from the tsar_read_mapping_grid program
  print, '   t1=        t         array from the tsar_read_mapping_grid program
  print, '   tau1=      tau       array from the tsar_read_mapping_grid program
  print, '   t2=        t         array from the tsar_read_georef       program
  print, '   tau2=      tau       array from the tsar_read_georef       program
  print, '   longitude= longitude array from the tsar_read_georef       program
  print, '   latitude=  latitude  array from the tsar_read_georef       program
  print, '   incidence= incidence array from the tsar_read_georef       program
  print, ' OUTPUT
  print, '   lf:  lf structure to which the new fit parameters have been added. 
  print, ' KEYWORDS:
  print, '   /quiet:       Do not print screen output
  print, '   /help:        Get help
  print, '   /diagnostic   Show diagnostic output
  return
endif

if printflag then begin
  print,' '
  print,'Computing fits of i, j, t1, tau1, t1,tau2, longitude, latitude, and incidence'
  hlp, i,j,t1,tau1,t2,tau2,longitude,latitude, incidence
endif
if lf.ave_pixel gt 1 then i= i/float(lf.ave_pixel)
if lf.ave_line  gt 1 then j= j/float(lf.ave_line)
;
; Compute relationship (i,j) and t1
;                      (i,j) and tau1
;

;---------------------------------------------------------------
; We wish to find equations of the form form
;  t1=    sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n
;  tau1=  sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n


ntotal=  long(n_elements(i))
i=       reform(i,     ntotal)
j=       reform(j,     ntotal)
t1=      reform(t1,    ntotal)
tau1=    reform(tau1,  ntotal)

if printflag then print, 'Compute fit of i/j => t1'
ans_compute_fit,i, j, t1,   lf.deg, t1_coef,   t1_xexp,   t1_yexp, quiet=quiet

if printflag then print, 'Compute fit of i/j => tau1'
ans_compute_fit,i, j, tau1, lf.deg, tau1_coef, tau1_xexp, tau1_yexp, quiet=quiet


if keyword_set(diagnostic) then begin
  t_test=   ans_poly_cal( i, j, t1_coef,   t1_xexp,  t1_yexp)
  tau_test= ans_poly_cal( i, j, tau1_coef, tau_xexp, tau_yexp)
  print, ''
  print, 'Comparison statistics:'
  print, '------------------------'
  print, 'Computing t from i and j'
  print, '  Mean difference=      ', mean(t1-t_test)
  print, '  Std Dev of difference=', stdev(t1-t_test)
  print, '  Fractional difference=', stdev(t1-t_test)/ mean(abs(t1))
  print, 'Computing tau from i and j'
  print, '  Mean difference=      ', mean(tau1-tau_test) 
  print, '  Std Dev of difference=', stdev(tau1-tau_test) 
  print, '  Fractional difference=', stdev(tau1-tau_test)/ mean(abs(tau1))
  stop
endif



;---------------------------------------------------------------
; We wish to find equations of the form form
;  i=    sum_m=0^deg sum_n=0^deg A(m,n) *t1^m tau1^n
;  j=    sum_m=0^deg sum_n=0^deg A(m,n) *t1^m tau1^n

if printflag then print, 'Compute fit of t1, tau1 => i'
ans_compute_fit, t1,  tau1,  i,  lf.deg, it_coef, it_xexp, it_yexp, quiet=quiet

if printflag then print, 'Compute fit of t1, tau1 => j'
ans_compute_fit, t1,  tau1,  j,  lf.deg, jt_coef, jt_xexp, jt_yexp, quiet=quiet
;

if keyword_set(diagnostic) then begin
  i_test=   ans_poly_cal( t1, tau1, it_coef, it_xexp, it_yexp)
  j_test=   ans_poly_cal( t1, tau1, jt_coef, jt_xexp, jt_yexp)
  print, ''
  print, 'Comparison statistics:'
  print, '------------------------'
  print, 'Computing i from t and tau'
  print, '  Mean difference=      ', mean(i-i_test)
  print, '  Std Dev of difference=', stdev(i-i_test)
  print, '  Fractional difference=', stdev(i-i_test)/ mean(abs(i))
  print, 'Computing j from t and tau'
  print, '  Mean difference=', mean(j-j_test)
  print, '  Std Dev of difference=',stdev(j-j_test) 
  print, '  Fractional difference=', stdev(j-j_test)/mean(abs(j))
  stop
endif

;
; Compute relationship (t,tau) and longitude, latitude, incidence
;                      (longitude, latitude) and t and tau
;
;---------------------------------------------------------------
; We wish to find equations of the form form
;  longitude=    sum_m=0^deg sum_n=0^deg A(m,n) *t^m tau^n
;  latitude=     sum_m=0^deg sum_n=0^deg A(m,n) *t^m tau^n
;  incidence=    sum_m=0^deg sum_n=0^deg A(m,n) *t^m tau^n


ntotal=    long(n_elements(t2))
t2=        reform(t2,        ntotal)
tau2=      reform(tau2,      ntotal)
longitude= reform(longitude, ntotal)
latitude=  reform(latitude,  ntotal)
incidence= reform(incidence, ntotal)

if printflag then print, 'Compute fit of t2, tau2 => longitude'
ans_compute_fit, t2,  tau2,  longitude,  lf.deg, lont_coef, lont_xexp, lont_yexp, quiet=quiet
if printflag then print, 'Compute fit of t2, tau2 => latitude'
ans_compute_fit, t2,  tau2,  latitude,   lf.deg, latt_coef, latt_xexp, latt_yexp, quiet=quiet
if printflag then print, 'Compute fit of t2, tau2 => incidence'
ans_compute_fit, t2,  tau2,  incidence,  lf.deg, inct_coef, inct_xexp, inct_yexp, quiet=quiet


if keyword_set(diagnostic) then begin
  lon_test=   ans_poly_cal( t2, tau2, lont_coef, lont_xexp, lont_yexp)
  lat_test=   ans_poly_cal( t2, tau2, latt_coef, latt_xexp, latt_yexp)
  inc_test=   ans_poly_cal( t2, tau2, inct_coef, inct_xexp, inct_yexp)
  print, ''
  print, 'Comparison statistics:'
  print, '------------------------'
  print, 'Computing longitude from t and tau'
  print, '  Mean difference=      ', mean(longitude-lon_test)
  print, '  Std Dev of difference=', stdev(longitude-lon_test)
  print, '  Fractional difference=', stdev(longitude-lon_test)/mean(abs(longitude))  
  print, 'Computing latitude from t and tau'
  print, '  Mean difference=      ', mean(latitude-lat_test)
  print, '  Std Dev of difference=', stdev(latitude-lat_test) 
  print, '  Fractional difference=', stdev(latitude-lat_test)/mean(abs(latitude)) 
  print, 'Computing incidence from t and tau'
  print, '  Mean difference=     ', mean(incidence-inc_test)
  print, '  Std Dev of difference=',stdev(incidence-inc_test)  
  print, '  Fractional difference=', stdev(incidence-inc_test)/mean(abs(incidence)) 
  stop
endif

;
; We now need to compute the relationships between (i,j) and longitude, latitude, and incidence angle
;
;
;---------------------------------------------------------------
; We wish to find equations of the form form
;  longitude=    sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n
;  latitude=     sum_m=0^deg sum_n=0^deg A(m,n) *i^m j^n
;  incidence=    sum_m=0^deg sum_n=0^deg A(m,n) *j^m j^n
;
; Compute i and j indices for the t and tau given in the GEOREF.xml file
ii= ans_poly_cal(t2, tau2, it_coef, it_xexp, it_yexp)
jj= ans_poly_cal(t2, tau2, jt_coef, jt_xexp, jt_yexp)

if printflag then print, 'Compute fit of ii, jj => longitude'
ans_compute_fit, ii, jj,  longitude,  lf.deg, lon_coef,   lon_xexp,   lon_yexp, quiet=quiet
if printflag then print, 'Compute fit of ii, jj => latitude'
ans_compute_fit, ii, jj,  latitude,   lf.deg, lat_coef,   lat_xexp,   lat_yexp, quiet=quiet
if printflag then print, 'Compute fit of ii, jj => incidence'
ans_compute_fit, ii, jj,  incidence,  lf.deg, incid_coef, incid_xexp, incid_yexp, quiet=quiet

if keyword_set(diagnostic) then begin
  lon_test=   ans_poly_cal( ii, jj, lon_coef,   lon_xexp,   lon_yexp   )
  lat_test=   ans_poly_cal( ii, jj, lat_coef,   lat_xexp,   lat_yexp   )
  inc_test=   ans_poly_cal( ii, jj, incid_coef, incid_xexp, incid_yexp )
  print, ''
  print, 'Comparison statistics:'
  print, '------------------------'
  print, 'Computing longitude from i and j'
  print, '  Mean difference=      ', mean(longitude-lon_test)
  print, '  Std Dev of difference=', stdev(longitude-lon_test)
  print, '  Fractional diffferece=', stdev(longitude-lon_test) / mean(abs(longitude))
  print, 'Computing latitude from i and j'
  print, '  Mean difference=      ', mean(latitude-lat_test)
  print, '  Std Dev of difference=' ,stdev(latitude-lat_test) 
  print, '  Fractional diffferece=', stdev(latitude-lat_test) / mean(abs(latitude)) 
  print, 'Computing incidence from i and j'
  print, '  Mean difference=      ', mean(incidence-inc_test)
  print, '  Std Dev of difference=' ,stdev(incidence-inc_test)   
  print, '  Fractional difference=', stdev(incidence-inc_test) / mean(abs(incidence))
  stop
endif

;
;---------------------------------------------------------------
; We wish to find equations of the form form
;  i=    sum_m=0^deg sum_n=0^deg A(m,n) *longitude^m latitude^n
;  j=    sum_m=0^deg sum_n=0^deg A(m,n) *longitude^m latitude^n
;
if printflag then print, 'Compute fit of longitude/latitude => i'
ans_compute_fit, longitude, latitude, ii, lf.deg, i_coef, i_xexp, i_yexp, quiet=quiet
if printflag then print, 'Compute fit of longitude/latitude => j'
ans_compute_fit, longitude, latitude, jj, lf.deg, j_coef, j_xexp, j_yexp, quiet=quiet


if keyword_set(diagnostic) then begin
  i_test=   ans_poly_cal( longitude, latitude, i_coef, i_xexp, i_yexp)
  j_test=   ans_poly_cal( longitude, latitude, j_coef, j_xexp, j_yexp)
  print, ''
  print, 'Comparison statistics:'
  print, '------------------------'
  print, 'Computing i from longitude and latitude'
  print, '  Mean difference=      ', mean(ii-i_test)
  print, '  Std Dev of difference=', stdev(ii-i_test)
  print, '  Fractional diffferece=', stdev(ii-i_test) / mean(abs(ii)) 
  print, 'Computing j from longitude and latitude'
  print, '  Mean difference=      ', mean(jj-j_test)
  print, '  Std Dev of difference=', stdev(jj-j_test)   
  print, '  Fractional diffferece=', stdev(jj-j_test) / mean(abs(jj)) 
  stop
endif

temp_lf= create_struct( lf, 'lon_coef',   lon_coef,   $
                            'lon_xexp',   lon_xexp,   $
                            'lon_yexp',   lon_yexp,   $
                            'lat_coef',   lat_coef,   $
                            'lat_xexp',   lat_xexp,   $
                            'lat_yexp',   lat_yexp,   $
                            'i_coef',     i_coef,     $
                            'i_xexp',     i_xexp,     $
                            'i_yexp',     i_yexp,     $                            
                            'j_coef',     j_coef,     $
                            'j_xexp',     j_xexp,     $
                            'j_yexp',     j_yexp,     $                              
                            'incid_coef', incid_coef, $
                            'incid_xexp', incid_xexp, $
                            'incid_yexp', incid_yexp  )
lf=    0
lf=    temp_lf
                            

end
