;-----------------------------------------------------
;+
; NAME: TSAR_R_LOOK
; PURPOSE:
;    The purpose of this program is to compute the
;    radar look direction in degrees with respect to 
;    North.  I.e. 90= east. Using the convention
;    of +/-180
; CALLING SEQUENCE:
;    tsar_r_look, lf
; INPUT:
;    lf,  Structure
; OUTPUT:
;    Added to structure
;    rlook_coef= Radar look direction wrt N in degrees, Coefficients
;                to compute r_look as a function of i,j
; KEYWORDS:
;    /quiet=           Print no messages while processing.
;    /help=            Get help.
; MODIFICATION HISTORY:
;   2003 Mar 29 Frank Monaldo                 
;   2009 Nov 25 Code changed to accommodate ANSWRS 2.0
;   2009 Dec 06 Frank Monaldo
;   2010 Jan 08 Change fix to long in lf.ny1 computation
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Jan 01 Frank Monaldo. Pass on quiet flag
; Copyright (C) 2003-2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_r_look, lf, r_look, help=help, quiet=quiet
		   
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: TSAR_R_LOOK
  print, ' PURPOSE:
  print, '    The purpose of this program is to compute the
  print, '    radar look direction in degrees with respect to 
  print, '    North.  I.e. 90= east. Using the convention
  print, '    of +/-180
  print, ' CALLING SEQUENCE:
  print, '    tsar_r_look, lf
  print, ' INPUT:
  print, '    lf,  Structure
  print, ' OUTPUT:
  print, '    Added to structure
  print, '    rlook_coef= Radar look direction wrt N in degrees, Coefficients
  print, '                to compute r_look as a function of i,j
  print, ' KEYWORDS:
  print, '    /quiet=           Print no messages while processing.
  print, '    /help=            Get help.
  return
endif

printflag=1
if keyword_set(quiet)   then printflag=0 
quiet= printflag eq 0

w=where(lf.sub_sat_track ne 0, count)
if count eq 0 then begin
  if printflag then print, 'There is no heading data embedded in the file...'
  if printflag then print, 'Inferring heading from lats and long at edges of the image...'
  ll2rb, lf.near_start_lon, lf.near_start_lat, lf.near_end_lon, lf.near_end_lat, $
          dist, azi 
  r_look=azi + 90.0  ; Assumes the SAR is right-looking.
  if printflag then print, 'Computed single look direction ' + string(r_look)
endif else begin
  if printflag then print, 'r_look from lf.sub_sat_track + 90' + string(mean(lf.sub_sat_track(w)))
endelse

;
; Computing two dimensional fit to radar look direction.First we must determine if the range 
; increases with pixel
;
range_direction= 0 ; Range increases with pixel.
first_inc=        ans_poly_cal(0,      0, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
last_inc=         ans_poly_cal(lf.nx1, 0, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
if last_inc lt first_inc then range_direction=  1
; 
look=   dblarr(13)
ii=     dblarr(13)
jj=     dblarr(13)

lf.nx1=   long( lf.nx0 / lf.ave_pixel )
lf.ny1=   long( lf.ny0 / lf.ave_line  )
;print, lf.nx0, lf.nx1
;print, lf.ny0, lf.ny1
if printflag then begin   
  print, '     i      j  look direction (d)'
  print, '------------------------------------------'
endif
for i=0, 12 do begin
  case i of
  0:  begin
      ii[i]= 0
      jj[i]= 0
      end
  1:  begin
      ii[i]= lf.nx1 / 2
      jj[i]= 0
      end  
  2:  begin
      ii[i]= lf.nx1 - 2
      jj[i]= 0
      end  
  3:  begin
      ii[i]= 0
      jj[i]= lf.ny1 / 2
      end
  4:  begin
      ii[i]= lf.nx1 / 2
      jj[i]= lf.ny1 / 2
      end  
  5:  begin
      ii[i]= lf.nx1 - 2
      jj[i]= lf.ny1 / 2
      end    
  6:  begin
      ii[i]= 0
      jj[i]= lf.ny1 - 2
      end
  7:  begin
      ii[i]= lf.nx1 / 2
      jj[i]= lf.ny1 - 2 
      end  
  8:  begin
      ii[i]= lf.nx1 - 2
      jj[i]= lf.ny1 - 2
      end      
  9:  begin
      ii[i]= lf.nx1 / 3.
      jj[i]= lf.ny1 / 3.
      end
  10: begin
      ii[i]= lf.nx1 / 3.
      jj[i]= lf.ny1 * 2. / 3. 
      end
  11: begin
      ii[i]= lf.nx1 * 2 / 3.
      jj[i]= lf.ny1 / 3. 
      end  
  12: begin
      ii[i]= lf.nx1 * 2. / 3.
      jj[i]= lf.ny1 * 2. / 3. 
      end 
  endcase   
  increment= 1
  if range_direction then increment= -1
  lon1= ans_poly_cal( ii[i],             jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
  lat1= ans_poly_cal( ii[i],             jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp )
  lon2= ans_poly_cal( ii[i] + increment, jj[i], lf.lon_coef, lf.lon_xexp, lf.lon_yexp )
  lat2= ans_poly_cal( ii[i] + increment, jj[i], lf.lat_coef, lf.lat_xexp, lf.lat_yexp )
  ll2rb, lon1, lat1, lon2, lat2, dist, azi
  if azi gt 180 then azi= azi - 360.0
  look[i] = double(azi) 
  if printflag then begin   
    print, string( ii[i],   format='(I6)'  ) + ' ' + $
           string( jj[i],   format='(I6)'  ) + ' ' + $
           string( look[i], format='(F9.3)')
  endif
endfor

if printflag then print, 'Compute fit of i.j=> rlook direction'
ans_compute_fit, ii,  jj,  look,  lf.deg, rlook_coef, rlook_xexp, rlook_yexp, quiet=quiet

;
; Add to structure
;

temp_lf= create_struct( lf, 'rlook_coef', rlook_coef, $
                            'rlook_xexp', rlook_xexp, $
                            'rlook_yexp', rlook_yexp  )
lf=      0
lf=      temp_lf


end
