;;-----------------------------------------------------
;+
; NAME: TSAR_READ_GEOREF.PRO
; PURPOSE:
;  Read main Terra SAR-X GEOREFile
; CALLING SEQUENCE
;   tsar_read_georef, filename, lf, incidence, latitude, longitude, t, tau, /help, /quiet
; INPUT
;  filename:  GEOREF files  
;  lf:        Strucuture which is update
; OUTPUT
;  incidence:      incidence angle array
;  latitude:       latitude array
;  longitude:      longitude array
;  t:              time array
;  tau:            tau array
; KEYWORDS
;  error:          error=-1 if bad
; MODIFICATION HISTORY:
; 2007 Oct 15 Frank Monaldo   
; 2007 Oct 18 Add provision for reading t and tau
; 2007 Oct 19 Read t and tau reference frames
; 2009 Dec 07 Rewritten for ANSWRS 2.0
; 2010 Jan 13 Frank Monaldo. Added more elements to structure.
;                            x,y,z position and velocity
; 2010 Jun 14 Frank Monaldo. Print help on no input.
; 2011 Feb 05 Frank Monaldo. Add error return if cannot read
; 2011 Mar 20 Frank Monaldo. Correct structure
; Copyright (C) 2007, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_read_georef, filename, lf, incidence, latitude, longitude,  t, tau, $
                      help=help, quiet=quiet, error=error

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: TSAR_READ_GEOREF.PRO
  print, ' PURPOSE:
  print, '  Read main Terra SAR-X GEOREFile
  print, ' CALLING SEQUENCE
  print, '   tsar_read_georef, filename, lf, incidence, latitude, longitude, t, tau, /help, /quiet
  print, ' INPUT
  print, '  filename:  GEOREF files  
  print, '  lf:        Strucuture which is update
  print, ' OUTPUT
  print, '  incidence:      incidence angle array
  print, '  latitude:       latitude array
  print, '  longitude:      longitude array
  print, '  t:              time array
  print, '  tau:            tau array
  print, ' KEYWORDS
  print, '  error:          error=-1 if bad
  return
endif       

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  


if printflag then begin
  print, 'Reading general header information with tsar_read_georef'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: tsar_read_georef' 
  return
endif

if printflag then print, 'Reading file: ' + filename
s= getfile(filename)
n= n_elements(s)

if printflag then print, 'Parsing:      ' + filename
;
;
;
; ----------------------------------------------------
;             Ellipsoid
; ---------------------------------------------------
ellpsoid=''       
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'ellipsoid')
w=where(index ge 0,count)
if count gt 0 then begin
  istart=    w[0]
  ss=        s(istart)
  ss=        repchr( ss, '<', ' ' )
  ss=        repchr( ss, '>', ' ' )
  ss=        repchr( ss, '/', ' ' )
  ellipsoid= getwrd( ss, 1)
endif

;
; ----------------------------------------------------
;             semiMajorAxis
; ---------------------------------------------------
semiMajorAxis='-999999.99'       
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'semiMajorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  istart=        w[0]
  ss=            s(istart)
  ss=            repchr( ss, '<', ' ' )
  ss=            repchr( ss, '>', ' ' )
  ss=            repchr( ss, '/', ' ' )
  semiMajorAxis= getwrd( ss, 1)
endif

;
; ----------------------------------------------------
;             semiMinorAxis
; ---------------------------------------------------
semiMinotAxis='-999999.99'         
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'semiMinorAxis')
w=where(index ge 0,count)
if count gt 0 then begin
  istart=        w[0]
  ss=            s(istart)
  ss=            repchr( ss, '<', ' ' )
  ss=            repchr( ss, '>', ' ' )
  ss=            repchr( ss, '/', ' ' )
  semiMinorAxis= getwrd( ss, 1)
endif


;
; ----------------------------------------------------
;             Datum Shifts
; ---------------------------------------------------
datashiftx=    '0.0000000000'
datashifty=    '0.0000000000'
datashiftz=    '0.0000000000'
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'datumShift')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=    w[0]
  iend=      w[1]
  for i=istart, iend do begin
    ss=  s[i]
    ss=  repchr( ss, '<', ' ' )
    ss=  repchr( ss, '>', ' ' )
    ss=  repchr( ss, '/', ' ' )
    key= getwrd(ss, 0)
    case key of 
    'x':    dataShiftx=getwrd(ss, 1)
    'y':    dataShifty=getwrd(ss, 1)
    'z':    dataShiftz=getwrd(ss, 1)
    ELSE: ; do nothing
    endcase
  endfor
endif

;---------------------------------
; Positions and velocities
;-------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'posX')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.x_position=double(getwrd(ss, 1))
endif

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'posY')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.y_position=double(getwrd(ss, 1))
endif

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'posZ')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.z_position=double(getwrd(ss, 1))
endif

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'velX')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.x_velocity=double(getwrd(ss, 1))
endif

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'velY')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.y_velocity=double(getwrd(ss, 1))
endif

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'velZ')
w=where(index ge 0,count)
if count gt 0 then begin
  ss=  s[w[0]]
  ss=  repchr( ss, '<', ' ' )
  ss=  repchr( ss, '>', ' ' )
  ss=  repchr( ss, '/', ' ' )
  lf.z_velocity=double(getwrd(ss, 1))
endif

;
; Add to lf
;
lf.ellipsoidName=        ellipsoid
lf.semiMajorAxis=        double(semiMajorAxis)
lf.semiMinorAxis=        double(semiMinorAxis)
lf.datumshiftparameters= [ double(datashiftx), double(datashifty),  double(datashiftz) ] 
if printflag then begin
   print, 'lf.ellipsoidName=', lf.ellipsoidName
   print, 'lf.semiMajorAxis=', lf.semiMajorAxis
   print, 'lf.semiMinorAxis=', lf.semiMinorAxis
endif



; ----------------------------------------------------
;                   GENERAL GET THE SIZE OF THE ARRAY
; ---------------------------------------------------
       
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'geolocationGrid')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=  w[0]
  iend=    w[1]
  index2=  index[w[0]:w[1]]
  s2=      s[w[0]:w[1]]
  n=       n_elements(s2)
  index2= intarr(n) & istart=-1 & iend=-1
  for i=0L, n-1 do index2[i]= strpos(s2[i], 'numberOfGridPoints')
  w=where(index2 ge 0,count)
  if count gt 0 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s2[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'azimuth':   nazimuth= long(getwrd(ss,1))
      'range':     nrange=   long(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif
; ----------------------------------------------------
;                   GENERAL GRID REFERNCE TIMES
; ---------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'gridReferenceTime')
w=where(index ge 0,count)
if count ge 2 then begin
  tReferenceTimeUTC=''
  tauReferenceTime= 0.0
  if count ge 2 then begin
    istart=  w[0]
    iend=    w[1]
    for i=istart, iend do begin
      ss= s[i]
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss, 0)
      case key of
      'tReferenceTimeUTC':    tReferenceTimeUTC=         getwrd(ss,1)
      'tauReferenceTime':     tauReferenceTime=   double(getwrd(ss,1))
      ELSE: ; No operation
      endcase
    endfor
  endif
endif


incidence= dblarr(nrange, nazimuth)
longitude= dblarr(nrange, nazimuth)
latitude=  dblarr(nrange, nazimuth)
t=         dblarr(nrange, nazimuth)
tau=       dblarr(nrange, nazimuth)

; ----------------------------------------------------
;                   FILL IN ARRAY
; ---------------------------------------------------
n= n_elements(s)      
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<gridPoint')
w1=where(index ge 0,count1)
for i=0L, n-1 do index[i]= strpos(s[i], '</gridPoint>')
w2=where(index ge 0,count2)
count= min([count1, count2])
count=long(count)
if count gt 0 then begin
  for cc=0L, count - 1L do begin
    istart=w1[cc]
    iend=  w2[cc]
    for i=istart, iend do begin
      if printflag and (i mod 10000) eq 0 then begin
         print, 'Parsing line: ' + string(i, format='(I8)') + ' for t, tau, lat, lon, inc...'
      endif
      ss=  s[i]
      ss=  repchr( ss, '<', ' ' )
      ss=  repchr( ss, '>', ' ' )
      ss=  repchr( ss, '/', ' ' )
      key= getwrd( ss,0)
      case key of
      'gridPoint': begin
                   irg=                  long(  getwrd(ss, 3, del='"'))-1
                   iaz=                  long(  getwrd(ss, 1, del='"'))-1
                   end
      't':         begin     
                   if irg ge 0 and iaz ge 0 then t(irg,iaz)=         double( getwrd(ss, 1) )  
                   end                                                                   
      'tau':       begin                                                                 
                   if irg ge 0 and iaz ge 0 then tau(irg,iaz)=       double( getwrd(ss, 1) ) 
                   end                                                                   
      'lat':       begin                                                                 
                   if irg ge 0 and iaz ge 0 then latitude(irg,iaz)=  double( getwrd(ss, 1) ) 
                   end                                                                   
      'lon':       begin                                                                 
                   if irg ge 0 and iaz ge 0 then longitude(irg,iaz)= double( getwrd(ss, 1) )
                   end                                                                   
      'inc':       begin                                                                 
                   if irg ge 0 and iaz ge 0 then incidence(irg,iaz)= double( getwrd(ss, 1) )
                   end
      ELSE: ; No operation 
      endcase
    endfor    ; End loop i
  endfor    ; End loop cc
endif       ; End if count gt 0

tau= tau +   tauReferenceTime
end
