;;-----------------------------------------------------
;+
; NAME: TSAR_READ_GHEADER.PRO
; PURPOSE:
;  Read main Terra SAR-X header file
; CALLING SEQUENCE
;   tsar_read_gheader, filename, ghead, /help, /quiet
; INPUT
;  filename:  XML file with general information                                   
; OUTPUT
;  ghead:     Structure with general heasder information.
; KEYWORDS:
;  error= error, error=1 if cannot read data
; MODIFICATION HISTORY:
;  2007 Oct 10 Frank Monaldo  
;  2007 Oct 14 Frank Monaldo. Reads the calibration and polarizations of different layers.
;  2007 Oct 15 Frank Monaldo. Add mapgrid and created arrays of files 
;  2010 Jan 13 Frank Monaldo. Added ellipsoidName, datumshiftparameters,
;                             semimajor/semiminor axes to ghead structure
;  2010 Jun 14 Frank Monaldo. Print help on no input.
;  2011 Feb 08 Frank Monaldo. Add error=-1 if cannot read file
;  2011 May 19 Frank Monaldo. Create a long integer index to correct error 
;  2011 May 19 Frank Monaldo. In parsing the xml file there was an error
;                             sometimes in getting the mapping grid file
;  2011 May 23 Frank Monaldo. Add jsbegin and jsend to structure
; Copyright (C) 2007, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_read_gheader, filename, ghead,  help=help, quiet=quiet, error=error

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help) or (n_params(0) lt 1 ) then begin 
  print, ' NAME: TSAR_READ_GHEADER.PRO
  print, ' PURPOSE:
  print, '  Read main Terra SAR-X header file
  print, ' CALLING SEQUENCE
  print, '   tsar_read_gheader, filename, ghead, /help, /quiet
  print, ' INPUT
  print, '  filename:  XML file with general information              
  print, ' OUTPUT
  print, '  ghead:     Structure with general heasder information.
  print, ' KEYWORDS:
  print, '  error= error, error=1 if cannot read data
  return
endif

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  

if printflag then begin
  print, 'Reading general header information with tsar_read_gheader'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: tsar_read_gheader' 
  return
endif

if printflag then print, 'Reading file: ' + filename
s= getfile(filename)
n= n_elements(s)

if printflag then print, 'Parsing:      ' + filename
;
; Declaring ghead structure
;
ghead={ mission:                  '',               $        
        source:                   '',               $   
        generationSystem:         '',               $
        generationSystemversion:  '',               $
        generationTime:           '',               $
        mainfile:                 strarr(10),       $     
        georeffile:               strarr(10),       $
        imagedatafile:            strarr(10),       $
        mapping_gridfile:         strarr(10),       $
        compositeQuiklookfile:    strarr(10),       $
        browseimagefile:          strarr(10),       $
        mapplotfile:              strarr(10),       $     
        quicklooksfile:           strarr(10),       $ 
        pollayer:                 strarr(10),       $
        layerindex:               strarr(10),       $
        orbitDirection:           '',               $
        absOrbit:                 '',               $
        sensor:                   '',               $
        imagingMode:              '',               $
        lookDirection:            '',               $
        productType:              '',               $
        projection:               '',               $
        pixelValueID:             '',               $
        imageDataType:            '',               $
        imageDataFormat:          '',               $
        imageDataDepth:           '',               $
        imageStorageOrder:        '',               $
        numberoflayers:           '',               $
        rowContent:               '',               $
        columnContent:            '',               $
        numberOfRows:             '',               $
        numberOfColumns:          '',               $
        rowSpacing:               '',               $
        columnSpacing:            '',               $
        groundRangeResolution:    '',               $
        azimuthResolution:        '',               $
        azimuthLooks:             '',               $
        rangeLooks:               '',               $
        starttimeUTC:             '',               $
        stoptimeUTC:              '',               $
        headingAngle:             '',               $
        centerlat:                '',               $
        centerlon:                '',               $
        centerincidenceangle:     '',               $
        calFactor:                strarr(10),       $
        tReferenceTimeUTC:        '',               $
        tauReferenceTime:         '',               $ 
        mapgridnumberofrows:      '',               $
        mapgridnumberofcolumns:   '',               $
        mapgridrowspacing:        '',               $
        mapgridcolumnspacing:     '',               $
        mapgridrefrow:            '',               $
        mapgridrefcol:            '',               $
        cornerrefrow:             strarr(4),        $
        cornerrefcolumn:          strarr(4),        $
        cornerlat:                strarr(4),        $
        cornerlon:                strarr(4),        $
        cornerincidenceangle:     strarr(4),        $
        centerfrequency:          '',               $
        ellipsoidname:            '',               $
        datumshiftparameter:      [0.0, 0.0,0.0],   $
        semimajoraxis:            0.0D,             $
        semiminoraxis:            0.0D,             $
        ellipsoidscale:           '',               $
        jsbegin:   -3155673600D,                    $   ; Default setting for start data
        jsend:     -3155673600D                     }   ; Default setting for end  data

; ----------------------------------------------------
;                   GENERAL HEADER
; ---------------------------------------------------
       
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'generalHeader')

w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i]
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'mission':                ghead.mission=                   getwrd(ss,1)
    'source':                 ghead.source=                    getwrd(ss,1)
    'generationSystem':       begin
                              ghead.generationsystemversion=   getwrd(getwrd(ss,1),1,del='"')
                              ghead.generationsystem=          getwrd(ss,2)
                              end
    'generationTime':         ghead.generationtime=            getwrd(ss,1)
    'ellipsoidID':            ghead.ellipsoid=                 getwrd(ss,1)
    'semiMajorAxis':          ghead.semiMajorAxis=             double(getwrd(ss,1))
    'semiMinorAxis':          ghead.semiMinorAxis=             double(getwrd(ss,1))
    ELSE: ; No operation
    endcase
  endfor
endif
; ----------------------------------------------------
;             PRODUCT COMPONENTS
; ---------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'productComponents')
w=where(index ge 0,count)
component=-1
lindex=0
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    if key eq 'type'               or    $     
       key eq 'imageData'          or    $
       key eq 'compositeQuicklook' or    $
       key eq 'browseImage'        or    $
       key eq 'mapPlot'            or    $
       key eq 'quicklooks'         then begin
       key2=''
      case key of 
      'type':                key2= getwrd(ss,1)
      'imageData':           key2= 'imageData'
      'compositeQuicklook':  key2= 'compositeQuicklook'
      'browseImage':         key2= 'browseImage'
      'mapPlot':             key2= 'mapPlot'
      'quicklooks':          key2= 'quicklooks'
      else:                  key2=''
      endcase
      layerindex= getwrd(ss, 1, del='"')
      lindex=0
      if layerindex ne '' then begin
        lindex= fix(layerindex)- 1
        if key2 eq 'imageData' then ghead.layerindex[lindex]= getwrd(ss, 1, del='"') 
      endif
      case key2 of 
      'MAIN':               component=1
      'GEOREF':             component=2
      'imageData':          component=3
      'MAPPING_GRID':       component=4
      'compositeQuicklook': component=5
      'browseImage':        component=6
      'mapPlot':            component=7
      'quicklooks':         component=8
      ELSE:                 component=1000
      endcase
    endif
    case key of
    'host':      begin
                 case component of
                 1:ghead.mainfile[lindex]=              getwrd(ss, 1) +  '/'
                 2:ghead.georeffile[lindex]=            getwrd(ss, 1) +  '/'
                 3:ghead.imagedatafile[lindex]=         getwrd(ss, 1) +  '/'
                 4:ghead.mapping_gridfile[lindex]=      getwrd(ss, 1) +  '/'
                 5:ghead.compositeQuiklookfile[lindex]= getwrd(ss, 1) +  '/'
                 6:ghead.browseimagefile[lindex]=       getwrd(ss, 1) +  '/'
                 7:ghead.mapplotfile[lindex]=           getwrd(ss, 1) +  '/'
                 8:ghead.quicklooksfile[lindex]=        getwrd(ss, 1) +  '/'
                 else: ; no operation
                 endcase
                 end
    'path':      begin
                 case component of
                 1:ghead.mainfile[lindex]=              ghead.mainfile[lindex]              + getwrd(ss, 1) +  '/'
                 2:ghead.georeffile[lindex]=            ghead.georeffile[lindex]            + getwrd(ss, 1) +  '/'
                 3:ghead.imagedatafile[lindex]=         ghead.imagedatafile[lindex]         + getwrd(ss, 1) +  '/'
                 4:ghead.mapping_gridfile[lindex]=      ghead.mapping_gridfile[lindex]      + getwrd(ss, 1) +  '/'
                 5:ghead.compositeQuiklookfile[lindex]= ghead.compositeQuiklookfile[lindex] + getwrd(ss, 1) +  '/'
                 6:ghead.browseimagefile[lindex]=       ghead.browseimagefile[lindex]       + getwrd(ss, 1) +  '/'
                 7:ghead.mapplotfile[lindex]=           ghead.mapplotfile[lindex]           + getwrd(ss, 1) +  '/'
                 8:ghead.quicklooksfile[lindex]=        ghead.quicklooksfile[lindex]        + getwrd(ss, 1) +  '/'         
                 else: ; no operation
                 endcase
                 end
    'filename':  begin
                 case component of
                 1:ghead.mainfile[lindex]=              ghead.mainfile[lindex]              + getwrd(ss, 1)
                 2:ghead.georeffile[lindex]=            ghead.georeffile[lindex]            + getwrd(ss, 1)
                 3:ghead.imagedatafile[lindex]=         ghead.imagedatafile[lindex]         + getwrd(ss, 1)
                 4:ghead.mapping_gridfile[lindex]=      ghead.mapping_gridfile[lindex]      + getwrd(ss, 1)
                 5:ghead.compositeQuiklookfile[lindex]= ghead.compositeQuiklookfile[lindex] + getwrd(ss, 1)
                 6:ghead.browseimagefile[lindex]=       ghead.browseimagefile[lindex]       + getwrd(ss, 1)
                 7:ghead.mapplotfile[lindex]=           ghead.mapplotfile[lindex]           + getwrd(ss, 1)
                 8:ghead.quicklooksfile[lindex]=        ghead.quicklooksfile[lindex]        + getwrd(ss, 1)
                 else: ; no operation
                 endcase
                 end
    else:                 ; No operation
    endcase
  endfor   
endif
;------------------------------------------------------------------------------
;                       POLLAYER INFORMATION
;------------------------------------------------------------------------------

for layer=1, 10 do begin
  index= intarr(n) & istart=-1 &iend=-1
  layerstr=strcompress(string(layer), /remove_all)
  for i=0L, n-1 do index[i]= strpos(s[i], '<imageData layerIndex="'+layerstr )
  w1=where(index ge 0, count1)
  if count1 gt 0 then begin
    index= index * 0
    index= index + (-1)
    for i=w1[0], n-1 do index[i]= strpos(s[i], '</imageData>'           )
    w2=where(index ge 0, count2)
    if count2 gt 0 then begin
      istart=w1[0]
      iend=  w2[0]
      for i=istart, iend do begin
        ss= s[i] 
        ss= repchr( ss, '<', ' ' )
        ss= repchr( ss, '>', ' ' )
        ss= repchr( ss, '/', ' ' )
        key= getwrd(ss, 0)
        case key of
        'imageData':         lindex=getwrd(ss, 1, del='"')-1
        'polLayer':          ghead.pollayer[lindex]= getwrd(ss, 1 )     
        else: ; No operation
        endcase 
      endfor  ; Loop lines looking for information
    endif     ; End check for count2
  endif       ; End check for count1
endfor        ; Loop through layers

;------------------------------------------------------------------------------
;                       MISSION INFO
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'missionInfo')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'absOrbit':       ghead.absorbit=       getwrd(ss, 1)
    'orbitDirection': ghead.orbitdirection= getwrd(ss, 1)
    else: ; No operation
    endcase
  endfor
endif 
;------------------------------------------------------------------------------
;                       ACQUISITION INFO
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'acquisitionInfo')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'sensor':         ghead.sensor=        getwrd(ss, 1)
    'imagingMode':    ghead.imagingMode=   getwrd(ss, 1)
    'lookDirection':  ghead.lookDirection= getwrd(ss, 1)
    else: ; No operation
    endcase
 endfor
endif 

;------------------------------------------------------------------------------
;                       PROJECTION VARIANT INFO
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 &iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'productVariantInfo')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'productType':    ghead.producttype=       getwrd(ss, 1)
    'projection':     ghead.projection=        getwrd(ss, 1)
    else: ; No operation
    endcase 
  endfor
endif 


;------------------------------------------------------------------------------
;                       IMAGE DATA INFO
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'imageDataInfo')
w=where(index ge 0,count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'pixelValueID':          ghead.pixelValueID=          getwrd(ss, 1)
    'imageDataType':         ghead.imageDataType=         getwrd(ss, 1)
    'imageDataFormat':       ghead.imageDataFormat=       getwrd(ss, 1)
    'numberOfLayers':        ghead.numberOfLayers=        getwrd(ss, 1)
    'imageDataDepth':        ghead.imageDataDepth=        getwrd(ss, 1)
    'imageStorageOrder':     ghead.imageStorageOrder=     getwrd(ss, 1)
    'rowContent':            ghead.rowContent=            getwrd(ss, 1)
    'columnContent':         ghead.columnContent=         getwrd(ss, 1)
    'numberOfRows':          ghead.numberOfRows=          getwrd(ss, 1)
    'numberOfColumns':       ghead.numberOfColumns=       getwrd(ss, 1)
    'rowSpacing':            ghead.rowSpacing=            getwrd(ss, 2)
    'columnSpacing':         ghead.columnSpacing=         getwrd(ss, 2)
    'groundRangeResolution': ghead.groundRangeResolution= getwrd(ss, 1)
    'azimuthResolution':     ghead.azimuthResolution=     getwrd(ss, 1)
    'azimuthLooks':          ghead.azimuthLooks=          getwrd(ss, 1)
    'rangeLooks':            ghead.rangeLooks=            getwrd(ss, 1)
    else: ; No operation
    endcase 
  endfor
endif 
;------------------------------------------------------------------------------
;                      SCENE INFO
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 &iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'sceneInfo')
w=where(index ge 0, count)
start=0
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'start':                 start=1
    'stop':                  start=2
    'timeUTC':               begin
                             if start eq 1 then ghead.starttimeUTC=  getwrd(ss, 1)  
                             if start eq 2 then ghead.stoptimeUTC=   getwrd(ss, 1)
                             end 
    'headingAngle':          ghead.headingAngle=            getwrd(ss, 1)              
    else: ; No operation
    endcase 
  endfor
endif

;------------------------------------------------------------------------------
;                      SCENE CENTER COORD
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 &iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'sceneInfo')
w=where(index ge 0, count)
for i=0L, n-1 do index[i]= strpos(s[i], 'sceneInfo')
w=where(index ge 0, count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'lat':            ghead.centerlat= getwrd(ss, 1 )
    'lon':            ghead.centerlon= getwrd(ss, 1 )
    'incidenceAngle': ghead.centerincidenceangle= getwrd(ss, 1 )           
    else: ; No operation
    endcase 
  endfor
endif


;------------------------------------------------------------------------------
;                       CALIBRATION CONSTANTS
;------------------------------------------------------------------------------

for layer=1, 10 do begin
  index= intarr(n) & istart=-1 &iend=-1
  layerstr=strcompress(string(layer), /remove_all)
  for i=0L, n-1 do index[i]= strpos(s[i], '<calibrationConstant layerIndex="'+layerstr )
  w1=where(index ge 0, count1)
  if count1 gt 0 then begin
    index= index * 0
    index= index + (-1)
    for i=w1[0], n-1 do index[i]= strpos(s[i], '</calibrationConstant>'           )
    w2=where(index ge 0, count2)
    if count2 gt 0 then begin
      istart=w1[0]
      iend=  w2[0]
      for i=istart, iend do begin
        ss= s[i] 
        ss= repchr( ss, '<', ' ' )
        ss= repchr( ss, '>', ' ' )
        ss= repchr( ss, '/', ' ' )
        key= getwrd(ss, 0)
        case key of
        'calibrationConstant':         lindex=getwrd(ss, 1, del='"')-1
        'calFactor':                   ghead.calFactor[lindex]= getwrd(ss, 1 )     
        else: ; No operation
        endcase 
      endfor  ; Loop lines looking for information
    endif     ; End check for count2
  endif       ; End check for count1
endfor        ; Loop through layers

;------------------------------------------------------------------------------
;                       CORNER CORRDINATES
;------------------------------------------------------------------------------

index= intarr(n) & istart=-1 & iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'sceneCornerCoord')
w=where(index ge 0,count)
if count ge 2 then begin
  ccmax= count/2
  for cc=0, ccmax-1 do begin
    cc2=  2*cc
    istart=w(cc2)
    iend=  w(cc2+1)
    for i=istart, iend do begin
      ss= s[i] 
      ss= repchr( ss, '<', ' ' )
      ss= repchr( ss, '>', ' ' )
      ss= repchr( ss, '/', ' ' )
      key= getwrd(ss,0)
      case key of
      'refRow':         ghead.cornerrefrow[cc]=         getwrd(ss, 1 )
      'refColumn':      ghead.cornerrefcolumn[cc]=      getwrd(ss, 1 )
      'lat':            ghead.cornerlat[cc]=            getwrd(ss, 1 )
      'lon':            ghead.cornerlon[cc]=            getwrd(ss, 1 )
      'incidenceAngle': ghead.cornerincidenceangle[cc]= getwrd(ss, 1 )
      else:             ; No operation
      endcase
    endfor
  endfor  ; End cc loop
endif

;------------------------------------------------------------------------------
;                       INSTRUMENT
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 &iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], '<instrument>')
w1=where(index ge 0, count)
for i=0L, n-1 do index[i]= strpos(s[i], '</instrument>')
w2=where(index ge 0, count)

if w1[0] gt 0 and w2[0] gt 0 then w=[w1[0], w2[0]]
count=n_elements(w)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'centerFrequency':            ghead.centerfrequency= getwrd(ss, 1 )     
    else: ; No operation
    endcase 
  endfor
endif

;------------------------------------------------------------------------------
;                      MAPPING GRID INFOR 
;------------------------------------------------------------------------------
index= intarr(n) & istart=-1 &iend=-1
for i=0L, n-1 do index[i]= strpos(s[i], 'mappignGridInfo')
w=where(index ge 0, count)
for i=0L, n-1 do index[i]= strpos(s[i], 'mappingGridInfo')
w=where(index ge 0, count)
if count ge 2 then begin
  istart=w[0]
  iend=  w[1]    
  for i=istart, iend do begin
    ss= s[i] 
    ss= repchr( ss, '<', ' ' )
    ss= repchr( ss, '>', ' ' )
    ss= repchr( ss, '/', ' ' )
    key= getwrd(ss,0)
    case key of
    'tReferenceTimeUTC': ghead.tReferenceTimeUTC=      getwrd(ss, 1 )
    'tauReferenceTime':  ghead.tauReferenceTime=       getwrd(ss, 1 )
    'refRow':            ghead.mapgridrefrow=          getwrd(ss, 1 )
    'refCol':            ghead.mapgridrefcol=          getwrd(ss, 1 )
    'numberOfRows':      ghead.mapgridnumberOfRows=    getwrd(ss, 1 )
    'numberOfColumns':   ghead.mapgridnumberOfColumns= getwrd(ss, 1 )
    'rowSpacing':        ghead.mapgridrowSpacing=      getwrd(ss, 1 )
    'columnSpacing':     ghead.mapgridcolumnSpacing=   getwrd(ss, 1 )
    else: ; No operation
    endcase 
  endfor
endif


end
