;;-----------------------------------------------------
;+
; NAME: TSAR_READ_IMAGE.PRO
; PURPOSE:
;  Read main TerraSAR-X TIFF image file and square the output
; CALLING SEQUENCE
;   tsar_read_image, filename, lf, image, /help, /quiet, /NOSQU
; INPUT
;  filename:  Image file in TIFF format
;  lf   :     Leaderfile structure
; OUTPUT      
;  image:     Averaged image in the digital values in the file
; KEYWORDS:
;  /help      Get help
;  /quiet     Do not print out intermediate output
;  /nosqu     Do not square output
;  error= error, error=1 if cannot read data
; MODIFICATION HISTORY:
;  2007 Oct 27 Frank Monaldo  
;  2007 Oct 30 Square the output
;  2009 Dec 07 Modify for ANSWRS 2.0. Remove df structure
;  2010 Jun 14 Frank Monaldo. Print help on no input.
; Copyright (C) 2007-2009, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_read_image, filename, lf, image, $
                     help=help, quiet=quiet, nosqu=nosqu, error=error

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: TSAR_READ_IMAGE.PRO
  print, ' PURPOSE:
  print, '  Read main TerraSAR-X TIFF image file and square the output
  print, ' CALLING SEQUENCE
  print, '   tsar_read_image, filename, lf, image, /help, /quiet, /NOSQU
  print, ' INPUT
  print, '  filename:  Image file in TIFF format
  print, '  lf:        Leaderfile structure
  print, ' OUTPUT      
  print, '  image:     Averaged image in the digital values in the file
  print, ' KEYWORDS:
  print, '  /help      Get help
  print, '  /quiet     Do not print out intermediate output
  print, '  /nosqu     Do not square output
  print, '  error= error, error=1 if cannot read data
  return    
endif    

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  

if printflag then begin
  print, 'Reading image data from TIFF file'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: tsar_read_image' 
  return
endif
;
;
nx=    long(lf.nx0)
ny=    long(lf.ny0)
nx2=   nx / long(lf.ave_pixel)
ny2=   ny / long(lf.ave_line)
if printflag then begin
  print, 'Original image size: ' + string(nx, format='(I7)') + string(ny,format='(I7)')
endif

nx3=       nx2 * lf.ave_pixel
sub_image= uintarr(nx3, lf.ave_line)
sub_image2=fltarr(nx3,  lf.ave_line)
image=     fltarr(nx2, ny2)
;
if printflag then print, 'Reading file TerraSAR-X: ' + filename
;
jstep= ny2/10
for j=0L, ny2-1 do begin
  jj=        j * lf.ave_line
  sub_image= read_tiff(filename, sub_rect=[0, jj, nx3, lf.ave_line] )
  if keyword_set(nosqu) then begin
    image[*,j]=     rebin(sub_image, nx2, 1)
    mu=  mean(sub_image)
    lab= 'Original data'
  endif else begin
    sub_image2=     float(sub_image) * float(sub_image)
    image[*,j]=     rebin(sub_image2, nx2, 1)
    mu=mean(sub_image2)
    lab= 'Original data squared.'
  endelse
  if printflag and (j mod jstep) eq 0 then begin
    percent_done= float(j) / ny2 * 100.0 
    print, 'Reading file. ' + string( percent_done, format='(F5.1)') + $
            '% done. ' + lab + ' Mean=' + string(mu)
  endif
endfor
if printflag then print, 'Finished readng file: ' + filename

end
