;;-----------------------------------------------------
;+
; NAME: TSAR_READ_MAPPING_GIRD
; PURPOSE:
;  Read mapping grid data and the relationship between
;  t and tau and image i and j
; CALLING SEQUENCE
;   tsar_read_mapping_grid, filename, ghead, t, tau, ii, jj, /help, /quiet
; INPUT
;  filename:  MAPPING_GRID file
;  ghead:     Header information from main file
; OUTPUT
;  t:         two-dimensional time array.
;  tau:       two-dimensional tau  array.
;  ii:        two-dimensional row index associated with the 
;             row index in the the image.
;  jj:        two-dimensional row index associated with the 
;             row index in the the image.
; KEYWORDS:
;  error= error, error=1 if cannot read data
; MODIFICATION HISTORY:
;   2007 Oct 16 Frank Monaldo  
;   2010 May 07 Frank Monaldo. Read t/tau whether written big or small endian
;   2010 Jun 14 Frank Monaldo. Print help on no input.
; Copyright (C) 2007, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_read_mapping_grid, filename, ghead, t, tau, ii, jj, $
                            help=help, quiet=quiet, error=error

printflag=1
if keyword_set(quiet) then printflag=0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: TSAR_READ_MAPPING_GIRD
  print, ' PURPOSE:
  print, '  Read mapping grid data and the relationship between
  print, '  t and tau and image i and j
  print, ' CALLING SEQUENCE
  print, '   tsar_read_mapping_grid, filename, ghead, t, tau, ii, jj, /help, /quiet
  print, ' INPUT
  print, '  filename:  MAPPING_GRID file
  print, '  ghead:     Header information from main file
  print, ' OUTPUT
  print, '  t:         two-dimensional time array.
  print, '  tau:       two-dimensional tau  array.
  print, '  ii:        two-dimensional row index associated with the 
  print, '             row index in the the image.
  print, '  ij:        two-dimensional row index associated with the 
  print, '              row index in the the image.
  print, ' KEYWORDS:
  print, '  error= error, error=1 if cannot read data
  return
endif

error= 1
catch, error_status  
;This statement begins the error handler:  

if Error_status ne 0 then begin  
   print, 'error index: ', error_status  
   print, 'error message: ', !error_state.msg  
   if error_status eq -249  or $
      error_status eq -252  or $
      error_status eq -261  or $
      error_status eq -262  then begin
      error= -1
      return
   endif
   catch, /cancel  
endif  

if printflag then begin
  print, 'Reading mapping grid information sar_read_mapping_grid...'
endif

res= file_search(filename, count=count)
if count  eq 0 then begin
  print, 'Cannot find file: ' + filename
  print, 'Exiting: tsar_read_mapping_grid' 
  return
endif

nx=   long(ghead.MAPGRIDNUMBEROFCOLUMNS)
ny=   long(ghead.MAPGRIDNUMBEROFROWS)
t=    fltarr( nx, ny)
tau=  fltarr( nx, ny)
if printflag then print, 'Start reading file: ' + filename

x= float(0)
y= float(0)
close, 1
openr, 1, filename
for j=0L, ny-1 do begin
  for i=0L, nx-1 do begin
    readu, 1, x,y
    t[i,j]=   x
    tau[i,j]= y
  endfor
endfor
close, 1


;
; Fudge
;
;t=   reform(transpose(t), nx, ny)
;tau= reform(transpose(tau), nx, ny)
if printflag then print, 'End reading file: ' + filename
  if printflag then begin
    print, 'Before flipping endian.'
    print, '   Max tau=', max(tau)
    print, '   Max t=  ', max(t)
  endif
if endian() ne 1  and max(abs(tau)) gt 1e30  then begin
  if printflag then print, 'Flipping endian..'
  t=     swap_endian(t)
  tau=   swap_endian(tau)
endif else begin
  if printflag then begin
    print, 'Not flipping endian.'
    print, '   Max tau=', max(tau)
    print, '   Max t=  ', max(t)
  endif
endelse

tau= tau + float(ghead.tauReferenceTime)
if printflag then print, 'Assigning i, j values...'
ii= fltarr( nx, ny)
jj= fltarr( nx, ny)
dx= float( ghead.MAPGRIDCOLUMNSPACING)  / float(ghead.columnspacing) 
dy= float( ghead.MAPGRIDROWSPACING)     / float(ghead.rowspacing    ) 

for i=0L, nx-1 do begin
  ii2= i*dx
  for j=0L, ny-1 do begin
    ii[i,j]= ii2
    jj[i,j]= j * dy
  endfor
endfor



end
