;----------------------------------------------------------
;+
; NAME: TSAR_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out TerraSAR-X data into ancillary data ascii file. These
;   data fit the SIO format
; CALLING SEQUENCE:'
;    tsar_write_ancil_sio, filename, lf,  scaling'
; INPUT:'
;   filename=     output file with ancillary data
;   lf=           leader structure'
;   scaling=      scaling for data in binary file of NRCS, default=1
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   
; MODIFICATION HISTORY:
;   2009 Dec 07 Frank Monaldo Derived from rsat2_write_ancil_sio for ANSWRS 2.0  
;   2009 Dec 23 Frank Monaldo. Corrected computation of heading  
;   2010 Jan 06 Frank Monaldo. Corrected for using ans_compute_fit
;   2010 Jan 08 Frank Monaldo. Correct misspelling
;   2010 Jan 13 Frank Monaldo. Modify to use ans_write_ancil_sio
;   2010 Jan 15 Frank Monaldo. Use heading from file
;   2010 Jan 22 Frank Monaldo. Re-compute illumination and direction and relative heading
;                              in accordance with Wackermans description
;   2010 Jan 23 Frank Monaldo. Corrected !dpi misspelling
;   2010 Jun 14 Frank Monaldo. Print help on no input.
;   2011 Jan 01 Frank Monaldo. Pass through quiet flag
; Copyright (C) 2007-2011, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro tsar_write_ancil_sio, filename, lf, scaling, quiet=quiet, help=help


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: TSAR_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out TerraSAR-X data into ancillary data ascii file. These
  print, '   data fit the SIO format
  print, ' CALLING SEQUENCE:'
  print, '    tsar_write_ancil_sio, filename, lf,  scaling'
  print, ' INPUT:'
  print, '   filename=     output file with ancillary data
  print, '   lf=           leader structure'
  print, '   scaling=      scaling for data in binary file of NRCS, default=1
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  return
endif

lf.datatake_id=          lf.product     
scansar=               0
if lf.range_pixel gt 30 or lf.azimuth_pixel gt 25 then scansar=1
asc_des=               strupcase(strmid(lf.orbitdirection,0,1))
lf.radar_frequency=    lf.radar_frequency *  1e9
sensor_velocity=       sqrt(lf.x_velocity^2 + lf.y_velocity^2 + lf.z_velocity^2)
sensor_velocity=       string(sensor_velocity, format='(F15.6)' ) 
if lf.heading gt 180 then lf.heading= lf.heading-360.
;
; Determine if the data are in slant range or ground range.
;
if lf.projection eq 'GROUNDRANGE' then begin
  range_type=  'GROUND RANGE'
  range_flag=  1
endif else begin
  range_type= lf.projection
  range_flag=  0  
endelse

element_spacing=  lf.pixel_spacing * lf.ave_pixel
lf.line_spacing=  lf.line_spacing  * lf.ave_line

heading_rel=         90
lf.sar_sensor=      'TerraSAR-X'
illumination_dir=    0
range_direction= 0 ; Range increases with pixel.
first_inc=        ans_poly_cal(0,      0, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
last_inc=         ans_poly_cal(lf.nx1, 0, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
if last_inc lt first_inc then range_direction=  1
if range_direction then illumination_dir=0
lo1=                ans_poly_cal(0,         0, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
lo2=                ans_poly_cal(0,    lf.ny1, lf.lon_coef, lf.lon_xexp, lf.lon_yexp)
la1=                ans_poly_cal(0,         0, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
la2=                ans_poly_cal(0,    lf.ny1, lf.lat_coef, lf.lat_xexp, lf.lat_yexp)
ll2rb, lo1,la1, lo2,la2, r, b
ang1=   !dpi * lf.heading
ang2=   !dpi * b
dang=   acos( cos(ang1) * cos(ang2) + sin(ang1) * sin(ang2) )
if dang > !dpI/2.0 then heading_rel=180

near_range_edge=    'L'
early_azimuth_edge= 'B'
center_incid=        ans_poly_cal(lf.nx1/2,lf.ny1/2, lf.incid_coef, lf.incid_xexp, lf.incid_yexp)
range_swath=         lf.range_pixel   * lf.ave_pixel * lf.nx1
azimuth_swath=       lf.azimuth_pixel * lf.ave_line  * lf.ny1
software_version=    lf.software_ver
datatake_id=         lf.product          
orbit_number=        lf.absorbit
platform_code=       'T1'
valuetype=           'intensity'
pol=                 lf.polarization
frequency_band=      'X'
integration_time=    '-99999.99'
nom_az_res=          '-99999.99'
nom_rg_res=          '-99999.99'
rcvr_gain=           '-99999.99'
proc_gain=           '-99999.99'
slant_range_near=     -9999.999          
slant_range_center=   -9999.999          
deg_incid=            lf.deg             
deg_rlook=            lf.deg             
flip=                 0       
rsat=                 sqrt(lf.x_position^2 + lf.y_position^2 + lf.z_position^2)
if rsat gt 10 then begin
  sin_lat=              lf.z_position / rsat
  sin_lat2=             sin_lat * sin_lat
  cos_lat=              sqrt(lf.x_position^2 + lf.y_position^2) / rsat
  cos_lat2=             cos_lat * cos_lat
  a=                    equatorradius
  b=                    polarradius
  rearth=               a* sqrt(b^4/a^4 * sin_lat2 + cos_lat2)
  rearth=               rearth / sqrt( 1 - (a - b^2/a^2) * sin_lat2 )
  rearth=               rearth * 1000.00  ; Convert from kilometers to meters
  lf.s_c_altitude =     rsat - rearth
endif else begin
  lf.s_c_altitude =     533000.00  ; Nominal value
endelse
if float( sensor_velocity ) lt 10 then begin 
  sensor_velocity= string(7608, format='(F15.6)' ) ; nominal value
endif
ellipsoidscale=       '1'

if not tag_test(lf, 'platform_code') then begin
  temp_lf= create_struct( lf, 'software_version',      software_version,    $
                              'orbit_number',          orbit_number,        $
                              'platform_code',         platform_code,       $
                              'scansar',               scansar,             $
                              'valuetype',             valuetype,           $
                              'pol',                   pol,                 $
                              'frequency_band',        frequency_band,      $
                              'asc_des',               asc_des,             $
                              'sensor_velocity',       sensor_velocity,     $
                              'integration_time',      integration_time,    $
                              'nom_az_res',            nom_az_res,          $      
                              'nom_rg_res',            nom_rg_res,          $      
                              'rcvr_gain',             rcvr_gain,           $       
                              'proc_gain',             proc_gain,           $
                              'range_type',            range_type,          $
                              'range_flag',            range_flag,          $
                              'scaling',               scaling,             $
                              'element_spacing',       element_spacing,     $
                              'heading_rel',           heading_rel,         $
                              'illumination_dir',      illumination_dir,    $ 
                              'near_range_edge',       near_range_edge,     $   
                              'early_azimuth_edge',    early_azimuth_edge,  $
                              'center_incid',          center_incid,        $      
                              'range_swath',           range_swath,         $      
                              'azimuth_swath',         azimuth_swath,       $
                              'slant_range_near',      slant_range_near,    $
                              'slant_range_center',    slant_range_center,  $
                              'deg_incid',             deg_incid,           $
                              'deg_rlook',             deg_rlook ,          $
                              'flip',                  flip,                $
                              'ellipsoidscale',        ellipsoidscale)   
  lf=      0                                    
  lf=      temp_lf                           
endif else begin
  lf.software_version=     lf.software_ver    
  lf.orbit_number=         lf.absorbit        
  lf.platform_code=        'T1'               
  lf.scansar=              scansar            
  lf.valuetype=            'intensity'        
  lf.pol=                  lf.polarization    
  lf.frequency_band=       'X'                
  lf.asc_des=              asc_des            
  lf.sensor_velocity=      sensor_velocity   
  lf.scaling=              scaling
  lf.integration_time=     '-99999.99'        
  lf.nom_az_res=           '-99999.99'          
  lf.nom_rg_res=           '-99999.99'          
  lf.rcvr_gain=            '-99999.99'           
  lf.proc_gain=            '-99999.99'        
  lf.range_type=           range_type         
  lf.range_flag=           range_flag        
  lf.element_spacing=      element_spacing    
  lf.heading_rel=          heading_rel       
  lf.illumination_dir=     illumination_dir  
  lf.near_range_edge=      near_range_edge   
  lf.early_azimuth_edge=   early_azimuth_edge
  lf.center_incid=         center_incid         
  lf.range_swath=          range_swath          
  lf.azimuth_swath=        azimuth_swath      
  lf.slant_range_near=     -9999.999        
  lf.slant_range_center=    -9999.999         
  lf.deg_incid=            lf.deg            
  lf.deg_rlook=            lf.deg  
  lf.flip=                 flip
  ellipsoidscale=          ellipsoidscale
endelse
;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, lf, quiet=quiet
 
end
