;-----------------------------------------------------
;+
; NAME: UAVSAR_FILLIN_INFO.PRO
; PURPOSE:
;    Read UAVSAR ancillary file and defaults file to fill in the lf structure with ancillary
;    information.   Note that the lon, lat, rlook, and incident coefs
;    are computed in this progam.  The lngswitch is set here as well.
; CALLING SEQUENCE
;    uavsar_fillin_info, ancil, deffile, info 
; INPUT
;   ancil      : UAVSAR ancillary data file
;   deffile    : Defaults file
; OUTPUT
;   info:      : info structure structure
; OPTIONS
;   /help      : Get help
;   /quiet     : Suppress intermediat results
;   /display   : Display fits for diagnostics purposes.
;   error      : < 0 bad
; MODIFICATION HISTORY:
;  2019 Dec 30 Frank Monaldo.  Note polarization filled with BLANK
;  2020 Jan 29 Frank Monaldo.  Use ancillary data file name to determine root file name
; Copyright (C) 2019, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro  uavsar_fillin_info, ancil, deffile, info  , $ 
                    help    =     help         , $   
                    quiet   =     quiet        , $
                    display =     display      , $
                    error   =     error      

printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0

if keyword_set(help)  or (n_params(0) lt 1 ) then begin 
  print, ' NAME: UAVSAR_FILLIN_INFO.PRO
  print, ' PURPOSE:
  print, '    Read UAVSAR ancillary file and defaults file to fill in the lf structure with ancillary
  print, '    information.   Note that the lon, lat, rlook, and incident coefs
  print, '    are computed in this progam.  The lngswitch is set here as well.
  print, ' CALLING SEQUENCE
  print, '    uavsar_fillin_info, ancil, deffile, info 
  print, ' INPUT
  print, '   ancil      : UAVSAR ancillary data file
  print, '   deffile    : Defaults file
  print, ' OUTPUT
  print, '   info:      : info structure structure
  print, ' OPTIONS
  print, '   /help      : Get help
  print, '   /quiet     : Suppress intermediat results
  print, '   /display   : Display fits for diagnostics purposes.
  print, '   error      : < 0 bad
  error = -1
  return
endif

speed_of_light   = 299792458.00      ; m/s 
rearth           = 6371.23 * 1000.00 ; m

;
;  Read ancillary data file file
; 
ans_read_uavsar_ancil, ancil, x, error=error
if error lt 0 then begin
  if printflag then begin
    print, 'Cannot read file : ' + ancil
    print, 'Exiting uavsar_fillin_info...'
  endif
  error = -2
  return
endif
;
;  Read defaults file
;
ans_read_defaults, deffile, df, quiet=quiet, error = error
if error lt 0 then begin
  if printflag then begin
    print, 'Cannot read file : ' + deffile
    print, 'Exiting uavsar_fillin_info...'
  endif
  error = -3
  return
endif
;
; Get the software version number
;
ans_gen_version, answrs_version, idlans_dir=df.idlans_dir, quiet=quiet
;
; Declaring info structure
;
;
; Read longitude and latitude arrays
;

absorbit                      = long(0)
current_js                    = dt_tm_tojs( systime(/utc) )
jsstart                       = current_js
processing_start              = dt_tm_fromjs( jsstart, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
sio_create_date_time          = dt_tm_fromjs( current_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
orbit_pass_flag               = 'NA'
sensor_wavelength_m           = double( x.Center_Wavelength_cm ) * 100.0
sensor_frequency_hz           = speed_of_light / sensor_wavelength_m  
product_type                  = 'GRD'
map_descriptor                = 'GROUND RANGE'
range_type                    = 1
sar_processor_version         = x.Post_Processor_Version_Number
processing_agency             = 'JPL'                             
processing_facility           = 'JPL'  
orbit_number                  = long(0)                  
platform_name                 = 'UAVSAR'                 
sar_platform_code             = 'UAVSAR'                 
sensor_imaging_mode           = x.Acquisition_Mode
calibration_flag              = 1                       
image_pixel_value_units       = 'Intensity'             
polarization                  = 'BLANK'
frequency_band                = 'L'
flag_sar_band_polarization    = '00'
orbit_pass_flag               = 'BLANK'
scaling                       = 1.00000
sar_beam_code                 = 'BLANK'                      
nominal_azimuth_resolution_m  = float(tag_value(x,'Azimuth_Resolution_m'  ))      
nominal_range_resolution_m    = -9999.9999                      
receiver_gain_db              = -9999.9999                      
relative_processor_gain_db    = -9999.9999                      
nrcs_slope                    = 1.00                              
nrcs_bias                     = 0.0     
sensor_vel_rel_ground_mps     = -9999.9999                      
integration_time_s            = -9999.9999                                              
ny0                           = long(tag_value(x,'grd_phase_set_rows_pixels'))              
nx0                           = long(tag_value(x,'grd_phase_set_cols_pixels'))
nx1                           = nx0 / long(df.ave_pixel)
ny1                           = ny0 / long(df.ave_line)
looks_range                   = float(tag_value(x,'Number_of_Range_Looks_in_GRD_'  ))
looks_azimuth                 = float(tag_value(x,'Number_of_Azimuth_Looks_in_GRD_'))
ref_ellipsoid                 = x.DEM_Datum
ref_semi_major_axis_m         = double(tag_value(x,'Ellipsoid_Semimajor_Axis_m'      )) 
esq                           = double(tag_value(x,'Ellipsoid_Eccentricity_Squared_' ))
ref_semi_minor_axis_m         = sqrt( ref_semi_major_axis_m^2 - esq ) 
datumshiftparameters          =  '0 0 0'    
case strupcase(x.Look_Direction) of
'LEFT'  : illumination_dir_deg = -90.0              
'RIGHT' : illumination_dir_deg =  90.0  
ELSE    : illumination_dir_deg =  90.0  
endcase
heading_north_deg             =  float(tag_value(x,'Peg_Heading_deg'))                   
if heading_north_deg lt 0 then heading_north_deg = heading_north_deg + 360.00
rlook                         = heading_north_deg + illumination_dir_deg
if rlook lt 0             then rlook             = rlook             + 360.00
flip_flag                     = 0
lngswitchflag                 = 0
slant_range_center_m          = -9999.9999                       
slant_range_near_m            = -9999.9999       
sensor_altitude_m             = float(tag_value(x,'Average_Altitude_m') )  
incid_angle_center_deg        = -9999.9999           
early_azimuth_edge            = -9999.9999                    
near_range_edge               = -9999.9999                    
range_swath_width_m           = -9999.9999 
azimuth_swath_width_m         = -9999.9999 
range_swath_width_m           = -9999.9999
azimuth_swath_width_m         = -9999.9999


filebreak, ancil, name=name 
root_file = name
;----------------------- 
; Start time parsing
;----------------------=
acquisition_start_date    = getwrd( x.Start_Time_of_Acquisition  , 0 )
acquisition_start_year    = strmid( acquisition_start_date  ,    7, 4 )
acquisition_start_month   = getwrd( acquisition_start_date  , 1 , del='-')
acquisition_start_day     = getwrd( acquisition_start_date  , 0 , del='-')
acquisition_start_time    = strmid( x.Start_Time_of_Acquisition  , 11, 8 )
acquisition_start_time_str= acquisition_start_year    + ' ' + $
                            acquisition_start_month   + ' ' + $ 
                            acquisition_start_day     + ' ' + $
                            acquisition_start_time  
acquisition_start_time_js = dt_tm_tojs(  acquisition_start_time_str )

acquisition_stop_date     = getwrd( x.Stop_Time_of_Acquisition  , 0 )
acquisition_stop_year     = strmid( acquisition_stop_date   ,    7, 4 )
acquisition_stop_month    = getwrd( acquisition_stop_date   , 1 , del='-')
acquisition_stop_day      = getwrd( acquisition_stop_date   , 0 , del='-')
acquisition_stop_time    = strmid( x.Stop_Time_of_Acquisition  , 11, 8 )
acquisition_stop_time_str = acquisition_stop_year    + ' ' + $
                            acquisition_stop_month   + ' ' + $ 
                            acquisition_stop_day     + ' ' + $
                            acquisition_stop_time  
acquisition_stop_time_js  = dt_tm_tojs(  acquisition_stop_time_str )                           
                            
center_js                 = ( acquisition_start_time_js + acquisition_stop_time_js ) / 2.0


acquisition_date          = dt_tm_fromjs( center_js, format='0n$-0d$-Y$' )
acquisition_time          = dt_tm_fromjs( center_js, format='h$:m$:s$' )
acq_date_time_js          = center_js
acq_date_time             = dt_tm_fromjs( center_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
absorbit                  = long(0)
current_js                = dt_tm_tojs( systime(/utc) )
sio_create_date_time      = dt_tm_fromjs( current_js, format='Y$-0n$-0d$ h$:m$:s$ UTC' )
processing_start          = x.Date_of_Processing
;----------------------- 
; End time parsing
;----------------------=
;
; Computing longitude and latitude of each pixel
;
; lambda[i]  = grd_pwr_col_addr_deg + i * grd_pwr_col_mult_deg_per_pixel
; phi[j]     = grd_pwr_row_addr_deg + j * grd_pwr_row_mult_deg_per_pixel
;
longitude_slope           = double( x.grd_pwr_col_mult_deg_per_pixel )
longitude_bias            = double( x.grd_pwr_col_addr_deg           )
latitude_slope            = double( x.grd_pwr_row_mult_deg_per_pixel )
latitude_bias             = double( x.grd_pwr_row_addr_deg           )
longitude_ll              = longitude_bias
latitude_ll               = latitude_bias
longitude_ur              = longitude_bias + double( nx0-1 ) * longitude_slope
latitude_ur               = latitude_bias  + double( ny0-1 ) * latitude_slope

ll2rb, longitude_ll, latitude_ll, longitude_ur,  latitude_ll , r , b
pixel_spacing_m           = double(r * rearth / nx0 )
ll2rb, longitude_ll, latitude_ll, longitude_ll,  latitude_ur , r , b
pixel_spacing_m           = float(r * rearth / nx0 )
line_spacing_m            = float(r * rearth / ny0 )



info= {root_file                     :  root_file                       , $
       answrs_version                :  answrs_version                  , $
       acquisition_date              :  acquisition_date                , $ 
       acquisition_time              :  acquisition_time                , $
       acq_date_time_js              :  acq_date_time_js                , $
       acq_date_time                 :  acq_date_time                   , $
       jsbegin                       :  acquisition_start_time_js       , $ 
       jsend                         :  acquisition_stop_time_js        , $
       processing_start              :  processing_start                , $
       sio_create_date_time          :  sio_create_date_time            , $
       product_type                  :  product_type                    , $
       processing_agency             :  processing_agency               , $
       processing_facility           :  processing_facility             , $
       sar_processor_version         :  sar_processor_version           , $
       scene_identifier              :  root_file                       , $
       scene_reference_number        :  root_file                       , $
       orbit_number                  :  orbit_number                    , $
       platform_name                 :  platform_name                   , $
       sar_platform_code             :  sar_platform_code               , $
       sensor_imaging_mode           :  sensor_imaging_mode             , $
       calibration_flag              :  calibration_flag                , $
       image_pixel_value_units       :  image_pixel_value_units         , $
       polarization                  :  polarization                    , $
       frequency_band                :  frequency_band                  , $
       flag_sar_band_polarization    :  flag_sar_band_polarization      , $
       orbit_pass_flag               :  orbit_pass_flag                 , $
       scaling                       :  scaling                         , $
       sar_beam_code                 :  sar_beam_code                   , $
       sensor_wavelength_m           :  sensor_wavelength_m             , $
       sensor_frequency_hz           :  sensor_frequency_hz             , $
       sensor_vel_rel_ground_mps     :  sensor_vel_rel_ground_mps       , $
       integration_time_s            :  integration_time_s              , $
       nominal_azimuth_resolution_m  :  nominal_azimuth_resolution_m    , $
       nominal_range_resolution_m    :  nominal_range_resolution_m      , $
       receiver_gain_db              :  receiver_gain_db                , $
       relative_processor_gain_db    :  relative_processor_gain_db      , $
       map_descriptor                :  map_descriptor                  , $
       pixel_spacing_m               :  pixel_spacing_m                 , $
       line_spacing_m                :  line_spacing_m                  , $
       range_type                    :  range_type                      , $
       heading_north_deg             :  heading_north_deg               , $
       heading_image_deg             :  0.0                             , $
       flip_flag                     :  flip_flag                       , $
       illumination_dir_deg          :  illumination_dir_deg            , $
       early_azimuth_edge            :  early_azimuth_edge              , $
       near_range_edge               :  near_range_edge                 , $
       range_swath_width_m           :  range_swath_width_m             , $
       azimuth_swath_width_m         :  azimuth_swath_width_m           , $
       sensor_altitude_m             :  sensor_altitude_m               , $
       incid_angle_center_deg        :  incid_angle_center_deg          , $
       slant_range_center_m          :  slant_range_center_m            , $
       slant_range_near_m            :  slant_range_near_m              , $
       lines_original_image          :  nx0                             , $
       pixels_original_image         :  ny0                             , $
       lines_sio_image               :  nx1                             , $
       pixels_sio_image              :  ny1                             , $
       nx0                           :  nx0                             , $
       ny0                           :  ny0                             , $
       nx1                           :  nx1                             , $
       ny1                           :  ny1                             , $
       ave_pixel                     :  df.ave_pixel                    , $
       ave_line                      :  df.ave_line                     , $
       looks_range                   :  looks_range                     , $
       looks_azimuth                 :  looks_azimuth                   , $
       ref_ellipsoid                 :  ref_ellipsoid                   , $
       ref_semi_major_axis_m         :  ref_semi_major_axis_m           , $
       ref_semi_minor_axis_m         :  ref_semi_minor_axis_m           , $
       datumshiftparameters          :  datumshiftparameters            , $
       datum_scale                   :  1                               , $
       nrcs_slope                    :  nrcs_slope                      , $
       nrcs_bias                     :  nrcs_bias                       , $
       lngswitchflag                 :  lngswitchflag                   , $
       icefile                       :  ''                              , $
       icefile_time_js               :  ''                              , $
       icefile_time                  :  ''                              , $
       deg                           :  df.deg                          , $
       deg_incid                     :  df.deg                          , $
       deg_rlook                     :  df.deg                            }
    

;
; Compute longitude and latitude arrays
;
; lambda[i]  = grd_pwr_col_addr_deg  + i * grd_pwr_col_mult_deg_per_pixel
;  phi[j]     = grd_pwr_row_addr_deg + j * grd_pwr_row_mult_deg_per_pixel
;
longitude  = fltarr(nx0,ny0) 
lng        = findgen(nx0) * x.grd_pwr_col_mult_deg_per_pixel + x.grd_pwr_col_addr_deg
for j = 0, ny0 -1 do  longitude[*,j] =lng  
latitude   = fltarr(nx0,ny0)
lat        = findgen(ny0) * x.grd_pwr_row_mult_deg_per_pixel + x.grd_pwr_row_addr_deg
for i = 0, nx0-1 do  latitude[i,*] =lat  
;
;  Compute fit 
;
;  Make sure that we start in the +/- 180 degrees basis
;
w = where( longitude gt 180, count)
if count gt 0 then longitude[w] = longitude[w] - 360.00

lonmax  =   max( longitude )
lonmin  =   min( longitude );

if lonmax gt 120 and lonmin lt -120 then info.lngswitchflag = 1;
if info.lngswitchflag then begin
  ; switch to 0-360 basis   
  w = where( longitude lt 0, count)
  if count gt 0 then longitude[w] = longitude[w] + 360.00
endif
pixels        = dblarr(nx0, ny0)
pixels_1d     = dindgen(nx0)
for j = 0, ny0 -1 do  pixels[*,j] =pixels_1d 
lines         = dblarr(nx0, ny0)
lines_1d      = dindgen(ny0)
for i = 0, nx0-1 do  lines[i,*] =lines_1d 


pixels    = reform( pixels   , n_elements(pixels)    )
lines     = reform( lines    , n_elements(lines)     )
longitude = reform( longitude, n_elements(longitude) )
latitude  = reform( latitude , n_elements(latitude ) )
; 
; sub sample by a factor of 100
;
factor =   n_elements(pixels) / 1000L
ii        = lindgen( n_elements(pixels) / factor)  * factor
pixels    = pixels[ii]
lines     = lines[ii]
longitude = longitude[ii]
latitude  = latitude[ii]
;
;
if printflag then hlp, pixels, lines, longitude, latitude
;

ans_compute_fit, pixels    , lines    ,  longitude , df.deg, lon_coef ,  lon_xexp , lon_yexp , quiet = quiet
ans_compute_fit, pixels    , lines    ,  latitude  , df.deg, lat_coef ,  lat_xexp , lat_yexp , quiet = quiet
ans_compute_fit, longitude , latitude ,  pixels    , df.deg, i_coef   ,  i_xexp ,   i_yexp   , quiet = quiet
ans_compute_fit, longitude , latitude ,  lines     , df.deg, j_coef   ,  j_xexp ,   j_yexp   , quiet = quiet


;
inc = 1 & ans_read_uavsar_ancil_img, ancil, xx, inc=inc, error=error

inc = inc * 180.00 /!pi  ; Convert to degrees

;
w = where( inc gt 10 and inc lt 70 , count )

if count le 10 then begin
  if printflag then begin
    print, 'Cannot compute incident angle fit'
    print, 'Exiting uavsar_fill_in.pro'
  endif
  error = -4
  return
endif

n      = n_elements(w)
xpixel = dblarr(n)
yline  = dblarr(n)
z      = dblarr(n)
ii     = -1L
if printflag then begin
  print, 'Number of incident angle points > 10 < 70 degrees ' + string(n)
  print, 'Arranging fit arrays for non-zero values'
end
for i = 0, nx0-1 do begin
  for j = 0, ny0-1 do begin
    if inc[i,j] gt 0 then begin
      ii         =  ii + 1L
      if ii lt count then begin
        xpixel[ii] =  i
        yline[ii]  =  j
        z[ii]      =  inc[i,j] 
      endif
    endif
  endfor
endfor

if printflag then hlp, xpixel , yline , z

 
factor    = n_elements(xpixel) / 1000L
ii        = lindgen( n_elements(xpixel) / factor)  * factor
xpixel    = xpixel[ii]
yline     = yline[ii]
z         = z[ii]
if printflag then hlp, xpixel , yline , z

ans_compute_fit, xpixel , yline , z , df.deg, incid_coef , incid_xexp , incid_yexp , quiet = quiet


; 
; Compute radar look direction with respect 
;


rlook_coef    = incid_coef * 0.0
rlook_coef[0] = rlook
rlook_xexp    = incid_xexp
rlook_yexp    = incid_yexp

; 
; Display fits
;
if keyword_set(display) then begin
  nx     = nx1 / 10
  ny     = ny1 / 10
  if printflag then print, 'Computing fits using array size of ' + string(nx) + ' x ' + string(ny)
  lng    = fltarr(nx,ny)
  lat    = lng
  incid  = lng
  rlk    = lng
  for i = 0, nx - 1L do begin
    ii  = double(i) * 10.00
    for j = 0, ny -1L do begin
       jj = double(j) * 10.00
       lng[i,j]    = ans_poly_cal( ii, jj, lon_coef  , lon_xexp  ,   lon_yexp )
       lat[i,j]    = ans_poly_cal( ii, jj, lat_coef  , lat_xexp  ,   lat_yexp )
       incid[i,j]  = ans_poly_cal( ii, jj, incid_coef, incid_xexp, incid_yexp ) 
       rlk[i,j]    = ans_poly_cal( ii, jj, rlook_coef, rlook_xexp, rlook_yexp ) 
    endfor
  endfor
  im = image( lng,  title= 'Longitude'  )
  stop
  im = image( lat,  title= 'Latitude'   )
  stop
  im = image(incid, title= 'Incidence' )
  stop
  im = image(rlk,   title= 'Radar Look' )
  stop
endif

info = create_struct( info,  'lon_coef'       ,  lon_coef    , $
                             'lon_xexp'       ,  lon_xexp    , $
                             'lon_yexp'       ,  lon_yexp    , $
                             'lat_coef'       ,  lat_coef    , $
                             'lat_xexp'       ,  lat_xexp    , $
                             'lat_yexp'       ,  lat_yexp    , $
                             'i_coef'         ,  i_coef      , $
                             'i_xexp'         ,  i_xexp      , $
                             'i_yexp'         ,  i_yexp      , $
                             'j_coef'         ,  j_coef      , $
                             'j_xexp'         ,  j_xexp      , $
                             'j_yexp'         ,  j_yexp      , $
                             'incid_coef'     ,  incid_coef  , $
                             'incid_xexp'     ,  incid_xexp  , $
                             'incid_yexp'     ,  incid_yexp  , $
                             'rlook_coef'     ,  rlook_coef  , $
                             'rlook_xexp'     ,  rlook_xexp  , $
                             'rlook_yexp'     ,  rlook_yexp  )
                         
                         
; compute center geographic position
;
ic         = nx0 / 2
jc         = ny0 / 2 
center_lon = ans_poly_cal( double(ic), double(jc), info.lon_coef, info.lon_xexp, info.lon_yexp)
center_lat = ans_poly_cal( double(ic), double(jc), info.lat_coef, info.lat_xexp, info.lat_yexp)

info  = create_struct(info, 'center_lat', center_lat, 'center_lon', center_lon ) 

end








