;----------------------------------------------------------
;+
; NAME: UAVSAR_WRITE_ANCIL_SIO'
; PURPOSE:'
;   To write out Sentinel-1 data into ancillary data ascii file. These
;   data fit the SIO format.
; CALLING SEQUENCE:'
;    uavsar_write_ancil_sio, filename, info
; INPUT:'
;   filename    = output file with ancillary data
;   info        = ancillary information structure'
; OUTPUT:
;   ASCII file in SIO format
; KEYWORDS:
;   /quiet=      Do not print intermediate results'
;   /help=       List help.
;   error     =  error ( <0 is bad )
; MODIFICATION HISTORY:
;   2020 Jan 10 Frank Monaldo.
;
; Copyright (C) 2007-2020, Johns Hopkins University/Applied 
; Physics Laboratory. This software may be used, copied, 
; or redistributed as long as it is not sold and this 
; copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied 
; warranties whatsoever.  
;-

pro uavsar_write_ancil_sio, filename, info,  quiet=quiet, help=help, error=error


printflag=1
if keyword_set(quiet) then printflag=0
quiet= printflag eq 0
error  = 0
if keyword_set(help)  or (n_params(0) lt 1 ) then begin
  print, ' NAME: UAVSAR_WRITE_ANCIL_SIO'
  print, ' PURPOSE:'
  print, '   To write out Sentinel-1 data into ancillary data ascii file. These
  print, '   data fit the SIO format.
  print, ' CALLING SEQUENCE:'
  print, '    uavsar_write_ancil_sio, filename, info
  print, ' INPUT:'
  print, '   filename    = output file with ancillary data
  print, '   info        = ancillary information structure'
  print, ' OUTPUT:
  print, '   ASCII file in SIO format
  print, ' KEYWORDS:
  print, '   /quiet=      Do not print intermediate results'
  print, '   /help=       List help.
  print, '   error     =  error ( <0 is bad )
  error  = -1
  return
endif

sar_sensor              = 'UAVSAR'
range_type              = 'GROUND RANGE'
range_flag              = 1
if info.product_type ne 'GRD' then begin
  range_type = 'SLANT RANGE'
  range_flag  = 1
endif
info = create_struct( info, 'name_dir'            ,  info.root_file                                             , $
                            'month'               ,  dt_tm_fromjs( info.acq_date_time_js, format='0n$'        ) ,  $
                            'day'                 ,  long( dt_tm_fromjs( info.acq_date_time_js, format='0d$') ) ,  $
                            'data_year'           ,  long( dt_tm_fromjs( info.acq_date_time_js, format='Y$' ) ) ,  $
                            'hour'                ,  long( dt_tm_fromjs( info.acq_date_time_js, format='h$' ) ) ,  $
                            'minute'              ,  long( dt_tm_fromjs( info.acq_date_time_js, format='m$' ) ) ,  $
                            'second'              ,  long( dt_tm_fromjs( info.acq_date_time_js, format='s$' ) ) ,  $
                            'radar_frequency'     ,  info.sensor_frequency_hz                                   ,  $
                            'pol'                 ,  info.polarization                                          ,  $
                            'range_pixel'         ,  info.pixel_spacing_m                                       ,  $
                            'azimuth_pixel'       ,  info.line_spacing_m                                        ,  $
                            'productType'         ,  info.product_type                                          ,  $
                            'proc_fac'            ,  info.processing_facility                                   ,  $
                            'software_version'    ,  info.sar_processor_version                                 ,  $
                            'datatake_id'         ,  info.scene_reference_number                                ,  $
                            'sar_sensor'          ,  sar_sensor                                                 ,  $
                            'platform_code'       ,  info.platform_name                                         ,  $
                            'sar_imaging_mode'    ,  info.sensor_imaging_mode                                   ,  $
                            'valuetype'           ,  info.image_pixel_value_units                               ,  $
                            'asc_des'             ,  info.orbit_pass_flag                                       ,  $
                            'beam_mode'           ,  info.sensor_imaging_mode                                   ,  $
                            'sensor_velocity'     ,  info.sensor_vel_rel_ground_mps                             ,  $
                            'integration_time'    ,  info.integration_time_s                                    ,  $
                            'nom_az_res'          ,  info.nominal_azimuth_resolution_m                          ,  $
                            'nom_rg_res'          ,  info.nominal_range_resolution_m                            ,  $
                            'rcvr_gain'           ,  string(info.receiver_gain_db)                              ,  $
                            'proc_gain'           ,  string(info.relative_processor_gain_db)                    ,  $
                            'range_flag'          ,  range_flag                                                 ,  $
                            'heading'             ,  info.heading_north_deg                                     ,  $   
                            'heading_rel'         ,  info.HEADING_NORTH_DEG                                     ,  $  
                            'flip'                ,  info.flip_flag                                             ,  $  
                            'illumination_dir'    ,  info.illumination_dir_deg                                  ,  $  
                            'range_swath'         ,  info.range_swath_width_m                                   ,  $
                            'azimuth_swath'       ,  info.azimuth_swath_width_m                                 ,  $
                            's_c_altitude'        ,  info.sensor_altitude_m                                     ,  $
                            'center_incid'        ,  info.incid_angle_center_deg                                ,  $ 
                            'slant_range_center'  ,  info.slant_range_center_m                                  ,  $
                            'slant_range_near'    ,  info.slant_range_near_m                                    ,  $
                            'range_looks'         ,  info.looks_range                                           ,  $ 
                            'azimuth_looks'       ,  info.looks_azimuth                                         ,  $ 
                            'ellipsoidName'       ,  info.ref_ellipsoid                                         ,  $
                            'semiMajorAxis'       ,  float(info.ref_semi_major_axis_m)                          ,  $ 
                            'semiMinorAxis'       ,  float(info.ref_semi_minor_axis_m)                          ,  $ 
                            'ellipsoidscale'      ,  info.datum_scale                                           ) 



;
; Call general ancilliary data routine
;
ans_write_ancil_sio, filename, info, quiet=quiet
 
end
